#!/usr/bin/perl
#==============================================================================
#-- File:         exitcode.pl
#-- Date:         09.03.2006
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Translate exit codes
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/03/09 : First implementation
#--   1.0.1 - 2007/06/22 : ANTms52361: ErrorCodes::xitString has to be ErrorCodes::ExitString
#--   1.0.2 - 2009/10/09 : ANTmt77547.  Need at least perl 5.6.0
#--   1.0.3 - 2010/03/04 : PBMT x86 Changes - ANTmt88407
#--   1.0.4 - 2010/05/02 : ALU01178887
#--   1.0.5 - 2016/07/15 : ALU02237966


my $VERSION = "1.0.5 -- 2016/07/15";
#==============================================================================
use strict;
require 5.6.0;
use warnings 'all';

use Getopt::Long;
#==============================================================================
#== Find the lib dir
use Cwd qw( realpath);
use File::Basename;
use Config;

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 
#==============================================================================

use ErrorCodes();

my $code;
my $options = GetOptions('code=i', \$code);

if (! $options) {
  print STDERR "Usage $0 --code <value>\n";
  exit;
}

my $str = "";
if (defined($code)) {
  if ($code < 1000) {
    $str = ErrorCodes::ExitString($code);
  } else {
    $str = ErrorCodes::CvtErrorString($code);
  }
  print "$str\n";
} else {
  print STDERR "Usage $0 --code <value>\n";
}
