package DownloadIsam7354;

=head1 ISAM 7354 SW download module

=begin html
This library contains the functions dedicated to perform prechecks for a 7302/7330 ISAM.

=end html

=cut

#==============================================================================
#-- File:         DownloadIsam7354.pm
#-- Date:         06.11.2005
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Script to download new OSWP to ISAM-PIZZA boxes
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2008/09/25 : First implementation
#--   1.0.1 - 2009/04/22 : ANTmt54584
#--   1.0.2 - 2009/04/24 : Release transaction if TO = 127.0.0.1
#--   1.0.3 - 2009/11/24 : ANTmt86148: 7354RU fixes
#--   1.0.4 - 2010/04/02 : ANTmt91104 / ALU00100007 PBMT: SW dir permissions not set when not reducing
#--   1.0.5 - 2011/02/02 : ALU01006984 - Undo workaround done for ALU00100007 since issue is solved in NT SW
#--   1.0.6 - 2011/04/01 : Changes for 7354
#--   1.0.7 - 2011/04/05 : ALU01031457 - Support for R4220 & R4230
#--   3.0.1 - 2012/03/29 : 7354 changes for SSH failure 
#--   3.0.2 - 2016/07/15 : ALU02237966 
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697


=head1 VERSION

=begin html
Current version: "DownloadIsam7354.pm - 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "DownloadIsam7354.pm - 3.1.0 -- 2018/06/20";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP<a/></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2217-DFZZA">Isam7354RUCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2086-DFZZA">Capability</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use File::stat;
use File::Basename;
use Getopt::Long;
use POSIX ":sys_wait_h";
use Socket;

use Net::TFTP;
use Net::SNMP;

use IsamCli;
use Isam7354RUCli;
use Crypt::Lite;
use Utils;  # -- BB --
require Octopus;
use Capability;
use ToolCommon;
use PBMTGlobal;
require FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :SWDL);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub Download($$);
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $config;
my $toolData;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 Download($$$)

=begin html
This function is called to perform a SW download of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>ntType</td><td>the type of NT inserted</td></tr>
</table>

=end html

=cut

# Note on coding principles:
# --------------------------
# I know that this coding is not realy according to the rules of well-structure programming but in the end it has been done like this
# to keep the process readable and the indentation level acceptable (hence the many intermediate return statements).
#
sub Download($$) {
  my ($isam, $ntType) = @_;
  my $cli = undef;
  my $swParSize;
  my ($newMajor, $newMinor, $newSuffix, $newRelNum, $newFullRelNum, $activeRelNum, $fullActiveRelNum);
  my $shubSwName;
  my ($time1, $time2);
  my $activeNt;
  my $possible = 0;
  my $reason = "";
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $dirName = FamilyCommon::GetToolDirName();
  my $childCount = FamilyCommon::GetChildNum();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();

  if (! defined($params->{OSWP})) {
    FamilyCommon::IncrErrorCount("  ERROR: no OSWP specified\n",
				 EC_OSWP_MISSING,
				 undef,
				 1);
    return 0;
  }
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  # Check OSWP for variant
  my $new = basename($params->{SWPNAME});
  my $regExp = REGEXP;
  if ($new =~ /$regExp/) {   # -- Start 1.0.4
    $newMajor = $1;
    $newMinor = $2;
    $newSuffix = $3;
    $newFullRelNum = Capability::ConvertRelease($newMajor, $newMinor, $newSuffix); # -- 1.2.0
    $newRelNum =  Capability::ConvertRelease($newMajor, $newMinor);
  }                          # -- End 1.0.4
#  ToolCommon::Log($nodeLogFH, "Check OSWP variant...");
#  if (! Capability::OswpVariant($params->{SWPNAME}, Capability::Isam7354RUFamily())) {
#    FamilyCommon::IncrErrorCount(sprintf("  ERROR: incorrect ISAM variant for %s: '%s', expecting '%s'\n", $params->{SWPNAME},
#					 Capability::GetError(), Capability::GetOswpVariant(Capability::Isam7354RUFamily(), $newRelNum)),
#				 EC_WRONG_VARIANT,
#				 undef,
#				 1);
#    return 0;
#  }
  $params->{ISAM7354RU} = ""; 
  #-- Open CLI Session
  ToolCommon::Log($nodeLogFH, "Open CLI...");
  $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
  if (!defined($cli)) {
     if(defined($params->{PASSWORD2})) {
            sleep(30);
            my $cliError = IsamCli::GetError(); 
	    if($cliError =~ /Login failed/i) {
		    ToolCommon::Log($nodeLogFH, "Open CLI failed with first password, Trying again with second password ...");
		    my $oldCliPassword = $params->{PASSWORD}; 
		    $params->{PASSWORD} = $params->{PASSWORD2}; 
		    $cli = ToolCommon::OpenCli($isam, $nodeLogFH, $toolData->{TELNETTIMEOUT});
		    $params->{PASSWORD} = $oldCliPassword; 
		 if (!defined($cli)) {
		    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli also with second password for %s: %s\n", FamilyCommon::GetFailedNode(), IsamCli::GetError()),
						 EC_OPENCLI_FAILED,
						 sprintf(": %s", ToolCommon::GetError()),
						 1);
		    return 0;
		 } else {
		    ToolCommon::Log($nodeLogFH, "  OK\n");
		 } 
            }
         } else {
            FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()),
				 EC_OPENCLI_FAILED,
				 sprintf(": %s", ToolCommon::GetError()),
				 1);
            return 0;
        }
  } else {
    ToolCommon::Log($nodeLogFH, "  OK\n");
  }
  FamilyCommon::SetCliSession2Close($cli);  # Store for IncrErrorCount
  ToolCommon::Log($nodeLogFH, sprintf("Telnet errmode: %s\n", $cli->errmode)) if ($params->{DEBUG});

  #-- STEP 0: acquire transaction
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
  return undef if( ToolCommon::AcquireTransaction() );
  Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

  #-- STEP 1: NT type information and check Active OSWP is committed
  #-- Get NT type information
  ToolCommon::Log($nodeLogFH, "Obtain extended NT information...");
  my $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, $ntType, undef,
					 $toolData->{RETRIES});
  if (! defined($oswpInfo)) {
    FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
				 EC_GET_OSWP_FAILED,
				 sprintf(": %s", ToolCommon::GetError()),
				 1);
    return 0;
  }
  #-- Verify migration path
  $fullActiveRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
						 $oswpInfo->{'ACTIVE'}{'MINOR'},
						 $oswpInfo->{'ACTIVE'}{'SUFFIX'});
  $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
					     $oswpInfo->{'ACTIVE'}{'MINOR'});

  my $newSwpIndex = $oswpInfo->{PASSIVE}{INDEX};
  if (! $params->{FORCEDL}) {
    if ($oswpInfo->{'ACTIVE'}{'DBVERSION'} ne $params->{DBVER_TGTREL} ) {
      #-- Verify Migrate path
      Logger::LogTrace($nodeLogFH, "Migrate restrictions...\n");
      my ($restricted, $reason) = Capability::IsMigrateRestricted( $ntType,
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Migrate restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_MIG_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    } else {
      #-- Verify upgrade path
      Logger::LogTrace($nodeLogFH, "Upgrade restrictions...\n");
      my ($restricted, $reason) = Capability::IsUpgradeRestricted( $ntType, 
                                                                   $oswpInfo->{'ACTIVE'}{'DBVERSION'}, $params->{DBVER_TGTREL},
                                                                basename($oswpInfo->{'ACTIVE'}{'NAME'}), basename($params->{OSWP}) );
      if (not $restricted) {
          Logger::LogTrace($nodeLogFH, "Upgrade restrictions... NONE\n");
      } else {
          FamilyCommon::ReportStatus( EC_UPGR_RESTRICTED, sprintf(": %s (%s) -> %s (%s) restricted: %s", 
                            $oswpInfo->{'ACTIVE'}{'NAME'}, $oswpInfo->{'ACTIVE'}{'DBVERSION'},
                            basename($params->{OSWP}), $params->{DBVER_TGTREL}, $reason ));
          if ($restricted) {
	          return 0 if ($params->{CONTINUE});
	          CORE::exit(UPG_IMPOSSIBLE_CODE);
          }
      }
    }
  } else {
    ToolCommon::Log($nodeLogFH, "Forced download, continue without checks...\n");
  }

  if (Capability::ReleaseCompare($activeRelNum, Capability::ConvertRelease(3,2), REGEXP, 0) < 0) {
    # From R3.1 onwards the same OSWP can be redownloaded so we have to allow it.
    ToolCommon::Log($nodeLogFH, "Checking new software already present...");
    if (basename($oswpInfo->{ACTIVE}{NAME}) eq (basename($params->{OSWP})) || # -- 1.2.3
	(basename($oswpInfo->{PASSIVE}{NAME}) eq basename($params->{OSWP}) && $oswpInfo->{PASSIVE}{AVAIL} eq 'enabled')) { #-- 1.2.3
      FamilyCommon::AdvanceToNext("Already present... skipping\n");
      return;
    }
    ToolCommon::Log($nodeLogFH, " OK\n");
  }

  ToolCommon::Log($nodeLogFH, "Checking Active Software is committed...");
  if ($oswpInfo->{ACTIVE}{COMMITED} ne "committed") {
    #-- STEP 2: If not committed -> commit to remove passive OSWP
    ToolCommon::Log($nodeLogFH, "  not committed.\n");
    ToolCommon::Log($nodeLogFH, "Committing Active OSWP...");
    $ret = Isam7354Cli::CommitSWP($cli, $oswpInfo->{ACTIVE}{INDEX});
    if (!defined($ret) || ! $ret) {
      FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				   EC_COMMIT_FAIL,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return 0;
    } else {
      ToolCommon::Log($nodeLogFH, " OK\n");
    }
  } else {
    ToolCommon::Log($nodeLogFH, " OK\n");
  }
  #-- STEP 3: Abort any active downloads
  ToolCommon::Log($nodeLogFH, "Aborting Active Downloads...");
  if (uc($oswpInfo->{PASSIVE}{NAME}) ne NO_OSWP) {
    if (!defined(Isam7354RUCli::AbortDownloadSWP($cli, $newSwpIndex))) {
      FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				   EC_ABORT_DL_FAIL,
				   sprintf(": %s", IsamCli::GetError()),
				   1);
      return 0;
    } else {
      #-- no error checking as it fails if no active one
      ToolCommon::Log($nodeLogFH, " OK\n");
      sleep 2;
    }
  } else {
    ToolCommon::Log($nodeLogFH, " OK (no passive OSWP)\n");
  }

  Logger::LogTrace( $nodeLogFH, "Any file/DB transfer on-going...\n");
  return undef if( ToolCommon::IsTransferOnGoing() );

  ToolCommon::Log($nodeLogFH, "Configure Download Parameters...");
  #-- STEP 4: Configure Download Parameters
  $ret = Isam7354RUCli::ConfigureSWPDownload($cli, $newSwpIndex,
				       $params->{PRIMTFTPIP});
  if (!defined($ret) || ! $ret) {
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				 EC_CONFIG_DL_FAILED,
				 undef,
				 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFH, " OK\n");
  }

  #-- STEP 5: Download SWP
  ToolCommon::Log($nodeLogFH, "Downloading OSWP-$newSwpIndex ...");
  $time1 = time();
  $ret = Isam7354RUCli::DownloadSWP($cli, $newSwpIndex, $params->{OSWP});
  if (!defined($ret) || ! $ret) {
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamCli::GetError()),
				 EC_DL_SWP_FAILED,
				 sprintf(": %s", IsamCli::GetError()),
				 1);
    return 0;
  } else {
    $time2 = time();
    ToolCommon::Log($nodeLogFH, sprintf("  OK, transfered in %d seconds\n", $time2 - $time1));
  }
  $oswpInfo = ToolCommon::GetOswpInfo($cli, $snmpSession, $dir, $isam, $ntType, undef, $toolData->{RETRIES});
  if (!defined($oswpInfo)) {
    FamilyCommon::IncrErrorCount("",
				 EC_GET_OSWP_FAILED,
				 "(after SWP download)",
				 1);
    return 0;
  }
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
   return undef if( ToolCommon::ReleaseTransaction() );
   Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");

  IsamCli::AllowAlarms($cli);
  FamilyCommon::CloseCliSession2Close();
  return 1;
}

1;
