package IsamTL1;

=head1 TL1 accessor module

=begin html
This PERL module contains the TL1 accesses made to the NE.

=end html

=cut

#==============================================================================
#-- File:         IsamTL1.pm
#-- Date:         27.11.2008
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Library with ISAM CLI commands
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel-Lucent 2008
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.1 - 2009/02/05   : Start First implementation
#--   1.0.2 - 2009/04/07   : Bug Fixing BDFam46650
#--   1.0.3 - 2009/04/27   : Bug Fixing BDFam47356 BDFam47624
#--   1.0.4 -    Fixed BDFam46905 BDFam47064 BDFam47358 BDFam49761 BDFam49842
#--   1.0.5 - 2009/05/25   : Bug Fix BDFam49771
#--   1.0.6 - 2009/08/18   : Bug Fix BDFam52833
#--   1.0.7 - 2009/08/25   : Bug Fix BDFam53089
#--   1.0.8 - 2009/08/28   : Change request BDFam53592
#--   1.0.9 - 2009/09/15   : Bug Fix BDFam53328 BDFam54385 BDFam53905 BDFam51905
#--   1.0.10- 2009/09/29   : Bug Fix BDFam53737
#--   1.0.11- 2009/10/11   : Bug Fix BDFam55491
#--   1.0.12- 2009/10/27   : Bug Fix BDFam56265
#--   3.0.00- 2010/04/21   : Converted version to 3.0.00 for converged PBMT
#--   3.0.01- 2010/05/06   : Converged PBMT Changes
#--   3.0.02- 2011/01/12   : LRM adaptations for lib-perl path
#--   3.0.03- 2011/03/27   : linux compatible changes
#--   3.0.04- 2011/05/05   : ALU01178887
#--   3.0.05- 2012/07/31   : ALU01804049
#--   3.0.06- 2012/11/12   : ALU01821914
#--   3.0.07- 2013/01/10   : ALU01849911
#--   3.0.08- 2013/04/29   : ALU01880147
#--   3.0.09- 2013/09/13   : ALU01910838
#--   3.0.10- 2013/11/14   : ALU01924625
#--   3.0.11- 2013/12/07   : introduce additional option DELAYACT
#--   3.0.12- 2014/03/10   : ALU01955321
#--   3.0.13- 2014/05/08   : ALU01936431
#--   3.0.14- 2015/04/10   : ALU02062081
#--   3.0.15- 2015/01/07   : ALU02160776
#--   3.0.16- 2016/02/15   : ALU02174145
#--   3.0.17- 2016/03/15   : ALU02189074
#--   3.0.18- 2016/03/18   : ALU02187728
#--   3.0.19- 2016/03/21   : ALU02181126, ALU02183524
#--   3.0.20- 2016/07/18   : ALU02237966
#--   3.0.21- 2016/09/28   : ALU02278259
#--   3.0.22- 2018/05/31   : ALU02470414
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.1.1  - 2018/12/13 : While getting new TL1 connection, Return immediately when found connection reaches maximum or Busy
=head1 VERSION

=begin html
Current version: "IsamTL1.pm - 3.1.1 -- 2018/12/13"

=end html

=cut

my $VERSION = "IsamTL1.pm - 3.1.1 -- 2018/12/13";
#==============================================================================

use strict;

use Cwd;
use File::Basename;
use vars qw ($tmpLogFile);
use Config;

#-- 1.2.4.1 -- BEGIN
BEGIN {
  my $dir = dirname($0);
  my $path;
  my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif( $dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl/$perlLibLocation");
  if ($Config{archname} =~ /sun4/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$path/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
}

use perlTL1::UdpChannel;
use perlTL1::SshChannel;
use ToolCommon qw(:CONSTANTS);
use ErrorCodes qw(:EXIT_CODES :COMMON);#New
use FamilyCommon;
use SNMPComm;
use PBMTGlobal;
#use Logger;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub SelectFH($);
sub SetDebug($);
sub GetError();
sub Connect($$$$;$$);
sub Disconnect($);
sub SendCommand($$;$);
sub Transaction($$);
sub InhibitAlarms($);
sub AllowAlarms($);
sub GetOswpInfo($$);
sub CommitSWP($$);
sub ActivateSWP($$);
sub AbortDownloadSWP($$);
sub DownloadSWP($$$$$$);
sub WaitForDownload($$$);
sub WaitForNTSync($);
sub backupDB_New($$$$$);
sub backupDB($$$);
sub WaitForBackupDB($$);
sub WaitForNTSync($);
sub GetNtExInfo($);
sub GetSwdxMode($);
sub getONTSWver($$);
sub getONTSWctrl($$);
sub removeONTSWP($$);
sub getONT($);
sub getEQPT($);
sub getNEopstat($);
sub getPrecheckAlarm($);
sub RestoreDB($$$$$);
sub WaitForUploadDB($$);
sub AddONTSWCTRL($$$$$$);
sub GetActiveDBVer($);
sub GetPassiveDBVer($);
sub RetriveActiveSwFileList($$);
sub setTL1Handler($$);
sub getTL1Hanlder();
sub getTL1ErrorMsg(); 
sub CheckErrorMib();  
sub ResetGlobals();

#==============================================================================
#== Variables
#==============================================================================
my $DEBUG = undef;
my $ERROR = "";
my $gTl1Hndlr = undef;
my $gisam = undef;
my $gCmdssntmo = 150;
my $gRemoteTmout = 30;
my $gCraftTmout = 10;
my $gAuthSuper= "LOCAL";
my $gAuthDef = "REJECTALL";
my $gSoakMode = "DISABLED";
my $gGponOltRngMode = "NORMAL";
my $ssh = 0;
my %ctagcmds = ();
my $ctag = 11000;

### Obsolete (or) Not used anywhere
my %tl1ErrorCode = ("EATN" => "Equipage, Not valid for Access Type.",
"ENDS" => "Equipage, Not equipped with Duplex Switching.",
"EQWT" => "Equipage, Wrong Type - Equipment is of the wrong type.",
"ICNC" => "Input, Command, Not Consistent - The command contains VERB,MODIFIER, or parameters that are incompatible.",
"IDNC" => "Input, Data Not Consistent - The command contains parameters that are incompatible.",
"ICNV" => "Input Command Not Valid.",
"IDNV" => "Input, Data Not Valid - The command contains invalid parameter.",
"IEAE" => "Input, Entity Already Exists - The entity to be created already exists.",
"IENE" => "Input, Entity Non-Existant - The referenced entity does not exist.",
"IIAC" => "Input, Invalid Access Identifier (AID) - The command contains an invalid AID.",
"IISP" => "Input, Invalid Syntax or Punctuation - The command has a syntax error.",
"IITA" => "Input, Invalid Target Identifier - The command contains an invalid TID.",
"IPEX" => "Input, Parameter, Extra - The command contains unsupported extra parameter.",
"IPMS" => "Input, Parameter, Missing - The command is missing a required parameter.",
"IPNC" => "Input ,Parameter, Not Consistent - Command contains incompatible parameters.",
"IPNV" => "Input, Parameter, Not Valid - The command contains invalid parameter.",
"MERR" => "Multiple Error.",
"PIUI" => "Privilege Illegal User Identity.",
"RABY" => "Resource, All Busy - All instances of requested resource are busy.",
"RNAN" => "Resource, requested NE access Address does Not exist.",
"RTBY" => "Resource, Requested TAP is Busy.",
"SAAL" => "Status, Already Allowed.",
"SAIN" => "Status, Already Inhibited.",
#"SARB" => "Status, All Resources Busy - The system is busy or overloaded.",
"SCSN" => "Status, Invalid Command Sequence.",
"SDNA" => "Status, Duplex unit Not Available.",
"SDNC" => "Status, Data Not Consistent - The input data in the command is inconsistent with the NE database.",
"SLEM" => "Status, List Exceed Maximum - Command processing exceeded system limit.",
"SNVS" => "Status, Not in Valid State - The entity is not in a state valid for performing the requested command.",
"SRCI" => "Status, Requested Command Inhibited.",
"SRCN" => "Status, Requested Condition already exists.",
"SROF" => "Status, Requested Operation Failed - Unable to perform requested operation.",
"SSRE" => "Status, System Resources Exceeded - The command exceeded the system limit on a parameter.",
"SSTP" => "Status, Stopped - The command execution was terminated because of a software/hardware problem."); 

=head1 FUNCTIONS

=head2 SelectFH()

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug()

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
}

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

sub ResetGlobals() {
   $ssh = 0;
}

=head2 GetSwdxMode($)

=begin html
Open NT Info on TL1 session. <br>
The following parameters are used:
<table>
  <tr><td>tl1session</td>An open TL1Session<td></td></tr>
</table>

=end html

=cut

#BDFam53727
sub GetSwdxMode($) {
  my ($session) = @_;
  my $switchAlw=undef;

  my $cmdResponse = SendCommand($session, "RTRV-SWDX-EQPT::NT:;");
  if (!defined($cmdResponse)) {
     return undef;
  }

  my @response = split(/\n/, $cmdResponse);
  foreach my $line (@response) {
   if($line=~/NT:ALW:NORM/) {
     $switchAlw=1;
   }
  }
  return $switchAlw;
}

=head2 GetNtExInfo($)

=begin html
Open NT Info on TL1 session. <br>
The following parameters are used:
<table>
  <tr><td>tl1session</td>An open TL1Session<td></td></tr>
</table>

=end html

=cut

sub GetNtExInfo($) {
  my ($session) = @_;
  my $index = undef;
  my $name = undef;
  my $status = undef;
  my $availability = undef;
  my $swpIndex;
  my $otherSwpIndex;
  my %oswpByNum;

  my %ntInfo = ('NTA' => { 'TYPE' => "EMPTY",
  			   'STATUS' => "EMPTY",
                           'ACTIVE' => "EMPTY",
                           'ICS'  => "EMPTY",
                           'STATE'  => "EMPTY"},
  		'NTB' => { 'TYPE' => "EMPTY",
  			   'STATUS' => "EMPTY",
                           'ACTIVE' => "EMPTY",
                           'ICS'    => "EMPTY",
                           'STATE'    => "EMPTY"});


    my $cmdResponse = SendCommand($session, "RTRV-EQPT::NT:;");
    if (!defined($cmdResponse)) {
        return undef;
     }

   my @response = split(/\n/, $cmdResponse);

      my ($position, $type, $enabled);
      foreach my $line (@response) {
        $position = 0;
        $type = "";
        $enabled = "";
        $index = "";
        if($line=~/\"NT(\w):(\D+)::(\D+)\"/) {
  	        $position = $1, $type = $2, $enabled = $3;
                $index = uc($position) eq 'A' ? 'NTA' : 'NTB';
                # Fix for BDFam52833
                #$ntInfo{$index}{'STATUS'} = uc($enabled) eq "IS-STBY," ? "PRESENT" : "EMPTY";

                $ntInfo{$index}{'STATUS'} = (uc($enabled) eq "IS-STBY," ||
                                             uc($enabled) eq "IS-NR," ||
                                             uc($enabled)  =~/OOS-AU,/  ) ? "PRESENT" : "EMPTY";
                $ntInfo{$index}{'STATE'}  = uc($enabled);

  	        $ntInfo{$index}{'ACTIVE'} = uc($enabled) eq "IS-NR," ? "ACTIVE" : "EMPTY";
                my @tp = split(',',$type);
                $ntInfo{$index}{'TYPE'} = uc($tp[0]); #if ( $tp[0] eq $tp[1] );
                $ntInfo{$index}{'ICS'} = uc($tp[1]);

                # intially set to present bcoz of NTA or NTB card found,while checking  the dectected
                # type and found EMPTY so setting it back to EMPTY, to consider as simplex machine

                if (($tp[1] eq "EMPTY") || (!$tp[0])) {
                 $ntInfo{$index}{'STATUS'} = "EMPTY";
                }

       }
     }
      return \%ntInfo;
   }

=head2 Connect($$$$$;$$)

=begin html
Open a TL1 session and establish TL1 login.  The opened telnet session is returned to the caller or undef in case of failure.<br>
The following parameters are used:
<table>
  <tr><td>ip</td><td>IP address of the NE</td></tr>
  <tr><td>udpPort</td><td>UDP port to be used for the TL1 session</td></tr>
  <tr><td>user</td><td>TL1 user name</td></tr>
  <tr><td>passwd</td><td>TL1 password</td></tr>
  <tr><td>params</td><td>pointer to a hash of command paramater values</td></tr>
  <tr><td>logFile</td><td>file handle for logging of TELNET activity (undef if not needed)</td></tr>
  <tr><td>ontParams</td><td>Optional parameter passed from ONT scripts</td></tr>
</table>

=end html

=cut
sub Connect($$$$;$$) {
  my ($isam, $udpPort, $userName, $passWord, $logFile,$ontParams) = @_;

  my $session;
  my $cmdResponse;
  my $ok = 0;
  my $response = undef;
  my $snmpAccBits;
  my $fh = FamilyCommon::GetWarnLogFile(); #added for 28th build

  my $configData = FamilyCommon::GetConfigData();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $timeout_schk = $toolConfigData->{TELNETTIMEOUT} if defined($toolConfigData->{TELNETTIMEOUT}) ;

  $snmpAccBits = ToolCommon::GetAccessettings($snmpSession, $params->{DEBUG});
  $ssh = 0;
  if (defined($snmpAccBits)) {
    if (!ToolCommon::TL1ViaUDP($snmpAccBits, $params->{DEBUG})) {
      if (ToolCommon::TL1ViaSSH($snmpAccBits, $params->{DEBUG})) {
        $ssh = 1;
      } else {
        $ERROR = "Ssh and udp disabled for TL1";
        $ok = undef;
        return $ok;  
      } 
    }
  } else {
    my $t = ToolCommon::GetError();
    FamilyCommon::SetAndLogError($fh, $t, "$t\n");
    $ok = undef;
    return $ok;
  }

  if ($ssh) {
    Logger::LogDebug($nodeLogFH, "Connecting via SSH...\n");
    my $port = "1022";
    $session = SshChannel::ConnectSSH($isam,$userName,$passWord,$timeout_schk,$logFile,$port);
    $ok = 1 if ($session);
    setTL1Handler($session,$isam) if ($ok);
  } else {
    Logger::LogDebug($nodeLogFH, "Connecting via UDP...\n");
    $session = new UdpChannel;
    $ok = $session->OpenChannel($isam, $udpPort, $logFile);
    if ($ok) {
      setTL1Handler($session,$isam) if ($ok);
      $response = SendCommand($session, "LOGOFF;");
      if((defined($response)) && (($response eq "Busy")||($response =~ /No Resources.... Maximum sessions/))){
        $ok = 0;
        FamilyCommon::SetAndLogError($fh, "Login failed: Busy/Maximum sessions reached!\n");
        return undef;
      }
      $cmdResponse = SendCommand($session, sprintf("ACT-USER::%s:UEACT::%s;", $userName, $passWord),$passWord);
      if (defined($cmdResponse)) {
        $ok = 1;
      } else {
        $ok = 0;
        FamilyCommon::SetAndLogError($fh, "Login failed:Incorrect TL1 username or password specified!\n");
        return undef;
      }
    }
  }

  RtrvTimeOut($session);    #BDFam56265

  #Fix for BDFam55491 starts here
  if ($ok) {
    SendCommand($session, "ED-SECU-CID::REMOTE::::TMOUT=180;"); 
    sleep(1);
    SendCommand($session, "ED-SECU-CID::CRAFT::::TMOUT=180;") ; 
    sleep(1);
    SendCommand($session, "SET-NE-ALL::COM::::CMDSSNTMO=0,AUTHSUPER=$gAuthSuper,GPONOLTRNGMODE=$gGponOltRngMode;"); 
    Disconnect($session);
  sleep (3);
  if ($ssh) {
    Logger::LogTrace($nodeLogFH, "Connecting via SSH...\n");
    my $port = "1022";
    $session = SshChannel::ConnectSSH($isam,$userName,$passWord,$timeout_schk,$logFile,$port);
    $ok = 1 if ($session);
    setTL1Handler($session,$isam) if ($ok);
  } else {
    Logger::LogDebug($nodeLogFH, sprintf("Connecting via UDP...\n")) if defined($DEBUG);
    $session = new UdpChannel;
    $ok = $session->OpenChannel($isam, $udpPort, $logFile);
    if ($ok) {
      setTL1Handler($session,$isam) if ($ok);
      $response = SendCommand($session, "LOGOFF;");
      if((defined($response)) && ($response eq "Busy")){
        $ok = 0;
        last;
      }
      $cmdResponse = SendCommand($session, sprintf("ACT-USER::%s:UEACT::%s;", $userName, $passWord),$passWord);
      if (defined($cmdResponse)) {
        $ok = 1;
      } else {
        $ok = 0;
      }
    }
  }
  }

  if($ok) {
    SendCommand($session, "ED-SECU-CID::REMOTE::::TMOUT=$gRemoteTmout;");
    sleep(1);
    SendCommand($session, "ED-SECU-CID::CRAFT::::TMOUT=$gCraftTmout;");
    sleep(1);
    if ($ontParams) {
    SendCommand($session, "SET-NE-ALL::COM::::CMDSSNTMO=$gCmdssntmo,AUTHSUPER=$gAuthSuper,AUTHDEF=$gAuthDef,SOAKMODE=$gSoakMode,GPONOLTRNGMODE=$gGponOltRngMode;");
    }
    else {
    SendCommand($session, "SET-NE-ALL::COM::::CMDSSNTMO=$gCmdssntmo,AUTHSUPER=$gAuthSuper,GPONOLTRNGMODE=$gGponOltRngMode;");
    }
  }

  return $ok ? $session : undef;
}

=head2 Disconnect($)

=begin html
Logout from the NE and close the TL1 session specified by $session.

=end html

=cut

sub Disconnect($) {
  my ($session) = @_;
  SendCommand($session, "CANC-USER;");

  if ($ssh) {
    SshChannel::Disconnect($session);
  } else {
    $session->CloseChannel();
  }
  $session = undef;
  setTL1Handler($session, undef);
}


sub GetCtagValue() {
    return $ctag;
}

sub SetCtagValue($) {
    my ($value) = @_;
    $ctag = $value;
}

=head2 SendCommand($$;$)

=begin html
Send a TL-1 command.<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>pointer to an opened TL-1 session</td></tr>
  <tr><td>cmd</td><td>TL-1 command to be sent</td></tr>
</table>

=end html

=cut

sub SendCommand($$;$) {
  my ($session, $cmd,$tl1Passwd) = @_;
  my $cnt =1;
  my $cmdSends = 0;
  my $ok;
  my $response = undef;
  my $fh = FamilyCommon::GetWarnLogFile();
  my $prevLine;
  my $snmpcheck = 0;
  my $configData = FamilyCommon::GetConfigData();
  my $toolData = FamilyCommon::GetToolData();
  $session = getTL1Hanlder();
  $ERROR = "";
  if(!defined($session)){
    $ERROR = "Invalid TL1 Session";
    return undef;
  }

  if ($cmd =~/;$/){
     chop($cmd);
  }
  my @cmdDetails=split(/:/, $cmd);
  my $cmdName = $cmdDetails[0];

  my ($cmdOriginallySent, $cmdToLog ) = (undef, undef );
  $cmdOriginallySent = $cmdToLog = $cmd;
  if( !($cmd =~ /ACT-USER|CANC-USER|LOGOFF/) ) {
      Logger::LogDebug($nodeLogFH, "Command W/O-CT : $cmdToLog\n");
      my $regex = "([^:]*:){3}";
      $cmd =~ /($regex)(.*)/;
      my $pattern = $1;
      ### Updating cmd as comnabd with CTAG
      if(defined($pattern) &&($pattern ne "")) {
         my @arr = split($pattern,$cmdOriginallySent);
         $cmd = $pattern . "$ctag";
         $cmd .=  $arr[1] if defined($arr[1]);
      } else {
        $cmd = $cmd . ":" . "$ctag";
      }
  }
  if (not ($cmd =~/;$/)) {
      $cmd .= ";";
  }
  my ( $thisRetryCount, $totalRetryCount) = (0,2);                                                             # BDFam55878
  while( $thisRetryCount < $totalRetryCount) {
      $thisRetryCount++;
      if ($snmpcheck) {
           my $snmpSession = FamilyCommon::GetSnmpSession2Close();
           if(!defined($snmpSession)){
              $ERROR = "Invalid SNMP Session";
              return undef;
           }
           if( $params->{'NOPING'} == 0 ) {
               if( ToolCommon::Ping($configData->{PING}, $params->{'ISAM'}) ) {
                 my $pingOk = 1;
               } else {
                  return undef;
               }
           }

           my $pctComplete = SNMPComm::GetValue('SYSTEM INIT', SYSTEMINITPROGRESSOID );
           return if (not defined $pctComplete or $pctComplete != 100) ;
      }
      if( !($cmd =~ /ACT-USER|CANC-USER|LOGOFF/) ) {
         ### Increment the Ctag alone;
         if( $cmd =~ /:$ctag:/  or $cmd =~ /:$ctag;/ ) {
            my $current_ctag_value = $ctag;
            $ctag++;
            my $ctag_pre = $`;
            my $ctag_part = $&;
            my $ctag_post = $';
            $ctag_part =~ s/$current_ctag_value/$ctag/;
            $cmd = "$ctag_pre$ctag_part$ctag_post";
            Logger::LogDebug($nodeLogFH, "CTAG Update : $cmd\n");
         }
         $ctagcmds{$ctag} = $cmdToLog = $cmd;
      } elsif( $cmd =~ /ACT-USER/ ) {
          $cmdToLog =~ s/$tl1Passwd/*********/;
      }
      Logger::LogDebug($nodeLogFH, sprintf("TL1-Command Sent : %s \n",$cmdToLog));
      if ($ssh) {
        $response = SshChannel::SendCommand($session,$cmd);
      } else {
        $response = $session->SendTL1Command($cmd);
      }
      if( !defined($response) ){                                                   # BDFam49771
          if ($ssh) {
            $ERROR = SshChannel::GetError();
          } else {
            $ERROR = $session->GetError();                                            # BDFam49771
            $snmpcheck = 1;
          }
          $ok = 0;                                                                  # BDFam49771
          Logger::LogTrace($nodeLogFH, "TL1-Command($cmdName) Response :\n ##NULL##; Error: $ERROR\n");

          if ($ERROR =~ /Connection to $params->{'ISAM'} closed/ || $ERROR =~ /timed-out/) {    #BDFam61407
              Logger::LogTrace($nodeLogFH, "Calling doLoginAgain()!!!!\n");
               my $ret = doLoginAgain();
               if (! $ret) {
                 $ok = 0;
                 $response = undef;
                 $ERROR = "Re-Login failed";
                 last;
               }
          }
          next;                                                                     # BDFam49771
      }
      Logger::LogTrace($nodeLogFH, "TL1-Command($cmdName) Response :\n $response\n") if ( $response );
      if($response eq "No Resources.... Maximum sessions (6) exceeded" ){     # BDFam49771
          $ERROR = "No Resources.... Maximum sessions (6) exceeded";
          $ok = 0;
          $response = undef;
          last;
      } elsif (($response =~ /Database is in use by another manager|MIB Locked by Different Manager/)) {                # BDFam55878
          sleep(15 * $cnt);                                                         # BDFam55878
          $cnt++;
          next;                                                                     # BDFam55878
      } elsif ($response =~ /Login Not Active/) {
          my $ret = doLoginAgain();
          if (! $ret) {
            $ok = 0;
            $response = undef;
            $ERROR = "Re-Login failed";
            last;
          }
          return SendCommand($session, $cmd,$tl1Passwd);
      } elsif ( ($response =~ /Already Allowed|Already Inhibited/) ) {
          $ok = 1;
          $ERROR = $response;
          $thisRetryCount = $totalRetryCount;                                                               # BDFam49771
          last;
      } else {
          my @responseArray = split(/\n/,$response);
          my( $thisLine, $prevLine ) = (  "", "" );
          my( $thisLineNo, $thisCmdResp, $foundDeny,$foundDenyRetry ) = (0, 0, 0, 0);
          my $totalLineNo = @responseArray;
          while( $thisLineNo < $totalLineNo ) {
               $prevLine=$thisLine;
               $thisLine = $responseArray[$thisLineNo];
               $thisLineNo++;
               my $possibleResponseToConsider = "$cmdName|UEACT";
               if(($cmd !~ /ACT-USER/) || ($cmd !~ /CANC-USER/) || ($cmd !~ /LOGOFF/)) {
                   $possibleResponseToConsider .= "|$ctag";
               }
               $thisCmdResp=1 if( $thisLine =~ /$possibleResponseToConsider/ );
               if($thisCmdResp == 1) {
                  if( $foundDeny ) {
                     if( $thisLine =~/^\s*SARB|SSTP\s*$/ ) {
                         $totalRetryCount = 5;
	                      # All resources busy, try again
	                      select(undef, undef, undef, 0.1); # 100 msec delay
                          my $retryWaitTimeout =  defined($configData->{'TL1_BUSY_RETRY_WAITTIME'}) ? $configData->{'TL1_BUSY_RETRY_WAITTIME'} : 60;
		                  sleep($retryWaitTimeout);
                          $foundDenyRetry = 1;
	                      $ERROR = "Could not send command, all resources busy";
	                      $ok = 0;
                          last;
                    }
                  } elsif($thisLine eq "No Resources... Maximum sessions (6) exceeded"){
                       $ERROR = "TL1 resources Busy. Try after some time";
                       $ok = 2;
                       last;
                  }elsif ( $thisLine =~/DENY/ || $prevLine=~/DENY/ ) {
	                   $ok = 0;
                       $foundDeny = 1;
                       $ERROR = "TL1 Operation Denied. "; 
                       $response = undef;
                  } elsif ($thisLine =~/PLNA/ || $prevLine=~/PLNA/) {
	                   # Privilege, Login not Active
	                   $ok = 0;
                       $thisRetryCount = $totalRetryCount;                                                               # BDFam49771
	                   $ERROR = "Privilege, Login not Active";
                       last;
                  } elsif ( $thisCmdResp and ( $thisLine =~/COMPLD/ || $prevLine=~/COMPLD/ ) ) {
	                   $ok = 1;
	                   $ERROR = $response;
                       $thisRetryCount = $totalRetryCount;                                                               # BDFam49771
                       last;
                  }
                }

                if($thisLine =~ /[^\s]+\s+(\d+)\sCOMPLD/){
                     my $compldTag = $1;
                     my $compldFlag = 0;
                     if (defined($compldTag) && $compldTag > 10000) {
                         my $ctagdiff = $ctag - $compldTag;
                         if ($ctagdiff == UdpChannel::GetRetrieveHeaderCount()) {
                           Logger::LogDebug($nodeLogFH, sprintf("Actual command $compldTag response received. \n"));
                           $compldFlag = 1;
                           $compldTag = $ctag;
                           UdpChannel::SetRetrieveHeaderCount(0);
                         }
                     }
                     if ($ctag eq $compldTag) {
                         if (!$compldFlag) {
                              Logger::LogDebug($nodeLogFH, sprintf("TL1 Operation completed for ctag $compldTag\n"));
                              $thisRetryCount = $totalRetryCount;                                                               # BDFam49771
                         }
                     } else {
                        if(($cmd !~ /ACT-USER/) && ($cmd !~ /CANC-USER/) && ($cmd !~ /LOGOFF/)) {
                          if (defined($compldTag)) {
                            Logger::LogDebug($nodeLogFH, sprintf("Response recieved for (CTAG:%s,%s) %s \n", $compldTag, $ctag, $cmd));
                             my $local = $ctag - $compldTag;
                             my $tmp;
                             for ($tmp=1; $tmp<=$local; $tmp++) {
                                 Logger::LogDebug($nodeLogFH, sprintf("Reading Remaining Response : %s \n",$tmp));
                                 $response = UdpChannel::ReadResponse($session);  #
                                 Logger::LogDebug($nodeLogFH, sprintf("Response : %s \n",$response));
                                 if ($response =~ /$ctag/) {
                                   Logger::LogDebug($nodeLogFH, sprintf("Actual Response  received for : %s \n",$ctag));
                                   $thisCmdResp = 1;
                                   $ok = 1;
                                   $thisRetryCount = $totalRetryCount;                                                               # BDFam49771
                                   $tmp = $local;
                                 }
                             }
                          }
                        }
                     }
                } elsif($thisLine =~ /[^\s]+\s+(\d+)\sDENY/) {
                      my $denyTag = $1;
                      $ERROR = "TL1 Operation Denied for ctag $denyTag" if($denyTag == $ctag);
                      Logger::LogDebug($nodeLogFH, sprintf("TL1 Operation completed for ctag $ctag\n")) if ($ctag == $denyTag);
                }
           } ## Inner-While

           if(!$thisCmdResp || !defined($ok)){
               if($cmd=~/ACT-USER/) {
                   my $cmd1= $cmd;
                   $cmd1=~ s/$tl1Passwd/*********/;
                   Logger::LogDebug($nodeLogFH, sprintf("Undefined response recieved from OLT: Retries:$thisRetryCount: Command is : $cmd1 \n"));
                   $snmpcheck = 1;
                   SendCommand($session, "CANC-USER");
                   sleep(5);   # Give Sufficient Time incase of ACT-USER command to avoid Max 6 Session issue in UDP mode
               } else {
                   $snmpcheck = 1;
                   Logger::LogDebug($nodeLogFH, sprintf("Undefined response recieved from OLT: Retries:$thisRetryCount: Command is : $cmd \n"));
                   if ($thisRetryCount < $totalRetryCount ) {
                     sleep($thisRetryCount * 60);  #SARB changes
                   }
               }
               $ok = 0;
            } elsif ( $foundDeny ) {
               $thisRetryCount = $totalRetryCount if( $foundDenyRetry == 0 );
            }
       }
 } ## while
 if(defined($response)){
  $response =~ s/\r//g;
  if($cmd=~/ACT-USER/){
      $cmd=~ s/$tl1Passwd/*********/;
      }
  }

  if($ok == 2){
    return "Busy";  #Busy condition
  }
  Logger::LogDebug($nodeLogFH, sprintf("Commandi (RC:$ok) is '$cmd'\n"));
  return $ok ? $response : undef;
}

=head2 Transaction($$)

=begin html
Acquire or release command transaction as specified by $action using TL1 session $session.<br>Returns 1 in case of success, 0 in case of failure

=end html

=cut

sub Transaction($$) {
  my ($session, $type) = @_;
  my $response;

  if ($type eq "acquire") {
    $response = SendCommand($session, "STA-CMDSSN::COM::;");
    if( !defined $response ) {
        FamilyCommon::ReportStatus( EC_CONFIG_LOCK_ACQUIRE_FAILED, GetError(), undef );
    } else {
        Logger::LogTrace( $nodeLogFH, "Acquired ownership for configuration-lock\n");
    }
  } else {
    $response = SendCommand($session, "STP-CMDSSN::COM::;");
    if( !defined $response ) {
        FamilyCommon::ReportStatus( EC_CONFIG_LOCK_RELEASE_FAILED, GetError(), undef );
    } else {
        Logger::LogTrace( $nodeLogFH, "Released ownership for configuration-lock\n");
    }
  }

  return defined($response) ? 1 : 0;
}

=head2 InhibitAlarms($)

=begin html
Inhibit generation of alarms using TL1 session $session.

=end html

=cut

sub InhibitAlarms($) {
  my ($session) = @_;
  my $response = SendCommand($session, "inh-msg-ALL::ALL:::CR&MJ&MN&NA;");
  return defined($response) ? 1 : 0;
}

=head2 AllowAlarms($)

=begin html
Allow generation of alarms using TL1 session $session.

=end html

=cut

sub AllowAlarms($) {
  my ($session) = @_;
  my $response = SendCommand($session, "alw-msg-ALL::ALL:::CR&MJ&MN&NA;");
  return defined($response) ? 1 : 0;
}

=head2 GetOswpInfo($$)

=begin html
Retrieve OSWP info using the opened TL1 session.  $regExp is used to determine the SW version numbers.
The undefined value is returned in case of error otherwise a reference to hash is returned.  It holds 2 entries: "ACTIVE" and "PASSIVE".  For each
entry the following is given:
<ul>
  <li>INDEX: the associated OSWP index</li>
  <li>NAME: the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>COMMITED: the OSWP SW commited status</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>

=end html

=cut

sub GetOswpInfo($$) {
  my ($session, $regExp) = @_;
  my $index = undef;
  my $name = undef;
  my $status = undef;
  my $availability = undef;
  my $swpIndex;
  my $otherSwpIndex;
  my $completed=0;
  my %oswpByNum;
  my $actOswp=0;
  my $passOswp;
  my $thisCmdResp = 0;
  my @response;
  my %oswpInfo = ('ACTIVE' => { 'INDEX' => 0,
				'NAME' => NO_OSWP,
				'COMMITED' => "",
				'AVAIL' => "",
				'MAJOR' => 0,
				'MINOR' => 0,
				'SUFFIX' => 0,
				'MAINT' => 0,
				'DLPROGRESS' => "" },
		  'PASSIVE' => { 'INDEX' => 0,
				 'NAME' => NO_OSWP,
				 'COMMITED' => "",
				 'AVAIL' => "",
				 'MAJOR' => 0,
				 'MINOR' => 0,
				 'SUFFIX' => 0,
				 'MAINT' => 0,
				 'DLPROGRESS' => ""} );
  my @active;
  my @passive;
  my $major = undef;    # -- 1.0.4
  my $minor = undef;    # -- 1.0.4
  my $maintRel = undef; # -- 1.2.0
  my $suffix = undef;   # -- 1.2.0
  my $progress;
  my $cmdResponse;

 while (!$completed) {

  $cmdResponse = SendCommand($session, "rept-opstat-oswp::all::;");

  if(!defined($cmdResponse)) {
    return undef;
  }
 @response = split(/\n/, $cmdResponse);
 foreach my $line (@response) {
  if(($line =~ /rept-opstat-oswp::all::/) || ($line =~ /rept-opstat-oswp::all:\d+:/))
  {  $thisCmdResp=1;
     $completed=1;
  }
   if($thisCmdResp == 1){
    if ($line =~ /\"OSWP-(\d):(\w+),(\w+)\"/) {
      my $oswpNum = $1;
      my $fieldName = $2;
      my $fieldConts = $3;
      $oswpByNum{$oswpNum}{$fieldName} = $fieldConts;
      $actOswp = $oswpNum if (($fieldName eq "ACTSTAT") && ($fieldConts eq "ACTIVE"));
    }
  }
 }

}
  $passOswp = $actOswp == 1 ? 2 : 1;


  $oswpInfo{'ACTIVE'}{'INDEX'} = $actOswp;
  $oswpInfo{'ACTIVE'}{'COMMITED'} = lc($oswpByNum{$actOswp}{COMMITSTAT});
  $oswpInfo{'ACTIVE'}{'AVAIL'} = lc($oswpByNum{$actOswp}{AVAILSTAT});
  $oswpInfo{'PASSIVE'}{'INDEX'} = $passOswp;
  $oswpInfo{'PASSIVE'}{'COMMITED'} = lc($oswpByNum{$passOswp}{COMMITSTAT});
  $oswpInfo{'PASSIVE'}{'AVAIL'} = lc($oswpByNum{$passOswp}{AVAILSTAT});
  $cmdResponse=undef;
$completed=0;

while (!$completed) {
  $cmdResponse = SendCommand($session, "rept-opstat-oswpdl::all::;");
  if(!defined($cmdResponse)) {
    return undef;
  }
  #  @response = split(/\n/, $cmdResponse);  #commented for BDFam53089

# Fix for BDFam53089 parsing the response for the packname name wrap to next line,
# now the fix will create and @response Array.

 my $full_line;
 foreach my $line (split(/\r?\n/, $cmdResponse))
 {
   if( $line =~ /\"(OSWP(-\d+)+):/ ) {
    $full_line = $line;
    push (@response,$line);
   }
   else
   {
    $line =~ s/^\s+//;
    $full_line .= $line;
   push (@response,$full_line);
   }
 }



  $thisCmdResp = 0;
  foreach my $line (@response) {
  if(($line =~ /rept-opstat-oswpdl::all::/) || ($line =~ /rept-opstat-oswpdl::all:\d+:/)) {
   $thisCmdResp = 1;
   $completed=1;
  }
  if($thisCmdResp==1){
    if ($line =~ /\"OSWP-(\d):PACKNAME,(.*)\"/) {
     my $oswpNum = $1;
       #BDFam4970
       my $major = 0;    # -- 1.0.4
       my $minor = 0;    # -- 1.0.4
       my $maintRel = 0; # -- 1.2.0
       my $suffix = 0;   # -- 1.2.0


      my $oswpName = basename($2);
      if ($oswpName =~ /$regExp/) {  # -- 1.0.4
	$major = $1;                                  # -- 1.0.4
	$minor = $2;                                  # -- 1.0.4
	$suffix = $3;                                 # -- 1.2.0
	$maintRel = $4;                               # -- 1.2.0
      }                                               # -- 1.0.4
      if ($oswpNum == $actOswp) {
	$oswpInfo{'ACTIVE'}{'MAJOR'} = $major;
	$oswpInfo{'ACTIVE'}{'MINOR'} = $minor;
	$oswpInfo{'ACTIVE'}{'SUFFIX'} = $suffix;
	$oswpInfo{'ACTIVE'}{'MAINT'} = $maintRel;
	$oswpInfo{'ACTIVE'}{'NAME'} = $oswpName;
      } else {
	$oswpInfo{'PASSIVE'}{'MAJOR'} = $major;
	$oswpInfo{'PASSIVE'}{'MINOR'} = $minor;
	$oswpInfo{'PASSIVE'}{'SUFFIX'} = $suffix;
	$oswpInfo{'PASSIVE'}{'MAINT'} = $maintRel;
	$oswpInfo{'PASSIVE'}{'NAME'} = $oswpName;
      }
    }
    elsif ($line =~ /\"OSWP-(\d):DLPROGRESS,(.*)\"/) {
	  my $oswpNum = $1;
	  if ($oswpNum == $actOswp) {
	$oswpInfo{'ACTIVE'}{'DLPROGRESS'} = $2;
        } else {
	$oswpInfo{'PASSIVE'}{'DLPROGRESS'} = $2;

        }

    }
  }
 }
}
  return \%oswpInfo;
}

=head2 WaitForDownload($$$)

=begin html
Wait for DB or SW download completion<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>oswpIndex</td><td>OSWP index</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForDownload($$$) {
  my ($session, $oswpIndex, $timeout) = @_;
  #-- check for completed download (or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $progress = undef;
  my $error;
  my $fileServer = "";
  my $errorFile = "";
  my $toReturn = undef;
  my $slept = 0;

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      sleep 25;

      $response = SendCommand($session, sprintf("rept-opstat-oswpdl::oswp-%d:;", $oswpIndex));

      if (!defined($response)) {
	if ($retries > 4) {
	  # 5 retries, seems that the first ones may yield no response...
	  $ERROR = sprintf("Maximum command retransmissions tried for rept-opstat-oswpdl::oswp-%d", $oswpIndex);
	  $toReturn = 0;
	  return;
	}
	$retries++;
	sleep 2;
	next;
      }
      $retries = 0;
      $progress = undef;
      my $avail;
      $fileServer = "";
      $errorFile = "";
      $error = "NOERROR";
      my @responseArray = split(/\n/, $response);

      foreach (@responseArray) {
	if (/OSWP\-\d:DLPROGRESS,([\w-]+)/) {
	  $progress = $1;
	}
	if (/OSWP\-\d:DLERROR,([\w-]+)/) {
	  $error = $1
	}
	if (/OSWP\-\d:ERRFILE,(.*)/) {
	  $errorFile = $1;
	  $errorFile =~ s/\s//g;
          $errorFile =~ s/\"//g;
	}
	if (/OSWP\-\d:SERVID,(\d+\.\d+\.\d+\.\d+)/) {
	  $fileServer = $1;
	}
	if (/^\d\d.\d\d.\d\d\s\d\d.\d\d.\d\d.*/) {
	  # Alarm received, continue
	  $progress = "DOWNLOADING";
	  $error = "NOERROR";
	}
	if (/OSWP\-\d:AVAILCOMPSET,([\w-]+)/) {
	  $avail = $1;
	}


      }
      Logger::LogDebug($nodeLogFH, sprintf("progress: %s, error: %s\n", defined($progress) ? $progress : "undef",
				      defined($error) ? $error : "undef"));
      if (defined($progress) && defined($error)) {
	if ($progress eq "ENABLED" && $error eq "NOERROR") {
	  $toReturn = 1;
	  return 1;
	} elsif ($progress eq "DOWNLOADING" &&
		 $error eq "NOERROR") {
      	  $slept = $slept + 7;
	  if($slept > $timeout){
		my $TERROR = " : Failed due to Software Download Timeout. Max time was $timeout Seconds";
		$toReturn = 0;
                my $abortRet=AbortDownloadSWP($session, $oswpIndex);
                if(!$abortRet){
                  $ERROR = $ERROR." ".$TERROR;
                 }else{
                   $ERROR = $TERROR;
                 }
		return 0;
	    }
	  sleep 2;
	} elsif ($progress eq "DISABLED") {
	    $ERROR = "Download failed. Error: $error (failed file: '$errorFile', server: '$fileServer'), availability: $progress";
	    $toReturn = 0;
	    return 0;
	} else {
	  $ERROR = sprintf("Internal error: Unused progress value (%s).\nFull response: %s\n", $progress, $response);
	  $toReturn = 0;
	  return 0;
	}
      } else {
	$ERROR = sprintf("No progress information available.\nFull response: %s\n", $response);
	$toReturn = 0;
	return 0;
      }
    }
  };
  Logger::LogDebug($nodeLogFH, "eval block ended ($evalRet)\n");
  if ($gotSigPipe) {
    Logger::LogDebug($nodeLogFH, "Local sigpipe handler called\n");
    $ERROR = "SIGPIPE (WaitForDownload)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 WaitForNTSync($$)

=begin html
Wait for NT Sync Completion on duplex NT<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>filename</td><td>TAR file name (dm_complete.tar)</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForNTSync($) {
  my ($session) = @_;
  #-- check for completed DB download(or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $error = "NOERROR";
  my $toReturn = undef;
  my $fh = FamilyCommon::GetWarnLogFile();
  my $thisCmdResp = 0;

  #Fix for BDFam50928
  my $timeout =0;
  my $toolConfigData = FamilyCommon::GetToolData();
  my $slept = 0;
  $timeout = $toolConfigData->{NT_SYNC_TIMEOUT} if defined($toolConfigData->{NT_SYNC_TIMEOUT}) ;

   my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = SendCommand($session, sprintf("REPT-OPSTAT-NE::COM::;"));
      if (!defined($response)) {
        Logger::LogDebug($nodeLogFH, sprintf("No response retry ..%d\n",$retries), 1);
        if ($retries > 4) {
          # 5 retries, seems that the first ones may yield no response...
          $ERROR = sprintf("Maximum command retransmissions tried for REPT-OPSTAT-NE;");
          $toReturn = 0;
          return;
        }
        $retries++;
        sleep 2;
        next;
      }
      $retries = 0;
      $error = "NOERROR";
      my @responseArray = split(/\n/,$response);
       $thisCmdResp = 0;
      foreach my $line (@responseArray) {
      if($line =~ /REPT-OPSTAT-NE/){ $thisCmdResp = 1; }
      if($thisCmdResp == 1){
      if (($line =~ /NTBSTATUS/) || ($line =~ /NTASTATUS/)){
      	if($line =~ /IDLE/ || $line =~ /COLDSTBY/){
      		Logger::LogDebug($nodeLogFH, "NT Sync in Progress\n", 1);
                sleep 25;
      		}
          if (($line =~ /HOTSTBY/)){
          	Logger::LogDebug($nodeLogFH, " NT is in Sync \n", 1);
          	$completed = 1;
          	$toReturn = 1;
            }
         }
       }
      }

      $slept = $slept + 30;
    if($slept > $timeout){
     my $TERROR = " : Failed due to NT Sync Timeout. Max time was $timeout Seconds";
     Logger::LogDebug($nodeLogFH, sprintf("Failed due to NT Sync Timeout. Max time was %s Seconds\n",$timeout));
     $toReturn = 0;
     return $toReturn;
    }

    }
  };
  Logger::LogDebug($nodeLogFH, "eval block ended ($evalRet)\n");
  if ($gotSigPipe) {
    Logger::LogDebug($nodeLogFH, "Local sigpipe handler called\n");
    $ERROR = "SIGPIPE (WaitForNTSync)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 ActivateSWP($$)

=begin html
Activate OSWP with index $swpIndex using the opened TL1 session $session.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub ActivateSWP($$) {
  my ($session, $swpIndex) = @_;
  my $response = SendCommand($session, sprintf("ed-oswp::oswp-%d::::ACTIVATION=WITHLINKEDDB,COMMITMENT=,ABORT=;", $swpIndex->{PASSIVE}{INDEX}));

  return defined($response) ? 1 : 0;
}



=head2 CommitSWP($$)

=begin html
Commit OSWP with index $swpIndex using the opened TL! session $session.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub CommitSWP($$) {
  my ($session, $swpIndex) = @_;
  my $response = SendCommand($session, sprintf("ed-oswp::oswp-%d::::ACTIVATION=,COMMITMENT=true,ABORT=;", $swpIndex));

  return defined($response) ? 1 : 0;
}

=head2 AbortDownloadSWP($$)

=begin html
Abort download of OSWP with index $swpIndex using the opened TL1 session $session.  Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub AbortDownloadSWP($$) {
  my ($session, $swpIndex) = @_;
  my $response = SendCommand($session, sprintf("ed-oswp::oswp-%d::::ACTIVATION=,COMMITMENT=,ABORT=true;", $swpIndex));

  return defined($response) ? 1 : 0;
}

=head2 DownloadSWP($$$$$$)

=begin html
Download OSWP and wait for completion.<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>swpIndex</td><td>OSWP index (1 or 2)</td></tr>
  <tr><td>oswpName</td><td>name of the OSWP</td></tr>
  <tr><td>primFtpIp</td><td>IP address of primary filetransfer server</td</tr>
  <tr><td>secFtpIp</td><td>IP address of primary filetransfer server</td</tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub DownloadSWP($$$$$$) {
  my ($session, $swpIndex, $oswpName, $primFtpIp, $secFtpIp, $timeout) = @_;
#  $oswpName =~ s/^\/// if ($oswpName =~ /^\//);
  my $response = SendCommand($session,
			     sprintf("dl-oswp:::::%s,%s,%s;", $primFtpIp, $secFtpIp, $oswpName));

  return 0 if (! defined($response));
  sleep 2;
  Transaction($session, "release");  # 1.2.5
  sleep 2;
  return WaitForDownload($session, $swpIndex, $timeout);
}

=head2 WaitForBackupDB($$)

=begin html
Wait for DB download completion<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>fileName</td><td>TAR file name (dm_complete.tar)</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForBackupDB($$) {
  my ($session, $fileName) = @_;
  #-- check for completed DB download(or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $error = "NOERROR";
  my $toReturn = undef;
  my $fh = FamilyCommon::GetWarnLogFile();
  my $sfname="dm_complete.tar";

  #Fix for BDFam54385
  my $toolConfigData = FamilyCommon::GetToolData();

  if ($toolConfigData->{NOTFTP}) {
   $fileName = $toolConfigData->{FTPDIR} . "/" . $fileName;
  }

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = SendCommand($session, sprintf("rept-opstat-dbul::::;"));
      if (!defined($response)) {
       Logger::LogDebug($nodeLogFH, sprintf("No response retry ..%d\n",$retries), 1);
        if ($retries > 4) {
          # 5 retries, seems that the first ones may yield no response...
          $ERROR = sprintf("Maximum command retransmissions tried for rept-opstat-dbul::::");
          $toReturn = 0;
          return;
        }
        $retries++;
        sleep 2;
        next;
      }
      $retries = 0;
      $error = "NOERROR";
      Logger::LogDebug($nodeLogFH, sprintf("TL1 command response as follows.\n %s\n",$response));
      my @responseArray = split(/\n/,$response);
      my $line=undef;

       $line=join("", @responseArray);
       $line=~s/\s+//g;

         if ($line =~ /$sfname/){
          if ($line =~ /ONGOING/){
            Logger::LogDebug($nodeLogFH, sprintf("%s DB backup is in progress \n",$fileName), 1);
            sleep 5;
            next;
          }elsif ($line =~/SUCCESS/){
            Logger::LogDebug($nodeLogFH, sprintf("%s DB backup completed sucessfully\n",$fileName), 1);
            $toReturn = 1;
          }elsif ($line =~/FAIL/){
            Logger::LogDebug($nodeLogFH, sprintf("%s DB backup Failed\n",$fileName), 1);
            $toReturn = 0;
          } else {
            Logger::LogDebug($nodeLogFH, sprintf("%s DB backup Failed\n",$fileName), 1);
            $toReturn = 0;
          }
          $completed = 1;
        }
    }
  };
  Logger::LogDebug($nodeLogFH, "eval block ended ($evalRet)\n");
  if ($gotSigPipe) {
    Logger::LogDebug($nodeLogFH, "Local sigpipe handler called\n");
    $ERROR = "SIGPIPE (WaitForDownload)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 GetActiveDBVer($$)

=begin html
Get Active DB Version from  NE<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>

</table>
Returns DB Version String in case of success or undef in case of failure.

=end html

=cut

sub GetActiveDBVer($) {
  my ($session) = @_;
  #-- check for completed DB download(or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $error = "NOERROR";
  my $toReturn = undef;
  my $fh = FamilyCommon::GetWarnLogFile();
  my @dbVer;

  while (!$completed) {
      $response = SendCommand($session, sprintf("rept-opstat-db::all:;"));
      if (!defined($response)) {
        Logger::LogDebug($nodeLogFH, sprintf("No response retry ..%d\n",$retries), 1);
        if ($retries > 4) {
          # 5 retries, seems that the first ones may yield no response...
          $ERROR = sprintf("Maximum command retransmissions tried for rept-opstat-dbul::::");
          $toReturn = 0;
          return;
        }
        $retries++;
        sleep 2;
        next;
      }
      $retries = 0;
      $error = "NOERROR";
      Logger::LogDebug($nodeLogFH, sprintf("TL1 command response as follows.\n %s\n",$response));
      my @responseArray = split(/\n/,$response);
      my $prevLine="";
      foreach my $line (@responseArray) {
       #  if ($line =~ /ACTUAL/ ){
       if (($line =~ /AOPERSTAT,ACTUAL/) && !($line =~ /NAOPERSTAT,ACTUAL/)){
             chop($prevLine);
             @dbVer=split(/,/, $prevLine);
             $completed=1;
             last;
          }
          $prevLine = $line;
    }
  };

	if(defined($dbVer[1])){
		return $dbVer[1];
	}else{
		return undef;
	}

}

=head2 GetPassiveDBVer($$)

=begin html
Get passive DB Version from NE<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>

</table>
Returns DB Version String in case of success or undef in case of failure.

=end html

=cut

sub GetPassiveDBVer($) {
  my ($session) = @_;
  #-- check for completed DB download(or error)
  my $response;
  my @db;
  my $passVer = undef;
  my $dbVer= undef;

     $response = SendCommand($session, sprintf("rept-opstat-db::all;"));
      if (!defined($response)) {
      return undef;
      }

      my @responseArray = split(/\n/,$response);
      foreach my $line (@responseArray) {
       if ($line =~/DBVERSION/ ){
          @db = split(/,/,$line);
          $dbVer = substr($db[1],1,5);
          $dbVer =~ s/\.//;
       }
       if ($line =~ /NAOPERSTAT,ACTUAL/){
         $passVer = $dbVer if defined($dbVer);
       }
     }
     return $passVer;
}

=head2 WaitForUploadDB($$)

=begin html
Wait for DB download completion<br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>fileName</td><td>TAR file name (dm_complete.tar)</td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub WaitForUploadDB($$) {
  my ($session, $fileName) = @_;
  #-- check for completed DB download(or error)
  my $completed = 0;
  my $retries = 0;
  my $gotSigPipe = 0;
  my $response;
  my $error = "NOERROR";
  my $toReturn = undef;
  my $fh = FamilyCommon::GetWarnLogFile();
  my $sfname="dm_complete.tar";

  #Fix for BDFam54385
  my $toolConfigData = FamilyCommon::GetToolData();

  if ($toolConfigData->{NOTFTP}) {
   $fileName = $toolConfigData->{FTPDIR} . "/" . $fileName;
  }

  my $evalRet = eval {
    local $SIG{'PIPE'} = sub { $gotSigPipe = 1; };
    while (!$completed) {
      $response = SendCommand($session, sprintf("rept-opstat-dbdl::::;"));
      if (!defined($response)) {
        Logger::LogDebug($nodeLogFH, sprintf("No response retry ..%d\n",$retries), 1);
        if ($retries > 4) {
          # 5 retries, seems that the first ones may yield no response...
          $ERROR = sprintf("Maximum command retransmissions tried for rept-opstat-dbul::::");
          $toReturn = 0;
          return;
        }
        $retries++;
        sleep 2;
        next;
      }
      $retries = 0;
      $error = "NOERROR";
      Logger::LogDebug($nodeLogFH, sprintf("TL1 command response as follows.\n %s\n",$response));
      my @responseArray = split(/\n/,$response);
      #my $prevLine="";
      my $line=undef;

       $line=join("", @responseArray);
       $line=~s/\s+//g;

        if ($line =~ /$sfname/){
          if ($line =~ /ONGOING/){
            Logger::LogDebug($nodeLogFH, sprintf("%s DB Restore to NE is in progress \n",$fileName), 1);
            sleep 5;
            next;
          }
          elsif ($line =~ /SUCCESS/){
            Logger::LogDebug($nodeLogFH, sprintf("%s DB Restore to NE completed sucessfully\n",$fileName), 1);
            $toReturn = 1;
          }elsif ($line =~ /FAIL/){
            Logger::LogDebug($nodeLogFH, sprintf("%s DB Restore to NE  Failed\n",$fileName), 1);
            $toReturn = 0;
          } else {
            Logger::LogDebug($nodeLogFH, sprintf("%s DB Restore Failed\n",$fileName), 1);
            $toReturn = 0;
          }
          $completed = 1;
        }

    }
  };
  Logger::LogDebug($nodeLogFH, "eval block ended ($evalRet)\n");
  if ($gotSigPipe) {
    Logger::LogDebug($nodeLogFH, "Local sigpipe handler called\n") ;
    $ERROR = "SIGPIPE (WaitForUpload)";
    kill 13, $$;
    $toReturn = 0;
  }
  return $toReturn;
}

=head2 backupDB($$)

=begin html
Backup the Active database of the Network Element. <br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>primTftpIp</td><td>Primary TFTP Server IP </td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub backupDB($$$) {
  my ($session, $primTftpIp,$isam) = @_;

  my $fileName = sprintf("%s/dm_complete.tar",$isam);
  my $dbtype = "AOSWPACTUAL";

  my $response = SendCommand($session, sprintf("ul-db:::::%s,%s,%s;", $primTftpIp, $fileName, $dbtype));

  return 0 if (! defined($response));
  Logger::LogDebug($nodeLogFH, sprintf("TL1 command response as follows.\n %s\n",$response));
  sleep 2;
  Logger::LogDebug($nodeLogFH, sprintf("%s DB backup started.\n",$fileName),1);
  return WaitForBackupDB($session,$fileName);
}


=head2 backupDB_New($$$$$)

=begin html
Backup the Active database of the Network Element. <br>
The following parameters are used:
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>primTftpIp</td><td>Primary TFTP Server IP </td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut

sub backupDB_New($$$$$) {

  my ($session, $primTftpIp, $isam, $bckDBName, $tftpYN) = @_;

  my $filename = undef;

  my $dbtype = "AOSWPACTUAL";

  my $response = SendCommand($session, sprintf("ul-db:::::%s,%s,%s;", $primTftpIp, $bckDBName, $dbtype));

  return 0 if (! defined($response));

  sleep 2;

  return WaitForBackupDB($session,$bckDBName);
}

=head2 RestoreDB($$$$$)

=begin html
Upload the Active database of the Network Element. <br>
to the passive bank.
<table>
  <tr><td>session</td><td>TL1 session context</td></tr>
  <tr><td>primTftpIp</td><td>Primary TFTP Server IP </td></tr>
</table>
Returns 1 in case of success or 0 in case of failure.

=end html

=cut


sub RestoreDB($$$$$) {

    my ($session, $primTftpIp, $isam, $uploadFileName,$ftpWorkDir, $noTFTP) = @_;

    my $filename = undef;


 #   if($noTFTP){
 #         $fileName = "$resultDir/$isam/ICReady/dm_complete.tar";
 #    }else{
 #      	$fileName =  "$ftpWorkDir/$isam/ICReady/dm_complete.tar";
 #   }

  my $response = SendCommand($session, sprintf("dl-db:::::%s,%s;", $primTftpIp, $uploadFileName));

  return 0 if (! defined($response));

  sleep 2;

  return WaitForUploadDB($session,$uploadFileName);

}#End of restore DB



sub RetriveOntSwCtrl($) {

  my ($session ) = @_;

  my $response = SendCommand($session, sprintf("RTRV-ONTSWCTRL::ALL:;"));

  # Sample response of RTRV-ONTSWCTRL::ALL
  # "1::HWVER=3FE51296AAAA01,VARNT=\"SIP\",PLNDSWVER=3FE50854DFVA12,
  # DLDSWVER=3FE50854DFVA12,LASTUPDATETIME=\"70-01-08,01:20:06\",
  # PREVPLNDSWVER=UNPLANNED,NUMREFONTS=1"

  if (defined ($response)) {

  my $current_ont;
  my $full_ont;
  my %ontSwCtrl;


   foreach my $line (split(/\r?\n/, $response)) {
	if( $line =~ /\"((\d+)+)::/ ) {
	    $current_ont = $1;
	    $full_ont = $line;
	} elsif(defined $current_ont) {
	    $line =~ s/^\s+//;
	    $full_ont .= $line;
	}

	if($line =~ /\"$/) {
	  $full_ont =~ s/^\=\\"(.*)\\\"$//g;
	  $full_ont =~ s/^\s*\"?//; # space and quote at beginning
	  $full_ont =~ s/\"?\s*$//; # quota and space at the end
	  $full_ont =~ s/\\"\\"//g;

	  my($ontswctrl_id, $ontswctrl_blk) = split(/::/, $full_ont);
	  my (@var_value) = $ontswctrl_blk =~ /[:,]?(\w+)=(\\\".*?\\\"|[^,:]*)/g;

		  # remove quotes around values
         for(my $i=0; $i < @var_value; $i++) {
	      $var_value[$i] =~ s/^\\"(.*)\\"/$1/;
         }

         $var_value[3] = "NULL" if ($var_value[3] eq "");

         $ontSwCtrl{$var_value[1]}{$var_value[3]}{'id'}=$ontswctrl_id;
         for(my $i=4; $i < @var_value; $i=$i+2) {
	   my $val = $i + 1;
	   $ontSwCtrl{$var_value[1]}{$var_value[3]}{$var_value[$i]}= $var_value[$val];
         }
      }
  }

    return (\%ontSwCtrl);
  } else {
    return 0;
  }
}

sub RetriveOntAid($$) {
  my ($session, $ctrl_id) = @_;

  my $cmd;
  my $current_ont;
  my $full_ont;
  my $ontAid = [];


  $cmd = "REPT-OPSTAT-ONTSWCTRL::$ctrl_id:;";

  my $response = SendCommand($session, sprintf("$cmd"));

  if (defined ($response)) {


	foreach my $line (split(/\r?\n/, $response)) {
	if( $line =~ /\"((\d+)+)::/ ) {
	    $current_ont = $1;
	    $full_ont = $line;
	} elsif(defined $current_ont) {
	    $line =~ s/^\s+//;
	    $full_ont .= $line;
	}

	if($line =~ /\"$/) {
	  $full_ont =~ s/^\=\\"(.*)\\\"$//g;
	  $full_ont =~ s/^\s*\"?//; # space and quote at beginning
	  $full_ont =~ s/\"?\s*$//; # quota and space at the end
	  $full_ont =~ s/\\"\\"//g;
	  my($ontswctrl_id, $empty, $empty1, $ont_aid) = split(/:/, $full_ont);

	  push (@{$ontAid}, $ont_aid);
      }
   }
   return $ontAid;
  }
  else {
    return undef $ontAid;
  }


}

sub getONTSWver($$){
  my ($session, $isam) = @_;
  my $count=0;

  my $response = SendCommand($session,"rtrv-ontswver::COM:::all");

  return undef if (! defined($response));

  # parse the response

  my %swinfo;

  my @responseArray = split(/\n/,$response);
  foreach my $line (@responseArray) {
    #Commented for BDFam53905 if ($line =~ /\s+\"COM::SWVER=\\\"(\w+)\\\",SWVERSIZE=(\d+)\"/){

    if ($line =~ /\s+\"COM::SWVER=\\\"(.*)\\\",SWVERSIZE=(\d+)\"/){
      $swinfo{$1} = $2;
      $count = 1;
    }
  }

  if (! $count){
    return 1;                                                         # BDFam52357
  }

  return \%swinfo;
}

#Function added for BDFam53328
sub getONTSWctrl($$){                                                 # BDFam53328
  my ($session, $isam) = @_;
  my $count=0;

  my $response = SendCommand($session,"rtrv-ONTSWCTRL::all:::;");

  return undef if (! defined($response));

  # parse the response

  my %swctrl;

  $response =~ s/\s*//g;
  my @responseArray = split(/[,]/,$response);
  foreach my $line ( @responseArray )
     {
     if ($line =~ /[\sA-Z]+SWVER=(\w+)/ && !($line =~ /UNPLANNED/)  && !($line =~ /DISABLED/) )
        {
        $swctrl{$1} = $1;
        $count = 1;
        }
     }

  if (! $count)
     {
     return 1;
     }

  return \%swctrl;

}

sub removeONTSWP($$) {
  my ($session,$ontSwp) = @_;
  my $response = SendCommand( $session,sprintf("dlt-ontswver::COM:::%s",$ontSwp) );
  return 0 if (! defined($response));

  return 1;
}

sub getONT ($) {
  my ($session) = @_;
  my $response = undef;

  $response = SendCommand($session,"RTRV-ONT::ALL:;");

  return $response;
}

=head2 getEQPT($)

=begin html
Retrive the EQPT details of the Network Element. <br>
The following parameters are used:
<table>
  <tr><td>tl1session</td><td>TL1 session context</td></tr>
</table>
Returns $result in case of success or 0 in case of failure.

=end html

=cut

sub getEQPT ($) {
  my ($session) = @_;
  my $response = undef;
  my $result = undef;

  $response = SendCommand($session,"RTRV-EQPT::ALL:;");

  if (!defined ($response)) {
    return $response;
  }


  foreach my $line (split(/\r?\n/, $response)) {

     $line =~ s/^\s*\"?//; # space and quote at beginning
     $line =~ s/\"?\s*$//; # quota and space at the end
    # $line =~ s/,/:/g;     # replace , with : as seperator to make uniform

    if( $line =~ /^(NT)/ || $line =~ /^(ACU)/  || $line =~ /^(SHELF)/ || $line =~ /^(LT)/ ) {

    my($aid_ont, $str, $eqpt_nblk, $end) = split(/:/, $line);
    my ($eqpt_type, $cur_type) = split(/,/, $str);
    my ($state, $sst) = split(/,/, $end);
    $eqpt_nblk =~s /,/:/g;   #To replace the nblk for LT card  LT-1-1-2:EQPT_TYPE=GLT2-C:CUR_TYPE=EMPTY:SWOVERRULE=NOOVERRULE,LSMPWR=UP:STATE=OOS-AU:SST=UEQ

    $result .= $aid_ont . ":EQPT_TYPE=" . $eqpt_type .  ":CUR_TYPE=". $cur_type . ":" . $eqpt_nblk . ":STATE=" . $state . ":SST=" . $sst . "\n" if (length($sst) > 0);
    $result .= $aid_ont . ":EQPT_TYPE=" . $eqpt_type .  ":CUR_TYPE=". $cur_type . ":" . $eqpt_nblk . ":STATE=" . $state .  "\n" if (length($sst) eq 0);

    # $result .= $x . "STATE=" . $state . "\n";
    }


  }


  return $result;
}

sub getNEopstat($) {
  my ($session) = @_;
  my $count=0;

  my $response = SendCommand($session,"rept-opstat-ne::COM::;");

  return undef if (! defined($response));

  # parse the response

  my %neinfo;

  my @responseArray = split(/\n/,$response);
  foreach my $line (@responseArray) {
    if ($line =~ /\s+\"COM:(\w+),(\w+)"/){
      $neinfo{$1} = $2;
      $count = 1;
    }
  }

  if (! $count){
    return undef;
  }

  return \%neinfo;
}

sub getPrecheckAlarm($) {
  my ($session) = @_;
  my $count=0;
  my $full_ont = undef;

  my %alarms;

  my $response = SendCommand($session,"rtrv-ne-all::com::;");

  return undef if (! defined($response));


  foreach my $line (split(/\r?\n/, $response)) {
        if( $line =~ /\"COM::/ ) {
          $count = 1;
          $full_ont = $line;
        } elsif ($count)  {
          $full_ont .= $line;

          if($line =~ /\"$/) {
            $count = 0;
          }
        }
  }

  $full_ont =~ s/^\s*\"?//; # space and quote at beginning
  $full_ont =~ s/\"?\s*$//; # quota and space at the end

  my($x, $y, $ont_blk) = split(/:/, $full_ont);

  my (@var_value) = $ont_blk =~ /[:,]?(\w+)=(\\\".*?\\\"|[^,:]*)/g;

  # remove quotes around values
  for(my $i=0; $i < @var_value; $i++) {
      $var_value[$i] =~ s/^\\\"(.*)\\\"$/$1/;
  }

  if(@var_value % 2 != 0) {
      return undef;
  }

  for(my $i=0; $i < @var_value; $i = $i + 2) {
      $alarms{$var_value[$i]} = $var_value[$i+1];
  }

  return \%alarms;
}

sub AddONTSWCTRL($$$$$$){
  my ($session, $hw, $var, $mode, $sw, $actsw) = @_;
  my $response = undef;;
  #$response = SendCommand($session, sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s,DLDSWVER=%s;",
  #                                 $hw,$var,$sw)) if ($mode == 1);

  if ($sw eq "UNPLANNED") {
  $response = SendCommand($session,
                          sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s;",
                          $hw,$var)) if ($mode == 2);
  $response = SendCommand($session,
                          sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s;",
                          $hw,$var)) if ($mode == 3 || $mode ==1);
  } else {
  $response = SendCommand($session,
                          sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s,PLNDSWVER=%s,PLNDSWVERCONF=%s,DLDSWVER=%s;",
                          $hw,$var,$actsw,$actsw,$sw)) if ($mode ==1);
  $response = SendCommand($session,
                          sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s,PLNDSWVER=%s,PLNDSWVERCONF=%s;",
                          $hw,$var,$sw,$sw)) if ($mode == 2);
  $response = SendCommand($session,
                          sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s,PLNDSWVER=%s,PLNDSWVERCONF=%s,DLDSWVER=%s;",
                          $hw,$var,$sw,$sw,$sw)) if ($mode == 3);
  $response = SendCommand($session,
                          sprintf("ENT-ONTSWCTRL::::::HWVER=%s,VARNT=%s,PLNDSWVER=%s,PLNDSWVERCONF=%s;",
                          $hw,$var,"DELAYACTIVATE","DELAYACTIVATE")) if ($mode == 4);   ### TODO :check val for PLNDSWVERCONF
  }
  return $response;
}



#This function is added for the RCR OSWP Speed up Feature. BDFam
=head2 ReteriveActiveSwFileList($$)

=begin html
Retrive the list of file in the Network Element. <br>
The following parameters are used:
<table>
  <tr><td>tl1session</td><td>TL1 session context</td></tr>
  <tr><td>Index</td><td>OSWP index</td></tr>
</table>
Returns $filesInNe in case of success or undef in case of failure.

=end html

=cut

sub RetriveActiveSwFileList($$) {
  my ($session, $index) = @_;

  my $i = 0;
  my $cnt = 0;
  my %filesInNe;

  my $response = SendCommand($session, sprintf("RTRV-MEM::OSWP-%s:;", $index));


  foreach my $line (split(/\r?\n/, $response)) {
     $line =~ s/^\s*\"?//; # space and quote at beginning
     $line =~ s/\"?\s*$//; # quota and space at the end

	if( $line =~ /^(OSWP-)/ ) {
		$i++;

    	my($tmp, $tmp1, $file_name, $end) = split(/,/, $line);

		$filesInNe{$file_name} = $file_name;
		$cnt =1;
	}
  }

  if (! $cnt){
    return undef;
  }


  return \%filesInNe;

 }

sub setTL1Handler($$) {
  my ($hndl, $isam) = @_;
  $gTl1Hndlr = $hndl;
  $gisam = $isam;
}

sub getTL1Hanlder() {
  return $gTl1Hndlr;
}

sub doLoginAgain() {
  my $toolData = FamilyCommon::GetToolData();

  my $ret = Connect($gisam, $toolData->{TL1UDPPORT}, $toolData->{TL1USER}, $toolData->{TL1PWD},undef);
  return undef if (! $ret);

  Transaction($ret, 'acquire');
  InhibitAlarms($ret);

  return 1;
}

#Fix for BDFam56265
sub RtrvTimeOut($) {
 my ($session) = @_;
 my $full_ont = undef;
 my $count =0;
 my $Tmout;
 my $authSuper; 
 my $authDef; 
 my $soakMode; 
 my $gponOltRngMode;
 my $response = SendCommand($session, "rtrv-ne-all::COM::;");

 if (defined($response))
 {
   ($Tmout, $authSuper, $authDef, $soakMode, $gponOltRngMode) = parse_timeout($response,"COM");
   if (defined($Tmout)) {
     $gCmdssntmo = $Tmout;
   }
   if (defined($authSuper)) {
     $gAuthSuper= $authSuper;
   }
   if (defined($authDef)) {
     $gAuthDef = $authDef;
   }
   if (defined($soakMode)) {
     $gSoakMode = $soakMode;
   }
   if (defined($gponOltRngMode)) {
     $gGponOltRngMode = $gponOltRngMode;
   }
 }

 $response = SendCommand($session, "rtrv-secu-cid::remote:;");

 if (defined($response))
 {
   ($Tmout, $authSuper, $authDef, $soakMode, $gponOltRngMode) = parse_timeout($response,"REMOTE");
   if (defined($Tmout)) {
     $gRemoteTmout = $Tmout;
   }
 }

 $response = SendCommand($session, "rtrv-secu-cid::craft:;");
 if (defined($response))
 {
   ($Tmout, $authSuper, $authDef, $soakMode, $gponOltRngMode) = parse_timeout($response,"CRAFT");
   if (defined($Tmout)) {
     $gCraftTmout = $Tmout;
   }
 }
}

#Fix for BDFam56265
sub parse_timeout($$) {
 my ($response, $type) = @_;
 my $full_ont = undef;
 my $count =0;
 my $Tmout;
 my $authSuper = undef;
 my $authDef = undef; 
 my $soakMode = undef; 
 my $gponOltRngMode = undef;


  foreach my $line (split(/\r?\n/, $response)) {
    if( $line =~ /\"$type:/ ) {
       $full_ont = $line;
       $count = 1;
    } elsif ($count)  {
      $full_ont .= $line;
       if($line =~ /\"$/) {
         $count = 0;
       }
   }
  }
  if(!defined($full_ont)) {
     return undef;
  }
  $full_ont =~ s/^\s*\"?//; # space and quote at beginning
  $full_ont =~ s/\"?\s*$//; # quota and space at the end

  my @responseArray = split(/[,]/,$full_ont);

  foreach my $line ( @responseArray )
  {
   if ($line =~ /TMOUT=(\w+)/  || $line =~ /CMDSSNTMO=(\w+)/ ) {
    $Tmout = $1;
    ###last;
   } elsif ($line =~ /AUTHSUPER=(\w+)/) { 
    $authSuper = $1; 
   } elsif ($line =~ /AUTHDEF=(\w+)/) { 
    $authDef = $1; 
   } elsif ($line =~ /SOAKMODE=(\w+)/) { 
    $soakMode = $1; 
   } elsif ($line =~ /GPONOLTRNGMODE=(\w+)/) {
	$gponOltRngMode = $1;
   }
  }
  return ($Tmout, $authSuper, $authDef, $soakMode, $gponOltRngMode);
}

sub CheckErrorMib() {
  my $errMsg = ""; 
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $asamErrCode; 
  my $asamErrorDescription; 

  if(defined($snmpSession)) {
    $asamErrCode = ToolCommon::DoSafeGet($snmpSession, ASAMERRORCODE);
    if(!defined($asamErrCode)) {
       $errMsg = "Could not retrieve system value of ASAM Error Code using SNMP.  SNMP correctly configured?"; 
    } else {
       if($asamErrCode eq "") {
          my $version = $snmpSession->version();
          $version++ if ($version < 3);
          $errMsg  = sprintf("ASAM Error Code not retrievable. SNMP configuration mismatch ?. Check SNMP version (locally SNMPv%d is configure) and SNMP username/password (if applicable)", $version);
       } else {
          $asamErrorDescription = ToolCommon::DoSafeGet($snmpSession, ASAMERRORDESCRIPTION);
          if(!defined($asamErrorDescription)) {
             $errMsg = "Could not retrieve system value of ASAM Error Description using SNMP.  SNMP correctly configured?"; 
          } else {
             if($asamErrorDescription eq "") {
                my $version = $snmpSession->version();
                $version++ if ($version < 3);
                $errMsg  = sprintf("ASAM Error Description not retrievable. SNMP configuration mismatch ?. Check SNMP version (locally SNMPv%d is configure) and SNMP username/password (if applicable)", $version);
             } else {
                  $errMsg  = " ASAM Error $asamErrCode : $asamErrorDescription"; 
             }
          }
       }
    }
  }
  return $errMsg; 
}

#-- End of library
1;
