package OntDlAct7302HiCap;

=head1 ONT Specific Module 

=begin html
This library contains the functions dedicated to perform a ONT download, activate operations.

=end html

=cut
#==============================================================================
#-- Copyright     :    All rights reserved. Passing on and copying of this document,
#--                    use and communication Of its contents not permitted without
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- File          :    OntDlAct7302HiCap.pm
#-- Date          :    04.02.2009
#-- Author        :    Palaniappan ramasamy / Suresh Chandrasekaran
#------------------------------------------------------------------------------
#-- Function      :    Script to backup the network element database
#------------------------------------------------------------------------------
#-- Disclaimer    :    This script contains references to proprietary information  
#--                    owned by Alcate-Lucent.
#------------------------------------------------------------------------------
#-- Copyright     :    All rights reserved. Passing on and copying of this document,
#--                    use and communication Of its contents not permitted without
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- Requirements  :
#--                 ANTms36033-029:
#--                 ANTms36033-030:
#--                 ANTms36033-031:
#--                 ANTms36033-032:
#------------------------------------------------------------------------------
#-- History       :
#-- 1.0.1         :2009/02/04 : First Implementation .
#-- 1.0.2         :2009/04/22 : DDTs Tickets resolved
#                                 BDFam48459
#                                 BDFam48398
#                                 BDFam48467
#                                 BDFam48532
#                                 BDFam48398
#-- 1.0.3         :2009/05/12 : DDTs tickets resolved
#                                 BDFam49289
#-- 1.0.4         :2009/05/25 : DDTs tickets resolved
#                                 BDFam49968
#                                 BDFam50319 
#-- 1.0.5         :2009/06/08 : OBR changes
#                                 BDFam50721
#-- 1.0.6         :2009/06/24 : DDTs tickets resolved
#                                 BDFam51205
#                                 BDFam48204 
#                                 BDFam51383
#                                 BDFam51387
#-- 1.0.7         :2009/07/08 : OBR chnages
#                                 BDFam51521
#-- 1.0.8         :2009/08/03 : DDTs tickets resolved
#                                 BDFam51842
#                                 BDFam51219
#                                 BDFam50321
#                                 BDFam51682
#                                 BDFam52012
#-- 1.0.9         :2009/08/21 : DDTs tickets resolved
#                                 BDFam52395
#-- 1.0.10        :2009/08/25 : DDTs tickets resolved
#                                 BDFam53196
#-- 1.0.11        :2009/09/01 : DDTs tickets resolved
#                                 BDFam51844
#                                 BDFam53261
#                                 BDFam53999
#-- 1.0.12        :2009/09/18 : DDTs tickets resolved
#                                 BDFam51905
#-- 1.0.13        :2009/10/01 : DDTs tickets resolved
#                                 BDFam55272
#-- 1.0.14        :2009/10/08 :  DDTs tickets resolved
#                                 BDFam53240
#-- 3.0.00        :2010/04/21 : Converted version to 3.0.00 for converged PBMT
#-- 3.0.01        :2010/05/24 : Converged PBMT Changes
#-- 3.0.02        :2010/10/29 : ALU00818866
#-- 3.0.03        :2012/07/31 : ALU01804049
#-- 3.0.04        :2012/08/13 : ALU01814430
#-- 3.0.05        :2012/11/12 : ALU01821914
#-- 3.0.06        :2013/11/14 : ALU01924625
#-- 3.0.07        :2013/12/07 : introduce additional option DELAYACT
#-- 3.0.08        :2014/03/10 : ALU01955321
#-- 3.0.09        :2014/03/18 : ALU01955037
#-- 3.0.10        :2014/08/30 : ALU02010326
#-- 3.0.11        :2014/09/10 : ALU02010106
#-- 3.0.12        :2014/10/10 : Ont sw download progress parameter setting done
#-- 3.0.13        :2014/11/28 : ALU02027128
#-- 3.0.14        :2015/03/15 : ALU02053378: Additional parameter for TL1::Connect 
#-- 3.0.15        :2015/03/25 : ALU02055812: Multiple SWVERCTLERR/SWVER-alarms generated 
#-- 3.0.16        :2015/08/12 : ALU02100233 and clone ALU02102265
#-- 3.0.17        :2016/03/21 : ALU02175510
#-- 3.0.18        :2016/04/15 : ALU02184944 and ALU02175120
#-- 3.0.19        :2016/07/15 : ALU02237966
#-- 3.0.20        :2017/11/01 : ALU02403385
#-- 3.0.21        :2018/02/12 : ALU02400908
#-- 3.0.22        :2018/05/31 : ALU02471106
#--   3.1.0       :2018/06/20 : ALU02460863 & ALU02471697
#--   3.1.1       :2018/06/20 : hash entry corrected for ONT state
#------------------------------------------------------------------------------

=head1 VERSION

=begin html
Current version: "OntDlAct7302HiCap.pm - 3.1.1 -- 2018/12/06"

=end html

=cut

my $VERSION = "OntDlAct7302HiCap.pm - 3.1.1 -- 2018/12/06";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use FileHandle;
use File::stat;
use Getopt::Long;
use POSIX ":sys_wait_h";
 
#==============================================================================
#== Find the lib dir
use Cwd;
use File::Basename;

BEGIN {
  my $dir = dirname($0);
  my $path;
  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif ($dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl");
}
#==============================================================================

use Net::SNMP;
use IsamTL1;
use ONT7302HiCap;
use Crypt::Lite;
use Utils;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :ONT_ERR);
use ToolCommon;
use PBMTGlobal;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub ProcessONT($);
sub ResetGlobals();
#==============================================================================
#== Constants
#==============================================================================
use constant {
    # download & actiavte mode
    ONT_DL    => 1, # use ONTSWCTRL (ONT DLSW=AUTO )
    ONT_ACT => 2,   # use ONTSWCTRL (ONT SWVERPLND=AUTO)
    ONT_DLACT    => 3,  # use ONTSWCTRL (ONT DLSW=AUTO SWVERPLND=AUTO)
    ONT_DELYACT    => 4  
};

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $STATUSFILE =       "result.txt";
my $toolData;
my $ontSw;   
my $ontSw2;
my $config;
my %mapInfo;
my %fullOnt;
#my %procInfo;
my $warnCount = 0;
my $failCount = 0;
my $warnThreshold = 0;
my $progressFile = undef;
my $failedOntListFile = undef;
my $complete = 0;
my $warnExceeds = 0;                      # BDFam51219
my %group = ();
my %failaid;
my %updated;
my $pcount = 0;
my %ontToMonitor = ();
#my $ontdlfile;
my $totalFail = 0;

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

sub ResetGlobals() {
	$pcount = 0;
	$complete = 0;
	$warnCount = 0;
	$failCount = 0;
	$warnThreshold = 0;
	$warnExceeds = 0;  
	$totalFail = 0;
}

=head2 SetProgressFail($)

=begin html
Set the progress file handler.
<br>
The following parameters are used:
<table>
  <tr><td>handle</td><td>the file handle of the progress file. Progress percentage is written to this file</td></tr>
</table>

=end html

=cut

sub SetProgressFail($){
  my ($handle) = @_;
  $progressFile = $handle;
}

=head2 SetFailontlistFile($)

=begin html
Set the fail onts file handler.
<br>
The following parameters are used:
<table>
  <tr><td>handle</td><td>the file handle of the failed ont file. Failed ONT aids as well as reason for faliure is written to this file</td></tr>
</table>

=end html

=cut

sub SetFailontlistFile($){
  my ($handle) = @_;
  $failedOntListFile = $handle;
}

=head2 getrefonts($$)

=begin html
Will return the onts which are refered by a particular control id in the ont software control table
<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>session handler to the already establlished tl1 session</td></tr>
  <tr><td>ctrlId</td><td>control id for the ont software control table</td></tr>
</table>

=end html

=cut

sub getrefonts {
  my ($tl1Session, $ctrlId) = @_;
  my $response = undef;
  my $list;
  my $current_Ont = undef;

  $response = IsamTL1::SendCommand($tl1Session,sprintf("rept-opstat-ontswctrl::%d:;",$ctrlId));

  return undef if (! $response);

  foreach my $line (split(/\r?\n/, $response)) {
    if( $line =~ /(ONT(-\d+)+)/ ) {
      if ( $current_Ont ) {
        $current_Ont .= ":";
      }
      $current_Ont .= $1;
    }
  }
  return $current_Ont;
}

=head2 getstatus($$$$$)

=begin html
The status of the download or the activate process is obtained using this function<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>session handler to the already establlished tl1 connection</td></tr>
  <tr><td>aid</td><td>aid of the onts for which the status is got</td></tr>
  <tr><td>m</td><td>defines the mode as in download activate or both</td></tr>
  <tr><td>proc</td><td>ont hash</td></tr>
  <tr><td>id</td><td>control id for the particular type of onts</td></tr>
</table>

=end html

=cut

sub getstatus {
  my ($tl1Session, $aid, $m, $proc, $id) = @_;
  my $response = undef;
  my $cmd = undef; 
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  $cmd = sprintf("rept-opstat-ontswdld::%s:;",$aid);
  $response = IsamTL1::SendCommand($tl1Session, $cmd);

  return undef if (! $response);
  my $current_Ont;
  my $full_Ont;

    foreach my $line (split(/\r?\n/, $response)) {
        if( $line =~ /\"(ONT(-\d+)+),:/ ) {
            $current_Ont = $1;
            $full_Ont = $line;
        } elsif(defined $current_Ont) {
            $line =~ s/^\s+//;
            $full_Ont .= $line;
        }

        if($line =~ /\"$/) {
          $full_Ont =~ s/^\s*\"?//; # space and quote at beginning
          $full_Ont =~ s/\"?\s*$//; # quota and space at the end

          # split
          my $res = 0;
          my($aid_Ont, $status) = split(/:/, $full_Ont);
          if ($status =~ /ONTSWACTFAIL/){
            $res = 2;
          } elsif ($status =~ /SWDLFAIL/) {
            $res = 2;
          } elsif ($status =~ /NTLTFAILURE/) {
            $res = 2;
          } elsif ($status =~ /OMCIFAILURE/) {
            $res = 2;
          } elsif ($status =~ /FAIL/) {
            $res = 2;
          } elsif ($status =~ /NODLDFILE/) {
            $res = 2;
          } elsif ($status =~ /NOMATCH/) {
            $res = 2;
          } elsif ($status =~ /NODLD/) {
            $res = 1;
          } elsif ($status =~ /DELAYACTIVATE/) {
            $res = 1;
          }
 
        if ( $ontToMonitor{$current_Ont} != 0) {   #BDFam63689 to avoid counter in -32

        if ( $res == 1) {
           #print STDERR "Current_ont : $current_Ont  Aid:$aid_ont\n";
           $ontToMonitor{$current_Ont} = 0;
           _update(1);
           $pcount = $pcount - 1;
        } elsif ( $res == 2) {
           $failaid{$current_Ont} = "ONT Operation failed";
           ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$current_Ont Reason:$failaid{$current_Ont}\n");
           $ontToMonitor{$current_Ont} = 0;
           _setfail(1);
           $pcount = $pcount - 1;
        }
        }
        $current_Ont = undef;

        }
    }

  return 1;
}

=head2 setmode($$$$$)

=begin html
Set the mode of the onts(DLSW / SWVERPLND) to auto<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>session handler to the already establlished tl1 connection</td></tr>
  <tr><td>aid</td><td>aid of the onts for which the status is got</td></tr>
  <tr><td>m</td><td>defines the mode as in download activate or both</td></tr>
  <tr><td>proc</td><td>ont hash</td></tr>
  <tr><td>id</td><td>control id for the particular type of onts</td></tr>
</table>

=end html

=cut

sub setmode {
  my ($tl1Session, $aid, $m, $proc, $id) = @_;
  my $response = undef;
  my $cmd = undef;
  my $number=0;
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  if ($m == ONT_DL ) {
    $cmd = sprintf("ed-ont::%s::::DLSW=AUTO;",$aid);
  } elsif ($m == ONT_ACT ) {
    $cmd = sprintf("ed-ont::%s::::SWVERPLND=AUTO;",$aid);
  } elsif ($m == ONT_DLACT) {
    $cmd = sprintf("ed-ont::%s::::DLSW=AUTO,SWVERPLND=AUTO;",$aid); 
  } elsif ($m == ONT_DELYACT) {
    $cmd = sprintf("ed-ont::%s::::SWVERPLND=DELAYACTIVATE;",$aid);
  }

  return undef if (! $cmd);

  $response = IsamTL1::SendCommand($tl1Session, $cmd);

  if (!$response) {
     my @ontarray = split(/&/,$aid);
     foreach my $ont (@ontarray) {
        next if defined($failaid{$ont});
        $failaid{$ont} = "Failed to set DLSW to auto";
        ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$ont Reason:$failaid{$ont}\n");
        $number++;
        _setfail(1);
     }
  } elsif ($response =~ /PRTL/) {
     my @responseArray = split(/\n/,$response);
     foreach my $line (@responseArray) {
        if ($line =~ /\"ONT-(\d+)-(\d+)-(\d+)-(\d+)-(\d+):ERRCDE=(\w+)\"/) {
           my $aid = "ONT-$1-$2-$3-$4-$5";
           next if defined($failaid{$aid});
           $failaid{$aid} = "Failed to set DLSW to auto";
           ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
           $number++;
           _setfail(1);
        }
     }
  }
  return $number ;

}

=head2 unsetmode($$$$)

=begin html
Unset the mode of the onts(DLSW / SWVERPLND) to earlier entries<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>session handler to the already establlished tl1 connection</td></tr>
  <tr><td>aid</td><td>aid of the onts for which the status is got</td></tr>
  <tr><td>m</td><td>defines the mode as in download activate or both</td></tr>
  <tr><td>proc</td><td>ont hash</td></tr>
  <tr><td>id</td><td>control id for the particular type of onts</td></tr>
  <tr><td>actSw</td><td>the active software version to which the control table is reset</td></tr>
</table>

=end html

=cut

sub unsetmode {
  my ($tl1Session, $aid, $m, $actSw) = @_;
  my $response = undef;
  my $cmd = undef;
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  if ($m == ONT_DL ) {
    $cmd = sprintf("ed-ont::%s::::DLSW=DISABLED;",$aid);
  } elsif ($m == ONT_ACT ) {
    #$cmd = sprintf("ed-ont::%s::::SWVERPLND=%s;",$aid,$actSw) if (defined($actSw));
    $cmd = sprintf("ed-ont::%s::::DLSW=DISABLED,SWVERPLND=%s;",$aid,$actSw) if (defined($actSw));
  } elsif ($m == ONT_DLACT) {
    $cmd = sprintf("ed-ont::%s::::DLSW=DISABLED,SWVERPLND=%s;",$aid,$actSw) if (defined($actSw));
  }

  return if (! $cmd);

  $response = IsamTL1::SendCommand($tl1Session, $cmd);

  if (!$response) {
     my @ontarray = split(/&/,$aid);
     foreach my $ont (@ontarray) {
        ToolCommon::Log($nodeLogFh,"ONTUnset: AID:$ont Reason:Unable to set mode to manual\n");
     }
  } elsif ($response =~ /PRTL/){
     my @responseArray = split(/\n/,$response);
     foreach my $line (@responseArray) {
        if ($line =~ /\"ONT-(\d+)-(\d+)-(\d+)-(\d+)-(\d+):ERRCDE=(\w+)\"/) {
           my $aid = "ONT-$1-$2-$3-$4-$5";
           ToolCommon::Log($nodeLogFh,"ONTUnset: AID:$aid Reason:Unable to set mode to manual\n");
        }
     }
  }
  return ;

}

=head2 triggertable($$$$)

=begin html
Edit the ont software control table for the particular software<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>session handler to the already establlished tl1 connection</td></tr>
  <tr><td>id</td><td>control id for the particular type of onts</td></tr>
  <tr><td>m</td><td>defines the mode as in download activate or both</td></tr>
  <tr><td>sw</td><td>software which is to be activated or downloaded</td></tr>
</table>

=end html

=cut

sub triggertable {
  my ($tl1Session, $id, $m, $sw) = @_;
  my $response = undef;
  my $cmd = undef;

  if ($m == ONT_DL ) {
    $cmd = sprintf("ed-ontswctrl::%s::::DLDSWVER=%s;",$id,$sw);
    #$cmd = sprintf("ed-ontswctrl::%s::::PLNDSWVER=%s,PLNDSWVERCONF=%s,DLDSWVER=%s;",$id,$sw,$sw,$sw);
  } elsif ($m == ONT_ACT ) {
    $cmd = sprintf("ed-ontswctrl::%s::::PLNDSWVER=%s,PLNDSWVERCONF=%s;",$id,$sw,$sw);
  } elsif ($m == ONT_DLACT) {
    $cmd = sprintf("ed-ontswctrl::%s::::PLNDSWVER=%s,PLNDSWVERCONF=%s,DLDSWVER=%s;",$id,$sw,$sw,$sw);
  } elsif ($m == ONT_DELYACT ) {
    $cmd = sprintf("ed-ontswctrl::%s::::PLNDSWVER=%s,PLNDSWVERCONF=%s;",$id,"DELAYACTIVATE","DELAYACTIVATE");
  }

  return undef if (! $cmd);

  $response = IsamTL1::SendCommand($tl1Session, $cmd);
  return $response;
 
}

=head2 _update($)

=begin html
Update the value written in the progress file according to the completion of processing.<br>
The following parameters are used:
<table>
  <tr><td>count</td><td>amount by which updation is done</td></tr>
</table>

=end html

=cut

sub _update ($) {
  my ($count) = @_;
  return if( $warnCount == 0 );
     
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();
  $complete  = $complete + $count;
  my $p = ($complete / $warnCount) * 100;
  seek($progressFile,0,0);                                   # BDFam53999
  $p = POSIX::ceil($p);                                      # BDFam51621
  print $progressFile $p . "% Complete\n";
  $progressFile->autoflush(1);                               # BDFam51387
  ToolCommon::Log($nodeLogFh, "Progress $p % Completed\n");

  return ;
}


=head2 _setfail($)

=begin html
Update the fail count for the onts<br>
The following parameters are used:
<table>
  <tr><td>count</td><td>amount by which updation of failcount is done</td></tr>
</table>

=end html

=cut


sub _setfail ($) {
  my ($count) = @_;
  $totalFail = $totalFail + $count; 
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  $failCount = $failCount + $count;   #BDFam49968
  if ( ( ( $warnCount > 0 ) and ($warnThreshold <= (($failCount / $warnCount) * 100)) )  or
       ( $warnCount == 0 and $warnExceeds == 0 )
     ) {
    ToolCommon::Log($nodeLogFh,"ONT operation failure reaches warning threshold");
    $warnExceeds = 1;                 # BDFam51219
    $warnThreshold  = 1000;
  }
  _update($count) if( $warnCount > 0 );
  return ;
}

=head2 _gotoSetPLNDVAR($$$)

=begin html
Set the planned variant for the onts where the planned variant is not initially set<br>
The following parameters are used:
<table>
  <tr><td>tl1Session</td><td>session handler to the already establlished tl1 connection</td></tr>
  <tr><td>voip</td><td>voip type to which the planned variant is to be set</td></tr>
  <tr><td>onts</td><td>group of onts which are changed</td></tr>
 </table>

=end html

=cut

sub _gotoSetPLNDVAR($$$) {
  my ($tl1Session,$voip, $onts) = @_;
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();
  my $response =IsamTL1::SendCommand($tl1Session,sprintf("ed-ont::%s::::PLNDVAR=%s;",$onts,$voip));
  if(! $response) {
    my @ontArray = split(/&/,$onts);
    foreach my $aid (@ontArray) {
      $failaid{$aid} = "Failed to set planned variant";
      ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
      _setfail(1);
    }
  } elsif ($response =~ /PRTL/){
    my @responseArray = split(/\n/,$response);
    foreach my $line (@responseArray) {
      if ($line =~ /\"ONT-(\d+)-(\d+)-(\d+)-(\d+)-(\d+):ERRCDE=(\w+)\"/) {
        my $aid = "ONT-$1-$2-$3-$4-$5";
        $failaid{$aid} = "Failed to set planned variant";
        ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
        _setfail(1);
      }
    }
  }
}

sub PrepareSetPlndVar($$$) {
      my ($tl1Session, $group, $hw) = @_;
      foreach my $voip ( sort keys %group) {
         my $iCount = 0;
         my $onts = undef;
         my $hwInGroup;
         foreach my $aid ( sort keys %{$group{$voip}} ) {
           $iCount++;
           $hwInGroup = $group{$voip}{$aid}{'HW'};
           if ( $hwInGroup =~ /$hw/) {
            if ($iCount > 31) {
             $onts = $onts . $aid ;
             _gotoSetPLNDVAR($tl1Session,$voip,$onts);
             $iCount = 0;
             $onts = undef;
            } else {
              if ($onts){
               $onts = $onts . $aid . "&";
              } else {
               $onts = $aid . "&";
              }
            }
           }
         }
		 if ($onts) {
			chop($onts) if ($iCount);
			_gotoSetPLNDVAR($tl1Session,$voip,$onts) if ($iCount);
		 }
      }
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 ProcessONT($$$)

=begin html
This function is called to perform SW backup of a node of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
</table>

=end html

=cut

sub ProcessONT($) {
  my ($isam) = @_;  
  my $tl1Session = undef;
  my $ret;
  my $dir = FamilyCommon::GetNeStorage();
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();
  my $nodeStatusLog = FamilyCommon::GetNodeStatusLog(); #for Transaction Logger..
  my $revertMode;
  my $monitor = 1;
  my $onlyUnset = defined($params->{UNSETONLY }) ? 1 : 0;

  my $ontTypeArr;
  my $ont;
  my $key;
  my $ctrl_Id;
  my $varnt;
  my $cnt; 
  my $sw;
  my $ontAid;
  my %SWTransfer= ();
  my $mode;
  my $ontParams = 1;
 
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  $revertMode = $params->{ONTDOWNLOADMODE};
  $warnThreshold = $params->{WARNTHRESHOLD};
  $mode = $params->{OPMODE};

  ToolCommon::Log($nodeLogFh,"INFO: Monitoring status of ONT Download || Activate set to $monitor\n");
  #-- Open TL1 Session 
  ToolCommon::Log($nodeLogFh, "Opening TL1 Session \n");
  
  $tl1Session = IsamTL1::Connect($isam->[0], $toolData->{TL1UDPPORT}, $toolData->{TL1USER}, $toolData->{TL1PWD}, $params->{DEBUG} ? $nodeLogFh : undef, $ontParams);
  if (!defined($tl1Session)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open TL1 session for %s: %s\n", 
                                 FamilyCommon::GetFailedNode(), IsamTL1::GetError()),
                                 EC_OPENTL1_FAILED,undef,
                                 1); 
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "Established TL1 Connection sucessfully \n");
  }
  FamilyCommon::SetTL1Session2Close($tl1Session);  # Store for IncrErrorCount

  #-- STEP 0: Acquire transaction
  ToolCommon::Log($nodeLogFh, "Attempting to Acquire transaction...\n");
  return 0 if( ToolCommon::TransactionReleased() );
  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'acquire')) );

  $ret = IsamTL1::InhibitAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, "Alarms Inhibited..\n");
  } else {
    ToolCommon::Log($nodeLogFh, " Error in Alarm Inhibit!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
			       EC_ISAM_INH_ALARM_FAILED,undef,
			       1);
    return 0;
  }

#Step 1: Get the ONT SW CTRL entries 
  # $ontSw =  IsamTL1::RetriveOntSwCtrl($tl1Session);    
  
  # if (!defined($ontSw))
  # {
  #   FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
  #                                 EC_DL_ACT_RTRV_ONTSWCTRL_FAILED, undef,1);  
	                                         
  #   return 0;     
  # }


#Step 1: Get the existing software for given ONT Type

   my %typeinfo;
   my $ont_List = undef;                                                            # BDFam48467
   my $res = 1;
   my $ontTypes;
   my @typeLists = split(',',$params->{FULLONTTYPES});
   my @typeDetails;
   my $typesInCommandLine = 0;
   my $connected = 0;
   my $notAuto = 0;
   my %ontDLManage=();
   my $Develop = 0;                         #Parameter used for viewing debugging information on the console to aid in development.
   my %ontDLManageSorted = ();
   my $OntWoutEqpt;  #BDFam62885

   print STDERR "PrcoessONT - Types received as command line input :\n"     if($Develop);
   print STDERR @typeLists if($Develop);
   print STDERR "\n"       if($Develop);

   ToolCommon::Log($nodeLogFh, "Get the existing software for given ONT Type\n");
   foreach my $typeList (@typeLists) {
     @typeDetails = split(":",$typeList);
     $fullOnt{uc($typeDetails[0])}{uc($typeDetails[1])} = 1; 
   }

   foreach my $ty ( sort keys %fullOnt) {
     if ( $ontTypes ) {
       $ontTypes .= ":" . $ty;
     } else {
       $ontTypes .= $ty;
     }
     foreach my $vp ( sort keys %{$fullOnt{$ty}} ){
         $typesInCommandLine++;
     }
   }

     $ont_List = ONT7302HiCap::getontlist($tl1Session,"ALL");

     if ((! $ont_List) || ($ont_List == 1) ){                                        # BDFam52428
                                                                                    #BDFam55272 - Changing error to warning
     ### FamilyCommon::IncrWarnCount(" FAILED: Get ONT list returns empty \n",        # BDFam48467
       ###                  EC_DL_ACT_GET_ONT_LIST_FAILED, undef                     # BDFam48467
         ###                );                                                        # BDFam48467
       $warnCount = 0;
      ### return 1;
     } else {
       $warnCount = @{$ont_List};
     }
     ToolCommon::Log($nodeLogFh, "INFO: Total number of ONTs returned by getontlist:$warnCount \n");
     if ( $warnCount == 0 ) {
       ToolCommon::Log($nodeLogFh, "INFO: No ONTs are connected\n");
     } else {
       foreach my $ont ( @{$ont_List} ) {

         print STDERR "Going into ONTLIST loop\n" if($Develop);
         # BDFam51844 begins here
         if (defined ($ont->{'state'}) && ( (uc($ont->{'state'}) eq "OOS-MA")
                                          || (uc($ont->{'state'}) eq "OOS-AUMA")) ) {
            $failaid{$ont->{'aid'}} = "ONT is in $ont->{'state'} state ";
            $warnCount--;
            ToolCommon::Log($nodeLogFh, "Ignore: AID:$ont->{'aid'} Reason:$failaid{$ont->{'aid'}}\n");
            next;
         }
         # BDFam51844 ends here

#Handle the ONT in OOS-AU state for which SWVERPLND is set to 3FEVALUE

       if ($mode == ONT_ACT) {
         if(defined($ont->{'nblk'}{SWVERPLND}) && $ont->{'nblk'}{SWVERPLND} eq "AUTO") {
            $failaid{$ont->{'aid'}} = "Already SWVERPLND is set to AUTO";
            next;
         }
         if (defined($ont->{'nblk'}{SWVERPLND}) && $ont->{'nblk'}{SWVERPLND} ne "AUTO"  && ($ont->{'state'} eq "OOS-AU") && (grep(/$ont->{'nblk'}{MNEMONIC}/, @typeDetails))) {
            push(@{$OntWoutEqpt},$ont->{'aid'});
#           ToolCommon::Log($nodeLogFh, "Process Later: AID:$ont->{'aid'}\n");
            $failaid{$ont->{'aid'}} = "EQPTVERNUM not defined for the ONT Process later to set SWVERPLD=AUTO";
            next;
         }
       }
       #BDFam51521 starts here ..
       if (! defined ($ont->{'nblk'}{EQPTVERNUM})){
         #$failaid{$ont->{'aid'}} = 1;                                  # BDFam51842
         $failaid{$ont->{'aid'}} = "EQPTVERNUM not defined for the ONT";                                  # BDFam51842
         $warnCount--;
	 ToolCommon::Log($nodeLogFh, "Ignore: AID:$ont->{'aid'} Reason: EQPTVERNUM not defined for the ONT\n");
         next;
       }
       #BDFam51521 ends here

       my $hwpartnum =$ont->{'nblk'}{EQPTVERNUM};
       #$hwpartnum = substr($hwpartnum,0,10);
       print STDERR "HW :$hwpartnum" if($Develop);


       if (!defined ($ont->{'nblk'}{MNEMONIC})) {
          $failaid{$ont->{'aid'}} = "Failed to retrieve mnemonic for Hw '$hwpartnum' from mapping file..";    # BDFam51842
          $warnCount--;
          ToolCommon::Log($nodeLogFh, "Ignore: AID:$ont->{'aid'} Reason:$failaid{$ont->{'aid'}}\n");
          next;
       }
       my $mnemonic =  $ont->{'nblk'}{MNEMONIC};
       print STDERR "   MNEM :$mnemonic\n" if($Develop);


      if (!defined ($ont->{'nblk'}{PLNDVAR})) {
          $failaid{$ont->{'aid'}} = "PLNDVAR not defined for the ONT";                                  # BDFam51842
          $warnCount--;
          ToolCommon::Log($nodeLogFh, "Ignore: AID:$ont->{'aid'} Reason:$failaid{$ont->{'aid'}}\n");
          next;
       }
      # my $var = $ont->{'nblk'}{PLNDVAR};
      # print STDERR "  VOIP :$var" if($Develop);

       my $var = undef; 
       if ((defined ($ont->{'nblk'}{PLNDVAR})) && (($ont->{'nblk'}{PLNDVAR} eq "\"\"") || ($ont->{'nblk'}{PLNDVAR} eq ""))) {
         if ( ! defined ($ont->{'nblk'}{VOIP})) {
           $failaid{$ont->{'aid'}} =  "No matching Variant found in mapping file";                       # BDFam51842
           ToolCommon::Log($nodeLogFh,"Ignore: AID:$ont->{'aid'}
                         Reason:No matching variant found in mapping file for Type '$mnemonic' \n");
           $warnCount--;
           next;
         }
         $var = $ont->{'nblk'}{VOIP};
         print STDERR "  VOIP :$var" if($Develop);
       } else {
          $var = $ont->{'nblk'}{PLNDVAR};
          print STDERR "  VOIP :$var" if($Develop);
       } 

       if (!defined ($fullOnt{$mnemonic}{$var}) ) {
          $warnCount--;
          ToolCommon::Log($nodeLogFh, "Ignore: AID:$ont->{'aid'} Reason:Not of type requested\n");
          next;
       } else {
          if ($fullOnt{$mnemonic}{$var} == 1){
             $connected++;
             $fullOnt{$mnemonic}{$var} = 2;
          }
       }

       if ( defined ($fullOnt{$mnemonic}{$var}) && (uc($var) ne uc($ont->{'nblk'}{PLNDVAR})) ) {
         $group{$var}{$ont->{'aid'}}{'type'} = $mnemonic; 
         $group{$var}{$ont->{'aid'}}{'HW'} = $ont->{'nblk'}{EQPTVERNUM};
       }

       if ($onlyUnset){
          if (defined($ont->{nblk}{SWVERPLND}) && (uc($ont->{nblk}{SWVERPLND}) ne "AUTO")){
             $notAuto++;
             ToolCommon::Log($nodeLogFh, "Ignore: AID:$ont->{'aid'} Reason:SWVERPLND is not auto\n");
             next;
           }
       }

       $ontDLManage{$mnemonic}{$var}{$hwpartnum}{$ont->{'aid'}} = 1;
       $ontDLManage{$mnemonic}{$var}{$hwpartnum}{'ACTSW'} = $ont->{'nblk'}{SWVERACT}  if (defined ($ont->{'nblk'}{SWVERACT}));

         if (defined ($typeinfo{$mnemonic}{$var}{'count'}) ) {
            $typeinfo{$mnemonic}{$var}{'count'}++;
         } else {
            $typeinfo{$mnemonic}{$var}{'count'} = 1;
         }

         if (defined ($typeinfo{$mnemonic}{$var}{$hwpartnum}{'count'})) {
            $typeinfo{$mnemonic}{$var}{$hwpartnum}{'count'}++;
         } else {
            $typeinfo{$mnemonic}{$var}{$hwpartnum}{'count'} = 1;
         }
      }



      my $ontsProcess = $warnCount - $notAuto;
      ToolCommon::Log($nodeLogFh, "INFO: Total number of ONTs taken for processing:$ontsProcess\n");

      print STDERR "Warncount at the end of ONT reading loop :$warnCount\n" if($Develop);
      if (!$warnCount) {
          FamilyCommon::IncrWarnCount(" FAILED: Get ONT list returns empty \n",        # BDFam48467
                                      EC_DL_ACT_GET_ONT_LIST_FAILED, undef,           # BDFam48467
                                      );                                               # BDFam48467
          return 0 if ($mode == ONT_ACT);
      }

      foreach my $mnemonic (sort keys %ontDLManage) {
         foreach my $var (sort keys %{$ontDLManage{$mnemonic}} ) {
            foreach my $hw (sort keys %{$ontDLManage{$mnemonic}{$var}} ) {

                ToolCommon::Log($nodeLogFh, "INFO: MNEM:$mnemonic VP:$var HW:$hw N:$typeinfo{$mnemonic}{$var}{$hw}{count}\n");
                my $i = 0;
                foreach my $index (keys %{$ontDLManage{$mnemonic}{$var}{$hw}} ){
                   $ontDLManageSorted{$typeinfo{$mnemonic}{$var}{$hw}{count} }{$mnemonic}{$var}{$hw}{$index} =
                   $ontDLManage{$mnemonic}{$var}{$hw}{$index};
                }
                my $modhw = substr($hw,0,10);
                my $sw = ONT7302HiCap::mnemonic_hw_var2sw($params->{RELEASE}, uc($mnemonic), uc($modhw), uc($var));
                print STDERR "Sw for $hw - $mnemonic - $var : $sw\n" if($Develop);
                if (!$sw) {
                   $ontDLManage{$mnemonic}{$var}{$hw}{'fail'} = "No information in release mapping file";
                   _setfail($typeinfo{$mnemonic}{$var}{$hw}{'count'});
                   ToolCommon::Log($nodeLogFh, "GroupFail: MNEM:$mnemonic VOIP:$var HW:$hw Reason:$ontDLManage{$mnemonic}{$var}{$hw}{'fail'}\n");
                   next;
                }

                $ontDLManage{$mnemonic}{$var}{$hw}{'SW'} = $sw;
          #     if ($mode != ONT_ACT) {
          #        $sw = substr($sw,1);
          #        my $filesize = -s "$toolData->{ONTRELDIR}/$sw";
          #        if ($filesize) {
          #           $ontDLManage{$mnemonic}{$var}{$hw}{'localfilesize'} = $filesize;
          #           #$stageSW = 1;
          #        } else {
          #           $ontDLManage{$mnemonic}{$var}{$hw}{'fail'} = "ONT SWP ($toolData->{ONTRELDIR}/$sw) does not exist";
          #           _setfail($typeinfo{$mnemonic}{$var}{$hw}{'count'});
          #           ToolCommon::Log($nodeLogFh,"GroupFail: MNEM:$mnemonic VOIP:$var HW:$hw Reason:$ontDLManage{$mnemonic}{$var}{$hw}
#{'fail'}\n");
#                     next;
          #        }
          #     }
            }
         }
      }

	 }
  
 my $notConnected = $typesInCommandLine - $connected;
 if($notConnected){
   FamilyCommon::IncrWarnCount("ONT Type List Partially Processed",
                      EC_LIST_PARTLY_PROCD, ": $notConnected ONT(s) given in command line is/are not connected to NE");
   ToolCommon::Log($nodeLogFh, "INFO: $notConnected ONT(s) given in command line is/are not connected to NE\n");
 }

#Step 2: Get the ONT SW CTRL entries 

   ToolCommon::Log($nodeLogFh, "Get the ONT SW CTRL entries\n");

   my $ontToProcess = $warnCount - $totalFail;
   if ($ontToProcess){
      $ontSw =  IsamTL1::RetriveOntSwCtrl($tl1Session);
      if (!defined($ontSw)){
         ToolCommon::Log($nodeLogFh, "INFO: ONT SWCTRL table could NOT be retrieved\n");
         FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                                      EC_DL_ACT_RTRV_ONTSWCTRL_FAILED, undef,1);
         return 0;
       } else {
          ToolCommon::Log($nodeLogFh, "ONT SWCTRL table retrieved sucessfully\n");
       }
   }



#Step 3: Populate the ONTs for given Type 

     ToolCommon::Log($nodeLogFh, "Populate the ONTs for given Type\n");
     my $updCtrl = 0;
     my $updPlndVar = 0;
     my @hw_array=();
     my @arr = ();

     foreach my $count (reverse sort {$a <=> $b} keys %ontDLManageSorted){
       foreach my $mnemonic (sort keys %{$ontDLManageSorted{$count}}){
         foreach my $var (sort keys %{$ontDLManageSorted{$count}{$mnemonic}} ) {
           foreach my $hw (sort keys %{$ontDLManageSorted{$count}{$mnemonic}{$var}} ) {
               next if ( defined ($ontDLManage{$mnemonic}{$var}{$hw}{'fail'}) );
               my $sw = $ontDLManage{$mnemonic}{$var}{$hw}{'SW'};                      #Storing into a variable for ease of use
               my $activeSw = (defined ($ontDLManage{$mnemonic}{$var}{$hw}{'ACTSW'})) ? $ontDLManage{$mnemonic}{$var}{$hw}{'ACTSW'}  : $sw ;  
   #-CTRL table entry------------------------------------------------------------------------------------
               print STDERR "CTRL Table entry creation ->BEGIn\n" if($Develop);


               my $newhw = $hw;
               my $modhw = substr($hw,0,10) . "*";
			   my $modhw1 = substr($hw,0,10);
               my $modhw2 = substr($hw,0,8) . "*";
               if (!defined ($ontSw->{$hw}{$var}) ){
                  if (!defined ($ontSw->{$modhw}{$var}) ){
					 if (!(grep(/$modhw1/, @arr))) {
						push (@arr, $modhw1);
					    if (!defined ($ontSw->{$modhw2}{$var}) ){
						   if (!IsamTL1::AddONTSWCTRL($tl1Session, $modhw, $var, $mode,$sw,$activeSw)) {
                           $ontDLManage{$mnemonic}{$var}{$hw}{'fail'} = "Failed to add ONT SW CTRL table entry";
                           _setfail($typeinfo{$mnemonic}{$var}{$hw}{'count'});
                           ToolCommon::Log($nodeLogFh,"GroupFail: MNEM:$mnemonic VOIP:$var HW:$hw Reason:$ontDLManage{$mnemonic}{$var}{$hw}{'fail'}\n");
                           #print $ontdlfile "GroupFail: MNEM:$mnemonic VOIP:$var HW:$hw Reason:$ontDLManage{$mnemonic}{$var}{$hw}{'fail'}\n";
                           next;
					    }
					   }
                     }
                   }
                     $updated{$hw}{$var} = 1;
                } else {
                     $newhw = $hw;
                }
               if (defined ($ontSw->{$modhw}{$var}) ){
                     $newhw = $modhw;
#                    $updated{$hw}{$var} = 1;
               }
               if (defined ($ontSw->{$modhw2}{$var}) ){
                     $newhw = $modhw2;
               }

               if (!(defined($updated{$newhw}{$var})) ) {
                  $updated{$newhw}{$var} = 1 if (($mode == ONT_DL) &&
                                                defined($ontSw->{$newhw}{$var}{DLDSWVER}) &&
                                                $ontSw->{$newhw}{$var}{DLDSWVER} eq $sw );

                  $updated{$newhw}{$var} = 1 if ($mode == ONT_ACT && (defined($ontSw->{$newhw}{$var}{PLNDSWVER}) &&
                                                $ontSw->{$newhw}{$var}{PLNDSWVER} eq $sw));

                  $updated{$newhw}{$var} = 1 if ($mode == ONT_DLACT  &&
                                                (defined($ontSw->{$newhw}{$var}{PLNDSWVER}) &&
                                                $ontSw->{$newhw}{$var}{PLNDSWVER} eq $sw) &&
                                                (defined($ontSw->{$newhw}{$var}{DLDSWVER}) &&
                                                $ontSw->{$newhw}{$var}{DLDSWVER} eq $sw) );
               }

               if (!(defined($updated{$newhw}{$var})) ) {
                  my $ctrlid = $ontSw->{$newhw}{$var}{"id"};
                  my $st = triggertable ($tl1Session, $ctrlid, $mode, $sw);
                  if (!$st) {
                     $ontDLManage{$mnemonic}{$var}{$hw}{'fail'} = "Failed to edit ONT SW CTRL table entry";
                     _setfail($typeinfo{$mnemonic}{$var}{$hw}{'count'});
                     ToolCommon::Log($nodeLogFh,"GroupFail: MNEM:$mnemonic VOIP:$var HW:$hw Reason:$ontDLManage{$mnemonic}{$var}{$hw}{'fail'}\n");
                     #print $ontdlfile "GroupFail: MNEM:$mnemonic VOIP:$var HW:$hw Reason:$ontDLManage{$mnemonic}{$var}{$hw}{'fail'}\n";
                     next;
                  } else {
                       $updated{$newhw}{$var} = 1; 
                  }
               }
               print STDERR "CTRL Table entry creation ->FINISH\n" if($Develop);

   # Set ONT to auto------------------------------------------------------------------------------------------------------
	       my $icount = 0;
	       my $onts = undef;
	       my $fail =0;
	       foreach my $aid (sort keys %{$ontDLManage{$mnemonic}{$var}{$hw}} ) {
		  next if (! ($aid =~ /(ONT(-\d+)+)/) );
		  next if (defined($failaid{$aid}));
		  my $ont = ONT7302HiCap::aid2info($aid);
		  print "$aid:$ont->{nblk}{DLSW}\n" if($Develop);
                  if(!$updPlndVar) {
                    if (!grep(/\Q$newhw\E/,@hw_array)) {
	              ToolCommon::Log($nodeLogFh,"Setting Planned variant for the ONT\n");
                      push (@hw_array, $newhw);
                      my $hardware = $newhw;  
                      $hardware =~ s/\*$//;
                      PrepareSetPlndVar($tl1Session, \%group,$hardware) if(scalar keys %group);  
                      #$updPlndVar = 1; 
		    }
                  }
		  if ($mode == ONT_DL) {
		     if (defined ($ont->{nblk}{DLSW}) && ($ont->{nblk}{DLSW} eq "AUTO") ){
			# BDFam52012 starts here
			if ((defined($ont->{state}) && $ont->{state} eq "OOS-AU")){     #BDFam61494
			   $ontToMonitor{$aid} = 0;
			   #_setfail(1);
			   #$failaid{$aid} = "ONT is in OOS-AU state"; 
			   #ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
			   next;
			}
			# BDFam52012 ends here

			if ((defined($ont->{nblk}{SWVERPSV}) && $ont->{nblk}{SWVERPSV} eq $sw) || 
			    (defined($ont->{nblk}{SWVERACT}) && $ont->{nblk}{SWVERACT} eq $sw ) ){
			   $ontToMonitor{$aid}=0;
			   _update(1);
			   ToolCommon::Log($nodeLogFh,"ONTReqState: AID:$aid\n");
			   next;
			}

			$ontToMonitor{$aid} = 1;
			$pcount++;
			next;
		     } 
		     #The else loop in the other two cases is altered in this case because we would have to set AUTO always in
		     # ONTDL for ODMT
		  } elsif ($mode == ONT_ACT) {
		     if (defined ($ont->{nblk}{SWVERPLND}) && ($ont->{nblk}{SWVERPLND} eq "AUTO") ) {
			if (defined($ont->{nblk}{SWVERACT}) && ($ont->{nblk}{SWVERACT} eq $sw) && defined($ont->{state}) &&
			   (uc($ont->{'state'}) eq "IS-NR") ){
			   $ontToMonitor{$aid}=0;
			   ToolCommon::Log($nodeLogFh,"ONTReqState: AID:$aid\n");
			   _update(1);
			   next;
			} elsif((defined($ont->{state}) && $ont->{state} eq "OOS-AU")){   #BDFam61494
                           $ontToMonitor{$aid} = 0;
                           #_setfail(1);
                           #$failaid{$aid} = "ONT is in OOS-AU state";
                           #ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
                           next;
                        }

			$ontToMonitor{$aid}=1;
			$pcount++;
			next;
		     } else {
			if (defined($ont->{nblk}{SWVERACT}) && ($ont->{nblk}{SWVERACT} eq $sw) && defined($ont->{state}) &&
			   (uc($ont->{'state'}) eq "IS-NR")){
			   #$ontToMonitor{$aid}=0; - No need to unset
			   _update(1);
			   ToolCommon::Log($nodeLogFh,"ONTReqState: AID:$aid\n");
			   next; 
			}
		     }
		  } elsif ($mode == ONT_DLACT) {
		     #TODO : Check for SWVERPLND since this would itself take care of download in the below loop [Merge into above loop]
		     if (defined ($ont->{nblk}{DLSW}) &&
		         defined ($ont->{nblk}{SWVERPLND}) &&
		         (uc($ont->{nblk}{SWVERPLND}) eq "AUTO") &&
		         (uc($ont->{nblk}{DLSW}) eq "AUTO") ){

			if (defined($ont->{nblk}{SWVERACT}) && ($ont->{nblk}{SWVERACT} eq $sw) && defined($ont->{state}) && 
			   (uc($ont->{'state'}) eq "IS-NR") ){
			   $ontToMonitor{$aid}=0;
			   _update(1);
			   ToolCommon::Log($nodeLogFh,"ONTReqState: AID:$aid\n");
			   next;
			} elsif((defined($ont->{state}) && $ont->{state} eq "OOS-AU")){  #BDFam61494
                           $ontToMonitor{$aid} = 0;
                           #_setfail(1);
                           #$failaid{$aid} = "ONT is in OOS-AU state";
                           #ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
                           next;
                        }

			$ontToMonitor{$aid}=1;
			$pcount++;
			next;
		     } else {
			if (defined($ont->{nblk}{SWVERACT}) && ($ont->{nblk}{SWVERACT} eq $sw) && defined($ont->{state}) &&
			   (uc($ont->{'state'}) eq "IS-NR") ){
			   #$ontToMonitor{$aid}=0;
			   _update(1);
			   ToolCommon::Log($nodeLogFh,"ONTReqState: AID:$aid\n");
			   next; 
			}
		     }
		  }
		  
		  $ontToMonitor{$aid}=1;
		  $pcount++;
		  $icount++;
		  if ($icount > 31) {
		     $onts = $onts . $aid ;
		     $fail =setmode($tl1Session, $onts, $mode);
		     $pcount -= $fail;
		     $icount=0;
		     $onts = undef;
		  } else {
		     if ($onts) { 
			$onts = $onts . $aid . "&";
		     } else {
			$onts = $aid . "&";
		     }
		  }
	       }
               if ($icount){
                  chop($onts);
                  $fail = setmode($tl1Session, $onts, $mode);
                  $pcount -= $fail;
               }


           }    ### 4
         }    ### 3
       }    ### 2
     }     ### 1

  print STDERR "Auto Setting  ->END\n" if($Develop);


##############################################################################################################################
#BDFam62885 Starts Here !!!
#Auto setting for OOS-AU without EQPTVERNUM handled here 

if ($mode == ONT_DL || $mode == ONT_ACT) {
   my $icount =0;
   my $forcemode = ONT_ACT;
   my $aid;
   my $fail;
   my $onts;
   #if (defined($OntWoutEqpt) || ($OntWoutEqpt != 1) ){}
   if (defined($OntWoutEqpt)) {
      ToolCommon::Log($nodeLogFh,"Handle the OOS-AU ONTs!!! Waiting few minutes before processing\n");
      sleep(300) if ($mode == ONT_ACT);
      foreach my $aid ( @{$OntWoutEqpt} ) {
       $icount++;
       if ($icount > 31) {
          $onts = $onts . $aid;
          #$fail = setmode($tl1Session, $onts, $forcemode);
          $fail = setmode($tl1Session, $onts, $mode);
          $icount=0;
          $onts = undef;
       } else {
         if ($onts) {
            $onts = $onts . $aid . "&";
         } else {
           $onts = $aid . "&";
         }
      }
   }

   if ($icount){
     chop($onts);
     $fail = setmode($tl1Session, $onts, $mode);
   }
  }
}

#BDFam62885 Starts Here !!!
##############################################################################################################################
   
#Ignoring those for which setting to auto failed
   foreach my $aid (sort keys %ontToMonitor){
     if (defined($failaid{$aid}) ){
        $ontToMonitor{$aid} = 0;
     } else {
        next if ($ontToMonitor{$aid} == 0);
        my $ont = ONT7302HiCap::aid2info($aid);
        my $mnemonic = $ont->{nblk}{MNEMONIC};
        my $hw = $ont->{nblk}{EQPTVERNUM};
        my $var = $ont->{nblk}{PLNDVAR};
        $sw = $ontDLManage{$mnemonic}{$var}{$hw}{'SW'};
        if (defined($sw)) {
        if ((defined($ont->{state}) && $ont->{state} eq "OOS-AU")){    #BDFam61494
           _setfail(1);
           $ontToMonitor{$aid}=0;
           $pcount--;
           $failaid{$aid} = "ONT is in OOS-AU state";
           ToolCommon::Log($nodeLogFh,"ONTPosFail: AID:$aid Reason:$failaid{$aid}\n");
           next;
        }
        if ($mode == ONT_DL){
           if ((defined($ont->{nblk}{SWVERPSV}) && $ont->{nblk}{SWVERPSV} eq $sw) ||
              (defined($ont->{nblk}{SWVERACT}) && $ont->{nblk}{SWVERACT} eq $sw ) ){
              $ontToMonitor{$aid} = 0;
              $pcount--;
              _update(1);
              ToolCommon::Log($nodeLogFh,"ONTReqState: AID:$aid\n");
           }
        }
        } 
     }
   }

###########################################################################################################################   
#BDFam62885 Starts Here !!!

   #Make sure all the ONT types passed for conneced / not connected are updated with DLDSWVER 
   #This helps when new ont connected it will automatically comes to IS-NR 

#  if ($mode == ONT_ACT || $mode == SUPER_DL) 
  if ($mode == ONT_DL || $mode == ONT_ACT) 
  { 
   ToolCommon::Log($nodeLogFh,"Create SWCTRL ENTRY for the ONT Types passed connected/not connected  \n");
   my $ontSw;
   $ontSw =  IsamTL1::RetriveOntSwCtrl($tl1Session);
   if (!defined($ontSw)){
       ToolCommon::Log($nodeLogFh, "INFO: ONT SWCTRL table could NOT be retrieved\n");
       FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                                   EC_DL_ACT_RTRV_ONTSWCTRL_FAILED, undef,1);
       return 0;
   } else {
     ToolCommon::Log($nodeLogFh, "ONT SWCTRL table retrieved sucessfully\n");
   }

   foreach my $ty ( sort keys %fullOnt) {
      print STDERR "$ty \n" if($Develop);
      foreach my $vp ( sort keys %{$fullOnt{$ty}} ){
       my $hw = ONT7302HiCap::mnemonic_voip2hw($params->{RELEASE}, uc($ty), uc($vp));
       if (defined($hw)) {
         my @hwtmp = split(":",$hw);
         my %tmp;
         print STDERR "$hw \n" if($Develop);
         @tmp{@hwtmp} = ();
         my @hwArr = keys %tmp;
         for (my $i=0; $i<=@hwArr -1 ;$i++)
         {
            my $modHw = $hwArr[$i] . "*";
            my $modHw2 = substr($hwArr[$i],0,8) . "*";
            my $sw = ONT7302HiCap::mnemonic_hw_var2sw($params->{RELEASE},uc($ty),uc($hwArr[$i]),uc($vp));
            my $modSw = substr($sw,1); #ALU01801103
            ## if(defined($SWTransfer{$modSw})) {
            ########
            my $toUpdate = 1;
            foreach my $currHwType ( sort keys %{$ontSw}) {
               foreach my $voipTyp ( sort keys %{$ontSw->{$currHwType}} ) {
                 if(defined($ontSw->{$currHwType}{$voipTyp}) && ($currHwType =~ /$hwArr[$i]/) && ($voipTyp =~ /$vp/)) {
                     $modHw = $currHwType;
                     if (defined($updated{$currHwType}{$voipTyp})) {
                           $toUpdate = 0  if($updated{$currHwType}{$voipTyp});
                           print STDERR "  Matched HW <$currHwType> Voip type <$voipTyp> Update <$toUpdate>\n" if($Develop);
                           ToolCommon::Log($nodeLogFh,"Matched HW <$currHwType> Voip type <$voipTyp> Update <$toUpdate>\n");
                           last; 
                     }
                 }
               }
            }
            #######
            #
            if($toUpdate) {
             if (!defined ($ontSw->{$modHw}{$vp}) ){
              if (!defined ($ontSw->{$modHw2}{$vp}) ){
                if (!IsamTL1::AddONTSWCTRL($tl1Session, $modHw, $vp, $mode,$sw,$sw)) {
                     $ontDLManage{$ty}{$vp}{$hwArr[$i]}{'fail'} = "Failed to add ONT SW CTRL table entry";
                     _setfail(1);
                     ToolCommon::Log($nodeLogFh,"SWCTRL ENTRY Creation failed::HW:$modHw MNEMONIC:$ty VOIP:$vp \n");
                }
              } else {
                 if (!((($mode == ONT_DL) && defined($ontSw->{$modHw2}{$vp}{DLDSWVER}) &&
                      $ontSw->{$modHw2}{$vp}{DLDSWVER} eq $sw) || ($mode == ONT_ACT && (defined($ontSw->{$modHw2}{$vp}{PLNDSWVER}) &&
                      $ontSw->{$modHw2}{$vp}{PLNDSWVER} eq $sw)))){
                         my $ctrlid = $ontSw->{$modHw2}{$vp}{"id"};
                         my $st = triggertable ($tl1Session, $ctrlid, $mode, $sw);
                         if (!$st) {
                           $ontDLManage{$ty}{$vp}{$hwArr[$i]}{'fail'} = "Failed to edit ONT SW CTRL table entry";
                           _setfail(1);
                           ToolCommon::Log($nodeLogFh,"TriggerTable:GroupFail: MNEM:$ty VOIP:$vp HW:$modHw2 SW:$sw ID:$ctrlid\n");
                         }

                  }
              }   
             } else {
                 if (!((($mode == ONT_DL) && defined($ontSw->{$modHw}{$vp}{DLDSWVER}) &&
                      $ontSw->{$modHw}{$vp}{DLDSWVER} eq $sw) || ($mode == ONT_ACT && (defined($ontSw->{$modHw}{$vp}{PLNDSWVER}) &&
                      $ontSw->{$modHw}{$vp}{PLNDSWVER} eq $sw)))){
                         my $ctrlid = $ontSw->{$modHw}{$vp}{"id"};
                         my $st = triggertable ($tl1Session, $ctrlid, $mode, $sw);
                         if (!$st) {
                           $ontDLManage{$ty}{$vp}{$hwArr[$i]}{'fail'} = "Failed to edit ONT SW CTRL table entry";
                           _setfail(1);
                           ToolCommon::Log($nodeLogFh,"TriggerTable:GroupFail: MNEM:$ty VOIP:$vp HW:$modHw SW:$sw ID:$ctrlid\n");
                         }

                  }
             }
            }
          ##}
         }
       }
     }
   } #for loop
  } #mode if
#BDFam62885 Ends Here !!!
############################################################################################################################


   if ($monitor) {                                 #this is added for making monitoring the progress of ONT Download optional
   my $starttime = time();
   my $currtime = time();
   my $difftime = $currtime - $starttime;
   print STDERR "Value before start is :$pcount\n" if($Develop);
   print STDERR "Timeout : $params->{ONTDLACT_TIMEOUT})\n" if($Develop);

   my $maxOntDldProgTimer = 0;
   my $maxOntDldTimeout = $toolData->{ONT_DWLD_PROGRESS_TIMEOUT};   ## Maximum ONT download progress timeout 
   my $prevpcount = $pcount;
   my $resetTime = time();
   my $release;  
   if (uc($params->{RELEASE}) =~ /^[LR][0-9]([1-9])\.[0-9]([0-9])\.[0-9]([0-9])/) {
       $release = $1*1000+$2*100+$3;
    }

   sleep(10); #BDFam63658  
   ToolCommon::Log($nodeLogFh, "Value before start:$pcount... Waiting few minutes before checking the status\n");
   while ($pcount && ($difftime < $params->{ONTDLACT_TIMEOUT})  ) {
      sleep(15);

      my $icount = 0;
      my $onts = undef;
      foreach my $aid (sort keys %ontToMonitor ) {
	 next if (!($aid =~ /(ONT(-\d+)+)/) );
	 next if (defined ($failaid{$aid}) );
	 next if ($ontToMonitor{$aid} == 0); 
	 
	 print STDERR "Aid n monitor loop : $aid\n" if($Develop); 
	 $icount++;
	 if ($icount > 31) {
	    $onts = $onts . $aid ;
	    getstatus ($tl1Session, $onts, $mode);
	    $icount=0;
	    $onts = undef;
	 } else {
	    if ($onts) {
	       $onts = $onts . $aid . "&";
	    } else {
	       $onts = $aid . "&";
	    }
	 }
      }

      chop($onts) if ($icount);
      getstatus ($tl1Session, $onts, $mode) if ($icount);
      $currtime = time();
      $difftime = $currtime - $starttime;
     
      if($release >= 5101) {    ### timer only for release above R05.01.01 and above
         $maxOntDldProgTimer = time() - $resetTime;
          if($maxOntDldProgTimer > $maxOntDldTimeout) {
                     last if($prevpcount == $pcount);
          }
          if($prevpcount > $pcount) {
              $prevpcount = $pcount;
              #$maxOntDldProgTimer= 0;
              $resetTime = time();
          }
      } 

      ToolCommon::Log($nodeLogFh, "ONTs that are in monitor loop:$pcount...\n");
      print STDERR "ONTs that are in monitor loop :$pcount\n" if($Develop); 

   }

#  if ($pcount) {
     foreach my $ont (sort keys %ontToMonitor){
        if ($ontToMonitor{$ont} == 1){
           $failaid{$ont} = "Operation Timed-out";
           _setfail(1);
        }

        if (!$failaid{$ont}) { 
            ToolCommon::Log($nodeLogFh,"ONTPass: AID:$ont \n");
        } else {
           ToolCommon::Log($nodeLogFh,"ONTFail: AID:$ont Reason:$failaid{$ont}\n");
        }
     }
#   }
    } else {
       _update($pcount) if ($pcount > 0);
    }

#-Setting mode to manual---------------------------------------------------------------------------------
   if (uc($revertMode) ne "AUTO") { 
      foreach my $mnemonic (sort keys %ontDLManage){
	 foreach my $vp (sort keys %{$ontDLManage{$mnemonic}} ) {
	    foreach my $hw (sort keys %{$ontDLManage{$mnemonic}{$vp}} ) {
	       my $icount = 0;
	       my $onts = undef;
	       foreach my $aid (sort keys %{$ontDLManage{$mnemonic}{$vp}{$hw}} ) {
		  next if (! ($aid =~ /(ONT(-\d+)+)/) );
                  #if($onlyUnset){
                     next if (defined($ontDLManage{$mnemonic}{$vp}{$hw}{'fail'}));
                  #} else {
                     next if (!defined($ontToMonitor{$aid}));
                  #}

		  $icount++;
		  if ($icount > 31) {
		     $onts = $onts . $aid ;
		     unsetmode($tl1Session, $onts, $mode, $ontDLManage{$mnemonic}{$vp}{$hw}{'SW'} );
		     $icount=0;
		     $onts = undef;
		  } else {
		     if ($onts) {
			$onts = $onts . $aid . "&";
		     } else {
			$onts = $aid . "&";
		     }
		  }
	       }
               if ($icount) {
	          chop($onts);
	          unsetmode($tl1Session, $onts, $mode, $ontDLManage{$mnemonic}{$vp}{$hw}{'SW'} );
               }
	    }
	 }
      }
   }


#--Printing failed ONTs into file----------------------------------------------------------------------------
   my $passONT = 0;
   my $failONT = 0;
   my $failprintcount = 1;
   foreach my $mnemonic (sort keys %ontDLManage){
      foreach my $vp (sort keys %{$ontDLManage{$mnemonic}} ) {
	 foreach my $hw (sort keys %{$ontDLManage{$mnemonic}{$vp}} ) {
	    my $fail;
	    if (defined($ontDLManage{$mnemonic}{$vp}{$hw}{'fail'})){
	       $fail = $ontDLManage{$mnemonic}{$vp}{$hw}{'fail'};
	       ToolCommon::Log($nodeLogFh,sprintf("Type:%s  Hw:%s  Voip:%s  Reason : %s\n", $mnemonic, $hw, $vp, $fail )); 
	       if ($fail !~ /ONT SW CTRL/){
		  $failONT += $typeinfo{$mnemonic}{$vp}{$hw}{'count'};
		  next;
	       }
	    }

	    foreach my $aid (sort keys %{$ontDLManage{$mnemonic}{$vp}{$hw}} ) {
	       next if (! ($aid =~ /(ONT(-\d+)+)/) );
	       my $failAid;
	       if (defined($fail) || defined($failaid{$aid}) ){
		  $failAid = defined($fail)? $fail:$failaid{$aid};
	       }
	       if ($failAid){
		  ToolCommon::Log($nodeLogFh,sprintf("Type:%s  Hw:%s  Voip:%s  OID:%s  Reason : %s\n",       # BDFam48398
        			 $mnemonic, $hw, $vp, $aid, $failAid)); 
		  #print $failedontlistfile $aid . "\n";
		  $failONT += 1;
		  $failprintcount = 0;
	       } else{
		  $passONT += 1;
	       }

	    }
	 }
      }
   }
   ToolCommon::Log($nodeLogFh, "INFO:Number of ONTs passed :$passONT, failed:$failONT\n");

     #foreach my $mnemonic (sort keys %otherSW){
     #  foreach my $voip (sort keys %{$otherSW{$mnemonic}} ){
     #     if(defined($otherSW{$mnemonic}{$voip}{'fail'})){
     #         my $fail = $otherSW{$mnemonic}{$voip}{'fail'};
     #         ToolCommon::Log($nodeLogFh,sprintf("Type:%s : Voip:%s  Reason:%s\n",$mnemonic,$voip,$fail));          
     #    }
     # }
     #} 



   #if($failprintcount) {
   #   print $failedontlistfile "\n";
   #}

 
   $ret = IsamTL1::AllowAlarms($tl1Session);
   if ($ret) {
      ToolCommon::Log($nodeLogFh, " Alarms allowed...\n");
   } else {
      ToolCommon::Log($nodeLogFh, " Failed to allow alarms!!!\n");
      FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
				 EC_ISAM_ALW_ALARM_FAILED,undef,
				 1);
      return 0;
   }

   return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'release')) );
   FamilyCommon::CloseTL1Session2Close();
   #FamilyCommon::CloseCliSession2Close();

# BDFam51219 starts
   if ($warnExceeds) { 
         FamilyCommon::IncrErrorCount("Error: Download failures exceeds The Warning Threshold",
                                       EC_DL_ACT_WARN_EXCEEDS_THRESHOLD,undef,
                                      1);
   }

 # BDFam51219 ends

   if ($failCount) {
      FamilyCommon::IncrWarnCount("ONT Type List Partially Processed : $failCount ONTs failed",
                                   EC_LIST_PARTLY_PROCD,undef, 
                                   );
   }

   if ($params->{DEBUG}){
      ToolCommon::Log($nodeLogFh,"Total ONTs : $warnCount . No of failed ONTs: $failCount\n");
   }
   return 1;
}

1;
