package PbmtConfig;

=head1 PBMT configuration definition

=begin html
This PERL module implements the access to PBMT parameter configuration

=end html

=cut

#==============================================================================
#-- File:         PbmtConfig.pm
#-- Date:         24.09.2007
#-- Company:      Alcatel
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     PBMT configuraton module
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2007/19/24 : First implementation
#--   1.0.1 - 2007/09/25 : ANTms65540: Encrypted parameters may contain '=' characters.  Adapt parsing to this
#--   1.0.2 - 2007/09/27 : Adapted to completely generate param.cfg.  It is no longer present as a default file.
#--   1.0.3 - 2008/01/18 : ANTms63550: Added a new function to configure access operatorId through cli
#--   1.0.4 - 2008/04/29 : ANTmt05023: NT backups on AMS are stored in a compressed manner.  Unzip before converting
#--                        the database.  The modified DB is left in tar format.
#--   1.0.5 - 2008/05/06 : Cypher of IVPS OFLMT changed from MJSH to NBTW.
#--   1.0.6 - 2008/05/14 : chmod 666 of param.cfg once it has been written.
#--   1.0.7 - 2008/05/26 : ANTmt08304.  Use default values for a parameter commented out in param.cfg.
#--   1.0.8 - 2009/10/14 : Support for other file transfer protocols then tftp.
#--   1.0.9 - 2008/10/28 : ANTmt29255.  Some customers do not allow the usage of ping.
#--   1.0.10 - 2009/03/11: ANTmt47249
#--   1.0.11 - 2009/03/19
#--   1.0.12 - 2009/05/25: ANTmt59036
#--   1.0.13 - 2009/06/02: ANTmt61239 Added 1540LS family
#--   1.0.14 - 2009/06/10: ANTmt62662.  Allow SSH authentication using public-key
#--   1.0.15 - 2009/11/11: changes for undocumented option -cdedir (for ATX)
#--   1.0.16 - 2010/02/26: ANTmt94769. TFTPDIR text for 5520 AMS
#--   3.0.00 - 2010/06/08: Converted version to 3.0.00 for converged PBMT
#--   3.0.01 - 2010/06/08: Converged PBMT Changes
#--   3.0.02 - 2010/10/18: ALU00804222 - Update default names for tools
#--   3.0.03 - 2010/11/03: Default NT Sychronization timeout changed to 600sec.
#--   3.0.04 - 2010/12/14: ALU00954124 - Revert back to old naming conventions (with .exe extension)  for tools
#--   3.0.05 - 2011/07/29: ALU01348790
#--   3.0.06 - 2011/09/19: ALU01497319
#--                      : Change default value of WAITTIME to 4000sec
#--   3.0.07 - 2011/09/24: Add Solaris x86 based IVPS tool configure option
#--   3.0.08 - 2011/10/11: Rename I386 related parameters since AMS only supports I386_NTOFLMT type formats
#--   3.0.09 - 2012/05/11: ALU01789702 - World writeable directories need sticky bit set 
#--   3.0.10 - 2012/05/30: ALU01719620 
#--   3.0.11 - 2012/08/23: ALU01797328
#--   3.0.12 - 2012/09/13: ALU01819262
#--   3.0.13 - 2012/10/30: VLAN Label can be configured for IHUB systems
#--   3.0.14 - 2013/01/30: option to specify use of second password if first one fails for cli over ssh
#--   3.0.15 - 2013/05/20: ALU01883846
#--   3.0.16 - 2013/10/24: Requirement to put the ontReleaseMap file is ISAM,
#--   3.0.17 - 2014/05/30: typo error correction
#--   3.0.18 - 2014/06/09 : ALU01937911
#--   3.0.19 - 2014/08/28 : add status file for Rollback script
#--   3.0.20 - 2014/10/10 : introduce ont sw download progress timer 
#--   3.0.21 - 2015/04/13 : ALU02073720
#--   3.0.22 - 2015/06/18 : ALU02025800
#--   3.0.23 - 2015/11/02 : ALU02142016
#--   3.0.24 - 2016/01/08 : ALU02146866
#--   3.0.25 - 2016/01/28 : ALU02171244
#--   3.0.26 - 2016/03/16 : ALU02190558
#--   3.0.27 - 2016/04/02 : ALU02193793
#--   3.0.28 - 2016/04/28 : ALU02206611
#--   3.0.29 - 2016/07/15 : ALU02237966
#--   3.0.30 - 2016/08/08 : ALU02251693
#--   3.0.31 - 2017/10/24 : ALU02400566

=head1 VERSION

=begin html
Current version: "PbmtConfig.pm - 3.0.31 - 2017/10/24"

=end html

=cut

my $VERSION = "PbmtConfig.pm - 3.0.31 - 2017/10/24";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use File::Basename;
use Config; 
use Data::Dumper;

use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use version qw(:ALL);
use ErrorCodes qw(:EXIT_CODES);
use vars qw(%opt82 %prompts);

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub ReadValue($$);
sub ChangeValue($;$);
sub UpdateConfig();
sub NewInstall();
sub DefaultInstall();
sub AdjustToolVars();
sub CleanUp($;$);
sub PopulateConfig($$);
sub WriteValues ($$$$) ;

my $instDir  = ($Config{archname} =~ /linux/) ? "" : "/usr"; 
my $utilPath = ($Config{archname} =~ /linux/) ? "/bin" : "/usr/sbin"; 

my $DEBUG = 0;
my $RetryCount = 5;

#==============================================================================
#== Constants
#==============================================================================

#### DEVEL-NOTE  ON DETAILED INFORMATION ON PROPERTIES ####
####  IMPACT  - Just used to ask Y (To proceed) or N (To Skip) some part of prompts
####		This propty wont update any value into configuration file, But will impact IMPACT listed prompt keys.
my %changeValuePrompt = (
			'STRING'  => "",
			'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
		        'COMMENT' => ["Y - To change value; N - To skip",],
                    );

my %prompts = ('VERSION' => {
		     'ORDER' 	=> 0,
		     'VALUE' 	=> {'TYPE' => "D", 'DEFAULT' => PBMT_VERSION},
		     'COMMENT' 	=> ["Version of the toolset."],
		    },
	       'HOSTSFILE' => {
		      'ORDER'   => 1,
		      'STRING'  => "Hosts filename",
		      'VALUE'   => { 'TYPE' => "S", 'DEFAULT' => "/etc/hosts", 'REQ' => 'Y' },
		      'COMMENT' => ["Name of the file containing a list of nodes to be handled.",
		       	            "Needs to be in the format of the /etc/hosts file.  Default value is \"/etc/hosts\""],
		    },
	       'TFTPDIR' => {
		      'ORDER'   => 2,
		      'STRING'  => "TFTP top-directory name (depends on the installation)",
		      'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "/var/opt/aws/common/tftpd/data", 'REQ' => 'Y'},
		      'COMMENT' => ["Directory where files transferred by tftp will be stored.",
				   "This directory needs to be accessible and writeable by the user performing the migration.",
				   "When run on AWS this is /var/opt/aws/equipment by default.",
				   "By default this directory is only writeable by root.",
				   "So either the migration is performed as root or the directory is made writeable.",
				   "To find the value on your system use the unix command ps -ef | grep tftpd or ps -auxww | grep tftpd",
				   "AWS R6.5.x: \"/var/opt/aws/equipment\"",
				   "AWS R7.0: \"/var/opt/aws/common/tftpd/data\"",
				   "AMS 5526: \"/opt/ALAams/systemdata\"",
                                   "AMS 5520 R8: \"/var/opt/ams/local/ams-4.x-yyyyy/ne\"",
				   "where yyyyy is the software package number of the 5520 AMS.",
				   "For most recent information refer SUMAP or AMS Documentation"],
		    },
	       'TFTPWORKDIR' => {
		       'ORDER'   => 3,
		       'STRING'  => "DSLAM working directory name (logfile storage etc.)",
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "/tmp", 'REQ' => "Y"},
		       'COMMENT' => ["Temporary directory for the migration process",  
		       	           "Will be concatenated with the value of TFTPDIR.",
		       	           "Default is \"/tmp\"."],
		    },
	       'RESULTSPACE' => {
		       'ORDER'   => 4,
		       'STRING'  => "Directory for storing the status result files",
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "./", 'REQ' => 'Y'},
                       'COMMENT' => ["Directory holding the status result files organized per NE in a separate sub-directory."],
                    },
	       'MAXTFTPSESSION' => {
			'ORDER'   => 5,
			'STRING'  => "Max. number of parallel SW downloads",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 5, 'REQ' => "Y"},
			'COMMENT' => ["Maximum number of parallel tftp sessions.",
				      "Making this value too high may result in 'dramatic' load on the server.",
				      "Used by the Software Download manager process.",
				      "Default value is 5."],
	            },
	       'MAXDLSESSION' => {
			'ORDER'   => 6,
			'STRING'  => "Max. number of parallel migrations/upgrades",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 3, 'REQ' => "Y"},
			'COMMENT' => ["Maximum number of parallel download sessions.",
					"Making this value too high may result in 'dramatic' load on the server.",
					"Used by the migration and upgrade processes.",
					"Default value is 3."],
		     },
	       'MAXCMPSESSION' => {
			'ORDER'   => 7,
			'STRING'  => "Max. number of status check compares",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 10, 'REQ' => "Y"},
			'COMMENT' => ["Maximum number of parallel comparisons.",
				       "Default value is 10."],
		     },
	       'DEFAULTCLIUSER' => {
			'ORDER'   => 11,
			'STRING'  => "CLI username",
			'SPECIAL' => 1,
			'VALUE'   => {  'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "Y", },
                        'COMMENT' => ["Default CLI username.  Configured using Config.pl and stored encrypted" ],
	            },
	       'DEFAULTCLIPWD'  => {
			'ORDER'   => 12,
			'STRING'  => "CLI password",
			'SPECIAL' => 1,
			'VALUE'   => {  'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "Y",
				        'VALID' => '^(.{6,})$', 'POSSIBLE' => 'Atleast 6 characters'},
			'COMMENT' => ["Default CLI password.  Configured using Config.pl and stored encrypted",
                                      "Value must be minimum of 6 characters"],
		    },
	     'ASK_SECONDCLIPWD' => {
			'ORDER'   => 13,
			'STRING'  => "Configure Second CLI password (only applicable for CLI over SSH)",
			'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
			'COMMENT' => [ "Configure Second CLI password", "Y - To Configure; N - To skip" ],
			'IMPACT'  => [ 'SECONDCLIPWD' ],
                    },
	       'SECONDCLIPWD'   => {
			'ORDER'   => 14,
			'STRING'  => "Second CLI password",
			'SPECIAL' => 1,
			'VALUE'   => {  'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "Y",
				        'VALID' => '^(.{6,})$', 'POSSIBLE' => 'Atleast 6 characters'},
			'COMMENT' => ["Second CLI password.  Configured using Config.pl and stored encrypted",
                                      "Value must be minimum of 6 characters"],
		    },
	       'DEFAULTTL1USER' => {
			'ORDER'   => 15,
			'STRING'  => "TL1 username",
			'SPECIAL' => 1,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "Y"},
			'COMMENT' => ["Default TL1 username.  Configured using Config.pl and stored encrypted."],
	            },
	       'DEFAULTTL1PWD' => {
			'ORDER'   => 16,
			'STRING'  => "TL1 password",
			'SPECIAL' => 1,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "Y",
				      'VALID' => '^(.{6,})$', 'POSSIBLE' => 'Atleast 6 characters'},
			'COMMENT' => ["Default TL1 password.  Configured using Config.pl and stored encrypted.",
                                      "Value must be minimum of 6 characters"],
                    },
            'DEFAULTSFTPUSER' => {
			'ORDER'   => 17,
			'STRING'  => "SFTP username",
			'SPECIAL' => 1,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "Y"},
			'COMMENT' => ["Default SFTP User.  Configured using Config.pl and stored encrypted."],
                   },
           'SYS_KNOWNHOSTS' => {
			'ORDER'   => 21,
			'STRING'  => "SSH - system-wide known hosts database",
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "/etc/ssh/ssh_known_hosts", 'REQ' => "Y"},
			'COMMENT' => ["SSH - system-wide known hosts database"],
                   },
	    'LOC_KNOWNHOSTS' => {
			'ORDER'   => 22,
			'STRING'  => "SSH - user-specific known hosts database",
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => ".ssh/known_hosts", 'REQ' => "Y"},
			'COMMENT' => ["SSH - user-specific known hosts database"],
                   },
	     'ASK_NT_COMMUNITY' => {
			'ORDER'   => 31,
			'STRING'  => "Configure NT community string",
			'VALUE'   => { 'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N", },
                        'COMMENT' => ["Y - To configure NT community; N - To skip"],
			'IMPACT'  => [ 'NT_COMMUNITY' ],
                   },
          'NT_COMMUNITY' => {
			'ORDER'   => 32,
			'STRING'  => "NT community string",
			'SPECIAL' => 1,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "public", 'REQ' => "N"},
                        'COMMENT' => ["SNMP community string for the NT.  Configured using Config.pl and stored encrypted."],
                    },
	       'SNMP_QUERY_TIMEOUT' => {
			'ORDER'   => 32,
			'STRING'  => "Wait time in seconds for SNMP Query",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 30, 'REQ' => "N",
				          'VALID' => '^([5-9]|[1-9][0-9]|[1-5][0-9][0-9]|600)$', 'POSSIBLE' => 'from 5 to 600 seconds'},
			'COMMENT' => ["Wait time in seconds for SNMP Query", 
                          "Possible values from 5 to 600 seconds",
                          "Default value is 30."],
		     },
          'ASK_SNMPV2' => {
                        'ORDER'   => 33,
                        'STRING'  => "Configure SNMPV2",
                        'VALUE'   => { 'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N", },
                        'COMMENT' => ["Y - To configure SNMPV2; N - To skip"],
                        'IMPACT'  => [ 'USE_SNMPV2' ],
                   },
          'USE_SNMPV2' => {
			'ORDER'   => 34,
                        'COMMENT' => ["Use SNMPv2c"],			     
			'STRING'  => "Use SNMPv2c",
			'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "Y"},
                        'COMMENT' => ["Y - To consider as SNMPv2; N - To skip"],
                     },
	      'ASK_SNMPV3' => {
			'ORDER'  => 35,
			'STRING' => "Configure SNMPV3 settings",
			'VALUE'  => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
                        'COMMENT' => ["Y - To configure SNMPv3; N - To skip"],
			'IMPACT' => [ 'SNMPV3_USERNAME', 'SNMPV3_AUTH_PASSWORD', 
                                      'SNMPV3_AUTH_PROTOCOL', 'SNMPV3_PRIV_PROTOCOL', 'SNMPV3_PRIV_PASSWORD' ],
                     },
          'SNMPV3_USERNAME' => {
                        'COMMENT' => ["Defines the username to be used for an SNMPv3 session."],
                        'ORDER'   => 36,
                        'STRING'  => "SNMPv3 session authentication username",
                        'SPECIAL' => 1,
                        'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                     },
              'SNMPV3_AUTH_PASSWORD' => {
                         'ORDER'   => 37,
                         'STRING'  => "SNMPv3 session authentication password",
                         'SPECIAL' => 1,
			 'VALUE'   => {  'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N",
				        'VALID' => '^(.{8,})$', 'POSSIBLE' => 'Atleast 8 characters'},
                         'COMMENT' => ["Defines the password to be used for an SNMPv3 session.",
                                       "Value must be minimum of 8 characters"],
                     },
               'SNMPV3_AUTH_PROTOCOL' => {
                         'ORDER'   => 38,
                         'STRING'  => "SNMPv3 authentication protocol",
                         'VALUE'   => { 'TYPE' => "S", 'DEFAULT' => "MD5", 'REQ' => "N",
                                        'VALID' => '^(SHA|SHA1|SHA512|SHA256|MD5)$', 'POSSIBLE' => 'SHA, SHA1(replica of SHA), SHA256, SHA512 and MD5', },
                         'COMMENT' => ["Defines the authentication protocol used for SNMPv3",
                                       "Possible values : SHA, SHA1(replica of SHA), SHA256, SHA512, MD5"],
		     },
              'SNMPV3_PRIV_PROTOCOL' => {
                         'ORDER'   => 39,
                         'STRING'  => "SNMPv3 privacy protocol (DES, 3DES or AES)",
			 'VALUE'   => {  'TYPE' => "S", 'DEFAULT' => "DES", 'REQ' => "N",
				        'VALID' => '^(3DES|DES|AES)$', 'POSSIBLE' => 'DES, 3DES and AES'},
                         'COMMENT' => [  "Defines the privacy protocol used for SNMPv3",
                                         "Possible values are DES, 3DES and AES"],
		    },
              'SNMPV3_PRIV_PASSWORD' => {
                         'ORDER'   => 40,
                         'STRING'  => "SNMPv3 session privacy password",
                         'SPECIAL' => 1,
                         'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N",
				       'VALID' => '^(.{8,})$', 'POSSIBLE' => 'Atleast 8 characters'},
                         'COMMENT' => ["Defines the privacy password to be used for an SNMPv3 session.",
                                       "Value must be minimum of 8 characters"],
                    },
	       'WAITTIME' => {
			 'ORDER'   => 51,
			 'STRING'  => "Max. time (in seconds) to wait before node is declared as unreachable after upgrade",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 4000, 'REQ' => "Y"},
                         'COMMENT' => ["Time (in seconds) to wait before giving up on a successful restart.",
			   	       "Default is 4000 seconds."],
                    },
		'ONT_DOWNLOAD_TIMEOUT' => {
			 'ORDER'   => 52,
                	 'STRING'  => "Max. time (in seconds) for ONT download",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 1200, 'REQ' => "Y"},
                         'COMMENT' => ["Timeout in seconds for ONT download in case of slow links.",
  		                       "Default is 1200 seconds.  Lower values could result in better results."],
                    },
		'ONT_ACTIVATE_TIMEOUT' => {
			 'ORDER'   => 53,
		         'STRING'  => "Max. time (in seconds) for ONT activate",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 1200, 'REQ' => "Y"},
                         'COMMENT' => ["Timeout in seconds for ONT activation.",
                       		       "Default is 1200 seconds.  Lower values could result in better results."],
                    },
	        'ONT_DWLD_PROGRESS_TIMEOUT' => {
			  'ORDER'   => 54,
			  'STRING'  => "Max. time (in seconds) for single ONT download",
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 600, 'REQ' => "Y"},
                          'COMMENT' => ["ONT sw download progress timer.",
		                        "Default is 600 seconds."],
                    },
		'NT_SYNC_TIMEOUT' => {
			  'ORDER'   => 55,
			  'STRING'  => "NT Sync Timeout",
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 600, 'REQ' => "Y"},
                          'COMMENT' => ["The number of seconds expected to be sufficient for NT sync.",
			                "Default is 600 seconds."],
                    },
	       'TELNETTIMEOUT' => {
			 'ORDER'   => 56,
			 'STRING'  => "TELNET timeout (seconds)",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 30, 'REQ' => "Y"},
                         'COMMENT' => ["Timeout in seconds for telnet sessions in case of slow links."],
                    },
	       'TELNETTIMEOUT_SCHK' => {
			 'ORDER'   => 57,
			 'STRING'  => "TELNET timeout in Status Check script(seconds)",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 600, 'REQ' => "Y"},
                         'COMMENT' => ["Timeout in seconds for telnet sessions for Status Check script alone."],
                    },
	       'TFTPTIMEOUT' => {
			 'ORDER'   => 58,
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 3},
                         'COMMENT' => ["Timeout in seconds for tftp sessions in case of slow links.",
			               "Default is 3 seconds.  Lower values could result in better results."],
                    },
		'DWLDONTRELMAPFILE' => {
			  'ORDER'   => 61,
                          'STRING'  => "Download ONT Release Map file to ISAM (Y/N)",
			  'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "Y"},
                          'COMMENT' => ["Download ONT Release Map file to ISAM", "Y - To download; N - To skip"],
                    },
	       'PRE_MIG_ACTION' => {
			 'ORDER'   => 71,
			 'STRING'  => "Any pre-migration script (specify absolute path)",
			 'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                         'COMMENT' => ["Possible pre-migration action."],
                    },
	       'POST_MIG_ACTION' => {
                         'COMMENT' => ["Possible post-migration action."],
			 'ORDER'   => 72,
			 'STRING'  => "Any post-migration script (specify absolute path)",
			 'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                    },
	       'PRE_UPG_ACTION' => {
			 'ORDER'   => 73,
			 'STRING'  => "Any pre-upgrade script (specify absolute path)",
			 'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                         'COMMENT' => ["Possible pre-upgrade action."],
                    },
	       'POST_UPG_ACTION' => {
			 'ORDER'   => 74,
			 'STRING'  => "Any post-upgrade script (specify absolute path)",
			 'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                         'COMMENT' => ["Possible post-upgrade action."],
                    },
	       'TL1_BUSY_RETRY_WAITTIME' => {
			'ORDER'   => 80,
			'STRING'  => "Wait time in seconds to retry for TL1's communication busy response (SARB|SSTP)",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 60, 'REQ' => "Y",
				          'VALID' => '^([5-9]|[1-9][0-9]|[1-5][0-9][0-9]|600)$', 'POSSIBLE' => 'from 5 to 600 seconds'},
			'COMMENT' => ["Retry timeout in seconds for TL1's communication is busy (SARB|SSTP)", 
                          "Possible values from 5 to 600 seconds",
                          "Default value is 60."],
		     },
	     'ASK_DHCPPORT' => {
			 'ORDER'  => 81,
			 'STRING' => "Configure Non-default DCHP port identification",
			 'VALUE'  => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
                         'COMMENT' => ["Y - To configure Non-default DCHP port; N - To skip"],
			 'IMPACT' => [ 'SYS_PORT_NUM_MED', 'ATM_BASED_SYNTAX', 'ETH_BASED_SYNTAX' ],
	            },
	       'SYS_PORT_NUM_MED' => {
                       'COMMENT' => [ "Defines the prefered format (configure system port-num-in-proto log-slot-num (1) |",
		         	        "                                                       position-based (2) |",
		         	        "                                                           type-based (3) |",
		         	        "                                                           legacy-num (4))"],
		       'ORDER'   => 82,
		       'STRING'  => "Customer specific DHCP port numbering [1..4]",
		       'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 4, 'VALID' => '^([1234])$', 'POSSIBLE' => '1 - log-slot-num, 2 - position-based, 3 - type-based, 4 - legacy-num', 'REQ' => "Y"},
		    },

              'ATM_BASED_SYNTAX' => {
			 'ORDER'   => 83,
                         'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "Access_Node_ID atm Rack/Frame/Slot/Port:VPI.VCI"},
                         'STRING'  => "ATM-based port identification",
                         'COMMENT' => ["ATM based"],
		    },
             'ETH_BASED_SYNTAX' => {
			 'ORDER'   => 84,
                         'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "Access_Node_ID eth Rack/Frame/Slot/Port:VPI.VCI"},
                         'STRING'  => "Ethernet-based port identification",
                         'COMMENT' => ["Ethernet based"],
	           },
	     'ASK_ARBERGTAG' => {
			'ORDER'   => 91,
			'STRING'  => "Configure ARGBERGTAG for configuring PPPOX-RELAY",
			'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
                        'COMMENT' => ["Y - To configure ARGBERGTAG for configuring PPPOX-RELAY; N - To skip"],
			'IMPACT'  => [ 'ARBERGTAG' ],
	           },
	       'ARBERGTAG' => {
		        'ORDER'   => 92,
		        'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 2, 'REQ' => "Y"},
		        'STRING'  => "ARBERG tag for PPPoX-Relay",
                        'COMMENT' => ["ARGBERGTAG for configuring PPPOX-RELAY cross-connect",
		         	      " 1 -- To insert",
		          	      " 2 -- Not to insert",
		         	      "During migration 2 is the default value."],
		  },
	       'LINE_MM_PCT' => {
			'ORDER'   => 101,
			'STRING'  => "Line out-of-servce mismatch percentage (status check compare)",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 5, 'REQ' => "Y"},
                        'COMMENT' => ["Line state mismatch percentage."],
                   },
	       'LT_MM_PCT' => {
			'ORDER'   => 102,
			'STRING'  => "LT out-of-service mismatch percentage (status check compare)",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 0, 'REQ' => "Y"},
                        'COMMENT' => ["LT state mismatch percentage."],
                   },
	       'AUTH_MM_PCT' => {
			'ORDER'   => 103,
			'STRING'  => "Authentication mismatch percentage (status check compare)",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 5, 'REQ' => "Y"},
                        'COMMENT' => ["Line authentication mismatch percentage."],
                   },
	       'BRDG_CONFMM_PCT' => {
		        'ORDER'   => 104,
		        'STRING'  => "Bridge configuration mismatch percentage (status check compare)",
		        'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 0, 'REQ' => "Y"},
                        'COMMENT' => ["Bridge config mismatcht threshold."],
                   },
	       'PHYSICAL_HOST_IP' => {
			'ORDER'   => 105,
			'STRING'  => "IP address of the IP interface to the DSLAM (for HA systems)",
			'VALUE'   => {'TYPE' => "@", 'DEFAULT' => "0.0.0.0", 'REQ' => "N"},
                        'COMMENT' => ["In case of HA systems we need the machine physical IP address."],
                   },

	       'CDEDIR' => {
		       'ORDER'   => 111,
		       'STRING'  => "CDE profile directory (relative to TFTP top directory).",
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "cde_profiles", 'REQ' => "N"},
                       'COMMENT' => ["CDE directory location on AWS.  Only 1 CDE directory per network is assumed.",
		   		  "Relative to TFTPDIR in case tftp is used as file transfer protocol to the node.",
		   		  "Absolute in case ftp or sftp is used as file transfer protocol to the node"],
                   },
	       'NEWCDEFILE' => {
			'ORDER'   => 112,
			'STRING'  => "Name of new CDE file to be included in database",
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                        'COMMENT' => ["New CDE file location on AWS.  Only 1 CDE file per network is assumed.",
				      "Relative to TFTPDIR/CDEDIR when tftp is used as file transfer mechanism to the node.",
				      "Relative to CDEDIR in case ftp or sftp is used as file transfer mechanism to the node"],
                   },
	       'ACTCDEFILE' => {
		       'ORDER'   => 113,
		       'STRING'  => "Name of CDE file currently loaded on the ISAM-V",
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", 'REQ' => "N"},
                       'COMMENT' => ["Active CDE file location on AWS.  Only 1 CDE file per network is assumed.",
		       	             "Relative to TFTPDIR/CDEDIR.",
		       	             "Used from R3.6 onwards."],
                   },
	       'CDE_REGEXP' => {
			'ORDER'   => 114,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => 'MSR2(..)(\d\d)\.(\d\d\d)'},
                        'COMMENT' => ["CDE-file regular expression to extract the customer ID e.g. MSR2AB32.123",
				      '$1 becomes customer ID (AB)',
				      '$2 becomes the compatible SW version (11: R1.1, 12: R2.5, 31: R3.1, 32: R3.2, 33: R3.3,....)',
				      '$3 becomes the CDE profile version (123)'],
		   },
	      'ASK_IGMP_MC_RANGE' => {
		        'ORDER'   => 121,
		        'STRING'  => "Configure IGMP Multicast Range setting",
		        'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
                        'COMMENT' => ["Y - To configure IGMP Multicast Range; N - To skip"],
		        'IMPACT'  => [ 'IGMP_MC_RANGE_SETTING' ],
                   },
	       'IGMP_MC_RANGE_SETTING' => {
			'ORDER'   => 122,
			'STRING'  => "Allow IGMP multicast entries",
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "UNU", 'REQ' => "Y",
                                      'VALID' => '^(SEL|FRC|UNU)$', 'POSSIBLE' => 'SEL - SELECTIVE, FRC - FORCED and UNU - UNUSED'},
                        'COMMENT' => ["To configure IGMP multicast (mc) entries",
				      "Values : SEL - SELECTIVE; FRC - FORCED; UNU - UNUSED",
				      "Default is UNU (Unused)."],
		    },
	       'ENABLE_IGMP_GROUPING' => {
			'ORDER'   => 126,
			'STRING'  => "Allow grouping of IGMP multicast entries",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 1, 'REQ' => "Y"},
                        'COMMENT' => ["Allow grouping of IGMP multicast entries.",
			              "Default is 1 (Enabled)."],
                   },
	       'MAX_GIP_GAP' => {
			'ORDER'   => 127,
			'STRING'  => "Gap size between multicast group ip-addresses",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 0, 'REQ' => "Y"},
                        'COMMENT' => ["Allowed gap size between multicast group ip-addresses while grouping.",
			              "Default is 0 (allow consecutive entries)."],
                   },
	       'GPON_ONU_RANGE_MODE' => {
			'ORDER'   => 128,
			'STRING'  => "Allow GPON ONU's to range in presence of ALIEN ONU's",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 0, 'REQ' => "Y"},
                        'COMMENT' => ["Allow GPON ONU's to range in presence of ALIEN ONU's.",
			              "Default is 0 (Disabled)."],
                   },
	       'ACT_VLANLABEL' => {
			'ORDER'   => 131,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => ""},
                        'COMMENT' => ["Existing Vlan Label for IHUB systems."],
                   },
	       'NEW_VLANLABEL' => {
			'ORDER'   => 132,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => ""},
                        'COMMENT' => ["New Vlan Label for IHUB systems."],
                   },
	       'SSMOUT_VLAN' => {
			'ORDER'   => 134,
			'STRING'  => "SSMOUT Vlan",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 0, 'REQ' => "Y"},
                        'COMMENT' => ["Unused Vlan value which can be used as ssm out vlan during migration.",
			              "Default is 0 (value can be provided by user when 4088 is already in use)."],
                   },
	       'ACCESS_OPERATOR_ID' => {
			'ORDER'   => 135,
			'STRING'  => "Access OperatorId",
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 0, 'REQ' => "Y"},
                        'COMMENT' => ["Configurable Nokia assigned Access OperatorID (range 0...65535)"],
                   },
               'FORCE_OPERATOR_ID' => {
                        'ORDER'   => 136,
                        'STRING'  => "Force Access OperatorId (Y/N)",
                        'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N" , 'REQ' => "Y"},
                        'COMMENT' => ["Force Nokia assigned OperatorID check", "Y - To enable force check; N - To skip"],
                   },
               'BUFSIZE' =>  {
                        'ORDER'   => 137,
                        'STRING'  => "CLI command line buffer",
                        'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "15" , 'REQ' => "Y"},
                        'COMMENT' => ["Size of CLI command return buffer"],
                   },
	       'REMAP_HOT_RESET' => {
			'ORDER'   => 138,
			'STRING'  => "Enable HOT RESET remapping (i.e. prevent NT hot reset)",
			'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "Y"},
                        'COMMENT' => ["Enable HOT RESET remapping (i.e. disable NT hot reset) prior to OSWP activation (requires octopus T&D access)", "Y - To enable; N - To skip"],
                   },
	       'SECOND_RUN' => {
			'ORDER'   => 139,
			'STRING'  => "Enable second OSWP activation of the new software in case of failure the first time",
			'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "Y"},
                        'COMMENT' => ["Execute a second OSWP activation run in case migration failed after the first run", "Y - To enable; N - To skip"],
                   },
	       'NO_PING' => {
			 'ORDER'   => 140,
			 'STRING'  => "Ping disabled in the network",
			 'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "Y"},
                         'COMMENT' => ["Indicates whether ping is allowed (0) or not (1) in the network", "Y - For ping exec; N - Not to exec"],
                   },
		'NT_SWP_MAPFILE' => {  
                          'ORDER'   => 141,
                          'COMMENT' => ["OSWP NT-TYPE mapping file from build image."],
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "NT_SWP_Mapping.txt"},
                    },
	    'UMASK_OVERRIDE' => {
	      'ORDER'   => 142,
	      'STRING'  => "Override umask value to 0 and perform chmod",
	      'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "Y", 'REQ' => "N"},
                      'COMMENT' => ["Override umask value to 0 and perform chmod ", "Y - Override umask and perform chmod; N - Not to ovreride"],
                },
	    'DOWNLOAD_VERIFY' => {
	      'ORDER'   => 143,
	      'STRING'  => "To perform implicit/auto SW load verification after downloading",
	      'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "N"},
                        'COMMENT' => ["To enable/disable auto verify of OSWP (SW build) after download ", 
                                     "Y - Enable auto-verify after downloading OSWP; N - Disable auto-verify the OSWP after download"],
                },
		'ONTRELDIR' => {  
                          'ORDER'   => 211,
                          'COMMENT' => ["Folder path of the ONT release mapping file."],
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/"},
                    },
		'ONTRELMAPFILE' => {    
                          'ORDER'   => 212,
                          'COMMENT' => ["ONT release mapping file name."],
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "ONT_Release_Mapping.txt"},
                    },
	       'SHUBSWPATH' => {
			  'ORDER'   => 213,
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "", },
                          'COMMENT' => ["Path to the location where the SHUB SW is stored.",
			                "Only applicable for releases < 2.3",
			                "This setting must be relative from TFTPDIR (i.e. has to be a sub-directory of it)."],
                    },
	       'SHUBSWNAME' => {
			  'ORDER'   => 214,
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "Lanx.Z"},
			  'COMMENT' => ["Name of the Lanx SW file.  This has to be filled in if the setting of SHUBSWPATH is not empty.",
				        "Default setting is \"Lanx.Z\"."],
		    },
	       'TL1UDPPORT' => {
			  'ORDER'   => 215,
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 13001},
                          'COMMENT' => ["TL-1 udp port for communicating to the ISAM."],
                    },
	       'TL1SSHPORT' => {
			  'ORDER'   => 216,
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 1022},
                          'COMMENT' => ["TL-1 SSH port for communicating to the ISAM."],
                    },
	       'RETRIES' => {
			  'ORDER'   => 217,
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 5},
                          'COMMENT' => ["Number of retries performed on an SNMP get.",
			     	        "Used to determine whether the node has realy restarted successfully.",
			     	        "Default value is 5."],
                    },

               'IVPSDBTOOLDIR' => {
			'ORDER'   => 231,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/IVPS"},
                        'COMMENT' => ["ISAM-V migration.",
			     	      "Directory holding IVPS offline migration tool."],
                   },
	       'IVPSOFLMT' => {
		        'ORDER'   => 232,
		        'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "NBTW.exe"},
                        'COMMENT' => ["Name of the IVPS offline migration tool for Solaris Sparc platform.",
		         	      "This name is relative to the directory IVPSDBTOOLDIR of the install-path of the migration toolset."],
                   },
	       'I386_IVPSOFLMT' => {
		        'ORDER'   => 233,
		        'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "NBTW_i386"},
                        'COMMENT' => ["Name of the IVPS offline migration tool for Solaris x86 platform.",
		         	     "This name is relative to the directory IVPSDBTOOLDIR of the install-path of the migration toolset."],
                   },
	       'LINUX_IVPSOFLMT' => {
			'ORDER'   => 234,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "NBTW_l.exe"},
                        'COMMENT' => ["Name of the IVPS offline migration tool for Linux platform.",
			 	     "This name is relative to the directory IVPSDBTOOLDIR of the install-path of the migration toolset."],
                   },
	       'IVPSDBCFG' => {
		       'ORDER'   => 235,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "dbtool.cfg"},
                       'COMMENT' => ["Name of the config file for the IVPS offline migration tool.",
				     "This name is relative to the directory IVPSDBTOOLDIR of the install-path of migration toolset."],
                   },
	       'IVPSINVENTORY' => {
		       'ORDER'   => 236,
		       'STRING'  => "xVPS inventory file specification (path and name)",
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "/opt/aws/isamv/voipim/script/getIVPSBoardInNE.pl", 'REQ' => "N"},
                       'COMMENT' => ["Directory and name of the IVPS-inventory script (AWS deliverable, check AMS documentation).",
			     	     "Returns list of IVPS boards with their IP address.",
			     	     "Default setting on AWS: /opt/aws/isamv/voipim/script/getIVPSBoardInNE.pl",
			     	     "Default setting on AMS 8.3: /opt/ni/bin/getIVPSBoardsInNE.sh"],
                   },
	       'NTDBTOOLDIR' => {
			  'ORDER'   => 241,
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/NT"},
                          'COMMENT' => ["Directory holding NT offline migration tool."],
                    },
	       'NTOFLMT' => {
			  'ORDER'   => 242,
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH.exe"},
                          'COMMENT' => ["Name of the NT offline migration tool for Solaris Sparc platform.",
			     	        "This name is relative to the directory NTDBTOOLDIR of the install-path of migration toolset."],
                    },
	       'I386_NTOFLMT' => {
			  'ORDER'   => 243,
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH_i386"},
                          'COMMENT' => ["Name of the NT offline migration tool for Solaris x86 platform.",
			      	        "This name is relative to the directory NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'LINUX_NTOFLMT' => {
			   'ORDER'  => 244,
			   'VALUE'  => {'TYPE' => "D", 'DEFAULT' => "MJSH_l.exe"},
                          'COMMENT' => ["Name of the NT offline migration tool for Linux platform.",
					"This name is relative to the directory NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'HICAP_NTDBTOOLDIR' => {
			  'ORDER'   => 245,
			  'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/NT_HICAP"},
                          'COMMENT' => ["Directory holding NT offline migration tool for the HiCAP NE family."],
                     },
	       'HICAP_NTOFLMT' => {
			  'ORDER'   => 246,
			  'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH.exe"},
                          'COMMENT' => ["Name of the NT offline migration tool of the HiCAP NE family.",
					"This name is relative to the directory HICAP_NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'ANSI_NTDBTOOLDIR' => {
			  'ORDER' => 251,
			  'VALUE' => {'TYPE' => "S", 'DEFAULT' => "tools/NT_ANSI"},
                          'COMMENT' => ["Directory holding NT offline migration tool for the ANSI NE family."],
                     },
	       'ANSI_NTOFLMT' => {
			'ORDER'   => 252,
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH.exe"},
                        'COMMENT' => ["Name of the NT offline migration tool for the ANSI NE family on Solaris Sparc platform.",
	        	              "This name is relative to directory ANSI_NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'I386_ANSI_NTOFLMT' => {
			'ORDER'   => 253,
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH_i386"},
                        'COMMENT' => ["Name of the NT offline migration tool for the ANSI NE family on Solaris x86 platform.",
			        	"This name is relative to the directory ANSI_NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'LINUX_ANSI_NTOFLMT' => {
			'ORDER'   => 254,
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH_l.exe"},
                        'COMMENT' => ["Name of the NT offline migration tool for the ANSI NE family on Linux platform.",
						"This name is relative to the directory ANSI_NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'LS_NTDBTOOLDIR' => {
			'ORDER'   => 256,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/NT_LS"},
                        'COMMENT' => ["Directory holding NT offline migration tool for the Litespan NE family."],
                     },
	       'LS_NTOFLMT' => {
			'ORDER'   => 257,
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH.exe"},
                        'COMMENT' => ["Name of the NT offline migration tool for the Litespan NE family.",
				      "This name is relative to the directoryLS_NTDBTOOLDIR of the install-path of migration toolset."],
                     },
	       'GPON_NTDBTOOLDIR' => {
			'ORDER'   => 301,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/NT_GPON"},
                        'COMMENT' => ["Directory holding NT offline migration tool for the GPON NE family."],
                     },
	       'GPON_NTOFLMT' => {
			'ORDER'   => 302,
			'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "MJSH.exe"},
                        'COMMENT' => ["Name of the NT offline migration tool for the GPON NE family.",
				      "This name is relative to the directory GPON_NTDBTOOLDIR of the install-path of migration toolset."],
                    },
	       'NTDBCFG' => {
		        'ORDER'   => 303,
		        'VALUE'   => {'TYPE' => "D", 'DEFAULT' => "dbtool.cfg"},
                        'COMMENT' => ["Name of the config file for the NT offline migration tool.",
		           	      "This name is relative to the directory [ANSI_ | HICAP_ | GPON_]NTDBTOOLDIR of the install-path of migration toolset."],
                    },
	       'LANXDBTOOLDIR' => {
			'ORDER'   => 304,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/Lanx"},
                        'COMMENT' => ["Directory holding SHUB offline migration tool."],
                    },
	       'ANSI_LANXDBTOOLDIR' => {
			'ORDER'   => 305,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/Lanx_ANSI"},
                        'COMMENT' => ["Directory holding SHUB offline migration tool for the ANSI stream."],
                    },
	       'LS_LANXDBTOOLDIR' => {
			'ORDER'   => 306,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/Lanx_LS"},
                        'COMMENT' => ["Directory holding SHUB offline migration tool for the Litespan ISAM."],
                    },
	       'LANXOFLMT' => {
			'ORDER'   => 307,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "Migrationtool.pl"},
                        'COMMENT' => ["Name of the LANX migration tool.",
			 	     "This name is relative to the directory LANXDBTOOLDIR of the install-path of migration toolset."],
                    },
	       'GPON_LANXDBTOOLDIR' => {
			'ORDER'   => 308,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/Lanx_GPON"},
                        'COMMENT' => ["Directory holding SHUB offline migration tool for the GPON NE family."],
                    },
	       'GPON_LANXOFLMT' => {
			'ORDER'   => 309,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "Migrationtool.pl"},
                        'COMMENT' => ["Name of the LANX migration tool for the GPON NE family.",
			    	      "This name is relative to the directory GPON_LANXDBTOOLDIR of the install-path of migration toolset."],
                   },
	       'OCTOPUSPATH' => {
			'ORDER'   => 310,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "tools/octopus"},
                        'COMMENT' => ["Name of the octopus application.",
			              "This name is relative to the installation directory of the toolset."],
                   },
	       'REGEXP' => {
		       'ORDER'   => 399,
                       'COMMENT' => ["REGEXP is a search pattern to extract the major and minor release IDs from an OSWP name.",
		            	  "\\w is for a single letter on that position",
		            	  "\\d is for a single digit on that position",
		            	  ". is for any character at that position",
		            	  "The digits that represent the major and minor and maintenance release ID have to set in parenthesis.",
		            	  "e.g. on AWS:",
		            	  "L6GPAA24.031 leads to a regexp value of L6GP..(\\d)(\\d)\.(\\d)(\\d\\d)",
		            	  "e.g. for AMS:",
		            	  "OSWPAA24.036 leads to a regexp value of OSWP..(\\d)(\\d)\.(\\d)(\\d\\d)",
		            	  "Default checks for any 6 characters followed by 2 digits again followed by a dot and finally followed by 3 digits"],
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => '......(\d)(\d)\.(\d)(\d\d)'},
			   },
	       'PING' => {
			'ORDER'   => 401,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$utilPath/ping"},
                        'COMMENT' => ["Path and name of the Unix ping command."],
                   },
	       'GREP' => {
                        'COMMENT' => ["Path and name of Unix grep command."],
			'ORDER'   => 402,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/grep"},
                   },
	       'PS' => {
			'ORDER'   => 403,
			'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/ps -ef"},
                        'COMMENT' => ["Path and name and option of Unix ps command."],
                  },
	       'RM'=> {
		       'ORDER'   => 404,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/rm"},
                       'COMMENT' => ["Path and name of Unix rm command."],
                  },
	       'SED'=> {
		       'ORDER'   => 404,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/sed"},
                       'COMMENT' => ["Path and name to file traversing."],
                  },
	       'CP' => {
		       'ORDER'   => 405,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/cp"},
                       'COMMENT' => ["Path and name of Unix cp command."],
                 },
	       'TOUCH' => {
		       'ORDER'   => 406,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/touch"},
                       'COMMENT' => ["Path and name of Unix touch command."],
                 },
	       'CD' => {
		       'ORDER'   => 407,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/cd"},
                       'COMMENT' => ["Path and name of Unix cd command."],
                 },
	       'MKDIR' => {
		       'ORDER'   => 408,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/mkdir"},
                       'COMMENT' => ["Path and name of Unix mkdir command."],
                 },
	       'SLEEP' => {
		       'ORDER'   => 409,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/sleep"},
                       'COMMENT' => ["Path and name of Unix sleep command."],
                 },
	       'ECHO' => {
		       'ORDER'   => 410,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/echo"},
                       'COMMENT' => ["Path and name of Unix echo command."],
                 },
	       'TAR' => {
		       'ORDER'   => 411,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$utilPath/tar"},
                       'COMMENT' => ["Path and name of Unix tar command."],
                 },
	       'PERL' => {
		       'ORDER'   => 412,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "/usr/bin/perl"},
                       'COMMENT' => ["Path and name of perl"],
                  },
	       'GZIP' => {
		       'ORDER'   => 413,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/bin/gzip"},
                       'COMMENT' => ["Path and name of gzip"],
                  },
	       'IFCONFIG' => {
		      'ORDER'   => 414,
		      'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "$instDir/sbin/ifconfig"},
                      'COMMENT' => ["Path and name of ifconfig"],
                  },
	       'SECUREDBSAVE' => {
		      'ORDER'   => 415,
		      'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 30},
                      'COMMENT' => ["Number of seconds to wait after the SHUB database command before allowing",
		            	    "a tftp to start (needed for R2.2 only)."],
                  },
	       'NODEPREFIX' => {
		      'ORDER'   => 416,
		      'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "AS-"},
                      'COMMENT' => ["Node name prefix in host file",
				    "AWS uses an AS- prefix for the nodes defined via AWS (as such it filters out IP addresses",
				    "not used by AWS that would be present in the /etc/hosts file."],
                  },
	       'MINIMALSWPARSIZE' => {
		      'ORDER'   => 417,
		      'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 126724096},
                      'COMMENT' => ["Minimum number of bytes in /dev/dha1 (the SW partition).",
		        	    "No migration is allowed when the SW partition size is smaller then the ammount of bytes specified below."],
                  },
	       'RED_CHARS' => {
		      'ORDER'   => 418,
		      'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "RP"},
                      'COMMENT' => ["Character identifying SW reduction."],
                  },
	       'MM_CHARS' => {
		       'ORDER'   => 419,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "ZZ"},
                       'COMMENT' => ["Character identifying SW minimizing."],
                  },
	       'SWDL_STATUS_FILE' => {    
		       'ORDER'   => 420,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "SWDLresult.txt"},
                       'COMMENT' => ["Status file name for SW download."],
                  },
	       'ONTSTG_STATUS_FILE' => {  
		       'ORDER'   =>421,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "ONTSTGresult.txt"},
                       'COMMENT' => ["Status file name for ont Staging"],
                  },
	       'ONTDWLDACT_STATUS_FILE' => {
		       'ORDER'   => 422,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "ONTDWLDACTresult.txt"},
                       'COMMENT' => ["Status file name for ont Download and Activate"],
                  },
	       'MIGR_STATUS_FILE' => {
		       'ORDER'   => 423,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "MIGRresult.txt"},
                       'COMMENT' => ["Status file name for migration."],
                  },
	       'UPGR_STATUS_FILE' => {
		       'ORDER'   => 424,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "UPGRresult.txt"},
                       'COMMENT' => ["Status file name for SW upgrade."],
                  },
	       'COMT_STATUS_FILE' => {
		       'ORDER'   => 425,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "COMTresult.txt"},
                       'COMMENT' => ["Status file name for SW commit."],
                  },
	       'PRCK_STATUS_FILE' => {
		       'ORDER'   => 426,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "PRCKresult.txt"},
                       'COMMENT' => ["Status file name for pre-check."],
                  },
	       'PRCK_WARN_FILE' => {
		       'ORDER'   => 426,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "PRCKwarnings.txt"},
                       'COMMENT' => ["Pre-check warning level file."],
                  },
	       'SCHK_STATUS_FILE' => {
		       'ORDER'   => 427,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "SCHKresult.txt"},
                       'COMMENT' => ["Status file name for post-check"],
                  },
	       'CHKC_STATUS_FILE' => {
		       'ORDER'   => 428,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "CHKCresult.txt"},
                       'COMMENT' => ["Status file for status check comparison."],
                  },
	       'CHKC_WARN_FILE' => {
		       'ORDER'   => 429,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "CHKCwarnings.txt"},
                       'COMMENT' => ["Status check warning level file."],
                  },
	       'CLUP_STATUS_FILE' => {
		       'ORDER'   => 430,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "CLUPresult.txt"},
                       'COMMENT' => ["Status file name for environment cleanup."],
                  },
	       'MINZ_STATUS_FILE' => {
		       'ORDER'   => 431,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "MINZresult.txt"},
                       'COMMENT' => ["Status file name for SW minimization."],
                  },
	       'RLBK_STATUS_FILE' => {
		       'ORDER' => 432,
		       'VALUE' => {'TYPE' => "S", 'DEFAULT' => "RLBKresult.txt"},
                       'COMMENT' => ["Status file name for OSWP Rollback."],
                  },
	       'ACT_PING_WAIT' => {
		       'ORDER'   => 433,
		       'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 30},
                       'COMMENT' => ["Wait time before start ping after OSWP activation for ISAM 7302/7330."],
                  },
	       'ACT_PING_WAIT_RU' => {
		       'ORDER'   => 434,
		       'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 45},
                       'COMMENT' => ["Wait time before start ping after OSWP activation for ISAM 7354RU."],
                  },
	       'STATUS_CHECKS' => {
		       'ORDER'   => 435,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "ltsts:xdsl:bridge:auth:alarm"},
                       'COMMENT' => ["Status checks for pre- and post- migration."],
                  },
	       'SSH' => {
		       'ORDER'   => 436,
		       'VALUE'   => {'TYPE' => "S", 'DEFAULT' => "/usr/bin/ssh"},
                       'COMMENT' => ["Path and name of ssh"],
                  },
		'EMERG_CALL_FEATURE' => {
			 'ORDER'   => 437,
			 'STRING'  => "Migration interrupt on emergency-call feature",
			 'VALUE'   => {'TYPE' => "B", 'DEFAULT' => "N", 'REQ' => "Y"},
             'COMMENT' => ["Indicates whether the migration interrupt on emergency-call feature is enabled or disabled.", "Y - Enabled; N - Disabled"],
      		 'IMPACT'  => [ 'EMERG_ALRM_POLL_TIME' ],
                   },
	       'EMERG_ALRM_POLL_TIME' => {
			 'ORDER'   => 438,
			 'STRING'  => "Polling time to fetch the emergency-call alarm (seconds)",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 120, 'REQ' => "Y",
				      'VALID' => '^([1-9]|[1-9][0-9]|[1-9][0-9][0-9]|1[01][0-9][0-9]|1200)$', 'POSSIBLE' => '1 to 1200 (seconds)'},
                         'COMMENT' => ["Max. time (in seconds) for emergency-call alarm polling.",
			               "Default is 120 seconds. Possible values - 1 to 1200 (seconds)"],
                    },
	       'SNMP_YANG_ACTIVATE_TIMEOUT' => {
			 'ORDER'   => 439,
			 'STRING'  => "Approximate wait time to check YANG build activated or not",
			 'VALUE'   => {'TYPE' => "D", 'DEFAULT' => 600, 'REQ' => "Y",
				      'VALID' => '^(3[0-9][0-9]|[4-9][0-9][0-9]|[12][0-9][0-9][0-9]|3[0-5][0-9][0-9]|3600)$', 'POSSIBLE' => '300 to 3600 (seconds)'},
                         'COMMENT' => ["Max. time (in seconds) to check yang build activation",
			               "Default is 600 seconds. Possible values - 300 to 3600 (seconds)"],
                    },
	      );

#==============================================================================
#== Variables
#==============================================================================
my $noEcho = 0;   # For signal handlers: restore echo when terminal echo was turned off.
my $renamed = 0;  # For signal handlers: restore param.cfg to original if needed.
my $created = 0;  # For signal handlers: remove param.cfg if incomplete.
my $DEFAULTCLIUSER = "";
my $DEFAULTCLIPWD = "";
my $DEFAULTTL1USER = "";
my $DEFAULTTL1PWD = "";
my $DEF_NT_COMMUNITY = "";
my $DEF_ACT_CDEFILE = undef;
my $DEF_NEW_CDEFILE = undef;
my $DEF_NEW_CDEDIR = undef;

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=begin html

=head2 ReadValue($$)

=begin html
Prompt for, read and parse a value from the commandline.  In case a default value is defined and no input has been given, the default value will
be used as the value to be read.  The accepted value is returned to the caller.<br>
The following parameters are used:
<table>
  <tr><td>secret</td><td>Indicates whether the value to read is secret or not (determines terminal echo or not)</td></tr>
  <tr><td>key</td><td>Indicates an entry from the parameter defintions (prompt hash)</td></tr>
  <tr><td>promptHash</td><td>A hash containing the prompts to be displayed to the user</td></tr>
  <tr><td>defValue</td><td>The default value to be used in case no input data has been given.  The undefined value if no default exists</td></tr>
</table>

=end html

=cut

sub ReadValue($$) {
  my ( $key, $promptHash) = @_;
  if( ( ! defined $key ) or ( $key eq "" ) ) {
	print "ERROR : KEY is not defined/Empty";
	return;
  }
  if( ( ! defined $promptHash->{STRING} ) or ( $promptHash->{STRING} eq "" ) ) {
	print "ERROR : $key property STRING is not defined/Empty";
	return;
  }
  my ($defValue, $defValueLabel) = ("","");
  if ( (defined $promptHash->{'VALUE'}{'DEFAULT'} ) and ( $promptHash->{'VALUE'}{'DEFAULT'} ne "" ) ) {
       $defValue =  $promptHash->{'VALUE'}{'DEFAULT'} ;
  }
  if( defined $promptHash->{'VALUE'}{'CONFIGVALUE'} ) {
       if( $key !~ /^ASK_/ ){
          $defValue = $promptHash->{'VALUE'}{'CONFIGVALUE'};
       }
  }
  my ( $ValidInput, $PossibleInput) = ("","","","");
  if ( (defined $promptHash->{'VALUE'}{'VALID'}) and 
       ( $promptHash->{'VALUE'}{'VALID'}) ne "" ) {
       $ValidInput = $promptHash->{'VALUE'}{'VALID'};
       $PossibleInput = ( defined $promptHash->{'VALUE'}{'POSSIBLE'}) ? $promptHash->{'VALUE'}{'POSSIBLE'} : "";
  } elsif ( $promptHash->{'VALUE'}{'TYPE'} eq "D" ) {
       $ValidInput = '^(\d+)$';
       $PossibleInput = "Only Digits";
  } elsif ($promptHash->{'VALUE'}{'TYPE'} eq "@") {
	$ValidInput = '\d+\.\d+\.\d+\.\d+';
        $PossibleInput = "Only IP address format (no netmask)";
  } elsif ($promptHash->{'VALUE'}{'TYPE'} eq "B") {
	$ValidInput = '^([YNyn])$';
        $PossibleInput = "Only Y or N";
  } else {
	$ValidInput = '.';
        $PossibleInput = "";
  }
  if( ( defined $promptHash->{STRING} ) and ( $promptHash->{STRING} ne "" ) ) {
       print STDERR "\n\n-- $promptHash->{STRING}\n";
       if (defined($promptHash->{COMMENT}) and (! defined $promptHash->{IMPACT})) {
           print STDERR "\t### $_\n" foreach (@{$promptHash->{COMMENT}}); 
       }
       if (defined($promptHash->{HELP}) and (! defined $promptHash->{IMPACT})) {
           print STDERR "\t--- $_\n" foreach (@{$promptHash->{HELP}}); 
       }
       if( defined $promptHash->{IMPACT} ) {
           print STDERR "\t--- Possible Values : Y (Proceed) or N (Skip)\n"; 
       }
  }
  if ( defined $promptHash->{SPECIAL} ) {
    system("stty -echo");
    $noEcho = 1;
  } elsif ( $defValue ne "" ) {
    $defValueLabel = "[$defValue]";
  }
  my ( $InputRetry, $IsRetryRequired, $read ) = ( 0, 1, "" );
  while ( $IsRetryRequired ) {
    print sprintf( "\t>>> Your input %40s : ", "$defValueLabel");
    $read = readline(*STDIN);
    chomp $read;
    $read = "" if ($read eq "\"\"" or $read eq "''" or ($read =~ /^\s*$/ ) );
    if (length($read) == 0) {
          $read = $defValue;
    }
    if ($noEcho) {
          print "*" x length($read);
          print "\n";
    }
    if ( ($promptHash->{'VALUE'}{'REQ'} eq "N") && (length($read) == 0) ) {
	 $IsRetryRequired = 0;
         last;
    } else  {
	 my $NoticeMesg = "";
         if( $read eq "" ) {
	     $NoticeMesg = "Input is NULL/EMPTY;";
         } elsif( ( $ValidInput ne "" ) and ( $read !~ /$ValidInput/ ) ){
           $NoticeMesg = "Input is INVALID;";
         }
         if( $NoticeMesg ne "" ) {
	   $NoticeMesg .= " Possible Values : $PossibleInput" if( $PossibleInput ne "" );
	   print STDERR "\t*** $NoticeMesg\n";
           $InputRetry++;
           if( $InputRetry == $RetryCount ) {
	      print STDERR "\t*** You have exceed maximum retry count($RetryCount)\n";
              CleanUp(undef,undef);
	   }
           next;
	 } else {
	    $IsRetryRequired = 0;
	    last;
	 }
     }
  }
  if ($noEcho) {
     system("stty echo");
     $noEcho = 0;
  }
  return $read;
}

=begin html

=head2 ChangeValue($;$)

=begin html
Asks for a confirmation whether a parameter needs to be changed or not.  Returns 1 in case a change is required, 0 if not.<br>
The following parameters are used:
<table>
  <tr><td>string</td><td>A string to display as prompt to the user</td></tr>
  <tr><td>key</td><td>Indicates an entry from the parameter defintions (prompt hash)</td></tr>
</table>

=end html

=cut

sub ChangeValue($;$) {
  my ($string, $key) = @_;
  my $response;
  my $label = "";
  $label .= "$string " if( defined $string );
  if (defined $key ) {
     if( defined($prompts{$key}{STRING})) {
         $label .= "\"$prompts{$key}{STRING}\" value ";
     } else {
        $label .= "$key " if( defined $key );
     }
  }
  $changeValuePrompt{'STRING'} = $label; 
  $response = ReadValue( "Change", \%changeValuePrompt );
  return ($response =~ /^[yY]$/);
}


sub PopulateConfig($$) {
    my $OUT = shift;
    my $promptsHash = shift;
   
    my ( $defValue, $confValue ) = ( "", "", "" );

    foreach my $key (sort { $promptsHash->{$a}{ORDER} <=> $promptsHash->{$b}{ORDER} } keys %$promptsHash) {
        $defValue  = (defined $promptsHash->{$key}{'VALUE'}{'DEFAULT'}) ? $promptsHash->{$key}{'VALUE'}{'DEFAULT'} : "";
        if( $DEBUG ) {
	    my $config = (defined $promptsHash->{$key}{'VALUE'}{'CONFIGVALUE'}) ? $promptsHash->{$key}{'VALUE'}{'CONFIGVALUE'} : "";
            if( defined($promptsHash->{$key}{SPECIAL} )) {
                $config = "Config:*****:" if( $config ne "" );
                my $defValueLabel = ( $defValue ne "" ) ? "*****" : "";
                my $string = sprintf("Order:%03d Key:%-25s Default:'%s' %s",$promptsHash->{$key}{ORDER}, "'$key'", $defValueLabel, $config);
                print "$string\n";
            } else {
                $config = "Config:$config:" if( $config ne "" );
                my $string = sprintf("Order:%03d Key:%-25s Default:'%s' %s",$promptsHash->{$key}{ORDER}, "'$key'", $defValue, $config);
                print "$string\n";
	    }
        }
        if( (defined $promptsHash->{$key}{'VALUE'}{'CONFIGVALUE'}) and ($key ne 'VERSION' ) ) {
	    $confValue = $promptsHash->{$key}{'VALUE'}{'CONFIGVALUE'};
        } else {
	    $confValue = $defValue;
        }
	if( defined $promptsHash->{$key}{'PROMPT-NA'} ) { 
           if ( (defined($promptsHash->{$key}{SPECIAL}) ) and
                (defined $promptsHash->{$key}{'VALUE'}{'CONFIGVALUE'}) ) {
	         $promptsHash->{$key}{SPECIAL} = undef;
           }
           WriteValues( $OUT, $promptsHash->{$key}, $key, $confValue );
	   next;
	}
        #if (defined($promptsHash->{$key}{EXTERNAL})) {
        #  # Special input routine
        #  &{$promptsHash->{$key}{EXTERNAL}}(\*OUT, $key, $defValue, 1, undef, $DEBUG, undef);
        #  next;
        #} 
        if (defined($promptsHash->{$key}{STRING})) {
	    my $isPromptReq = 1;
            ### While modify existing configuration,
            ### Prompt a user to continue with old value or not for SPECIAL(encrypt)
            if ( defined $promptsHash->{$key}{SPECIAL} ) {
                 if( defined $promptsHash->{$key}{'VALUE'}{'CONFIGVALUE'} ) { 
                    if( ! ChangeValue("CONFIRM to change ", $key) ) {
		       $isPromptReq = 0;
                       $promptsHash->{$key}{SPECIAL} = undef;
	            }
                 }
            }
            $confValue = ReadValue( $key, $promptsHash->{$key} ) if( $isPromptReq );
	}
        if( defined $promptsHash->{$key}{IMPACT} and ( $promptsHash->{$key}{'VALUE'}{'TYPE'} eq "B" ) ) {
	    if( $confValue =~ /^([nN])$/ ) {
                print "Not to prompt; Impacting keys are : @{$promptsHash->{$key}{IMPACT}} \n" if $DEBUG;
		for my $ImpactKeys ( @{$promptsHash->{$key}{IMPACT}} ) {
			if( defined $promptsHash->{$ImpactKeys} ) {
			    $promptsHash->{$ImpactKeys}{'PROMPT-NA'} = 1;
			}
		}
	    }
        }
        WriteValues( $OUT, $promptsHash->{$key}, $key, $confValue );
   }
}

sub WriteValues ($$$$) {
    my $OUT = shift;
    my $promptHash = shift;
    my $key = shift;
    my $value = shift;
    my $crypt = Crypt::Lite->new('debug' => 0);
    if ( defined($promptHash->{COMMENT}) ) {
        print $OUT "#$_\n" foreach (@{$promptHash->{COMMENT}}); 
    }
    if (defined($promptHash->{SPECIAL})) {
        my $encoded = $crypt->encrypt($value, CYPHER);
        print OUT "$key=\"$encoded\"\n";
    } elsif ($promptHash->{'VALUE'}{'TYPE'} eq "D") {
        print OUT "$key=$value\n";
    } elsif ($promptHash->{'VALUE'}{'TYPE'} eq "B") {
       $value =~ tr/[a-z]/[A-Z]/;
       print OUT "$key=\"$value\"\n";
    } else {
        print OUT "$key=\"$value\"\n";
    }
    return;
}

sub RenameConfigFile ($$) {
  my $sourceFile = shift;
  my $destFile  = shift;
  if( ! -f $sourceFile ) {
       print STDERR "File $sourceFile not exits\n";
       exit -1;
  }
  if (!rename("$sourceFile", "$destFile")) {
    print STDERR "Failed to rename from $sourceFile to $destFile: $!\n";
    exit -1;
  }
  if (! chmod(0664, "$destFile")) {
      print STDERR "Failed to chmod 664 $destFile: $!\n";
      exit -1;
  }
  return;
}


=head2 NeedsUpdate($)

=begin html
This function returns true if the configuration passed in $currentConfig requires an update when compared to the contents of the prompts hash table
as defined inside the PBMT configuration module.

=end html

=cut
sub NeedsUpdate($) {
  my ($currentConfig) = @_;
  my $needUpdate = 0;

  foreach my $key (sort { $prompts{$a}{ORDER} <=> $prompts{$b}{ORDER} } keys %prompts) {
#    $needUpdate = 1 if (!defined($currentConfig->{$key}));
    if (defined($currentConfig->{$key})) {
      $needUpdate = 1 if (!defined($prompts{$key}{STRING}) and ($prompts{$key}{VALUE}{DEFAULT} ne $currentConfig->{$key}) and
                         ($key eq "VERSION"));
    } else {
      # New key, so an update is required for sure.
      $needUpdate = 1;
    }
  }
  return $needUpdate;
}



=begin html

=head2 UpdateConfig($)

=begin html
Update an existing parameter configuration file:
<ul>
  <li>Read the exisiting file</li>
  <li>Ask whether the parameters currently present need modification and prompt for them if yes</li>
  <li>Build a list of new parameters introduced in this version and prompt for the value</li>
</ul>

=end html

=cut

sub UpdateConfig() {

    my $dirName = dirname($0);
    my %config = Utils::ReadConfig("$dirName/param.cfg");

    $DEBUG = (defined $params->{DEBUG} and $params->{DEBUG} == 1 ) ? 1 : 0;

    RenameConfigFile( "$dirName/param.cfg", "$dirName/param.cfg.sav" );
    $renamed = 1;

    ### >>UPDATE-CHECK Update required or Not
    my $key = 'VERSION';
    my $updateExisting = 0;
    my ($forceUpdate,$versionUpdate,$newKeys) = (0, 0, "");
    if ( !defined $config{$key} ) {
       print STDERR "NOTICE : VERSION not exists, Update is required\n";
       $forceUpdate = 1;
    } else {
       my $updateExisting = ChangeValue("Modify existing configuration (Y/N) [N]", undef);
       $forceUpdate = 1 if( $updateExisting );
       if( (!$forceUpdate) and
           (defined $prompts{$key}{VALUE}{DEFAULT}) and
           ($prompts{$key}{VALUE}{DEFAULT} ne $config{$key} )
       ) {
           print STDERR "NOTICE : VERSION is not same, Update is required\n";
           $versionUpdate = 1;
       }
       ### Check any mandate key is not exists in configuration file, With help of optional jeys
       foreach my $key (keys %prompts) {
           if( defined($config{$key}) ) {
             $prompts{$key}{'PROMPT-NA'} = 1 if(!$forceUpdate);
             $prompts{$key}{'VALUE'}{'CONFIGVALUE'} = $config{$key};
           } else {
             $newKeys .= "$key ";
           }
       }
       print STDERR "NOTICE : new Prompt(s) are found, Update is required\n" if(!$forceUpdate and $newKeys ne "");
    }

    if( $forceUpdate == 0 and $versionUpdate == 0 and $newKeys eq "" ) {
       print STDERR "NOTICE : Updating Congfiguration file is not required\n\n";
       RenameConfigFile( "$dirName/param.cfg.sav", "$dirName/param.cfg" );
       $renamed = 0;
       return 0;
    }
    ### >>UPDATE-CHECK ENDS HERE


    print "Proceeding to update existing configuration\n" if $DEBUG;
    if( !$versionUpdate ) {
       print "\t+-----------------------------------------------------------------+\n";
       print "\t| Use \"\" in case you want to reset a parameter to an empty string |\n";
       print "\t+-----------------------------------------------------------------+\n";
    }

    if (! open(OUT, ">" . "$dirName/param.cfg")) {
      print STDERR "Failed to open $dirName/param.cfg for writing: $!\n";
      RenameConfigFile( "$dirName/param.cfg.sav", "$dirName/param.cfg" );
      $renamed = 0;
      return;
    }
    PopulateConfig(\*OUT, \%prompts );
    close OUT;
    if (! chmod(0664, "$dirName/param.cfg")) {
      print STDERR "Failed to chmod 664: $!\n";
    }
    if( -f "$dirName/param.cfg.sav" ) {
	unlink "$dirName/param.cfg.sav" or print STDERR "Failed to remove $dirName/param.cfg.sav : $!\n";
    }
    $renamed = 0;
    return 0;
}

=begin html

=head2 NewInstall($)

=begin html
Create a new parameter configuration file by prompting for the values.

=end html

=cut

sub NewInstall() {
  my $dirName = dirname($0);
  $DEBUG = (defined $params->{DEBUG} and $params->{DEBUG} == 1) ? 1 : 0;

  print "Create new configuration\n" if $DEBUG;

  if (! open(OUT, ">" . "$dirName/param.cfg")) {
    print STDERR "Failed to open $dirName/param.cfg for writing: $!\n";
    #RenameConfigFile( "$dirName/param.cfg.sav", "$dirName/param.cfg" );
    $created = 0;
    return;
  }
  $created = 1;
  PopulateConfig( \*OUT, \%prompts );
  close OUT;
  if (! chmod(0664, "$dirName/param.cfg")) {
    print STDERR "Failed to chmod 664: $!\n";
  }
  $created = 0;
}

=begin html

=head2 DefaultInstall($)

=begin html
Create a new parameter configuration file by configuring all default values.

=end html

=cut

sub DefaultInstall() {
  my $dirName = dirname($0);

  $DEBUG = (defined $params->{DEBUG} and $params->{DEBUG} == 1) ? 1 : 0;

  print "Create new configuration\n" if $DEBUG;

  if (! open(OUT, ">" . "$dirName/param.cfg")) {
    print STDERR "Failed to open $dirName/param.cfg for writing: $!\n";
    RenameConfigFile( "$dirName/param.cfg.sav", "$dirName/param.cfg" );
    $created = 0;
    return;
  }
  $created = 1;
  ### Assign all prompt to set PROMPT-NA to enable non-interactive mode
  $prompts{$_}{'PROMPT-NA'} = 1 foreach (keys %prompts);
  $prompts{'DEFAULTCLIUSER'}{VALUE}{DEFAULT}   = "$DEFAULTCLIUSER";
  $prompts{'DEFAULTCLIPWD'}{VALUE}{DEFAULT}    = "$DEFAULTCLIPWD";
  $prompts{'DEFAULTTL1USER'}{VALUE}{DEFAULT}   = "$DEFAULTTL1USER";
  $prompts{'DEFAULTTL1PWD'}{VALUE}{DEFAULT}    = "$DEFAULTTL1PWD";
  ### Do we need to NULLify the SNMPv3, Y not default value ?? ###FIX-IT
  $prompts{'NT_COMMUNITY'}{VALUE}{DEFAULT}     = "$DEF_NT_COMMUNITY";
  ### Why can't we check NOT-NULL?? ##FIX-IT
  if( defined $DEF_ACT_CDEFILE) {
     $prompts{'ACTCDEFILE'}{VALUE}{DEFAULT} = "$DEF_ACT_CDEFILE";
  }
  ### Why can't we check NOT-NULL?? ##FIX-IT
  if( defined $DEF_NEW_CDEFILE) {
     $prompts{'NEWCDEFILE'}{VALUE}{DEFAULT} = "$DEF_NEW_CDEFILE";
  }
  if( defined $DEF_NEW_CDEDIR and $DEF_NEW_CDEDIR ne "" ) {
     $prompts{'NEWCDEFILE'}{VALUE}{DEFAULT} = "$DEF_NEW_CDEDIR";
  }
  ### NEED TO CLARIFY
  #   1)Why we set NULL for SNMP, Why not default
  #   2)Why we missed to encrypt the specify value type
  PopulateConfig(\*OUT, \%prompts );
  close OUT;
  if (! chmod(0664, "$dirName/param.cfg")) {
    print STDERR "Failed to chmod 664: $!\n";
  }
  $created = 0;
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file.

=end html

=cut

sub AdjustToolVars() {
  #my $crypt = Crypt::Lite->new('debug' => 0);
  my @defaults;

  if (defined($params->{DEFAULT})) {
    @defaults = split(':', $params->{DEFAULT});
    #$DEFAULTCLIUSER = $crypt->encrypt($defaults[0], CYPHER);
    #$DEFAULTCLIPWD = $crypt->encrypt($defaults[1], CYPHER);
    #$DEFAULTTL1USER = $crypt->encrypt($defaults[2], CYPHER);
    #$DEFAULTTL1PWD = $crypt->encrypt($defaults[3], CYPHER);
    #$DEF_NT_COMMUNITY = $crypt->encrypt($defaults[4], CYPHER);
    $DEFAULTCLIUSER = $defaults[0] if (defined($defaults[0]));;
    $DEFAULTCLIPWD = $defaults[1] if (defined($defaults[1]));
    $DEFAULTTL1USER = $defaults[2] if (defined($defaults[2]));
    $DEFAULTTL1PWD = $defaults[3] if (defined($defaults[3]));
    $DEF_NT_COMMUNITY = $defaults[4] if (defined($defaults[4]));
    $DEF_ACT_CDEFILE = $defaults[5] if (defined($defaults[5]));
    $DEF_NEW_CDEFILE = $defaults[6] if (defined($defaults[6]));
  }

  if (defined($params->{CDEDIROPT})) {
    $DEF_NEW_CDEDIR = $params->{CDEDIROPT};
  }
}

=head2 CleanUp($;$)

=begin html
Called when the user aborts the input using Ctrl-C

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $dirName = dirname($0);
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  print STDERR sprintf("%s::%s - Caught signal '%s'\n", PBMT_VERSION, $VERSION, $signal) if( defined $signal);
  if ($renamed) {
    print STDERR "Failed to rename $dirName/param.cfg.sav: $!\n" if (!rename("$dirName/param.cfg.sav", "$dirName/param.cfg"));
  }
  system("stty echo") if $noEcho;
  if ($created) {
    print STDERR "Failed to delete $dirName/param.cfg: $!\n" if ! unlink "$dirName/param.cfg";
  }
  exit 0;
}


#-- end of library
1;
