
=head1 PBMT online SW reduction module

=begin html
This PERL module implements the reduction of the online SW.

=end html

=cut

#==============================================================================
#-- File:         ReduceSWP.pm
#-- Date:         13.08.2004
#-- Company:      Alcatel Bell - TEC
#-- Author:       Raghu Raman Nagarajan
#------------------------------------------------------------------------------
#-- Function:     Script to reduce the 5.x sandisk package
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#--               There is no warranty about it, use of this script is at own
#--               risk
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel BND 2001
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 03082004 : First implementation,BDFhw15550
#--   1.0.1 - 13082004 : BDFhw23431,BDFhw23684
#--   1.0.2 - 20082004 : BDFhw24043
#--   1.0.3 - 17092004 : BDFhw26222
#--   1.0.4 - 13102004 : BDFhw28354 (joby.georged@wipro.com)
#--   1.0.5 - 09022005 : BDFhw37521 (eric.kwanten@alcatel.be)
#--                      -> 9Telecom problem
#--                      BDFhw..... (eric.kwanten@alcatel.be)
#--                      -> Add version in script output
#--   1.0.6 - 16012006 : Incorporate in Push-button migration script set **BBOG**
#--   1.0.7 - 22032006 : Fix for BDFhw83406: don't tftp in /Sw put in
#--                      /pub/Sw/Download as this fools the redundancy system.
#--   1.0.8 - 22082006 : Extra debugging and one more exit code if no OSWP to activate
#--                      could be found.
#--   1.0.9 - 06102006 : ANTms17041: Print MIB version read from the node in case we do
#--                      not find the expected MIB version pattern.
#--   1.0.10 - 16102006: SW pkg reduction identifiers added: no longer alter the
#--                      SW version but changes characters 5 and 6 of the OSWP name.
#--   1.0.11 - 18102006: ANTms18614 Syntax error on line 432
#--   1.4.3  - 20102006: Converted to perl package for error reporting (integration
#--                      in AMS/AWS (main code body not altered).
#--            07112006: Claim transaction before start OSWP and SWP DL and performing
#--                      SNMP sets
#--   1.4.35 - 15112006: ANTms22602 SW not reduced
#--   1.4.36 - 16112006: ANTms19172 - use IP@ from OSWP file to set transaction and not
#--                      the IP@ of the host.
#--   1.4.37 - 2007/03/29 : ANTms40542: check on error code when reseting transaction
#--                          owner
#--   1.4.38 - 2007/06/08 : ANTms49877: Timing issue between RP dld, SetValueAtIndex and GetValueAtIndex
#--   1.4.39 - 2007/10/30 : ANTms60696: SW reduction fails when '\r' characters are present in the SWP file.
#--   1.4.40 - 2007/09/20 : included more debugging on SW management in case (SNMP) OSWP manipulation would
#--                         result in non-success.
#--   1.4.41 - 2008/12/18 : ANTmt36711 - Support R4.x MIB
#--   1.4.42 - 2009/03/25 : ANTmt49436: Use IP@ of ISAM rather then node name when opening up tftp.
#--   1.4.43 - 2009/12/01 : fix for TYPE-K file 
#--   1.4.44 - 2010/03/09 : ANTmg01050 - Total_MiB label change from Ex.x.y.z to Vx.x.x.y.z
#--   1.4.45 - 2010/04/18 : ALU01123774
#--   1.4.46 - 2012/05/01 : ALU01778224 
#--   1.4.47 - 2012/05/10 : Extend fix ALU01778224 to previous Releases 
#--   1.4.48 - 2012/07/24 : ALU01788919
#--   1.4.49 - 2012/07/29 : ALU01813641
#--   1.4.50 - 2013/10/14: ALU01913931 Adapted for enable read/write on /Sw/OSWPx
#--   1.4.51 - 2013/10/19: AT&T issues ALU01913513, ALU01913928, ALU01913529
#--   1.4.52 - 2013/10/19: ALU01913491, ALU01913501
#--   1.4.53 - 2013/10/19: ALU01913513
#--   1.4.54 - 2013/11/27: ALU01933941
#--   1.4.55 - 2014/01/27: ALU01944943 & ALU01944556 
#--   1.4.56 - 2014/02/02: ALU01948427
#--   1.4.57 - 2014/02/08: Modification for getting linux server ip for linux 5.x 
#--   1.4.58 - 2014/03/02: ALU01956469
#--   1.4.59 - 2014/03/26: ALU01963856
#--   1.4.60 - 2014/03/28: ALU01959833 and clone FR ALU01962435
#--   1.4.61 - 2014/04/06: ALU02061738
#--   1.4.62 - 2015/07/03: ALU02095810
#--   1.4.63 - 2015/07/06: ALU02097337
#--   1.4.64 - 2015/07/23: ALU02098661
#--   1.4.65 - 2015/07/29: ALU02106765
#--   1.4.66 - 2015/08/27: ALU02107668
#--   1.4.67 - 2015/09/01: ALU02119959 
#--   1.4.68 - 2015/09/09: ALU02119959 : Ensure T&D debug mode is from R52, Not from R53
#--   1.4.69 - 2015/01/07: ALU02163612
#--   1.4.70 - 2015/02/02: ALU02167196, ALU02168421
#--   1.4.71 - 2015/05/13: ALU02202972, ALU02213657
#--   1.4.72 - 2016/05/27: ALU02188053
#--   1.4.73 - 2016/06/24: ALU02237966
#--   1.4.74 - 2016/07/15: ALU02237966
#--   1.4.75 - 2016/08/31: ALU02237462
#--   2.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "ReduceSWP.pm - 2.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "ReduceSWP.pm - 2.1.0 -- 2018/06/20";
#==============================================================================

package ReduceSWP;

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2101-DFZZA">Net::Telnet</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use warnings 'all';
use FileHandle;
use POSIX ":sys_wait_h";
use File::Basename;
use Config; 
use Sys::Hostname;
use Socket;

use Net::SNMP;
use Net::TFTP;

use ErrorCodes qw(:SWRD);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use ErrorCodes qw(:EXIT_CODES :SWDL);#New
require Octopus;
require Utils;
require FamilyCommon;
require IsamCli;
use SecureDebugTAndD;#New
use SNMPComm;
use vars qw(%toolConsts);#New
#==============================================================================
#== Constants
#==============================================================================
*toolConsts = \%ToolCommon::toolConsts; #New

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetError();
sub GetValueAtIndex($$$$$);
sub GetValuesAtIndex($$$$$);
sub GetColumn($$$$$$);
sub GetMibVersion($$$$);
sub GetBoardList($$$$);
sub GetActiveSwInfo($$$$$$$);
sub ReduceSwpFile($$$$$$$);
sub MakeNewOswpFile($$$$$$$$);
sub DownloadAndCommitNewSW($$$$$$$$$;$$);
sub ReduceActiveSWP($$$$$$$$;$);
sub ResetGlobals();
#==============================================================================
# Constants

my $MIBVERSIONOID=".1.3.6.1.4.1.637.61.1.9.13.0";
my $TRANSACTOWNER=".1.3.6.1.4.1.637.61.1.11.1.0";
my $BOARD_5X_OID = '.1.3.6.1.4.1.637.61.1.23.3.1';
my $PLANNEDBOARD_5X_OID ="$BOARD_5X_OID.2";
my $INSERTEDBOARD_5X_OID ="$BOARD_5X_OID.3";

my $ACTIVEOSWP_5X_OID = ".1.3.6.1.4.1.637.61.1.24.1.1.1.4";
my $SWPKG_5X_OID = ".1.3.6.1.4.1.637.61.1.24.1.1.1.2";
my $OSWPINDEX_5X_OID = ".1.3.6.1.4.1.637.61.1.24.1.1.1.1";
my $FREESPACEOID = ".1.3.6.1.4.1.637.61.1.24.1.7.0";
my $OPERAVAILSTATUS=".1.3.6.1.4.1.637.61.1.24.1.1.1.3";
my %oswpAvailStates = ("empty" => 1,
		       "enabled" => 2,
		       "disabled" => 3,
		       "downloading" => 4,
		       "aborting" => 5);
my %oswpAvailEnum = (1 => "empty",
		     2 => "enabled",
		     3 => "disabled",
		     4 => "downloading");
my $OSWPACTIVATION = ".1.3.6.1.4.1.637.61.1.24.1.1.1.6";
my $OSWPOPERACTSTATUS = ".1.3.6.1.4.1.637.61.1.24.1.1.1.4";
my %oswpActStates = ("active" => 1,
		     "notActive" => 2);
my %oswpActEnum = (1 => "active",
		   2 => "notActive");
my $OPERCOMMITSTATUS = ".1.3.6.1.4.1.637.61.1.24.1.1.1.5";
my %oswpCommitStates = ("committed" => 1,
			"unCommitted" => 2,
			"committing" => 3);
my %oswpCommitEnum = (1 => "committed",
		      2 => "unCommitted",
		      3 => "committing");
my $OSWPCOMMITMENT = ".1.3.6.1.4.1.637.61.1.24.1.1.1.7";
my $OSWPDLERROR = ".1.3.6.1.4.1.637.61.1.24.1.1.1.12";
my %ftpErrors = (0 => "notDefined",
		 1 => "fileNotFound",
		 2 => "accessViolation",
		 3 => "diskFull",
		 4 => "illegalTftpOperation",
		 5 => "unknownTransferId",
		 6 => "fileAlreadyExists",
		 7 => "noSuchUser",
		 8 => "syntaxErrorDLFile",
		 9 => "lackOfStorageResources",
		 10 => "systemRestart",
		 11 => "noError",
		 12 => "syncFailure",
		 13 => "invalidTftpDeamon",
		 14 => "invalidSftpDeamon",
		 15 => "invalidFtpDeamon");
my $OSWPERRORFILENAME = ".1.3.6.1.4.1.637.61.1.24.1.1.1.13";
my $OSWPERRORFILESERVER = ".1.3.6.1.4.1.637.61.1.24.1.1.1.14";
my $OSWPSYNTAXRE = "[\\w\\.\\_\\-\\:\\/]+";
my $RETRIES = 5;
my $OSWPNameMatch = 0;
my $TmpSWDMinVar = "W";
my $TmpSWDRedVar = "Y";
my $ReductionId = "R";

my $ScriptName = "ReduceSWP.pl";

my $ERROR = "";

my $config; 

my $connSSH; ##new


=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 GetError()

=begin html
Return the error (string) detected when accessing the ISAM via the CLI.

=end html

=cut

sub GetError() {
  return $ERROR;
}

sub ResetGlobals() {
   $connSSH = undef;
   $OSWPNameMatch = 0;
}


=head2 GetValueAtIndex($$$$$)

=begin html
Get the value of $oid using the opened SNMP session context $session for the NE defined by $asam.  $verbose indicates whether additional debugging
needs to be switched on (debug info is written to the file identified by $fh).<br>
The value read is returned in case of success otherwise the undefined value is returned.

=end html

=cut

sub GetValueAtIndex($$$$$) {
  my ($fh, $asam, $session, $oid, $verbose) = @_;   # -- 1.0.6

  my (@oids, $reply);
  $oids[0] = $oid;
  my $version = $session->version();
  if ($version < 2) {
      # SNMPv1/SNMPv2
      $reply = $session->get_request(-varbindlist => \@oids);
  } else {
      # SNMPv3
      $reply = $session->get_request(-varbindlist => \@oids,
                                     -contextname => NT_CONTEXT);
  }
  my $value = $reply->{$oids[0]};

  if (! defined($value)) {
    ToolCommon::Log($fh, "Get $oid value FAILED ($asam->[1])\n  ERROR: " . $session->error() . "\n");  # -- 1.0.6
    $ERROR = sprintf("SNMP error. Get $oid value FAILED: %s", $session->error());
    return undef;
  }
  return $value;
}

=head2 GetValuesAtIndex($$$$$)

=begin html
Get the value of $oid using the opened SNMP session context $session for the NE defined by $asam.  $verbose indicates whether additional debugging
needs to be switched on (debug info is written to the file identified by $fh).<br>
The value read is returned in case of success otherwise the undefined value is returned.

=end html

=cut

sub GetValuesAtIndex($$$$$) {
  my ($fh, $asam, $session, $oids, $verbose) = @_;   # -- 1.0.6
  my (@oids, $reply);
  my $version = $session->version();
  if ($version < 2) {
      # SNMPv1/SNMPv2
      $reply = $session->get_request(-varbindlist => \@{$oids});
  } else {
      # SNMPv3
      $reply = $session->get_request(-varbindlist => \@{$oids},
                                     -contextname => NT_CONTEXT);
  }

  if (! defined($reply)) {
    ToolCommon::Log($fh, "Get values FAILED ($asam->[1])\n  ERROR: " . $session->error() . "\n");  # -- 1.0.6
    $ERROR = sprintf("SNMP error. Get values FAILED: %s", $session->error());
    return undef;
  }
  return $reply;
}

=head2 GetColumn($$$$$$)

=begin html
Get a column defined by $oid using the opened SNMP session context $session for the NE defined by $asam.  $verbose indicates whether additional
debugging needs to be switched on (in which case $desc represents a description of the OID being used in this get_table request.  Debug info is
written to the file identified by $fh).<br>
A reference to the values read is returned in case of success otherwise the undefined value is returned.

=end html

=cut

sub GetColumn($$$$$$) {
  my ($fh, $asam, $session, $oid, $descr, $verbose) = @_;   # -- 1.0.6

  if (defined($descr)) {
    ToolCommon::Log($fh, "Getting $descr...\n") if ($verbose);
  }

  my $result;
  my $version = $session->version();
  if ($version == 0) {
    $result = $session->get_table($oid);
  } else {
    $result = $session->get_table(-baseoid => $oid,-maxrepetitions => 40);  ## for SNMP V2 & V3
  }

  if( !defined( $result)) {
    ToolCommon::Log($fh, "Get $descr FAILED ($asam->[1])\n  ERROR: " .  $session->error() ."\n");   # -- 1.0.6
    $ERROR = sprintf("SNMP error.  Get $descr FAILED: %s",  $session->error());
    return undef;
  }
  return $result;
}

=head2 GetMibVersion($$$$)

=begin html
Get the MIB version of the NE defined by $asam using the opened SNMP session $session.  $verbose indicates whether additional debugging needs to be
switched on (debug info is written to the file identified by $fh).<br>
The values read is returned in case of success (and a known MIB version has been read) otherwise the undefined value is returned.

=end html

=cut

sub GetMibVersion($$$$) {
  my ($fh, $asam, $session, $verbose) = @_;
  my $reply;
  my $value = undef;
  my $retries = 0;

  ToolCommon::Log($fh, "Getting Mib Version ($asam->[1])...\n") if ($verbose);   # -- 1.0.6
  my $version = $session->version();
  my @oids;
  $oids[0] = $MIBVERSIONOID;
  while (! defined($value) && $retries < 5) {
     if ($version < 2) {
         # SNMPv1/SNMPv2
         $reply = $session->get_request(-varbindlist => \@oids);
     } else {
         # SNMPv3
         $reply = $session->get_request(-varbindlist => \@oids,
                                        -contextname => NT_CONTEXT);
     }
    $value = $reply->{$MIBVERSIONOID};
    if( defined( $value)) {
      $value =~ s/\000//;
    } else {   
      $retries++;
    }
  }
  if (! defined($value)) {
    ToolCommon::Log($fh, "Get MibVersion FAILED ($asam->[1])\n");   # -- 1.0.6
    $ERROR = "Failed to retrieve MIB version";
  } else {
    if( ! ($value =~ /^3FE.*_(E[2-4]|V[4-6]).*/) ) {
      ToolCommon::Log($fh, "Not appropriate 2.x/3.x/4.x/5.x Mib Version: $value\n");   # -- 1.0.6
      $ERROR = "MIB version not supported ($value)";
      $value = undef;
    }
  }
  return $value;
}

=head2 GetBoardList($$$$)

=begin html
This function determines the list of boards planned and configured in the node.

=end html

=cut

sub GetBoardList($$$$) {
  my ($logFile, $asam, $snmpSession, $verbose) = @_;
  my $plannedBoards = GetColumn($logFile, $asam, $snmpSession, $PLANNEDBOARD_5X_OID, "Planned boards", $verbose);    # -- 1.0.6
  my $insertedBoards = GetColumn($logFile, $asam, $snmpSession,  $INSERTEDBOARD_5X_OID, "Inserted boards", $verbose);   # -- 1.0.6
  my @oid;
  my $slot;
  my %a_boards;
  my $boardState;
  my $plannedBoard;
  my $insertedBoard;

  ToolCommon::Log($logFile, "Getting planned and inserted boards...");
  if (defined($plannedBoards) && defined($insertedBoards)) {  # -- 1.0.6
    foreach (Net::SNMP::oid_lex_sort(keys(%{$plannedBoards}))) {
      chomp;
      @oid = split(/\./, $_);
      $slot = $oid[-1];    # Slot is last identifier in the OID
      $insertedBoard = uc($insertedBoards->{"$INSERTEDBOARD_5X_OID.$slot"});
      $plannedBoard = uc($plannedBoards->{"$PLANNEDBOARD_5X_OID.$slot"});
      next if ((($insertedBoard eq "EMPTY") || ($insertedBoard eq "NOT_ALLOWED") || ($insertedBoard eq "UNKNOWN")) &&
	       (($plannedBoard eq "ANY_BOARD") || ($plannedBoard eq "NOT_PLANNED")));
      if (($plannedBoard ne "ANY_BOARD") && ($plannedBoard ne "NOT_PLANNED")) {
	if (!defined($a_boards{$plannedBoard})) {
	  $a_boards{$plannedBoard} = 1;
	  ToolCommon::Log($logFile, "Added planned board $plannedBoard\n") if $verbose;
	} else {
	  ToolCommon::Log($logFile, "$plannedBoard already present\n") if $verbose;
	}
      }
      if (($insertedBoard ne "EMPTY") && ($insertedBoard ne "NOT_ALLOWED") && ($insertedBoard ne "UNKNOWN") && $insertedBoard ne $plannedBoard) {
        if (!defined($a_boards{$insertedBoard})) {
	  $a_boards{$insertedBoard} = 1;
	  ToolCommon::Log($logFile, "Added inserted board $insertedBoard\n") if $verbose;
	} else {
	  ToolCommon::Log($logFile, "$insertedBoard already present\n") if $verbose;
	}
      }
      # chomp;
    }
    ToolCommon::Log($logFile, "  OK\n");
    if ($verbose) {
      ToolCommon::Log($logFile, "The available boards are:\n") if ($verbose);
      foreach my $key (sort keys %a_boards) {
	ToolCommon::Log($logFile, "'$key'\n");
      }
    }
  } else {
    ToolCommon::Log($logFile, "  Failed\n");
  }
  return \%a_boards;
}

=head2 GetActiveSwInfo$$$$$$$$()

=begin html
This function determines the active OSWP and related SWP package.

=end html

=cut

sub GetActiveSwInfo($$$$$$$) {
  my ($logFile, $asam, $snmpSession, $dir, $tftp, $verbose, $path) = @_;
  my $activeOswpStatus = GetColumn($logFile, $asam, $snmpSession, $ACTIVEOSWP_5X_OID, "Active OSWP States", $verbose);   # -- 1.0.6
  # Find the Active OSWP;
  ToolCommon::Log($logFile, "Finding the active OSWP ...");
  my @swoid;
  my @actOswpLine; 
  my $act;
  my $actOswPkg;
  my $actOswpInd;

  foreach (Net::SNMP::oid_lex_sort(keys(%{$activeOswpStatus}))) {
    @swoid = split(/\./, $_);
    $act = $swoid[-1];
    next if($activeOswpStatus->{"$ACTIVEOSWP_5X_OID.$act"} ne $oswpActStates{"active"});
    $actOswPkg = GetValueAtIndex($logFile, $asam, $snmpSession, "$SWPKG_5X_OID.$act", $verbose);   # -- 1.0.6
    if (!defined($actOswPkg)) {
      ToolCommon::Log($logFile, "  FAILED: retrieving active OSWP package failed\n");
      return (EC_NO_ACT_SWP, undef, undef);   # -- 1.0.6
    }
    @actOswpLine = split(/\/+/,$actOswPkg);
    $actOswPkg = $actOswpLine[-1]; 
    $actOswpInd  = GetValueAtIndex($logFile, $asam, $snmpSession, "$OSWPINDEX_5X_OID.$act", $verbose);   # -- 1.0.6
    if (!defined($actOswpInd)) {
      ToolCommon::Log($logFile, "  FAILED: retrieving active OSWP index failed\n");
      return (EC_NO_ACT_OSWP, undef, undef);   # -- 1.0.6
    }
  }
  ToolCommon::Log($logFile, "  OK\n");
        my $swOSWPX = "/Sw/OSWPx";
	ToolCommon::Log($logFile, "Going to enable read/write on /Sw/OSWPx...");
        if (defined($connSSH)) {
           ToolCommon::Log($logFile, "Reduced software download is happening on T&D via Secured Debug mode !!\n");
           if(SecureDebugTAndD::SshEnableSwWrite($connSSH, $dir,$swOSWPX)) {
            $ERROR = "Enable write on /Sw/OSWPx failed";
            return EC_SEC_SSH_ENABLE_WRITE_FAIL;
           }
        } else {
	    if (! Octopus::EnableSwWrite($verbose, $dir, $asam, $path, $swOSWPX)) {
                $ERROR = "Enable write on /Sw/OSWPx failed";
        	return EC_OCTO_FAIL;   # -- 1.0.6
            }
        }

  if(!$params->{NOTFTP}) {
	ToolCommon::Log($logFile, "Retrieving OSWP descriptor file using TFTP...");
	$tftp->binary();
	my $ret = $tftp->get("/Sw/OSWP$actOswpInd/$actOswPkg", "$dir/$actOswPkg");
	if (not defined($ret) or ($ret ne "")) {
		ToolCommon::Log($logFile, "Ist attempt failed and going ahead with 2nd ATTEMPT: tftp get /Sw/OSWP$actOswpInd/$actOswPkg...");
                $tftp = undef;
		$tftp = Net::TFTP->new($asam->[0]);   # Changed to use IP@
		if (! defined($tftp)) {
	  		ToolCommon::Log($logFile, "  FAILED: Cannot open tftp to '$asam->[1]'\n");
	  		return EC_TFTP_OPEN_FAIL;
		}
		sleep(2);
	        my $ret = $tftp->get("/Sw/OSWP$actOswpInd/$actOswPkg", "$dir/$actOswPkg");
	        if (not defined($ret) or ($ret ne "")) {
			ToolCommon::Log($logFile, sprintf("  FAILED: 2nd ATTEMPT: tftp get /Sw/OSWP$actOswpInd/$actOswPkg failed: %s\n", $tftp->error()));
			$ERROR = sprintf("tftp get /Sw/OSWP$actOswpInd/$actOswPkg failed: %s", $tftp->error());
			return (EC_XFTP_GET_OSWP_FAIL, undef, undef);
		} 
	} 
  } else {
	  # use sftp xfer
	    #my $cmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD}\' \'$params->{ISAM}\' '/Sw/OSWP$actOswpInd/$actOswPkg|$dir/$actOswPkg' 'get'"; 
        my $ret = ToolCommon::SFTPClientWrap( $logFile, $config, $path, "/Sw/OSWP$actOswpInd/$actOswPkg|$dir/$actOswPkg", 'get', $asam->[0] );
        if ($ret) {
		ToolCommon::Log($logFile, sprintf("  FAILED: sftp get /Sw/OSWP$actOswpInd/$actOswPkg failed: \n"));
		$ERROR = sprintf("sftp get /Sw/OSWP$actOswpInd/$actOswPkg failed");
		return (EC_XFTP_GET_OSWP_FAIL, undef, undef);  
	} 
  }
  if (! open(OSWP, "$dir/$actOswPkg")) {
    ToolCommon::Log($logFile, "  Failed: Opening $dir/$actOswPkg FAILED: $!\n");
    $ERROR = "Opening $dir/$actOswPkg FAILED: $!";
    return (EC_OPEN_OSWP_FAIL, undef, undef);
  } else {
    ToolCommon::Log($logFile, "  OK\n");
  }
  #Finding the active SWP
  my $actSwPkg = "";
  my $actSwPkg1 = "";
  my $actOswPkg1 = "";
  my $row;
  my $asamCoreStr = "";

  ToolCommon::Log($logFile, "Finding the active SWP ...\n");
  while ($row = <OSWP>) {
    $row =~ s/\r//g;
    chomp $row;
    ToolCommon::Log($logFile, "Line read: $row\n") if $verbose;
    if ($row =~ /ASAM-CORE\s+\:\s+($OSWPSYNTAXRE)\s+(\d+\.\d+\.\d+\.\d+)\s+(\d+\.\d+\.\d+\.\d+);/) {
      my $oswpName = $1;
      ToolCommon::Log($logFile, "Found '$oswpName'\n") if $verbose;
      $actSwPkg = basename($oswpName);
      last;
    } else { #  parse ASAM-CORE information string and TFTP server ip-addr spread across multiple lines and concatenate it
       $asamCoreStr = $asamCoreStr . ' ' . $row;
       if ($row =~ /;\s*$/) {
           if($asamCoreStr =~ /ASAM-CORE\s+\:\s+($OSWPSYNTAXRE)\s+(\d+\.\d+\.\d+\.\d+)\s+(\d+\.\d+\.\d+\.\d+)/) {
              my $oswpName = $1;
              ToolCommon::Log($logFile, "Found '$oswpName'\n") if $verbose;
              $actSwPkg = basename($oswpName);
              last;
           }
       }
    }
  }
  close(OSWP);
  if ($actSwPkg ne "") {
         $actSwPkg1 = $actSwPkg;
         if($actSwPkg eq $actOswPkg) {
            $OSWPNameMatch = 1;
            $actSwPkg1 =~  s/(....).(.......)/$1$TmpSWDMinVar$2/;
         }
	

	ToolCommon::Log($logFile, "Going to enable read/write on /Sw ...");
		if (defined($connSSH)) {
           ToolCommon::Log($logFile, "Reduced software download is happening on T&D via Secured Debug mode !!\n");
           if(SecureDebugTAndD::SshEnableSwWrite($connSSH, $dir)) {
            $ERROR = "Enable write on /Sw failed";
            return EC_SEC_SSH_ENABLE_WRITE_FAIL;
           }
        } else {
	    if (! Octopus::EnableSwWrite($verbose, $dir, $asam, $path)) {
                $ERROR = "Enable write on /Sw failed";
        	return EC_OCTO_FAIL;  
            }
        }
 if(!$params->{NOTFTP}) {
		ToolCommon::Log($logFile, "Retrieving SWP descriptor file using TFTP...");
		if ($tftp->get("/Sw/$actSwPkg", "$dir/$actSwPkg1") ne "") {
			ToolCommon::Log($logFile, "1st attempt failed and going ahead with 2nd ATTEMPT tftp get /Sw/$actSwPkg...");
			$tftp = undef;
			$tftp = Net::TFTP->new($asam->[0]);   # Changed to use IP@
			if (! defined($tftp)) {
	  			ToolCommon::Log($logFile, "  FAILED: Cannot open tftp to '$asam->[1]'\n");
	  			return EC_TFTP_OPEN_FAIL;
			}
			sleep(2);
		        my $ret = $tftp->get("/Sw/$actSwPkg", "$dir/$actSwPkg1");
			if ($ret ne "") {
				ToolCommon::Log($logFile, sprintf("  FAILED :2nd ATTEMPT tftp get /Sw/$actSwPkg failed: %s\n", $tftp->error()));
				$ERROR = sprintf("tftp get /Sw/$actSwPkg failed: %s\n", $tftp->error());
				return (EC_XFTP_GET_SWP_FAIL, undef, undef);
			} 
		} 
	 } else {
		  # use sftp xfer
		        #my $cmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD}\' \'$params->{ISAM}\' '/Sw/$actSwPkg|$dir/$actSwPkg1' 'get'"; 
                my $ret = ToolCommon::SFTPClientWrap( $logFile, $config, $path, "/Sw/$actSwPkg|$dir/$actSwPkg1", 'get', $asam->[0] );
                if ($ret) {
		   ToolCommon::Log($logFile, sprintf("  FAILED: sftp get /Sw/$actSwPkg failed: \n"));
		   $ERROR = sprintf("sftp get /Sw/$actSwPkg failed: ");
		   return (EC_XFTP_GET_SWP_FAIL, undef, undef);
                }
	  }
	
      ToolCommon::Log($logFile, " Transfer OK\n");
    
    $actOswPkg1 = $actOswPkg;
  } else {
    $ERROR = "Failed to determine ASAM-CORE SWP in uploaded OSWP file.  Please check $dir/$actOswPkg";
    return (EC_XFTP_GET_SWP_FAIL, undef, undef);
  }
  #-- 1.0.5

  if ($verbose) {
    ToolCommon::Log($logFile, "***********************\n");
    ToolCommon::Log($logFile, "SWP File: $actOswPkg1\n");
    ToolCommon::Log($logFile, "***********************\n");
  }
  return (EC_SUCCESS, $actOswPkg, $actSwPkg1, $actOswpInd);
}

=head2 ReduceSwpFile($$$$$$$)

=begin html
This function reduces the SWP file based on the list of boards configured and planned.

=end html

=cut

sub ReduceSwpFile($$$$$$$) {
  my ($logFile, $dir, $actSwp, $newActSwp, $delAll, $boardList, $verbose) = @_;

  ToolCommon::Log($logFile, "Reducing SWP descriptor file to desired boardlist...");
  my $row;
  if (! open(MAIN, "$dir/$actSwp")) {
    ToolCommon::Log($logFile, "  FAILED: Could not open $dir/$actSwp for retrieving information: $!\n");
    $ERROR = " Could not open $dir/$actSwp: $!";
    return EC_OPEN_SWP_FAIL;
  }
  if (! open(NEW, ">$dir/$newActSwp")) {
    ToolCommon::Log($logFile, "  FAILED: Could not open $dir/$newActSwp for updating information: $!\n");
    $ERROR = "Could not open $dir/$newActSwp: $!";
    return EC_CREATE_SWP_FAIL;
  }
  # Note BBOG
  # This code heavily depends on the way the SWP file has been created.  If it would change this parsing
  # has to be revisited.  Suggestion is to make this parsing only dependant from the SWP syntax!!
  # End note BBOG
  my $includeDesc = 1;
  my $board = "";
  while (<MAIN>) {
    s/\r//g;  # Remove LF characters if present.
    $row = $_;
    chomp $row;
    $row =~ s/^\s+//;
    ToolCommon::Log($logFile, "Line read: $row\n") if ($verbose);

    if ($row =~ /DESCRIPTION\s+(\w+\-\w+)/) {
      $board = uc($1);
      if (! defined($delAll)) {
	$includeDesc = defined($boardList->{$board}) ? 1 : 0;
      } else {
	$includeDesc = 1 if !($board =~ /NT/);
      }
    } elsif ($row =~ /DESCRIPTOR\-FILE\s+(\w+\.\w+)/) {
      my $origSwpDesc = $1;
      my $newActSwpDesc = $newActSwp;
      if($OSWPNameMatch == 1){
        $newActSwpDesc = $origSwpDesc;
        $newActSwpDesc =~ s/(....).(.......)/$1$ReductionId$2/;
      }
      $_ = "DESCRIPTOR-FILE $newActSwpDesc\n";
    }
    if ($includeDesc) {
      if (($board =~ /ECNT/) || ($board =~ /NANT/)) {
	if (! (($row =~ /TYPE\-K/) && ($row =~ /MY1UAA00\.000/))) {
	  print NEW "$_" if ($row ne "");
	}
      } else {
	print NEW "$_" if ($row ne "");
      }
    } else {
      $includeDesc = 1 if ($row =~ /^END/);
    }
  }
  close(MAIN);
  close(NEW);
  ToolCommon::Log($logFile, "  OK\n");
  return EC_SUCCESS;
}

=head2 MakeNewOswpFile($$$$$$$$$)

=begin html
This function creates a new OSWP file for the reduced SWP.

=end html

=cut

sub MakeNewOswpFile($$$$$$$$) {
  my ($logFile, $dir, $actOswp, $newActOswp, $actSwp, $newActSwp, $verbose, $asam) = @_;
  my $node = $params->{NONAME} ? $asam->[0] : $asam->[1];

  #Changing OSWP descriptor file 
  ToolCommon::Log($logFile, "Changing OSWP descriptor file...");
  if (! open(OSWP, "$dir/$actOswp")) {
    ToolCommon::Log($logFile, "  FAILED: Could not open $dir/$actOswp for reading: $!\n");
    $ERROR = "Could not open $dir/$actOswp: $!";
    return EC_OPEN_OSWP_NEW_FAIL;
  }
  if (! open(NEWOSWP,">$dir/$newActOswp")) {
    ToolCommon::Log($logFile, "  FAILED: Could not open $dir/$newActOswp for writing: $!\n");
    $ERROR = "Could not open $dir/$newActOswp: $!";
    return EC_CREATE_OSWP_FAIL;
  }
  while (<OSWP>) {
    s/\r//g;
    if (/OVERALL\-DESCRIPTOR\-FILE\s+(\w+\.\w+)/) {
      my $origOswpDesc = $1;
      my $newActOswpDesc = $newActOswp;
      if($OSWPNameMatch == 1){
        $newActOswpDesc = $origOswpDesc;
        $newActOswpDesc =~ s/(....).(.......)/$1$ReductionId$2/;
      }
      print NEWOSWP "OVERALL-DESCRIPTOR-FILE $newActOswpDesc\n";
    } elsif (/(\s+ASAM\-CORE\s+:)\s+([^\s]+)\s+(\d+\.\d+\.\d+\.\d+);/) {
      my $p1 = $1; my $swpName = $2; my $ip1 = $3;
      $swpName =~ s/$actSwp$/$newActSwp/;
      $swpName = substr($swpName,rindex($swpName,'/')+1);
      my $toolConfigData = FamilyCommon::GetToolData();
      if(!$params->{NOTFTP}) {
           if (defined ($params->{HOSTS}) && (basename($dir) =~ /$toolConfigData->{NODEPREFIX}/))
           {
            my $hostDir = basename($dir);
            $swpName = "$toolConfigData->{TFTPWORKDIR}/$hostDir/$swpName";
           }
           else {
            $swpName = "$toolConfigData->{TFTPWORKDIR}/$node/$swpName"; 
           }
      } else {
           ## for SFTP ##
           if (defined ($params->{HOSTS}) && (basename($dir) =~ /$toolConfigData->{NODEPREFIX}/))
           {
            my $hostDir = basename($dir);
            $swpName = "$toolConfigData->{TFTPDIR}/$toolConfigData->{TFTPWORKDIR}/$hostDir/$swpName";
           }
           else {
      	    $swpName = "$toolConfigData->{TFTPDIR}/$toolConfigData->{TFTPWORKDIR}/$node/$swpName"; 
		   }
      }
      print NEWOSWP "$p1 $swpName $ip1;\n";
      ToolCommon::Log($logFile, "'$p1', '$swpName', '$ip1'\n") if $verbose;
    } elsif (/(\s+ASAM\-CORE\s+:)\s+([^\s]+)\s+(\d+\.\d+\.\d+\.\d+)\s+(\d+\.\d+\.\d+\.\d+);/) {
      my $p1 = $1; my $swpName = $2; my $ip1 = $3; my $ip2 = $4;
      $swpName =~ s/$actSwp$/$newActSwp/;
      $swpName = substr($swpName,rindex($swpName,'/')+1);
      $swpName =~ s/(....).(.......)/$1$TmpSWDRedVar$2/  if($OSWPNameMatch == 1);
      my $toolConfigData = FamilyCommon::GetToolData();
      if(!$params->{NOTFTP}) {
           if (defined ($params->{HOSTS}) && (basename($dir) =~ /$toolConfigData->{NODEPREFIX}/))
           {
            my $hostDir = basename($dir);
            $swpName = "$toolConfigData->{TFTPWORKDIR}/$hostDir/$swpName";
           }
	   else {
            $swpName = "$toolConfigData->{TFTPWORKDIR}/$node/$swpName"; 
	   }
      } else {
           ## for SFTP ##
           if (defined ($params->{HOSTS}) && (basename($dir) =~ /$toolConfigData->{NODEPREFIX}/))
           {
            my $hostDir = basename($dir);
            $swpName = "$toolConfigData->{TFTPDIR}/$toolConfigData->{TFTPWORKDIR}/$hostDir/$swpName";
           }
           else {
      		 $swpName = "$toolConfigData->{TFTPDIR}/$toolConfigData->{TFTPWORKDIR}/$node/$swpName"; 
		   }
      }

      my $ipAddr = undef;
      if ($Config{archname} =~ /linux/) {
         my $host = hostname();
         my $i_addr =  scalar gethostbyname( $host ) if(defined($host)); 
         if( defined $i_addr) {
            $ipAddr = inet_ntoa($i_addr);
         } else {
            my @data = `$config->{IFCONFIG}`;
            my @data1=split (/ +/, $data[1]);
            my @data2=split (/:/, $data1[2]);
            $ipAddr = $data2[1];
         }
      } else {
         my $host = hostname();
         $ipAddr = inet_ntoa(scalar gethostbyname( $host )) if(defined($host));
      }

      if(defined($ipAddr)) {
          chomp($ipAddr); 
          $ip1 = $ipAddr  if($ipAddr =~ /\d+\.\d+\.\d+\.\d+/); 
      }

      print NEWOSWP "$p1 $swpName $ip1 $ip2;\n";
      ToolCommon::Log($logFile, "'$p1', '$swpName', '$ip1', '$ip2'\n") if $verbose;
    } else {
      print NEWOSWP $_;
    }
#    if ($row =~/$actSwp\/$actSwp/ ) {
#      my @a_asamcore = split(/$actSwp\/$actSwp/, $row);
#      ToolCommon::Log($logFile, "1: $row -> @a_asamcore\n") if ($verbose);
#      print NEWOSWP "   $a_asamcore[0]$newActSwp/$newActSwp$a_asamcore[1]";
#      next;	
#    } elsif ($row =~/$actSwp/ ) {
#      my @a_asamcore = split(/$actSwp/, $row);
#      ToolCommon::Log($logFile, "2: $row -> @a_asamcore\n") if ($verbose);
#      print NEWOSWP "   $a_asamcore[0]$newActSwp$a_asamcore[1]";
#      next;
#    }
#    print NEWOSWP  $_; 
  } 
  close(OSWP);
  close(NEWOSWP);      
  ToolCommon::Log($logFile, "  OK\n");
  return ;
}

=head2 DownloadAndCommitNewSW($$$$$$$$$$;$$)

=begin html
This function downloads, activates and commits the reduced OSWP/SWP.

=end html

=cut

sub DownloadAndCommitNewSW($$$$$$$$$;$$) {
  my ($logFile, $dir, $tftp, $snmpSession, $asam, $newActSwp, $newActOswp, $verbose, $path, $cli, $actOswpIndx) = @_;

  # Download SWP before we download the OSWP.  When receiving the OSWP the NT will find out that the SWP is already present.
  # During this download it is not expected that the NE will retrieve any SW file from the TFTP server since all files have
  # been downloaded previously.
  ToolCommon::Log($logFile, "Downloading SWP descriptor file to NE...");
  my $setSwpInd = ""; 
  my $setOswpInd = ""; 
  my $tftpSwp = 1; 
  my $tftpOswp = 1; 
  my $pasOswpIndx = 3 - $actOswpIndx; 

  ToolCommon::Log($logFile, "Active OSWP index <$actOswpIndx> Passive OSWP index <$pasOswpIndx>...\n");
  
	  ToolCommon::Log($logFile, "Calling ListDescfiles\n");
    my $swpDescFile = undef; 
    $swpDescFile =  IsamCli::ListSwpDescFiles($cli,$newActSwp) if(defined($cli));
    my $actOswp = ""; 
    my $actSwp = ""; 
    ### if(defined($swpDescFile) && defined($swpDescFile->{$newActSwp}{TYPE})) 
    if(defined($swpDescFile)) { 
       my $swpfileSize = -s "$dir/$newActSwp"; 
       my $oswpfileSize = -s "$dir/$newActOswp"; 
       foreach my $file (keys %{$swpDescFile}) {
            if($file=~ /$newActSwp/) {
            ToolCommon::Log($logFile, "Found <$newActSwp>  <$file>\n");
              
		if ( $swpDescFile->{$file}{SIZE} ne $swpfileSize) {
			$setSwpInd= "yes";
			#Call octopus to remove the L6GQRN
			ToolCommon::Log($logFile,  "Calling octopus (size not equal )to remove:$newActSwp\n");
			$actSwp = $file;
		}
		else {
                        $tftpSwp = 0; 
			ToolCommon::Log($logFile, "File size equal..:$newActSwp!!\n");
		}
            }
            if($file =~ /$newActOswp/) {
                ToolCommon::Log($logFile, "Found OSWP -->  <$newActOswp>  <$file>\n");
		if ( $swpDescFile->{$file}{SIZE} ne $oswpfileSize ) {
			ToolCommon::Log($logFile,  "Calling octopus (size not equal )to remove:$newActOswp\n");
			$setOswpInd= "yes";
			$actOswp = $file;
                } else {
                        $tftpOswp = 0; 
			ToolCommon::Log($logFile, "File size equal..:$newActOswp!!\n");
                }
            }
      }
    }
    if (( $setSwpInd ne "" ) || ($setOswpInd ne "")) {
	$actSwp= $setSwpInd ? $actSwp : undef;
	$actOswp= $setOswpInd ? $actOswp : undef;
        ToolCommon::Log($logFile, sprintf("call error routine SWP:<%s>, OSWP:<%s>", defined($actSwp) ? $actSwp : "not in list", defined($actOswp) ? $actOswp : "not in list"));
        if (defined($connSSH)) {
           if (SecureDebugTAndD::SshRemoveSwPkg($connSSH, $dir,$actSwp,$actOswp) ) {
              $ERROR = "Remove for the SWP pkg on /Sw failed";
              return EC_SEC_SSH_REMOVE_SWPKG_FAIL; 
           }    
        } else {
          if (! Octopus::RemoveSwPkg($verbose, $dir, $asam, $path, $actSwp,$actOswp)) {
              $ERROR = sprintf("Remove for the SWP pkg on /Sw failed: %s", Octopus::GetError());
              return EC_OCTO_FAIL;   # -- 1.0.6
  	  }
  	}
    }
if(!$params->{NOTFTP}) {
    if ($tftpSwp) {
      ToolCommon::Log($logFile, sprintf(" TFTP put $dir/$newActSwp /pub/Sw/Download/$newActSwp...\n")) if ($verbose);
      if ($tftp->put("$dir/$newActSwp", "/pub/Sw/Download/$newActSwp") ne "") {
         ToolCommon::Log($logFile, sprintf("  FAILED: put $dir/$newActSwp /pub/Sw/Download/$newActSwp failed: %s\n", $tftp->error()));
         $ERROR = sprintf("tftp put $dir/$newActSwp /pub/Sw/Download/$newActSwp failed: %s", $tftp->error());
         return EC_XFTP_PUT_SWP_FAIL;
      } else {
         ToolCommon::Log($logFile, "  OK\n");
      }
    } else {
      ToolCommon::Log($logFile, "  OK - File already present\n");
    }
  } else {
	  if ($tftpSwp) {
       #my $cmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD}\' \'$params->{ISAM}\' '$dir/$newActSwp|/pub/Sw/Download/$newActSwp' 'put'"; 
       my $ret = ToolCommon::SFTPClientWrap( $logFile, $config, $path, "$dir/$newActSwp|/pub/Sw/Download/$newActSwp" , 'put', $asam->[0] );
       if ($ret) {
           ToolCommon::Log($logFile, sprintf("  FAILED: sftp  put $dir/$newActSwp /pub/Sw/Download/$newActSwp failed: \n"));
           $ERROR = sprintf("sftp put $dir/$newActSwp /pub/Sw/Download/$newActSwp failed");
           return EC_XFTP_PUT_SWP_FAIL; 
       }
	  } else {
		ToolCommon::Log($logFile, "  OK - File already present\n");
    }
  }
    my $actoswp;
    ToolCommon::Log($logFile, "Retrieve OSWP name...");
    my $oswpname = GetColumn($logFile, $asam, $snmpSession, $SWPKG_5X_OID, "OSWP Name", $verbose);   # -- 1.0.6
    if (!defined($oswpname)) {
      ToolCommon::Log($logFile, "  FAILED!!\n");
      return EC_GET_OSWP_NAME_FAIL;   # -- 1.0.6
    } else {
      ToolCommon::Log($logFile, "  OK!!\n");
    }
    my $act;
    my @swoid;
    foreach (Net::SNMP::oid_lex_sort(keys(%{$oswpname}))) {
      @swoid = split(/\./, $_);
      $act = $swoid[-1];
      ToolCommon::Log($logFile, sprintf("oid: '%s', act: '%s', oswpname: '%s'\n", $_, $act, $oswpname->{"$SWPKG_5X_OID.$act"})) if ($verbose); # 1.0.8
      if ($oswpname->{"$SWPKG_5X_OID.$act"} eq NO_OSWP) {
      ToolCommon::Log($logFile, "Downloading OSWP descriptor file to NE...");
      if(!$params->{NOTFTP}) {
        if ($tftpOswp) {
           if($OSWPNameMatch == 1) {
              ToolCommon::Log($logFile, sprintf("  put $dir/$newActOswp /Sw/OSWP$pasOswpIndx/$newActOswp...\n")) if ($verbose);
           } else {
              ToolCommon::Log($logFile, sprintf("  put $dir/$newActOswp /pub/Sw/Download/$newActOswp...\n")) if ($verbose);
           }
           my $retcode = ($OSWPNameMatch == 1) ? $tftp->put("$dir/$newActOswp", "/Sw/OSWP$pasOswpIndx/$newActOswp") : $tftp->put("$dir/$newActOswp", "/pub/Sw/Download/$newActOswp");
           if ($retcode ne "") {
	     ToolCommon::Log($logFile, sprintf("  FAILED: put $dir/$newActOswp /pub/Sw/Download/$newActOswp failed: %s\n",
					  $tftp->error()), $verbose);
	     $ERROR = sprintf("tftp put $dir/$newActOswp /pub/Sw/Download/$newActOswp failed: %s", $tftp->error());
	     return EC_XFTP_PUT_OSWP_FAIL;
           } else {
	     ToolCommon::Log($logFile, "  OK\n");
           }
        } else {
	  ToolCommon::Log($logFile, "  OK, file already present\n");
        }
      } else {
         my $ret; 
	 if ($tftpOswp) {
         if($OSWPNameMatch == 1) {
            #$cmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD}\' \'$params->{ISAM}\' '$dir/$newActOswp|/Sw/OSWP$pasOswpIndx/$newActOswp' 'put'"; 
            $ret = ToolCommon::SFTPClientWrap( $logFile, $config, $path, "$dir/$newActOswp|/Sw/OSWP$pasOswpIndx/$newActOswp" , 'put', $asam->[0] );
         } else {
            #$cmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD}\' \'$params->{ISAM}\' '$dir/$newActOswp|/pub/Sw/Download/$newActOswp' 'put'"; 
            $ret = ToolCommon::SFTPClientWrap( $logFile, $config, $path, "$dir/$newActOswp|/pub/Sw/Download/$newActOswp" , 'put', $asam->[0] );
         }
         if ($ret) {
           ToolCommon::Log($logFile, sprintf("  FAILED: sftp  put $dir/$newActOswp /pub/Sw/Download/$newActOswp failed failed: \n"));
           $ERROR = sprintf("sftp put $dir/$newActOswp /pub/Sw/Download/$newActOswp failed");
           return EC_XFTP_PUT_OSWP_FAIL;
         }
	 } else {
	ToolCommon::Log($logFile, "  OK, file already present\n");
        }
      }
      # ANTms49877 : sleep for 2 secs after OSWP download
      sleep(2);
      # Updating new OSWP  descriptor file 
      ToolCommon::Log($logFile, "Updating new OSWP descriptor filename in the MIB...");
      $actoswp = $act;
      if( not SNMPComm::SetValue( 'OSWP Descriptor filename', "$SWPKG_5X_OID.$act", $newActOswp ) ) {
  	       ToolCommon::Log($logFile, "  FAILED!!\n");
	       return EC_SET_OSWP_FAIL;   # -- 1.0.6
      } else {
  	       ToolCommon::Log($logFile, "  OK!!\n");
      }                                                                                          # -- 1.0.6
      # ANTms49877 : sleep for 2 secs after snmpset
      sleep(2);
      last;
    }
  }
  if (!defined($actoswp)) {   # 1.0.8
    ToolCommon::Log($logFile, "No active OSWP determined!!!\n");
    return EC_NO_ACT_OSWP_2ACTIVATE;
  }                           # 1.0.8
  ToolCommon::Log($logFile, "Checking if new OSWP is enabled... ");
  my $counter = 0;
  my @oids;
  push(@oids, "$OPERAVAILSTATUS.$actoswp");
  push(@oids, "$OSWPDLERROR.$actoswp");
  push(@oids, "$OSWPERRORFILENAME.$actoswp");
  push(@oids, "$OSWPERRORFILESERVER.$actoswp");
  my $availState = GetValuesAtIndex($logFile, $asam, $snmpSession, \@oids, $verbose); # -- 1.0.6
  if (!defined($availState)) {  # -- 1.0.6
    ToolCommon::Log($logFile, " FAILED!!\n");
    return EC_GET_OSWP_FAIL;   # -- 1.0.6
  }  # -- 1.0.6
  while ((defined($availState) && ($availState->{"$OPERAVAILSTATUS.$actoswp"} == $oswpAvailStates{"downloading"})) && ($counter < 1200)) {
    ToolCommon::Log($logFile, ".") if (($counter % 30) == 0);   # -- 1.0.6
    sleep(1);
    $counter++;
    $availState = GetValuesAtIndex($logFile, $asam, $snmpSession, \@oids, $verbose);   # -- 1.0.6
  }
  if (!defined($availState)) { # -- 1.0.6
    ToolCommon::Log($logFile, " FAILED!!\n");
    return EC_GET_OSWP_FAIL;   # -- 1.0.6
    ToolCommon::Log($logFile, "  FAILED!!\n");
  } else {
    if ($availState->{"$OPERAVAILSTATUS.$actoswp"} != $oswpAvailStates{"enabled"}) {
      $ERROR = sprintf("making OSWP available failed (state '%s', ftp error: '%s', error file name/server: '%s/%s')",
		       defined($oswpAvailEnum{$availState->{"$OPERAVAILSTATUS.$actoswp"}}) ?
		       $oswpAvailEnum{$availState->{"$OPERAVAILSTATUS.$actoswp"}} : "??" . $availState->{"$OPERAVAILSTATUS.$actoswp"} . "??",
		       defined($ftpErrors{$availState->{"$OSWPDLERROR.$actoswp"}}) ? $ftpErrors{$availState->{"$OSWPDLERROR.$actoswp"}} :
		       "??" . $availState->{"$OSWPDLERROR.$actoswp"} . "??",
		       $availState->{"$OSWPERRORFILENAME.$actoswp"}, $availState->{"$OSWPERRORFILESERVER.$actoswp"});
      return EC_OSWP_OPER_FAIL;   # -- 1.0.6
    } else {
      ToolCommon::Log($logFile, "  OK!!\n");
      ToolCommon::Log($logFile, sprintf("ftp error: '%s', error file name/server: '%s/%s'\n",
					defined($ftpErrors{$availState->{"$OSWPDLERROR.$actoswp"}}) ?
					$ftpErrors{$availState->{"$OSWPDLERROR.$actoswp"}} :
					"??" . $availState->{"$OSWPDLERROR.$actoswp"} . "??",
					$availState->{"$OSWPERRORFILENAME.$actoswp"}, $availState->{"$OSWPERRORFILESERVER.$actoswp"})) if $verbose;
    }
  } # -- 1.0.6
  sleep 5; # for safety: redundant systems.
  #Activating new descriptor files via MIB

  ToolCommon::Log($logFile, "Activating OSWP via MIB.....");
  if( not SNMPComm::SetValue( 'ACTIVATE OSWP', "$OSWPACTIVATION.$actoswp", "1" ) ) {
    ToolCommon::Log($logFile, "  FAILED!!\n");
    return EC_SNMP_ACT_FAIL;   # -- 1.0.6
  } else {
    ToolCommon::Log($logFile, "  OK!!\n");
  }# -- 1.0.6

  ToolCommon::Log($logFile, "Checking if OSWP is active ...");
  my $active = GetValueAtIndex($logFile, $asam, $snmpSession, "$OSWPOPERACTSTATUS.$actoswp", $verbose);   # -- 1.0.6
  if (!defined($active)) { # -- 1.0.6
    ToolCommon::Log($logFile, "  FAILED!!\n");
    return EC_GET_ACTIVATE_FAIL;   # -- 1.0.6
  }                        # -- 1.0.06

  $counter = 0;
  while ((defined($active) && ($active != $oswpActStates{"active"})) && ($counter < 1200)) {
    ToolCommon::Log($logFile, ".") if (($counter % 30) == 0);   # -- 1.0.6
    sleep(1);
    $counter++;
    $active = GetValueAtIndex($logFile, $asam, $snmpSession, "$OSWPOPERACTSTATUS.$actoswp", $verbose);   # -- 1.0.6
  }
  if (!defined($active) || (($active != $oswpActStates{"active"}) && ($counter == 1200))) {
    ToolCommon::Log($logFile, "  FAILED!!\n");
    $ERROR = sprintf("activating OSWP failed (state %s, $counter)",
		     defined($active) ? (defined($oswpActEnum{$active}) ? $oswpActEnum{$active} : "??$active??") : "undefined");
    return EC_ACTIVATE_FAIL;   # -- 1.0.6
  } else {
    ToolCommon::Log($logFile, "  OK!!\n");
  }

  ToolCommon::Log($logFile, "Committing OSWP...");
  if( not SNMPComm::SetValue( 'COMMIT OSWP', "$OSWPCOMMITMENT.$actoswp", "1" ) ) {
    ToolCommon::Log($logFile, "  FAILED!!\n");
    return EC_SET_COMMIT_FAIL;   # -- 1.0.6
  } else {
    ToolCommon::Log($logFile, "  OK!!\n");
  }
  ToolCommon::Log($logFile, "Waiting for the OSWPOPERCOMMITSTATUS to reach committed...");
  my $commit = GetValueAtIndex($logFile, $asam, $snmpSession, "$OPERCOMMITSTATUS.$actoswp", $verbose);   # -- 1.0.6
  if (!defined($commit)) { # -- 1.0.6
    ToolCommon::Log($logFile, "  FAILED!!\n");
    return EC_GET_COMMIT_FAIL;   # -- 1.0.6
  } # -- 1.0.6
      
  $counter = 0;
  while ((defined($commit) && ($commit != $oswpCommitStates{"committed"})) && ($counter < 1200)) {
    sleep(1);
    $counter++;
    $commit = GetValueAtIndex($logFile, $asam, $snmpSession, "$OPERCOMMITSTATUS.$actoswp", $verbose);   # -- 1.0.6
  }

  if (!defined($commit) || (($commit != $oswpCommitStates{"committed"}) && ($counter == 1200))) {
    ToolCommon::Log($logFile, "  FAILED!!\n");
    $ERROR = sprintf("Commiting OSWP failed (state %s, $counter)",
		     defined($commit) ? (defined($oswpCommitEnum{$commit}) ? $oswpCommitEnum{$commit} : "??$commit??") : "undefined");
    return EC_COMMIT_FAIL;   # -- 1.0.6
  } else {
    ToolCommon::Log($logFile, "  OK!!\n");
  }
  return EC_SUCCESS;
}

=head2 ReduceActiveSWP($$$$$$$$$;$)

=begin html
Reduce the online SW package of the NE.<br>
The following parameters are used:
<table>
  <tr><td>asam</td><td>isam context.  ->[0] holds IP address, ->[1] holds ISAM name</td></tr>
  <tr><td>snmpSession</td><td>reference to an opened SNMP context</td></tr>
  <tr><td>dir</td><td>the NE output directory</td></tr>
  <tr><td>logFile</td><td>the file handle used to log progress and debug messages</td></tr>
  <tr><td>verbose</td><td>additional debugging needed?</td></tr>
  <tr><td>octopusPath</td><td>location of the octopus scripts</td></tr>
  <tr><td>redId</td><td>string to be used on position 5 and 6 of the SWP and OSWP when creating the reduced OSWP and SWP</td></tr>
  <tr><td>delAll</td><td>Delete all files or not (PBMT leaves all SW related to the boards planned and present untouched)</td></tr>
  <tr><td>tftpIp</td><td>the IP address of the TFTP server</td></tr>
</table>

=end html

=cut

sub ReduceActiveSWP($$$$$$$$;$) {
  my ($asam, $snmpSession, $dir, $logFile, $path, $redId, $delAll, $tftpIp,$cli) = @_;

  $ERROR = "";
  my $tftp;
  my $verbose = 1;  
  $verbose = 0 if (!defined ($verbose));
  ToolCommon::Log($logFile, "Options:\n");
  ToolCommon::Log($logFile,         "   NE                      : $asam->[0]\n");   # Changed to use IP@
  ToolCommon::Log($logFile,         "   dir                     : $dir\n");
  ToolCommon::Log($logFile, sprintf("   Extra logging           : %s\n", $verbose ? "Yes" : "No"));
  ToolCommon::Log($logFile,         "   SW reduction indentifier: $redId\n");

  $config = FamilyCommon::GetConfigData();
  #Octopus session to enable read access permissions
  if (defined($cli)) {
  my $debugViaSSH = IsamCli::GetSecureDebugSetting($cli);
  if (!defined($debugViaSSH)) {
    FamilyCommon::IncrErrorCount("", EC_SW_RED_FAILED, sprintf(": %s", "Error in fetching Secure debug setting for SSH"),1);
              return EC_SW_RED_FAILED;
   }

  my $actRelNum = GetActiveRelNum($cli);
  my $ntType = ToolCommon::DetermineNtType( );
  
  my $debugSshSupportedHwEsti = (Capability::ReleaseCompare($actRelNum, 
                                                            Capability::ConvertRelease(5, 2), 
                                                            $toolConsts{REGEXP}, 
                                                            0) >= 0 
                                 && 
                                 Capability::IsIsamHiCapEtsiFamily($ntType, 
                                                                   $params->{DBVER_TGTREL}));
 
  my $debugSshSupportedHwAnsi = (Capability::ReleaseCompare($actRelNum, 
                                                            Capability::ConvertRelease(4, 4), 
                                                            $toolConsts{REGEXP},
                                                            0) >= 0
                                 &&
                                 Capability::IsIsamAnsiFamily($ntType, 
                                                              $params->{DBVER_TGTREL}));

  if ($debugViaSSH && ($debugSshSupportedHwEsti || $debugSshSupportedHwAnsi))   {
   ToolCommon::Log($logFile, "Reduced software download is happening on T&D via Secured Debug mode !!\n");
   #$connSSH = IsamCli::ConnectSecDebugViaSSH($asam->[0], $config->{'TELNETTIMEOUT'}, $params->{DEBUG} ? $logFile: undef);
   $params->{USEDEBUGVIASSH} = 1;
   if( Capability::ReleaseCompare($actRelNum, Capability::ConvertRelease(5, 5), $toolConsts{REGEXP}, 0) >= 0 ) {
      $params->{DEBUGSSHPORT}=11130;
   } else {
      $params->{DEBUGSSHPORT}=30;
   }
   $connSSH = ToolCommon::OpenCliWrap( $asam, $params->{DEBUG} ? $logFile: undef, $config->{'TELNETTIMEOUT'} );
   if (!defined($connSSH)) {
	  FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open cli for %s: %s\n", FamilyCommon::GetFailedNode(), ToolCommon::GetError()), EC_OPENCLI_FAILED, "", 1 );
          return EC_OPENCLI_FAILED;
   }
   $params->{USEDEBUGVIASSH} = 0;
   SecureDebugTAndD::SetVars($logFile);
   SecureDebugTAndD::ReduceSwFs($connSSH, $dir); 
   SecureDebugTAndD::UpdateNewFs($connSSH, $dir); 
   if (SecureDebugTAndD::SshEnableSwWrite($connSSH, $dir)) {
    $ERROR = "Enable write on /Sw failed";
    return EC_SEC_SSH_ENABLE_WRITE_FAIL;
   } 
  } 
   if(!$debugViaSSH) {
     Octopus::ReduceSwFs($verbose, $dir, $asam, $path); 
     Octopus::UpdateNewFs($verbose, $dir, $asam, $path); 
     if (! Octopus::EnableSwWrite($verbose, $dir, $asam, $path)) {
     $ERROR = "Enable write on /Sw failed";
     return EC_OCTO_FAIL;   # -- 1.0.6
     }
   }
 } else {
  if (! Octopus::EnableSwWrite($verbose, $dir, $asam, $path)) {
     $ERROR = "Enable write on /Sw failed";
    return EC_OCTO_FAIL;   # -- 1.0.6
  }
  }
  my $mib = GetMibVersion($logFile, $asam, $snmpSession, $verbose);   # -- 1.0.6
  if (!defined($mib)) {
    ToolCommon::Log($logFile, "Mib version not found.\n");
    return EC_NO_MIB_VERSION;   # -- 1.0.6
  }
  ToolCommon::Log($logFile, "MIB version = $mib\n") if ($verbose);

  # Printing the existing free space 
  my $freespace = GetValueAtIndex($logFile, $asam, $snmpSession, $FREESPACEOID, $verbose);   # -- 1.0.6
  if (!defined($freespace)) {  # -- 1.0.6
    ToolCommon::Log($logFile, "Retrieving free space failed\n");
    return EC_NO_FREE_SPACE;   # -- 1.0.6
  }                            # -- 1.0.6
  ToolCommon::Log($logFile, "Free space available before cleanup of unused software = $freespace bytes\n");
#
  # Check for file Xfer protocol in the isam viz. TFTP/SFTP
    my $fxrData = ToolCommon::GetFxrSettings($snmpSession, $RETRIES);
    if (not defined($fxrData)) {
       ToolCommon::Log($logFile, "obtained empty  file transfer protocol details\n");
    } else {
       if($fxrData->{PROTOCOL} != TFTP_FXR) {
         $params->{NOTFTP}=1;
	 ToolCommon::Log($logFile, ">>>>>>> To use SFTP Xfer \n");
       }
    }
#
    
  my $a_boards = GetBoardList($logFile, $asam, $snmpSession, $verbose);
  if (defined($a_boards)) {
    if(!$params->{NOTFTP}) {
	$tftp = Net::TFTP->new($asam->[0]);   # Changed to use IP@
	if (! defined($tftp)) {
	  ToolCommon::Log($logFile, "  FAILED: Cannot open tftp to '$asam->[1]'\n");
	  return EC_TFTP_OPEN_FAIL;
	}
    } else {
	$tftp = undef; 
    }

    my ($ret, $actOswp, $actSwp, $actOswpIndc) = GetActiveSwInfo($logFile, $asam, $snmpSession, $dir, $tftp, $verbose, $path);      
    if (! $ret) {
      if (($actOswp =~ /^.*(999)$/) || ($actOswp =~ /^.*(998)$/) ||  # Original reduction indication
	  ($actOswp =~ /....$redId......./)) {                           # New reduction indication
	ToolCommon::Log($logFile, "The cleanup of unused s/w is already done.\n") if ($verbose);
	return EC_SUCCESS; 	   # -- 1.0.6
      }
      my $newActOswp = $actOswp;
      $newActOswp =~ s/(....).(.......)/$1$redId$2/;
      my $newActSwp = $actSwp;
      if($OSWPNameMatch == 1){ 
        $newActSwp =~  s/(....).(.......)/$1$TmpSWDRedVar$2/;
      } else {
        $newActSwp =~  s/(....).(.......)/$1$redId$2/;
      }
      ToolCommon::Log($logFile, "New OSWP: '$newActOswp', new SWP: '$newActSwp'.\n") if ($verbose);
      #Reducing SWP descriptor file to desired boardlist

      $ret = ReduceSwpFile($logFile, $dir, $actSwp, $newActSwp, $delAll, $a_boards, $verbose);
      return $ret if $ret;

      $ret = MakeNewOswpFile($logFile, $dir, $actOswp, $newActOswp, $actSwp, $newActSwp, $verbose, $asam);

      return $ret if $ret;
      #Downloading SWP & OSWP descriptor files to ASAM 
      Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership...\n");
      return undef if( ToolCommon::AcquireTransaction() );
      Logger::LogTrace($nodeLogFH, "Acquire config-lock ownership... DONE\n");

      $ret = DownloadAndCommitNewSW($logFile, $dir, $tftp, $snmpSession, $asam, $newActSwp, $newActOswp, $verbose, $path,$cli, $actOswpIndc);
      return $ret if $ret;

      #Getting new free space
      
      ToolCommon::Log($logFile, "Get free space available after cleanup of unused software...");
      $freespace = GetValueAtIndex($logFile, $asam, $snmpSession, $FREESPACEOID, $verbose);   # -- 1.0.6
      if (!defined($freespace)) {
	      ToolCommon::Log($logFile, "  FAILED!!\n");
	      return EC_GET_NEW_FREE_FAIL;   # -- 1.0.6
      } else {
	      ToolCommon::Log($logFile, "  OK!! ($freespace bytes)\n");
      }
      Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership...");
      return undef if( ToolCommon::ReleaseTransaction() );
      Logger::LogTrace($nodeLogFH, "Release configuration-lock ownership... DONE");
    } else {
      return $ret;
    }
  } else {
    return EC_NO_BOARDS;   # -- 1.0.6
  }
  return EC_SUCCESS;
}

sub GetActiveRelNum($) {
 my ($cli) = @_;
 my $oswpInfo = IsamCli::GetOswpInfo($cli, $toolConsts{REGEXP});
 my $dbVersInfo = IsamCli::GetDbVersionInfo($cli);

  my $activeRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'}, $oswpInfo->{'ACTIVE'}{'MINOR'}, 0, $dbVersInfo->{$oswpInfo->{'ACTIVE'}{'INDEX'}});
 return $activeRelNum;
}

1;
