package StageONT7302HiCap;

=head1 Stage ONT Software

=begin html
This library contains the functions dedicated to perform a Stage ONT .

=end html

=cut
#==============================================================================
#-- Copyright     :    All rights reserved. Passing on and copying of this document,
#--                    use and communication Of its contents not permitted without
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- File          :    StageONT7302HiCap.pm
#-- Date          :    11-03-2009
#-- Company       :    Alcatel-Lucent
#-- Author        :    Palaniappan Ramasamy
#------------------------------------------------------------------------------
#-- Function      :    Script to perform Staging ONT software to OLT
#------------------------------------------------------------------------------
#-- Disclaimer    :    This script contains references to proprietary information
#--                    which is owned by Alcatel- Lucent.
#------------------------------------------------------------------------------
#-- Copyright     :    All rights reserved. Passing on and copying of this document,
#--                    use and communication Of its contents not permitted without
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- Requirements  :
#--                    ANTms36033-071:
#--                    ANTms36033-072:
#--                    ANTms36033-073:
#--                    ANTms36033-074:
#--                    ANTms36033-091:
#------------------------------------------------------------------------------
#-- History       :
#-- 1.0.1         :    2009/03/11 : First Implementation . 
#-- 1.0.2         :    2009/04/27 : DDT's tickets resolved
#                        BDFam48538
#                        BDFam48532
#                        BDFam48398
#                        BDFam48332 - Strings are not enclosed between the braces.
#-- 1.0.3         :    2009/05/04 : DDT's tickets resolved
#                        BDFam48534
#                        BDFam48393 
#-- 1.0.4         :    2009/05/12 : DDT's tickets resolved
#                        BDFam49289
#                        BDFam49953 
#-- 1.0.5         :    2009/06/24 : OBR changes
#                        BDFam51227 
#                        BDFam50249 
#                        BDFam51387
#-- 1.0.6         :    2009/07/08 : DDTs tickets resolved
#                        BDFam51621 
#                        BDFam51620
#                        BDFam51619
#                        BDFam48538
#-- 1.0.7         :    2009/08/03 : DDTs tickets resolved
#                        BDFam51219
#                        BDFam52649
#                        BDFam52670
#-- 1.0.8         :    2009/08/18 : DDTs tickets resolved
#                        BDFam52357
#                        BDFam53999
#-- 1.0.9         :    2009/09/16 : DDTs tickets resolved
#                        BDFam53543
#                        BDFam54235
#-- 3.0.00        :    2010/04/21 : Converted version to 3.0.00 for converged PBMT

#-- 3.0.01         :   2010/05/24 : First Implementation for converged PBMT - JanakiRaman and Janith
#-- 3.0.02         :   2011/01/12 : LRM updates for lib-perl path 
#-- 3.0.03         :   2011/03/27 : linux compatible changes
#-- 3.0.04         :   2011/05/05 : ALU01178887 
#-- 3.0.05         :   2013/04/29 : ALU01880147
#-- 3.0.06         :   2013/10/24 : Requirement to put the ontReleaseMap file is ISAM
#-- 3.0.07         :   2015/03/15 : ALU02053378: Additional parameter for TL1::Connect 
#-- 3.0.08         :   2015/05/05 : ALU02063817 
#-- 3.0.09         :   2015/05/25 : ALU02063817 
#-- 3.0.10         :   2015/06/17 : ALU02087249
#-- 3.0.11         :   2016/07/15 : ALU02237966
#-- 3.0.12         :   2016/09/28 : ALU02278259
#-- 3.0.13         :   2016/11/07 : ALU02286690
#------------------------------------------------------------------------------

=head1 VERSION

=begin html
Current version: "StageONT7302HiCap.pm - 3.0.12 - 2016/09/28"

=end html

=cut
my $VERSION = "StageONT7302HiCap.pm - 3.0.12 - 2016/09/28";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2216-DFZZA">FamilyCommon</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;

use FileHandle;
use File::stat;
use Getopt::Long;
use POSIX ":sys_wait_h";

#==============================================================================
#== Find the lib dir
use Cwd;
use File::Basename;
use Config; 

BEGIN {
  my $dir = dirname($0);
  my $path;
  my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($dir =~ /^\//) {
    #--absolute path
    $path = $dir;
  } elsif ($dir eq ".") {
    #-- relative, current dir
    $path = getcwd();
  } else {
    #-- relative, but not from current dir
    $dir =~ s/^\.\///;
    $path = getcwd()."/$dir";
  }
  unshift(@INC, "$path/lib");
  unshift(@INC, "$path/lib-perl/$perlLibLocation");
  if ($Config{archname} =~ /sun4/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
    unshift(@INC, "$path/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$path/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$path/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
}
#==============================================================================

use Net::SNMP;
use Net::TFTP;
use ONT7302HiCap;
use IsamTL1;
use Crypt::Lite;
use Utils;
use FamilyCommon;
use ErrorCodes qw(:EXIT_CODES :ONT_ERR :COMMON);
use ToolCommon;
use PBMTGlobal;
use FileTransfer;
#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub getNEdiskspace($$);
sub StageONT($$);
sub ResetGlobals();
#==============================================================================
#== Constants
#==============================================================================

#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================
my $STATUSFILE =       "result.txt";
my $FREE_SPACE = ".1.3.6.1.4.1.637.61.1.24.1.7.0";
my $toolData;
my $config;
my %OntInfo;
my %stageSw;
my $count = 0;
my $complete = 0;
my $progressfile = undef;
my $statusFile = undef;
my $warnCount = 0;
my $failCount = 0;
my $succ_Count = 0;
my $warnThreshold = 0;
my $warnExceeds = 0;                         # BDFam51219
my $OntRelMapFileSize = 0; 
my $ontrelmapName = "ONT_Mapping"; 

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}


sub ResetGlobals() {
	$count = 0;
	$complete = 0;
 	$warnCount = 0;
	$failCount = 0;
	$succ_Count = 0;
	$warnThreshold = 0;
	$warnExceeds = 0;             
	$OntRelMapFileSize = 0; 
}

=head2 SetProgressFail($)

=begin html
Set the progress file handler.
<br>
The following parameters are used:
<table>
  <tr><td>handle</td><td>the file handle of the progress file. Progress percentage is written to this file</td></tr>
</table>

=end html

=cut

sub SetProgressFail($){
  my ($handle) = @_;
  $progressfile = $handle;
}

=head2 SetStatusFail($)

=begin html
Set the status file handler.
<br>
The following parameters are used:
<table>
  <tr><td>handle</td><td>the file handle of the status file. Failed ONT aids as well as reason for faliure is written to this file</td></tr>
</table>

=end html

=cut
sub SetStatusFail($){
  my ($handle) = @_;
  $statusFile = $handle;
}

=head2 getNEdiskspace($$)

=begin html
Get the free diskspace from the NE
<br>
The following parameters are used:
<table>
 <tr><td>session</td><td>the SNMP session</td></tr>
 <tr><td>retry</td><td>no of retires allowed in that session</td></tr>
</table>

=end html

=cut

sub getNEdiskspace($$) {
  my ($session, $retry) = @_;

  my $freeSpace = ToolCommon::DoSafeGet($session, $FREE_SPACE, $retry);

  return $freeSpace;
}

=head2 _update($)

=begin html
Update the progress file to show the percentage of completion
<br>
The following parameters are used:
<table>
 <tr><td>count</td><td>the quantum by which staging is completed in the present attempt</td></tr>
</table>

=end html

=cut

sub _update ($) {
  my ($count) = @_;
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  $complete  = $complete + $count;
  my $p = ($complete / $warnCount) * 100;
  seek($progressfile,0,0);                                   # BDFam53999
  $p = POSIX::ceil($p);                                      # BDFam51621
  print $progressfile $p . "% Complete\n";
  $progressfile->autoflush(1);                               # BDFam51387
  ToolCommon::Log($nodeLogFh, "Progress $p % Completed\n");

  return ;
}

=head2 _setfail($)

=begin html
Update the faliure count for staging and compare with the warning threshold
<br>
The following parameters are used:
<table>
 <tr><td>count</td><td>the quantum by which staging is completed in the present attempt</td></tr>
</table>

=end html

=cut

sub _setfail ($) {
  my ($count) = @_;
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();

  $failCount = $failCount + $count;   #BDFam49968
  if ($warnThreshold <= (($failCount / $warnCount) * 100)) {
    ToolCommon::Log($nodeLogFh,"ONT operation failure reaches warning threshold\n");
    $warnExceeds = 1;                 # BDFam51219
    $warnThreshold  = 1000;
  }
  _update($count);
  return ;
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 StageONT($$$)

=begin html
This function is called to perform Stage ONT software of the ISAM family.<br>
The following parameters are used:
<table>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>isam</td><td>the ID of the ISAM: [0] being the IP address and [1] being either the name of the IP address.</td></tr>
  <tr><td>nttype</td><td>NT Type.</td></tr>
</table>

=end html

=cut

sub StageONT($$) {
  my ($isam, $ntType) = @_;
  my ($newMajor, $newMinor, $newSuffix, $newFullRelNum);
  my $activeNt;
  my $tl1Session = undef;
  my $ftSession = undef;
  my $fxrData = undef;
  my $ret;
  my ($type, $hw);
  my $dir = FamilyCommon::GetNeStorage();
  my $nodeLogFh = FamilyCommon::GetWarnLogFile();
  my $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
  my $snmpSession = FamilyCommon::GetSnmpSession2Close();
  my $toolConfigData = FamilyCommon::GetToolData();
  my $configData = FamilyCommon::GetConfigData();
  my $ontParams = 1;
  FamilyCommon::ClearError();
  $config = $configData;
  $toolData = $toolConfigData;
  $warnThreshold = $toolData->{WARN};

#-- Step 1: Open TL1 Session
  ToolCommon::Log($nodeLogFh, "Opening TL1 Session \n");
  $tl1Session = IsamTL1::Connect($isam->[0], $toolData->{TL1UDPPORT}, $toolData->{TL1USER}, $toolData->{TL1PWD},
                                     $params->{DEBUG} ? $nodeLogFh : undef,$ontParams);

  if (!defined($tl1Session)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open TL1 session for %s: %s\n",
                                 FamilyCommon::GetFailedNode(), IsamTL1::GetError()),
                                 EC_OPENTL1_FAILED,undef,
                                 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "Established TL1 Connection sucessfully\n");
  }
  FamilyCommon::SetTL1Session2Close($tl1Session);  # Store for IncrErrorCount

#-- Step 2: Aquire Transaction
  ToolCommon::Log($nodeLogFh, "Attempting to Acquire transaction...\n");
  return 0 if( ToolCommon::TransactionReleased() );
  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'acquire')) );

  #BDFam48393 starts ..
  $ret = IsamTL1::InhibitAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, "Alarms Inhibited..\n");
  } else {
    ToolCommon::Log($nodeLogFh, "Error in Alarm Inhibit!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                               EC_ISAM_INH_ALARM_FAILED,undef,
                               1);
    return 0;
  }
  #BDFam48393 ends ..
ToolCommon::Log($nodeLogFh, "Attempt to stage ONT!!!\n");
#-- Step 3: Populate the ONT SWP for respective ONT Types

   my @typeLists = split(',',$toolData->{ONTTYPES});
   my @typeDetails;
   foreach my $typeList (@typeLists) {
     @typeDetails = split(":",$typeList);
     $warnCount++;
   }
   my $sw = undef;
   my $lCount = 0;
   foreach my $typeList (@typeLists) {
     @typeDetails = split(":",$typeList);
     $sw = undef;
     my $swArrRef = ONT7302HiCap::mnemonic_var2sw_multi($toolData->{RELEASE},uc($typeDetails[0]),uc($typeDetails[1]));
     if ($swArrRef) {
      my @swArr = @{$swArrRef};
      if (@swArr > 1 ) {
        $warnCount = $warnCount + @swArr - 1;
      }
      foreach $sw (@swArr) {
       $sw = substr($sw,1);
       $OntInfo{uc($typeDetails[0])}{uc($typeDetails[1])}{$sw}{'software'} = $sw;
       if ( defined($stageSw{$sw}{'failreason'}) ) {                             # BDFam51619
         $warnCount--;                                                           # BDFam51619
       } else {                                                                  # BDFam51619 
         $stageSw{$sw}{'failreason'} = "OK";                                     # BDFam51620
       }                                                                         # BDFam51619
      }                                                                         # BDFam51619
     } else {
       $OntInfo{uc($typeDetails[0])}{uc($typeDetails[1])}{'unknown'}{'software'} = "unknown";
       $stageSw{'unknown'}{'failreason'} = "Type not supported";
       _setfail(1);
       $lCount++;
     }
   }
   if($toolData->{DWLDONTRELMAPFILE}) {
         $warnCount++;                             
   }

if ($warnCount != $lCount) {                                                  # BDFam53543

#-- Step 4: Check whether the required ONT SWP are exist in local path
  ToolCommon::Log($nodeLogFh,"Attempt to Check the ONT SW package existence in local path\n");
 
  my $fileSize = undef;
  foreach my $sw (sort keys %stageSw) {
    if (defined $stageSw{$sw} ) {

      if ( uc($sw) eq 'UNKNOWN') {
        next;
      }  

      $fileSize = -s "$toolData->{LOCALDIR}/$sw";
      if ($fileSize) {
        $stageSw{$sw}{'localfilesize'} = $fileSize;
      } else {
        $stageSw{$sw}{'failreason'} = "ONT SWP ($toolData->{LOCALDIR}/$sw) does not exist";
        _setfail(1);
      }
    }
  }
      if($toolData->{DWLDONTRELMAPFILE}) {
        $OntRelMapFileSize = -s "$toolData->{ONTRELDIR}/$toolData->{ONTRELMAPFILE}"; 
        if (!$OntRelMapFileSize) {
          ToolCommon::Log($nodeLogFh,"ONT Releasemap file does not exist in <$toolData->{ONTRELDIR}/$toolData->{ONTRELMAPFILE}>\n");
          _setfail(1);
        }
      }
  #Logger::Log($nodeStatusLog, '[Update]', 'Local SW file existence check is done');

#-- Step 4.1: Populate the NE software info
  my $swInfo = IsamTL1::getONTSWver($tl1Session,$isam);
 
  # BDFam52357 starts here 
  if (! defined($swInfo) ) {
    FamilyCommon::IncrErrorCount("FAILED: Get ONT SW list returns empty \n",
                                    EC_STAGE_GET_ONTSWVER_FAILED, undef,
                                    1);
    return 0;
  }

  if ( $swInfo == 1 ) {
    ToolCommon::Log($nodeLogFh,"No ONT SWP found on OLT \n");
    $swInfo = undef;
  }
  # BDFam52357 ends here

#-- Step 6: Perform the Staging Type by Type for all given types
  my $totalSize = 0;
  foreach my $sw (sort keys %stageSw) {

    #-- Step 6.1: Check whether the Software is already exist in OLT
    if (! $stageSw{$sw}{'localfilesize'}) {
      if ( uc($sw) eq 'UNKNOWN') {
        next;
      }
    
      next if ( $stageSw{$sw}{'failreason'} ne "OK" );                 # BDFam51620

      $stageSw{$sw}{'failreason'} = "Local file does not exist";
      _setfail(1);
      next;
    }
 
    #my $swName = substr($sw,1);
    my $swName = $sw;
    if( defined($swInfo->{$swName}) ) {

      #-- Step 6.2: Compare the size, file available in OLT and local path
      if ( $stageSw{$sw}{'localfilesize'} != $swInfo->{$swName} ) { 
 
        ToolCommon::Log($nodeLogFh,"Attempt to remove software $swName from OLT");         
        #-- Step 6.3: Remove the already exisiting ONT SWP from OLT, in case of error
        $ret = IsamTL1::removeONTSWP($tl1Session, $swName);
        if (!$ret) {
          $stageSw{$sw}{'failreason'} = "Failed to remove already existing old file";
          _setfail(1);
          next;
        }
      } else {
        ToolCommon::Log($nodeLogFh,"Latest Software already exist..\n");
        $stageSw{$sw}{'failreason'} = "SUCCESS";                          # BDFam51620
        _update(1);
        next;
      }
    } 
    $totalSize = $totalSize + $stageSw{$sw}{'localfilesize'} if ( defined ($stageSw{$sw}{'localfilesize'}) );
    
  }

      if($toolData->{DWLDONTRELMAPFILE}) {
        if(defined($swInfo->{$ontrelmapName})) {
          if ($OntRelMapFileSize != $swInfo->{$ontrelmapName} ) { 
            $ret = IsamTL1::removeONTSWP($tl1Session, $ontrelmapName);
            if (!$ret) {
              ToolCommon::Log($nodeLogFh,"Failed to remove already existing old file\n");
              _setfail(1);
            }
          } else {
              ToolCommon::Log($nodeLogFh,"Latest ONT Releasemap File already exist..\n");
              ## $stageSw{$sw}{'failreason'} = "SUCCESS"; 
              _update(1);
          }
        }
      }

#-- Step 6.4: Check the available disk space
#   ToolCommon::Log($nodeLogFh,"Attempt to get NE disk space..\n");
#   Commented for BDFam55826
#    $ret = getNEdiskspace($snmpSession,$toolData->{RETRY});
#    if(!$ret) {
#      FamilyCommon::IncrErrorCount("Error: Failed to get free disk space \n",
#                             EC_STAGE_GET_DISK_SPACE_FAILED,undef,
#                             1);
#      return 0;
#    }

#    if ($ret && ($totalSize > $ret) ) {
#      FamilyCommon::IncrErrorCount("Error: Failed to get free disk space \n",
#                             EC_STAGE_GET_DISK_SPACE_FAILED,  
#                             sprintf("- Totalsize required %s, Available Free space %s",$totalSize, $ret),
#                             1);
#      return 0;
#    }

  #BDFam48393 starts here ..
  $ret = IsamTL1::AllowAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, "Alarms allowed...\n");
  } else {
    ToolCommon::Log($nodeLogFh, "Failed to allow alarms!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                               EC_ISAM_ALW_ALARM_FAILED,undef,
                               1);
    return 0;
  }
  #BDFam48393 ends here..

#-- Release the TL1 connection and Close
  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'release')) );
  FamilyCommon::CloseTL1Session2Close();


if ($complete != $warnCount ) {

#-- Step 7: Make a File transfer conncetion
  $ftSession = FileTransfer::OpenTelnet($toolData->{FTPMODE}, $isam->[0], $params->{USERNAME},
                          $params->{PASSWORD}, 10,undef);
  if (! $ftSession) {
    FamilyCommon::IncrErrorCount(sprintf("Failed to open Telnet session using given credentials\n"),
                               EC_STAGE_TELNET_OPEN_FAILED,undef,
                               1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "Telnet sesssion opened sucessfully\n");
  }
#  Logger::Log($nodeStatusLog, '[Update]', 'Connecting to telnet is successful');


#-- Step 8: Perform Staging ONT software
  foreach my $sw (sort keys %stageSw) {

    next if ( $stageSw{$sw}{'failreason'} ne "OK" );                 # BDFam51620
    
    ToolCommon::Log($nodeLogFh, "Attempt to stage file ..\n");
    #my $newSw = substr ($sw,1);
    my $newSw = $sw;
    ToolCommon::Log($nodeLogFh,sprintf("Software %s to OLT path %s\n",
                             "$toolData->{LOCALDIR}/$sw","$toolData->{SWDIR}/$newSw")); #BDFam48332

    my $res = FileTransfer::PUT($ftSession,"$toolData->{LOCALDIR}/$sw","$toolData->{SWDIR}/$newSw"); # BDFam52649 
    if (! $res) {
      my $err = FileTransfer::GetError();
      $stageSw{$sw}{'failreason'} = "Staging failed : $err";
      _setfail(1);
      if ( $err =~ /no write permission/ || 
           $err =~ /Connection reset/ ||                              # BDFam54235
           $err =~ /Broken pipe/ ||                                   # BDFam54235
           $err =~ /Permission denied/ ){                             # BDFam54235
        FamilyCommon::IncrErrorCount("Error: PUT failed \n",
                                     EC_STAGE_PUT_FAILED, $err,0);
        return 0;
      }
      next;
    }

    $stageSw{$sw}{'failreason'} = "SUCCESS";                           # BDFam51620
    $succ_Count++;
    _update(1);
  } # end of stageSw

#### Step 8.1 Perform transfer of the ONT RelMap file    TODO: check for R5.0 stream to put the ont relmapfile 
    if($toolData->{DWLDONTRELMAPFILE}) {
       ToolCommon::Log($nodeLogFh,sprintf("ONT RelMap File  %s to OLT path %s\n",
                             "$toolData->{ONTRELDIR}/$toolData->{ONTRELMAPFILE}","$toolData->{SWDIR}/$ontrelmapName"));

       my $res = FileTransfer::PUT($ftSession,"$toolData->{ONTRELDIR}/$toolData->{ONTRELMAPFILE}","$toolData->{SWDIR}/$ontrelmapName"); # BDFam52649 
       if (! $res) {
         my $err = FileTransfer::GetError();
         $stageSw{$sw}{'failreason'} = "Staging ONT RelMap file failed : $err";
         _setfail(1);
         if ( $err =~ /no write permission/ || 
           $err =~ /Connection reset/ ||                              # BDFam54235
           $err =~ /Broken pipe/ ||                                   # BDFam54235
           $err =~ /Permission denied/ ){                             # BDFam54235
           FamilyCommon::IncrErrorCount("Error: PUT failed \n",
                                     EC_STAGE_PUT_FAILED, $err,0);
           return 0;
         }
       }
    }

#  Logger::Log($nodeStatusLog, '[Update]', 'Staging completed');

#-- Step 9: Close the File Transfer connection
  FileTransfer::closeTelnet($ftSession);
  $ftSession = undef;
  ToolCommon::Log($nodeLogFh, "Telnet session closed sucessfully\n");
#  Logger::Log($nodeStatusLog, '[Update]', 'Telnet session closed');

}
if ($succ_Count) {
  my $ontParams = 1;
  $tl1Session = IsamTL1::Connect($isam->[0], $toolData->{TL1UDPPORT}, 
                                     $toolData->{TL1USER}, $toolData->{TL1PWD},
                                     $params->{DEBUG} ? $nodeLogFh : undef,$ontParams);
  if (!defined($tl1Session)) {
    FamilyCommon::IncrErrorCount(sprintf("  ERROR: Could not open TL1 session for %s: %s\n",
                                 FamilyCommon::GetFailedNode(), IsamTL1::GetError()),
                                 EC_OPENTL1_FAILED,undef,
                                 1);
    return 0;
  } else {
    ToolCommon::Log($nodeLogFh, "Established TL1 Connection sucessfully\n");
  }
  FamilyCommon::SetTL1Session2Close($tl1Session);  # Store for IncrErrorCount

  ToolCommon::Log($nodeLogFh, "Attempting to Acquire transaction...\n");
  return 0 if( ToolCommon::TransactionReleased() );
  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'acquire')) );

  $ret = IsamTL1::InhibitAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, "Alarms Inhibited..\n");
  } else {
    ToolCommon::Log($nodeLogFh, "Error in Alarm Inhibit!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                               EC_ISAM_INH_ALARM_FAILED,undef,
                               1);
    return 0;
  }

  my $swInfoUpd = IsamTL1::getONTSWver($tl1Session,$isam);

  # BDFam52357 starts here
  if (! defined($swInfoUpd) ) {
    FamilyCommon::IncrErrorCount("FAILED: Get ONT SW list returns empty \n",
                                    EC_STAGE_GET_ONTSWVER_FAILED, undef,
                                    1);
    return 0;
  }

  if ( $swInfoUpd == 1 ) {                                        
    ToolCommon::Log($nodeLogFh,"No ONT SWP found on OLT \n");
    $swInfoUpd = undef;
  }
  # BDFam52357 ends here

  if (defined($swInfoUpd) ) {
    foreach my $sw (sort keys %stageSw) {
      next if ( $stageSw{$sw}{'failreason'} ne "SUCCESS" );                 

      #my $swNameUpd = substr($sw,1);
      my $swNameUpd = $sw;
      if( defined($swInfoUpd->{$swNameUpd}) ) {
        if ( $stageSw{$sw}{'localfilesize'} != $swInfoUpd->{$swNameUpd} ) {
          $ret = IsamTL1::removeONTSWP($tl1Session, $swNameUpd);
          if (!$ret) {
            $stageSw{$sw}{'failreason'} = "Failed to remove incorrect file";
            #_setfail(1);
            next;
          }

          $stageSw{$sw}{'failreason'} = "Failed to stage software, stagged file is incorrect";
          #_setfail(1);
        }
      }

    } 
  }

  $ret = IsamTL1::AllowAlarms($tl1Session);
  if ($ret) {
    ToolCommon::Log($nodeLogFh, "Alarms allowed...\n");
  } else {
    ToolCommon::Log($nodeLogFh, "Failed to allow alarms!!!\n");
    FamilyCommon::IncrErrorCount(sprintf(" FAILED: %s\n", IsamTL1::GetError()),
                               EC_ISAM_ALW_ALARM_FAILED,undef,
                               1);
    return 0;
  }

#-- Release the TL1 connection and Close
  return 0 if( not ($ret = IsamTL1::Transaction($tl1Session, 'release')) );
  FamilyCommon::CloseTL1Session2Close();

}  # end of if ($succ_Count)

}  # BDFam53543 # end of if ($warnCount != $lCount)

#-- Step 10: Process the status file
  # Print all the status info
  foreach $type (sort keys %OntInfo) {
    foreach my $var (sort keys %{$OntInfo{$type}}) {
     foreach my $sw (sort keys %{$OntInfo{$type}{$var}}) {
      if(defined $OntInfo{$type}{$var}{$sw}{'software'}) {
        if (defined $stageSw{$OntInfo{$type}{$var}{$sw}{'software'}} ){
          if ( $stageSw{$OntInfo{$type}{$var}{$sw}{'software'}}{'failreason'} ne "SUCCESS" ) {
            print $statusFile sprintf("%s %s: software '%s' - %s",$type,$var,
                                                   $OntInfo{$type}{$var}{$sw}{'software'},
                                                   $stageSw{$OntInfo{$type}{$var}{$sw}{'software'}}{'failreason'}) . "\n";
          }
        }
      }
     }
    }
  }

# BDFam51219 starts 
  if ($warnExceeds) {
    FamilyCommon::IncrErrorCount("Error : Software Staging Failures exceeds The Warning Threshold",
                               EC_STAGE_WARN_EXCEEDS_THRESHOLD,undef,
                               1);
    return 0;
  }
# BDFam51219 ends

  if ($failCount) {
    FamilyCommon::IncrWarnCount("ONT Type List Partially Processed - $failCount ONTs failed",
                                 EC_LIST_PARTLY_PROCD, undef);
  }
  
  if ($params->{DEBUG}){
    ToolCommon::Log($nodeLogFh,"Total Files : $succ_Count. No of failed files: $failCount\n");
  }

  return 1;
}

1;
