=head1 PBMT common resource module

=begin html
This PERL module implements the common resources of PBMT.

=end html

=cut

#==============================================================================
#-- File:         ToolCommon.pm
#-- Date:         26.04.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Commons for the toolset
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel 2006
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.2.0 - 2006/03/22 : Introduction of capability model.
#--   1.4.2 - 2006/10/11 : Added max command line length of std unix cmd shell.
#--           2006/10/16 : SW package reduction ID.
#--   1.4.3 - 2006/10/18 : Timestamp logging
#--   1.4.3.4 - 2006/11/13 : ANTms22321 - transaction timeour values in units of msecs.
#--   1.4.3.5 - 2006/11/25 : ANTms22602 SW not reduced
#--   1.5.0   - 2006/12/11 : ANTms14531: pre- and post config/status checks
#--   1.5.1   - 2007/09/03 : Add optional retrycount parameter to DoSafeGet.
#--   1.5.2   - 2007/09/24 : ANTms64938 remove all traces of readable username and passwords.
#--   1.8.0   - 2007/08/03 : ANTms53180: ISAM-V migration, R3.5
#--                          Include version number of offline migration tools
#--   1.8.1   - 2008/01/04 : ANTms78847: New NT OFLMT version is 12.1.4
#--                                      New SHUB OFLMT version is 1.39
#--   1.8.2   - 2008/01/09: ANTms78942.  OSWP activation accompanied by an NT reset makes the OSWP activation
#--                         'fail': only the NT is activated (with the previous load).
#--   1.8.3   - 2008/01/22 : ANTms81426.  NT OFLMT version 12.1.4 -> 12.1.5
#--   1.8.4   - 2008/01/24 : ANTms81903.  An FD in extended-LT mode doesn not have an entry
#--                                       in the EQPT MIB for the second NT.
#--   1.8.5   - 2008/01/28 : ANTms82688. xVPS framework version changed to 1.2.1
#--   1.8.6   - 2008/01/30 : ANTms82853. NT framework version changed to 12.1.6
#--   1.8.7   - 2008/02/05 : Defined max-cc for mac@ concentration (convertion script)
#--   1.8.8   - 2008/02/20 : ANTms84960.  Latest SHUB OFLMT is 1.42
#--   1.8.9   - 2008/02/11 : ANTms83783: Check whether NT died during T&D handling.
#--   1.8.10  - 2008/02/25 : ANTms85750.  Allow ':' in SWP name
#--   1.8.11  - 2008/03/10 : ANTms87671.  New version for SHUB OFLMT
#--                          xVPS oflmt becomes 1.2.2
#--   1.8.12  - 2008/03/20 : ANTms89118.  New highest NT framework version.
#--   1.8.13  - 2008/04/07 : ANTmt01212: 13.2.0 is new highest NT OFLMT FW version
#--   1.8.14  - 2008/04/14 : ANTmt02718: 13.2.1 is new highest NT OFLMT FW version
#--   1.8.15  - 2008/04/22 : ANTmt04156: Latest IVPS OFLMT is 2.2.0
#--   1.8.16  - 2008/04/29 : ANTmt04599: additional check on 999/998 in combination
#--                          with a reduced SWP.
#--   1.8.17  - 2008/04/29 : ANTmt05521: IVPS OFLMT version changed to 2.2.1 for R3.6V
#--                          Updated NT OFLMT from 13.2.1 to 13.2.2 for R3.6
#--   1.8.18  - 2008/05/29 : ANTmt09138.  LT numbering starts from 1 in type-based
#--                          operator profiles.  Incorrect LT number is used to calculate
#--                          an ifIndex.
#--   1.8.19  - 2008/06/05 : ANTmt10248.  NT OFLMT version is now 13.2.3
#--   1.8.20  - 2008/06/24 : ANTmt13530 NT OFLMT version is now 13.2.4
#--   1.8.21  - 2008/07/03 : ANTmt14453.  Remove leading \s in regxep when checking for
#--                          voice capable boards.
#--   1.8.22  - 2008/07/13 : ANTmt16192.  New xVPS OFLMT version (2.2.2)
#--   1.8.23  - 2008/07/22 : ANTmt16653.  Latest NT OFLMT version is 14.0.0
#--   1.8.24  - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/07/25 : ANTmt17868.  Latest SHUB OFLMT is 1.45
#--             2008/08/18 :              Latest NT OFLMT is 14.0.1
#--                          ANTmt19864.  Latest IVPS OFLMT is 3.0.1
#--             2008/08/21 : ANTmt16940.  Sum of max number of MAC addresses per line
#--                                       may not exceed 1024 per LT.
#--   1.8.25 -  2008/09/16 : ANTmt23612 NT OFLMT v14.0.2
#--   1.8.26 -  2008/09/22 : ANTmt24391 SHUB OFLMT V1.46
#--   1.8.27 -  2008/09/29 : ANTmt25387 SHUB OFLMT V1.47
#--   1.8.28 -  2008/10/09 : ANTmt26750 xVPS OFLMT 3.0.2
#--   1.8.29 -  2008/10/13 : ANTmt27224 xVPS OFLMT 4.0.0
#--   1.8.30 -  2008/10/16 : ANTmt27866 NT OFLMT 14.0.3
#--   1.9.0  -  2008/09/22 : R3.7.10 and ISAM-IPD family
#--   1.9.1  -  2009/03/06 : For some reason R3.7 returns SNMP control but has attributes not writeable.
#                            keep in that wait loop until the attribute can be written
#--   1.9.2  -  2009/03/18 : ANTmt48688.  Maximum size for SNMP packets.
#--   1.9.3  -  2009/04/22 : Reverse retries and waitforever in TransactionRelease
#--   1.9.4  -  2009/04/30 : Added GetActiveOswpInfoBySnmp
#--   1.9.5  -  2009/05/06 : Moved function to set access operator id to this library.
#--   1.9.6  -  2009/05/20 : ANTmt58787/ANTmt59036.  TL1 can be disable or over SSH.  Added function to retrieve
#--                          and determine access security settings
#--   1.9.7  -  2009/05/25 : ANTmt60011.  Bitshift is >> rather then >
#--   1.9.8  -  2009/06/10 : ANTmt62662.  Allow SSH authentication using public-key
#--   1.9.9  -  2009/07/24 : ANTmt71238: ECNT-E conversion Changes
#--   1.9.10 -  2009/08/27 : Access operator ID retrieval via SNMP
#--   1.9.11 -  2009/09/10 : ReadHostsFile debugging prints confusing.
#--   1.9.12 -  2009/09/16 : ANTmt75993: determine NT type fails is NT at pos-B: passed @oid in stead of \@oid in the 2nd snmp-get
#--   1.9.13 -  2009/10/12 : Retry 3 CLI Connect attempts in case a 'no data read' error was detected.
#--   1.9.14 -  2009/10/21 : ANTmt80806. SWDL failure when SNMP v2 configured. Also added a Check for empty response 
#--                          on DoSafeGet for SNMPv3 session
#--   1.9.15 -  2009/11/12 : Cope with SNMPv2/v3 when deteriming NT type: the RU returns an empty string when an OID is not there. 
#--   1.9.16 -  2009/11/27 : ANTmt83962 & ANTmt86148
#--   3.0.00 -  2010/04/21 : Converted version to 3.0.00 for converged PBMT
#--   3.0.01 -  2010/05/24 : Converged PBMT changes
#--   3.0.02 -  2010/10/01 : for ALU00747293 - added a toolConstant to hold a default hash value
#--   3.0.03 -  2010/12/06 : ALU00926734
#--   3.0.04 -  2011/01/11 : LRM support added
#--   3.0.05 -  2011/02/09 : ALU01033842 
#--   3.0.06 -  2011/03/29 : ALU01031457 - Support for R4220 & R4230
#--   3.0.07 -  2011/05/17 : ALU01152509
#--   3.0.08 -  2011/05/24 : ALU01224557 
#--   3.0.09 -  2011/06/09 : ALU01235324 
#--   3.0.10 -  2011/06/16 : ALU01242257 - enable User-to-User Hairpin feature for R3.6.04k/g to R4.3 migration
#--   3.0.11 -  2012/02/20 : ALU01755206 - detect 7354 DB version from descriptor file
#--   3.0.12 -  2012/08/27 : eliminate warnings which occur while trying to print empty string 
#--   3.0.13 -  2012/10/09 : ALU01828441
#--   3.0.14 -  2013/02/06 : enable usage of second password for ssh if first one fails
#--   3.0.15 -  2013/02/15 : reduce sleep timing in OpenCli wrapper
#--   3.1.00 -  2013/02/27 : to suppress warnings while trying to set File Xfer settings when SFTP user information
#--                          is not configured in the ISAM
#--   3.1.01 -  2013/04/30 : Handle Ping command which goes uninterrupted on Linux
#--   3.1.02 -  2014/06/22 : ALU01991387 - Workaround for Release transaction owner issue
#--   3.1.03 -  2014/10/10 : ALU02016489
#--   3.1.04 -  2014/11/04 : ALU02020212
#--   3.1.05 -  2014/12/02 : Add subroutine for new swpfile command parameter for rollback script
#--   3.1.06 -  2015/02/13 : ALU02026412 : Transaction release owner period expired 
#--   3.1.07 -  2015/07/03 : ALU02095810 
#--   3.1.08 -  2015/07/29 : ALU02106765
#--   3.1.09 -  2015/08/04 : redirect the STDOUT and STDERR to a SFTPClient.txt log for sftpclient.pl execution
#--   3.1.10 -  2015/09/09 : ALU02119959; Ensure T&D is from R52, Not from R53
#--   3.1.11 -  2015/11/19 : ALU02144580
#--   3.1.12 -  2015/11/27 : ALU02145791
#--   3.1.13 -  2015/12/14 : ALU02156544
#--   3.1.14 -  2015/12/30 : ALU02160777
#--   3.1.15 -  2015/01/07 : ALU02163612
#--   3.1.16 -  2015/02/02 : ALU02167196, ALU02168421
#--   3.1.17 -  2016/03/01 : ALU02182211
#--   3.1.18 -  2016/03/15 : ALU02188504
#--   3.1.19 -  2016/03/16 : ALU02190562, ALU02190558
#--   3.1.20 -  2016/04/28 : ALU02206611
#--   3.1.21 -  2016/05/27 : ALU02188053
#--   3.1.22 -  2016/07/27 : Transaction Release error in TransactionReleased (before through cli lock/acruire became obselete)
#--   3.1.23 -  2016/07/29 : ALU02237966
#--   3.1.24 -  2016/08/26 : ALU02263616
#--   3.1.25 -  2016/11/21 : ALU02285181
#--   3.1.26 -  2016/12/13 : ALU02310440
#--   3.1.27 -  2017/04/12 : ALU02345022
#--   3.1.28 -  2017/07/17 : ALU02373989
#--   3.1.29 -  2017/08/09 : ALU02380215
#--   3.2.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--   3.2.1  - 2018/08/22 : ONT Mig via CLI
#--   3.2.2  - 2019/02/28 : ALU02550344   
=head1 VERSION

=begin html
Current version: "ToolCommon.pm -- 3.2.2 -- 2019/02/28"

=end html

=cut

my $VERSION = "ToolCommon.pm -- 3.2.2 -- 2019/02/28";
#==============================================================================

package ToolCommon;

use strict;
use warnings 'all';
use PBMTGlobal;
require SNMPComm;
use Logger qw(:ALL);
use IO::Socket::INET;

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub GetError();
sub SelectFH($);
sub SetDebug($);
sub OpenCli($$$);
sub NotReadyForSNMP($$);
sub DoSafeGet($$;$);
sub DoSafeGetNext($$;$$);
sub GetActiveOswpInfoBySnmp($$;$);
sub GetSwpBySnmp($$$$;$);
sub ExtractSwFileName($$$$);
sub GetNtLoadBySnmp($$$$;$);
sub GetOswpInfo($$$$$$;$);
sub GetOswpInfoTL1($$$$$$$;$);
sub CheckEnv($);
sub Ping($$);
sub Log($$;$$);
sub GetSwParSize(;$$$);
sub ActivateSwp($);
sub ReadHostsFile($$$;$);
sub ReadReleaseFile($);
sub TransactionReleased(;$$$$);
sub ReleaseTransaction(;$$$$);
sub AcquireTransaction(;$$$$$);
sub DetermineNtType();
sub IsamHasVoice($);
sub IsamHasVoiceServer($);
sub BuildShubConfigFile($$$$$$$$$$;$);
sub NtIsUp();
sub ValidIP($);
sub IsTransferOnGoing();
sub CheckUnicastMacLimits($$$$);
sub CountPackages($$);
sub CheckMcPackages($$$);
sub PrepareCliEnv($$);
sub GetFxrSettings($$);
sub FxrServerIsThisNode($$);
sub AccessOprIdConfigure($$$$);
sub GetAccessettings($$);
sub TL1AccessEnabled($$);
sub TL1ViaSSH($$);
sub TL1ViaUDP($$);
sub CliViaSSH($$);
sub CliViaTelnet($$);
sub SshPossible($$$);
sub GetAccessOperId($$);
sub GetNEdiskspace($$$);
sub CheckVlanUniqHostId($); 
sub GetDBVersion($); 
sub CheckMiscLT($); 
sub CheckLTSWPresence($);
sub SFTPClientWrap($$$$$$);
sub KnownHostCheck($$);
sub AlarmRetrieval($$$);
sub CreateDirectory($$);

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS
            %toolConsts @PBMT_CONSTANTS);

use Exporter();
@ISA = qw(Exporter);

my $connSSH;#New

=head1 EXPORTED DATA

=head2 toolConsts

=begin html
A list of functions returning the associated data value as defined in the private toolcommon data hash table.<br>
To include the 'CONSTANT' into the code use the following construction:<br>
<br>
<tt>use ToolCommon qw(:CONSTANTS);
<br>

=end html

=cut

@PBMT_CONSTANTS = qw(SYSUPTIME CYPHER REGEXP MIBVERSIONOID PLANNEDBOARD_5X_OID INSERTEDBOARD_5X_OID ACTIVEOSWP_5X_OID OSWPACTIVATION_OID VARIANT_RELEASE_REGEXP 
                     TRANSACTOWNER MAXCC_MACCONC SWPSYNTAX UPLOADACTACTDB ASAMDBMUPLOADSERVERID ASAMDBMUPLOADPATH ASAMDBMUPLOADDBSELECTION
		     ASAMDBMUPLOADPROGRESS ASAMDBMUPLOADERROR DBUPLOADONGOING DBUPLOADOK DBUPLOADERROR TRANSACT_TIME MINIMALSWPARSIZE
		     MINIMALSWPARSIZE_ECNTC IFOPERSTATUS_OID XDSLLNPWRDS_OID XDSLCHFEACTBRDS_OID XDSLCHFEATTBRDS_OID XDSLCHFEACTIMPNUS_OID
		     XDSLCHACTBRUS_OID XDSLCHATTBRUS_OID XDSLCHIMPNDS_OID NO_OSWP FXRPROTOCOL_OID FXRSERVERUSERNAME_OID DISK_FREE_SPACE TFTP_FXR
		     FTP_FXR SFTP_FXR FXRPROTOCOL_NAME SYSTEMINITPROGRESSOID MAXSNMPMSGSIZE INTERNALIP MAX_SNMPV2_3_REP NT_CONTEXT MAX_REP
		     SFTPCLIENTUSERNAMEOID ASAMERRORCODE ASAMERRORDESCRIPTION HASHDEFVAL PREOPERCHECKFILETAG POSTOPERCHECKFILETAG VLANVMACADDR);

@EXPORT = ();
@EXPORT_OK = (@PBMT_CONSTANTS);
%EXPORT_TAGS = (CONSTANTS => [@PBMT_CONSTANTS],
		ALL => [@EXPORT_OK]);


sub SYSUPTIME()                     { $toolConsts{"SYSUPTIME"} }
sub CYPHER()                        { $toolConsts{"CYPHER"} }
sub REGEXP()                        { $toolConsts{"REGEXP"} }
sub VARIANT_RELEASE_REGEXP()        { $toolConsts{"VARIANT_RELEASE_REGEXP"} }
sub MIBVERSIONOID()                 { $toolConsts{"MIBVERSIONOID"} }
sub SYSTEMINITPROGRESSOID()         { $toolConsts{"SYSTEMINITPROGRESSOID"} }
sub PLANNEDBOARD_5X_OID()           { $toolConsts{"PLANNEDBOARD_5X_OID"} }
sub INSERTEDBOARD_5X_OID()          { $toolConsts{"INSERTEDBOARD_5X_OID"} }
sub ACTIVEOSWP_5X_OID()             { $toolConsts{"ACTIVEOSWP_5X_OID"} }
sub OSWPACTIVATION_OID()            { $toolConsts{"OSWPACTIVATION"} }
sub TRANSACTOWNER()                 { $toolConsts{"TRANSACTOWNER"} }
sub SWPSYNTAX()                     { $toolConsts{"SWPSYNTAX"} }
sub MAXCC_MACCONC()                 { $toolConsts{"MAXCC_MACCONC"} }
sub UPLOADACTACTDB()                { $toolConsts{"UPLOADDBSELECTION"}{"ACTUALACTIVE"} }
sub ASAMDBMUPLOADSERVERID()         { $toolConsts{"ASAMDBMUPLOADSERVERID"} }
sub ASAMDBMUPLOADPATH()             { $toolConsts{"ASAMDBMUPLOADPATH"} }
sub ASAMDBMUPLOADDBSELECTION()      { $toolConsts{"ASAMDBMUPLOADDBSELECTION"} }
sub ASAMDBMUPLOADPROGRESS()         { $toolConsts{"ASAMDBMUPLOADPROGRESS"} }
sub ASAMDBMUPLOADERROR()            { $toolConsts{"ASAMDBMUPLOADERROR"} }
sub DBUPLOADONGOING()               { $toolConsts{"UPLOADPROGRESS"}{"ONGOING"} }
sub DBUPLOADOK()                    { $toolConsts{"UPLOADPROGRESS"}{"FINISHEDOK"} }
sub DBUPLOADERROR($)                { my $err = shift;
				      defined($toolConsts{UPLOADERRORS}{$err}) ? $toolConsts{UPLOADERRORS}{$err} : "unknown error: $err";
				    }
sub TRANSACT_TIME()                 { $toolConsts{"TRANSACT_TIME"} }
sub MINIMALSWPARSIZE()              { $toolConsts{"MINIMALSWPARSIZE"} }
sub MINIMALSWPARSIZE_ECNTC()        { $toolConsts{"MINIMALSWPARSIZE_ECNTC"} }
sub IFOPERSTATUS_OID()              { $toolConsts{"IFOPERSTATUS"} }
sub XDSLLNPWRDS_OID()               { $toolConsts{"XDSLLNPWRDS"} }
sub XDSLCHFEACTBRDS_OID()           { $toolConsts{"XDSLCHFEACTBRDS"} }
sub XDSLCHFEATTBRDS_OID()           { $toolConsts{"XDSLCHFEATTBRDS"} }
sub XDSLCHFEACTIMPNUS_OID()         { $toolConsts{"XDSLCHFEACTIMPNUS"} }
sub XDSLCHACTBRUS_OID()             { $toolConsts{"XDSLCHACTBRUS"} }
sub XDSLCHATTBRUS_OID()             { $toolConsts{"XDSLCHATTBRUS"} }
sub XDSLCHIMPNDS_OID()              { $toolConsts{"XDSLCHIMPNDS"} }
sub NO_OSWP()                       { $toolConsts{"NO_OSWP"} }
sub FXRPROTOCOL_OID()               { $toolConsts{"FXRPROTOCOL_OID"} }
sub FXRSERVERUSERNAME_OID()         { $toolConsts{"FXRSERVERUSERNAME_OID"} }
sub DISK_FREE_SPACE()               { $toolConsts{"DISK_FREE_SPACE"} }     
sub TFTP_FXR()                      { $toolConsts{"FXRSETTINGS"}{"tftp"} }
sub FTP_FXR()                       { $toolConsts{"FXRSETTINGS"}{"ftp"} }
sub SFTP_FXR()                      { $toolConsts{"FXRSETTINGS"}{"sftp"} }
sub MAXSNMPMSGSIZE()                { $toolConsts{"MAXSNMPMSGSIZE"} }
sub FXRPROTOCOL_NAME($)             { my $num = shift;
				      defined($toolConsts{FXRNAMES}{$num}) ? $toolConsts{FXRNAMES}{$num} : "unknown protocol: $num";
				    }
sub INTERNALIP()                    { $toolConsts{"INTERNALIP"} }
sub MAX_SNMPV2_3_REP()              { $toolConsts{"MAX_REP"} }
sub NT_CONTEXT()                    { $toolConsts{"NT_CONTEXT"} }
sub MAX_REP()                       { $toolConsts{"MAX_REP"} }
sub SFTPCLIENTUSERNAMEOID()         { $toolConsts{"SFTPCLIENTUSERNAMEOID"} }
sub ASAMERRORCODE()                 { $toolConsts{"ASAMERRORCODE"} }
sub ASAMERRORDESCRIPTION()          { $toolConsts{"ASAMERRORDESCRIPTION"} }
sub HASHDEFVAL()                    { $toolConsts{"HASHDEFVAL"} }
sub PREOPERCHECKFILETAG()           { $toolConsts{"PREOPERCHECKFILETAG"} }
sub POSTOPERCHECKFILETAG()          { $toolConsts{"POSTOPERCHECKFILETAG"} }
sub VLANVMACADDR()                  { $toolConsts{"VLANVMACADDR"} }

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>File::Basename</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2101-DFZZA">Net::Telnet</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2099-DFZZA">Net::Ping</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2087-DFZZA">IsamCli</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2091-DFZZA">Octopus</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
    </ul>
  </li>
</ul>

=end html

=cut

use File::Basename;
use Config;

#==============================================================================

# Don't include use/require Capabilty.  There seems to an issue (Capabilty uses ToolCommon already).  One way or another if
# this IS done then toolConsts{xxx} seems to have an issue in the Capability module.  I'll try to find out why that is but
# as long as no explanation has been found: a warned person....

require IsamCli;
require IsamTL1;
require Octopus;
use Net::Ping;
use Net::SNMP;
use Net::SSH::Perl::Key;
use ErrorCodes qw(:EXIT_CODES :ONT_ERR :COMMON);

%toolConsts  = ('NO_OSWP'                       => 'NO_OSWP',
		'UPLOADDBSELECTION'             => {'ACTUALACTIVE' => 1},
		'UPLOADPROGRESS'                => {'ONGOING' => 1,
						    'FINISHEDOK' => 2,
						    'FINISHEDERR' => 3},
		'FXRSETTINGS'                   => {'tftp' => 1,
						    'ftp' => 2,
						    'sftp' => 3},
		'FXRNAMES'                      => {1 => 'tftp',
						    2 => 'ftp',
						    3 => 'sftp'},
		'OSWPPROGRESS'                  => {'EMPTY' => 1,
						    'ENABLED' => 2,
						    'DISABLED' => 3,
						    'DOWNLOADING' => 4,
						    'ABORTING' => 5},
		'UPLOADERRORS'                  => {0 => 'Undefined',
						    1 => 'File not found',
						    2 => 'Access violation',
						    3 => 'Disk full/allocation exceeded',
						    4 => 'Illegal TFTP operation',
						    5 => 'Unknown transfer ID',
						    6 => 'File already exists',
						    7 => 'No such user',
						    8 => 'Selected DB not available',
						    9 => 'System restart',
						    10 => 'No error'},
		'SYSUPTIME'                     => '.1.3.6.1.2.1.1.3.0',
		'OSWPACTIVATION'                => '.1.3.6.1.4.1.637.61.1.24.1.1.1.6',
		'OSWPOPERACTSTATUS'             => '.1.3.6.1.4.1.637.61.1.24.1.1.1.4',
		'OSWPOPERAVAILSTATUS'           => '.1.3.6.1.4.1.637.61.1.24.1.1.1.3',
		'OSWPNAME'                      => '.1.3.6.1.4.1.637.61.1.24.1.1.1.2',
		'ASAMSWMCONSIDEREDSWPNAME'      => '.1.3.6.1.4.1.637.61.1.24.1.3.1.2',
		'ASAMSWMFILEBOARDTYPE'          => '.1.3.6.1.4.1.637.61.1.24.1.5.1.3',
		'FREESPACEOID'                  => '.1.3.6.1.4.1.637.61.1.24.1.6.0',
		'ASAMDBMDATABASEOPERSTATUS'     => '1.3.6.1.4.1.637.61.1.24.2.1.1.4',
		'ASAMDBMDATABASEVERSION'        => '1.3.6.1.4.1.637.61.1.24.2.1.1.5',
		'ASAMDBMDNLOADSERVERID'         => '.1.3.6.1.4.1.637.61.1.24.2.2.0',
		'ASAMDBMDNLOADPATH'             => '.1.3.6.1.4.1.637.61.1.24.2.3.0',
		'ASAMDBMDNLOADPROGRESS'         => '.1.3.6.1.4.1.637.61.1.24.2.4.0',
		'ASAMDBMDNLOADERROR'            => '.1.3.6.1.4.1.637.61.1.24.2.10.0',
		'ASAMDBMUPLOADSERVERID'         => '.1.3.6.1.4.1.637.61.1.24.2.6.0',
		'ASAMDBMUPLOADPATH'             => '.1.3.6.1.4.1.637.61.1.24.2.7.0',
		'ASAMDBMUPLOADDBSELECTION'      => '.1.3.6.1.4.1.637.61.1.24.2.8.0',
		'ASAMDBMUPLOADPROGRESS'         => '.1.3.6.1.4.1.637.61.1.24.2.9.0',
		'ASAMDBMUPLOADERROR'            => '.1.3.6.1.4.1.637.61.1.24.2.10.0',
		'ASAMATMVCLENCAPSAUTODETECT'    => '.1.3.6.1.4.1.637.61.1.4.1.75.1.2',
		'QOSSCHEDULERPROFILENAME'       => '.1.3.6.1.4.1.637.61.1.47.3.4.1.2',
		'ASAMIFEXTCUSTOMERID'           => '.1.3.6.1.4.1.637.61.1.6.5.1.1',
		'ASAMIFEXTALMREPORTINGSEVERITY' => '.1.3.6.1.4.1.637.61.1.6.5.1.2',
		'MIBVERSIONOID'                 => '.1.3.6.1.4.1.637.61.1.9.13.0',
		'SYSTEMINITPROGRESSOID'         => '.1.3.6.1.4.1.637.61.1.9.15.0',
		'SYSTEMACCESSOPERATORID'        => '.1.3.6.1.4.1.637.61.1.9.33.0',
		'PLANNEDBOARD_5X_OID'           => '.1.3.6.1.4.1.637.61.1.23.3.1.2',
		'INSERTEDBOARD_5X_OID'          => '.1.3.6.1.4.1.637.61.1.23.3.1.3',
		'ACTIVEOSWP_5X_OID'             => '.1.3.6.1.4.1.637.61.1.24.1.1.1.4',
		'TRANSACTOWNER'                 => '.1.3.6.1.4.1.637.61.1.11.1.0',
		'ASAMTRANSACTIONTIMEOUT'        => '.1.3.6.1.4.1.637.61.1.11.10.0',
		'IGMPCHNLFULLVIEW'              => '.1.3.6.1.4.1.637.61.1.29.2.1.6',
		'IGMPCHNLPREVIEW'               => '.1.3.6.1.4.1.637.61.1.29.2.1.30',
		'XDSLCHFEACTBRDS'               => '.1.3.6.1.4.1.637.61.1.39.6.1.1.1.1',
		'XDSLCHFEATTBRDS'               => '.1.3.6.1.4.1.637.61.1.39.6.1.1.1.2',
		'XDSLCHFEACTIMPNUS'             => '.1.3.6.1.4.1.637.61.1.39.6.1.1.1.4',
		'XDSLCHACTBRUS'                 => '.1.3.6.1.4.1.637.61.1.39.5.1.1.1.1',
		'XDSLCHATTBRUS'                 => '.1.3.6.1.4.1.637.61.1.39.5.1.1.1.2',
		'XDSLCHIMPNDS'                  => '.1.3.6.1.4.1.637.61.1.39.5.1.1.1.4',
		'XDSLLNPWRDS'                   => '.1.3.6.1.4.1.637.61.1.39.3.8.1.1.3',
		'EXTENDVLANPORTMAXNUMMACADDR'   => '.1.3.6.1.4.1.637.61.1.31.2.5.1.3',
		'EXTENDSTATICUNICASTALWTOGOTO'  => '.1.3.6.1.4.1.637.61.1.31.3.1.1.2',
		'FXRPROTOCOL_OID'               => '.1.3.6.1.4.1.637.61.1.74.1.0',
		'FXRSERVERUSERNAME_OID'         => '.1.3.6.1.4.1.637.61.1.74.2.1.2',
		'DISK_FREE_SPACE'       	=> '.1.3.6.1.4.1.637.61.1.24.1.7.0',
		'IFTYPE'                        => '.1.3.6.1.2.1.2.2.1.3',
		'IFADMINSTATUS'                 => '.1.3.6.1.2.1.2.2.1.7',
		'IFOPERSTATUS'                  => '.1.3.6.1.2.1.2.2.1.8',
		'IFLINKUPDOWNTRAPENABLE'        => '.1.3.6.1.2.1.31.1.1.1.14',
		'DOT1DBASEPORTIFINDEX'          => '.1.3.6.1.2.1.17.1.4.1.2',
		'SFTPCLIENTUSERNAMEOID'         => '.1.3.6.1.4.1.637.61.1.57.5.0',
		'ASAMERRORCODE'                 => '.1.3.6.1.4.1.637.61.1.12.1.1.1',
		'ASAMERRORDESCRIPTION'          => '.1.3.6.1.4.1.637.61.1.12.1.1.2',
		'MINIMALSWPARSIZE'              => 126724096,
		'MINIMALSWPARSIZE_ECNTC'        => 151539712,
		'REDUCEDSWP'                    => 998,
		'REGEXP'                        => "......(\\d)(\\d)\.(\\d)(\\d\\d)",
		'VARIANT_RELEASE_REGEXP'        => "....(.)(.)((\\d)(\\d)\.(\\d)(\\d\\d))",
		'RETRIES'                       => 5,
		'MAXEXECSIZE'                   => 750000,
		'CMDLENGTH'                     => 255,
		'TRANSACT_TIME'                 => 360000,
		'XDSLLINETYPE'                  => 249,
		'SWPSYNTAX'                     => "[^\\w|\\.|\\/|:|-]",
		'CYPHER'                        => "!@#\$ab12'",
		'EQPTSLOTACTUALTYPE'            => "1.3.6.1.4.1.637.61.1.23.3.1.3",
		'EQPTINVTYPENAME'               => "1.3.6.1.4.1.637.61.1.23.3.1.14",
		'ACCESS_SEC_OID'                => "1.3.6.1.4.1.637.61.1.57.2.0",
		'PUBLIC_KEY_OID'                => "1.3.6.1.4.1.637.61.1.57.1.1.4.1",
		'NTA_EQPT_IDX'                  => 0x1101,
		'NTB_EQPT_IDX'                  => 0x1102,
		'MAXCC_MACCONC'                 => 8,
		'MAXNUMMACADDRPERLT'            => 1024,    # Default maximum.  Values per LT maintained in
		'DYNAMICIFINDEXBIT'             => 0x08000000,
		'DYNAMICLTMASK'                 => 0x01f00000,
		'DYNAMICRACKMASK'               => 0x70000000,
		'DYNAMICRACKSHIFT'              => 28,
		'DYNAMICRACKOFFSET'             => 1,
		'DYNAMICSHELFMASK'              => 0x06000000,
		'DYNAMICSHELFSHIFT'             => 25,
		'DYNAMICSHELFOFFSET'            => 1,
		'DYNAMICLTSHIFT'                => 20,
		'DYNAMICLTOFFSET'               => 2,
		'STATICRACKMASK'                => 0x70000000,
		'STATICRACKSHIFT'               => 28,
		'STATICRACKOFFSET'              => 0,
		'STATICSHELFMASK'               => 0x07000000,
		'STATICSHELFSHIFT'              => 24,
		'STATICSHELFOFFSET'             => 0,
		'STATICLTMASK'                  => 0x00ff0000,
		'STATICLTSHIFT'                 => 16,
		'STATICLTOFFSET'                => 1,
		'HOLDERRACKSHIFT'               => 12,
		'HOLDERSHELFSHIFT'              => 8,
		'MAXSNMPMSGSIZE'                => 65534,
		'INTERNALIP'                    => "127.0.0.1",
		'MAX_REP'                       => 10,
                'HASHDEFVAL'                    => 0,
		'NT_CONTEXT'                    => "nt",
		'PREOPERCHECKFILETAG'           => "preOperCheck",
		'POSTOPERCHECKFILETAG'          => "postOperCheck",
                'VLANVMACADDR'                  => 8191
	       );


my $ERROR = "";
my $DEBUG = undef;
my $AcquireThroughCLI = 0;

sub GetVersion() {
  return $VERSION;
}

#==============================================================================
#== Functions
#-- All functions set $ERROR in case of failure.
#==============================================================================

=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetError() {
  return $ERROR;
}

sub CleanUp() {
  $ERROR = "";
  $connSSH = undef;
  if( defined $params ) {
      if( defined $params->{USEDEBUGVIASSH} ) {
         $params->{USEDEBUGVIASSH} = 0;
      }
      if( defined $params->{USESSH} ) {
         $params->{USESSH} = 0;
      }
  }
}


=head2 SelectFH($)

=begin html
Make sure that the file handle supplied by $handle can be used as a file handle by this PERL module when writing debug information.

=end html

=cut

sub SelectFH($) {
  my ($handle) = @_;

  select((select($handle), $|=1)[$[]);
  $handle;
}

=head2 SetDebug($)

=begin html
Enable additional debugging in this module.

=end html

=cut

sub SetDebug($) {
  my ($fh) = @_;

  $DEBUG = &SelectFH($fh);
#  print $DEBUG "Testing...\n";
}


################################################################################
### Routine to create directory and change the ownership
################################################################################
sub CreateDirectory($$) {
   my ($passedLogFH, $dirPath) = @_;
   if( !defined $dirPath or $dirPath eq "" ) {
       LogError( $passedLogFH, "Directory is not passed/NULL\n");
       exit CREATE_DIR_FAILED_CODE;
   }
   if (! -d $dirPath) {
      if (!mkdir( $dirPath, 0777)) {
         LogError( $passedLogFH, "Unable to create directory '$dirPath' : $!\n");
         exit CREATE_DIR_FAILED_CODE;
      }
    }
    if ($params->{UMASK_CHMOD} and ! chmod(01777, $dirPath)) {
       LogWarn( $passedLogFH,  "Failed to chmod 1777 for directory '$dirPath' : $!\n");
    }
}



=head2 OpenCli($$$$)

=begin html
Open the CLI and retry up to 5 times in case of failure with an interval of 10 seconds.  After opening the CLI we will inhit the alarms and turn
off the interactive mode of the CLI.<br>
The following parameters are used:
<table>
  <tr><td>isam</td><td>isam context.  ->[0] holds IP address, ->[1] holds ISAM name</td></tr>
  <tr><td>fh</td><td>pointer to an open file (or undef when not needed).  If a pointer is specified it is used to log all TELNET activity.</td></tr>
  <tr><td>params</td><td>pointer to the tool command options</td></tr>
  <tr><td>timeout</td><td>telnet open timeout value</td></tr>
</table>
Returns a pointer to the opened telnet context or the undefined value in case of error.

=end html

=cut

sub OpenCli($$$) {
  my ($isam, $fh, $timeout) = @_;
  my $cli = undef;
  my $tries = 0;

  CheckEnv("OpenCli");
  while (! defined($cli) && $tries < 3) {
    if (defined $params->{USEDEBUGVIASSH} && $params->{USEDEBUGVIASSH}) {
      $cli = IsamCli::ConnectSecDebugViaSSH($isam->[0], $timeout, $fh );
    } elsif ($params->{USESSH}) {
      $cli = IsamCli::ConnectSSH($isam->[0], $timeout, $fh);
    } else {
      $cli = IsamCli::Connect($isam->[0], $timeout,
			      $params->{DEBUG} ? $fh : undef);
    }
    if (!defined($cli)) {
      if (IsamCli::TelnetNoDataRead(IsamCli::GetError())) {
	Log($DEBUG, " EOF detected, retying\n");
	$tries++;
	sleep 5;
      } else {
	$tries = 4;
      }
    }
  }
  if (!defined $cli) {
    $ERROR = IsamCli::GetError();
  } else {
    if (defined $params->{USEDEBUGVIASSH} && $params->{USEDEBUGVIASSH}) {
        Log($DEBUG, "Got CLI\n");
        return $cli;
    }
    IsamCli::SetCliConnection($cli);
    if (!PrepareCliEnv($cli, $params->{DEBUG})) {
      IsamCli::Disconnect($cli);
      $cli = undef;
      IsamCli::SetCliConnection($cli);
    }
  }
  Log($DEBUG, "Got CLI\n");
  return $cli;
}

=head2 NotReadyForSNMP($$)

=begin html

=end html

=cut

sub NotReadyForSNMP($$) {
  my ($session, $debug) = @_;
  my $err = $session->error();

  Log($DEBUG, "Testing SNMP error '$err'") if $debug;
  return (($err =~ /No response/) || ($err =~ /noSuchName|noSuchObject/)) ? 1 : 0;
}

=head2 DoSafeGet($$;$)

=begin html
Perform an SNMP get (with retries in case of failure - interval 1 sec) on $oid using the opened SNMP session context $session.  Either the value read
is returned or the undefined value in case of failure.

=end html

=cut

sub DoSafeGet($$;$) {
  my ($session, $oid, $retryCount) = @_;
  my ($reply, $value);
  my $count = -1;
  my $retries = defined $retryCount ? $retryCount : $toolConsts{RETRIES};
  my $version = $session->version();
  my @oids;

  CheckEnv("DoSafeGet");
  $oids[0] = $oid;
  LogDebug( $DEBUG,"\n", 1);
  LogDebug( $DEBUG,"--------------- OID($retries): $oid ---------------\n");
  while ($count < $retries && !defined($value)) {
    if ($version < 2) {
      # SNMPv1/SNMPv2
      $reply = $session->get_request(-varbindlist => \@oids);
    } else {
      # SNMPv3
      $reply = $session->get_request(-varbindlist => \@oids,
				     -contextname => NT_CONTEXT);
    }
    $value = $reply->{$oids[0]};
    $count++;
    if (!defined($value)) {
      sleep 1;
    }
  }
  my $valuemesg = defined $value ? $value : "#NULL#";
  LogDebug( $DEBUG,"--------------- VALUE : $valuemesg ---------------\n\n");
  return $value;
}

=head2 DoSafeGetNext($$;$$)

=begin html
Perform an SNMP get-next (with retries in case of failure - interval 1 sec) on $oid using the opened SNMP session context $session.  Either the value
read is returned or the undefined value in case of failure.

=end html

=cut

sub DoSafeGetNext($$;$$) {
  my ($session, $oid, $retryCount, $debug) = @_;
  my $reply;
  my $value = undef;
  my $count = 0;
  my @oids;
  my $retries = defined $retryCount ? $retryCount : $toolConsts{RETRIES};
  my $version = $session->version();
  my $lastKey;

  CheckEnv("DoSafeGetNext");
  $oids[0] = $oid;
  LogDebug( $DEBUG,"\n", 1);
  LogDebug( $DEBUG,"--------------- OID($retries) : $oid ---------------\n");
  while ($count < $retries && !defined($value)) {
    if ($version < 2) {
      # SNMPv1/SNMPv2
      $reply = $session->get_next_request(-varbindlist => \@oids);
    } else {
      # SNMPv3
      $reply = $session->get_next_request(-varbindlist => \@oids,
					  -contextname => NT_CONTEXT);
    }
    if (defined($reply)) {
      foreach my $key (keys %{$reply}) {
	Log($DEBUG, sprintf("key: %s -> %s\n", $key, defined($reply->{$key}) ? $reply->{$key} : "not defined")) if (defined($debug) && $debug);
	$lastKey = $key;
      }
      $value = $reply->{$lastKey};
    }
    $count++;
    if (!defined($value)) {
      sleep 1;
    }
  }
  my $valuemesg = defined $value ? $value : "#NULL#";
  LogDebug( $DEBUG,"--------------- VALUE : $valuemesg ---------------\n\n");
  return $value;
}


=head2 GetActiveOswpInfoBySnmp($$;$)
Retrieve the OSWP name of the active OSWP.  Returns undef in case of failure.

=begin html

=end html

=cut

my $OSWP_ACTIVE = 1;
my $OSWP_NOT_ACTIVE = 2;

sub GetActiveOswpInfoBySnmp($$;$) {
  my ($snmpSession, $debug, $retries) = @_;
  my @oids;
  my $result = undef;
  my $name = undef;
  my $version = $snmpSession->version();

  $oids[0] = $toolConsts{OSWPOPERACTSTATUS} . ".1";
  $oids[1] = $toolConsts{OSWPOPERACTSTATUS} . ".2";
  if ($version < 2) {
    # SNMPv1/SNMPv2
    $result = $snmpSession->get_request(-varbindlist => \@oids);
  } else {
    # SNMPv3
    $result = $snmpSession->get_request(-varbindlist => \@oids,
					-contextname => NT_CONTEXT);
  }
  if (defined($result) && ($snmpSession->error() eq "")) {
    if ($result->{$oids[0]} == $OSWP_ACTIVE) {
      @oids = ();
      $oids[0] = $toolConsts{OSWPNAME} . ".1";
    } else {
      @oids = ();
      $oids[0] = $toolConsts{OSWPNAME} . ".2";
    }
    if ($version < 2) {
      # SNMPv1/SNMPv2
      $result = $snmpSession->get_request(@oids);
    } else {
      # SNMPv3
      $result = $snmpSession->get_request(-varbindlist => \@oids,
					  -contextname => NT_CONTEXT);
    }
    if (!defined($result) || ($snmpSession->error() ne "")) {
      $ERROR = sprintf("failed to retrieve active OSWP name: %s", $snmpSession->error());
    } else {
      $name = $result->{$oids[0]};
    }
  } else {
    $ERROR = sprintf("failed to retrieve OSWP state info: %s", $snmpSession->error());
  }
  return $name;
}

sub GetSourceDetails() {
     my %oidDetails = (
                     "$toolConsts{OSWPNAME}.1" => 'OSWP-1 NAME',
                     "$toolConsts{OSWPNAME}.2" => 'OSWP-2 NAME',
                     "$toolConsts{OSWPOPERACTSTATUS}.1" => 'OSWP-1 OPR-STATUS',
                     "$toolConsts{OSWPOPERACTSTATUS}.2" => 'OSWP-2 OPR-STATUS',
                     );
     $params->{SOURCE_DBVERSION} = "";
     my $snmpResponse = SNMPComm::GetValueList( 'OSWP DETAILS', \%oidDetails );
     my $activeOswpIndex =  ($snmpResponse->{"$toolConsts{OSWPOPERACTSTATUS}.1"} == $OSWP_ACTIVE ) ? 1 : 2;
     $params->{SOURCE_OSWPNAME} = $snmpResponse->{"$toolConsts{OSWPNAME}.$activeOswpIndex"};
     for my $dbContainer ( 1, 2, 3 ) {
         my $index = "$dbContainer.$activeOswpIndex";
         my $dbOprStatus = SNMPComm::GetValue("DB $index OPR-STATUS", "$toolConsts{ASAMDBMDATABASEOPERSTATUS}.$index"  );
         if ( defined $dbOprStatus and $dbOprStatus == 1 ) {
            my $dbVersion = SNMPComm::GetValue("ACTIVE DB ($index) NAME", "$toolConsts{ASAMDBMDATABASEVERSION}.$index"  );
            $params->{SOURCE_DBVERSION} = $dbVersion;
            last;
         }
     }
}

=head2 GetSwpBySnmp($$$$;$)

=begin html
Retrieve the SWP info for both the active and/or passive OSWP.  This fuction is called in case the OSWP name does not meet the default syntax of
L6GPAAnn.nnn.  In that case we get the OSWP information using SNMP from the MIB.<br>
The following parameters are used:
<table>
  <tr><td>snmpSession</td><td>an opened SNMP session</td></tr>
  <tr><td>debug</td><td>if debug info is needed</td></tr>
  <tr><td>oswpAct</td><td>the active OSWP index (1 or 2) or undef if not required to be retrieved</td></tr>
  <tr><td>oswpPas</td><td>the passive OSWP index (1 or 2) or undef if not required to be retrieved</td></tr>
</table>
In case of success the undefined value is returned, otherwise a reference to a hash with 2 (hash) entries is returned: 'ACTIVE', 'PASSIVE'.
Each entry contains:
<ul>
  <li>SWP the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>

=end html

=cut

sub GetSwpBySnmp($$$$;$) {
  my ($snmpSession, $debug, $oswpAct, $oswpPass, $retries) = @_;
  my $value;
  my %swpInfo = ('ACTIVE' => { 'SWP' => '',
			       'MAJOR' => 0,
			       'MINOR' => 0,
			       'SUFFIX' => 0,
			       'MAINT' => 0 },
		 'PASSIVE' => { 'SWP' => '',
				'MAJOR' => 0,
				'MINOR' => 0,
				'SUFFIX' => 0,
				'MAINT' => 0});

  if (defined($oswpAct)) {
       $value =  SNMPComm::GetValueNext('ACTIVE SWP NAME', $toolConsts{ASAMSWMCONSIDEREDSWPNAME} . "." . $oswpAct, $retries  );
       if ( not defined($value) ) {
            $ERROR = sprintf("Unable to retrieve Active SWP name : %s\n", SNMPComm::GetError() );
            Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
            return undef;
       }
       Logger::LogDebug( $nodeLogFH, "Retrived Active SWP Name : $value\n" );
       if ($value =~ /$toolConsts{REGEXP}/) {
             $swpInfo{'ACTIVE'}{'MAJOR'} = $1;
             $swpInfo{'ACTIVE'}{'MINOR'} = $2;
             $swpInfo{'ACTIVE'}{'SUFFIX'} = $3;
             $swpInfo{'ACTIVE'}{'MAINT'} = $4;
             $swpInfo{'ACTIVE'}{'SWP'} = $value;
       } else {
             $ERROR = "Retrived Active SWP Name ($value) not acording to expected format '".$toolConsts{REGEXP}."\n";
             Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
             return undef;
       }
  }
  if (defined($oswpPass)) {
       $value = SNMPComm::GetValueNext('PASSIVE SWP NAME', $toolConsts{ASAMSWMCONSIDEREDSWPNAME} . "." . $oswpPass, $retries  );
       if ( not defined($value) ) {
            $ERROR = sprintf("Unable to retrieve Passive SWP name : %s\n", SNMPComm::GetError() );
            Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
            return undef;
       }
       Logger::LogDebug( $nodeLogFH, "Retrived Passive SWP Name : $value\n" );
       if ($value =~ /$toolConsts{REGEXP}/) {
	         $swpInfo{'PASSIVE'}{'MAJOR'} = $1;
	         $swpInfo{'PASSIVE'}{'MINOR'} = $2;
	         $swpInfo{'PASSIVE'}{'SUFFIX'} = $3;
	         $swpInfo{'PASSIVE'}{'MAINT'} = $4;
	         $swpInfo{'PASSIVE'}{'SWP'} = $value;
      } else{
             $ERROR = "Retrived Passive SWP Name ($value) not acording to expected format '".$toolConsts{REGEXP}."\n";
             Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
             return undef;
      }
    }    
  return \%swpInfo;
}
=head2 ActivateSwp($)

=begin html
Activation of the new SW using an SNMP set.  Returns 0 in case of error.
<br>The following parameters are used:
<table>
  <tr><td>oswpInfo</td><td>pointer of the OSWP information as retrieved from the NE (both active and passive OSWP)</td></tr>
</table>

=end html

=cut

sub ActivateSwp($) {
  my ($oswpInfo) = @_;


  my $logOswpDetails = "Index : ".$oswpInfo->{PASSIVE}{INDEX}.", Name : ".$oswpInfo->{PASSIVE}{NAME};
  my $errMesg = "Failed to activate oswp ($logOswpDetails)";
  my $status = 0;
  Logger::LogTrace( $nodeLogFH, "Activating oswp ($logOswpDetails)\n" );
  if ( $AcquireThroughCLI ) {
       my $cli = IsamCli::GetCliConnection();
       $status = IsamCli::ActivateSwp($cli,$oswpInfo->{PASSIVE}{INDEX});
       if ($status == 0 ) {
          Logger::LogNotify( $nodeLogFH, "Failed to Activate OSWP ($logOswpDetails)\n");
          FamilyCommon::ReportStatus( EC_OSWP_ACT_FAILED, IsamCli::GetError(), $errMesg );
          if (not $params->{CONTINUE}) {
            CORE::exit(CHG_OSWP_FAIL_CODE);
          }
          return $status;
       } else {
          Logger::LogNotify( $nodeLogFH, "Successfully Activated oswp ($logOswpDetails)\n" );
       }
       ### Disconnect existing CLL Connection
       IsamCli::Disconnect();
       FamilyCommon::ClearCliSession2Close();
  } else {
      ### Disconnect existing CLL Connection
      IsamCli::Disconnect();
      FamilyCommon::ClearCliSession2Close();
      $status = SNMPComm::SetValue('ACTIVATE OSWP', OSWPACTIVATION_OID . ".$oswpInfo->{PASSIVE}{INDEX}", 1);
      if ( $status == 0  ) {
         Logger::LogError( $nodeLogFH, "Activating oswp ($logOswpDetails) is failed : ".SNMPComm::GetError() ."\n", undef, $Logger::LogOnlyHandle );
         FamilyCommon::ReportStatus( EC_OSWP_ACT_FAILED, SNMPComm::GetError(), $errMesg );
         if (not $params->{CONTINUE}) {
           CORE::exit(CHG_OSWP_FAIL_CODE);
         }
         return $status;
      }
      Logger::LogNotify( $nodeLogFH, "Successfully Activated oswp ($logOswpDetails)\n" );
  }
  ActivationInProgress($oswpInfo);
  return $status;
}

sub ActivationInProgress($)
{
  my ($oswpInfo) = @_;

  my %isamStatus = ( 
                      0 => 'Not Reachable', 
                      1 => 'Active', 
                      2 => 'Not Active', 
                      3 =>'Activation In-Progress',
                      4 =>'Activation In-Progress',
                    );
        
  my ($inprogress_completed, $oswpPassiveStatus ) = (0, 0);
  my ( $retry_current_count, $retry_max_count ) = (0, 120);
  my ($startTime, $endTime) = (undef, undef);
  $startTime = time(); 
  my $passiveOSWPStatusOid = $toolConsts{OSWPOPERACTSTATUS} . ".". $oswpInfo->{'PASSIVE'}{'INDEX'};
  my $logOswpDetails = "Index : ".$oswpInfo->{PASSIVE}{INDEX}.", Name : ".$oswpInfo->{PASSIVE}{NAME};
  Logger::LogTrace( $nodeLogFH, "Activation In-Progress oswp ($logOswpDetails)\n" );
  ### For Duplex setup / ISSU compatible LT's isam will not go for reboot immediately
  do {
      sleep 60; 
      $oswpPassiveStatus = SNMPComm::GetValue('ACTIVATED OSWP STATUS', $passiveOSWPStatusOid, 2, 60  );
      if( defined $oswpPassiveStatus ) {
          ### Incase passive oswp status is active or not active, Retry once again
          if (  $oswpPassiveStatus == 1 || $oswpPassiveStatus == 2 ) {
              $retry_current_count = $retry_max_count;
          } else {
              $retry_max_count = 120;
          }
      } else {
          if ( $inprogress_completed == 0 ) {
             $retry_max_count = $retry_current_count + 2;
             $inprogress_completed = 1;
          }
          $oswpPassiveStatus = 0;
      }
      $retry_current_count++;
  } while($retry_current_count < $retry_max_count);
  $endTime = time(); 
  Logger::LogNotify( $nodeLogFH, sprintf("Activation In-Progress oswp (%s) Ends (after %d seconds);\n",$logOswpDetails, ($endTime - $startTime)) );
  Logger::LogNotify( $nodeLogFH, sprintf("Activated OSWP status : %s\n", defined( $isamStatus{$oswpPassiveStatus} ) ? $isamStatus{$oswpPassiveStatus} : $oswpPassiveStatus ) ); 
  FamilyCommon::SetOSWPStatus($oswpPassiveStatus);
}

=head2 ExtractSwFileName($$$$)

=begin html
Extract SW Filename from OID containing 3 keys: swpName, SW Filename and board type as octet strings.  An octet string is a length field followed by
the decimal representation of the character codes so we simply rebuild the strings.<br>
The following parameters are used:
<table>
  <tr><td>column</td><td>the SNMP column returned from the SNMP operation</td></tr>
  <tr><td>swpNameToFind</td>the SW name to look for in the returned results<td></td></tr>
  <tr><td>prefix</td><td>the expected prefix for this NT type</td></tr>
  <tr><td>debug</td><td>Need extra debugging?</td></tr>
</table>
In case of success the SW is returned.  The undefined value is returned when no match is found.

=end html

=cut

sub ExtractSwFileName($$$$) {
  my ($column, $swpNameToFind, $prefix, $debug) = @_;
  my $swFileName = "";
  my $swpName = "";
  my $loop = 0;
  my $i = 0;
  my @keyArray = Net::SNMP::oid_lex_sort(keys %{$column});

  Log($DEBUG, "\nLook for SWP '$swpNameToFind', SW prefix '$prefix'") if $debug;
  while ($i < $#keyArray) {
    my $key = $keyArray[$i];
    Log($DEBUG, "\n$i: key: '$key'") if $debug;
    if ($key =~ /$toolConsts{ASAMSWMFILEBOARDTYPE}\.(.*)/) {
      my @remainingOidArr = split(/\./, $1);
      $swpName = "";
      $loop = 0;
      # Get length of SWP name
      my $length = shift @remainingOidArr;
      while ($loop < $length) {
	$swpName .= chr(shift @remainingOidArr);
	$loop++;
      }
      Log($DEBUG, " swp: '$swpName'") if $debug;
      if (basename($swpName) eq $swpNameToFind) {
	# Reset loop and get length of SW Filename
	$swFileName = "";
	$length = shift @remainingOidArr;
	$loop = 0;
	while ($loop < $length) {
	  $swFileName .= chr(shift @remainingOidArr);
	  $loop++;
	}
	Log($DEBUG, " filename: '$swFileName'") if $debug;
	if ($swFileName =~ /$prefix/) {
	  Log($DEBUG, "\n") if $debug;
	  return ($swFileName);
	}
      }
    }
    $i++;
  }
  Log($DEBUG, "\n") if $debug;
  return undef;
}

=head2 GetNtLoadBySnmp($$$$;$)

=begin html
Get the NT SW load using SNMP.  This routine is called in case the CLI response is not containing the expected information.<br>
The following parameters are used:
<table>
  <tr><td>snmpSession</td><td>the context of an opened SNMP session</td></tr>
  <tr><td>debug</td><td>Need extra debugging?</td></tr>
  <tr><td>oswpIndex</td>the index of the required OSWP<td></td></tr>
  <tr><td>prefix</td><td>the expected prefix for this NT type</td></tr>
</table>
In case of success the undefined value is returned, otherwise a reference to a hash with 2 (hash) entries is returned: 'ACTIVE', 'PASSIVE'.
Each entry contains:
<ul>
  <li>SWP the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>

=end html

=cut

sub GetNtLoadBySnmp($$$$;$) {
  my ($snmpSession, $debug, $oswpIndex, $prefix, $retries) = @_;
  my %ntLoadInfo = ('SWP' => '',
		    'MAJOR' => 0,
		    'MINOR' => 0,
		    'SUFFIX' => 0,
		    'MAINT' => 0 );

  CheckEnv("GetNtLoadBySnmp");
  # First get SWP name
  my $swpInfo = GetSwpBySnmp($snmpSession, $debug, $oswpIndex, undef, $retries);
  if( not defined $swpInfo ) {
        $ERROR = "SWP Name not found";
        Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
	    return undef;
  }
  # Next walk asamSwmFileBoardType and find swp name and entry starting with the prefix
  # Walk returns combined index of swpName, SW file name and board Type
  $ntLoadInfo{SWP} = $swpInfo->{'ACTIVE'}{'SWP'};
  Log($DEBUG, "Get SWFile info for prefix '$prefix' via SNMP column walk...");
  my $column = SNMPComm::GetTable( 'FILE BOARD TYPE', $toolConsts{ASAMSWMFILEBOARDTYPE}  );
  if (defined($column)) {
       my $swFileName = ExtractSwFileName($column, $ntLoadInfo{'SWP'}, $prefix, $debug);
       if (defined($swFileName) and ($swFileName =~ /$toolConsts{REGEXP}/) ) {
             $ntLoadInfo{'MAJOR'} = $1;
             $ntLoadInfo{'MINOR'} = $2;
             $ntLoadInfo{'SUFFIX'} = $3;
             $ntLoadInfo{'MAINT'} = $4;
       } else {
             $ERROR = ( defined $swFileName ) ? "SW File Name $swFileName not in expected fromat" : 
                                                "NT SW FIle name not found :" .SNMPComm::GetError();
             Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
             return undef;
       }
  } else {
     $ERROR = "SNMP Table Walk for  'FILE BOARD TYPE' failed :" .SNMPComm::GetError();
     Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
     return undef;
  }
  return \%ntLoadInfo;
}

sub CheckISSU($) {
  my ($session_isam ) = @_;

  my $debug = $params->{DEBUG};

  CheckEnv("CheckISSU");
  $ERROR = "";
  ## skip if sw replacement mode is not lt by lt 
  my $status = IsamCli::IsSwReplacementModeLtByLt($session_isam);
  return 0 if( $status == 0 );
  ### Just dump the cli command execution
  my @response = IsamCli::SendCommand($session_isam, "show software-mngt upgrade-plan");
  ## Check NT boards are in Hold-Standby-Or-Not
  my $need_to_wait = 0;
  $status = IsamCli::WaitForNTSync($session_isam, $need_to_wait);
  if( $status == 0 ) {
      $ERROR = "Either Setup is not a Duplex NT or Duplex NTs are not in valid state";
      return 1;
  }
  ## port protection should be up
  $status = IsamCli::IsPortProtectionUp($session_isam);
  if( $status == 0 ) {
      $ERROR = "Either port protection details not available or configured ports state are not up";
      return 1;
  }
  return 0;
}

=head2 GetOswpInfo($$$$$$$;$)

=begin html
Retrieve the OSWP info for both the active and passive OSWP using the CLI.  In case information is missing alternative methods are tried:
<ol>
  <li>Using T&D by retrieving the FW information (only needed in cases where a migration from R2.0/R2.1 is done to R2.3 or higher.</li>
  <li>Using SNMP in case alternative OSWP naming has been used by the customer.</li>
</ol>
The following parameters are used:
<table>
  <tr><td>session</td><td>the context of an opened telnet CLI session</td></tr>
  <tr><td>snmpSession</td><td>the context of an opened SNMP session</td></tr>
  <tr><td>debug</td><td>Need extra debugging?</td></tr>
  <tr><td>dir</td><td>directory for T&amp;D usage</td></tr>
  <tr><td>isam</td><td>isam context.  ->[0] holds IP address, ->[1] holds ISAM name</td></tr>
  <tr><td>ntType</td><td>the NT type</td></tr>
  <tr>td>oswpInfoEmpty</td><td>An optional reference to an int that will indicate whether the CLI returned empty OSWP info (1) or not (0)</td></tr>
  <tr><td>retries</td><td>the number of retries on SNMP</td></tr>
</table>
I M P O R T A N T   N O T E<br>
The reduced SW PKG naming has been changed: we now no longer change the version but we alter characters 5 and 6.  This fuction however has to live
with both version so we still verify extension 998 and 999 but we also verify whether the reduction chars are stored at positions 5 and 6<br>
In case of success the undefined value is returned, otherwise a reference to a hash with 2 (hash) entries is returned: 'ACTIVE', 'PASSIVE'.
Each entry contains:
<ul>
  <li>SWP: the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>REDUCED: SWP has been reduced (1) or not (0)</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>


=end html

=cut

sub GetOswpInfo($$$$$$;$) {
  my ($session, $snmpSession, $dir, $isam, $ntType, $oswpInfoEmpty, $retries) = @_;
  my ($oswpSnmpInfo, $oswpFwInfo, $ntLoadInfoAct, $ntLoadInfoPas);
  my $getActive = undef;
  my $getPassive = undef;

  my $debug = $params->{DEBUG};


  $ERROR = "";
  CheckEnv("GetOswpInfo");
  Logger::LogTrace( $nodeLogFH, "Getting OSWP info..\n");
  my $oswpInfo = IsamCli::GetOswpInfo($session, $toolConsts{REGEXP});
  if (not defined($oswpInfo) ) {
      $ERROR = "Failed to obtain OSWP details";
      Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
      $ERROR = "$ERROR : ".IsamCli::GetError();
      return undef;
  }

  my ($oswpIndexActive, $oswpIndexPassive ) = (undef, undef);
  if( $oswpInfo->{'ACTIVE'}{'NAME'} ne $toolConsts{NO_OSWP} ) {
    $oswpIndexActive = $oswpInfo->{'ACTIVE'}{'INDEX'};
  }
  if (($oswpInfo->{'PASSIVE'}{'NAME'} ne $toolConsts{NO_OSWP} ) and ( $oswpInfo->{PASSIVE}{AVAIL} ne "disabled" ) ) {
        $oswpIndexPassive  = $oswpInfo->{'PASSIVE'}{'INDEX'};
        $oswpInfo->{'PASSIVE'}{'SWP'} = $oswpInfo->{'PASSIVE'}{'NAME'};
  }
  $oswpSnmpInfo = GetSwpBySnmp($snmpSession, $debug, $oswpIndexActive, $oswpIndexPassive, $retries);
  if (not defined($oswpSnmpInfo)) {
     Logger::LogError( $nodeLogFH, "Failed to fetch SWP details through SNMP\n", undef, $Logger::LogOnlyHandle );
     return undef;
  } 

  if ($oswpInfo->{'ACTIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
     $oswpInfo->{'ACTIVE'}{$_} = $oswpSnmpInfo->{'ACTIVE'}{$_} for qw(MAJOR MINOR SUFFIX MAINT SWP);
  }
  if ( ($oswpInfo->{'PASSIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) and ( $oswpInfo->{PASSIVE}{AVAIL} ne "disabled" )) {
     $oswpInfo->{'PASSIVE'}{$_} = $oswpSnmpInfo->{'PASSIVE'}{$_} for qw(MAJOR MINOR SUFFIX MAINT SWP);
  }
  $$oswpInfoEmpty = 0;

  Log($DEBUG, "Getting DB Version Information ...");
  my $dbVersInfo = IsamCli::GetDbVersionInfo($session);
  return undef if (not defined($dbVersInfo)) ;
  for my $oswpMode ( ('ACTIVE', 'PASSIVE') ) {
      if( defined $dbVersInfo->{$oswpInfo->{$oswpMode}{INDEX}} and $dbVersInfo->{$oswpInfo->{$oswpMode}{INDEX}} ne '' ) {
          $oswpInfo->{$oswpMode}{DBVERSION} = $dbVersInfo->{$oswpInfo->{$oswpMode}{INDEX}};
      }
  }

  if ($oswpInfo->{ACTIVE}{NAME} eq $toolConsts{NO_OSWP}) {
      # This case happens when going from R2.0/R2.1 to R2.3 or higer in one step.
      # Other case: OSWP activation did not result in NT reset.  Even after 1 hot reset there is only an empty response
      my $config = FamilyCommon::GetConfigData();
      my $dbVersionActive = $dbVersInfo->{ $oswpInfo->{'ACTIVE'}{'INDEX'} };
      my $dbVersionTarget = $params->{DBVER_TGTREL};
      Logger::LogWarn( $nodeLogFH, "OSWP SW Name missing: Determine from FirmWare...", undef, $Logger::LogOnlyHandle );
      my $actRelNum = Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'}, $oswpInfo->{'ACTIVE'}{'MINOR'}, 0, $dbVersionActive );
      my $debugViaSSH = IsamCli::GetSecureDebugSetting($session);
      if (not defined($debugViaSSH)) {
          FamilyCommon::ReportStatus( EC_SW_RED_FAILED. "Error in fetching Secure debug setting for SSH", undef );
          return 0;
      }    
      my ($debugSshSupportedHwEsti, $debugSshSupportedHwAnsi ) = (0, 0);
      if( $dbVersionActive ge "A5.200" && Capability::IsIsamHiCapEtsiFamily($ntType, $dbVersionTarget ) ) {
          $debugSshSupportedHwEsti = 1;
      }
      if( $dbVersionActive ge "A4.400" && Capability::IsIsamAnsiFamily($ntType, $dbVersionTarget ) ) {
          $debugSshSupportedHwAnsi= 1;
      }

      if ($debugViaSSH && ($debugSshSupportedHwEsti || $debugSshSupportedHwAnsi))   {
          if (!defined($connSSH)) {
            $params->{USEDEBUGVIASSH} = 1;
            #$connSSH = IsamCli::ConnectSecDebugViaSSH($isam->[0], $config->{'TELNETTIMEOUT'}, $params->{DEBUG} ? $DEBUG: undef);
            $connSSH = OpenCliWrap( $isam, $params->{DEBUG} ? $DEBUG: undef, $config->{'TELNETTIMEOUT'} );
            if( !$connSSH ) {
                  FamilyCommon::ReportStatus( EC_OPENCLI_FAILED, ToolCommon::GetError(), undef );
            	  return 0;
            }

            $params->{USEDEBUGVIASSH} = 0;
          }    
          SecureDebugTAndD::SetVars($DEBUG);
          $oswpFwInfo = SecureDebugTAndD::SshGetOswpFromFW($connSSH, $debug, $dir, $isam, "Name", $toolConsts{REGEXP});
      } else {
          $oswpFwInfo = Octopus::GetOswpFromFW($debug, $dir, $isam, "Name", $toolConsts{REGEXP});
      }    
      if (not defined($oswpFwInfo)) {
          $ERROR = "Failed to fetch OSWP Info from FirmWare";
          Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
          return undef;
      }
      if (($oswpFwInfo->{'ACTIVE'}{'MAJOR'} != 0) || ($oswpFwInfo->{'PASSIVE'}{'MAJOR'} != 0)) {
	       $$oswpInfoEmpty = 1 if defined($oswpInfoEmpty);
	       $oswpInfo->{'FRMW_OSWP'} = $oswpFwInfo->{'FRMW_OSWP'};
	       $oswpInfo->{'ACT_PENDING'} = $oswpFwInfo->{'ACT_PENDING'};
	       Logger::LogTrace( $nodeLogFH, "Activation pending for OSWP $oswpFwInfo->{'ACT_PENDING'}, firmware OSWP: $oswpFwInfo->{'FRMW_OSWP'}\n");
           $oswpInfo->{'ACTIVE'}{$_} = $oswpFwInfo->{'ACTIVE'}{$_} for qw(NAME MAJOR MINOR SUFFIX MAINT INDEX);
           $oswpInfo->{'PASSIVE'}{$_} = $oswpFwInfo->{'PASSIVE'}{$_} for qw(NAME MAJOR MINOR SUFFIX MAINT INDEX);
      } else {
	       if (Octopus::NtIsUp()) {
	         $ERROR = "Failed to obtain FW version info";
	       } else {
	         $ERROR = "Failed to obtain FW version info, NT died during the process";
	       }
           Logger::LogError( $nodeLogFH, "$ERROR\n", undef, $Logger::LogOnlyHandle );
	       return undef;
      }
  }
  for my $oswpMode ( ('ACTIVE', 'PASSIVE') ) {
     Logger::LogNotify( $nodeLogFH, sprintf("OSWP Details %-7s : %02d %-13s %-13s %-6s %d.%d.%d%02d\n",
                                    $oswpMode, $oswpInfo->{$oswpMode}{INDEX},
                                    $oswpInfo->{$oswpMode}{NAME},
                                    ( defined $oswpInfo->{$oswpMode}{SWP} ) ? $oswpInfo->{$oswpMode}{SWP} : "NO_SWP",
                                    ( defined $oswpInfo->{$oswpMode}{DBVERSION} ) ? $oswpInfo->{$oswpMode}{DBVERSION} : "NO_DB",
                                    ( defined $oswpInfo->{$oswpMode}{MAJOR} ) ? $oswpInfo->{$oswpMode}{MAJOR} : "0",
                                    ( defined $oswpInfo->{$oswpMode}{MINOR} ) ? $oswpInfo->{$oswpMode}{MINOR} : "0",
                                    ( defined $oswpInfo->{$oswpMode}{SUFFIX} ) ? $oswpInfo->{$oswpMode}{SUFFIX} : "0",
                                    ( defined $oswpInfo->{$oswpMode}{MAINT} ) ? $oswpInfo->{$oswpMode}{MAINT} : "00"
                                   ) );
  }
  return $oswpInfo;
}

=head2 GetOswpInfoTL1($$$$$$$;$)

=begin html
Retrieve the OSWP info for both the active and passive OSWP using TL1.  In case information is missing alternative methods are tried:
<ol>
  <li>Using T&D by retrieving the FW information (only needed in cases where a migration from R2.0/R2.1 is done to R2.3 or higher.</li>
  <li>Using SNMP in case alternative OSWP naming has been used by the customer.</li>
</ol>
The following parameters are used:
<table>
  <tr><td>session</td><td>the context of an opened telnet CLI session</td></tr>
  <tr><td>snmpSession</td><td>the context of an opened SNMP session</td></tr>
  <tr><td>debug</td><td>Need extra debugging?</td></tr>
  <tr><td>dir</td><td>directory for T&amp;D usage</td></tr>
  <tr><td>isam</td><td>isam context.  ->[0] holds IP address, ->[1] holds ISAM name</td></tr>
  <tr><td>ntType</td><td>the NT type</td></tr>
  <tr>td>oswpInfoEmpty</td><td>An optional reference to an int that will indicate whether the CLI returned empty OSWP info (1) or not (0)</td></tr>
  <tr><td>retries</td><td>the number of retries on SNMP</td></tr>
</table>
I M P O R T A N T   N O T E<br>
The reduced SW PKG naming has been changed: we now no longer change the version but we alter characters 5 and 6.  This fuction however has to live
with both version so we still verify extension 998 and 999 but we also verify whether the reduction chars are stored at positions 5 and 6<br>
In case of success the undefined value is returned, otherwise a reference to a hash with 2 (hash) entries is returned: 'ACTIVE', 'PASSIVE'.
Each entry contains:
<ul>
  <li>SWP: the OSWP name of (NO_NAME in case no OSWP has been loaded)</li>
  <li>REDUCED: SWP has been reduced (1) or not (0)</li>
  <li>MAJOR: the major number of the release (the 2 of L6GPAA23.456)</li>
  <li>MINOR: the minor number of the release (the 3 of L6GPAA23.456)</li>
  <li>SUFFIX: the suffix number of the release (the 4 of L6GPAA23.456)</li>
  <li>MAINT: the maintenace number of the release (the 45 of L6GPAA23.456)</li>
</ul>


=end html

=cut

sub GetOswpInfoTL1($$$$$$$;$) {
  my ($session, $snmpSession, $debug, $dir, $isam, $ntType, $oswpInfoEmpty, $retries) = @_;
  my ($actOswpRelNumExt, $pasOswpRelNumExt, $actFwRelNumExt, $pasFwRelNumExt);
  my ($oswpSnmpInfo, $oswpFwInfo, $ntLoadInfoAct, $ntLoadInfoPas);
  my $getActive = undef;
  my $getPassive = undef;
  my $actPrefix = "";
  my $passPrefix = "";

  CheckEnv("GetOswpInfoTL1");
  Log($DEBUG, "Getting Oswp Info...");
  $ERROR = "";
  my $oswpInfo = IsamTL1::GetOswpInfo($session, $toolConsts{REGEXP});
  if (defined($oswpInfo)) {
    $$oswpInfoEmpty = 0;
    Log($DEBUG, "  OK\n");
  } else {
    Log($DEBUG, "  FAILED\n");
    $ERROR = IsamCli::GetError();
    return undef;
  }
  if ($oswpInfo->{'ACTIVE'}{'NAME'} =~ /$toolConsts{REGEXP}/) {
    $actOswpRelNumExt = $3*100 + $4;
    $actFwRelNumExt = $oswpInfo->{'ACTIVE'}{'SUFFIX'} * 100 + $oswpInfo->{'ACTIVE'}{'MAINT'};
  } else {
    $actOswpRelNumExt = 0;
    $actFwRelNumExt = 0;
    if ($oswpInfo->{'ACTIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
      $getActive = 1;
    }
  }
  if ($oswpInfo->{'PASSIVE'}{'NAME'} =~ /$toolConsts{REGEXP}/) {
    $pasOswpRelNumExt = $3 * 100 + $4;
    $pasFwRelNumExt = $oswpInfo->{'PASSIVE'}{'SUFFIX'} * 100 + $oswpInfo->{'PASSIVE'}{'MAINT'};
  } else {
    $pasOswpRelNumExt = 0;
    $pasFwRelNumExt = 0;
    if ($oswpInfo->{'PASSIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
      $getPassive = 1;
    }
  }
  my ($oswpIndexActive, $oswpIndexPassive ) = (undef, undef);
  if( $oswpInfo->{'ACTIVE'}{'NAME'} ne $toolConsts{NO_OSWP} ) {
    $oswpIndexActive = $oswpInfo->{'ACTIVE'}{'INDEX'};
  }
  if( ( $oswpInfo->{'PASSIVE'}{'NAME'} ne $toolConsts{NO_OSWP} ) and ( $oswpInfo->{PASSIVE}{AVAIL} ne "disabled" ) ) {
    $oswpIndexPassive  = $oswpInfo->{'PASSIVE'}{'INDEX'};
    $oswpInfo->{'PASSIVE'}{'SWP'} = $oswpInfo->{'PASSIVE'}{'NAME'};
  }
  $oswpSnmpInfo = GetSwpBySnmp($snmpSession, $debug, $oswpIndexActive, $oswpIndexPassive, $retries);
  if (! defined($oswpSnmpInfo)) {
    Log($DEBUG, "  FAILED, check logs\n");
#    $ERROR = $snmpSession->error();
    return undef;
  } else {
    if ($oswpInfo->{'ACTIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
      $oswpInfo->{'ACTIVE'}{'MAJOR'} = $oswpSnmpInfo->{'ACTIVE'}{'MAJOR'} if
	$oswpSnmpInfo->{'ACTIVE'}{'MAJOR'} != $oswpInfo->{'ACTIVE'}{'MAJOR'};
      $oswpInfo->{'ACTIVE'}{'MINOR'} = $oswpSnmpInfo->{'ACTIVE'}{'MINOR'} if
	$oswpSnmpInfo->{'ACTIVE'}{'MINOR'} != $oswpInfo->{'ACTIVE'}{'MINOR'};
      $oswpInfo->{'ACTIVE'}{'SUFFIX'} = $oswpSnmpInfo->{'ACTIVE'}{'SUFFIX'} if
	$oswpSnmpInfo->{'ACTIVE'}{'SUFFIX'} != $oswpInfo->{'ACTIVE'}{'SUFFIX'};
      $oswpInfo->{'ACTIVE'}{'MAINT'} = $oswpSnmpInfo->{'ACTIVE'}{'MAINT'} if
	$oswpSnmpInfo->{'ACTIVE'}{'MAINT'} != $oswpInfo->{'ACTIVE'}{'MAINT'};
      $oswpInfo->{'ACTIVE'}{'SWP'} =  $oswpSnmpInfo->{'ACTIVE'}{'SWP'};
    }
    if ($oswpInfo->{'PASSIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
      $oswpInfo->{'PASSIVE'}{'MAJOR'} = $oswpSnmpInfo->{'PASSIVE'}{'MAJOR'} if
	$oswpSnmpInfo->{'PASSIVE'}{'MAJOR'} != $oswpInfo->{'PASSIVE'}{'MAJOR'};
      $oswpInfo->{'PASSIVE'}{'MINOR'} = $oswpSnmpInfo->{'PASSIVE'}{'MINOR'} if
	$oswpSnmpInfo->{'PASSIVE'}{'MINOR'} != $oswpInfo->{'PASSIVE'}{'MINOR'};
      $oswpInfo->{'PASSIVE'}{'SUFFIX'} = $oswpSnmpInfo->{'PASSIVE'}{'SUFFIX'} if
	$oswpSnmpInfo->{'PASSIVE'}{'SUFFIX'} != $oswpInfo->{'PASSIVE'}{'SUFFIX'};
      $oswpInfo->{'PASSIVE'}{'MAINT'} = $oswpSnmpInfo->{'PASSIVE'}{'MAINT'} if
	$oswpSnmpInfo->{'PASSIVE'}{'MAINT'} != $oswpInfo->{'PASSIVE'}{'MAINT'};
      $oswpInfo->{'PASSIVE'}{'SWP'} =  $oswpSnmpInfo->{'PASSIVE'}{'SWP'};
    }
  }
  $actPrefix = Capability::NtLoadPrefix(uc($ntType), Capability::ConvertRelease($oswpInfo->{'ACTIVE'}{'MAJOR'},
										$oswpInfo->{'ACTIVE'}{'MINOR'}, undef));
  $passPrefix = Capability::NtLoadPrefix(uc($ntType), Capability::ConvertRelease($oswpInfo->{'PASSIVE'}{'MAJOR'},
										 $oswpInfo->{'PASSIVE'}{'MINOR'}, undef));
  $oswpInfo->{'ACTIVE'}{'REDUCED'} = basename($oswpInfo->{'ACTIVE'}{'NAME'}) ne $toolConsts{NO_OSWP} ?
    Capability::OswpReduced(basename($oswpInfo->{'ACTIVE'}{'SWP'}), $ntType) : 0;
  $oswpInfo->{'PASSIVE'}{'REDUCED'} = basename($oswpInfo->{'PASSIVE'}{'NAME'}) ne $toolConsts{NO_OSWP} ?
    Capability::OswpReduced(basename($oswpInfo->{'PASSIVE'}{'SWP'}), $ntType) : 0;

  if ((($actOswpRelNumExt >= $toolConsts{REDUCEDSWP}) && ($actOswpRelNumExt == $actFwRelNumExt)) ||
      (($pasOswpRelNumExt >= $toolConsts{REDUCEDSWP}) && ($pasOswpRelNumExt == $pasFwRelNumExt))) {
    # OSWP has been reduced by SWP reduction script: last three digits of SWP name are either 998 or 999
    Log($DEBUG, "Reduced SWP: determine real release from NT load...");
    Log($DEBUG, "act '$actOswpRelNumExt' <> '$actFwRelNumExt' || pas '$pasOswpRelNumExt' <> '$pasFwRelNumExt'\n") if $debug;
    $oswpInfo->{'ACTIVE'}{'REDUCED'} = 1 if ($actOswpRelNumExt >= $toolConsts{REDUCEDSWP});
    $oswpInfo->{'PASSIVE'}{'REDUCED'} = 1 if ($pasOswpRelNumExt >= $toolConsts{REDUCEDSWP});
    if ($oswpInfo->{'ACTIVE'}{'REDUCED'}) {
      $ntLoadInfoAct = GetNtLoadBySnmp($snmpSession, $debug, $oswpInfo->{'ACTIVE'}{'INDEX'}, $actPrefix, $retries);
      if (!defined($ntLoadInfoAct)) {
	Log($DEBUG, "  FAILED for active, check logs\n");
	return undef;
      }
    }
    if ($oswpInfo->{'PASSIVE'}{'REDUCED'}) {
      $ntLoadInfoPas = GetNtLoadBySnmp($snmpSession, $debug, $oswpInfo->{'PASSIVE'}{'INDEX'}, $passPrefix, $retries);
      if (!defined($ntLoadInfoPas)) {
	Log($DEBUG, "  FAILED for passive, check logs\n");
	return undef;
      }
    }
    if (($oswpInfo->{'ACTIVE'}{'REDUCED'} && ($ntLoadInfoAct->{'MAJOR'} != 0)) ||
	($oswpInfo->{'PASSIVE'}{'REDUCED'} && ($ntLoadInfoPas->{'MAJOR'} != 0))) {
      Log($DEBUG, "  OK\n");
      if ($oswpInfo->{'ACTIVE'}{'REDUCED'}) {
	$oswpInfo->{'ACTIVE'}{'MAJOR'} = $ntLoadInfoAct->{'MAJOR'};
	$oswpInfo->{'ACTIVE'}{'MINOR'} = $ntLoadInfoAct->{'MINOR'};
	$oswpInfo->{'ACTIVE'}{'SUFFIX'} = $ntLoadInfoAct->{'SUFFIX'};
	$oswpInfo->{'ACTIVE'}{'MAINT'} = $ntLoadInfoAct->{'MAINT'};
      }
      if ($oswpInfo->{'PASSIVE'}{'REDUCED'}) {
	$oswpInfo->{'PASSIVE'}{'MAJOR'} = $ntLoadInfoPas->{'MAJOR'};
	$oswpInfo->{'PASSIVE'}{'MINOR'} = $ntLoadInfoPas->{'MINOR'};
	$oswpInfo->{'PASSIVE'}{'SUFFIX'} = $ntLoadInfoPas->{'SUFFIX'};
	$oswpInfo->{'PASSIVE'}{'MAINT'} = $ntLoadInfoPas->{'MAINT'};
      }
    } else {
      Log($DEBUG, "  FAILED to obtain version, check logs\n");
      $ERROR = "FAILED to obtain NT version info by SNMP";
      return undef;
    }
  }
  if ($oswpInfo->{ACTIVE}{NAME} eq $toolConsts{NO_OSWP}) {
    # Other case: OSWP activation did not result in NT reset.  Even after 1 hot reset there is only an empty response
    Log($DEBUG, "OSWP SW Name missing: determine real release...");
    $oswpFwInfo = Octopus::GetOswpFromFW($debug, $dir, $isam, "Name", $toolConsts{REGEXP});
    if (!defined($oswpFwInfo)) {
      Log($DEBUG, "  FAILED, check logs\n");
      $ERROR = "Failed to get OSWP info from firmware";
      return undef;
    } else {
      if (($oswpFwInfo->{'ACTIVE'}{'MAJOR'} != 0) && ($oswpFwInfo->{'PASSIVE'}{'MAJOR'} != 0)) {
	Log($DEBUG, "  OK\n");
	$$oswpInfoEmpty = 1 if defined($oswpInfoEmpty);
	$oswpInfo->{'FRMW_OSWP'} = $oswpFwInfo->{'FRMW_OSWP'};
	$oswpInfo->{'ACT_PENDING'} = $oswpFwInfo->{'ACT_PENDING'};
	$oswpInfo->{'ACTIVE'}{'NAME'} = $oswpFwInfo->{'ACTIVE'}{'NAME'};
	$oswpInfo->{'ACTIVE'}{'MAJOR'} = $oswpFwInfo->{'ACTIVE'}{'MAJOR'};
	$oswpInfo->{'ACTIVE'}{'MINOR'} = $oswpFwInfo->{'ACTIVE'}{'MINOR'};
	$oswpInfo->{'ACTIVE'}{'SUFFIX'} = $oswpFwInfo->{'ACTIVE'}{'SUFFIX'};
	$oswpInfo->{'ACTIVE'}{'MAINT'} = $oswpFwInfo->{'ACTIVE'}{'MAINT'};
	$oswpInfo->{'ACTIVE'}{'INDEX'} = $oswpFwInfo->{'ACTIVE'}{'INDEX'} if ($oswpInfo->{'ACTIVE'}{'INDEX'} == 0);
	$oswpInfo->{'PASSIVE'}{'NAME'} = $oswpFwInfo->{'PASSIVE'}{'NAME'};
	$oswpInfo->{'PASSIVE'}{'MAJOR'} = $oswpFwInfo->{'PASSIVE'}{'MAJOR'};
	$oswpInfo->{'PASSIVE'}{'MINOR'} = $oswpFwInfo->{'PASSIVE'}{'MINOR'};
	$oswpInfo->{'PASSIVE'}{'SUFFIX'} = $oswpFwInfo->{'PASSIVE'}{'SUFFIX'};
	$oswpInfo->{'PASSIVE'}{'MAINT'} = $oswpFwInfo->{'PASSIVE'}{'MAINT'};
	$oswpInfo->{'PASSIVE'}{'INDEX'} = $oswpFwInfo->{'PASSIVE'}{'INDEX'} if ($oswpInfo->{'PASSIVE'}{'INDEX'} == 0);
      } else {
	if (Octopus::NtIsUp()) {
	  Log($DEBUG, "  FAILED to obtain version, check logs\n");
	  $ERROR = "Failed to obtain FW version info";
	  return undef;
	} else {
	  Log($DEBUG, "  FAILED to obtain version, NT died during the process\n");
	  $ERROR = "Failed to obtain FW version info";
	  return undef;
	}
      }
    }
  }
  if (defined($oswpInfo->{PASSIVE}{MAJOR})) {
    Log($DEBUG, sprintf("  Active OSWP : %s - %d.%d.%d%02d (%d)\n                       Passive OSWP: %s - %d.%d.%d%02d (%d)\n",
			$oswpInfo->{ACTIVE}{NAME}, $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR},
			 $oswpInfo->{ACTIVE}{SUFFIX}, $oswpInfo->{ACTIVE}{MAINT}, $oswpInfo->{ACTIVE}{INDEX},
			 $oswpInfo->{PASSIVE}{NAME}, $oswpInfo->{PASSIVE}{MAJOR}, $oswpInfo->{PASSIVE}{MINOR},
			 $oswpInfo->{PASSIVE}{SUFFIX}, $oswpInfo->{PASSIVE}{MAINT}, $oswpInfo->{PASSIVE}{INDEX}));
  } else {
    Log($DEBUG, sprintf("  Active OSWP : %s - %d.%d.%d%02d (%d)\n                       Passive OSWP: %s - 0.0.00 (%d)\n",
			 $oswpInfo->{ACTIVE}{NAME}, $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR},
			 $oswpInfo->{ACTIVE}{SUFFIX}, $oswpInfo->{ACTIVE}{MAINT}, $oswpInfo->{ACTIVE}{INDEX},
			 $oswpInfo->{PASSIVE}{NAME}, $oswpInfo->{PASSIVE}{INDEX}));
  }
  return $oswpInfo;
}

=head2 CheckEnv($)

=begin html
Environment check:  A file handle for output of information is required.  Every function in this packacge should call this.<br>
The following parameters are used:
<table>
  <tr><td>caller</td><td>string holding the name of the calling function</td></tr>
</table>
The tool exists in case no init has been done.

=end html

=cut

sub CheckEnv($) {
  my ($caller) = @_;

  if (!defined($DEBUG)) {
    print STDERR "ToolCommon::$caller - environment not set!!!\n";
    CORE::exit(COMMON_ENV_NOT_SET_CODE);
  }
}

sub GetIPAddress($) {
    my ($asam) = @_;
    my $config = FamilyCommon::GetConfigData();
    my $ipAddress = undef;
    if( ! Ping( $config->{PING}, $asam ) ) {
        Logger::LogError( $nodeLogFH, "Host '$asam' is not reachable\n");
        return $ipAddress;
    }
    my $port = (defined $params->{USESSH} and $params->{USESSH} == 1 ) ? 22 : 23;
    my $sock = IO::Socket::INET->new(
                           PeerAddr=> "$asam",
                           PeerPort=> $port,
                           Proto   => "tcp") or die ("Unable to connect $asam");
    
    if( defined $sock) {
       $ipAddress = $sock->sockhost;
       Logger::LogNotify( $nodeLogFH, "PBMT IP-Address is '$ipAddress'\n" );
    }
    return $ipAddress;
}

=head2 Ping($$)

=begin html
Ping the isam using UDP ping.  If that fails try ICMP ping via UNIX command line.<br>
The following parameters are used:
<table>
  <tr><td>pingCmd</td><td>ping command to be executed</td></tr>
  <tr><td>asam</td><td>isam context.  ->[0] holds IP address, ->[1] holds ISAM name</td></tr>
</table>
Returns 1 in case of success, 0 in case of failure.

=end html

=cut

sub Ping($$) {
  my ($pingCmd, $asam) = @_;
  my ( $cmd, $cmd_status ) = ( "", 0 );
  return 1 if ( defined $params->{NOPING} and $params->{NOPING} );
  if ($Config{archname} =~ /linux/) {
     $cmd = "$pingCmd -c 2 $asam 2>&1 "
  } else {
     $cmd = "$pingCmd $asam";
  }
  Logger::LogDebug( $nodeLogFH, "Ping ($cmd)...\n" );
  my @cmd_op = `$cmd`;
  $cmd_status = $?;
  foreach ( @cmd_op ) {
     Logger::LogDebug( $nodeLogFH, "$_" );
  }
  my $logMesg = sprintf( "Ping to Node(%s)... %s\n", $asam, ( $cmd_status ) ? "Failed" : "Succeed" );
  Logger::LogNotify( $nodeLogFH, $logMesg );
  if( $cmd_status ) {
     return 0
  }
  return 1;
}

=head2 Log($$;$$)

=begin html
Print a line to the logfile and optionally to standard error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>file handle to be used for logging</td></tr>
  <tr><td>txt</td><td>string to be written</td></tr>
  <tr><td>stdout</td><td>when defined will also print to STDOUT</td></tr>
  <tr><td>noTime</td><td>when defined: do not prefix the string with a time indication</td></tr>
</table>

=end html

=cut

sub Log($$;$$) {
  my ($logFH, $txt, $stdout, $noTime) = @_;
  my $calledFrom = (caller(1))[3];
  my $lineNo = (caller(0))[2];
  if (defined($calledFrom)) {
    if ($calledFrom =~ /(eval)/) {
      $calledFrom = (caller(2))[3];
    } elsif (($calledFrom =~ /::(SetAndLogError|ReleaseCompare|SelectFamily)$/)) {
      $calledFrom = (caller(2))[3];
      $lineNo = (caller(1))[2];
    }
  }
  my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime(time()))[0,1,2,3,4,5];
  my $str = defined($noTime) ? $txt :
    sprintf("%04d-%02d-%02d %02d:%02d:%02d [%s:%d] %s", $year+1900, $month+1, $dayOfMonth, $hour, $minute, $second,
	    defined($calledFrom) ? $calledFrom : "main::main", $lineNo, $txt);

  #Remove all ascii control characters from the response except "new line" char (0x0A)
  $str =~ s/[\x00-\x09\x0B-\x1F]//g;
  #print $fh $str;
  #print STDOUT $str if (defined($stdout));
  LogDump($logFH, $str);
  LogDisplay($str) if(defined($stdout));

}

=head2 ReadHostsFile($$$;$)

=begin html
Read and parse the hostsfile into ISAM IP, ISAM name couples.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>file handle to be used for logging</td></tr>
  <tr><td>debug</td><td>extra debugging required?</td></tr>
  <tr><td>hosts</td><td>full name of the hosts file</td></tr>
  <tr><td>prefix</td><td>node names on AWS are prefixed, providing this prefix here will make sure that only AWS-defined nodes are read</td></tr>
</table>
A reference to the array of arrays is returned in case of success, the undefined value is returned in case of error.

=end html

=cut

sub ReadHostsFile($$$;$) {
  my ($fh, $debug, $hosts, $prefix) = @_;
  my @isams = ();
  my $nodePrefix = defined($prefix) ? $prefix : "";

  if (!open(HOSTS, $hosts)) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    return undef;
  }
  while (<HOSTS>) {
    next if (/^\s*\#/);
    chomp();
    if (/\w+/) {
      my ($ip, $hostname) = split(/\s+/, $_);
      if ($nodePrefix ne "") {
	if ($hostname =~ /$nodePrefix/) {   # -- 1.0.3
	  push(@isams, [$ip, $hostname]);
	  Log($fh, "  Added $hostname\n", 1) if ($debug);
	}
      } else {
	push(@isams, [$ip, $hostname]);
	Log($fh, "  Added $hostname\n", 1) if ($debug);
      }
    }
  }
  close(HOSTS);

  if (!scalar(@isams)) {
	Log($fh,"ERROR: Hosts file '$hosts' does not contain the Hostnames in proper format\n",1); 
  	exit HOSTS_FILE_OPEN_FAILED_CODE; 
  }

  return \@isams;
}

sub ReadReleaseFile($) {

  my ($isamRelFile) = @_;
  my @isamRel = ();
  my $formatOK = 1; 

  if (!open(ISAMREL, $isamRelFile)) {
    return undef;
  }
  while (<ISAMREL>) {
    next if (/^\s*\#/);
    chomp();
    if (/\s*(\d+\.\d+)/) {
      my $rel = $1;  
       push(@isamRel, $rel); 
    } else {
     $formatOK = 0; 
    }
  }
  close(ISAMREL);

  if (!scalar(@isamRel) || (!$formatOK)) {
	print "ERROR: ISAM SWP file '$isamRelFile' does not contain the ISAM Releases in proper format\n"; 
  	exit SWP_FILE_OPEN_FAILED_CODE; 
  } 
   ## else {
     ## print join(", ", @isamRel);
   ##}

  return \@isamRel;
} 

=head2 TransactionReleased(;$$$$)

=begin html
Wait for the transaction owner to be released.<br>
The following parameters are used:
<table>
  <tr><td>debug</td><td>extra debugging required?</td></tr>
  <tr><td>session</td><td>an opened SNMP session</td></tr>
  <tr><td>retries</td><td>number of retries on SNMP gets</td></tr>
  <tr><td>waitForEver</td><td>wait forever when defined</td></tr>
</table>
The IP address contained in the transaction owner is returned. "0.0.0.0" indicates a released transaction.<br>
The undefined value indicates an SNMP error.

=end html

=cut

sub TransactionReleased(;$$$$) {
  my ($debug, $session, $retries, $waitForEver) = @_;
  my $tries = 0;
  my $ipAddr = undef;

  while( $tries <= 5 ) {
     $ipAddr = TransactionOwner();
     if( (defined $ipAddr) and ($ipAddr eq "0.0.0.0") ) {
         Logger::LogDebug($nodeLogFH, "(Already) Released configuration-lock ownership\n");
         return 0;
     }
     $tries++;
  }
  $ERROR = (defined $ipAddr) ? "Already Acquired by other owner '$ipAddr'" : SNMPComm::GetError();
  my $reportStatus = FamilyCommon::ReportStatus( EC_CONFIG_LOCK_ALREADY_TAKEN, $ERROR, undef );
  return 1 if( $reportStatus );
  return 0;
}


=begin html
Get current transaction owner
The following parameters are used:
<table>
  <tr><td>debug</td><td>extra debugging required?</td></tr>
  <tr><td>session</td><td>an opened SNMP session</td></tr>
  <tr><td>retries</td><td>number of retries on SNMP gets</td></tr>
  <tr><td>waitForEver</td><td>wait forever when defined</td></tr>
</table>
The IP address contained in the transaction owner is returned.
The undefined value indicates an SNMP error.

=end html

=cut

sub TransactionOwner(;$$$$) {
  #my ($debug, $session, $retries, $waitForEver) = @_;
  return  SNMPComm::GetValue('TRANSACTION OWNER', $toolConsts{TRANSACTOWNER}, 2  );
}


=head2 ReleaseTransaction($$;$$)

=begin html
Release all pending transactions by setting 255.255.255.255 as transaction ownew IP address.<br>
The following parameters are used:
<table>
  <tr><td>debug</td><td>extra debugging required?</td></tr>
  <tr><td>session</td><td>an opened SNMP session</td></tr>
  <tr><td>retries</td><td>number of SNMP get retries</td></tr>
  <tr><td>delay</td><td>delay required before SNMP get is done</td></tr>
</table>
The IP address contained in the transaction owner is returned. "0.0.0.0" indicates a released transaction.
The undefined value indicates an SNMP error.

=end html

=cut

sub ReleaseTransaction(;$$$$) {
  my ($debug, $session, $retries, $timeDelay) = @_;
  my $transactionIp =  defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP};
  my $possibleIpAddress = "$transactionIp|$toolConsts{'INTERNALIP'}";
  my $ipAddr = TransactionOwner();
  if( (defined $ipAddr) and ($ipAddr eq "0.0.0.0") ) {
      Logger::LogDebug($nodeLogFH, "(Already) Released configuration-lock ownership'\n");
      return 0;
  } elsif( (defined $ipAddr) and ( $ipAddr =~ /^($possibleIpAddress)$/ ) ) {
      Logger::LogDebug($nodeLogFH, "Acquired configuration-lock ownership by same owner '$ipAddr'\n");
  } else {
      $ERROR = (defined $ipAddr) ? "Already Acquired by other owner '$ipAddr'" : SNMPComm::GetError();
      $reportStatus = FamilyCommon::ReportStatus( EC_CONFIG_LOCK_ALREADY_TAKEN, $ERROR, undef );
      return 1 if( $reportStatus );
  }
  my $cli = IsamCli::GetCliConnection();
  if( defined $cli and IsamCli::Transaction( $cli, 'release')  ) {
      $ipAddr = TransactionOwner();
      if( (defined $ipAddr) and ($ipAddr eq "0.0.0.0") ) {
          Logger::LogDebug($nodeLogFH, "Released configuration-lock ownership through CLI\n");
          return 0;
      }
  }

  $ipAddr = "1.1.1.1";
  my $snmpRetries = 0;
  while (defined($ipAddr) && ($ipAddr ne "0.0.0.0") && ($snmpRetries < 5)) {
    if( ! SNMPComm::SetValue( 'RELEASE OWNER', $toolConsts{TRANSACTOWNER}, "255.255.255.255" ) ) {
        $ERROR = sprintf("Failed to set transaction owner: %s", SNMPComm::GetError());
        sleep 2;
        $snmpRetries++;
        next;
    }
    sleep($timeDelay)   if(defined($timeDelay)); 
    $ipAddr = TransactionOwner();
    $snmpRetries++;
  }
  if( (defined $ipAddr) and ($ipAddr eq "0.0.0.0") ) {
      Logger::LogDebug($nodeLogFH,"Released Transaction config-lock through SNMP\n");
      return 0;
  } else {
      $ERROR = (defined $ipAddr) ? "Acquired by owner '$ipAddr'" : SNMPComm::GetError() ;
      $reportStatus = FamilyCommon::ReportStatus( EC_CONFIG_LOCK_RELEASE_FAILED, $ERROR, undef );
      return 1 if( $reportStatus );
  }
}

=head2 AcquireTransaction(;$$$$$)

=begin html
Acquire transaction by setting the transaction owner IP address.<br>
The following parameters are used:
<table>
  <tr><td>debug</td><td>extra debugging required?</td></tr>
  <tr><td>session</td><td>an opened SNMP session</td></tr>
  <tr><td>transIp</td><td>transaction owner IP addr</td></tr>
  <tr><td>retries</td><td>number of SNMP get retries</td></tr>
  <tr><td>delay</td><td>delay required before SNMP get is done</td></tr>
</table>
The IP address contained in the transaction owner is returned. '$transIp' indicates an acquired transaction.
The undefined value indicates an SNMP error.

=end html

=cut


sub AcquireTransaction (;$$$$$) {
  my ($debug, $session, $transactionIp, $retries, $timeDelay) = @_;
  $AcquireThroughCLI = 0;
  ### Check config-lock transactio owner
  $transactionIp =  defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP};
  my $possibleIpAddress = "$transactionIp|$toolConsts{'INTERNALIP'}";
  my $ipAddr = TransactionOwner();
  if( (defined $ipAddr) and ( $ipAddr =~ /^($possibleIpAddress)$/ ) ) {
      Logger::LogDebug($nodeLogFH, "(Already) Acquired configuration-lock ownership by same owner '$ipAddr'\n");
  } elsif( (defined $ipAddr) and ($ipAddr eq "0.0.0.0") ) {
      Logger::LogDebug($nodeLogFH, "Config-Lock onwership is free to acquire\n");
  } else {
      $ERROR = (defined $ipAddr) ? "Already Acquired by other owner '$ipAddr'" : SNMPComm::GetError() ;
      $reportStatus = FamilyCommon::ReportStatus( EC_CONFIG_LOCK_ALREADY_TAKEN, $ERROR, undef );
      return 1 if( $reportStatus );
  }
  
  my $cli = IsamCli::GetCliConnection();
  if( defined $cli and  IsamCli::Transaction( $cli, 'acquire') ) {
      $ipAddr = TransactionOwner();
      if( (defined $ipAddr) and ( $ipAddr =~ /^($possibleIpAddress)$/ ) ) {
          Logger::LogDebug($nodeLogFH, "Acquired configuration-lock ownership through CLI\n");
          $AcquireThroughCLI = 1;
          return 0;
      }
  }
  $ipAddr = "1.1.1.1";
  my $snmpRetries = 0;
  while (defined($ipAddr) && ($ipAddr ne $transactionIp) && ($snmpRetries < 5)) {
      if( ! SNMPComm::SetValue( 'ACQUIRE OWNER', $toolConsts{TRANSACTOWNER}, $transactionIp ) ) {
          sleep 2;
          $snmpRetries++;
          next;
      }
      sleep($timeDelay)   if(defined($timeDelay)); 
      $ipAddr = TransactionOwner();
      $snmpRetries++;
  }
  if( (defined $ipAddr) and ( $ipAddr =~ /^($possibleIpAddress)$/ ) ) {
      Logger::LogDebug($nodeLogFH, "Acquired configuration-lock ownership through SNMP by same owner '$ipAddr'\n");
      return 0;
  } else {
      $ERROR = (defined $ipAddr) ? "Acquired by other owner '$ipAddr'" : SNMPComm::GetError() ;
      $reportStatus = FamilyCommon::ReportStatus( EC_CONFIG_LOCK_ACQUIRE_FAILED, $ERROR, undef );
      return 1 if( $reportStatus );
  }
}



=head2 DetermineNtType()

=begin html
This function is called to determine the NT type using SNMP requests (CLI show equipment flat is not supported by GPON).  The undefined value is
returned in case of failure otherwise the retrieved NT type is returned (as a string)<br>
The following parameters are used:
<table>
</table>

=end html

=cut

sub DetermineNtType() {
  my @oids;
  my $result;
  my %ntOidDetails = (
                      "$toolConsts{EQPTSLOTACTUALTYPE}.$toolConsts{NTA_EQPT_IDX}" => 'EQPT-ACT-NTA',
                      "$toolConsts{EQPTSLOTACTUALTYPE}.$toolConsts{NTB_EQPT_IDX}" => 'EQPT-ACT-NTB',
                      "$toolConsts{EQPTINVTYPENAME}.$toolConsts{NTA_EQPT_IDX}"    => 'EQPT-INV-NTA',
                      "$toolConsts{EQPTINVTYPENAME}.$toolConsts{NTB_EQPT_IDX}"    => 'EQPT-INV-NTB'
                     );
                  
  my ($snmpResponse, $ntTypeValue, $ntTypeFound ) = (undef, undef, 0);
  foreach my $snmpOid (sort keys %ntOidDetails) {
      $ntTypeValue = SNMPComm::GetValue( "NT-TYPE $ntOidDetails{$snmpOid}", $snmpOid );
      if( defined $ntTypeValue and $ntTypeValue !~ /^(EMPTY|)$/ ) {
          $ntTypeFound = 1;
          last;
      }
  }
  if( not $ntTypeFound ) {
      $ERROR = "NT(s) tyoe not found/holds EMPTY : ". SNMPComm::GetError();
	  FamilyCommon::ReportStatus( EC_GET_NT_TYPE_FAILED, $ERROR );
      $ntTypeValue = undef;
  }
  return $ntTypeValue;
}

=head2 IsamHasVoice($$)

=begin html
This function returns 1 in case this ISAM has POLT or BALT boards for for which CDE files are needed, 0 if no voice boards are found.  The undefined
variable is returned in case an error did occur.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>cli</td><td>opened CLI session</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub IsamHasVoice($) {
  my ($cli) = @_;
  my $ltConfig = IsamCli::GetSlotConfig($cli);
  my $isIsamV = undef;

  $ERROR = "";
  if (defined($ltConfig)) {
    $isIsamV = 0;
    foreach my $slot (keys %{$ltConfig}) {
      Log($DEBUG, "  '$slot' -> '$ltConfig->{$slot}'\n") if $params->{DEBUG};
      if (($ltConfig->{$slot} =~ /(po|ba)lt\-/) || ($ltConfig->{$slot} =~ /nbat\-/) || ($ltConfig->{$slot} =~ /pot\-/)
              || ($ltConfig->{$slot} =~ /nvps\-/)) {
        $isIsamV = 1;
      }
    }
  } else {
    $ERROR = sprintf("Retrieving LT configuration failed: %s", IsamCli::GetError());
  }
  return $isIsamV;
}

=head2 IsamHasVoiceServer($$)

=begin html
This function returns 1 in case this ISAM has IVPS boards for which database migration is needed, 0 if no voice boards are found.  The undefined
variable is returned in case an error did occur.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>cli</td><td>opened CLI session</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
</table>

=end html

=cut

sub IsamHasVoiceServer($) {
  my ($cli) = @_;
  my $ltConfig = IsamCli::GetSlotConfig($cli);
  my $isIsamV = undef;

  $ERROR = "";
  if (defined($ltConfig)) {
    $isIsamV = 0;
    foreach my $slot (keys %{$ltConfig}) {
      Log($DEBUG, "  '$slot' -> '$ltConfig->{$slot}'\n") if $params->{DEBUG};
      if ($ltConfig->{$slot} =~ /[inv]vps\-/) {
        $isIsamV = 1;
      }
    }
  } else {
    $ERROR = sprintf("Retrieving LT configuration failed: %s", IsamCli::GetError());
  }
  return $isIsamV;
}



=head2 AlarmRetrieval($$$$)

=begin html
This function returns 1 in case if any of the emergency call alarms in the list $emergencyCallList is active 
and 0 if the alarm is not active and the entry is not present.
The following parameters are used:
<table>
  <tr><td>fh</td><td>File handle of the child process logfile</td></tr>
  <tr><td>cli</td><td>opened CLI session</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>toolData</td><td>a pointer to the tool configuration</td></tr>
</table>

=end html

=cut

sub AlarmRetrieval($$$) {
	my ($fh, $cli, $toolData) = @_;
	my $isActive = 0;
	my $tries = $toolData->{'RETRIES'};
	my $ok = 0;

	  while ($tries) {
		 $isActive = IsamCli::AlarmStatus($cli, 'emergency-cal');	
		 if ($isActive) {            #emergency call alarm is active
			 ToolCommon::Log($fh, "Emergency-call alarm is active. Waiting for $toolData->{'EMERG_ALRM_POLL_TIME'} seconds...\n") if $params->{DEBUG};
			 sleep($toolData->{'EMERG_ALRM_POLL_TIME'});
			 $tries--;
			 $ok = 1;
		 } else {	
			 ToolCommon::Log($fh, "Emergency-call alarm is non-active...\n") if $params->{DEBUG};
			 $tries = 0;
			 $ok = 0;
		 }
	  }
	return $ok;
}

=head2 BuildShubConfigFile($$$$$$$$$$$;$)

=begin html
Build the SHUB parameter config file.  The contents is determined using the SHUB OFLMT version.  Returns 0 in case of error.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>dir</td><td>NE output directory</td></tr>
  <tr><td>from</td><td>the NE release coming from</td></tr>
  <tr><td>to</td><td>the NE release going to</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>path</td><td>directory holding the SHUB OFLMT configuration file</td></tr>
  <tr><td>file</td><td>Name of the SHUB database file</td></tr>
  <tr><td>ntType</td><td>NT type</td></tr>
  <tr><td>paramFileName</td><td>Name of the SHUB OFLMT parameter file to be created</td></tr>
  <tr><td>cfgFile</td><td>Name of the OFLMT config file</td></tr>
  <tr><td>toDbName</td><td>Filename of the output database</td></tr>
</table>

=end html

=cut

sub BuildShubConfigFile($$$$$$$$$$;$) {
  my ($fh, $dir, $from, $to, $path, $file, $ntType, $paramFileName, $cfgFile, $toDbName, $u2uHairPin) = @_;
  my $ok = 1;
  my $shubNtType = Capability::ShubBoardType($ntType);
  my $ifBase = Capability::InterfaceBase($ntType);

  $ERROR = "";
  ToolCommon::Log($fh, "Build SHUB migration parameter file...") if $params->{DEBUG};
  if (open(CFG, ">$dir/$paramFileName")) {
    print CFG "PARAMS START\n";
    print CFG "Inputconffile=$file\n";
    print CFG "Datfile=$path/$cfgFile\n" if ($params->{SHUBOFLMTVERS} < 128);
    print CFG "Outputconffile=$toDbName\n";
    print CFG "Versionfrom=$from\n";
    print CFG "Versionto=$to\n";
    print CFG "Modefrom=$shubNtType\n";
    if ($params->{TOECNTC}) {
      print CFG sprintf("Modeto=%s\n", Capability::ShubBoardType("ECNT-C"));
    } elsif ($params->{TONANTB}) {
      print CFG sprintf("Modeto=%s\n", Capability::ShubBoardType("NANT-B"));
    } elsif ($params->{TOECNTE}) {
      print CFG sprintf("Modeto=%s\n", Capability::ShubBoardType("ECNT-E"));
    } else {
      print CFG "Modeto=$shubNtType\n";
    }
    print CFG "Trace=trace\n";
    print CFG "Bandwidth=$params->{BWREDUCE}\n";
    print CFG "Miglogfile=lanx_migration.log\n" if ($params->{SHUBOFLMTVERS} < 128);
    if ($params->{SHUBOFLMTVERS} >= 127) {
      if ($params->{TOECNTC} || $params->{TOECNTE}) {
	print CFG sprintf("Macmode=%s\n", defined($params->{MACMODE}) ? $params->{MACMODE} : "Legacy");
      } else {
	if ($ntType eq Capability::ShubBoardType("ECNT-A")) {
	  print CFG "Macmode=Legacy\n";
	} else {
	  print CFG "Macmode=Native\n";
	}
      }
    }
    print CFG "InterfaceBase=$ifBase\n" if ($params->{SHUBOFLMTVERS} >= 136);
    if(($from eq "R3.6") && ($to ge "R4.3")) {  
      if(defined($u2uHairPin)){
          print CFG sprintf("Hairpinstatus=%s\n", $u2uHairPin ? "TRUE" : "FALSE");   
      }
    }
    print CFG "PARAMS END\n";
    close(CFG);
  } else {
    $ERROR = "Creation of $dir/$paramFileName failed: $!";
    $ok = 0;
  }
  return $ok;
}

=head2 NtIsUp()

=begin html
This function returns true in case the NT is up or 0 in case the NT is dead.

=end html

=cut

sub NtIsUp() {
  return Octopus::NtIsUp();
}

=head2 ValidIP($)

=begin html
This function returns true in case the IP address provided to it is a valid IP address.  0 is returned in case of error.

=end html

=cut

sub ValidIP($) {
  my ($ipAddress) = @_;
  my $ok = 0;
  my ($ip1, $ip2, $ip3, $ip4, $mask);

  if ($ipAddress =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)\/(\d+)/) {
    $ip1 = $1;
    $ip2 = $2;
    $ip3 = $3;
    $ip4 = $4;
    $mask = $5;
  } elsif ($ipAddress =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)/) {
    $ip1 = $1;
    $ip2 = $2;
    $ip3 = $3;
    $ip4 = $4;
    $mask = undef;
  }
  if (($ip1 > 0) && ($ip1 <= 255) &&
      ($ip2 >= 0)&& ($ip2 <= 255) &&
      ($ip3 >= 0)&& ($ip3 <= 255) &&
      ($ip4 >= 0)&& ($ip4 <= 255)) {
    $ok = 1;
  } else {
    $ERROR = "IP $ip1.$ip2.$ip3.$ip4 address not valid";
  }
  if (defined($mask)) {
    if (($mask < 0) || ($mask > 32)) {
      $ERROR = "netmask $mask not valid";
      $ok = 0;
    }
  }
  return $ok;
}

=head2 IsTransferOnGoing()

=begin html
This function returns true in case a download to the ISAM is ongoing (can be either a SW DL, DB up- or download.  The second parameter indicates whether debug logging has been enabled (true) or not.

=end html

=cut
sub IsTransferOnGoing() {
  my %oidDetails = (
                     "$toolConsts{ASAMDBMDNLOADPROGRESS}" => 'DB-DOWNLOAD-PROGRESS',
                     "$toolConsts{ASAMDBMUPLOADPROGRESS}" => 'DB-UPLOAD-PROGRESS',
                     "$toolConsts{OSWPOPERAVAILSTATUS}.1" => 'OSWP1-OPR-AVAIL-STATUS',
                     "$toolConsts{OSWPOPERAVAILSTATUS}.2" => 'OSWP2-OPR-AVAIL-STATUS',
                     );
                  
  my $returnValue = 0;
  my ($startTime, $endTime, $totalWaitTime, $snmpResponse, $onGoing ) = (undef, undef, 600, undef, 0 );
  $startTime = $endTime = time();
  do {
      $onGoing = 0;
      $snmpResponse = SNMPComm::GetValueList( 'DNLD-UPLD ONGOING', \%oidDetails );
      if( defined $snmpResponse ) {
          my ($snmpOid, $statusValue, $snmpOidLabel ) = ("", "", "" );
          foreach $snmpOid (sort keys %oidDetails) {
              if( defined $snmpResponse->{$snmpOid} ) {
                  $statusValue = $snmpResponse->{$snmpOid};
                  next if( $statusValue =~ /^\s+$/ );
                  $snmpOidLabel = (defined $oidDetails{$snmpOid}) ? $oidDetails{$snmpOid} : "";
                  if( $snmpOidLabel =~ /^DB/ ) {
                     $onGoing = 1 if( $statusValue == $toolConsts{UPLOADPROGRESS}{ONGOING} );
                  } else {
                     $onGoing = 1 if( $statusValue >= $toolConsts{OSWPPROGRESS}{DOWNLOADING} );
                  }
              }
          }
          return 0 if( not $onGoing );
      } else {
          FamilyCommon::ReportStatus( EC_TRANSFER_SNMPGET_FAILED, SNMPComm::GetError() );
          return 1;
      }
      $endTime = time();
  } while( ($endTime-$startTime) <= $totalWaitTime );
  FamilyCommon::ReportStatus( EC_TRANSFER_WAITTIME_EXCEED, "Wait time out($totalWaitTime) exceeds" );
  return 1;
}


=head2 CheckUnicastMacLimits($$$$)

=begin html
Determine the sum of all maxNumOfMacAddr per line on each LT.  The sum may not exceed 1024.  Check the maximum number of static unicast MAC
addresses.  The total number may not exceed 1024.<br>
The following parameters are used:
<table>
  <tr><td>snmpSession</td><td>A pointer to an opened SNMP session</td></tr>
  <tr><td>profMode</td><td>The CLI profile mode used for this CLI user</td></tr>
  <tr><td>ntType</td><td>the type of the NT</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
</table>

=end html

=cut

sub CheckUnicastMacLimits($$$$) {
  my ($snmpSession, $profMode, $ntType, $debug) = @_;
  my $maxMacList;
  my $ifIndexList;
  my $ltList;
  my $plannedLtList;
  my $staticUnicastFwd;
  my $ifIndex;
  my $ret = 1;
  my $basePort;
  my $oid;
  my %ltList = ();
  my $maxMac;
  my ($rackNum, $shelfNum, $slotNum, $cliSlotNum);
  my ($holderId, $holderIndex, $plannedIndex);
  my $version = $snmpSession->version();

  my $snmpErr = "";
  $ERROR = "";

  $maxMacList        = SNMPComm::GetTable('MAC-ADDR', $toolConsts{EXTENDVLANPORTMAXNUMMACADDR}, 2, 5 );
  if (!defined $maxMacList) {
      $snmpErr = SNMPComm::GetError();
      if (!($snmpErr =~ /Requested table is empty/)) {
        $ERROR = "Failed to retrieve maximum number of MAC addresses per line: $snmpErr";
        return 0;
      }
  }
  $ifIndexList       = SNMPComm::GetTable('IFINDEX', $toolConsts{DOT1DBASEPORTIFINDEX}, 2, 5 );
  if (!defined $ifIndexList) {
      $snmpErr = SNMPComm::GetError();
      if (!($snmpErr =~ /Requested table is empty/)) {
        $ERROR = "Failed to retrieve maximum number of IFIndexes : $snmpErr";
        return 0;
      }
  }
  $ltList            = SNMPComm::GetTable('INSERTED BOARD', $toolConsts{INSERTEDBOARD_5X_OID}, 2, 5 );
  if (!defined $ltList) {
      $snmpErr = SNMPComm::GetError();
      if (!($snmpErr =~ /Requested table is empty/)) {
        $ERROR = "Failed to retrieve maximum number of Inserted Boards : $snmpErr";
        return 0;
      }
  }
  $plannedLtList     = SNMPComm::GetTable('PLANNED BOARD', $toolConsts{PLANNEDBOARD_5X_OID}, 2, 5 );
  if (!defined $plannedLtList ){
      $snmpErr = SNMPComm::GetError();
      if (!($snmpErr =~ /Requested table is empty/)) {
        $ERROR = "Failed to retrieve maximum number of Planned Boards : $snmpErr";
        return 0;
      }
  }
  foreach my $key (Net::SNMP::oid_lex_sort(keys %{$maxMacList})) {
	  $maxMac = $maxMacList->{$key};
	  $basePort = $key;
      Log($DEBUG, ":$maxMac:, :$basePort:\n");
	  $basePort =~ s/$toolConsts{EXTENDVLANPORTMAXNUMMACADDR}\.//;
	  $oid = "$toolConsts{DOT1DBASEPORTIFINDEX}.$basePort";
	  $ifIndex = $ifIndexList->{$oid};
      Log($DEBUG, ":$maxMac:, :$basePort:, :$oid:, :$ifIndex:\n");
	  if (($ifIndex & $toolConsts{DYNAMICIFINDEXBIT}) != 0) {
	    $rackNum = (($ifIndex & $toolConsts{DYNAMICRACKMASK}) >> $toolConsts{DYNAMICRACKSHIFT}) + $toolConsts{DYNAMICRACKOFFSET};
	    $shelfNum = (($ifIndex & $toolConsts{DYNAMICSHELFMASK}) >> $toolConsts{DYNAMICSHELFSHIFT}) + $toolConsts{DYNAMICSHELFOFFSET};
	    $slotNum = (($ifIndex & $toolConsts{DYNAMICLTMASK}) >> $toolConsts{DYNAMICLTSHIFT}) + $toolConsts{DYNAMICLTOFFSET};
	    # Dynamic ifIndex
	  } else {
	    # Static ifIndex
	    $rackNum = (($ifIndex & $toolConsts{STATICRACKMASK}) >> $toolConsts{STATICRACKSHIFT}) + $toolConsts{STATICRACKOFFSET};
	    $shelfNum = (($ifIndex & $toolConsts{STATICSHELFMASK}) >> $toolConsts{STATICSHELFSHIFT}) + $toolConsts{STATICSHELFOFFSET};
	    $slotNum = (($ifIndex & $toolConsts{STATICLTMASK}) >> $toolConsts{STATICLTSHIFT}) + $toolConsts{STATICLTOFFSET};
	  }
	  $holderId = ($rackNum << $toolConsts{HOLDERRACKSHIFT}) + ($shelfNum << $toolConsts{HOLDERSHELFSHIFT}) + $slotNum-1;
	  $holderIndex = "$toolConsts{INSERTEDBOARD_5X_OID}.$holderId";
	  $plannedIndex = "$toolConsts{PLANNEDBOARD_5X_OID}.$holderId";
	  $cliSlotNum = Capability::ConvertLtInternalNum($profMode, $ntType, $slotNum);
      if (defined $cliSlotNum ) {
	     Log($DEBUG, sprintf("[max MAC] basePort: '%d', oid: '%s', ifIndex: '0x%08x' (lt: '%d/%d/%d'), LT int num: %d, maxMac: '%d'\n",
	  		$basePort, $oid, $ifIndex, $rackNum, $shelfNum, $cliSlotNum, $slotNum, $maxMac)) if $debug;
      } else {
        Log($DEBUG, sprintf("Could not calculate internal LT number for profile mode %s, NT type %s and LT %d\n",
                         $profMode, $ntType, $slotNum)) if $debug;
        next;
      }
	  Log($DEBUG, sprintf("Holder ID: 0x%x, holder index: %s, type: %s (planned: %s)\n",
	  		$holderId, $holderIndex, (defined $ltList->{$holderIndex}) ? $ltList->{$holderIndex} : "NA",
              (defined $plannedLtList->{$plannedIndex}) ? $plannedLtList->{$plannedIndex} : "NA" )) if $debug;
      next if( !defined $plannedLtList->{$plannedIndex} or !defined $ltList->{$holderIndex} );
	  if (defined($ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMAC})) {
	    $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMAC} += $maxMac;
	  } else {
	    $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMACPERLSM} =
	  ( defined $ltList->{$holderIndex} and uc($ltList->{$holderIndex}) ne "EMPTY") ? Capability::MaxMacPerLsm($ltList->{$holderIndex}) :
	    (defined $plannedLtList->{$plannedIndex}) ? Capability::MaxMacPerLsm($plannedLtList->{$plannedIndex}): 0;
	    $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{TYPE} =
	  uc($ltList->{$holderIndex}) ne "EMPTY" ? uc($ltList->{$holderIndex}) :
	    sprintf("planned %s", uc($plannedLtList->{$plannedIndex}));
	    $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMAC} = $maxMac;
	  }
	  Log($DEBUG, sprintf("New value for maxmac for LT %d/%d/%d: %d\n",
	  		$rackNum, $shelfNum, $cliSlotNum, $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMAC})) if $debug;
	  if ($ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMAC} > $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXMACPERLSM}) {
	    $ret = 0;
	  }
  }
  if (! $ret) {
    $ERROR = "  FAILED!! MAC address limits exceeded:\n";
    foreach $slotNum (sort keys %ltList) {
      if ($ltList{$slotNum}{MAXMAC} > $ltList{$slotNum}{MAXMACPERLSM}) {
    $ERROR .= sprintf("  Sum of max MAC addresses per line (%d) exceeds limit of %d for LT %s (%s)\n",
    		  $ltList{$slotNum}{MAXMAC}, $ltList{$slotNum}{MAXMACPERLSM}, $slotNum, $ltList{$slotNum}{TYPE});
      }
    }
  }
  $staticUnicastFwd  = SNMPComm::GetTable('UNICAST FWD', $toolConsts{EXTENDSTATICUNICASTALWTOGOTO}, 2, 5 );
  if (!defined $staticUnicastFwd) {
      $snmpErr = SNMPComm::GetError();
      if (!($snmpErr =~ /Requested table is empty/)) {
        $ERROR = "Failed to retrieve maximum number of Unicast FWD : $snmpErr";
        return 0;
      }
  }
  foreach my $key (Net::SNMP::oid_lex_sort(keys %{$staticUnicastFwd})) {
	    $basePort = $staticUnicastFwd->{$key};
	    $oid = "$toolConsts{DOT1DBASEPORTIFINDEX}.$basePort";
	    $ifIndex = $ifIndexList->{$oid};
	    if (($ifIndex & $toolConsts{DYNAMICIFINDEXBIT}) != 0) {
	        $rackNum = (($ifIndex & $toolConsts{DYNAMICRACKMASK}) >> $toolConsts{DYNAMICRACKSHIFT}) + $toolConsts{DYNAMICRACKOFFSET};
	        $shelfNum = (($ifIndex & $toolConsts{DYNAMICSHELFMASK}) >> $toolConsts{DYNAMICSHELFSHIFT}) + $toolConsts{DYNAMICSHELFOFFSET};
	        $slotNum = (($ifIndex & $toolConsts{DYNAMICLTMASK}) >> $toolConsts{DYNAMICLTSHIFT}) + $toolConsts{DYNAMICLTOFFSET};
	        # Dynamic ifIndex
	    } else {
	        # Static ifIndex
	        $rackNum = (($ifIndex & $toolConsts{STATICRACKMASK}) >> $toolConsts{STATICRACKSHIFT}) + $toolConsts{STATICRACKOFFSET};
	        $shelfNum = (($ifIndex & $toolConsts{STATICSHELFMASK}) >> $toolConsts{STATICSHELFSHIFT}) + $toolConsts{STATICSHELFOFFSET};
	        $slotNum = (($ifIndex & $toolConsts{STATICLTMASK}) >> $toolConsts{STATICLTSHIFT}) + $toolConsts{STATICLTOFFSET};
	    }
	    $cliSlotNum = Capability::ConvertLtInternalNum($profMode, $ntType, $slotNum);
        if (defined $cliSlotNum ) {
	       Log($DEBUG, sprintf("[max MAC] basePort: '%d', oid: '%s', ifIndex: '0x%08x' (lt: '%d/%d/%d'), LT int num: %d, maxMac: '%d'\n",
	    		$basePort, $oid, $ifIndex, $rackNum, $shelfNum, $cliSlotNum, $slotNum, $maxMac)) if $debug;
        } else {
          Log($DEBUG, sprintf("Could not calculate internal LT number for profile mode %s, NT type %s and LT %d\n",
                           $profMode, $ntType, $slotNum)) if $debug;
          next;
        }
	    if (defined($ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXUNICAST})) {
	        $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXUNICAST}++;
	    } else {
	        $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXUNICAST} = 1;
	    }
	    Log($DEBUG, sprintf("New value for max static unicast MAC for LT %d/%d/%d: %d\n",
	  		  $rackNum, $shelfNum, $cliSlotNum, $ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXUNICAST})) if $debug;
	    if ($ltList{"$rackNum/$shelfNum/$cliSlotNum"}{MAXUNICAST} > $toolConsts{MAXNUMMACADDRPERLT}) {
	        $ret = 0;
	    }
  }
  if (! $ret) {
	  foreach $slotNum (sort keys %ltList) {
	       if (defined($ltList{$slotNum}{MAXUNICAST})) {
	         if ($ltList{$slotNum}{MAXUNICAST} > $ltList{$slotNum}{MAXMACPERLSM}) {
	           $ERROR .= sprintf("  Max static unicast MAC addresses (%d) exceeds limit of %d for LT %s (%s)\n",
	       		      $ltList{$slotNum}{MAXUNICAST}, $ltList{$slotNum}{MAXMACPERLSM}, $slotNum, $ltList{$slotNum}{TYPE});
	         }
	       }
	  }
  }
  return $ret;
}

=head2 CountPackages($$)

=begin html
Count the number of packages in the bitmask array assoicated to this multicast source.  Returns the number of packages used.
The following parameters are used:
<table>
  <tr><td>bitmask</td><td>a character string representing the bitmask of used packages</td></tr>
  <tr><td>debug</td><td>Extra debugging enabled (1) or not (0)</td></tr>
</table>

=end html

=cut

sub CountPackages($$) {
  my ($bitMask, $debug) = @_;
  my @bitMaskArr = split(//, $bitMask);
  my $packages = 0;
  my $highestPkg = 0;
  my $dbgStr = "";

  foreach my $bitValue (@bitMaskArr) {
    $bitValue = ord($bitValue);
    $dbgStr .= sprintf("0x%02x ", $bitValue) if $debug;;
    if ($bitValue == 0xff) {
      $packages += 8;
      $highestPkg += 8;
    } elsif ($bitValue != 0) {
      my $loop = 0;
      while ($loop < 8) {
	if ($bitValue & (1 << $loop)) {
	  $packages++;
          $highestPkg += ($loop+1);
	}
	$loop++;
      }
    }
  }
  if ($debug) {
    $dbgStr = substr($dbgStr, 0, -1);
    Log($DEBUG, "Values read: '$dbgStr'\n");
  }
  return ($packages, $highestPkg);
}

=head2 CheckMcPackages($$$)

=begin html
Check the number of packages used in a multicast source.  An error is returned in case more then the allowed amount is used.
The following parameters are used:
<table>
  <tr><td>snmpSession</td><td>a pointer to an opened SNMP session</td></tr>
  <tr><td>debug</td><td>Extra debugging enabled (1) or not (0)</td></tr>
  <tr><td>actRelNum</td>The release currently active in numeric representation</td></tr>
</table>

=end html

=cut

sub CheckMcPackages($$$) {
  my ($snmpSession, $debug, $actRelNum) = @_;
  my $oid = Capability::GetMcastSrcOID($actRelNum);
  my $pkgBitMasks;
  my $mcSrcAddr = "";
  my ($pkgCount, $highestPkgNum);
  my $errStr = "";
  my $t = "";
  my $ret = 1;

  if ($snmpSession->version() == 0) {
    # SNMPv1
    $pkgBitMasks = $snmpSession->get_table(-baseoid => $oid);
  } elsif ($snmpSession->version() == 1) {
    # SNMPV2C
    $pkgBitMasks = $snmpSession->get_table(-baseoid => $oid,
					   -maxrepetitions => $toolConsts{MAX_REP});
  } else {
    $pkgBitMasks = $snmpSession->get_table(-baseoid => $oid,
					   -contextname => NT_CONTEXT,
					   -maxrepetitions => $toolConsts{MAX_REP});
  }
  if (defined($pkgBitMasks)) {
    foreach my $key (Net::SNMP::oid_lex_sort(keys %{$pkgBitMasks})) {
      $mcSrcAddr = $key;
      $mcSrcAddr =~ s/$oid\.//;
      ($pkgCount, $highestPkgNum) = CountPackages($pkgBitMasks->{$key}, $debug);
      Log($DEBUG, sprintf("  %d packages for multicast source %s - OK\n", $pkgCount, $mcSrcAddr)) if $debug;
      $errStr = Capability::McastPackageCountOK($pkgCount, $highestPkgNum, $mcSrcAddr);
      $t .= $errStr if ($errStr ne "");
    }
    if ($t ne "") {
      $ERROR = substr($t, 0, -1);
      $ret = 0;
    }
  } else {
    my $snmpErr = $snmpSession->error();
    if (!($snmpErr =~ /Requested table is empty/)) {
      $ERROR = sprintf("Failed to retrieve Multi-cast source package bitmask: %s", $snmpErr);
      $ret = 0;
    }
  }
  return $ret;
}

=head2 PrepareCliEnv($$)

=begin html
Prepare CLI environment:<br>
<ul>
  <li>Adjust prompt to 'pbmt%d%c'</li>
  <li>Enable batch mode</li>
  <li>Disable more prompting</li>
</ul>
Return 0 in case of failure.

=end html

=cut

sub PrepareCliEnv($$) {
  my ($session, $debug) = @_;
  my @response;
  my $ret = 0;

  @response = IsamCli::SendCommand($session, "environment print no-more");
  if (IsamCli::CommandError(join("", @response))) {
    $ERROR = sprintf("Disabling operator interaction failed: %s", IsamCli::GetError());
    Log($DEBUG, "  FAILED... $ERROR\n");
  } else {
    @response = IsamCli::SendCommand($session, "environment mode batch");
    # Don't check response as this may fail: only supported from R2.5 onwards
    @response = IsamCli::SendCommand($session, "environment prompt \"%n%d%c _PBMT_#\"");
    $session->prompt("/_PBMT_#/");
    if (IsamCli::CommandError(join("", @response))) {
      $ERROR = sprintf("Setting environment prompt failed: %s", IsamCli::GetError());
      Log($DEBUG, "  FAILED... $ERROR\n");
    } else {
      if (IsamCli::InhibitMore($session)) {
	IsamCli::InhibitAlarms($session);
	my $cliSessions = IsamCli::ListCliSessions($session);
	if (defined($cliSessions)) {
	  foreach my $sessId (sort keys %{$cliSessions}) {
	    Log($DEBUG, sprintf("Id: %-4d (%s) From: %s Type: %s\n", $sessId,
				$cliSessions->{$sessId}{OPER},
				$cliSessions->{$sessId}{FROM},
				$cliSessions->{$sessId}{TYPE})) if ($debug);
	  }
	  $ret = 1;
#	} else {
#	  $ERROR = sprintf("Failed to retrieve session info: %s", IsamCli::GetError());
#	  Log($DEBUG, "  FAILED... $ERROR\n");
# Don't error on this,  It's a pitty if we don't get the result but let's continue anyhow.
	}
      }
    }
  }
}

=head2 GetFxrSettings($$$)

=begin html
Retrieve the filetransfer settings of the node:<br>
<ul>
  <li>the protocol used: ftp, sftp or tftp.  In case the file transfer protocol OID does not exist it will be defaulted to TFTP.</li>
  <li>a list of servers each whith their login stored as %hash{IP address} = &lt;login user&gt;</li>
</ul>
Return undef in case of failure.

=end html

=cut

sub GetFxrSettings($$) {
  my ($snmpSession, $retries) = @_;
  my %frxSettings = ('PROTOCOL' => -1,
		     'SERVERS' => {});
  my $snmpError;
  my $fxrProtocol =  SNMPComm::GetValue('FILE TRANSFER PROTOCOL', FXRPROTOCOL_OID  );
  if ( not defined($fxrProtocol) ) {
      $frxSettings{PROTOCOL} = TFTP_FXR;
  } else {
    $frxSettings{PROTOCOL} = $fxrProtocol;
    my $xferProtocolStr = sprintf("%s", $fxrProtocol);
    if ($xferProtocolStr !~  /^\d+$/) {
      $frxSettings{PROTOCOL} = TFTP_FXR;
    }
    # Node is supporting more then tftp alone so continue and fetch server/user name couples
    if ($frxSettings{PROTOCOL} != TFTP_FXR) {
      my $serverOid = FXRSERVERUSERNAME_OID ."123";
       my $table = SNMPComm::GetTable( 'FILE SERVER', $serverOid );
       if (!defined($table)) {
	         if ($frxSettings{PROTOCOL} == FTP_FXR) {
	               $ERROR = sprintf("failed to fetch filetransfer server table: %s", SNMPComm::GetError() );
	               return undef;
	         } else {
	               # File transfer protocol is sftp, get sftp username
                   my $sftpClientUsername =  SNMPComm::GetValue('SFTP USER NAME', SFTPCLIENTUSERNAMEOID, $retries  );
	               if (not defined($sftpClientUsername)) {
	                     $ERROR = sprintf("Failed to retrieve SFTP username: %s", SNMPComm::GetError() );
	                     return undef;
	               } elsif( defined ($params->{PRIMTFTPIP}) ) {
	                     $frxSettings{SERVERS}{ $params->{PRIMTFTPIP} } = $sftpClientUsername;
	               }
	          }
        } else {
	          my $serverIp;
	          foreach my $fullOid (Net::SNMP::oid_lex_sort(keys(%{$table}))) {
	            $serverIp = $fullOid;
	            $serverIp =~ s/$serverOid\.//g;
	            $frxSettings{SERVERS}{$serverIp} = $table->{$fullOid};
	            Log($DEBUG, "Storing user '$table->{$fullOid}' for server '$serverIp'\n") if $params->{DEBUG};
	          }
        }
    }
  }
  return \%frxSettings
}

=head2 FxrServerIsThisNode($$$)

=begin html
This function verifies whether this UNIX box is acting as a file server.  The IP address of the file server is returned in case it acts as a file server.  Otherwise the empty string is returned.<br>
The following parameters are used:
<table>
  <tr><td>fxrData</td><td>The file transfer settings as retrieved from the ISAM</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td>config</td><td>a pointer to the tool configuration</td></tr>
</table>

=end html

=cut

sub FxrServerIsThisNode($$) {
  my ($fxrData, $config) = @_;
  my $cmd = "$config->{IFCONFIG} -a|";
  my $look;
  my $line;
  my $goOn = 1;
  my $ip = "";

  Log($DEBUG, "CMD = $cmd\n") if $params->{DEBUG};
  if (open($look, $cmd)) {
    while ((defined($line = <$look>)) && $goOn) {
      Log($DEBUG, "read '$line'\n") if $params->{DEBUG};
      if ($line =~ /^\s*inet\s+(\d+\.\d+\.\d+\.\d+)/) {
	$ip = $1;
	Log($DEBUG, "Found IP: $ip\n") if $params->{DEBUG};
	my $servers = join(" ", keys %{$fxrData->{SERVERS}});
	$goOn = 0 if (index($servers, $ip) != -1);
      }
    }
  } else {
    my ($exitCode, $signal) = Utils::ConvertExitCode($?);
    $ERROR = "'$cmd' FAILED (signal $signal, exitcode $exitCode): $!\n";
    return undef;
  }
  return $goOn ? "" : $ip;
}
=head2 GetSwParSize($$$)

=begin html
Return the free space in the SW partition of the NT in bytes using the opened SNMP session $session.
In case of failure the undefined value is returned.

=end html

=cut

sub GetSwParSize(;$$$) {
  my ($session, $debug, $retries) = @_;
  my $swParSize = SNMPComm::GetValue('SW PARTITION SIZE', $toolConsts{FREESPACEOID} );
  return $swParSize ;
}

=head2 AccessOprIdConfigure($$$)

=begin html
Configures access operatoId assigned by ALU. Returns 0 in case of error (Unable to configure access operatorId).
This functionality is supported starting R3.6<br>
NOTE: Only ALU assigned access operatorId must be used.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the child process.  Debug traces and progress messages are written to this file</td></tr>
  <tr><td>debug</td><td>a boolean indicating whether extra logging is enabled</td></tr>
  <tr><td>cli</td><td>an opened CLI session</td></tr>
</table>

=end html

=cut

#------------------------------------------------------------------------------
#-- AccessOprIdConfigure
#------------------------------------------------------------------------------
sub AccessOprIdConfigure($$$$) {
  my ($fh, $debug, $cli, $operId) = @_;
  my $ok = 0;

  if (defined($operId)) {
    if (($operId >= 0) && ($operId <= 65535)) {
      my $ret = IsamCli::ConfigureAccessOprId($cli, $operId);
      if (!$ret) {
        $ERROR = sprintf("  Failed: %s", IsamCli::GetError());
      } else {
	$ok = 1;
      }
    } else {
     $ERROR = "ACCESS_OPERATOR_ID has wrong value ($operId): can only be 0..65535";
    }
  } else {
    $ERROR = "ACCESS_OPERATOR_ID not defined";
  }
  return $ok;
}

=head2 GetAccessSettings($$)

=begin html
Return the ssh access settings for both CLI and TL1 in a hash structure:
<ul>
  <li>0: no access possible (not support for CLI)</li>
  <li>1: acess via SSH</li>
  <li>2: access via Telnet/UDP</li>
  <li>3; access via both SSH and Telnet/UDP</li>
</ul>
undef is returned in case of failure.

=end html

=cut

sub GetAccessettings($$) {
  my ($snmpSession, $debug) = @_;
  my @oids;
  my $secSettings;
  my %accSecBits = ('CLI' => -1,
		    'TL1' => -1,
		    'PUBKEY' => "");

  $oids[0] = $toolConsts{ACCESS_SEC_OID};
  $oids[1] = $toolConsts{PUBLIC_KEY_OID};
  if ($snmpSession->version() < 2) {
    # SNMPv1/SNMPv2
    $secSettings = $snmpSession->get_request(-varbindlist => \@oids);
  } else {
    # SNMPv3
    $secSettings = $snmpSession->get_request(-varbindlist => \@oids,
					     -contextname => NT_CONTEXT);
  }
  if (defined($secSettings) && ($snmpSession->error() eq "")) {
    $accSecBits{CLI} = ($secSettings->{$toolConsts{ACCESS_SEC_OID}} & 0x06) >> 1;
    $accSecBits{TL1} = ($secSettings->{$toolConsts{ACCESS_SEC_OID}} & 0x18) >> 3;
    $accSecBits{PUBKEY} = $secSettings->{$toolConsts{PUBLIC_KEY_OID}};
    Log($DEBUG, "Access security settings - CLI: '$accSecBits{CLI}', TL1: '$accSecBits{TL1}'\n") if $debug;
    Log($DEBUG, "Access security settings - public key: '$accSecBits{PUBKEY}'\n") if $debug;
  } else {
    $ERROR = sprintf("failed to retrieve access security settings: %s", $snmpSession->error());
    return undef;
  }
  return \%accSecBits;
}

=head2 TL1AccessEnabled($$)

=begin html
Check whether TL-1 access is possible.  $accessBits is a reference to a hash as returned by GetAccessSettings.  TRUE is returned in case TL1 acccess
is possible, FALSE in case no TL-1 access is allowed.

=end html

=cut

sub TL1AccessEnabled($$) {
  my ($accessBits, $debug) = @_;

  Log($DEBUG, "TL1 access bits: $accessBits->{TL1}\n") if $debug;
  return($accessBits->{TL1} != 0 ? 1 : 0);
}

=head2 TL1ViaSSH($$)

=begin html
Check whether TL-1/SSH access is possible.  $accessBits is a reference to a hash as returned by GetAccessSettings.  TRUE is returned in case TL1/SSH
acccess is possible, FALSE in case no TL-1/SSH access is allowed.

=end html

=cut

sub TL1ViaSSH($$) {
  my ($accessBits, $debug) = @_;

  Log($DEBUG, "TL1 access bits: $accessBits->{TL1}\n") if $debug;
  return(($accessBits->{TL1} &0x01) != 0 ? 1 : 0);
}

=head2 TL1ViaUDP($$)

=begin html
Check whether TL-1/UDP access is possible.  $accessBits is a reference to a hash as returned by GetAccessSettings.  TRUE is returned in case TL1/UDP
acccess is possible, FALSE in case no TL-1/UDP access is allowed.

=end html

=cut

sub TL1ViaUDP($$) {
  my ($accessBits, $debug) = @_;

  Log($DEBUG, "TL1 access bits: $accessBits->{TL1}\n") if $debug;
  return(($accessBits->{TL1} &0x02) != 0 ? 1 : 0);
}

=head2 CliViaSSH($$)

=begin html
Check whether CLI/SSH access is possible.  $accessBits is a reference to a hash as returned by GetAccessSettings.  TRUE is returned in case CLI/SSH
acccess is possible, FALSE in case no CLI/SSH access is allowed.

=end html

=cut

sub CliViaSSH($$) {
  my ($accessBits, $debug) = @_;

  Log($DEBUG, "CLI access bits: $accessBits->{CLI}\n") if $debug;
  return(($accessBits->{CLI} &0x01) != 0 ? 1 : 0);
}

=head2 CliViaTelnet($$)

=begin html
Check whether CLI/Telnet access is possible.  $accessBits is a reference to a hash as returned by GetAccessSettings.  TRUE is returned in case
CLI/Telnet acccess is possible, FALSE in case no CLI/Telnet access is allowed.

=end html

=cut

sub CliViaTelnet($$) {
  my ($accessBits, $debug) = @_;

  Log($DEBUG, "CLI access bits: $accessBits->{CLI}\n") if $debug;
  return(($accessBits->{CLI} &0x02) != 0 ? 1 : 0);
}

=head2 SshPossible($$$)

=begin html
Check whether SSH access to $isam is possible.  Check ~home/.ssh/known_hosts and /etc/ssh/ssh_known_hosts whether the ISAMs IP address can be found
in one of both files.  If the IP address is not found FALSE will be returned and $ERROR will be set accordingly.

=end html

=cut

sub SshPossible($$$) {
  my ($isam, $debug, $config) = @_;
  my $homeDir = $ENV{HOME};
  my $possible = 0;
  my $file;
  my $line;
  my $sysDbFound = 0;
  my $locDbFound = 0;

  $ERROR = "Host not found in known_hosts database";
  Log($DEBUG, "Home directory: '$homeDir'\n") if $debug;
  if (-r "$config->{SYS_KNOWNHOSTS}") {
    $sysDbFound = 1;
    if (open($file, "<$config->{SYS_KNOWNHOSTS}")) {
      ToolCommon::Log($DEBUG, "Reading $config->{SYS_KNOWNHOSTS} looking for $isam...\n") if $debug;
      while (defined($line=<$file>)) {
	chomp $line;
	ToolCommon::Log($DEBUG, "Read '$line'\n") if $debug;
	if (index($line, $isam) != 1) {
	  $ERROR = "";
	  $possible = 1;
	  last;
	}
      }
      close($file)
    } else {
      $ERROR = "Could not open $config->{SYS_KNOWNHOSTS} for read: $!";
    }
  }
  if (! $possible) {
    if (-r "$homeDir/$config->{LOC_KNOWNHOSTS}") {
      $locDbFound = 1;
      if (open($file, "<$homeDir/$config->{LOC_KNOWNHOSTS}")) {
	ToolCommon::Log($DEBUG, "Reading $homeDir/$config->{LOC_KNOWNHOSTS} looking for $isam...\n") if $debug;
	while (defined($line=<$file>)) {
	  chomp $line;
	  ToolCommon::Log($DEBUG, "Read '$line'\n") if $debug;
	  if ($line =~ /^$isam\s.+/) {
	    $ERROR = "";
	    $possible = 1;
	    last;
	  }
	}
	close($file)
      } else {
	$ERROR = "Could not open $homeDir/$config->{LOC_KNOWNHOSTS} for read: $!";
      }
    }
  }
  if (!($locDbFound + $sysDbFound)) {
    $ERROR = "This system does not have a known hosts database";
  }
  return $possible;
}

sub CreateTmpKeyFile($$$) {
  my ($dir, $accessInfo, $debug) = @_;
  my $file;
  my $ret = 1;
#  my $key = Net::SSH::Perl::Key->new("DSA", $accessInfo->{PUBKEY});
#  my $bits = $key->size;
#  my $kfp = $key->fingerprint;

  if (open($file, ">$dir/tempkey.txt")) {
    print $file $accessInfo->{PUBKEY};
    close $file;
  } else {
    $ERROR = "Failed to create $/dir/tempkey.txt: $!";
    $ret = 0;
  }
#  ToolCommon::Log($DEBUG, "keysize: $bits, key fingerprint: $kfp\n") if $debug;
  return $ret;
}

=head2 GetAccessOperId($$$)

=begin html
This function retrieves the access operator ID using SNMP.  Retrieving by CLI could result in a telnet timeout in case the timeout is not
high enough (the info config system command hangs a long time on the pae section).

=end html

=cut

sub GetAccessOperId($$) {
  my ($fh, $snmpSession) = @_;
  my $accOperId =  SNMPComm::GetValue('SYSTEM ACCESS OPERATOR', $toolConsts{SYSTEMACCESSOPERATORID}, 2  );
  if( not defined $accOperId ) {
       $ERROR = sprintf("Failed to retrieve Access Operator ID: %s", SNMPComm::GetError() );
  }
  return $accOperId;
}

=head2 GetNEdiskspace($$$)

=begin html
This function retrieves the Diskspace available in the NE using SNMP.  

=end html

=cut

sub GetNEdiskspace($$$) {
  my ($session, $retry, $debug) = @_;
  my $diskFreeSpace =  SNMPComm::GetValue('NE DISK FREE-SPACE', DISK_FREE_SPACE, $retry  );
  if( not defined $diskFreeSpace) {
       $ERROR = SNMPComm::GetError() ;
  }
  return $diskFreeSpace;
}

sub CheckVlanUniqHostId($) {
  my ($cli) = @_;
  my $ret = 0;  

  my $vmacAddr = IsamCli::GetVlanUniqHostId($cli);
  if(defined($vmacAddr)) {
        if($vmacAddr != 0) {
           Log($DEBUG, "Fetched Vlan Uniq host-id : $vmacAddr\n") if $params->{DEBUG};
        } else {
           Log($DEBUG, "Vlan Uniq host-id not set\n") if $params->{DEBUG};
        } 
        if($vmacAddr > $toolConsts{VLANVMACADDR}) {
           my $ERROR = "Vlan unique Host-id not in acceptable range (1 to 8191)\n"; 
	   Log($DEBUG, "  FAILED, $ERROR");
        } else {
           $ret = 1; 
           Log($DEBUG, "  OK\n");
        }
  } else {
    Log($DEBUG, "  FAILED\n");
    $ERROR = IsamCli::GetError();
  } 
  
  return $ret; 
}

sub GetDBVersion($) {
  my ($oswpDesc) = @_; 
  my $line;
  my $dbVersion = undef;

  if (! open(DESC, $oswpDesc)) {
          print STDERR "Could not open $oswpDesc file for read \n";
          exit FILE_OPEN_FAIL_CODE;
  } 

  while (defined($line = <DESC>)) {
      chomp $line;
      $line =~ s/\r//g;
      ### DBASE-VERSION  : A4.200;
      if($line =~ /DBASE-VERSION\s+:\s+([^\s]+)\s*;/) {
          $dbVersion = $1;
          last;
      ### SWP_COMPATIBLE_DATABASE_VERSION_NUMBER = DB1.101 ;
      } elsif($line =~ /SWP_COMPATIBLE_DATABASE_VERSION_NUMBER\s+=\s+([^\s]+)\s*;/) {
          $dbVersion = $1;
          last;
      }
  }
  close (DESC);

  return $dbVersion;
}

sub CheckMiscLT($) {
  my ($LT) = @_; 
  return  Capability::IsMiscLT(uc($LT)); 
}

sub CheckLTSWPresence($) {
my ($cli) = @_;
my %ltNoSWFound = ();
my $ltInfo = IsamCli::GetLtConfigState($cli);

 foreach my $lt (keys %{$ltInfo}) {
   if (exists $ltInfo->{$lt}{ERRSTATE}) {
     if ($ltInfo->{$lt}{ERRSTATE} eq "WAITING-FOR-SW") {
       $ltNoSWFound{$lt}{TYPE} = $ltInfo->{$lt}{TYPE};
       $ltNoSWFound{$lt}{ERRSTATE} = $ltInfo->{$lt}{ERRSTATE};
     } 
   } 
 }
  return (\%ltNoSWFound);
}

=head2 OpenCliWrap($$$$)

=begin html
This function is more of a wrapper around the OpenCli subroutine. For CLI SSH setting in the ISAM the routine tries to use the second password to make a CLI connection in case the cli connection is not established with the first password. Its written on the lines of a programming macro to eliminate unnecessary code replication in the libraries.  Since this function does not generate any error messages, it is upto the calling function to log the errors got during  cli connection establishment.

=end html

=cut

sub OpenCliWrap($$$) {
 my ($isam, $fh, $timeout) = @_;

 my $cli = OpenCli($isam, $fh, $timeout);

  if (!defined($cli)) {
    if( ( defined($params->{PASSWORD2}) ) &&
        ( ( (defined $params->{USEDEBUGVIASSH}) && ($params->{USEDEBUGVIASSH}) ) ||
        ( (defined $params->{USESSH}) && ($params->{USESSH}) ) )
       ) {
            sleep(5);
	    Log($fh, "ToolCommon::Open CLI Wrap failed with first password, Trying again with second password ...");
	    my $oldCliPassword = $params->{PASSWORD}; 
	    $params->{PASSWORD} = $params->{PASSWORD2}; 
	    $cli = OpenCli($isam, $fh, $timeout);
	    $params->{PASSWORD} = $oldCliPassword; 
    }
  }
  return $cli; 
}

=head2 SFTPClientWrap($$$$)

=begin html
This function is more of a wrapper around the sftpClient script execution.
To download files it requires USER and PASSWORD, We have option to use SECOND PASSWORD for establish a connection

=end html
=cut

sub SFTPClientWrap($$$$$$) {
     my ($logFile, $config, $path, $SWPFile, $type, $isam ) = @_;
	 my $dir = FamilyCommon::GetNeStorage();
     ## use sftp xfer
     ToolCommon::Log($logFile, "Retrieving SWP descriptor file using SFTP...\n");
     ToolCommon::Log($logFile, " Executing SFTP command to $type $SWPFile ...\n");
     my $sftpClientCmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD}\' '$isam' '$SWPFile' '$type'";
     my $cmd = sprintf("$sftpClientCmd > %s 2>&1", defined($dir) ? "$dir/sftpClient.txt" : "/dev/null");
     my $ret = system($cmd);
     if ($ret && (defined($params->{PASSWORD2}) )) {
         sleep(5);
         Log($logFile, "ToolCommon::SFTPClientWrap failed with first password, Trying again with second password ...\n");
         $sftpClientCmd = "$config->{PERL} '$path/sftpClient.pl' \'$params->{USERNAME}\' \'$params->{PASSWORD2}\' '$isam' '$SWPFile' '$type'";
         $cmd = sprintf("$sftpClientCmd >> %s 2>&1", defined($dir) ? "$dir/sftpClient.txt" : "/dev/null");
         $ret = system($cmd);
     }
     return $ret;
}

=head2 KnownHostCheck($$)

=begin html
This function checks whether the known hosts file and it's parent directory exists or not and whether the file and the directory has read/write 
permission. This check is done both for the user specific known hosts file and systen specific knwon hosts file.

=end html

=cut

sub KnownHostCheck($$) {
	my ($hostAutoUpd, $knownHostsFile) = @_;
	my $err = "";
	my $retVal = 0;

	if (-d "$knownHostsFile"){
		  return ( 1, "Known host file given is a directory '$knownHostsFile'." );
	}

	my $dir = dirname($knownHostsFile);
	if (! -d "$dir"){
		  return ( 1, "Known host file parent directory '$dir' does not exist" );
	}

    if ($hostAutoUpd) {
		if (! -w "$dir"){	## if the host autoupdate is enabled and the file does not exist, checks whether the directory has write permission to create new file.
		  return ( 1, "Known host file parent directory '$dir' does not have write permission" );
		} elsif ((-e "$knownHostsFile") && (! -w "$knownHostsFile")) {	## if the host autoupdate is enabled and the file exists, check whether it has write permission.
		  return ( 1, "Known host file '$knownHostsFile' does not have write permission" );
		}
    } else {
	  	if (! -e "$knownHostsFile" ) {	## if the host autoupdate is disabled, checks whether the file exists.
		  return ( 1, "Known host file '$knownHostsFile' does not exist" );
		} elsif ((-e "$knownHostsFile") && (! -r "$knownHostsFile")){	## if the host autoupdate is disabled and the file exists, checks whether it has read permission.
		  return ( 1, "Known host file '$knownHostsFile' does not have read permission" );
        }
    }
	return (0, "");
}


sub GetOswpInfoCLI($$$$$$$;$) {
  my ($session, $snmpSession, $debug, $dir, $isam, $ntType, $oswpInfoEmpty, $retries) = @_;
  my ($actOswpRelNumExt, $pasOswpRelNumExt, $actFwRelNumExt, $pasFwRelNumExt);
  my ($oswpSnmpInfo, $oswpFwInfo, $ntLoadInfoAct, $ntLoadInfoPas);
  my $getActive = undef;
  my $getPassive = undef;
  my $actPrefix = "";
  my $passPrefix = "";

  CheckEnv("GetOswpInfoTL1");
  Log($DEBUG, "Getting Oswp Info...");
  $ERROR = "";
  my $oswpInfo = IsamCli::GetOswpInfo($session, $toolConsts{REGEXP});
  if (defined($oswpInfo)) {
    $oswpInfoEmpty = 0;
    Log($DEBUG, "  OK\n");
  } else {
    Log($DEBUG, "  FAILED\n");
    $ERROR = IsamCli::GetError();
    return undef;
  }
  if ($oswpInfo->{'ACTIVE'}{'NAME'} =~ /$toolConsts{REGEXP}/) {
    $actOswpRelNumExt = $3*100 + $4;
    $actFwRelNumExt = $oswpInfo->{'ACTIVE'}{'SUFFIX'} * 100 + $oswpInfo->{'ACTIVE'}{'MAINT'};
  } else {
    $actOswpRelNumExt = 0;
    $actFwRelNumExt = 0;
    if ($oswpInfo->{'ACTIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
      $getActive = 1;
    }
  }
  if ($oswpInfo->{'PASSIVE'}{'NAME'} =~ /$toolConsts{REGEXP}/) {
    $pasOswpRelNumExt = $3 * 100 + $4;
    $pasFwRelNumExt = $oswpInfo->{'PASSIVE'}{'SUFFIX'} * 100 + $oswpInfo->{'PASSIVE'}{'MAINT'};
  } else {
    $pasOswpRelNumExt = 0;
    $pasFwRelNumExt = 0;
    if ($oswpInfo->{'PASSIVE'}{'NAME'} ne $toolConsts{NO_OSWP}) {
      $getPassive = 1;
    }
  }
  if (defined($oswpInfo->{PASSIVE}{MAJOR})) {
    Log($DEBUG, sprintf("  Active OSWP : %s - %d.%d.%d%02d (%d)\n                       Passive OSWP: %s - %d.%d.%d%02d (%d)\n",
			$oswpInfo->{ACTIVE}{NAME}, $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR},
			 $oswpInfo->{ACTIVE}{SUFFIX}, $oswpInfo->{ACTIVE}{MAINT}, $oswpInfo->{ACTIVE}{INDEX},
			 $oswpInfo->{PASSIVE}{NAME}, $oswpInfo->{PASSIVE}{MAJOR}, $oswpInfo->{PASSIVE}{MINOR},
			 $oswpInfo->{PASSIVE}{SUFFIX}, $oswpInfo->{PASSIVE}{MAINT}, $oswpInfo->{PASSIVE}{INDEX}));
  } else {
    Log($DEBUG, sprintf("  Active OSWP : %s - %d.%d.%d%02d (%d)\n                       Passive OSWP: %s - 0.0.00 (%d)\n",
			 $oswpInfo->{ACTIVE}{NAME}, $oswpInfo->{ACTIVE}{MAJOR}, $oswpInfo->{ACTIVE}{MINOR},
			 $oswpInfo->{ACTIVE}{SUFFIX}, $oswpInfo->{ACTIVE}{MAINT}, $oswpInfo->{ACTIVE}{INDEX},
			 $oswpInfo->{PASSIVE}{NAME}, $oswpInfo->{PASSIVE}{INDEX}));
  }
  return $oswpInfo;
}

#-- End of lib
1;
