#!/usr/bin/perl -w

=head1 ISAM ONT DownloadandActivate 

=begin html
This script will download and activate the new ONT software 
<ul>
  <li>perform a number of pre-checks to make sure NT is reachable </li> 
  <li>download and activate new OSWP onto the node</li>
</ul>

The operations of the script are influenced by:
<ul>
  <li>command line options</li>
  <li>a parameter configuration file</li>
</ul>

=end html

=cut

#==============================================================================
#-- File:         ontDownloadandActivate.pl
#-- Date:         03.10.2009
#-- Company:      Alcatel-Lucent
#-- Authors:      Palaniappan RAMASAMY / Suresh Chandrasekaran
#------------------------------------------------------------------------------
#-- Function:     Script to download and activate the ONT software 
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel AND 2005
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#-- #-- Requirements: for 7302HiCap are defined below:
#--               ANTms36033-029:
#--               ANTms36033-030:
#--               ANTms36033-031:
#--               ANTms36033-032:
#--               TRS - 3FE-52193-AYAA-DTZZA
#--
#--  1.0.1 - 2009/02/25  : First Version for ontDownloadandActivate.pl
#--  1.0.2 - 2009/04/22  : DDTs ticket resolved - BDFam48453, BDFam48335, BDFam47022
#--  1.0.3 - 2009/05/12  : DDTs ticket resolved - BDFam49289
#--  1.0.4 - 2009/06/02  : DDTS ticket resolved - BDFam50346
#--  1.0.5 - 2009/06/08  : OBR Changes - BDFam50721
#--  1.0.6 - 2009/07/08  : DDTs ticket resolved - BDFam49973
#--  1.0.7 - 2009/07/16  : Common changes pertaining to DDTS tickets BDFam47286,BDFam50360,BDFam51427.
#--  1.0.8 - 2009/07/26  : Log directory related changes pertaining to DDTS tickets BDFam51948.
#--  1.0.9 - 2009/08/05  : DDTs tickets resolved
#--                          BDFam52496
#--                          BDFam51685
#--                          BDFam52498
#--                          BDFam52435 
#--                          BDFam53287
#--                          BDFam53845
#--                          BDFam53774
#--  1.0.10 - 2009/09/16  : DDTs tickets resolved BDFam53859
#--  1.0.11 - 2009/10/08  : DDTs tickets resolved 
#--  3.0.1  - 2010/05/24  : First Implementation for converged PBMT - JanakiRaman and Janith
#--  3.0.2  - 2010/07/06  : Added X86 changes
#--  3.0.3  - 2011/01/05  : new LRM option --id (suppresses the --logfile option). Must not be used in combination with --hosts
#--  3.0.4  - 2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--  3.0.5  - 2011/05/02 : ALU01178887
#--  3.0.6  - 2012/10/22 : ALU01761938
#--  3.0.7  - 2012/11/06 : ALU01837839
#--  3.0.8  - 2013/04/29 : ALU01880147
#--  3.0.9  - 2013/07/29 : ALU01899054
#--  3.0.10 - 2013/12/04 : introduce additional option DELAYACT
#--  3.0.11 - 2014/10/10 : ont sw download progress timer parameter taken in
#--  3.0.12 - 2015/05/25 : ALU02060588
#--  3.0.13 - 2015/06/10 : ALU02083077
#--  3.0.14 - 2015/10/28 : ALU02139820
#--  3.0.15 - 2016/03/01 : ALU02120557
#--  3.0.16 - 2016/07/15 : ALU02237966
#--  3.1.0 -- 2018/06/20 : ALU02278259
#--  3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697
#--  3.1.1  - 2018/09/13 : ONT Migration via CLI 
#--  3.1.2  - 2018/12/02 : added cli buffer
#--  3.1.3  - 2019/02/28 : Fix for reading SNMPV3 configs from param.cfg 
#------------------------------------------------------------------------------

=head1 VERSION

=begin html
Current version: "ontDownloadandActivate.pl - 3.1.3 -- 2019/02/28 "

=end html

=cut

my $VERSION = "ontDownloadandActivate.pl - 3.1.3 -- 2019/02/28";
#==============================================================================

=head1 PERL MODULES USED

=begin html
The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::stat</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h";</li>
      <li>Socket</li>
      <li>Cwd</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2098-DFZZA">Net::SNMP<a/></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2156-DFZZA">IsamTL1</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

use strict;
use FileHandle;
use File::stat;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#==============================================================================
#== Find the lib dir
use Cwd qw( realpath getcwd);
use File::Basename;
use Config;

my $scriptDir;
BEGIN {
$scriptDir = dirname(realpath($0));
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 


#==============================================================================
use Net::TFTP;
use Net::SNMP;

use Crypt::Lite;
use Utils;  # -- BB --
use PbmtConfig;
use Logger qw(:ALL);
use ErrorCodes qw(:EXIT_CODES :ONT_ERR);
use FamilyCommon;
use Octopus;
use Capability;
use OntDlAct7302HiCap;
use OntDlAct7302HiCapCLI;
use IsamTL1;
use IsamCli;
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use version qw(:ALL);

END {
    LogCloseFile($nodeLogFH);
    LogCloseFile($execLogFH);
}


#==============================================================================
#== Prototypes
#==============================================================================
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$);
sub handleONT($$$$);
sub AdjustToolVars($);
sub CleanUp($;$);

#==============================================================================
#== Constants
#==============================================================================
use constant {
    # download & actiavte mode
    ONT_DL    => 1, # use ONTSWCTRL (ONT DLSW=AUTO )
    ONT_ACT => 2,   # use ONTSWCTRL (ONT SWVERPLND=AUTO)
    ONT_DLACT    => 3,  # use ONTSWCTRL (ONT DLSW=AUTO SWVERPLND=AUTO)
    ONT_DELYACT    => 4  # delayact opt 
};
#==============================================================================
#== Vars  -- Modif BB --
#==============================================================================

my $STATUSFILE = "result.txt";
my $ID_FILENAME = "ONTDLACT";
my $sessionCount = 0;
my $logExecOrNode = 0;
my %toolData = (HOSTSFILE => "/etc/hosts",
		FTPDIR =>         "/var/opt/aws/equipment",	
		ONTRELDIR  => "tools/",
                ONTRELMAPFILE => "ONT_Release_Mapping.txt",
		TL1USER =>     "",
		TL1PWD =>     "",
                MAXSESSIONS =>     5,
		RETRIES =>         5,
                BUFSIZE =>        100,
		FTPTIMEOUT =>     5,
		TELNETTIMEOUT =>   10,
                OCTOPUSSCRIPTS =>   "tools/octopus",
                OCTOPUSPATH => "tools/octopus",
                ONT_DOWNLOAD_TIMEOUT => 1200,
                ONT_ACTIVATE_TIMEOUT => 1200,
                ONT_DWLD_PROGRESS_TIMEOUT => 600,
		TL1UDPPORT =>      13001,
		# FXRDATA is a pointer to hash being filled in at a later stage.  The hash contains the following:
		# 'PROTOCOL': set to the filetransfer protocol being used by the ISAM
		# 'SERVERS': the filetransfer server used by the ISAM.  Only set when the protocol is different from tftp.
		#            Is a hash consisting of a list of IP-addresses and usernames (only username per filetransfer server is possible).
		FXRDATA =>         undef);

=head1 USAGE

=head2 How to run?

=begin html
Invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] ontDownloadandActivate.pl --isam $lt;ip&gt;|--hosts &lt;file&gt; \ <br> 
           --onttypes &lt;onttype1&gt;[Load1],&lt;onttype2&gt;[Load2] \ <br>
           --ontdownloadandactivatemode &lt;mode&gt;  \ <br> 
           --operationmode &lt;dlactmode&gt; [--ontreleasemap <path>] \ <br>
           [--multi] [--seplog | --noseplog] [--sessions &lt;nr&gt;] \ <br>
           [--warn &ltThreshold&gt] [--tl1username &lt;username&gt;] [--tl1password &lt;password&gt;] \ <br>
           [--v] [--debug] --logfile &lt;filename&gt; [--noname] \ <br> 
           [--comm &lt;SNMP community string&gt;] [--noping] --release &lt;text&gt; [--v2]\ <br>
           [--v3auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;[privProtocol&gt;:&lt;password&gt;]]</tt>


=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>isam</td><td>ip or the host name of the NE</td></tr>
  <tr><td>hosts</td><td>name of the host file</td></tr>
  <tr><td>logfile</td><td>name of the logfile</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>ontdownloadandactivatemode</td><td>onts will be in auto or manual mode </td></tr> 
  <tr><td>operationmode</td><td>Determine wether the script will do the ont download, activation or both </td></tr>
  <tr><td>onttypes</td><td>Specified ONT types </td></tr>
  <tr><td>ontreleasemap</td><td>path of the ont release mapping file</td></tr>
  <tr><td>tl1username</td><td>TL1 username</td></tr>
  <tr><td>tl1password</td><td>TL1 password</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td>
</tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>debug</td><td>enables additional debugging traces</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                         </ul><br>
                         <ul><li>Default value for the Privacy protocol is DES.</li>
                         </ul><br>e.g. --v3Auth user:password:authprotocol.Incase when Auth User is configured.</td></tr>
                         </ul><br>e.g. --v3Auth user:password:authprotocol[privproto:password].When AuthPrivUser is configured.</td></tr>
</table>

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("\n%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <IP|hostname> | --hosts <file>  --logfile <log filename> | -id <id> \\ \n";
  print STDERR "      --onttypes <Type1:Load1,Type2:Load2,..TypeN:LoadN> --release <Release>  \\ \n";
  print STDERR "      --ontdownloadandactivatemode < Auto|Manual > \\ \n";
  print STDERR "      --operationmode <DLONLY | ACTONLY | DLANDACT | DELAYACT> [--sessions <no>] \\ \n";
  print STDERR "      [--ontreleasemap <path>] [--noname] [--noping]\\ \n";
  print STDERR "      [--cliusername <username>] [--clipassword <password>] \\ \n";
  print STDERR "      [--tl1username <username>] [--tl1password <password>] [--v] [--debug] [--comm <SNMP community string>][--v2] \\ \n";
  print STDERR "      [--multi] [--seplog | --noseplog] \\  \n";
  print STDERR "      [--v3auth <user>[:<password>:<authProtocol>[:<privProtocol>:<password>]]] [--warn <Threshold>] \\ \n";
  print STDERR "\n";
  print STDERR "    Params:\n";
  print STDERR "       isam            : ip or the host name of the NE.\n";
  print STDERR "       hosts           : hosts file name \n";
  print STDERR "       sessions        : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         id           : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                        The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "       operationmode   : Specify wether download or activate or even both is required \n"; 
  print STDERR "       ontdownloadandactivatemode : Mode of which ONT activation should happen [Auto | Manual]\n";
  print STDERR "       release         : Release info \n";
  print STDERR "       onttypes        : Specify the types of ONT & Loads for multiple types\n";
  print STDERR "       ontreleasemap   : path of directory where ONT Release Mapping file is located\n";
  print STDERR "       warn            : Threshold for failure rate detection\n";
  print STDERR "       noname          : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "       noping          : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "       tl1username     : TL1 username\n";
  print STDERR "       tl1password     : TL1 password\n";
  print STDERR "       cliusername     : CLI username \n";
  print STDERR "       clipassword     : CLI password \n";
  print STDERR "       multi           : include Unix username in NE logging directory path\n";
  print STDERR "       seplog          : if set create a log file per node, otherwise logging gets appended in the child log file \n";
  print STDERR "       noseplog        : keep all NE logging in a single child log file (default is a single logfile per NE) \n";
  print STDERR "       v               : Print version and exit\n";
  print STDERR "       debug           : enable debug message, which will print in log file in addition to log message. \n";
  print STDERR "       SNMP settings   :\n"; 
  print STDERR "       comm            : NT SNMP community string when using SNMPv1 or SNMPv2.  Default is \"public\"\n";
  print STDERR "       v2              : Use SNMPv2c\n";
  print STDERR "       v3auth          : username, password and authentication protocol when using SNMPv3Auth\n";
  print STDERR "                         The password should be at least 6 characters long.\n";
  print STDERR "                         Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                         All fields are mandatory when this option is used.\n";
  print STDERR "                         optional setting when using SNMPv3 auth protocol providing protocol and password.\n";
  print STDERR "                         Default value for the Privacy protocol is DES.\n";
  print STDERR "                         e.g. --v3Auth<user>:<password>:<authprotocol>[:<privprotocol>:<password>].Used when AuthPrivUser is configured.\n";
  print STDERR "                         e.g. --v3Auth<user>:<password>:<authprotocol>.Used when AuthUser is configured.\n";
  print STDERR "                         e.g. --v3auth<user>. Used when NoAuth and NoPriv is configured.\n";
  exit CMD_INCOMPL_CODE;
}

=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of errors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $fullOntTypes, $warnThreshold,$ontDownloadandActivateMode, $tl1Username, $tl1Password , $release,
      $debug, $version, $noPing, $v3Auth, $communityString, $noName, $id,  $cliUsername, $cliPassword);
  my ($noSepLog, $sepLog, $multi, $logFile, $hosts, $snmpV2,$sessions, $relMapDirName);
  my $operationMode;  
  my $regExp;
  my %params;
  my $crypt = Crypt::Lite->new('debug' => 0);  
  my $options = GetOptions('isam=s', \$isam,
                           'hosts=s', \$hosts,
                           'sessions=i', \$sessions,
			   'onttypes=s', \$fullOntTypes,
                           'ontreleasemap=s', \$relMapDirName,
	                   'warn=i', \$warnThreshold,
                           'operationmode=s', \$operationMode,
	                   'ontdownloadandactivatemode=s', \$ontDownloadandActivateMode,
			   'debug', \$debug,
			   'tl1username=s', \$tl1Username,
			   'tl1password=s', \$tl1Password,
                           'username|cliusername=s', \$cliUsername,
                           'password|clipassword=s', \$cliPassword,
                           'multi', \$multi,
                           'seplog', \$sepLog,
                           'noseplog', \$noSepLog,
                           'logfile=s', \$logFile,
			   'v', \$version,			   
			   'comm=s', \$communityString,			   
                           'v2', \$snmpV2,
			   'noping', \$noPing,
                           'release=s', \$release,
                           'noname', \$noName,
			   'v3auth=s', \$v3Auth,
                           'id=s', \$id);

  #-- check options
  if (! $options) {
    Usage();
  }
  
  if ($ARGV[0])
  {
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }
  
  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit SUCCESS_CODE;
  }
  
  if (defined($isam) and defined($hosts)) {
    print STDERR "ERROR: Target isam ip or hosts file should be specified\n";
    Usage();	
  }

  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
 
  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
    $params{PROGFILE} = $id . "_" . $ID_FILENAME . ".update";
    $params{ONT_STATUSFILE} = $id . "_" . $ID_FILENAME . ".status";
  } else {
    $params{LOGFILE} = $logFile ;
    $params{PROGFILE} = $logFile . ".update";
    $params{ONT_STATUSFILE} = $logFile . ".status";
  }

  if (!defined($operationMode)) {
    print STDERR "ERROR: --operationMode not specified \n";
    Usage();
  } else {
      if (uc($operationMode) eq "DLONLY") {
        $params{OPMODE} = ONT_DL;
      } elsif( uc($operationMode) eq "ACTONLY" ) {
        $params{OPMODE} = ONT_ACT;
      } elsif( uc($operationMode) eq "DLANDACT" ) {
        $params{OPMODE} = ONT_DLACT;
      } elsif( uc($operationMode) eq "DELAYACT" ) {
        $params{OPMODE} = ONT_DELYACT;
      } else {
        print STDERR "Error: Invalid value specified for script operation mode : $operationMode\n";
        Usage();
      }
  }

  
  if (!defined($release) ) {
    print STDERR "Error: --release missing ..\n";
    Usage();
  } else {
    if ( uc($release) =~ /^[LR][0-9][1-9]\.[0-9][0-9]\.[0-9][0-9]$/ || uc($release) =~ /^[LR][0-9][1-9]\.[0-9][0-9]\.[0-9][0-9][A-Z]$/ ) {
      $params{RELEASE} = $release;
    } else {
      print STDERR "Error : Invalid release info - '$release'\n";
      Usage (); # not sure about exit code.
    }
  }

  if (defined($fullOntTypes)) {
    my @fulltypes = split(',',$fullOntTypes);
    my $ontTypes= undef;
    foreach my $types (@fulltypes) {
      my @type = split(':',$types);
      my $nooftypes = scalar @type;

      if ( $nooftypes > 2 || $nooftypes < 2) {
        print STDERR "Error: Invalid input - @type \n";
        Usage();
      }

      #if (! (uc($type[0]) =~ /^[IOB]-(\d+)[A-Z]-[A-Z]$/) ){ # BDFam48335
      #  print STDERR sprintf("%s\n","Invalid ONT Type : $type[0]");
      #  Usage();
      #}
   
      if ((uc($type[1]) !~ /^H.248$/) && 
          (uc($type[1]) !~ /^SIP$/) && 
          (uc($type[1]) !~ /^VEIP_SIP$/) && 
          (uc($type[1]) !~ /^DO$/) ){
        print STDERR "Error: Invalid Load type : $type[1]"; 
        Usage();
      }
    }
    $params{FULLONTTYPES} = uc($fullOntTypes);
    $params{ONTTYPES} = uc($ontTypes);
  } else {
    print STDERR "Error: --ontTypes missing ..";
    Usage();
  }

  if (defined($warnThreshold) && (0 > $warnThreshold || $warnThreshold > 100)) {
    print STDERR "Error: Invalid value specified for warn : $warnThreshold\n";
    Usage();
  } elsif (defined($warnThreshold)) {
    $params{WARNTHRESHOLD} = $warnThreshold;
  } else {
    $params{WARNTHRESHOLD} = 0;                 # BDFam49973
  }

  if (defined ($ontDownloadandActivateMode)) {
    if (uc($ontDownloadandActivateMode) eq "AUTO" || uc($ontDownloadandActivateMode) eq "MANUAL") {
	$params{ONTDOWNLOADMODE} = $ontDownloadandActivateMode;
    } else {
      print STDERR "Error: Invalid value specified for ONT DownloadandActivateMode : $ontDownloadandActivateMode\n";
      Usage();
    }	
  } else {
    print STDERR "--ontDownloadandActivateMode missing ..\n";
    Usage();
  }
   #Fix for BDFam53859 starts here
  if (defined($tl1Username) && ($tl1Username eq "")) {
   print STDERR "Error : Invalid value specified for usernameTL1 \n";
   Usage();
  }

  if (defined($tl1Password) && ($tl1Password eq "")) {
   print STDERR "Error : Invalid value specified for passwordTL1 \n";
   Usage();
  }
  #Fix for BDFam53859 ends here
 
  if (defined($tl1Username)) {
    $toolData{TL1USER} = $tl1Username;
  }

  if (defined($tl1Password)) {
    $toolData{TL1PWD} = $tl1Password;
  }

  if (defined($communityString)&& defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
        "($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }

  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }

  if (defined($relMapDirName)) {
     if($relMapDirName ne ""){
        $toolData{ONTRELDIR} = $relMapDirName;
     } 
  }

  if(! -r "$toolData{ONTRELDIR}/$toolData{ONTRELMAPFILE}"){
    print STDERR "ERROR: ONT Release mapping file not found in the location $toolData{ONTRELDIR}. \n";
    Usage();
  }
  
 #-- fill hash, set defaults if needed
  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if( defined($debug));
  $params{NONAME} = 1 if ( defined($noName) );
  $params{NOTFTP} = 0;
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";
  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});


  if (defined($cliUsername)) {
    if ($cliUsername eq "") {
      print STDERR "Error : Invalid value specified for CLI username  :'$cliUsername' \n";
      Usage();
    }
    $params{USERNAME} = $cliUsername;
    $toolData{FTPMODE} = "SFTP";
    $params{NOTFTP} = 1;
    } else {
    $params{USERNAME} = $toolData{USERNAME};
  }


  if (defined($cliPassword)) {
    if ($cliPassword eq "") {
      print STDERR "Error : Invalid value specified for CLI password  :'$cliPassword' \n";
      Usage();
    }
    $params{PASSWORD} = $cliPassword;
    $toolData{FTPMODE} = "SFTP";

  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }


  if (($toolData{ONT_DOWNLOAD_TIMEOUT} !~ /^\d{1,}$/)) {
    print STDERR "Error: Invalid value specified for ONT_DOWNLOAD_TIMEOUT : $toolData{ONT_DOWNLOAD_TIMEOUT} \n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if (($toolData{ONT_ACTIVATE_TIMEOUT} !~ /^\d{1,}$/)) {
    print STDERR "Error: Invalid value specified for ONT_ACTIVATE_TIMEOUT: $toolData{ONT_ACTIVATE_TIMEOUT} \n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }

  if($params{OPMODE} == ONT_DL) {
    $params{ONTDLACT_TIMEOUT} = $toolData{ONT_DOWNLOAD_TIMEOUT};
  } elsif ($params{OPMODE} == ONT_ACT ) {
    $params{ONTDLACT_TIMEOUT} = $toolData{ONT_ACTIVATE_TIMEOUT};
  } else {
    $params{ONTDLACT_TIMEOUT} = $toolData{ONT_DOWNLOAD_TIMEOUT} + $toolData{ONT_ACTIVATE_TIMEOUT};
  }
  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
 
  $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  $params{SNMP_V1} = 1 if (defined($params{SNMP}{V1}{NT_COMMUNITY}) and !defined($snmpV2) and !defined($v3Auth));
  if (!defined($snmpV2) && !(defined($communityString))) { #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  } else {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
    $params{SNMP_V2} = 1 ;
    $params{SNMP}{V1}{NT_COMMUNITY} = undef;
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  }

  if (defined($v3Auth)) {
    my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    my $snmpV3AuthParmsLength = scalar(@snmpV3AuthParms);
    if( ($snmpV3AuthParmsLength != 1) && ($snmpV3AuthParmsLength != 2) && 
        ($snmpV3AuthParmsLength != 3) && ($snmpV3AuthParmsLength !=5) ){
         print STDERR "ERROR: Invalid arguments specified for SNMPv3 parameter.\n";
         Usage();
    }
    if ( $snmpV3AuthParms[0] ne "" ) {
        $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
    } else {
	print STDERR "SNMP V3 Username is NULL/EMPTY";
	Usage();
    }
    my @pwdAr = ();
    if( $snmpV3AuthParmsLength > 1 ) {
      $snmpV3AuthParms[2] = "" if ($snmpV3AuthParmsLength == 2);
      if( $snmpV3AuthParms[1] eq "" ) {
          $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
          $params{SNMP}{V3}{AUTH}{PROTO} =  defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
      } else {
          @pwdAr = split(//, $snmpV3AuthParms[1]);
          if (scalar @pwdAr > 5) {
            if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA256|SHA512|MD5)$/ ) or ($snmpV3AuthParms[2] eq "")) {
              $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
              $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
              $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
              if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
                 $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
              }
            } else {
              print STDERR "SNMPv3 authentication protocol can only be SHA or SHA256 or SHA512 or MD5.\n";
              Usage();
            }
          } else {
            print STDERR "SNMPv3 authentication password must be at least 6 characters long.\n";
            Usage();
          }
      }
      $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
      $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
      if( $snmpV3AuthParmsLength > 3 ) {
           if( $snmpV3AuthParms[4] eq "" ) {
               $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
               $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
           } else {
               @pwdAr = split(//, $snmpV3AuthParms[4]);
               if( scalar(@pwdAr) > 7 ) {
                   if ((uc($snmpV3AuthParms[3]) eq 'DES') or (uc($snmpV3AuthParms[3]) eq 'AES') or ($snmpV3AuthParms[3] eq "")) {
                     $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3AuthParms[3] eq "" ? "DES" : uc($snmpV3AuthParms[3]);
                     $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3AuthParms[4];
                   } else {
                     print STDERR "SNMPv3 privacy protocol can be either AES or DES.\n";
                     Usage();
                   }
               }else {
                 print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
                 Usage();
               } 
           }
       }
     }
### END
  } else {
    if (!defined($communityString)) {
      $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
      $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
      $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
      if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
         $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
      }

      if (defined($toolData{SNMPV3_PRIV_PROTOCOL})) {
       $params{SNMP}{V3}{PRIV}{PROTO} = $toolData{SNMPV3_PRIV_PROTOCOL};
       $params{SNMP}{V3}{PRIV}{PASSWORD} = $toolData{SNMPV3_PRIV_PASSWORD};
      }
    }
  }
#added till here for SNMPv3 auth and priv changes by Indhu on 09/07/2009-BDFam47286.


  if (($toolData{TL1PWD} eq "") || ($toolData{TL1USER} eq "")) {
    print STDERR "No username or password defined in param.cfg.  Run Config.pl!!!\n";
    exit PARAM_CFG_NOT_OK_CODE;
  }

  if (! (defined($params{SNMP}{V1}{NT_COMMUNITY})) || ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) {
   if (! (defined($params{SNMP}{V3}{AUTH}{USERNAME})) || ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    if (! (defined($params{SNMP}{V2}{NT_COMMUNITY})) || ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) {
      print STDERR "SNMP must be configure to either use v1 , v2 or v3.  Run Config.pl and configure one of the options.\n";
    exit PARAM_CFG_NOT_OK_CODE;
   }
  }
  }
  $params{BUFSIZE} = 5 * $toolData{BUFSIZE};  
  $params{BUFSIZE} = 150 if ($params{BUFSIZE} > 150);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes

  $params{TELNETTIMEOUT_SCHK} = $toolData{TELNETTIMEOUT_SCHK};

  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'}); # BDFam50346
  $params{NOPING} = 1 if defined($noPing);                                                    # BDFam50346
  
  return \%params;
}

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Octopus
                       OntDlAct7302HiCap
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}


=head2 PrintOptionsSummary($$$$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of a problem is required as it indicates what actions will be undertaken
by the script.

=end html

=cut

sub PrintOptionsSummary($$) {
  my($argLogFH, $logOnScreen) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Release" , $params->{RELEASE} ) ) if defined($params->{RELEASE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ontTypes" , $params->{FULLONTTYPES} ) ) if defined($params->{FULLONTTYPES});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Failed ONT list file" , $params->{FAILONTFILE} ) ) if defined($params->{FAILONTFILE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Warning Threshold" , $params->{WARNTHRESHOLD} ) ) if defined($params->{WARNTHRESHOLD});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation Mode" , $params->{ONTDOWNLOADMODE} ) ) if defined($params->{ONTDOWNLOADMODE});
  if(defined($params->{OPMODE})){
    if($params->{OPMODE} == ONT_DL) {
        Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation(s) " , "Download Only" ) );
    } elsif ($params->{OPMODE} == ONT_ACT ) {
        Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation(s) " , "Activation Only" ) );
    } elsif ($params->{OPMODE} == ONT_DELYACT) {
        Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation(s) " , "Delayed Activation" ) );
    } else {
        Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Operation(s) " , "Download and Activation" ) );
    }
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Telnet session timeout in seconds" , $params->{TELNETTIMEOUT_SCHK} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ( !defined $params->{SNMP}{V3}{AUTH}{USERNAME} or $params->{SNMP}{V3}{AUTH}{USERNAME} =~ /^\s*$/) {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , "'$params->{SNMP}{V3}{AUTH}{USERNAME}'") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "CLI buffer size" , ($params->{BUFSIZE} / 1048576) ) );

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
}

#------------------------------------------------------------------------------
#-- Do the job!!!
#------------------------------------------------------------------------------

=head2 HandleONT($$$$$)

=begin html
The main body of the script defining the complete SW download and activate process.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the parent process.  Progress and failure messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by this child process.
      Every array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
  <tr><td>config</td><td>a pointer to the hash containing all configuration data</td></tr>
</table>

=end html

=cut

sub handleONT($$$$) {
  #$execLogFH is taken from PBMT, fh is file handle for parent file
  my ($execLogFH, $isams, $sessionCount, $config) = @_;
  my $isam;
  my $ret;
  my ($time1, $time2);
  my $startTime;
  my $nodeLogFh;                               # Filehandle of node specific log
  my $nodeLogFile;                             # Name of node specific log
  my $nodeStatusLog;                           # Filehandle of the node status result file
  my $dirName = dirname($0);
  my $ntType;
  my $isamNode;

  my $statFile;
  my $progFile;

  #Below arguments are taken from the PBMT
  my $newLog = "$params->{LOGFILE}_$sessionCount";    # Name of child logfile
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $fxrData; 
  my $dir;

  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolDirName($dirName);

  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetConfigData($config);


   ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
   if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );
  }

  PrintOptionsSummary($nodeLogFH, undef);


my $isamCount = -1;
  foreach $isam (@{$isams}) {
    $isamCount++;
    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1];
    FamilyCommon::SetFailedNode($isamNode);
    
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    
    SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
    IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});	
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});	
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);
    IsamTL1::ResetGlobals();
    IsamCli::ResetGlobals();
	OntDlAct7302HiCap::ResetGlobals();
	OntDlAct7302HiCapCLI::ResetGlobals();

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

  
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

   if ( defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef);
    }
    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));

    #status files - result.txt logs SUCCESS or Error code, incase of failure
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
    }
  
  $nodeStatusLog->autoflush(1);
  FamilyCommon::SetNodeStatusLog($nodeStatusLog);

  #progress file - % of download and activated successful ONTs
  if (!open($progFile, ">$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}")) {
    print STDERR "Cannot open $toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE} for writing: $!\n";
    exit CREATE_FILE_FAIL_CODE;
  }
  
  OntDlAct7302HiCap::SetProgressFail($progFile);
  OntDlAct7302HiCapCLI::SetProgressFail($progFile);
  truncate($progFile, 0 );
  print $progFile "0% Complete\n";
  $progFile->autoflush(1);

  #To log list of failed ONTs
  if (!open($statFile, ">$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}")) {
    print STDERR "Cannot open $toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE} for writing: $!\n";
    exit CREATE_FILE_FAIL_CODE;
  }

  $statFile->autoflush(1);  
  OntDlAct7302HiCap::SetFailontlistFile($statFile);
  OntDlAct7302HiCapCLI::SetFailontlistFile($statFile);
    
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::CloseTL1Session2Close();

#    if ($toolData{RESULTSPACE} =~ /^\//) {
#    Logger::LogTrace($execLogFH, "PBMT script logged in file     : '$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}'\n", 1);
#    Logger::LogTrace($execLogFH, "Writing result to              : '$toolData{RESULTSPACE}/$isamNode/$params->{STATUSFILE}'\n", 1);
#    } else {
#    Logger::LogTrace($execLogFH, sprintf("PBMT script logged in file     : '%s/$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}'\n", getcwd()), 1);
#    Logger::LogTrace($execLogFH, sprintf("Writing result to              : '%s/$toolData{RESULTSPACE}/$isamNode/$STATUSFILE'\n", getcwd()), 1);
#    }
    
    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }
 
    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});   # send SNMP debug info to the child logfile first
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
	      FamilyCommon::IncrErrorCount( "FAILED : ".SNMPComm::GetError(),
	        		                    EC_SNMP_SESS_ESTB_ERR, ": ".SNMPComm::GetError(), 1);
	      next;
    }
    my $mibVersion = SNMPComm::GetValue( 'MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::IncrErrorCount(sprintf("  FAILED to obtain MIB version: %s\n", SNMPComm::GetError() ),
	       		   EC_SNMP_NO_GET, sprintf(": %s", SNMPComm::GetError() ), 1);
         next;
    }
    
    Logger::LogTrace($nodeLogFH, "Attempt to get file transfer data...\n");
    $fxrData = ToolCommon::GetFxrSettings($session, $toolData{RETRIES});
    if (!defined($fxrData)) {
      Logger::LogError($nodeLogFH, "obtained empty  file transfer protocol details\n");
      FamilyCommon::IncrErrorCount("\n",
                                  EC_GET_FTP_FAILED,
                                  undef,
                                  1);
      next;
    } else {
      if($fxrData->{PROTOCOL} != TFTP_FXR) {
        $params->{NOTFTP}=1;
	    Logger::LogTrace($nodeLogFH, "  OK\n");
      }
    }
    umask(0) if ( $params->{UMASK_CHMOD} );
    my $tmpDir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}";

    if (! -d $tmpDir) {
      if (!mkdir( $tmpDir, 0777)) {
        Logger::LogError($execLogFH, "Cannot create directory $tmpDir: $!\n");
        exit CREATE_DIR_FAILED_CODE;
      } else {
        Logger::LogDebug($execLogFH, "Logging to $tmpDir\n");
      }
    }

    my $dir = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode";
    
    #MULTI support, multiple session, multiple ISAM are inherited from PBMT
    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $newLog, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};


    FamilyCommon::SetNeStorage($dir);

    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile = sprintf("ontDownloadandActivate_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        PrintOptionsSummary($nodeLogFH, undef);
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --sepLog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    }

    if ($toolData{RESULTSPACE} =~ /^\//) {
         Logger::LogInfo($nodeLogFH, "Ont download activate progress  to : '$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}'\n");
         Logger::LogInfo($nodeLogFH, "Ont download activate status to    : '$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}'\n");
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Ont download activate progress to : '%s/$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}'\n",getcwd()) );
      Logger::LogInfo($nodeLogFH, sprintf("Ont download activate status to   : '%s/$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}'\n",getcwd()));
    }

   # 1 -> ONT Download , 2 -> ONT DownloadandActivate , 3 -> Download and Activate and 4 -> Delayed Activate 
   # params{OPMODE} will have this value now in converged platform 

    Logger::LogTrace($nodeLogFH, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      next if( FamilyCommon::ReportStatus( EC_GET_ACC_BITS_FAILED, ToolCommon::GetError(), undef ) );
    } else {
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

   if( ! ToolCommon::TL1ViaUDP($accBits, $params->{DEBUG}) && ! ToolCommon::TL1ViaSSH($accBits, $params->{DEBUG}) )
   {
        if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
               $params->{USESSH} = 1;
               Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
        }
        $params->{PHYSICAL_HOST_IP} = ToolCommon::GetIPAddress($isam->[0]);
        $ret = OntDlAct7302HiCapCLI::ProcessONT($isam);
   } else {
        $ret = OntDlAct7302HiCap::ProcessONT($isam);
   }

   FamilyCommon::CloseSnmpSession2Close();
   FamilyCommon::CloseTL1Session2Close();

   my $endTime = time();
   ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );

   
   Logger::LogInfo($execLogFH, sprintf(" ONT download and activate Completed in %d seconds", $endTime - $startTime));


   if (defined($nodeStatusLog) and ($ret)) {
      FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    }

   if ($params->{SEPLOG}) {
       FamilyCommon::CloseChildLogFile();
       LogCloseFile($nodeLogFH);
   }
   FamilyCommon::CloseChildLogFile() if ($params->{SEPLOG});

    close $progFile if (defined($progFile));
    $progFile = undef;

    close $statFile if (defined($statFile));
    $statFile = undef;
  }

  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (!defined($params->{ID})) {
    if (! $params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile(); 
        LogCloseFile($nodeLogFH);
    }
  }
  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);
}

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file.

=end html

=cut

sub AdjustToolVars($) {
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{FTPDIR} = $config->{'FTPDIR'} if defined($config->{'FTPDIR'});
  $toolData{FTPTIMEOUT} = $config->{'FTPTIMEOUT'} if defined($config->{FTPTIMEOUT});
  $toolData{MAXSESSIONS} = $config->{'MAXTFTPSESSION'} if defined($config->{'MAXTFTPSESSION'});
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT}); 
  $toolData{TELNETTIMEOUT_SCHK} = $config->{'TELNETTIMEOUT_SCHK'} if defined($config->{'TELNETTIMEOUT_SCHK'}); 
  $toolData{RETRIES} = $config->{'RETRIES'} if defined($config->{'RETRIES'});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{RESULTSPACE} =~ s/\/$//;
  $toolData{TL1UDPPORT} = $config->{TL1UDPPORT} if defined($config->{TL1UDPPORT});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
                                                                                 ($config->{NT_COMMUNITY} ne ""));

  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL}
                                    if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER)
                                    if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER)
                                    if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER)
                                    if defined($config->{SNMPV3_PRIV_PASSWORD});

  $toolData{TL1PWD} =  $crypt->decrypt($config->{'DEFAULTTL1PWD'}, CYPHER)
             if (defined($config->{'DEFAULTTL1PWD'}) && ($config->{'DEFAULTTL1PWD'} ne ""));

  $toolData{TL1USER} =  $crypt->decrypt($config->{'DEFAULTTL1USER'}, CYPHER)
             if (defined($config->{'DEFAULTTL1USER'}) && ($config->{'DEFAULTTL1USER'} ne ""));

  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
                                                                                 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
                                                                                ($config->{'DEFAULTCLIPWD'} ne ""));

  $toolData{ONT_DOWNLOAD_TIMEOUT} = $config->{'ONT_DOWNLOAD_TIMEOUT'} if defined($config->{'ONT_DOWNLOAD_TIMEOUT'});
  $toolData{ONT_ACTIVATE_TIMEOUT} = $config->{'ONT_ACTIVATE_TIMEOUT'} if defined($config->{'ONT_ACTIVATE_TIMEOUT'});
  $toolData{ONT_DWLD_PROGRESS_TIMEOUT} = $config->{'ONT_DWLD_PROGRESS_TIMEOUT'} if defined($config->{'ONT_DWLD_PROGRESS_TIMEOUT'});
  $toolData{NTDBTOOLDIR} = $config->{'NTDBTOOLDIR'} if defined($config->{'NTDBTOOLDIR'}); # BDFam53774 
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $STATUSFILE = $config->{ONTDWLDACT_STATUS_FILE} if defined($config->{ONTDWLDACT_STATUS_FILE});
  $toolData{ONTRELDIR} = $config->{'ONTRELDIR'} if defined($config->{'ONTRELDIR'});
  $toolData{ONTRELMAPFILE} = $config->{'ONTRELMAPFILE'} if defined($config->{'ONTRELMAPFILE'});
  $toolData{BUFSIZE} = $config->{'BUFSIZE'} if defined($config->{'BUFSIZE'});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});


}

#==============================================================================
#== Main program
#==============================================================================
my @pids;
my $pid;
my $warnCount = 0;

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written into the log files for later analysis (no warnings should be
       generated during the run of program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  Logger::LogTrace($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					    defined($ownPackage) ? $ownPackage : "undefined",
					    defined($ownFileName) ? $ownFileName: "undefined",
					    defined($ownLine) ? $ownLine : "undefined",
					    defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			   defined($ownPackage) ? $ownPackage : "undefined",
			   defined($ownFileName) ? $ownFileName: "undefined",
			   defined($ownLine) ? $ownLine : "undefined",
			   defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($warnLogFile, "  Aborting children...\n") if defined($warnLogFile);
    foreach my $pid (@pids) {
      if ($pid > 0) {
	print STDERR "Killing process $pid...\n";
	Logger::LogTrace($warnLogFile, "Killing process $pid...\n") if defined($warnLogFile);
	kill($pid);
      }
    }
  }
  my $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
  if (defined($nodeStatusLog)) {
    print $nodeStatusLog sprintf("%s '%s'\n", ErrorCodes::ExitString($exitCode), $signal);
    FamilyCommon::CloseNodeStatusLog();
  }
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();

  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($warnLogFile, "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr \n") if (defined($warnLogFile));
    print STDERR "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr";
    $warnCount++;
  }
};
$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};
$SIG{"PIPE"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($warnLogFile);
    Logger::LogTrace($warnLogFile, "SIGPIPE received (ignored): $msgStr") if defined($warnLogFile);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
	print STDERR sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
			     defined($ownPackage) ? $ownPackage : "undefined",
			     defined($ownFileName) ? $ownFileName: "undefined",
			     defined($ownLine) ? $ownLine : "undefined",
			     defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
	Logger::LogTrace($warnLogFile, sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
					      defined($ownPackage) ? $ownPackage : "undefined",
					      defined($ownFileName) ? $ownFileName: "undefined",
					      defined($ownLine) ? $ownLine : "undefined",
					      defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
	$i++;
      } else {
	$framePresent = 0;
      }
    }
    CleanUp("PIPE", EXIT_DIED_CODE);
  }
};

#-- Read parameter config file and adjust tool defaults if needed -- BB--

my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End modif BB

#-- Handle command line
$params = ParseCommandLine(\%config);

$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1);
Utils::SetDebug($execLogFH) if ($params->{DEBUG}); # 1.2.2

#-- Get a list of all Isams, This is not used, since we are supporting only one ISAM. But we planned to maintain this code for future.
my $isamsRef;
if (defined( $params->{ISAM})) {
    if ($params->{ISAM} =~ /^\d+(.\d+)*$/) {
     if (ToolCommon::ValidIP($params->{ISAM}))
     {
       my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
       if (! defined($name)) {
        $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
       } else {
       $isamsRef = [ [$params->{ISAM}, $name] ];
      }
    }
    else
    {
      print STDERR "ERROR: Invalid ISAM IP\n";
      exit INVALID_ISAM_IP_CODE;
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = undef;

    if(defined($addr[4])){
      @ip = unpack( "C4", $addr[4]);

        my $ipAddr = join( '.', @ip );
        $isamsRef = [ [$ipAddr, $params->{ISAM}]];
    } else {
    print STDERR "ERROR: Invalid Hostname\n";
      exit UNABLE_TO_RESOLVE_HOST_NAME_CODE;
    }
  }
}
else #taken from pbmt
{
      #Read the list of NE from the host file
      Logger::LogInfo($execLogFH, "Reading Hostfile...\n");
      $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX}); 
 }

 #below check is taken from pbmt 
exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

#umask(0); # Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );


my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      handleONT($execLogFH, \@isamPart, $sessionCount, \%config); 
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      Logger::LogInfo($execLogFH, sprintf("Session %d: exit code: '%d' (%s), signal: '%d'\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal) );
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
#end - pbmt code
