#!/usr/bin/perl -X

=head1 ISAM OSWP Rollback

=begin html
This script allows us to rollback the passive OSWP of the network element.
=end html

=cut

#------------------------------------------------------------------------------
#-- File          :rollback.pl
#-- Date          :10-07-2014
#-- Company:      Alcatel-Lucent
#-- Author        :K Ravi
#------------------------------------------------------------------------------
#-- Function:     Script to rollback the passive OSWP of the network element
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    All rights reserved. Passing on and copying of this document, 
#--               use and communication Of its contents not permitted without 
#--               written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- Requirements: 
#--               RCR - ALU01965295
#------------------------------------------------------------------------------
#-- History:
#--     1.0.1  :    10-07-2014 - First Implementation of Rollback.pl (ALU2009355)
#--     1.0.2  :    26-09-2014 - Rollback for BellCanada (ALU02014478)
#--     1.0.3  :    28-11-2014 - rollback changes for EtsiNtt family
#--     1.0.4  :    02-12-2014 - rollback changes for --swpfile option in ANSI
#--     1.0.5  :    13-02-2015 - ALU02046572
#--     1.0.6  :    25-05-2015 - ALU02060588
#--     1.0.7  :    10-06-2015 - ALU02083077
#--     1.0.8  :    04-08-2015 - ALU02104915
#--     1.0.9  :    28-10-2015 - ALU02139820
#--     1.0.10 :    02-11-2015 - ALU02142016
#--		1.0.11 :    14-12-2015 - ALU02156544
#--     1.0.12 :    30-12-2015 - ALU02160793
#--     1.0.13 :    16-03-2016 - ALU02190558
#--     1.0.14 :    2016/07/15 - ALU02237966
#--     1.0.15 :    2016/09/28 - ALU02278259
#--     1.0.16 :    2016/11/21 - ALU02285181
#--     1.1.0  :    2018/06/20 : ALU02460863 & ALU02471697

#------------------------------------------------------------------------------

=head1 VERSION

=begin html
Current version: "rollback.pl - 1.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "rollback.pl - 1.1.0 -- 2018/06/20";
#==============================================================================

=head1 ISAM OLT Rollback 

=begin html
This script is used to perform the NE rollback, which activates the passive OSWP 
software of the given NE. ONT rollback will be done automatically by OLT.

The operations of the script are influenced by:
<ul>
  <li>Command line options</li>
  <li>A Environment variable configuration file</li>
</ul>

The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h"</li>
      <li>Socket</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut


#------------------------------------------------------------------------------
#-- Inclusions
#------------------------------------------------------------------------------
use strict;
require 5.6.0;
use warnings 'all';

use FileHandle;
use File::stat;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Text::ParseWords;

#------------------------------------------------------------------------------
#-- Find the lib dir, Add the lib path to environment variable
#------------------------------------------------------------------------------
#use Cwd;
use Cwd qw( realpath getcwd);
use File::Basename;
use Config; 

my $scriptDir; 
BEGIN { 
$scriptDir = dirname(realpath($0)); 
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 

use Net::SNMP;

use Logger qw(:ALL);
use IsamCli;
use IsamTL1;
use Crypt::Lite;
use Utils;
use Capability;
use PbmtConfig;
use ErrorCodes qw(:EXIT_CODES :RLBK);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
#use Logger;
use RollbackIsam;
use ToolCommon();
use SNMPComm;
use version qw(:ALL);

#------------------------------------------------------------------------------
#-- Prototypes
#------------------------------------------------------------------------------
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$$);
sub RollbackIsams($$$$);
sub AdjustToolVars($);
sub CleanUp($;$);

#------------------------------------------------------------------------------
#-- Global Declarations
#------------------------------------------------------------------------------
my $ID_FILENAME = "RLBK";
my $STATUSFILE  = "result.txt";


#-----
my %toolData = (
                "SHUBSWCHECK" => "shubSwCheck.sh",
                "HOSTSFILE" => "/etc/hosts",
                "TFTPDIR" => "/var/opt/aws/equipment",
                "TFTPWORKDIR" => "tmp",
                "MAXSESSIONS" => 5,
                "USERNAME" => "",
                "PASSWORD" => "",
                "PASSWORD2" => "",
                "RETRIES" => 5,
                "ACTPINGWAIT" => 10,
                "NODEPREFIX" => "AS-",
                "TFTPTIMEOUT" => 5,
                "TELNETTIMEOUT" => 10,
				"EMERG_ALRM_POLL_TIME" => 120,
				"MAXWAITTIME" => 1800,
                "NO_IP" => "0.0.0.0",
                "NT_COMMUNITY" => "public",
);
#-----
my $sessionCount = 0;
my @pids;
my $pid;
my $warnCount = 0;
my $logExecOrNode = 0;

#------------------------------------------------------------------------------
#-- Sub routines 
#------------------------------------------------------------------------------

=head1 USAGE
 

=head2 How to run?
 

=begin html

Invoke the script by running it from a perl command such as:<br>

<br>

<tt>[perl] rollback.pl --isam &lt;ip&gt; --log &lt;filename&gt; \ <br>
                    [--cliusername &lt;user&gt;] [--clipassword &lt;password&gt;] [--noname]  \ <br>
                    [--noping] [--v] [--debug] [--comm &lt;SNMP community string&gt;] \ <br>
                    [--v3Auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;[privProtocol&gt;:&lt;password&gt;]] </tt>
=end html 

=head2 Command options

 =begin html

<table>

  <tr><td>isam</td><td>ip or the host name of the NE</td></tr>
  <tr><td>log</td><td>name of the logfile</td></tr>
  <tr><td>cliusername</td><td>CLI username</td></tr>
  <tr><td>clipassword</td><td>CLI password</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory (typically used by LRM)</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed.</td></tr>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>debug</td><td>enables additional debugging traces</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>continue</td><td>continue SW download for other ISAMs in stead of exiting the current child process</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                             <li>All fields are mandatory when option is used.</li>
                         </ul><br>
  <tr><td>v3priv</td><td>optional setting when using SNMPv3 privacy protocol providing protocol and password<br>
                         <ul><li>Default value for the Privacy protocol is DES.</li>
                         </ul><br>e.g. --v3Auth user:password:authprotocol.Incase when Auth User is configured.</td></tr>
                         </ul><br>e.g. --v3Auth user:password:authprotocol[privproto:password].When AuthPrivUser is configured.</td></tr>
</table> 

=end html

=head1 FUNCTIONS

=head2 Usage()

=begin html
Called in case of incorrect usage.  Prints all command line options and their explanations and exists with exit code "command-line incomplete".
 
=end html

=cut
 
sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <ip> | --hosts <file> --logfile <file> | --id <id>   \\\n";
  print STDERR "       --swp <path-to-oswp-descriptor>  |  --swpfile <file containing list of source releases>     \\\n";
  print STDERR "       [--cliusername <username>] [--clipassword <password>]  [--clipassword2 <password>] \\\n";
  print STDERR "       [--noname] [--noping] [--v] [--debug] [--comm <SNMP community string>]  [--v2] \\ \n";
  print STDERR "       [--seplog | --noseplog] [--continue] [--sessions <nr>] \\ \n";
  print STDERR "       [--v3auth <user>:<password>:<authProtocol>] [-v3priv <privProtocol>:<password>] \n";
  print STDERR "\n";
  print STDERR "       Params:\n";
  print STDERR "         isam               : IP or the hostname of the NE.\n";
  print STDERR "         id                 : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                              The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         swp                : path to oswp descriptor file (either relative or absolute)\n";
  print STDERR "                              (relative to TFTP root - $toolData{TFTPDIR})\n";
  print STDERR "         swpfile            : file containing list of source releases to rollback (option provided only for ANSI) \n";
  print STDERR "         sessions           : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         cliusername        : ISAM CLI username\n";
  print STDERR "         clipassword        : ISAM CLI password\n";
  print STDERR "         clipassword2       : Second ISAM CLI password (option to use a second password if first password fails).\n";
  print STDERR "                              The second password option will be used only when CLI over SSH is configured.\n";
  print STDERR "         noname             : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         noping             : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "         seplog             : if set create a log file per node, otherwise logging gets appended in the child log file\n";
  print STDERR "         noseplog           : keep all NE logging in a single child log file (default is a single logfile per NE)\n";
  print STDERR "         continue           : continue rollback for other ISAMs in stead of exiting the current child process\n";
  print STDERR "         v                  : print version and exit\n";
  print STDERR "         debug              : enables additional debugging traces\n";
  print STDERR "         SNMP settings      :\n";
  print STDERR "         comm               : NT SNMP community string when using SNMPv1.  Default is \"public\"\n";
  print STDERR "         v2                 : Use SNMPv2c\n";
  print STDERR "         v3auth             : username, password and authentication protocol when using SNMPv3\n";
  print STDERR "                              The password should be at least 8 characters long.\n";
  print STDERR "                              Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                              All fields are mandatory when an option is used.\n";
  print STDERR "                              e.g. --v3auth user:password:: will result in MD5 as protocol.\n";
  print STDERR "         v3priv             : optional setting when using SNMPv3 privacy protocol providing protocol and password.\n";
  print STDERR "                              Privacy protocol is DES, 3DES or AES, DES is default protocol.\n";
  exit CMD_INCOMPL_CODE;
}

#------------------------------------------------------------------------------
#-- sub ParseCommandLine($); # Parse the received command line parameters
#------------------------------------------------------------------------------
sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $swp, $swpFile, $debug, $logFile, $id, $hosts, $cliUserName, $cliPassword, $cliPassword2, $sessions, $version, $sepLog, $noSepLog, 
      $continue, $noName, $communityString, $snmpV2, $noPing, $noName, $v3Auth, $v3Priv );
  my ($id);
  my $regExp;
  my %params;
  my $dbVersion; 
  my $crypt = Crypt::Lite->new('debug' => 0);
  my $dirName = getcwd();

  my $options = GetOptions('isam=s', \$isam,
                           'debug', \$debug,
                           'id=s', \$id,
                           'swp=s', \$swp,
                           'swpfile=s', \$swpFile,
                           'sessions=i', \$sessions,
                           'hosts=s', \$hosts,
                           'username|cliusername=s', \$cliUserName,
                           'password|clipassword=s', \$cliPassword,
                           'clipassword2=s', \$cliPassword2,     # undocumented, for 7354
                           'logfile=s', \$logFile,
                           'v', \$version,
                           'v2', \$snmpV2,
                           'seplog', \$sepLog,
                           'noseplog', \$noSepLog,
                           'continue', \$continue,
                           'comm=s', \$communityString,
                           'noping', \$noPing,
                           'noname', \$noName,
                           'v3auth=s', \$v3Auth,
                           'v3priv=s', \$v3Priv); 

  #-- check options
  if (! $options) {
    Usage();
  }
  
  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }

  if (defined($version)) {
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }
 
  #-- Check for mandatory parameters  
  #if (!defined($isam)) {      
  #  print STDERR "ERROR: Target isam ip or hostname should be specified\n";
  #  Usage();
  #}

  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }

  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }

  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }

#  if (!defined($swp) && !defined($swpFile)) {
#      print STDERR "ERROR: Either --swp or --swpFile should be specified\n";
#      Usage();
#  }

  if (!defined($swp)) {
    if (!defined($swpFile)) {
      print STDERR "ERROR: Either --swp or --swpFile should be specified\n";
      Usage();
    }
  } else {
    if (defined($swpFile)) {
      print STDERR "ERROR: --swp can't be used in combination with --swpFile\n";
      Usage();
    }
  }

  if(defined($swpFile)) {
   if(! -r $swpFile) {
     print STDERR "ERROR: accessing swp File '$swpFile' failed: $!\n";
     Usage();
   } else {
     ###my $swpFileData = ToolCommon::ReadReleaseFile($swpFile);
     $params{SWPREL}= ToolCommon::ReadReleaseFile($swpFile);
   }
  }

  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }
  if (defined($cliUserName)) {
    $params{USERNAME} = $cliUserName;
  } else {
    $params{USERNAME} = $toolData{USERNAME};
  }
  if (defined($cliPassword)) {
    $params{PASSWORD} = $cliPassword;
  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }
  if (defined($cliPassword2)) {
    $params{PASSWORD2} = $cliPassword2;
  } else {
    $params{PASSWORD2} = ($toolData{PASSWORD2} ne "") ? $toolData{PASSWORD2} : undef ;
  }

  #-- fill hash, set defaults if needed
  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
        "($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }

  if (defined($communityString) && defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  if (defined($communityString) && defined($v3Priv)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }

  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = 0;
  if( defined($debug)) {
      LogSetDebug();
      $params{DEBUG} = 1 
  }
  #$params{OSWP} = $swp if (defined($swp));
  $params{OSWP} = defined($swp) ? $swp : undef; 

#----
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
##  $params{RESULTFILE} = $id . "_" . $ID_FILENAME . ".result";
    $params{PROGFILE} = $id . "_" . $ID_FILENAME . ".update";
  } else {
    $params{LOGFILE} = $logFile;
    if ($params{LOGFILE} =~ /([^\/]+)\/(.*)/) {
      my $logdir = $1;
      if (! -d $logdir) {
        if (!mkdir( $logdir, 0777)) {
          print STDERR "Cannot create directory '$logdir': $!\n";
          exit CREATE_DIR_FAILED_CODE;
        }
      }
    }
  }
 
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
 
  #  $params{PHYSICAL_HOST_IP}=Utils::getLocalHostIP(); 
  #$params{PHYSICAL_HOST_IP} = $config->{PHYSICAL_HOST_IP} ; ##if (defined($config->{PHYSICAL_HOST_IP}) &&
                                                           ##   ($config->{PHYSICAL_HOST_IP} ne "0.0.0.0"));
  $params{NONAME} = defined($noName) ? 1 : 0;
  if (!defined($snmpV2) && !(defined($communityString))) { #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  }

  if (defined($snmpV2)) {
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
  }

  if (defined($v3Auth)) {
     my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    if (scalar @snmpV3AuthParms >= 2) {
      my @pwdAr = split(//, $snmpV3AuthParms[1]);
      $snmpV3AuthParms[2] = "" if (scalar @snmpV3AuthParms == 2);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA512|SHA256|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
	  $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
	  $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
      if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
         $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
      }
	} else {
	  print STDERR "SNMPv3 authentication protocol can only be SHA or SHA512 or SHA256 or MD5.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 authentication password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
       if(defined($snmpV3AuthParms[0]) && ($snmpV3AuthParms[0] ne  "")) {
	  $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
       } else {
          print STDERR sprintf("SNMPv3 authentication requires at least 2 parameters: username, password and protocol, only %d provided\n",
			   scalar @snmpV3AuthParms);
          Usage();
       }
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
    $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
    $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
    if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
       $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
    }
  } #ALU01524313-indhu-15/12/2011
  }
  if (defined($v3Priv)) {  
     my @snmpV3PrivParms = quotewords(':', 0, $v3Priv);
    if( $v3Priv eq "" ) {
        $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
        $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    } elsif (scalar @snmpV3PrivParms == 2) {
      my @pwdAr = split(//, $snmpV3PrivParms[1]);
      if (scalar @pwdAr > 7) {
	if ((uc($snmpV3PrivParms[0]) eq 'DES') or (uc($snmpV3PrivParms[0]) eq '3DES') or
	    (uc($snmpV3PrivParms[0]) eq 'AES') or ($snmpV3PrivParms[0] eq "")) {
	  $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3PrivParms[0] eq "" ? "DES" : uc($snmpV3PrivParms[0]);
	  $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3PrivParms[1];
	} else {
	  print STDERR "SNMPv3 privacy protocol can only be DES, 3DES or AES.\n";
	  Usage();
	}
      } else {
	print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
	Usage();
      }
    } else {
      print STDERR "SNMPv3 privacy requires  parameters: password and protocol\n";
      Usage();
    }
  } else {
  if (!defined($communityString)) { #ALU01524313-indhu-15/12/2011
    $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
    $params{SNMP}{V3}{PRIV}{PASSWORD} = defined($toolData{SNMPV3_PRIV_PASSWORD}) ? $toolData{SNMPV3_PRIV_PASSWORD} : "";
  } #ALU01524313-indhu-15/12/2011
  }

  if (($params{PASSWORD} eq "") || ($params{USERNAME} eq "")) {
    print STDERR "No username or password defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }
  if ((defined($params{SNMP}{V1}{NT_COMMUNITY}) and ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) and
      (defined($params{SNMP}{V2}{NT_COMMUNITY}) and ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) and
      ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2c or v3.  Run Config.pl and configure one of the options.\n";
    Usage();
  }

  $params{NOPING} = 0;
  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";

  $params{UPD_KNOWN_HOST_FILE} = 0;
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});

  $params{EMERG_CALL_FEATURE} = 0;
  $params{EMERG_CALL_FEATURE} = uc($config->{'EMERG_CALL_FEATURE'}) eq "Y" ? 1 : 0 if defined($config->{'EMERG_CALL_FEATURE'});

  $params{CONTINUE} = defined($continue) ? 1 : 0;

  #if(defined($communityString)){
  #  $params{SNMP}{V1}{NT_COMMUNITY} = $communityString;
  #} else {
  #  $params{SNMP}{V1}{NT_COMMUNITY} = $toolData{NT_COMMUNITY};
  #}
  #$params{SNMP_V1} = 1 if (defined($communityString)); #added by Indhu for SNMP ddts on 25/06/2009

  #if (!defined($params{SNMP}{V1}{NT_COMMUNITY}) and (!defined($params{SNMP}{V3}{AUTH}{USERNAME}))) {
  #  print STDERR "SNMP must be configure to either use v1 or v3.  Run Config.pl and configure one of the options.\n";
  #  exit(PARAM_CFG_NOT_OK_CODE);
  #}

  $params{PHYSICAL_HOST_IP} = $config->{PHYSICAL_HOST_IP} if (defined($config->{PHYSICAL_HOST_IP}) &&
                                                            ($config->{PHYSICAL_HOST_IP} ne $toolData{NO_IP}));

  
  # Timeout Changes OBR Added by Suresh on 29-June-2009
  if (!defined($toolData{MAXWAITTIME})) {
     $toolData{MAXWAITTIME} = 1800;  #This be default timeout when not specified in param.cfg
   }
   
   if (! ($toolData{MAXWAITTIME} =~ /^\d{1,}$/)) {
    print STDERR "Error: Invalid value specified for ROLLBACK_TIMEOUT : $toolData{MAXWAITTIME} \n";
    exit(PARAM_CFG_NOT_OK_CODE);
   }
 

  #Fix for BDFam50928 changes added by Suresh on 10/07/2009  
  # if (! ($toolData{NT_SYNC_TIMEOUT} =~ /^\d{1,}$/)) {
   #  print STDERR "Error: Invalid value specified for NT_SYNC_TIMEOUT : $toolData{NT_SYNC_TIMEOUT} \n";
    #exit(PARAM_CFG_NOT_OK_CODE);
   #}
 
  if (defined($swp)) { 
    my $baseName = basename($swp);
    $regExp = defined($toolData{REGEXP})? $toolData{REGEXP}:REGEXP;
    if (!($baseName =~ /$regExp/)) {
      print STDERR sprintf("Software package name (%s, read from $swp) not conforming to regexp (%s).  Check param.cfg!!!\n",
                           $swp, $regExp);
      exit PARAM_CFG_NOT_OK_CODE;
    }

if(!($baseName =~ /(AN|ZN|RN)/)) # AN,ZN and RN represents ANSI VARIANT
{
	(my $releaseBuild) = $baseName =~ /(\d+\.\d+)/;
	(my $release) = $releaseBuild =~ /(\d+)/;
	 if (!($release >= 51))
	  {
	    print STDERR "Error: Rollback supported only from R5.1 release \n";
		exit EC_NOT_SUPPORTED_LOWER_RELEASE;
	  } 
}	    

my $oswp2Consider = $swp;
  $params{NOTFTP} = 0;
  # First try the file as specfied (could be an absolute path when using ftp e.g.
  if (! -r $oswp2Consider) {
    # Next try the SWP relative to the login home directory
    $oswp2Consider = "$ENV{HOME}/$swp";
    $oswp2Consider =~ s/\/\//\//g;
    if (! -r $oswp2Consider) {
      # Finally check as tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $oswp2Consider =~ s/\/\//\//g;
      if (! -r  $oswp2Consider) {
	print STDERR "ERROR: Accessing OSWP '$oswp2Consider' failed: $!\n";
	Usage();
      } else {
	$params{OSWP_HOME} = $toolData{TFTPDIR};
      }
    } else {
      $params{OSWP_HOME} = $ENV{HOME};
      $params{NOTFTP} = 1;
    }
  } else {
    if (index($toolData{TFTPDIR}, $dirName) != -1) {
      # PBMT installed relative to TFTP directory so still tftp
      $oswp2Consider = "$toolData{TFTPDIR}/$swp";
      $params{OSWP_HOME} = $toolData{TFTPDIR};
    } else {
      $params{OSWP_HOME} = "";
      $params{NOTFTP} = 1;
    }
  }
  
  
 my $tftpInfo = Utils::FindTftpInfo(undef, $oswp2Consider);
    if (!defined($tftpInfo->{PRIMIP}) || ($tftpInfo->{PRIMIP} eq "")) {
      print STDERR sprintf("%s\n", ErrorCodes::ExitString(OSWP_FILE_NO_IP_CODE));
    exit OSWP_FILE_NO_IP_CODE;
  }
  $params{PRIMTFTPIP} = $tftpInfo->{PRIMIP};


  }


  return \%params;
}

#------------------------------------------------------------------------------
#-- sub PrintLibVersions($$);
#------------------------------------------------------------------------------

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut

sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli Octopus 
                       RollbackIsam
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}

#------------------------------------------------------------------------------
#-- sub PrintOptionsSummary($$$);
#------------------------------------------------------------------------------

=head2 PrintOptionsSummary($$$$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of 
a problem is required as it indicates what actions will be undertaken by the 
script.

=end html

=cut

sub PrintOptionsSummary($$$){
  my($argLogFH, $logOnScreen, $config) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "XDescriptor build type" , $params->{BTYPE} ) ) if defined($params->{BTYPE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "OSWP Descriptor file" , $params->{OSWP} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- SWP Name" , $params->{SWPNAME} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- DB Version" , $params->{DBVER_TGTREL} ) ) if defined($params->{DBVER_TGTREL});

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Transaction IP Address" , 
			       defined($params->{PHYSICAL_HOST_IP}) ? $params->{PHYSICAL_HOST_IP} :  $params->{PRIMTFTPIP}) );
            
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "FTP Details :-") );
  if (! $params->{NOTFTP}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "TFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Timeout in seconds" , $toolData{TFTPTIMEOUT}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- TFTP (HOME) Directory" , $toolData{TFTPDIR}) );
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "SFTP") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Server IP Address" , $params->{PRIMTFTPIP}) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ($params->{SNMP}{V3}{AUTH}{USERNAME} eq "") {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , $params->{SNMP}{V3}{AUTH}{USERNAME}) );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );
  if ($params->{CONTINUE}) {
    Logger::LogWish( $logMode, $argLogFH, "!!!WARNING!!!  USE AT OWN RISK!!!\n" );
    Logger::LogWish( $logMode, $argLogFH, "    Rollback process will continue even in case an ISAM is lost in the process of migration" );
    Logger::LogWish( $logMode, $argLogFH, "    Hit Ctrl-C now if you do not want to continue\n!!!WARNING!!!\n" );
  }

}


#------------------------------------------------------------------------------
#-- sub RollbackIsams($$$$);
#------------------------------------------------------------------------------

=head2 RollbackIsams($$$$$)

=begin html
The main body of the script defining the complete NE Rollback process.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the parent process.  Progress and failure messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated by 
    this child process.  Every array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
</table>

=end html

=cut

sub RollbackIsams($$$$){

  my ($execLogFH, $isams, $sessionCount, $config) = @_;
  my $isam;
  my $cli = undef;
  my $ret;
  my $startTime;
  my $nodeLogFile;
  my $nodeStatusLog;
  my $userName = $params->{MULTI} ? getpwuid($<) : "";       
  my $statusLogFh;
#  my $nodeLogFH = undef;
  my $newLog = "$params->{LOGFILE}_$sessionCount";
  my $dirName = dirname($0);
  my $ntType;
  my $thisTftpDir;
  my $fxrData;

  my $isamNode;
  my $dir;

  #my ($session, $snmpError);

  FamilyCommon::SetOwnPid($$);
  FamilyCommon::SetToolData(\%toolData);
  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetConfigData($config);

  Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Started as '%d'\n", $sessionCount, FamilyCommon::GetOwnPid()));
  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );

      FamilyCommon::SetFailedNode($isamNode);
      FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and
      FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
      IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
      IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
      Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
      ToolCommon::SetDebug($nodeLogFH);
      Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG}); 
      PrintOptionsSummary($nodeLogFH, undef, $config);
  }

  my $isamCount = -1;
  foreach $isam (@{$isams}) {
    $isamCount++;
    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1]; # Store for IncrErrorCount

    $startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

    if ( defined($params->{ID})) {
        $logExecOrNode = 0;
        $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
        Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
        FamilyCommon::SetParentLogFile($execLogFH);
    } elsif( $params->{SEPLOG} ) {
        $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$isamCount);
        $logExecOrNode = 1;
        Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
        PrintOptionsSummary($nodeLogFH, undef, $config);
    }

    FamilyCommon::SetFailedNode($isamNode);
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);  
    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG}); 
    $params->{USESSH} = 0; #for each isam the value is reset

    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
    }

    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);
    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::ClearCliSession2Close();
   ##  $execLogFH->autoflush(1);
   ##  FamilyCommon::SetWarnLogFile($execLogFH);
    STDOUT->autoflush(1);    
   ##  ToolCommon::SetDebug($execLogFH);

   #Fix for BDFam53454
   # my $prog;
   # if (! open($prog, ">$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}")) {
   #   print STDERR "Cannot open $toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE} for writing: $!\n";
   #   CORE::exit(CREATE_STAT_FILE_FAIL_CODE);
   # }

   #$prog->autoflush(1);
    #FamilyCommon::SetProgressFile($prog);
    
    ##  PrintOptionsSummary($execLogFH, 1);
    #PrintOptionsSummary($execLogFH, undef, $config);


    
    #Logger::LogTrace($nodeLogFH, "Child $sessionCount: Using the following file storage locations:\n", 1);
    #Logger::LogTrace($nodeLogFH, sprintf("   PBMT script logfiles stored in        '%s'\n", getcwd()), 1);
    #Logger::LogTrace($nodeLogFH, "   NE specific logging to                '$dir/$nodeLogFile'\n", 1) if $params->{SEPLOG};

   # if ($toolData{RESULTSPACE} =~ /^\//) {
   #   Logger::LogTrace($execLogFH, "Rollback script logfiles stored in : '$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}'\n", 1);
#  #    Logger::LogTrace($execLogFH, "Writing status result to       : '$toolData{RESULTSPACE}/$isamNode/$params->{RESULTFILE}'\n", 1);
   # } else {
   #   Logger::LogTrace($execLogFH, sprintf("Rollback script logfiles stored in : '%s/$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}'\n", getcwd()), 1);
## #   Logger::LogTrace($execLogFH, sprintf("Writing status result to       : '%s/$toolData{RESULTSPACE}/$isamNode/$params->{RESULTFILE}'\n", getcwd()), 1);
   # }
		    

    ###  Net::SNMP->snmp_debug(0x3e, $execLogFH) if ($params->{DEBUG});

    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }

    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
	      FamilyCommon::IncrErrorCount( "FAILED : ".SNMPComm::GetError(),
	        		                    EC_SNMP_SESS_ERR, ": ".SNMPComm::GetError(), 1);
	      next;
    }
    $snmpStatus = SNMPComm::MaximizeMsgSize();
    if( $snmpStatus ) {
         FamilyCommon::IncrErrorCount( "Failed to set maximum SNMP message size : ". SNMPComm::GetError(),
	       		             EC_SNMP_MAX_SIZE_FAIL, " : ".SNMPComm::GetError(), undef);
         next;
    }
    my $mibVersion = SNMPComm::GetValue( 'MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::IncrErrorCount(sprintf("  FAILED to obtain MIB version: %s\n", SNMPComm::GetError() ),
	       		   EC_SNMP_NO_GET, sprintf(": %s", SNMPComm::GetError() ), 1);
         next;
    }
    Logger::LogTrace($nodeLogFH, "  OK\n");


    Logger::LogTrace($nodeLogFH, "Prepare logging directories...\n");
    ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($session, $nodeLogFH, $userName, $isamNode);

    if (not defined($fxrData)) {
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
                                   EC_DETERMINE_LOG_FAILED,
                                   sprintf(": %s", FamilyCommon::GetError()),
                                   1);
      next;
    } else {
      $toolData{FXRDATA} = $fxrData;
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g;
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $newLog, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};

    FamilyCommon::SetNeStorage($dir);

    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile = sprintf("rlbk_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
        Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --seplog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    }

    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.  Only then we continue 
    $ntType = ToolCommon::DetermineNtType( );
    next if (not defined($ntType));

    Logger::LogTrace($nodeLogFH, "Check access security settings...\n");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      FamilyCommon::IncrErrorCount(ToolCommon::GetError(),
                                   EC_GET_ACC_BITS_FAILED,
                                   undef,
                                   1);
      next;
    } else {
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

   if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
	 $params->{USESSH} = 1;
	 Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
   }

    ToolCommon::GetSourceDetails();
    if( !defined $params->{SOURCE_OSWPNAME} ) {
         FamilyCommon::ReportStatus(EC_GET_OSWP_FAILED,ToolCommon::GetError(), undef );
         next;
    }

    $params->{'BOARD-FAMILY'} = "";
    Capability::LogNeFamily($ntType, $params->{SOURCE_DBVERSION});
    my $possibleBoardFamily = Capability::IsamHiCapEtsiFamily() . "|" .  Capability::IsamHiCapAnsiFamily(); 
    $possibleBoardFamily   .= Capability::IsamAnsiFamily();
    $possibleBoardFamily   .= "|". Capability::Isam7367_SXFamily() ."|". Capability::Isam7367Family();
    $possibleBoardFamily   .= "|". Capability::Isam7363Family();
    if ($params->{'BOARD-FAMILY'} !~ /^($possibleBoardFamily)$/ ) {
       FamilyCommon::ReportStatus( EC_NT_NOT_SUPPORTED, undef, 
                             sprintf("Operation not permitted for NT-Type '%s', DB-Version '%s' and Family '%s'\n",
                                      $ntType, $params->{DBVER_TGTREL}, $params->{'BOARD-FAMILY'} ) );
       next;
    } elsif ( $params->{'BOARD-FAMILY'} eq Capability::IsamHiCapEtsiFamily() ) {
        if($params->{SWPREL}) {
			Logger::LogError($nodeLogFH, "Option --swpfile not supported for HiCap family... \n");
			exit EC_SWPFILE_NOT_SUPPORTED;
        }
		Logger::LogInfo($nodeLogFH, "Checking whether precheck is run or not... \n");
		my $file = "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isamNode/RestoreDB/dm_complete.tar";
  		if (! -e $file) {
             my $logMesg = "The copy of DB - dm_complete.tar should be made available under $toolData{TFTPDIR}/$toolData{TFTPWORKDIR}/$isam     Node/RestoreDB for rollback purpose !!! \n";
             Logger::LogError($nodeLogFH, $logMesg );
             FamilyCommon::ReportStatus( EC_PRECHECK_NOTRUN, undef, $logMesg );
		     next;
	 	}
	    $ret =  RollbackIsam::Rollback($isam, $ntType);
    } else {
	    $ret =  RollbackIsam::Rollback($isam, $ntType);
    }
    Logger::LogTrace($nodeLogFH, "Done!\n");
    FamilyCommon::CloseSnmpSession2Close();

    $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
    if (defined($nodeStatusLog)) {
       #Logger::LogTrace($nodeStatusLog, "",sprintf("Performing Rollback..."));      
    }

    
    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );

    if($ret) {
       FamilyCommon::UpdateNodeStatusLog( SUCCESS_CODE );
    }

   if ($params->{SEPLOG}){
        FamilyCommon::CloseChildLogFile();
        LogCloseFile($nodeLogFH);
   }
  }

#-----
  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (!defined($params->{ID})) {
      if (!$params->{SEPLOG}){
           FamilyCommon::CloseChildLogFile();
           LogCloseFile($nodeLogFH);
      }
  }

  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);

#-----
  
  #Logger::LogTrace($execLogFH, sprintf("Rollback Script execution completed...\n"));
  
  #Logger::LogTrace($execLogFH, sprintf("Rollback OLT: %d error(s) occured for the following ISAMs, please verify %s\n",
  #				 FamilyCommon::GetErrorCount(), "$params->{LOGFILE}."), 1);


  #if(FamilyCommon::GetErrorCount() > 0){
  #        CORE::exit(FamilyCommon::GetOverAllError());
  #   }
      
  #    if(FamilyCommon::GetWarningCount() > 0) {
  #       CORE::exit(FamilyCommon::GetOverAllWarning());    
  #   }
     
  # CORE::exit(SUCCESS_CODE);
  
}

#------------------------------------------------------------------------------
#-- sub AdjustToolVars($);
#------------------------------------------------------------------------------
sub AdjustToolVars($){
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);

  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});
  $toolData{MAXSESSIONS} = $config->{'MAXTFTPSESSION'} if defined($config->{'MAXTFTPSESSION'});

  $toolData{FTPTIMEOUT} = $config->{'FTPTIMEOUT'} if defined($config->{FTPTIMEOUT});
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT}); 
  $toolData{EMERG_ALRM_POLL_TIME} = $config->{'EMERG_ALRM_POLL_TIME'} if defined($config->{EMERG_ALRM_POLL_TIME});            
  $toolData{RETRIES} = $config->{'RETRIES'} if defined($config->{'RETRIES'});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{RESULTSPACE} =~ s/\/$//;
  $toolData{TL1UDPPORT} = $config->{TL1UDPPORT} if defined($config->{TL1UDPPORT});
   #$toolData{MAXWAITTIME} = $config->{'WAITTIME'} if defined($config->{'WAITTIME'});

  $toolData{MAXWAITTIME} = $config->{'ROLLBACK_TIMEOUT'} if defined($config->{'ROLLBACK_TIMEOUT'});
  $toolData{NODEPREFIX} = $config->{'NODEPREFIX'} if defined($config->{'NODEPREFIX'}); # -- 1.0.3
  $toolData{TFTPTIMEOUT} = $config->{'TFTPTIMEOUT'} if defined($config->{TFTPTIMEOUT});       


  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
                                      ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
                                      ($config->{'DEFAULTCLIPWD'} ne ""));
  $toolData{PASSWORD2} = $crypt->decrypt($config->{'SECONDCLIPWD'}, CYPHER) if (defined($config->{'SECONDCLIPWD'}) &&
                                      ($config->{'SECONDCLIPWD'} ne ""));
  
  $toolData{ACTPINGWAIT} = $config->{ACT_PING_WAIT} if defined($config->{ACT_PING_WAIT});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
                                                                                 ($config->{NT_COMMUNITY} ne ""));
   $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} if defined($config->{SNMPV3_AUTH_PROTOCOL});
   $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) if defined($config->{SNMPV3_USERNAME});
   $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) if defined($config->{SNMPV3_AUTH_PASSWORD});
   $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
   $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) if defined($config->{SNMPV3_PRIV_PASSWORD});

  $STATUSFILE = $config->{RLBK_STATUS_FILE} if defined($config->{RLBK_STATUS_FILE});

  $toolData{NT_SYNC_TIMEOUT} = uc($config->{NT_SYNC_TIMEOUT}) if defined($config->{NT_SYNC_TIMEOUT});

  $toolData{MIGRATE_TIMEOUT} = defined($config->{'MIGRATE_TIMEOUT'})? $config->{'MIGRATE_TIMEOUT'}: "1200";
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});
}

#------------------------------------------------------------------------------
#-- Main Program 
#------------------------------------------------------------------------------

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process 
      or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written 
      into the log files for later analysis (no warnings should be generated 
      during the run of a GPON program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written 
      to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  Logger::LogTrace($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                                            defined($ownPackage) ? $ownPackage : "undefined",
                                            defined($ownFileName) ? $ownFileName: "undefined",
                                            defined($ownLine) ? $ownLine : "undefined",
                                            defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                           defined($ownPackage) ? $ownPackage : "undefined",
                           defined($ownFileName) ? $ownFileName: "undefined",
                           defined($ownLine) ? $ownLine : "undefined",
                           defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    print STDERR "  Aborting children...\n";
    Logger::LogTrace($warnLogFile, "  Aborting children...\n") if defined($warnLogFile);
    foreach my $pid (@pids) {
      if ($pid > 0) {
        print STDERR "Killing process $pid...\n";
        Logger::LogTrace($warnLogFile, "Killing process $pid...\n") if defined($warnLogFile);
        kill($pid);
      }
    }
  }
  FamilyCommon::UpdateNodeStatusLog( $exitCode, $signal );
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  
  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();

  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($warnLogFile, "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($warnLogFile));
    print STDERR "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr";
    $warnCount++;
  }
};

$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};
$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};

$SIG{"PIPE"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($warnLogFile);
    Logger::LogTrace($warnLogFile, "SIGPIPE received (ignored): $msgStr") if defined($warnLogFile);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
        print STDERR sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                             defined($ownPackage) ? $ownPackage : "undefined",
                             defined($ownFileName) ? $ownFileName: "undefined",
                             defined($ownLine) ? $ownLine : "undefined",
                             defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
        Logger::LogTrace($warnLogFile, sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                                              defined($ownPackage) ? $ownPackage : "undefined",
                                              defined($ownFileName) ? $ownFileName: "undefined",
                                              defined($ownLine) ? $ownLine : "undefined",
                                              defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
        $i++;
      } else {
        $framePresent = 0;
      }
    }
    CleanUp("PIPE", EXIT_DIED_CODE);
  }
};

#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);
$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");

#$config{"SSHOPTIONS"} = "Y";  # run rollback always in ssh mode to aviod max session issue in udp

if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End modif BB

#-- Handle command line
$params = ParseCommandLine(\%config);

STDOUT->autoflush(1);

$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1, \%config);

if ($params->{DEBUG}) {
   LogSetDebug();
   Utils::SetDebug($execLogFH);
}

#-- Get a list of all Isams
my $isamsRef;
if (defined( $params->{ISAM})) {
  if ($params->{ISAM} =~ /\d+\.\d+\.\d+\.\d+/) {
    my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
    if (! defined($name)) {
      $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
    } else {
      $isamsRef = [ [$params->{ISAM}, $name] ];
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = unpack( "C4", $addr[4]);
    my $ipAddr = join( '.', @ip );
    $isamsRef = [ [$ipAddr, $params->{ISAM}]];
  }
} else {
  Logger::LogInfo($execLogFH, "Reading Hostfile...\n");
  $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX});
}

exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));

#umask(0); # Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );

my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsPerSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsPerSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      RollbackIsams($execLogFH, \@isamPart, $sessionCount, \%config);
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
        if ($kid == $pids[$pidIdx]) {
          $childIdx = $pidIdx;
          $pids[$pidIdx] = 0;
          last;
        }
        $pidIdx++;
      }
      Logger::LogInfo($execLogFH, sprintf("Session %d: exit code: '%d' (%s), signal: '%d'\n",
                                        $childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal), 1);
      if ($exitCode > 9) {
        $newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
        $newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
     }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
