#!/usr/bin/perl -w

=head1 PBMT configuration

=begin html
This script allows you to stage the ont software to the NE.

=end html

=cut
#=============================================================================
#-- File          :    stageont.pl
#-- Date          :    11-03-2009
#-- Author        :    Palaniappan Ramasamy
#------------------------------------------------------------------------------
#-- Function      :    Script to download the ONTSWP from local machine to OLT 
#------------------------------------------------------------------------------
#-- Disclaimer    :    This script contains references to proprietary information
#--                    which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright     :    All rights reserved. Passing on and copying of this document, 
#--                    use and communication Of its contents not permitted without 
#--                    written authorization from Alcatel-Lucent
#------------------------------------------------------------------------------
#-- Requirements  : 
#--                   ANTms36033-071:
#--                   ANTms36033-072:
#--                   ANTms36033-073:
#--                   ANTms36033-074:
#--                   ANTms36033-091:
#------------------------------------------------------------------------------
#-- History       :
#--   1.0.1       :   11-03-2009 :  First Implementation of stageont.pl for GPON7342
#--   1.0.2       :   21-04-2009 :  DDTS tickets resolved - BDFam48335, BDFam48326, BDFam47022
#--   1.0.3       :   04-05-2009 :  DDTs tickets resolved - BDFam48943
#--   1.0.4       :   12-05-2009 :  DDTs tickets resolved - BDFam49289
#--   1.0.5       :   24-06-2009 :  OBR changes
#--   1.0.6       :   08-07-2009 :  DDTs tickets resolved - BDFam51206, BDFam51622
#--   1.0.7       :   16-07-2009 :  Common changes pertaining to DDTS tickets BDFam47286,BDFam50360,BDFam51427.
#--   1.0.8       :   26-07-2009 :  Log directory related changes pertaining to DDTS tickets BDFam51948.
#--                              :  DDTs tickets resolved - BDFam52215
#--   1.0.9       :   05-08-2009 :  DDTs tickets resolved - BDFam52496
#--                                   BDFam52498
#--   1.0.10      :   22-08-2009 :  DDTs tickets resolved - BDFam53171
#--                                   BDFam53845   
#--                                   BDFam53774
#--   1.0.11      :   16-09-2009 :  DDTs tickets resolved - BDFam53859
#--   1.0.12      :   2009/10/08 : DDTs tickets resolved
#--   3.0.1       :   2010/05/24 : First Implementation for converged PBMT - JanakiRaman and Janith 
#--   3.0.2       :   2010/07/06 : Added X86 changes
#--   3.0.3       :   2011/01/05 : new LRM option --id (suppresses the --logfile option). Must not be used in combination with --hosts
#--   3.0.4       :   2011/03/19 : ALU00966683 PBMT support for 32bit and 64bit Linux environment
#--   3.0.5       :   2011/05/02 : ALU01178887
#--   3.0.6       :   2011/10/14 : ALU01558101
#--   3.0.7       :   2012/03/26 : ALU01748774
#--   3.0.8       :   2012/10/22 : ALU01761938
#--   3.0.9       :   2012/11/06 : ALU01837839
#--   3.0.10      :   2013/04/29 : ALU01880147
#--   3.0.11      :   2013/07/29 : ALU01899054
#--   3.0.12      :   2013/10/23 : Requirement to put the ontReleaseMap file is ISAM
#--   3.0.13      :   2015/05/25 : ALU02060588
#--   3.0.14      :   2015/06/10 : ALU02083077
#--   3.0.15      :   2015/10/28 : ALU02139820
#--   3.0.16      :   2016/03/01 : ALU02120557
#--   3.0.17      :   2016/07/15 : ALU02237966
#--   3.0.18      :   2016/09/28 : ALU02278259
#--   3.1.0       :   2018/06/20 : ALU02460863 & ALU02471697
#--   3.1.1       :   2018/07/06 : Migration via CLI
#--   3.1.2       :   2018/11/08 : changes for FR ALU02510303
#--   3.1.3       :   2018/12/02 : introduce cli bufsize
#--   3.1.4       :   2019/02/28 : Fix for reading SNMPV3 configs from param.cfg 
# 

#------------------------------------------------------------------------------

=head1 Stage ONT software

=begin html
This script is used to perform the download the ONT software from local machine 
to OLT (Server). The set of given ONT type software will be PUT onto OLT (server)
from the local machine where the UE runs.

The operations of the script are influenced by:
<ul>
  <li>Command line options</li>
  <li>A Environment variable configuration file</li>
</ul>

The following modules are used by this script:
<ul>
  <li>Standard PERL:
    <ul>
      <li>FileHandle</li>
      <li>File::Copy</li>
      <li>File::Basename</li>
      <li>Getopt::Long</li>
      <li>POSIX ":sys_wait.h"</li>
      <li>Socket</li>
    </ul>
  </li>
  <li>CPAN:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2098-DFZZA">Net::SNMP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2100-DFZZA">Net::TFTP</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2105-DFZZA">Crypt::Lite</a></li>
    </ul>
  </li>
  <li>PBMT:
    <ul>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2088-DFZZA">Utils</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2089-DFZZA">ToolCommon</a></li>
      <li><a href="http://aww.ngn.bel.alcatel.be/scm-lib4/view.cgi?
           number=3HH-00057-2085-DFZZA">ErrorCodes</a></li>
      <li>version</li>
    </ul>
  </li>
</ul>

=end html

=cut

#------------------------------------------------------------------------------
#-- Find the lib dir, Add the lib path to environment variable
#------------------------------------------------------------------------------
use Cwd qw( realpath getcwd);
use File::Basename;
use Config;
use Text::ParseWords;

my $scriptDir;

BEGIN {
$scriptDir = dirname(realpath($0));
my $perlLibLocation = "standalone";
  if(defined($ENV{'LOAD_PERL_LIBS_LOCATION'})) {
       $perlLibLocation = $ENV{'LOAD_PERL_LIBS_LOCATION'};
  }

  if ($Config{archname} =~ /sun4/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/sun4-solaris-64int");
  } elsif ($Config{archname} =~ /i86pc/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i86pc-solaris-64int");
  } elsif ($Config{archname} =~ /i386-linux-thread-multi/) {
                unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/i386-linux-thread-multi");
  } elsif ($Config{archname} =~ /x86_64-linux-thread-multi/) {
                if($Config{version} =~ /(\d+)\.(\d{1,2})/) {
                   my $perlVersion = ($1 *100 + $2);
                   if( $perlVersion < 510) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.8");
                   } elsif($perlVersion < 516) {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.10/");
                   } else {
                      unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation/x86_64-linux-thread-multi/5.16/");
                   }
                }
  }
  unshift(@INC, "$scriptDir/lib-perl/$perlLibLocation");
}

use lib "$scriptDir/lib"; 



#------------------------------------------------------------------------------
#-- Inclusions
#------------------------------------------------------------------------------
use strict;
use FileHandle;
use File::Copy;
use Getopt::Long;
use POSIX ":sys_wait_h";
use POSIX qw(ceil);
use Socket;
use Net::SNMP;
use Net::TFTP;

use Crypt::Lite;
use Utils;
use PbmtConfig;
use Logger qw(:ALL);
use ErrorCodes qw(:EXIT_CODES :ONT_ERR);
use ToolCommon qw(:CONSTANTS);
use PBMTGlobal;
use FamilyCommon;
use Octopus;
use Capability;
use StageONT7302HiCap;
use StageONT7302HiCapCLI;
use IsamTL1;
use version qw(:ALL);

END {
    LogCloseFile($nodeLogFH);
    LogCloseFile($execLogFH);
}

#------------------------------------------------------------------------------
#-- Prototypes
#------------------------------------------------------------------------------
sub Usage();
sub ParseCommandLine($);
sub PrintLibVersions($$);
sub PrintOptionsSummary($$);
sub StageONTs($$$$); 
sub AdjustToolVars($);

#------------------------------------------------------------------------------
#-- Global Declarations
#------------------------------------------------------------------------------
my $VERSION = "stageont.pl - 3.1.2 -- 2018/12/02";
my $FILENAME = "STAGE";
my $STATUSFILE = "result.txt";
my $ID_FILENAME = "ONTSTAGE";

my %toolData = (HOSTSFILE => "/etc/hosts",
                FTPDIR => "/global/aws_var/aws/equipment",
                USERNAME => "",
                PASSWORD => "",
                TL1USER => "",
                TL1PWD => "",
                MAXSESSIONS => 5,
                TELNETTIMEOUT => 10,
                ACTPINGWAIT => 0,
                LOCALDIR => "tmp",
                ONTRELDIR => "tools/",
                ONTRELMAPFILE => "ONT_Release_Mapping.txt",
                DWLDONTRELMAPFILE => "N",
                OCTOPUSSCRIPTS =>   "tools/octopus",
                OCTOPUSPATH => "tools/octopus",
                SWDIR => "/pub/OntSw/Download",		
#                SWDIR => "/pub/Sw/Download",
                FTPMODE => "TFTP",
                ONTTYPES => "",
                ALLTYPES => 0,
                WARN => 0,                                       # BDFam51206
                RELEASE => "",
                RETRIES => 5,
                BUFSIZE => 30);



my $sessionCount = 0;
my $logExecOrNode = 0;
my @pids;
my $pid;
my $warnCount = 0;

#------------------------------------------------------------------------------
#-- Sub routines 
#------------------------------------------------------------------------------

=head1 USAGE

=head2 How to run?

=begin html
invoke the script by running it from a perl command such as:<br>
<br>
<tt>[perl] stageont.pl --isam $lt;ip&gt;|--hosts &lt;file&gt; \ <br> 
           --servdir $lt;serverOntSoftwareDirectory&gt; \ <br>
           --release &lt;text&gt; --logfile <&lt;file&gt;> [--warn &ltThreshold&gt] \ <br> 
           --onttypes &lt;onttype1&gt;[Load1],&lt;onttype2&gt;[Load2] \<br>
           [--ontreleasemap <&lt;path&gt;>] \<br>
           [--tl1username &lt;username&gt;] [--tl1password &lt;password&gt;] \<br>
           [--cliusername &lt;user&gt;] [--clipassword &lt;password&gt;] \<br> 
           [--multi] [--seplog | --noseplog] [--sessions &lt;nr&gt;] \<br>
           [--v] [--debug] [--noname] [--noping][--comm &lt;SNMP community string&gt;]\ <br>
           [--v3auth &lt;user&gt;:&lt;password&gt;:&lt;authProtocol&gt;[privProtocol&gt;:&lt;password&gt;]]] </tt>
=end html

=head2 Commmand options

=begin html
<table>
  <tr><td>isam</td><td>ISAM identifier IP / NAME</td></tr>
  <tr><td>hosts</td><td>name of the host file</td></tr>
  <tr><td>servdir</td><td>Path in Local machine where ONT SWP resides </td></tr>
  <tr><td>release</td><td>target release info </td></tr>
  <tr><td>logfile</td><td>Identifier for the logfiles that will be created </td></tr>
  <tr><td>warn</td><td>Warning threshold to indicate the operator </td></tr>
  <tr><td>onttypes</td><td>Specified ONT types </td></tr>
  <tr><td>ontreleasemap</td><td>path of the ont release mapping file</td></tr>
  <tr><td>tl1username</td><td>TL1 username</td></tr>
  <tr><td>tl1password</td><td>TL1 password</td></tr>
  <tr><td>cliusername</td><td>ISAM CLI username</td></tr>
  <tr><td>clipassword</td><td>ISAM CLI password</td></tr>
  <tr><td>seplog</td><td>if set create a log file per node, otherwise logging gets appended in the child log file</td></tr>
  <tr><td>noseplog</td><td>if set do not create a logfile per node (default is a logfile per node)</td></tr>;
  <tr><td>multi</td><td>include Unix username in NE logging directory path</td></tr>
  <tr><td>sessions</td><td>number of parallel ISAMs to be treated</td></tr>
  <tr><td>v</td><td>print version and exit</td></tr>
  <tr><td>debug</td><td>enables additional debuging informartion</td></tr>
  <tr><td>noname</td><td>force the IP address in the naming of the temporary NE output directory</td></tr>
  <tr><td>noping</td><td>suppress the usage of ping for networks where usage of ping is not allowed</td></tr>
  <tr><td>comm</td><td>NT SNMP community string.  Default is "public"</td></tr>
  <tr><td>v2</td><td>Use SNMP V2c</td></tr>
  <tr><td>v3auth</td><td>username, password and authentication protocol when using SNMPv3<br>
                         <ul><li>The password should be at least 8 characters long</li>
                             <li>Authentication protocol is SHA or MD5.  Default is MD5</li>
                         </ul><br>
                         <ul><li>Default value for the Privacy protocol is DES.</li>
                         </ul><br>e.g. --v3Auth user:password:authprotocol.Incase when Auth User is configured.</td></tr>
                         </ul><br>e.g. --v3auth user:password:authprotocol:[privproto:password].When AuthPrivUser is configured.</td></tr>
</table>

=end html

=cut

#------------------------------------------------------------------------------
#-- sub Usage(); # To display the usage on receiving incorrect input
#------------------------------------------------------------------------------

=head1 FUNCTIONS

=head2 Usage()

=begin html
Invoked in case of incorrect usage.  Prints all command line options and their 
explanations and exists with exit code "command-line incomplete".

=end html

=cut

sub Usage() {
  print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
  print STDERR "Usage: $0 --isam <IP|hostname> | --hosts <file>  --servdir <serverOntSoftwareDirectory> \\ \n"; 
  print STDERR "       --release <text> --logfile <file> | -id <id> [--warn <Threshold>] \\ \n"; # BDFam48326
  print STDERR "       --onttypes <Type1:Load1,Type2:Load2,..TypeN:LoadN> \\  \n";
  print STDERR "       [--ontreleasemap <path>] \\ \n";
  print STDERR "       [--tl1username <username>] [--tl1password <password>] \\  \n";
  print STDERR "       [--cliusername <username>] [--clipassword <password>] \\ \n";
  print STDERR "       [--multi] [--seplog | --noseplog] [--sessions <no>] \\ \n"; 
  print STDERR "       [--v] [--debug] [--noname] [--noping] [--comm <SNMP community string>] [--v2] \\ \n";
  print STDERR "       [--v3auth <user>[:<password>:<authProtocol>[:<privProtocol>:<password>]]] \n";
  print STDERR "\n";
  print STDERR "     Params:\n";
  print STDERR "         isam              : IP or the hostname of the NE.\n";
  print STDERR "         hosts             : hosts file name \n";
  print STDERR "         servdir           : ONT SWP location on local machine \n";
  print STDERR "         release           : target release info \n";
  print STDERR "         warn              : Warning threshold to indicate the operator \n";
  print STDERR "         id                : a session identifier (used by LRM).  All logfiles are fixed named and contain this ID value.\n";
  print STDERR "                             The logfile option will be igonored.  This option is also not possible in combination with the --hosts option.\n";
  print STDERR "         onttypes          : Specify the types of ONT & Loads for multiple types\n";
  print STDERR "         ontreleasemap     : path of directory where ONT Release Mapping file is located\n";
  print STDERR "         tl1username       : TL1 username\n";
  print STDERR "         tl1password       : TL1 password\n";
  print STDERR "         cliusername       : CLI username \n";
  print STDERR "         clipassword       : CLI password \n";
  print STDERR "         multi             : include Unix username in NE logging directory path\n";
  print STDERR "         seplog            : if set create a log file per node, otherwise logging gets appended in the child log file \n";
  print STDERR "         noseplog          : keep all NE logging in a single child log file (default is a single logfile per NE) \n";
  print STDERR "         sessions          : number of parallel ISAMs to be treated (MAX=$toolData{MAXSESSIONS})\n";
  print STDERR "         v                 : print version and exit\n";
  print STDERR "         debug             : enables additional debugging traces\n";
  print STDERR "         noname            : force the IP address in the naming of the temporary NE output directory\n";
  print STDERR "         noping            : suppress the usage of ping for networks where usage of ping is not allowed.\n";
  print STDERR "         comm              : NT SNMP community string when using SNMPv1.  Default is \"public\"\n";
  print STDERR "         SNMP settings     :\n";
  print STDERR "         v2                : Use SNMPv2c\n";
  print STDERR "         v3auth            : username, password and authentication protocol when using SNMPv3Auth\n";
  print STDERR "                              The password should be at least 6 characters long.\n";
  print STDERR "                              Authentication protocol is SHA or SHA1(replica of SHA) or SHA256 or SHA512 or MD5.  Default is MD5\n";
  print STDERR "                              All fields are mandatory when this option is used.\n";
  print STDERR "                              optional setting when using SNMPv3 auth protocol providing protocol and password.\n";
  print STDERR "                              Default value for the Privacy protocol is DES.\n";
  print STDERR "                              e.g. --v3auth<user>:<password>:<authprotocol>[:<privprotocol>:<password>].Used                                               when AuthPrivUser is configured.\n";
  print STDERR "                              e.g. --v3auth<user>:<password>:<authprotocol>.Used when AuthUser is configured.\n";
  print STDERR "                              e.g. --v3auth<user>. Used when NoAuth and NoPriv is configured.\n";
  exit(CMD_INCOMPL_CODE);
}


#------------------------------------------------------------------------------
#-- sub ParseCommandLine($); # Parse the received command line parameters
#------------------------------------------------------------------------------

=head2 ParseCommandLine($)

=begin html
Parse the command line options and check the validity of the parameter values.  The tool usage will be displayed in case of e
rrors and the script
will end.

=end html

=cut

sub ParseCommandLine($) {
  my ($config) = @_;
  my ($isam, $servDir, $noName, $cliUsername, $cliPassword,
      $warn, $tl1Username, $tl1Password, $release, $debug, 
      $communityString, $noPing, $v3Auth, $version, $fullOntTypes);
  my ($noSepLog, $sepLog, $multi, $logFile, $hosts, $snmpV2, $sessions, $relMapDirName, $id);
      
  my $regExp;
  my %params;

  my $options = GetOptions('servdir=s', \$servDir,
                           'onttypes=s', \$fullOntTypes,
                           'ontreleasemap=s', \$relMapDirName,
                           'release=s', \$release,
                           'logfile=s', \$logFile,
                           'warn=i', \$warn,
                           'debug', \$debug,
                           'hosts=s', \$hosts,
                           'isam=s', \$isam,
                           'tl1username=s', \$tl1Username,
                           'tl1password=s', \$tl1Password,
                           'username|cliusername=s', \$cliUsername,
                           'password|clipassword=s', \$cliPassword,
                           'multi', \$multi,
                           'seplog', \$sepLog,
                           'noseplog', \$noSepLog,
                           'sessions=i', \$sessions,
                           'v', \$version,
                           'comm=s', \$communityString,
                           'v2', \$snmpV2,
                           'noping', \$noPing,
                           'noname', \$noName,
                           'v3auth=s', \$v3Auth, 
                           'id=s', \$id);

  #-- check options
  if (! $options) {
    Usage();
  }

  if ($ARGV[0]){
    print STDERR sprintf("Invalid Parameter : @ARGV \n");
    Usage();
  }

  if (defined($version)) {
    print STDERR sprintf("%s -- Version: %s::%s\n", $0, PBMT_VERSION, $VERSION);
    PrintLibVersions(undef, undef);
    exit(SUCCESS_CODE);
  }

  #-- Check for mandatory parameters  
  if ((defined($isam)) && (defined($hosts))) {
      print STDERR "ERROR: Either Target isam ip or hostname  should be specified\n";
      Usage();
  }

  if (defined($hosts) && ! -r $hosts) {
    print STDERR "ERROR: accessing hostsfile '$hosts' failed: $!\n";
    Usage();
  }

  if (!defined($logFile) && !defined($id)) {
    print STDERR "ERROR: no logfile given (either --logfile or --id must be used)\n";
    Usage();
  }
  if (defined($id) && defined($hosts)) {
    print STDERR "ERROR: --id can't be used in combination with --hosts, use --logfile instead\n";
    Usage();
  }
  if (defined($id)) {
    $params{ID} = $id;
    $params{LOGFILE} = $id . "_" . $ID_FILENAME . ".log";
    $params{PROGFILE} = $id . "_" . $ID_FILENAME . ".update";
    $params{ONT_STATUSFILE} = $id . "_" . $ID_FILENAME . ".status";
  } else {
    $params{LOGFILE} = $logFile ;
    $params{PROGFILE} = $logFile . ".update";
    $params{ONT_STATUSFILE} = $logFile . ".status";
  }

 
  if (defined($sepLog) and defined($noSepLog)) {
    print STDERR "ERROR: Either --selog or --noseplog only should be specified\n";
    Usage();
  }
 
  if (!defined($servDir)  || (defined($servDir) && $servDir eq "")) {
    print STDERR "ERROR: Local path of file does not defined\n";
    Usage();
  } else {
    $toolData{LOCALDIR} = $servDir;
    if (! (-d $toolData{LOCALDIR}) ) {
      print STDERR "Directory does not exist in the given path : $toolData{LOCALDIR}\n";
      Usage();  # not sure about exit code.
    }
  }

  if (!defined($release) ) {
    print STDERR "Error: --release missing ..\n";
    Usage();
  } else {
    if ( uc($release) =~ /^[LR][0-9][1-9]\.[0-9][0-9]\.[0-9][0-9]$/ || uc($release) =~ /^[LR][0-9][1-9]\.[0-9][0-9]\.[0-9][0-9][A-Z]$/ ) {
      $toolData{RELEASE} = $release;
    } else {
      print STDERR "Error : Invalid release info - '$release'\n";
      Usage (); # not sure about exit code.
    } 
  }

  if (!defined($sessions)) {
    $params{SESSIONS} = 1;
  } else {
    if ($sessions > $toolData{MAXSESSIONS}) {
      print STDERR "ERROR: # sessions is larger than the maximum ".
        "($sessions > $toolData{MAXSESSIONS})\n";
      Usage();
    }
    $params{SESSIONS} = $sessions;
  }


  my %typeList; 
  if (defined($fullOntTypes)) {
    my @fullTypes = split(',',$fullOntTypes);
    foreach my $types (@fullTypes) {
      my @type = split(':',$types);
      my $noOfTypes = scalar @type;

      if ( $noOfTypes > 2 || $noOfTypes < 2) {
        print STDERR "Error: Invalid input - @type \n";
        Usage();
      }

      #if (! (uc($type[0]) =~ /^[IOB]-(\d+)[A-Z]-[A-Z]$/) ){ # BDFam48335
      #  print STDERR sprintf("%s\n","Invalid ONT Type : $type[0]");
      #  Usage();
      #}

      if ( (uc($type[1]) !~ /^H.248$/) &&
           (uc($type[1]) !~ /^SIP$/) &&
           (uc($type[1]) !~ /^VEIP_SIP$/) &&
           (uc($type[1]) !~ /^DO$/) ){
        print STDERR "Error: Invalid Load type : $type[1]\n";
        Usage();
      }
      if ( defined ($typeList{uc($type[0])}{uc($type[1])}) ) {
        print STDERR "Error : Duplicate entry : $type[0]:$type[1] \n";
        Usage();
      } else {
        $typeList{uc($type[0])}{uc($type[1])} = 1;
      }
    }
    $toolData{ONTTYPES} = $fullOntTypes;
  } else {
    print STDERR "Error: --ontTypes missing ..";
    Usage();
  }

  if (defined($communityString)&& defined($v3Auth)) {
    print STDERR "ERROR: define any one method of SNMP connection\n";
    Usage();
  }
  $params{NOTFTP}=0;
  $params{SYS_KNOWNHOSTS} = defined($toolData{SYS_KNOWNHOSTS}) ? $toolData{SYS_KNOWNHOSTS} : "";
  $params{LOC_KNOWNHOSTS} = defined($toolData{LOC_KNOWNHOSTS}) ? $toolData{LOC_KNOWNHOSTS} : "";
  $params{UPD_KNOWN_HOST_FILE} = 0; 
  $params{UPD_KNOWN_HOST_FILE} = uc($config->{'UPD_KNOWN_HOST_FILE'}) eq "Y" ? 1 : 0 if defined($config->{'UPD_KNOWN_HOST_FILE'});





  if (defined($cliUsername)) {
    if ($cliUsername eq "") {                                                                  
      print STDERR "Error : Invalid value specified for CLI username  :'$cliUsername' \n";    
      Usage();                                                                                  
    }                                       
    $params{USERNAME} = $cliUsername;
    #$toolData{FTPMODE} = "SFTP";
    #$params{NOTFTP} = 1;
    } else {
    $params{USERNAME} = $toolData{USERNAME};
  }


  if (defined($cliPassword)) {
    if ($cliPassword eq "") {
      print STDERR "Error : Invalid value specified for CLI password  :'$cliPassword' \n";
      Usage();
    }
    $params{PASSWORD} = $cliPassword;
    #$toolData{FTPMODE} = "SFTP";

  } else {
    $params{PASSWORD} = $toolData{PASSWORD};
  }


  if (defined($warn) && (0 > $warn || $warn > 100))
  {
    print STDERR "Error: Invalid value specified for warn : $warn\n";
    Usage();
  } elsif (defined($warn)) {
    $toolData{WARN} = $warn;
  } 

  #Fix for BDFam53859 starts here
  if (defined($tl1Username) && ($tl1Username eq "")) {
    print STDERR "Error : Invalid value specified for usernameTL1 \n";
    Usage();
  }

  if (defined($tl1Password) && ($tl1Password eq "")) {
    print STDERR "Error : Invalid value specified for passwordTL1 \n";
    Usage();
  }
  #Fix for BDFam53859 ends here

  
  if (defined($tl1Username)) {
    $toolData{TL1USER} = $tl1Username;
  }

  if (defined($tl1Password)) {
    $toolData{TL1PWD} = $tl1Password;
  }

  # BDFam53171 begins here
  #if ((defined($cliUsername) && (! defined($cliPassword)) ) || 
  #    (defined($cliPassword) && (! defined($cliUsername)))) {
  #  print STDERR "Incompleted SFTP user credentials \n";
  #  Usage();
  #}
  # BDFam53171 ends here

  if (($toolData{TL1PWD} eq "") || ($toolData{TL1USER} eq "")) {
    print STDERR "No TL1 username or password defined in param.cfg.  Run Config.pl!!!\n";
    exit(PARAM_CFG_NOT_OK_CODE);
  }

  #if (uc($toolData{FTPMODE}) eq 'SFTP' && ($params{USERNAME} eq "" || $params{PASSWORD} eq "") ) {
  #  print STDERR "SFTP requires user name and password \n";
  #  Usage (); 
  #}
  if (defined($hosts)) {
    $params{HOSTS} = $hosts;
  } else {
    $params{HOSTS} = $toolData{HOSTSFILE};
  }

  if ( defined($relMapDirName) ) {                  
     if($relMapDirName ne ""){
        $toolData{ONTRELDIR} = $relMapDirName;
     } 
  }

  if(! -r "$toolData{ONTRELDIR}/$toolData{ONTRELMAPFILE}"){
    print STDERR "ERROR: ONT Release mapping file not found in the location $toolData{ONTRELDIR}. \n";
    Usage();
  }

  $params{UMASK_CHMOD} = ( (! defined $config->{UMASK_OVERRIDE}) or $config->{UMASK_OVERRIDE} eq "Y" ) ? 1 : 0;
  $params{ISAM} = $isam if( defined($isam));
  $params{DEBUG} = 0;
  $params{DEBUG} = 1 if( defined($debug));
  $params{NONAME} = 1 if( defined($noName));
  $params{MULTI} = defined($multi) ? 1 : 0;
  $params{SEPLOG} = 1;
  $params{SEPLOG} = 0 if defined($noSepLog) || defined($id);
  $params{BUFSIZE} = 2 * $toolData{BUFSIZE};
  $params{BUFSIZE} = 100 if ($params{BUFSIZE} > 100);
  $params{BUFSIZE} *= 1048576; # Convert to MBytes



  ### param.cfg Config SNMP_QUERY_TIMEOUT is treated as $params->{SNMP}{QUERY_TIMEOUT}
  $params{SNMP}{QUERY_TIMEOUT} = defined($toolData{SNMP_QUERY_TIMEOUT}) ? $toolData{SNMP_QUERY_TIMEOUT} : 30;
  if(defined($communityString)){
    $params{SNMP}{V1}{NT_COMMUNITY} = $communityString;
  } else {
    $params{SNMP}{V1}{NT_COMMUNITY} = $toolData{NT_COMMUNITY};
  }
  $params{SNMP_V1} = 1 if (defined($params{SNMP}{V1}{NT_COMMUNITY}) and !defined($snmpV2) and !defined($v3Auth)); 
  if (!defined($snmpV2) && !(defined($communityString))) { #ALU01524313-indhu-15/12/2011
    $snmpV2 = 1 if ($config->{'USE_SNMPV2'} eq "Y");
  } else { 
    $params{SNMP}{V2}{NT_COMMUNITY} = defined($communityString) ? $communityString : $toolData{NT_COMMUNITY};
    $params{SNMP_V2} = 1 ;
    $params{SNMP}{V1}{NT_COMMUNITY} = undef;
	$params{SNMP}{V3}{AUTH}{USERNAME} = "";
  } 
  


  #added from here for SNMPv3 auth and priv changes by Indhu on 09/07/2009-BDFam47286.
  if (defined($v3Auth)) {
    my @snmpV3AuthParms = quotewords(':', 0, $v3Auth);
    my $snmpV3AuthParmsLength = scalar(@snmpV3AuthParms);
    if( ($snmpV3AuthParmsLength != 1) && ($snmpV3AuthParmsLength != 2) && 
        ($snmpV3AuthParmsLength != 3) && ($snmpV3AuthParmsLength !=5) ){
         print STDERR "ERROR: Invalid arguments specified for SNMPv3 parameter.\n";
         Usage();
    }
    if ( $snmpV3AuthParms[0] ne "" ) {
        $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
    } else {
	print STDERR "SNMP V3 Username is NULL/EMPTY";
	Usage();
    }
    my @pwdAr = ();
    if( $snmpV3AuthParmsLength > 1 ) {
      $snmpV3AuthParms[2] = "" if ($snmpV3AuthParmsLength == 2);
      if( $snmpV3AuthParms[1] eq "" ) {
          $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
          $params{SNMP}{V3}{AUTH}{PROTO} =  defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
          if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
             $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
          }
      } else {
          @pwdAr = split(//, $snmpV3AuthParms[1]);
          if (scalar @pwdAr > 5) {
            if ((uc($snmpV3AuthParms[2]) =~ /^(SHA|SHA1|SHA512|SHA256|MD5)$/) or ($snmpV3AuthParms[2] eq "")) {
              $params{SNMP}{V3}{AUTH}{USERNAME} = $snmpV3AuthParms[0];
              $params{SNMP}{V3}{AUTH}{PASSWORD} = $snmpV3AuthParms[1];
              $params{SNMP}{V3}{AUTH}{PROTO} = ($snmpV3AuthParms[2] eq "") ? "MD5" : uc($snmpV3AuthParms[2]);
              if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
                 $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
              }
            } else {
              print STDERR "SNMPv3 authentication protocol can only be SHA or SHA512 or SHA256 or MD5.\n";
              Usage();
            }
          } else {
            print STDERR "SNMPv3 authentication password must be at least 6 characters long.\n";
            Usage();
          }
      }
      $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
      $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
      if( $snmpV3AuthParmsLength > 3 ) {
           if( $snmpV3AuthParms[4] eq "" ) {
               $params{SNMP}{V3}{PRIV}{PASSWORD} = "";
               $params{SNMP}{V3}{PRIV}{PROTO} = defined($toolData{SNMPV3_PRIV_PROTOCOL}) ? $toolData{SNMPV3_PRIV_PROTOCOL} : "DES";
           } else {
               @pwdAr = split(//, $snmpV3AuthParms[4]);
               if( scalar(@pwdAr) > 7 ) {
                   if ((uc($snmpV3AuthParms[3]) eq 'DES') or (uc($snmpV3AuthParms[3]) eq 'AES') or ($snmpV3AuthParms[3] eq "")) {
                     $params{SNMP}{V3}{PRIV}{PROTO} = $snmpV3AuthParms[3] eq "" ? "DES" : uc($snmpV3AuthParms[3]);
                     $params{SNMP}{V3}{PRIV}{PASSWORD} = $snmpV3AuthParms[4];
                   } else {
                     print STDERR "SNMPv3 privacy protocol can be either DES or AES.\n";
                     Usage();
                   }
               }else {
                 print STDERR "SNMPv3 privacy password must be at least 8 characters long.\n";
                 Usage();
               } 
           }
       }
     }
  } else {
    if (!defined($communityString)) {
      $params{SNMP}{V3}{AUTH}{USERNAME} = defined($toolData{SNMPV3_USERNAME}) ? $toolData{SNMPV3_USERNAME} : "";
      $params{SNMP}{V3}{AUTH}{PASSWORD} = defined($toolData{SNMPV3_AUTH_PASSWORD}) ? $toolData{SNMPV3_AUTH_PASSWORD} : "";
      $params{SNMP}{V3}{AUTH}{PROTO} = defined($toolData{SNMPV3_AUTH_PROTOCOL}) ? $toolData{SNMPV3_AUTH_PROTOCOL} : "MD5";
      if ($params{SNMP}{V3}{AUTH}{PROTO}  eq "SHA" ) {
         $params{SNMP}{V3}{AUTH}{PROTO} = "SHA1";
      }
      if (defined($toolData{SNMPV3_PRIV_PROTOCOL})) {
       $params{SNMP}{V3}{PRIV}{PROTO} = $toolData{SNMPV3_PRIV_PROTOCOL};
       $params{SNMP}{V3}{PRIV}{PASSWORD} = $toolData{SNMPV3_PRIV_PASSWORD};
      }
    }
  }
  #added till here for SNMPv3 auth and priv changes by Indhu on 09/07/2009-BDFam47286.

  if (! (defined($params{SNMP}{V1}{NT_COMMUNITY})) || ($params{SNMP}{V1}{NT_COMMUNITY} eq "")) {
   if (! (defined($params{SNMP}{V3}{AUTH}{USERNAME})) || ($params{SNMP}{V3}{AUTH}{USERNAME} eq "")) {
    if (! (defined($params{SNMP}{V2}{NT_COMMUNITY})) || ($params{SNMP}{V2}{NT_COMMUNITY} eq "")) {
    print STDERR "SNMP must be configure to either use v1, v2 or v3.  Run Config.pl and configure one of the options.\n";
    exit(PARAM_CFG_NOT_OK_CODE);
    }
   }
  }

  $params{NOPING} = uc($config->{'NO_PING'}) eq "N" ? 0 : 1 if defined($config->{'NO_PING'});
  $params{NOPING} = 1 if defined($noPing);

  return \%params;
}

#------------------------------------------------------------------------------
#-- sub PrintLibVersions($$);
#------------------------------------------------------------------------------

=head2 PrintLibVersions($$)

=begin html
Prints version information on the PERL modules used by the script.

=end html

=cut


sub PrintLibVersions($$) {
  my ($argLogFH, $logMode) = @_;
  my $logHandleOnly = undef;
  if( !defined $argLogFH ) {
     $argLogFH = *STDERR;
     $logHandleOnly = 1;
  } 
  if( !defined $logMode ) {
      $logMode = $Logger::LogModeInfo;
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("%s : %s\n", "PBMT Tool" , PBMT_VERSION), undef, $logHandleOnly );
  my( $library, $versionFunction, $versionValue ) =  ("", "", "");
  $versionValue = $VERSION;
  if( $versionValue  =~ /\s+-+\s+/ ) {
     my @versionValues = split(/\s+-+\s+/, $versionValue);
     $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
  } else {
     $versionValue = sprintf("    %s\n", $versionValue );
  }
  Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );

  my @libraryList = qw( 
                       PbmtConfig Utils ToolCommon FamilyCommon ErrorCodes Capability IsamCli IsamTL1 Octopus 
                       );
  Logger::LogWish( $logMode,  $argLogFH, "Library versions:\n", undef, $logHandleOnly );
  foreach $library ( @libraryList ) {
      $versionFunction = $library."::GetVersion();";
      $versionValue = eval($versionFunction);
      if( $versionValue  =~ /\s+-+\s+/ ) {
         my @versionValues = split(/\s+-+\s+/, $versionValue);
         $versionValue = sprintf("    %-30s -- %-8s -- %s\n", $versionValues[0],  $versionValues[1],  $versionValues[2]);
      } else {
         $versionValue = sprintf("    %s\n", $versionValue );
      }
      Logger::LogWish( $logMode,  $argLogFH, $versionValue, undef, $logHandleOnly );
  }
  return;
}
#------------------------------------------------------------------------------
#-- sub PrintOptionsSummary($$);
#------------------------------------------------------------------------------

=head2 PrintOptionsSummary($$$)

=begin html
Print the tool option summary.  This summary is usefull whenever debugging of 
a problem is required as it indicates what actions will be undertaken by the 
script.

=end html

=cut

sub PrintOptionsSummary($$) {
  my($argLogFH, $logOnScreen) = @_;

  my $logMode = $Logger::LogModeTrace;
  if( defined $logOnScreen) {
      $logMode = $Logger::LogModeInfo;
  }

  PrintLibVersions($argLogFH, $logMode );

  Logger::LogWish( $logMode, $argLogFH, "Options:\n"  );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ISAM(s)" , 
                defined $params->{ISAM} ? "$params->{ISAM} (Only one)" : "List from '$params->{HOSTS}'" ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Access through Ping" , $params->{NOPING} ? "Not-Allowed" : "Allowed")  ); 
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Nr Sessions" , $params->{SESSIONS} ) );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Session ID" , $params->{ID} ) ) if defined($params->{ID});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Update in Known hosts file" , 
                $params->{UPD_KNOWN_HOST_FILE} ? "Yes, Automatically" : "Not Allowed") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Local Dir" , $params->{LOCALDIR} ) ) if defined($params->{LOCALDIR});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Release" , $params->{RELEASE} ) ) if defined($params->{RELEASE});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "ontTypes" , $params->{FULLONTTYPES} ) ) if defined($params->{FULLONTTYPES});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Warning Threshold" , $params->{WARN} ) ) if defined($params->{WARN});
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "Transfer Mode" , $toolData{FTPMODE}) ) if ($toolData{FTPMODE} ne "");
#  if ( uc($toolData{FTPMODE}) eq 'SFTP' ) {
  if (defined($params->{USERNAME}) && defined($params->{PASSWORD})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- User name" , $params->{USERNAME}) ) if ($params->{USERNAME} ne "");
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Password" , '*******') ) if ($params->{PASSWORD} ne "");
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "SNMP Settings :-") );
  if ( !defined $params->{SNMP}{V3}{AUTH}{USERNAME} or $params->{SNMP}{V3}{AUTH}{USERNAME} =~ /^\s*$/) {
    if (defined($params->{SNMP}{V1}{NT_COMMUNITY})) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V1") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V1}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V2c") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- NT Community string" , $params->{SNMP}{V2}{NT_COMMUNITY}) ) if ($params->{DEBUG});
    }
  } else {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Mode" , "Using SNMP V3") );
    if ($params->{DEBUG}) {
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Authentication :-") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Username" , "'$params->{SNMP}{V3}{AUTH}{USERNAME}'") );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') );
      Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{AUTH}{PROTO} ) )  if(defined($params->{SNMP}{V3}{AUTH}{PROTO}) );
      if( defined($params->{SNMP}{V3}{PRIV}{PROTO}) ) {
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "-- Privacy :-") );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Protocol" , $params->{SNMP}{V3}{PRIV}{PROTO}) );
           Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "----- Password" , '********') ) if(defined($params->{SNMP}{V3}{PRIV}{PASSWORD}) && ($params->{SNMP}{V3}{PRIV}{PASSWORD} ne ""));
      }
    }
  }

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "CLI buffer size" , ($params->{BUFSIZE} / 1048576) ) );

  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s\n", "Logging Details :-") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Debugging" , "Enabled") ) if (defined($params->{DEBUG}) && $params->{DEBUG});
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Session based logging" ) );
  } elsif ( defined($params->{ID})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Execution based logging" ) );
  } elsif ( defined($params->{SEPLOG})) {
       Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Log file mode" , "Node based logging" ) );
  }
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
			       defined($params->{NONAME}) ? "Use ISAM's IP-Adress, Instead NAME" : "Use ISAM's Name") );
  Logger::LogWish( $logMode, $argLogFH, sprintf("  %-30s : %s\n", "-- Directory in Log path" , 
                 $params->{MULTI} ? sprintf("Create Unix (current user: %s) based path", getpwuid($<)) : "Don't create unix user based path") );


}


#------------------------------------------------------------------------------
#-- sub StageONTs($$$$);
#------------------------------------------------------------------------------

=head2 StageONTs($$$$$)

=begin html
The main body of the script defining the complete staging process.<br>
The following parameters are used:
<table>
  <tr><td>fh</td><td>the file handle of the parent process.  Progress and failure messages are written to this file</td></tr>
  <tr><td>params</td><td>a pointer to the hash containing all command option</td></tr>
  <tr><td valign="top">isamsRef</td><td>pointer to an array of arrays containing the identities of the ISAMs to be migrated b
y this child process.
      Every array entry consists of 2 element:
      <ul>
        <li>[0]: the IP address of the ISAM</li>
        <li>[1]: the ISAM name in case IP to name translation was possible otherwise the IP address again</li>
     </ul></td></tr>
  <tr><td>count</td><td>the child process number (there may be multiple child processes active)</td></tr>
  <tr><td>config</td><td>a pointer to the hash containing all configuration data</td></tr>
</table>

=end html

=cut

sub StageONTs($$$$){
#$execLogFH is taken from PBMT
  my ($execLogFH, $isamsRef, $sessionCount, $config) = @_;
  my $isam;
  my $ret;
  my $startTime;
  my $nodeLogFile;                             # Name of node specific log 
  my $nodeStatusLog;                           # Filehandle of the node status result file
  my $dirName = dirname($0);
  my $ntType;
  my $isamNode;
  my ($session, $snmpError);
  my $fxrData;
  my $statFile;
  my $progFile;

  #Below arguments are taken from the PBMT
  my ($time1, $time2);
  my $newLog = "$params->{LOGFILE}_$sessionCount";    # Name of child logfile
  my $userName = $params->{MULTI} ? getpwuid($<) : "";
  my $dir;
  my $thisTftpDir;


  $dirName = getcwd() if ($dirName eq ".");
  FamilyCommon::SetToolDirName($dirName);
  FamilyCommon::SetOwnPid($$);
  #umask(0); # Make sure directory remains 0777
  umask(0) if ( $params->{UMASK_CHMOD} );
  FamilyCommon::SetParentLogFile($execLogFH);
  FamilyCommon::SetChildNum($sessionCount);
  FamilyCommon::SetToolData(\%toolData);
  FamilyCommon::SetConfigData($config);


  ### Ensure and create session based logged, When execution starts with --noseplog and without --id parameter passed
  if ( !(defined $params->{ID}) and $params->{SEPLOG} == 0 ) {
      $nodeLogFile = sprintf("%s_%d",$params->{LOGFILE}, $sessionCount);
      $logExecOrNode = 1;
      Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Session ($sessionCount) " );
      PrintOptionsSummary($nodeLogFH, undef);
  }

 
  my $isamCount = -1;
  foreach $isam (@{$isamsRef}) {
    $isamCount++;
    $isamNode = $params->{NONAME} ? $isam->[0] : $isam->[1]; # Store for IncrErrorCount
    FamilyCommon::SetFailedNode($isamNode);
    FamilyCommon::SetWarnLogFile($nodeLogFH) if $nodeLogFH != $execLogFH;   # Make sure warnings and 
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    
    SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
    IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);
    StageONT7302HiCap::ResetGlobals();
	IsamTL1::ResetGlobals();
    $params->{USESSH} = 0; #for each isam the value is reset

	$startTime = time();
    my ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($startTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Started processing Node $isamCount : '$isamNode'\n");

    ##### Create ResultSpace directory #####
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
    ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}/$isamNode" );

    if ( defined($params->{ID})) {
         $logExecOrNode = 0;
         $nodeLogFile = "$toolData{RESULTSPACE}/$isamNode/$params->{LOGFILE}";
         Logger::LogHandleUpdate( $logExecOrNode, $nodeLogFile, "Execution");
         FamilyCommon::SetParentLogFile($execLogFH);
    } elsif ( $params->{SEPLOG} ) {
         $nodeLogFile = sprintf("%s_%d_%d",$params->{LOGFILE},$sessionCount,$isamCount);
         $logExecOrNode = 1;
         Logger::LogHandleCreate( $logExecOrNode, $nodeLogFile, "Node ($isamNode)" );
         PrintOptionsSummary($nodeLogFH, undef);
    }
    FamilyCommon::SetWarnLogFile($nodeLogFH);
    FamilyCommon::SetChildLogFile($nodeLogFH);  # logging is sent to the child process' logfile.
    # Switch debugging (if any) to an opened filehandle...
    IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
    Octopus::SetVars($toolData{OCTOPUSSCRIPTS}, $toolData{OCTOPUSPATH}, $nodeLogFH);
    Capability::SetDebug($nodeLogFH) if ($params->{DEBUG});
    ToolCommon::SetDebug($nodeLogFH);

    $STATUSFILE = $params->{ID} . "_" . $ID_FILENAME . ".result" if (defined($params->{ID}));
    if (! open($nodeStatusLog, ">$toolData{RESULTSPACE}/$isamNode/$STATUSFILE")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create status file '$toolData{RESULTSPACE}/$isamNode/$STATUSFILE' for $isamNode: $!\n");
      CORE::exit(CREATE_STAT_FILE_FAIL_CODE)
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : Result status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$STATUSFILE" ) );
    }
    
    $nodeStatusLog->autoflush(1);
    FamilyCommon::SetNodeStatusLog($nodeStatusLog);

    if (!open($progFile, ">$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create prgress file '$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}' for $isamNode: $!\n");
      exit CREATE_FILE_FAIL_CODE;
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : ONT Progress file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$params->{PROGFILE}" ) );
    }

    truncate($progFile, 0 );                        # BDFam51622
    print $progFile "0% Complete\n";                # BDFam51622
    $progFile->autoflush(1);                        # BDFam51622

    StageONT7302HiCap::SetProgressFail($progFile);
    StageONT7302HiCapCLI::SetProgressFail($progFile);
    if (!open($statFile, ">$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}")) {
      LogError( $execLogFH, "Session $sessionCount: Node ($isamCount - $isamNode ) : Unable to create ont status file '$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}' for $isamNode: $!\n");
      exit CREATE_FILE_FAIL_CODE;
    } else {
      Logger::LogInfo($nodeLogFH, sprintf("Node ($isamNode ) : ONT Status file '%s%s'\n",
                                ($toolData{RESULTSPACE} =~ /^\//) ? "" : getcwd()."/",
                                "$toolData{RESULTSPACE}/$isamNode/$params->{ONT_STATUSFILE}" ) );
    }

    $statFile->autoflush(1);                        
    StageONT7302HiCap::SetStatusFail($statFile);
    StageONT7302HiCapCLI::SetStatusFail($statFile);

    FamilyCommon::ClearSnmpSession2Close();
    FamilyCommon::CloseTL1Session2Close();

    Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});  # send SNMP debug info to the child logfile first

    if (not ToolCommon::Ping($config->{PING}, $isam->[0]) ) {
        FamilyCommon::ReportStatus( EC_NO_PING, SNMPComm::GetError(), undef );
        next;
    }
    Logger::LogTrace($execLogFH, "Attempt to make SNMP connectivity...\n");
    my ( $snmpStatus, $session ) = SNMPComm::EstablishSNMP( $isam->[0] ); 
    if( $snmpStatus ) {
	      FamilyCommon::IncrErrorCount( "FAILED : ".SNMPComm::GetError(),
	        		                    EC_SNMP_SESS_ESTB_ERR, ": ".SNMPComm::GetError(), 1);
	      next;
    }
    my $mibVersion = SNMPComm::GetValue('MIB VERSION', MIBVERSIONOID );
    if( !defined($mibVersion) ) {
         FamilyCommon::IncrErrorCount(sprintf("  FAILED to obtain MIB version: %s\n", SNMPComm::GetError() ),
	       		   EC_SNMP_NO_GET, sprintf(": %s", SNMPComm::GetError() ), 1);
         next;
    }
#KR---------------
    Logger::LogTrace($nodeLogFH, "Attempt to get file transfer data...\n");
    $fxrData = ToolCommon::GetFxrSettings($session, $toolData{RETRIES});
    if (!defined($fxrData)) {
      Logger::LogError($nodeLogFH, "obtained empty  file transfer protocol details\n");
      FamilyCommon::IncrErrorCount("\n",
                                  EC_GET_FTP_FAILED,
                                  undef,
                                  1);
      next;
    } else {
      if($fxrData->{PROTOCOL} != TFTP_FXR) {
        $params->{NOTFTP}=1;
        $toolData{FTPMODE} = "SFTP";
            Logger::LogTrace($nodeLogFH, "  OK\n");
      }
    }
#---------------

    Logger::LogTrace($execLogFH, "Attempt to prepare logging directories...\n");
     #Multi support is inherited from PBMT
    ($dir, $thisTftpDir, $fxrData) = FamilyCommon::PrepareLogging($session, $nodeLogFH, $userName, $isamNode);
    #below check is taken from PBMT
	if (not defined($fxrData)) { 
      FamilyCommon::IncrErrorCount(sprintf("  FAILED: %s\n", FamilyCommon::GetError()),
				   EC_TRX_PROTOCOL_MISMATCH,
				   sprintf(": %s", FamilyCommon::GetError()),
				   1);
      next;
    } else {
      $toolData{FXRDATA} = $fxrData;
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }



    $dir =~ s/\/\//\//g;         # On unix this does not make a difference but let's convert double forward slashes into a single one
    $thisTftpDir =~ s/\/\//\//g;
    #-- first create temporary directory for tftp.  ISAM-V nodes prior to R3.5 only know tftp as file transfer
    Logger::LogDebug($nodeLogFH, "Creating workspace for '$dir'\n");
    CORE::exit(CREATE_DIR_FAILED_CODE)
      if (! FamilyCommon::CreateTmpDirFile($execLogFH, $nodeLogFH, $dir, $userName, $newLog, $sessionCount, 0));
    $dir .= "/$userName" if $params->{MULTI};
    $thisTftpDir .= "/$userName" if $params->{MULTI};


    FamilyCommon::SetNeStorage($dir);



    if ($params->{SEPLOG}) {
        $logExecOrNode = 1;
        $nodeLogFile = sprintf("ontstage_%04d-%02d-%02d_%02d_%02d.log", $year+1900, $month+1, $dayOfMonth, $hour, $minute);
        Logger::LogHandleUpdate( $logExecOrNode, "$dir/$nodeLogFile", "Node ($isamNode)" );

        SshChannel::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamCli::SetDebug($nodeLogFH) if ($params->{DEBUG});
        IsamTL1::SetDebug($nodeLogFH) if ($params->{DEBUG});
        Utils::SetDebug($nodeLogFH) if ($params->{DEBUG});
        ToolCommon::SetDebug($nodeLogFH);           # -- 1.2.5 END
        FamilyCommon::SetWarnLogFile($nodeLogFH);   # Adjust warning and child logfile handles.
        FamilyCommon::SetChildLogFile($nodeLogFH);  # This only makes a difference in case --sepLog has been used otherwise the settings remain as is
        Net::SNMP->snmp_debug(0x3e, $nodeLogFH) if ($params->{DEBUG});
    }

    # Determine NE family: retrieve NT-A and NT-B type and status using SNMP.  Only then we continue 
    $ntType = ToolCommon::DetermineNtType( );
    next if( not defined($ntType) );

    ##$ret = StageONT7302HiCap::StageONT($isam, $ntType);
    Logger::LogTrace($nodeLogFH, "Check access security settings...");
    my $accBits = ToolCommon::GetAccessettings($session, $params->{DEBUG});
    if (! defined($accBits)) {
      next if( FamilyCommon::ReportStatus( EC_GET_ACC_BITS_FAILED, ToolCommon::GetError(), undef ) );
    } else {
      Logger::LogTrace($nodeLogFH, "  OK\n");
    }

    if( ! ToolCommon::TL1ViaUDP($accBits, $params->{DEBUG}) && ! ToolCommon::TL1ViaSSH($accBits, $params->{DEBUG}) )
    {
        if ( ! ToolCommon::CliViaTelnet($accBits, $params->{DEBUG})) {
               $params->{USESSH} = 1;
	       Logger::LogInfo($nodeLogFH, "CLI Communication through SSH\n");
        }
        $params->{PHYSICAL_HOST_IP} = ToolCommon::GetIPAddress($isam->[0]); 

        $ret = StageONT7302HiCapCLI::StageONT($isam, $ntType);
    } else {
        $ret = StageONT7302HiCap::StageONT($isam, $ntType);
    } 

    FamilyCommon::CloseSnmpSession2Close(); # Reset for IncrErrorCount in case the post migration script would fail.
    FamilyCommon::CloseTL1Session2Close();

    my $endTime = time();
    ($second, $minute, $hour, $dayOfMonth, $month, $year) = (localtime($endTime))[0,1,2,3,4,5];
    Logger::LogInfo( $execLogFH, sprintf("Session (%d) : Finished processing Node %d : '%s' in %d seconds\n",
                         $sessionCount, $isamCount, $isamNode, ($endTime - $startTime)) );
    $nodeStatusLog = FamilyCommon::GetNodeStatusLog();

   if ($params->{SEPLOG}) {
       FamilyCommon::CloseChildLogFile();
       LogCloseFile($nodeLogFH);
   }
    FamilyCommon::CloseChildLogFile() if ($params->{SEPLOG});
    if (defined($nodeStatusLog) and ($ret)) {
      print $nodeStatusLog sprintf("%s\n", ErrorCodes::ExitString(SUCCESS_CODE));
      FamilyCommon::CloseNodeStatusLog();
    }
    
    close $progFile if (defined($progFile));
    $progFile = undef;

    close $statFile if (defined($statFile));
    $statFile = undef;
  }


  if (FamilyCommon::GetErrorCount()) {
      my $isamNodes = FamilyCommon::GetFailedNodeList();
      my @isamNodes = (sort keys %$isamNodes);
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  %d error(s) occurred for processing Node(s) '%s'\n",
	  			 $sessionCount, FamilyCommon::GetErrorCount(), @isamNodes ) );
	  Logger::LogInfo( $execLogFH, sprintf("Session (%d) :  For more details, Please verify '%s'\n",
	  			 $sessionCount, $nodeLogFile) );
  }
  if (!defined($params->{ID})) {
    if (! $params->{SEPLOG}) {
        FamilyCommon::CloseChildLogFile();
        LogCloseFile($nodeLogFH);
    }
  }
  Logger::LogInfo( $execLogFH, "Session ($sessionCount) : Finished...\n");
  CORE::exit(FamilyCommon::GetErrorCount() ? EXIT_WARN_CODE : SUCCESS_CODE);

}

#------------------------------------------------------------------------------
#-- sub AdjustToolVars($);
#------------------------------------------------------------------------------

=head2 AdjustToolVars($)

=begin html
Adjust Tool defaults if needed (i.e. when defined in the parameter config file.

=end html

=cut

sub AdjustToolVars($){
  my ($config) = @_;
  my $crypt = Crypt::Lite->new('debug' => 0);
  $toolData{HOSTSFILE} = $config->{'HOSTSFILE'} if defined ($config->{'HOSTSFILE'});
  $toolData{FTPDIR} = $config->{'FTPDIR'} if defined($config->{'FTPDIR'});
  $toolData{FTPTIMEOUT} = $config->{'FTPTIMEOUT'} if defined($config->{FTPTIMEOUT});
  $toolData{MAXSESSIONS} = $config->{'MAXTFTPSESSION'} if defined($config->{'MAXTFTPSESSION'});
  $toolData{TELNETTIMEOUT} = $config->{'TELNETTIMEOUT'} if defined($config->{TELNETTIMEOUT}); 
  $toolData{RETRIES} = $config->{'RETRIES'} if defined($config->{'RETRIES'});
  $toolData{RESULTSPACE} = $config->{RESULTSPACE} if (defined($config->{RESULTSPACE}));
  $toolData{RESULTSPACE} =~ s/\/$//;
  $toolData{TL1UDPPORT} = $config->{TL1UDPPORT} if defined($config->{TL1UDPPORT});
  $STATUSFILE = $config->{ONTSTG_STATUS_FILE} if defined($config->{ONTSTG_STATUS_FILE});
  $toolData{NT_COMMUNITY} = $crypt->decrypt($config->{NT_COMMUNITY}, CYPHER) if (defined($config->{NT_COMMUNITY}) &&
                                                                                 ($config->{NT_COMMUNITY} ne ""));

  $toolData{SNMPV3_AUTH_PROTOCOL} = $config->{SNMPV3_AUTH_PROTOCOL} 
                                    if defined($config->{SNMPV3_AUTH_PROTOCOL});
  $toolData{SNMPV3_USERNAME} = $crypt->decrypt($config->{SNMPV3_USERNAME}, CYPHER) 
                                    if defined($config->{SNMPV3_USERNAME});
  $toolData{SNMPV3_AUTH_PASSWORD} = $crypt->decrypt($config->{SNMPV3_AUTH_PASSWORD}, CYPHER) 
                                    if defined($config->{SNMPV3_AUTH_PASSWORD});
  $toolData{SNMPV3_PRIV_PROTOCOL} = $config->{SNMPV3_PRIV_PROTOCOL} if defined($config->{SNMPV3_PRIV_PROTOCOL});
  $toolData{SNMPV3_PRIV_PASSWORD} = $crypt->decrypt($config->{SNMPV3_PRIV_PASSWORD}, CYPHER) 
                                    if defined($config->{SNMPV3_PRIV_PASSWORD});

  $toolData{TL1USER} =  $crypt->decrypt($config->{'DEFAULTTL1USER'}, CYPHER)
             if (defined($config->{'DEFAULTTL1USER'}) && ($config->{'DEFAULTTL1USER'} ne ""));

  $toolData{TL1PWD} =  $crypt->decrypt($config->{'DEFAULTTL1PWD'}, CYPHER)
             if (defined($config->{'DEFAULTTL1PWD'}) && ($config->{'DEFAULTTL1PWD'} ne ""));

  $toolData{USERNAME} = $crypt->decrypt($config->{'DEFAULTCLIUSER'}, CYPHER) if (defined($config->{'DEFAULTCLIUSER'}) &&
                                                                                 ($config->{'DEFAULTCLIUSER'} ne ""));
  $toolData{PASSWORD} = $crypt->decrypt($config->{'DEFAULTCLIPWD'}, CYPHER) if (defined($config->{'DEFAULTCLIPWD'}) &&
                                                                                ($config->{'DEFAULTCLIPWD'} ne ""));

  #$toolData{FTPMODE} =  uc($config->{'FTPMODE'}) if (defined($config->{'FTPMODE'}));  # BDFam48943
  $toolData{NTDBTOOLDIR} = $config->{'NTDBTOOLDIR'} if defined($config->{'NTDBTOOLDIR'}); # BDFam53774

  $toolData{TFTPDIR} = $config->{'TFTPDIR'} if defined($config->{'TFTPDIR'});
  $toolData{TFTPWORKDIR} = $config->{'TFTPWORKDIR'} if defined($config->{'TFTPWORKDIR'});

  $toolData{ONTRELDIR} = $config->{'ONTRELDIR'} if defined($config->{'ONTRELDIR'});
  $toolData{ONTRELMAPFILE} = $config->{'ONTRELMAPFILE'} if defined($config->{'ONTRELMAPFILE'});
  $toolData{DWLDONTRELMAPFILE} = uc($config->{'DWLDONTRELMAPFILE'}) eq "N" ? 0 : 1  if defined($config->{'DWLDONTRELMAPFILE'});
  $toolData{SYS_KNOWNHOSTS} = $config->{'SYS_KNOWNHOSTS'} if defined($config->{'SYS_KNOWNHOSTS'});
  $toolData{LOC_KNOWNHOSTS} = $config->{'LOC_KNOWNHOSTS'} if defined($config->{'LOC_KNOWNHOSTS'});
  $toolData{BUFSIZE} = $config->{'BUFSIZE'} if defined($config->{'BUFSIZE'});


}

#------------------------------------------------------------------------------
#-- Main Program 
#------------------------------------------------------------------------------

=head1 SIGNAL HANDLING

=begin html
Signal handlers are installed for the following:
<ul>
  <li>'INT': Ctrl-C typed on the Unix commandline to abort the running process 
      or a SIGINT has been received</li>
  <li>'__WARN__': a PERL warning has been generated.  The warning is written 
      into the log files for later analysis (no warnings should be generated 
      during the run of a PBMT program.</li>
  <li>'__DIE__': a PERL die has been called.  Offender and message is written 
      to the log files and the status result file and the log files are
      gracefully closed and the ISAM is released (as far as possible).
  <li>'KILL': a unix SIGKILL is received.</li>
</ul>

=end html

=cut

sub CleanUp($;$) {
  my ($signal, $exitCode) = @_;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my $ownPid = FamilyCommon::GetOwnPid();
  $exitCode = EXIT_SIGNAL_CODE if ! defined($exitCode);
  my $string = sprintf("[PID %s] - %s::%s - Caught signal '%s'\n", $ownPid, PBMT_VERSION, $VERSION, $signal);
  print STDERR $string;
  Logger::LogTrace($warnLogFile, $string) if defined($warnLogFile);

  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $framePresent = 1;
  my $i = 0;

  while ($framePresent) {
    $ownPackage = (caller($i))[0];
    $ownFileName = (caller($i))[1];
    $ownLine = (caller($i))[2];
    $ownRoutine = (caller($i))[3];
    if (defined($ownPackage)) {
      Logger::LogTrace($warnLogFile, sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                                            defined($ownPackage) ? $ownPackage : "undefined",
                                            defined($ownFileName) ? $ownFileName: "undefined",
                                            defined($ownLine) ? $ownLine : "undefined",
                                            defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
      print STDERR sprintf("Stackframe $i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                           defined($ownPackage) ? $ownPackage : "undefined",
                           defined($ownFileName) ? $ownFileName: "undefined",
                           defined($ownLine) ? $ownLine : "undefined",
                           defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
      $i++;
    } else {
      $framePresent = 0;
    }
  }
  if (scalar @pids) {
    Logger::LogTrace($warnLogFile, "  Aborting children...\n") if defined($warnLogFile);
    foreach my $pid (@pids) {
      if ($pid > 0) {
        print STDERR "Killing process $pid...\n";
        Logger::LogTrace($warnLogFile, "Killing process $pid...\n") if defined($warnLogFile);
        kill($pid);
      }
    }
  }
  my $nodeStatusLog = FamilyCommon::GetNodeStatusLog();
  if (defined($nodeStatusLog)) {
    print $nodeStatusLog sprintf("%s '%s'\n", ErrorCodes::ExitString($exitCode), $signal);
    FamilyCommon::CloseNodeStatusLog();
  }
  FamilyCommon::CloseWarnLogFile() if (! defined(FamilyCommon::GetChildNum()));  
  # In a child the childFh and the warning logfile are the same.
  FamilyCommon::ToolExit($exitCode);
}

FamilyCommon::SetOwnPid($$);
$SIG{'INT'} = \&CleanUp;
$SIG{__WARN__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my ($package, $fileName, $line) = caller();

  if (!($msgStr =~ /Zlib/)) {
    Logger::LogTrace($warnLogFile, "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr") if (defined($warnLogFile));
    print STDERR "Warning [package '$package', file: '$fileName', line:$line]:\n$msgStr";
    $warnCount++;
  }
};

$SIG{__DIE__} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Died [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("__DIE__", EXIT_DIED_CODE);
};

$SIG{"KILL"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  my $warnLogFile = FamilyCommon::GetWarnLogFile();
  my ($package, $fileName, $line) = caller();
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  Logger::LogTrace($warnLogFile, "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr") if (defined($warnLogFile));
  print STDERR "Killed [package '$package', file: '$fileName', line: $line]:\n$msgStr";
  CleanUp("KILL", EXIT_DIED_CODE);
};

$SIG{"PIPE"} = sub {
  my @msgs = @_;
  my $msgStr = "";
  foreach (@msgs) {
    $msgStr .= "$_\n";
  }
  my $ownPackage;
  my $ownFileName;
  my $ownLine;
  my $ownRoutine;
  my $warnLogFile = FamilyCommon::GetWarnLogFile();

  if (!($msgStr =~ /Zlib/)) {
    my $framePresent = 1;
    my $i = 0;
    print STDERR "SIGPIPE received (ignored): $msgStr" if !defined($warnLogFile);
    Logger::LogTrace($warnLogFile, "SIGPIPE received (ignored): $msgStr") if defined($warnLogFile);
    while ($framePresent) {
      $ownPackage = (caller($i))[0];
      $ownFileName = (caller($i))[1];
      $ownLine = (caller($i))[2];
      $ownRoutine = (caller($i))[3];
      if (defined($ownPackage)) {
        print STDERR sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                             defined($ownPackage) ? $ownPackage : "undefined",
                             defined($ownFileName) ? $ownFileName: "undefined",
                             defined($ownLine) ? $ownLine : "undefined",
                             defined($ownRoutine) ? $ownRoutine : "undefined") if !defined($warnLogFile);
        Logger::LogTrace($warnLogFile, sprintf("$i:\n\tPackage: %s\n\tFile: %s\n\tLineNo: %s\n\tRoutine: %s\n",
                                              defined($ownPackage) ? $ownPackage : "undefined",
                                              defined($ownFileName) ? $ownFileName: "undefined",
                                              defined($ownLine) ? $ownLine : "undefined",
                                              defined($ownRoutine) ? $ownRoutine : "undefined")) if defined($warnLogFile);
        $i++;
      } else {
        $framePresent = 0;
      }
    }
    CleanUp("PIPE", EXIT_DIED_CODE);
  }
};

#-- Read parameter config file and adjust tool defaults if needed -- BB--
my $dirName = dirname($0);

$dirName = getcwd() if ($dirName eq ".");
my %config = Utils::ReadConfig("$dirName/param.cfg");
if (%config) {
  if (PbmtConfig::NeedsUpdate(\%config)) {
    print STDERR "PBMT's configuration needs an update, please run $dirName/Config.pl first\n";
    exit PARAM_CFG_NEEDS_UPDATE_CODE;
  }
} else {
  print STDERR "$dirName/param.cfg not found.  Please run $dirName/Config.pl first!!\n";
  exit PARAM_CFG_MISSING_CODE;
}
AdjustToolVars(\%config);
FamilyCommon::SetConfigData(\%config);
# -- End modif BB

#-- Handle command line
$params = ParseCommandLine(\%config);

#Taken from PBMT

$logExecOrNode = 0;
Logger::LogHandleCreate( $logExecOrNode, $params->{LOGFILE}, "Execution" );
FamilyCommon::SetWarnLogFile($execLogFH);
Logger::LogInfo( $execLogFH, sprintf("Execution (pid :%d) started...\n", FamilyCommon::GetOwnPid()));

#-- Print summary of options
PrintOptionsSummary($execLogFH, 1);
Utils::SetDebug($execLogFH) if ($params->{DEBUG}); # 1.2.2


#-- Get a list of all Isams
my $isamsRef;

if (defined( $params->{ISAM})) {
   if ($params->{ISAM} =~ /^\d+(.\d+)*$/) {
  
     if (ToolCommon::ValidIP($params->{ISAM}))
     {
        #below function is taken from UE
       my $name = gethostbyaddr(inet_aton($params->{ISAM}), AF_INET);
       if (! defined($name)) {
        $isamsRef = [ [$params->{ISAM}, $params->{ISAM}] ];
       } else {
       $isamsRef = [ [$params->{ISAM}, $name] ];
      }
    }
    else
    {
      print STDERR "ERROR: Invalid ISAM IP\n";
      exit INVALID_ISAM_IP_CODE;
    }
  } else {
    my @addr = gethostbyname($params->{ISAM});
    my @ip = undef;

    if(defined($addr[4])){
      @ip = unpack( "C4", $addr[4]);

        my $ipAddr = join( '.', @ip );
        $isamsRef = [ [$ipAddr, $params->{ISAM}]];
   }
   else {
   print STDERR "ERROR: Invalid Hostname\n";   
   exit UNABLE_TO_RESOLVE_HOST_NAME_CODE;
   }
  }
 }
 else #taken from pbmt
 {
      #Read the list of NE from the host file
      Logger::LogInfo($execLogFH, "Reading Hostfile...\n");
      $isamsRef = ToolCommon::ReadHostsFile($execLogFH, $params->{DEBUG}, $params->{HOSTS}, $toolData{NODEPREFIX}); 
 }
 
#below check is taken from pbmt 
exit HOSTS_FILE_OPEN_FAILED_CODE if (!defined($isamsRef));
#-- Create work dir
#umask(0); # Make sure directory remains 0777
umask(0) if ( $params->{UMASK_CHMOD} );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{TFTPDIR}/$toolData{TFTPWORKDIR}"  );
ToolCommon::CreateDirectory( $execLogFH, "$toolData{RESULTSPACE}" );
 
 
my @isams = @{$isamsRef};

while ($sessionCount < $params->{SESSIONS}) {
  my $isamsInSessions = ceil( scalar(@isams) / ($params->{SESSIONS} - $sessionCount) );
  my @isamPart = splice(@isams, 0, $isamsInSessions);
  if (scalar(@isamPart)) {
    if ($pid = fork()) {
      push(@pids, $pid);
    } else {
      StageONTs($execLogFH, \@isamPart, $sessionCount, \%config); 
      exit SUCCESS_CODE;
    }
  }
  $sessionCount++;
  sleep 1;
}

my $signal,
my $exitCode;
my $overAllExitCode = SUCCESS_CODE;
my $newExitCode = SUCCESS_CODE;
my $childIdx = -1;
my $pidIdx = 0;
if (scalar(@pids)) {
  my $kid;
  do {
    $kid = waitpid(-1, &WNOHANG);
    ($exitCode, $signal) = Utils::ConvertExitCode($?);
    if (($kid != 0) && ($kid != -1)) {
      $childIdx = -1;
      $pidIdx = 0;
      while (($childIdx == -1) && ($pidIdx < scalar(@pids))) {
	if ($kid == $pids[$pidIdx]) {
	  $childIdx = $pidIdx;
	  $pids[$pidIdx] = 0;
	  last;
	}
	$pidIdx++;
      }
      Logger::LogInfo($execLogFH, sprintf("Session %d: exit code: '%d' (%s), signal: '%d'\n",
					$childIdx, $exitCode, ErrorCodes::ExitString($exitCode), $signal));
      if ($exitCode > 9) {
	$newExitCode =  EXIT_CHILD_FATAL_CODE;
      } elsif ($exitCode > 0) {
	$newExitCode =  EXIT_WARN_CODE;
      }
      # Exit with the highest code.
      $overAllExitCode = $newExitCode if ($newExitCode > $overAllExitCode);
    }
    sleep 2;
  } until ($kid == -1);
  sleep 1;
  Logger::LogInfo($execLogFH, "All childs finished, exiting\n");
  FamilyCommon::CloseWarnLogFile();
  print STDERR "$warnCount warnings encountered...\n" if ($warnCount);
}
exit $overAllExitCode;
#end - pbmt code
