#!/bin/sh

#==============================================================================
#-- file:         shubSwCheck.sh
#-- Date:         06.03.06
#-- Company:      Alcatel Bell - TEC
#-- Author:       Bart Bogaert, JA54
#-- function:     Check presence of Lanx.Z in /Sw
#-- synopsis:     see below.
#-- dependencies: 1) octopus tool should be referenced in the path
#--               2) must be run on the AWS connected to the ISAM.
#--               3) current directory must be writable
#------------------------------------------------------------------------------
#-- disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel. The script is for TAC use only.
#--               There is no warranty about it's running except in a
#--               controlled manner by qualified personnel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel CID 2005
#--               All rights reserved.
#------------------------------------------------------------------------------
Version="shubSwCheck.sh version 1.0.1 - (C)Alcatel"
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 060306 : first version
#--   1.0.1 - 2015/09/01 : ALU02119959 : To exit with proper exit code
#--   1.0.2 - 2015/11/25 : ALU02148679
#==============================================================================

#
# set debug option
#
# set -x


#
# timing test loop - strips some commands
#
TIMINGTEST=0


#
#
#
help() {
	echo "$Version"
	echo "usage: $0 <ip>"
	echo "	<ip	>	dotted-id or hostname of ASAM"
	exit 1
}


#
# ----------------------------------------------------------------
# variables
#
TRACETIME=30
MONITORTIME=0
MONITORCNT=1000
CLEARCOUNTERS=0
CLEARERRORS=0
LOOP=0
FULL=1
OCTOPUSPATH=""

#
while [ -n "$1" ]
do
    case $1 in

    #
    # timeout flag
    #
    -[tT]*)
	shift
        TRACETIME=$1
        ;;

    #
    # clear counters requested
    #
    -[cC]*)
        CLEARCOUNTERS=1
        ;;

    #
    # clear error_recs requested
    #
    -[eE]*)
        CLEARERRORS=1
        ;;

    #
    # statistics only 
    #
    -[iI]*)
        FULL=0
        ;;

    #
    # continuous loop
    #
    -[lL]*)
	shift
        MONITORCNT=$1
        ;;

    #
    # continuous loop
    #
    -[mM]*)
	shift
        MONITORTIME=$1
        ;;
    #
    # octopus path
    #
    -[oO]*)
        shift
        OCTOPUSPATH=$1
        ;;
    #
    # Logfile
    #
    -[gG]*)
	shift
	LOGFILE=$1
        ;;
    #
    # other arguments
    #
    *)
        IPADDR=$1
        ;;
    esac
    shift
done


#
# input file
#
INPUT="tmp_input$$"
MGT="tmp_mgt$$"

#==============================================================================
# Constants - BDFhw14578 <revuri.reddy@wipro.com>
#==============================================================================
LOGFILE1="tmp1_log$$"
INPUT1="tmp1_input$$"
MGT1="tmp1_mgt$$"

NTSWVERSION="_"
NERELEASENAME="_"
#=============== BDFhw19380 <krishnamurthy.pavan@wipro.com> ===================
OSWPDESCFILE="_"
ACTIVEOSWP="_"
NTTYPE="_"
NE_REL_NO=4

#
which nawk >/dev/null 2>&1
if [ $? -eq 0 ]
then
        AWK="nawk"
else
        AWK="awk"
fi

#==============================================================================
# END <revuri.reddy@wipro.com>
#==============================================================================

#
# ----------------------------------------------------------------
# check for correct synopsis.
#
if [ -z "$IPADDR" ]
then
	help
fi


#
# ----------------------------------------------------------------
# functions
#
trc_echo() {
    echo "$*" 
    echo "$*" >>$INPUT
}

#==============================================================================
# BDFhw14578 - Adapt network summary script for R5 <revuri.reddy@wipro.com>
#==============================================================================
trc1_echo() {
    echo "$*"
    echo "$*" >>$INPUT1
}

trc_sleep() {
    echo "sleeping $1"
    sleep $1
}


#
GREPOPTION=""
grep --version >/dev/null 2>&1
if [ $? -eq 0 ]
then
	GREPOPTION="--text"
fi


#
#
#
if [ $TIMINGTEST -eq 0 ]
then

#
# ----------------------------------------------------------------
# patchLevel verification (just to be sure)
# (octopus crashing on 105181-16 or 105181-17)
#
    solver=`uname -v | cut -f2 -d'_'`
    if [ "$solver" = "105181-16" -o "$solver" = "105181-17" ]
    then
    	echo "You are running a solaris patch release ($solver) which will "
    	echo "crash the system while running this script ... bailing out !!!"
    	echo "Please upgrade your solaris patches."
    	exit 1
    fi


#
# ----------------------------------------------------------------
# verify ASAM connectivity
#
    platform=`uname`
    if [ "$platform" = "Linux" ] ; then
      /bin/ping -c 2 $IPADDR >/dev/null 2>&1 
    else
    /usr/sbin/ping $IPADDR >/dev/null 2>&1
    fi

    if [ $? -gt 0 ]
    then
       	echo "ASAM $IPADDR not reachable"
        echo "ASAM $IPADDR not reachable"  > $IPADDR
        echo "echo host = `hostname`"     >> $IPADDR
        exit 1
    fi


#
# ----------------------------------------------------------------
# start the octopus
#
#    which octopus >/dev/null 2>&1
#    if [ $? -gt 0 ]
#    then
#       	echo "octopus is not in the path"
#        exit 1
#    fi
	
    rm -f $LOGFILE      >/dev/null 2>&1
    rm -f $INPUT        >/dev/null 2>&1

    touch $LOGFILE

    $OCTOPUSPATH/octopus $IPADDR:udp:23 \
    	$LOGFILE\#out,append \
    	PSEUDO:$INPUT\#i \
    	PSEUDO:$MGT\#m &
    trc_sleep 5


fi	# TIMINGTEST


#
# ----------------------------------------------------------------
# reset cleanup trap
#
trap " \

echo 'exit' >> $INPUT1
echo 'quit' >> $MGT1
rm -f $INPUT1
rm -f $LOGFILE1
rm -f $MGT1 
echo 'exit' >> $INPUT;\
echo 'cleaning up ...';\
echo 'quit' >> $MGT ;\
rm -f $INPUT;\
rm -f $MGT;\
exit $1;\
" 0 1 2 3 15


#
# ----------------------------------------------------------------
# log in
#
echo "logging in"
echo "" >>$INPUT
trc_sleep 2
echo "" >>$INPUT
trc_sleep 2
if [ "$platform" = "Linux" ] ; then
  /bin/echo -ne "shell\r" >>$INPUT
else
  /bin/echo "shell\r\c" >>$INPUT
fi
trc_sleep 2
if [ "$platform" = "Linux" ] ; then
  /bin/echo -ne "nt\r" >>$INPUT
else
  /bin/echo "nt\r\c" >>$INPUT
fi
trc_sleep 10
echo "" >>$INPUT
trc_sleep 2
# BDFhw32819 - Extra delay added
trc_sleep 10

if [ $TIMINGTEST -eq 0 ]
then
    fgrep "T&D console " $LOGFILE  >/dev/null 2>&1
    if [ $? -gt 0 ]
    then
        echo login NT failed
        echo "ISAM $IPADDR login not possible"  >> $IPADDR
        echo "echo host = `hostname`"           >> $IPADDR
        exit 1
    fi
fi


#
trc_echo ""
trc_sleep 1

#   >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#  /            Here goes the non-generic stuff                  /

LOCAL=`grep $GREPOPTION '^{nt' $LOGFILE | sed '
1,$ s/\-.*//g
1,$ s/.*\[//g
' | uniq`

#
# no inactivity timeout, since we're polling this ...

trc_echo "login inactivity 0"
echo "Start Executing T & D Commands for release 5.X"
       
#
trc_echo "dbg suspend $LOCAL"
trc_sleep 30

trc_echo "dir /Sw"
trc_sleep 10

trc_echo "dbg resume EPRT"
trc_sleep 1


#
# ----------------------------------------------------------------
# kill off octopus - trapped
#


#

