=head1 PBMT Errorcodes module

=begin html
This PERL module defines the error and exit codes used by the various PBMT tools.

=end html

=cut

#==============================================================================
#-- File:         ErrorCodes.pm
#-- Date:         08.03.2006
#-- Company:      Alcatel-Lucent
#-- Author:       Bart Bogaert
#------------------------------------------------------------------------------
#-- Function:     Error codes generated by the Push-button toolset
#------------------------------------------------------------------------------
#-- Disclaimer:   This script contains references to proprietary information
#--               which is owned by Alcatel.
#------------------------------------------------------------------------------
#-- Copyright:    COPYRIGHT Alcatel 2006
#--               All rights reseved.
#------------------------------------------------------------------------------
#-- History:
#--   1.0.0 - 2006/03/06 : First implementation
#--   1.2.0 - 2006/03/22 : Introduction of capability model.
#--   1.4.3   - 2006/10/18 : - Timestamp logging
#--                          - AMS/AWS integration
#--   1.4.3.4 - 2006/11/13 : ANTms22321 - transaction timeour values in units of msecs.
#--   1.4.3.5 - 2006/11/15 : Prepare for R3.2
#--   1.5.0   - 2006/12/11 : ANTms14531: pre- and post config/status checks
#--   1.6.1   - 2007/02/28 : ANTms35659 - Added code to handel SFP provisioning when
#--                          migrating from R 2.4 or 2.5 to R 3.3 only
#--                          [Implemented by Haq Furqan]
#--   1.6.2   - 2007/03/29 : ANTms40542: check on error code when reseting transaction
#--                          owner
#--   1.7.0   - 2007/05/02 : ANTms45206: prepare for R3.4/R3.5
#--   1.7.1   - 2007/08/25 : ANTms55192: Reset NT using T&D in case the NT is still
#--                          running after an OSWP activation.
#--   1.7.2   - 2007/10/30 : ANTms60696. \r in SWP file causes trouble.
#--   1.8.0   - 2007/09/12 : ISAM-v offline migration.
#--   1.8.1   - 2008/01/18 : ANTms63550: Added a new function to configure access operatorId through cli
#--   1.8.2   - 2008/02/07 : ANTms83783
#--   1.8.3   - 2008/04/25 : ANTmt04759
#--   1.8.4   - 2008/06/03 : ANTmt09937: New TDC script
#--   1.8.5   - 2008/07/24 : ANTmt17699.  Add robustness for the FT (SIGPIPE) migration failure.
#--             2008/08/21 : ANTmt16940.
#--   1.8.6   - 2008/10/01 : Workaround for ANTmt25573.
#--           - 2008/10/02 : ANTmt23353.  Implement workaround for random initvalues in vlanStaTbl definition.
#--                                       Set init-value to 0 for vSysMod, vStaPri and vVerNbr if init-value not set to 0
#--   1.9.0   - 2008/10/03 : R3.7.10/R4.0
#--   1.9.1   - 2009/03/13 : ANTmt47687.  SHUB OFLMT 1.49 required for migrationm to R4.0.
#--   1.9.2   - 2009/03/18 : ANTmt48688.
#--   1.9.3   - 2009/04/29 : New codes for NANT-A to NANT-D convertion.
#--   1.9.4   - 2009/05/06 : ANTmt54684
#--   1.9.5   - 2009/05/30 : ANTmt60596.  Modified some error code strings (change NW VLAN).
#--   1.9.6   - 2009/06/02 : ANTmt61239 Added 1540LS family
#--   1.9.7   - 2009/06/11.
#--   1.9.8   - 2009/07/24 : ANTmt71238: ECNTE Migration error codes added
#--                          ANTmt70751: solve minor warnings occuring during script run.
#--   1.9.9   - 2009/08/27 : New error codes w.r.t. file count in migrated DB.
#--   1.9.10  - 2009/11/13 : ANTmt82666 new error code added.
#--   3.0.00  - 2010/04/21 : Converted version to 3.0.00 for converged PBMT 
#--   3.0.01  - 2010/05/24 : Converged PBMT Changes
#--   3.0.02  - 2010/07/06 : Add h248 to SIP error codes
#--                          Add h248 to SIP (voice IP inventory) 
#--                          error codes.
#--   3.0.03  - 2010/10/18 : ALU00813833 new error code added for ONT Release Map file missing
#--   3.0.04  - 2010/12/10 : NANT-D to NANT-E
#--   3.0.05  - 2011/02/10 : ALU01033842 
#--   3.0.06  - 2011/04/08 : ALU01031457
#--   3.0.07  - 2012/07/29 : ALU01813641
#--   3.0.08  - 2012/09/13 : Adapted NANT-A2E errorcodes
#--   3.0.09  - 2013/09/23 : ALU01901387
#--   3.0.10  - 2013/10/17 : ALU01913431
#--   3.0.11  - 2014/07/15 : ALU01980121
#--   3.0.12  - 2014/07/23 : Error codes added  for new script rollback
#--   3.0.13  - 2014/09/26 : ALU02014478 
#--   3.0.14  - 2014/10/10 : ALU02016489 
#--   3.0.15  - 2014/10/10 : Change error description for snmp failure in rollback
#--   3.0.16  - 2014/11/04 : ALU02020212 
#--   3.0.17  - 2014/11/19 : add error description for passive OSWP empty in rollback
#--   3.0.18  - 2014/12/02 : new error code for rollback swpfile option
#--   3.0.19  - 2015/01/27 : ALU02042831: SW Reduce not displaying proper error when Control card waiting for SW
#--   3.0.20  - 2015/02/10 : ALU02045085 
#--   3.0.21  - 2015/02/13 : ALU02046572
#--   3.0.22  - 2015/03/26 : ALU02055435
#--   3.0.23  - 2015/05/04 : ALU02062664
#--   3.0.24  - 2015/05/12 : Error message misspelling correction
#--   3.0.25  - 2015/06/10 : ALU02087591 - Secure Debug via SSH implementation 
#--   3.0.26  - 2015/10/13 : ALU02135551
#--   3.0.27  - 2016/01/28 : ALU02169438
#--   3.0.28  - 2016/03/16 : ALU02190558
#--   3.0.29  - 2016/04/28 : ALU02206611
#--   3.0.30  - 2016/05/23 : ALU02216697
#--   3.0.31  - 2016/07/27 : ALU02240675
#--   3.0.32  - 2016/08/17 : ALU02253040
#--   3.0.33  - 2016/09/12 : ALU02271083
#--   3.1.0  - 2018/06/20 : ALU02460863 & ALU02471697

=head1 VERSION

=begin html
Current version: "ErrorCodes.pm -- 3.1.0 -- 2018/06/20"

=end html

=cut

my $VERSION = "ErrorCodes.pm -- 3.1.0 -- 2018/06/20";
#==============================================================================

package ErrorCodes;

use strict;
use warnings 'all';

#==============================================================================
#== Prototypes
#==============================================================================
sub GetVersion();
sub WarningLevelString($);
sub WarningLevelIsError($);
sub WarningLevelIsWarning($);
sub WarningLevelIsIgnore($);
sub ExitString($);
sub CvtErrorString($);
sub ErrorString($);
sub IsErrorUsedIIn($$);
sub IsPreCheckError($);
sub IsSwDlError($);
sub IsMigrationError($);
sub IsUpgradeError($);
sub IsCommitError($);
sub IsStatusCheckError($);
sub IsStatusCompareError($);
sub IsSwReduceError($);
sub IsSwMinimizeError($);
sub MakePrckWarningFile($$);
sub MakeChckWarningFile($$);

use vars qw(
	    $VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS
	    @EXITCODES @COMMON_ERRORS
        @OCTOPUS_ERRORS
	    @SWDL_ERRORS @PRCK_ERRORS @CHKC_ERRORS @UPGR_ERRORS @MIGR_ERRORS @COMT_ERRORS @SWRD_ERRORS @SWMM_ERRORS @SCHK_ERRORS
	    @IBRG_ERRORS @CCCE_ERRORS @PPPC_ERRORS @CNWV_ERRORS @TONANTD_ERRORS @TONANTE_ERRORS @TOSIP_ERRORS @IPINV_ERRORS
        @RLBK_ERRORS @ONT_ERRORS @CLUP_ERRORS %errorCodes %exitCodes
	   );

use Exporter();
@ISA = qw(Exporter);

=head1 EXPORTED DATA

=head2 exitCodes

=begin html
A list of functions returning the associated exit code as defined in the private exitCodes hash table.<br>
To include the 'CONSTANT' into the code use the following construction:<br>
<br>
<tt>use ErrorCodes qw(:EXIT_CODES);</tt>
<br>

=end html

=cut

@EXITCODES = qw(
		SUCCESS_CODE CONDITONAL_SUCCESS_CODE EXIT_WARN_CODE EXIT_CHILD_FATAL_CODE EXIT_SIGNAL_CODE EXIT_DIED_CODE CMD_INCOMPL_CODE PARAM_CFG_NOT_OK_CODE
		CREATE_DIR_FAILED_CODE CREATE_ISS_CONF_FAIL_CODE CREATE_NVRAM_FAIL_CODE NT_RESTART_FAIL_CODE SHUB_RESTART_FAIL_CODE
		SHUB_CONF_SELECT_FAIL_CODE CHG_OSWP_FAIL_CODE CREATE_FILE_FAIL_CODE OPENDIR_FAIL_CODE FILE_OPEN_FAIL_CODE
		OSWP_FILE_NO_IP_CODE PATH_DEAD_END_CODE UPG_IMPOSSIBLE_CODE UPG_RESTRICTED_CODE NO_NT_OFLMT_VER_CODE
		NO_SHUB_OFLMT_VER_CODE NT_OFLMT_NO_EXE_CODE OCTOPUS_NO_EXE_CODE VERS_DET_FAILED_CODE CREATE_WORK_DIR_FAILED_CODE
		CREATE_NODE_FILE_FAIL_CODE CREATE_STAT_FILE_FAIL_CODE HOSTS_FILE_OPEN_FAILED_CODE SHUB_OFLMT_VERSION_2_HIGH_CODE
		CREATE_CHCK_FILE_FAILED_CODE OSWP_FILE_INV_CHARS_CODE PARAM_INV_MACMODE_CODE CALC_IP_CKSUM_NO_EXEC_CODE
		NT_OFLMT_VERSION_2_HIGH_CODE PARAM_BOARD_CONV_ERR_CODE WORK_NOT_WRITEABLE_CODE NO_IVPS_OFLMT_VER_CODE IVPS_OFLMT_NO_EXE_CODE
		IVPS_OFLMT_VERSION_2_HIGH_CODE IVPS_INV_FAILED_CODE NO_IVPS_OFLMT_CODE NO_NT_OFLMT_CODE PARAM_CFG_MISSING_CODE
		PARAM_CFG_NEEDS_UPDATE_CODE BAD_CKSUM_SYNTAX_CODE UNKNOWN_SCRIPT_CODE BAD_CKSUM_CODE OCTO_ENV_NOT_SET_CODE NT_CAP_NOTSET_CODE
		NT_CAP_ALT_NOTSET_CODE NT_SWPARSIZE_NOTSET_CODE COMMON_ENV_NOT_SET_CODE SHUB_OFLMT_VER_TOO_LOW PERL_FATAL_RUNTIME_ERR_CODE 
		INVALID_ISAM_IP_CODE UNABLE_TO_RESOLVE_HOST_NAME_CODE START_ACTIVITY_CODE SWP_FILE_OPEN_FAILED_CODE
	       );

=head2 errorCodes

=begin html
A list of functions returning the associated exit code as defined in the private errorCodes hash table.<br>
To include the 'CONSTANT' into the code use the following construction:<br>
<br>
<tt>use ErrorCodes qw(:MIGR); # includes the migration related error codes.
</tt>
<br>
<br>
The following error codes categories are defined:
<ul>
  <li>MIGR: migration related error codes</li>
  <li>SWDL: SW download related error codes</li>
  <li>PRCK: precheck related error codes</li>
  <li>CHCK: pre and post check comparison related error codes</li>
  <li>UPGR: SW upgrade related error codes</li>
  <li>COMT: SW commit related error codes</li>
  <li>SWRD: online SW reduction related error codes</li>
  <li>SWMM: SW minimization related error codes</li>
  <li>SCHK: Status check related error codes.</li>
  <li>RLBK: Rollback related error codes.</li>
</ul>

=end html

=cut

@COMMON_ERRORS = qw(
          EC_NO_PING
          EC_CONFIG_LOCK_ALREADY_TAKEN EC_CONFIG_LOCK_ACQUIRE_FAILED EC_CONFIG_LOCK_RELEASE_FAILED
          EC_OPENCLI_FAILED  EC_SW_RED_FAILED
          EC_OSWP_ACT_FAILED
          EC_GET_NT_INFO_FAILED EC_INCOMPAT_NT EC_NT_TYPE_UNKNOWN
          EC_GET_NT_TYPE_FAILED
          EC_TRANSFER_SNMPGET_FAILED EC_TRANSFER_WAITTIME_EXCEED 
          );

@OCTOPUS_ERRORS = qw( EC_OCTO_ACTIVE EC_OCTO_FAIL EC_OCTO_PS_ERROR );

@SWDL_ERRORS = qw(
          EC_ABORT_DL_FAIL EC_ALW_ALARM_FAILED EC_COMMIT_FAIL EC_CONFIG_DL_FAILED EC_DB_VER_UNDEFINED
          EC_DEL_SHUB_SW_FAILED EC_DETERMINE_LOG_FAILED EC_DIR_SW_LIST_FAILED EC_DL_SWP_FAILED EC_ENABLE_WR_FAILED
          EC_FILEXFRMODE_NOTSUPPORTED EC_GET_ACC_BITS_FAILED EC_GET_DB_INFO_FAILED EC_GET_FILEXFR_FAILED EC_GET_LT_FAILED
          EC_GET_OSWP_FAILED EC_GET_SHUB_CONF_FAILED EC_GET_SW_PARSIZE_FAILED
          EC_INHBT_MORE_FAILED EC_INH_ALARM_FAILED EC_LIST_SWFILES_FAILED EC_LT_NOT_SUPPORTED
          EC_MIG_NOT_ADVISED EC_MIG_RESTRICTED EC_MISSING_FAILED EC_NEW_OSWP_FILE_FAILED EC_NO_LT_SWP_FOUND EC_NO_PASS_SWPIDX_FOUND
          EC_NO_SHUB_SW EC_NO_SNMP_VERSION_SEL EC_NO_SSH_POSS EC_NTTYPE_MAPFILE_FAILED EC_NTTYPE_OSWP_FAILED
          EC_NTTYPE_SWPNAME_FAILED EC_NT_NOT_IN_SWP EC_NT_NOT_SUPPORTED EC_OCTO_ACTIVE EC_OCTO_PS_ERROR
          EC_OPENCLI_FAILED EC_OPENTL1_FAILED EC_OSWP_MISSING EC_PARSE_SWP_FAILED EC_PAR_SIZE_TOO_SMALL EC_REDUCE_OPR_NOTALLOW
          EC_REL_CVT_FAILED EC_SHUB_NO_TFTP_BOOT EC_SHUB_SW_DL_FAILED EC_SNMP_MAX_SIZE_FAIL EC_SNMP_NO_GET
          EC_SNMP_SESS_ERR EC_SW_RED_FAILED EC_SYNC_TO_STBY_FAILED EC_TFTP_MISSING_FAILED EC_TFTP_OPEN_FAILED EC_TFTP_PUT_FAILED
          EC_UPGR_NOT_ADVISED EC_UPGR_RESTRICTED EC_WRONG_VARIANT
		 );

@UPGR_ERRORS = qw(
		EC_ACC_OPERID_MISMATCH EC_CONFIG_OPRID_FAILED EC_DETERMINE_LOG_FAILED EC_DETERMINE_OPER_FAILED
        EC_EMERGENCY_CALL_ACTIVE EC_GET_ACC_BITS_FAILED EC_GET_ACC_OPERID_FAILED 
        EC_GET_OSWP_FAILED EC_INHBT_MORE_FAILED EC_INIT_COPY_FAILED EC_MAP_ERR_REC_FAILED EC_NEEDS_MIGRATION
        EC_NEW_SWP_NOT_ENABLED EC_NEW_SWP_NOT_PRESENT EC_NEW_SW_NOT_ACT EC_NO_SNMP_VERSION_SEL EC_NO_SSH_POSS
        EC_NTTYPE_MAPFILE_FAILED EC_NT_DB_BU_FAIL EC_NT_NOT_SUPPORTED EC_OCTO_ACTIVE EC_OCTO_PS_ERROR
        EC_OPENCLI_FAILED EC_POST_UPGR_ERROR EC_PRE_UPGR_ERROR EC_RESET_NT_FAILED
        EC_RESTART_FAILED EC_SETUP_ISSU_FAILED EC_SET_TRANSACTION_FAILED EC_SNMP_MAX_SIZE_FAIL EC_SNMP_NO_GET EC_SNMP_SESS_ERR 
        EC_SW_NAME_ERROR EC_UPGR_NOT_ADVISED EC_UPGR_RESTRICTED
		 );

@MIGR_ERRORS = qw(
		EC_ACC_OPERID_MISMATCH EC_CDE_NOTFOUND EC_CONFIG_OPRID_FAILED EC_COPY_OFLDB_FAILED 
        EC_CORR_VLAN_TBL_FAILED EC_CREATE_ISS_FAIL EC_CREATE_NVR_FAIL EC_DB_AUTO_ACTIVATE_ON_DWLD EC_DB_COMBINE_FAILED 
        EC_DB_COUNT_FILES_FAILED EC_DB_DL_FAILED EC_DB_TOO_MANY_FILES EC_DB_VERS_UNEXPECTED EC_DEL_SHUB_SW_FAILED EC_DETERMINE_LOG_FAILED
        EC_DETERMINE_OPER_FAILED EC_EMERGENCY_CALL_ACTIVE EC_EMPTY_NT_DETECTED EC_ENV_NOT_OK EC_EXTR_MCAST_FAILED 
        EC_FILEXFRMODE_NOTSUPPORTED EC_GETCOMMTY_ERROR EC_GET_ACC_BITS_FAILED EC_GET_ACC_OPERID_FAILED EC_GET_CLIMODE_FAILED
        EC_GET_DB_INFO_FAILED EC_GET_FILEXFR_FAILED EC_GET_OSWP_FAILED EC_GET_SFP_ASSIGNMENT_FAILED
        EC_GET_TRANS_TO_FAILED EC_INCOMPAT_NT_ICS_STATE EC_INCORR_SHUB_OFLMT EC_INCORR_SHUB_OFLMT_NANT_B
        EC_INHBT_MORE_FAILED EC_INIT_COPY_FAILED EC_INPUTDB_PROBLEM EC_INVALID_2_ECNT_C EC_INVALID_2_ECNT_E EC_INVALID_2_NANT_B
        EC_ISAMV_OFLMIG_NO_CONF EC_ISAM_HAS_VOICE EC_ISAM_NT_SYNC_FAILED EC_IVPS_ADD_CDE_FAILED EC_IVPS_DB_FAIL EC_IVPS_GETDEP_ERROR
        EC_IVPS_GETIP_ERROR EC_IVPS_MIG_FAILED EC_MAC_LIMITS_EXCEEDED EC_MAP_ERR_REC_FAILED EC_MIG_NOT_ADVISED EC_MIG_RESTRICTED
        EC_MKDIR_FAILED EC_NEEDS_UPGRADE EC_NEW_CDE_NOTFOUND EC_NEW_SW_NOT_ACT EC_NO_ALT_NT_CAP EC_NO_NT_CAP 
        EC_NO_SNMP_VERSION_SEL EC_NO_SSH_POSS EC_NO_TTL0_FWD EC_NTTYPE_MAPFILE_FAILED EC_NT_DB_BU_FAIL EC_NT_DB_COPY_FAILED EC_NT_DB_EMPTY
        EC_NT_DB_NOT_PRESENT EC_NT_MIG_FAILED EC_NT_NOT_ECNT_A EC_NT_NOT_ECNT_A_OR_C EC_NT_NOT_NANT_A EC_NT_NOT_SUPPORTED EC_NT_SHUB_SW_MM
        EC_OCTO_ACTIVE EC_OCTO_PS_ERROR EC_OFLDB_NOTREQ EC_OPENCLI_FAILED EC_OPEN_WARN_FILE_FAILED 
        EC_POST_UPGR_ERROR EC_PRE_UPGR_ERROR EC_RESET_NT_FAILED EC_RESTART_FAILED EC_SETUP_ISSU_FAILED
        EC_SET_SFP_ASSIGNMENT_FAILED EC_SET_TRANSACTION_FAILED EC_SET_TRANS_TO_FAILED EC_SHOW_EQPT_FAILED EC_SHUB_BU_FAIL
        EC_SHUB_DB_DL_FAILED EC_SHUB_DB_SAVE_FAIL EC_SHUB_MIG_FAILED EC_SHUB_REBOOT_FAILED EC_SHUB_SET_NEW_CONF_FAILED EC_SHUB_SW_NOT_FOUND
        EC_SW_CHECKS_NOK EC_VMAC_HOST_ID_LIMITS_EXCEED EC_SW_CHECKS_NOK EC_UNKNOWN_NE_FAM
        EC_SNMP_NO_GET EC_SNMP_MAX_SIZE_FAIL EC_SNMP_SESS_ERR EC_TFTP_SUSPEND_FAILED EC_TTL_0_FWD_FAILED
);

@SWRD_ERRORS = qw(
		EC_ACTIVATE_FAIL EC_COMMIT_FAIL EC_CREATE_OSWP_FAIL EC_CREATE_SWP_FAIL EC_GET_ACTIVATE_FAIL EC_GET_COMMIT_FAIL 
        EC_GET_NEW_FREE_FAIL EC_GET_OSWP_FAIL EC_GET_OSWP_NAME_FAIL EC_NO_ACT_OSWP EC_NO_ACT_OSWP_2ACTIVATE EC_NO_ACT_SWP
        EC_NO_ASAMCORE_FOUND EC_NO_BOARDS EC_NO_FREE_SPACE EC_NO_MIB_VERSION EC_OCTO_FAIL EC_OPEN_OSWP_FAIL EC_OPEN_OSWP_NEW_FAIL
        EC_OPEN_SWP_FAIL EC_OSWP_OPER_FAIL EC_RESET_TRANSACTION_FAILED EC_SEC_SSH_ENABLE_WRITE_FAIL EC_SEC_SSH_REMOVE_SWPKG_FAIL
        EC_SET_COMMIT_FAIL EC_SET_OSWP_FAIL EC_SET_TRANSACTION_FAILED EC_SNMP_ACT_FAIL EC_SUCCESS EC_TFTP_OPEN_FAIL 
        EC_XFTP_GET_OSWP_FAIL EC_XFTP_GET_SWP_FAIL EC_XFTP_PUT_OSWP_FAIL EC_XFTP_PUT_SWP_FAIL
	 );

@SWMM_ERRORS = qw(
		  EC_CMD_INCOMPL EC_CONF_FILE_READ_FAILED EC_COPY_FAILED EC_CREATE_OSWP_FAIL EC_CREATE_SWP_FAIL EC_CREATE_TARGET_DIR_FAILED 
          EC_NO_MIB_VERSION EC_NTTYPE_MAPFILE_FAILED EC_NTTYPE_OSWP_FAILED EC_NTTYPE_SWPNAME_FAILED EC_OPEN_OSWP_FAIL EC_OPEN_SWP_FAIL
          EC_OSWP_FILE_NO_IP EC_PARAM_CFG_NOT_OK EC_SNMP_OPEN_FAIL EC_SUCCESS
		 );

@COMT_ERRORS = qw(
		EC_COMMIT_FAILED EC_DETERMINE_LOG_FAILED EC_GET_ACC_BITS_FAILED 
        EC_GET_OSWP_FAILED EC_INHBT_MORE_FAILED EC_NEW_SW_NOT_ACT EC_NO_SNMP_VERSION_SEL EC_NO_SSH_POSS
        EC_NT_NOT_SUPPORTED EC_OPENCLI_FAILED EC_SNMP_NO_GET EC_SNMP_SESS_ERR 
		 );

@PRCK_ERRORS = qw(
            EC_CDE_NOTFOUND EC_CREATE_RESTOREDIR_FAILED EC_DB_AUTO_ACTIVATE_ON_DWLD EC_DETERMINE_LOG_FAILED 
            EC_EMERGENCY_CALL_ACTIVE EC_FILEXFRMODE_NOTSUPPORTED EC_GET_ACC_BITS_FAILED EC_GET_CLIMODE_FAILED EC_GET_DB_INFO_FAILED 
            EC_GET_FILEXFR_FAILED EC_GET_LT_FAILED EC_GET_LT_INFO_FAILED EC_GET_OSWP_FAILED 
            EC_GET_SHUB_CONF_FAILED EC_GET_SHUB_SWSIZE_FAILED EC_GET_SW_PARSIZE_FAILED EC_INHBT_MORE_FAILED EC_ISAM_HAS_VOICE 
            EC_LT_NOT_SUPPORTED EC_MAC_LIMITS_EXCEEDED EC_MAKE_KEYFILE_FAILED EC_MIG_NOT_ADVISED EC_MIG_RESTRICTED EC_NEW_SW_PRESENT 
            EC_NO_ALT_NT_CAP EC_NO_NT_CAP EC_NO_SNMP_VERSION_SEL EC_NO_SSH_POSS EC_NTTYPE_MAPFILE_FAILED EC_NTTYPE_OSWP_FAILED
            EC_NTTYPE_SWPNAME_FAILED EC_NT_ICS_ISSUE EC_NT_NOT_IN_SWP EC_NT_NOT_SUPPORTED EC_OCTO_ACTIVE 
            EC_OCTO_PS_ERROR EC_OPENCLI_FAILED EC_PAR_SIZE_TOO_SMALL EC_SETUP_ISSU_FAILED EC_SHOW_EQPT_FAILED EC_SHUB_NO_TFTP_BOOT 
            EC_SNMP_CLIENT_CREDENTIAL_MISMATCH EC_SNMP_MAX_SIZE_FAIL EC_SNMP_NO_GET EC_SNMP_SESS_ERR EC_SW_MGNT_SIZE_NOK EC_TOO_MANY_MC_SRC_PKG
            EC_UPGR_NOT_ADVISED EC_UPGR_RESTRICTED EC_UPLOAD_DB_FAILED EC_VMAC_HOST_ID_LIMITS_EXCEED
		 );

@CHKC_ERRORS = qw(
	    EC_AUTHOOS_THR_EXCEEDED EC_BRIDGECONFMM_THR_EXCEEDED EC_DETERMINE_LOG_FAILED EC_INVALID_FILE_CONTENT EC_LINEOOS_THR_EXCEEDED
        EC_LTOOS_THR_EXCEEDED EC_NEW_ALARM EC_NO_INPUT_DIR EC_NO_STATUS_FILE EC_ONTOOS_THR_EXCEEDED EC_ONT_RELMAP_MISSING 
        EC_OPEN_POST_CHECK_FAILED EC_OPEN_PRE_CHECK_FAILED EC_RTRV_REL_FAILED EC_RTRV_REL_INFO_FAILED EC_SNMP_MAX_SIZE_FAIL
        EC_SNMP_NO_GET EC_SNMP_SESS_ERR EC_THRESHOLD_WARN EC_THRESHOLD_WARN_EXCEED
		 );

@SCHK_ERRORS = qw(
		EC_ALW_ALARM_FAILED EC_CLI_PREPARE_FAILED EC_DETERMINE_LOG_FAILED EC_GET_ACC_BITS_FAILED EC_GET_ALARM_FAILED 
        EC_GET_AUTHENT_FAILED EC_GET_BRIDGE_FAILED EC_GET_CLIMODE_FAILED EC_GET_LT_FAILED EC_GET_OSWP_FAILED
        EC_GET_XDSL_FAILED EC_INH_ALARM_FAILED EC_ISAM_TRANSACT_TAKEN 
        EC_NO_SNMP_VERSION_SEL
        EC_NO_SSH_POSS EC_NT_NOT_SUPPORTED EC_OPENCLI_FAILED EC_OPENTL1_FAILED EC_RTRV_ONT_EMPTY EC_RTRV_ONT_FAILED EC_RTRV_REL_FAILED
        EC_SET_SESSION_TO_FAILED EC_SNMP_MAX_SIZE_FAIL EC_SNMP_NO_GET EC_SNMP_SESS_ERR
		 );

@RLBK_ERRORS = qw(
			EC_DETERMINE_LOG_FAILED EC_DOWNLOAD_DB_FAILED EC_EMERGENCY_CALL_ACTIVE EC_ETSI_NOT_SUPPORTED
            EC_GET_ACC_BITS_FAILED EC_GET_OSWP_FAILED EC_HIGHER_PASSIVE_RESTRICTED 
            EC_NEW_SWP_NOT_ENABLED EC_NEW_SW_NOT_ACT EC_NOT_SUPPORTED_LOWER_RELEASE EC_NO_SSH_POSSIBLE EC_NT_NOT_INSYNC 
            EC_NT_NOT_SUPPORTED EC_OPENCLI_FAILED EC_PASSIVE_DB_FAILED EC_PASSIVE_DB_FAILED EC_PASSIVE_OSWP_EMPTY
            EC_PASSIVE_SWP_NOT_COMMITTED EC_PASSIVE_SWP_NOT_ENABLED EC_PING_NO_RESP EC_PRECHECK_NOTRUN EC_SETUP_ISSU_FAILED
            EC_SET_TRANSACTION_FAILED EC_SNMP_MAX_SIZE_FAIL EC_SNMP_NO_GET EC_SNMP_SESS_ERR EC_SWPFILE_NOT_SUPPORTED 
            EC_TARGET_DIFF_FROM_ACTIVE_OSWP EC_TARGET_DIFF_FROM_PASSIVE_OSWP 
            EC_RESTART_FAILED 
          );

@IBRG_ERRORS = qw(
		EC_BACKUPDB_FAILED EC_BATCHMODE_FAILED EC_CREATE_CONFIG_ERROR EC_DEL_CONFIG_ERROR EC_DHCPRELAY_PARSE_ERROR
        EC_EMPTY_RESPONSE EC_GET_ACC_BITS_FAILED EC_GET_ATMPVC_ERROR EC_GET_BRIDGEINFO_ERROR EC_GET_IGMPCH_INFO_ERROR 
        EC_INFO_CONF_DHCPRELAY_ERROR EC_INFO_CONF_QOS_ERROR EC_INFO_CONF_VLANBIND_ERROR EC_INFO_CONF_VLAN_ERROR EC_INFO_CONF_VRF_ERROR
        EC_INHIBIT_ALRM_FAILED EC_INVALID_CONFIG EC_INV_MIB_VERSION EC_NO_SSH_POSS EC_NO_SUCH_VLAN EC_NO_SUCH_VRF 
        EC_OPENCLI_FAILED EC_PPPOESESS_PARSE_ERROR EC_PRNT_NOMORE_FAILED EC_PROMPT_FAILED EC_QOS_PARSE_ERROR EC_RELEASE_TOO_LOW
        EC_ROLLBACK_ERROR EC_SET_TIMEOUT_FAILED EC_SHOW_PPPOESESS_ERROR EC_SHUB_VLAN_PARSE_ERROR EC_SNMP_NO_GET 
        EC_SNMP_SESS_ERR EC_VLANBIND_PARSE_ERROR EC_VLAN_PARSE_ERROR EC_VRF_PARSE_ERROR
		 );

@CCCE_ERRORS = qw(
		EC_BACKUPDB_FAILED EC_BATCHMODE_FAILED EC_BRIDGEPORT_PARSE_ERROR EC_CLIENTPORT_PARSE_ERROR
        EC_CREATE_CONFIG_ERROR EC_DEL_CONFIG_ERROR EC_EMPTY_RESPONSE EC_ENGINE_PARSE_ERROR EC_GET_ACC_BITS_FAILED
        EC_INFO_CONF_BRIDGEPORT_ERROR EC_INFO_CONF_CLIENTPORT_ERROR EC_INFO_CONF_ENGINE_ERROR EC_INFO_CONF_VLAN_PP_ERROR
        EC_INHIBIT_ALRM_FAILED EC_INV_MIB_VERSION EC_MAC_CONC_ENABLED EC_NO_SSH_POSS EC_NO_SUCH_BRIDGEPORT 
        EC_NO_SUCH_CLIENTPORT EC_NO_SUCH_ENGINE EC_NO_SUCH_VLAN_PP EC_OPENCLI_FAILED EC_PRNT_NOMORE_FAILED EC_RELEASE_TOO_LOW
        EC_ROLLBACK_ERROR EC_SET_TIMEOUT_FAILED EC_SNMP_NO_GET EC_SNMP_SESS_ERR 
        EC_VLAN_PP_PARSE_ERROR
		 );

@PPPC_ERRORS = qw(
		EC_BACKUPDB_FAILED EC_BATCHMODE_FAILED EC_CLIENTPORT_PARSE_ERROR EC_CREATE_CONFIG_ERROR EC_DEL_CONFIG_ERROR 
        EC_EMPTY_RESPONSE EC_GET_ACC_BITS_FAILED EC_INFO_CONF_CLIENTPORT_ERROR EC_INFO_CONF_INTERFACE_ERROR EC_INFO_CONF_PVCCONFIG_ERROR
        EC_INFO_CONF_VLAN_ERROR EC_INHIBIT_ALRM_FAILED EC_INV_MIB_VERSION EC_ITFCONFIG_PARSE_ERROR EC_NO_SSH_POSS
        EC_NO_SUCH_CLIENTPORT EC_NO_SUCH_IPVLAN EC_NO_SUCH_PPPVLAN EC_OPENCLI_FAILED EC_PRNT_NOMORE_FAILED EC_PVCCONFIG_PARSE_ERROR
        EC_RELEASE_TOO_LOW EC_ROLLBACK_ERROR EC_SET_TIMEOUT_FAILED EC_SNMP_NO_GET EC_SNMP_SESS_ERR 
		 );

@CNWV_ERRORS = qw(
		EC_BACKUPDB_FAILED EC_BATCHMODE_FAILED EC_BRIDGEPORT_PARSE_ERROR EC_CREATE_CONFIG_ERROR EC_DEL_CONFIG_ERROR
        EC_EMPTY_RESPONSE EC_GET_ACC_BITS_FAILED EC_INFO_CONF_BRIDGEPORT_ERROR EC_INFO_CONF_CCPORT_ERROR EC_INFO_CONF_MCAST_ERROR
        EC_INFO_CONF_NTVRF_ERROR EC_INFO_CONF_NT_VLAN_ERROR EC_INFO_CONF_SHUBVLAN_ERROR EC_INFO_CONF_SHUBVRF_ERROR EC_INFO_CONF_SHUB_BRG_ERROR
        EC_INFO_CONF_SHUB_VLAN_ERROR EC_INFO_CONF_VLANPORT_ERROR EC_INFO_CONF_VLANPP_ERROR EC_INHIBIT_ALRM_FAILED EC_INV_MIB_VERSION
        EC_INV_NT_RB_TYPE EC_INV_SHUB_RB_TYPE EC_MCASTSRC_PARSE_ERROR EC_NO_SSH_POSS EC_NO_SUCH_NT_VLAN EC_NO_SUCH_SHUB_VLAN 
        EC_NT_VLAN_PARSE_ERROR EC_OPENCLI_FAILED EC_PREPARE_CREATE_ERROR EC_PRNT_NOMORE_FAILED EC_RELEASE_TOO_LOW 
        EC_ROLLBACK_ERROR EC_SET_TIMEOUT_FAILED EC_SHUB_VLAN_PARSE_ERROR EC_SNMP_NO_GET EC_SNMP_SESS_ERR EC_TO_NTVLAN_EXISTS EC_TO_SHUBVLAN_EXISTS
        EC_USED_CCPORT EC_USED_NT_VRF EC_USED_SHUBPVID EC_USED_SHUB_ROUTEDEST EC_USED_VLANPP EC_VLANPORT_PARSE_ERROR EC_VLAN_USED
		 );

@IPINV_ERRORS = qw(
		EC_DETERMINE_LOG_FAILED EC_GET_VOICEIP_FAILED EC_GET_VVLAN_FAILED EC_OPENCLI_FAILED EC_SNMP_SESS_ERR
          );

@ONT_ERRORS = qw(
            EC_PING_NO_RESP EC_SNMP_SESS_ESTB_ERR EC_SNMP_NO_GET EC_GET_FTP_FAILED
			EC_LIST_PARTLY_PROCD EC_OPENTL1_FAILED  EC_ISAM_TRANSACT_TAKEN  
			EC_ISAM_ALW_ALARM_FAILED EC_ISAM_INH_ALARM_FAILED  
			EC_TRX_PROTOCOL_MISMATCH OPEN_MAPPING_FILE_FAILED EC_GET_ALL_ONT_FAILED EC_GET_WARN_TYPE_COULD_NOT_FOUND EC_GET_ACC_BITS_FAILED 
            
           EC_STAGE_TYPE_FILE_MAPPING_FAILED EC_STAGE_GET_DISK_SPACE_FAILED EC_STAGE_TELNET_OPEN_FAILED 
           EC_STAGE_TELNET_CLOSE_FAILED EC_STAGE_TELNET_INVALID_OPTIONS EC_STAGE_PUT_FAILED 
           EC_STAGE_WARN_EXCEEDS_THRESHOLD EC_STAGE_GET_ONTSWVER_FAILED 
         EC_DL_ACT_RTRV_ONTSWCTRL_FAILED EC_DL_ACT_GET_ONT_LIST_FAILED EC_DL_ACT_WARN_EXCEEDS_THRESHOLD );

@CLUP_ERRORS = qw(CLUP_DELETE_SW_FAILED CLUP_GET_ONTSWVER_FAILED CLUP_GET_ONTSWCTRL_FAILED CLUP_RTRV_ONT_FAILED);

@TONANTD_ERRORS = qw(
		     EC_NO_SNMP_VERSION_SEL EC_SNMP_SESS_ERR EC_DETERMINE_LOG_FAILED EC_OPEN_WARNFILE_FAILED
		     EC_OPENCLI_FAILED EC_NT_DB_BU_FAIL EC_NT_DB_CVT_FAIL EC_SHUB_PARSE_FAIL EC_CVT_TO_IHUB_FAIL
		     EC_SHOW_EQPT_FAIL EC_CVT_COMBINE_FAIL EC_GET_SNMP_SEC_FAIL EC_GET_CLIMODE_FAIL
		     EC_GET_SHELFCONFIG_FAIL EC_NOT_SUPPORTED EC_GET_ACC_BITS_FAILED
		    );

@TONANTE_ERRORS = qw(
		     EC_NO_SNMP_VERSION_SEL EC_SNMP_SESS_ERR EC_DETERMINE_LOG_FAILED EC_OPENCLI_FAILED EC_NT_DB_BU_FAIL
		     EC_NT_DB_CVT_FAIL EC_CVT_IHUB_FAIL EC_CVT_COMBINE_FAIL EC_GET_CLIMODE_FAIL EC_NOT_SUPPORTED
		     EC_OPEN_WARNFILE_FAILED EC_SHOW_EQPT_FAIL EC_GET_SNMP_SEC_FAIL EC_GET_SHELFCONFIG_FAIL
		     EC_SHUB_PARSE_FAIL EC_CVT_TO_IHUB_FAIL EC_GET_ACC_BITS_FAILED
		    );

@TOSIP_ERRORS = qw(
		   EC_SNMP_SESS_ERR EC_DETERMINE_LOG_FAILED EC_OPENCLI_FAILED EC_GET_CLIMODE_FAIL EC_GET_SNMP_SEC_FAIL
		   EC_NT_DB_BU_FAIL EC_NT_DB_CVT_FAIL EC_CVT_COMBINE_FAIL 
		   EC_DB_DL_FAILED EC_MAP_ERR_REC_FAILED EC_SET_TRANSACTION_FAILED EC_GET_OSWP_FAILED
		   EC_RESET_NT_FAILED EC_INIT_COPY_FAILED EC_RESTART_FAILED 
		   EC_SIP_TERM_ERROR EC_SIP_TERM_MISSING EC_GET_TFTPIP_FAIL EC_SHOW_EQPT_FAILED EC_NOT_VOICE_SRV
		   EC_PARSE_SIP_SERV_FAILED EC_PARSE_SIP_SITE_FAILED EC_GET_H248_FAILED EC_RELEASE_TOO_LOW EC_GET_OSPF_FAILED
		   EC_OPENFILE_FAILED EC_CREATE_WORKSPACE_FAILED EC_NEW_DB_MISSING EC_NEW_DB_NOT_ACTIVE EC_FILE_OPEN_FAILED
		   EC_CLI_CMD_FAILED EC_GET_SNMP_ACC_FAILED EC_SHUB_RECONFIG_MISSING EC_RB_FAILED EC_INVALID_TFTP_IP
		   EC_PREP_RECONF_FAIL
		  );





@EXPORT = ();
@EXPORT_OK = (@EXITCODES, @COMMON_ERRORS, @OCTOPUS_ERRORS, @SWDL_ERRORS, @PRCK_ERRORS, @CHKC_ERRORS, @UPGR_ERRORS, @MIGR_ERRORS, @COMT_ERRORS, @SWRD_ERRORS, @SWMM_ERRORS,
	      @SCHK_ERRORS, @IBRG_ERRORS, @CCCE_ERRORS, @PPPC_ERRORS, @CNWV_ERRORS, @TONANTD_ERRORS, @TONANTE_ERRORS,
	      @TOSIP_ERRORS, @IPINV_ERRORS, @ONT_ERRORS, @RLBK_ERRORS, @CLUP_ERRORS );

%EXPORT_TAGS = (EXIT_CODES => [@EXITCODES],
        OCTO     => [@OCTOPUS_ERRORS, @COMMON_ERRORS ],
		SWDL     => [@SWDL_ERRORS, @COMMON_ERRORS],
		PRCK     => [@PRCK_ERRORS, @COMMON_ERRORS],
		CHKC     => [@CHKC_ERRORS, @COMMON_ERRORS],
		UPGR     => [@UPGR_ERRORS, @COMMON_ERRORS],
		MIGR     => [@MIGR_ERRORS, @COMMON_ERRORS],
		COMT     => [@COMT_ERRORS, @COMMON_ERRORS],
        RLBK     => [@RLBK_ERRORS, @COMMON_ERRORS],
		SWRD     => [@SWRD_ERRORS, @COMMON_ERRORS],
		SWMM     => [@SWMM_ERRORS, @COMMON_ERRORS],
		SCHK     => [@SCHK_ERRORS, @COMMON_ERRORS],
		IBRG     => [@IBRG_ERRORS, @COMMON_ERRORS],
		CCCE     => [@CCCE_ERRORS, @COMMON_ERRORS],
		PPPC     => [@PPPC_ERRORS, @COMMON_ERRORS],
		CNWV     => [@CNWV_ERRORS, @COMMON_ERRORS],
		TONANTD  => [@TONANTD_ERRORS, @COMMON_ERRORS],
		TONANTE  => [@TONANTE_ERRORS, @COMMON_ERRORS],
		TOSIP    => [@TOSIP_ERRORS, @COMMON_ERRORS],
		IPINV    => [@IPINV_ERRORS, @COMMON_ERRORS],
        CLUP     => [@CLUP_ERRORS, @COMMON_ERRORS],
		ONT_ERR  => [@ONT_ERRORS, @COMMON_ERRORS],
        COMMON   => [@COMMON_ERRORS],
		ALL      => [@EXPORT_OK]);

# Exit code related codes
# -----------------------
sub SUCCESS_CODE()                   { $exitCodes{"SUCCESS"}[0] }
sub CONDITONAL_SUCCESS_CODE()        { $exitCodes{"CONDITONAL_SUCCESS"}[0] }
sub EXIT_WARN_CODE()                 { $exitCodes{"EXIT_WARN"}[0] }
sub EXIT_CHILD_FATAL_CODE()          { $exitCodes{"EXIT_CHILD_FATAL"}[0] }
sub EXIT_SIGNAL_CODE()               { $exitCodes{"EXIT_SIGNAL"}[0] }
sub EXIT_DIED_CODE()                 { $exitCodes{"EXIT_DIED"}[0] }
sub CMD_INCOMPL_CODE()               { $exitCodes{"CMD_INCOMPL"}[0] }
sub PARAM_CFG_NOT_OK_CODE()          { $exitCodes{"PARAM_CFG_NOT_OK"}[0] }
sub CREATE_DIR_FAILED_CODE()         { $exitCodes{"CREATE_DIR_FAILED"}[0] }
sub CREATE_ISS_CONF_FAIL_CODE()      { $exitCodes{"CREATE_ISS_CONF_FAIL"}[0] }
sub CREATE_NVRAM_FAIL_CODE()         { $exitCodes{"CREATE_NVRAM_FAIL"}[0] }
sub NT_RESTART_FAIL_CODE()           { $exitCodes{"NT_RESTART_FAIL"}[0] }
sub SHUB_RESTART_FAIL_CODE()         { $exitCodes{"SHUB_RESTART_FAIL"}[0] }
sub SHUB_CONF_SELECT_FAIL_CODE()     { $exitCodes{"SHUB_CONF_SELECT_FAIL"}[0] }
sub CHG_OSWP_FAIL_CODE()             { $exitCodes{"CHG_OSWP_FAIL"}[0] }
sub CREATE_FILE_FAIL_CODE()          { $exitCodes{"CREATE_FILE_FAIL"}[0] }
sub OPENDIR_FAIL_CODE()              { $exitCodes{"OPENDIR_FAIL"}[0] }
sub FILE_OPEN_FAIL_CODE()            { $exitCodes{"FILE_OPEN_FAIL"}[0] }
sub OSWP_FILE_NO_IP_CODE()           { $exitCodes{"OSWP_FILE_NO_IP"}[0] }
sub PATH_DEAD_END_CODE()             { $exitCodes{"PATH_DEAD_END"}[0] }
sub UPG_IMPOSSIBLE_CODE()            { $exitCodes{"UPG_IMPOSSIBLE"}[0] }
sub UPG_RESTRICTED_CODE()            { $exitCodes{"UPG_RESTRICTED"}[0] }
sub NO_NT_OFLMT_VER_CODE()           { $exitCodes{"NO_NT_OFLMT_VER"}[0] }
sub NO_SHUB_OFLMT_VER_CODE()         { $exitCodes{"NO_SHUB_OFLMT_VER"}[0] }
sub NT_OFLMT_NO_EXE_CODE()           { $exitCodes{"NT_OFLMT_NO_EXE"}[0] }
sub OCTOPUS_NO_EXE_CODE()            { $exitCodes{"OCTOPUS_NO_EXE"}[0] }
sub VERS_DET_FAILED_CODE()           { $exitCodes{"VERS_DET_FAILED"}[0] }
sub CREATE_WORK_DIR_FAILED_CODE()    { $exitCodes{"CREATE_WORK_DIR_FAILED"}[0] }
sub CREATE_NODE_FILE_FAIL_CODE()     { $exitCodes{"CREATE_NODE_FILE_FAIL"}[0] }
sub CREATE_STAT_FILE_FAIL_CODE()     { $exitCodes{"CREATE_STAT_FILE_FAIL"}[0] }
sub HOSTS_FILE_OPEN_FAILED_CODE()    { $exitCodes{"HOSTS_FILE_OPEN_FAILED"}[0] }
sub SHUB_OFLMT_VERSION_2_HIGH_CODE() { $exitCodes{"SHUB_OFLMT_VERSION_2_HIGH"}[0] }
sub CREATE_CHCK_FILE_FAILED_CODE()   { $exitCodes{"CREATE_CHCK_FILE_FAILED"}[0] }
sub OSWP_FILE_INV_CHARS_CODE()       { $exitCodes{"OSWP_FILE_INV_CHARS"}[0] }
sub PARAM_INV_MACMODE_CODE()         { $exitCodes{"PARAM_INV_MACMODE"}[0] }
sub CALC_IP_CKSUM_NO_EXEC_CODE()     { $exitCodes{"CALC_IP_CKSUM_NO_EXEC"}[0] }
sub NT_OFLMT_VERSION_2_HIGH_CODE()   { $exitCodes{"NT_OFLMT_VERSION_2_HIGH"}[0] }
sub PARAM_BOARD_CONV_ERR_CODE()      { $exitCodes{"PARAM_BOARD_CONV_ERR"}[0] }
sub WORK_NOT_WRITEABLE_CODE()        { $exitCodes{"WORK_NOT_WRITEABLE"}[0] }
sub NO_IVPS_OFLMT_VER_CODE()         { $exitCodes{"CREATE_FILE_FAIL"}[0] }
sub IVPS_OFLMT_NO_EXE_CODE()         { $exitCodes{"IVPS_OFLMT_NO_EXE"}[0] }
sub IVPS_OFLMT_VERSION_2_HIGH_CODE() { $exitCodes{"IVPS_OFLMT_VERSION_2_HIGH"}[0] }
sub IVPS_INV_FAILED_CODE()           { $exitCodes{"IVPS_INV_FAILED"}[0] }
sub NO_IVPS_OFLMT_CODE()             { $exitCodes{"NO_IVPS_OFLMT"}[0] }
sub NO_NT_OFLMT_CODE()               { $exitCodes{"NO_NT_OFLMT"}[0] }
sub PARAM_CFG_MISSING_CODE()         { $exitCodes{"PARAM_CFG_MISSING"}[0] }
sub PARAM_CFG_NEEDS_UPDATE_CODE()    { $exitCodes{"PARAM_CFG_NEEDS_UPDATE"}[0] }
sub BAD_CKSUM_SYNTAX_CODE()          { $exitCodes{"BAD_CKSUM_SYNTAX"}[0] }
sub UNKNOWN_SCRIPT_CODE()            { $exitCodes{"UNKNOWN_SCRIPT"}[0] }
sub BAD_CKSUM_CODE()                 { $exitCodes{"BAD_CKSUM"}[0] }
sub SHUB_OFLMT_VER_TOO_LOW()         { $exitCodes{"SHUB_OFLMT_VER_TOO_LOW"}[0] }
sub OCTO_ENV_NOT_SET_CODE()          { $exitCodes{"OCTO_ENV_NOT_SET"}[0] }
sub NT_CAP_NOTSET_CODE()             { $exitCodes{"NT_CAP_NOTSET"}[0] }
sub NT_CAP_ALT_NOTSET_CODE()         { $exitCodes{"NT_CAP_ALT_NOTSET"}[0] }
sub NT_SWPARSIZE_NOTSET_CODE()       { $exitCodes{"NT_SWPARSIZE_NOTSET"}[0] }
sub COMMON_ENV_NOT_SET_CODE()        { $exitCodes{"COMMON_ENV_NOT_SET"}[0] }
sub SWP_FILE_OPEN_FAILED_CODE()      { $exitCodes{"SWP_FILE_OPEN_FAILED"}[0] }
#Added for ONT;taken from UE
sub INVALID_ISAM_IP_CODE ()             { $exitCodes{"INVALID_ISAM_IP"}[0]}
sub UNABLE_TO_RESOLVE_HOST_NAME_CODE () { $exitCodes{"UNABLE_TO_RESOLVE_HOST_NAME"}[0] }
sub PERL_FATAL_RUNTIME_ERR_CODE()       { $exitCodes{"PERL_FATAL_RUNTIME_ERR"}[0] }
sub START_ACTIVITY_CODE()            { $exitCodes{"START_ACTIVITY"}[0] }
 
sub EC_OCTO_ACTIVE                        { $errorCodes{ 'EC_OCTO_ACTIVE' }{'CODE-NO'}  }
sub EC_OCTO_FAIL                          { $errorCodes{ 'EC_OCTO_FAIL' }{'CODE-NO'}  }
sub EC_OCTO_PS_ERROR                      { $errorCodes{ 'EC_OCTO_PS_ERROR' }{'CODE-NO'}  }
sub EC_ABORT_DL_FAIL                      { $errorCodes{ 'EC_ABORT_DL_FAIL' }{'CODE-NO'}  }
sub EC_ACC_OPERID_MISMATCH                { $errorCodes{ 'EC_ACC_OPERID_MISMATCH' }{'CODE-NO'}  }
sub EC_CONFIG_LOCK_ACQUIRE_FAILED         { $errorCodes{ 'EC_CONFIG_LOCK_ACQUIRE_FAILED' }{'CODE-NO'}  }
sub EC_CONFIG_LOCK_RELEASE_FAILED         { $errorCodes{ 'EC_CONFIG_LOCK_RELEASE_FAILED' }{'CODE-NO'}  }
sub EC_CONFIG_LOCK_ALREADY_TAKEN          { $errorCodes{ 'EC_CONFIG_LOCK_ALREADY_TAKEN' }{'CODE-NO'}  } 
sub EC_ACTIVATE_FAIL                      { $errorCodes{ 'EC_ACTIVATE_FAIL' }{'CODE-NO'}  }
sub EC_ALW_ALARM_FAILED                   { $errorCodes{ 'EC_ALW_ALARM_FAILED' }{'CODE-NO'}  }
sub EC_AUTHOOS_THR_EXCEEDED               { $errorCodes{ 'EC_AUTHOOS_THR_EXCEEDED' }{'CODE-NO'}  }
sub EC_BACKUPDB_FAILED                    { $errorCodes{ 'EC_BACKUPDB_FAILED' }{'CODE-NO'}  }
sub EC_BATCHMODE_FAILED                   { $errorCodes{ 'EC_BATCHMODE_FAILED' }{'CODE-NO'}  }
sub EC_BRIDGECONFMM_THR_EXCEEDED          { $errorCodes{ 'EC_BRIDGECONFMM_THR_EXCEEDED' }{'CODE-NO'}  }
sub EC_BRIDGEPORT_PARSE_ERROR             { $errorCodes{ 'EC_BRIDGEPORT_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_CDE_NOTFOUND                       { $errorCodes{ 'EC_CDE_NOTFOUND' }{'CODE-NO'}  }
sub EC_CLIENTPORT_PARSE_ERROR             { $errorCodes{ 'EC_CLIENTPORT_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_CLI_CMD_FAILED                     { $errorCodes{ 'EC_CLI_CMD_FAILED' }{'CODE-NO'}  }
sub EC_CLI_PREPARE_FAILED                 { $errorCodes{ 'EC_CLI_PREPARE_FAILED' }{'CODE-NO'}  }
sub EC_CMD_INCOMPL                        { $errorCodes{ 'EC_CMD_INCOMPL' }{'CODE-NO'}  }
sub EC_COMMIT_FAIL                        { $errorCodes{ 'EC_COMMIT_FAIL' }{'CODE-NO'}  }
sub EC_COMMIT_FAILED                      { $errorCodes{ 'EC_COMMIT_FAILED' }{'CODE-NO'}  }
sub EC_CONFIG_DL_FAILED                   { $errorCodes{ 'EC_CONFIG_DL_FAILED' }{'CODE-NO'}  }
sub EC_CONFIG_OPRID_FAILED                { $errorCodes{ 'EC_CONFIG_OPRID_FAILED' }{'CODE-NO'}  }
sub EC_CONF_FILE_READ_FAILED              { $errorCodes{ 'EC_CONF_FILE_READ_FAILED' }{'CODE-NO'}  }
sub EC_COPY_FAILED                        { $errorCodes{ 'EC_COPY_FAILED' }{'CODE-NO'}  }
sub EC_COPY_OFLDB_FAILED                  { $errorCodes{ 'EC_COPY_OFLDB_FAILED' }{'CODE-NO'}  }
sub EC_CORR_VLAN_TBL_FAILED               { $errorCodes{ 'EC_CORR_VLAN_TBL_FAILED' }{'CODE-NO'}  }
sub EC_CREATE_CONFIG_ERROR                { $errorCodes{ 'EC_CREATE_CONFIG_ERROR' }{'CODE-NO'}  }
sub EC_CREATE_ISS_FAIL                    { $errorCodes{ 'EC_CREATE_ISS_FAIL' }{'CODE-NO'}  }
sub EC_CREATE_NVR_FAIL                    { $errorCodes{ 'EC_CREATE_NVR_FAIL' }{'CODE-NO'}  }
sub EC_CREATE_OSWP_FAIL                   { $errorCodes{ 'EC_CREATE_OSWP_FAIL' }{'CODE-NO'}  }
sub EC_CREATE_RESTOREDIR_FAILED           { $errorCodes{ 'EC_CREATE_RESTOREDIR_FAILED' }{'CODE-NO'}  }
sub EC_CREATE_SWP_FAIL                    { $errorCodes{ 'EC_CREATE_SWP_FAIL' }{'CODE-NO'}  }
sub EC_CREATE_TARGET_DIR_FAILED           { $errorCodes{ 'EC_CREATE_TARGET_DIR_FAILED' }{'CODE-NO'}  }
sub EC_CREATE_WORKSPACE_FAILED            { $errorCodes{ 'EC_CREATE_WORKSPACE_FAILED' }{'CODE-NO'}  }
sub EC_CVT_COMBINE_FAIL                   { $errorCodes{ 'EC_CVT_COMBINE_FAIL' }{'CODE-NO'}  }
sub EC_CVT_IHUB_FAIL                      { $errorCodes{ 'EC_CVT_IHUB_FAIL' }{'CODE-NO'}  }
sub EC_CVT_TO_IHUB_FAIL                   { $errorCodes{ 'EC_CVT_TO_IHUB_FAIL' }{'CODE-NO'}  }
sub EC_DB_AUTO_ACTIVATE_ON_DWLD           { $errorCodes{ 'EC_DB_AUTO_ACTIVATE_ON_DWLD' }{'CODE-NO'}  }
sub EC_DB_COMBINE_FAILED                  { $errorCodes{ 'EC_DB_COMBINE_FAILED' }{'CODE-NO'}  }
sub EC_DB_COUNT_FILES_FAILED              { $errorCodes{ 'EC_DB_COUNT_FILES_FAILED' }{'CODE-NO'}  }
sub EC_DB_DL_FAILED                       { $errorCodes{ 'EC_DB_DL_FAILED' }{'CODE-NO'}  }
sub EC_DB_TOO_MANY_FILES                  { $errorCodes{ 'EC_DB_TOO_MANY_FILES' }{'CODE-NO'}  }
sub EC_DB_VERS_UNEXPECTED                 { $errorCodes{ 'EC_DB_VERS_UNEXPECTED' }{'CODE-NO'}  }
sub EC_DB_VER_UNDEFINED                   { $errorCodes{ 'EC_DB_VER_UNDEFINED' }{'CODE-NO'}  }
sub EC_DEL_CONFIG_ERROR                   { $errorCodes{ 'EC_DEL_CONFIG_ERROR' }{'CODE-NO'}  }
sub EC_DEL_SHUB_SW_FAILED                 { $errorCodes{ 'EC_DEL_SHUB_SW_FAILED' }{'CODE-NO'}  }
sub EC_DETERMINE_LOG_FAILED               { $errorCodes{ 'EC_DETERMINE_LOG_FAILED' }{'CODE-NO'}  }
sub EC_DETERMINE_OPER_FAILED              { $errorCodes{ 'EC_DETERMINE_OPER_FAILED' }{'CODE-NO'}  }
sub EC_DHCPRELAY_PARSE_ERROR              { $errorCodes{ 'EC_DHCPRELAY_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_DIR_SW_LIST_FAILED                 { $errorCodes{ 'EC_DIR_SW_LIST_FAILED' }{'CODE-NO'}  }
sub EC_DL_ACT_GET_ONT_LIST_FAILED         { $errorCodes{ 'EC_DL_ACT_GET_ONT_LIST_FAILED' }{'CODE-NO'}  }
sub EC_DL_ACT_RTRV_ONTSWCTRL_FAILED       { $errorCodes{ 'EC_DL_ACT_RTRV_ONTSWCTRL_FAILED' }{'CODE-NO'}  }
sub EC_DL_ACT_WARN_EXCEEDS_THRESHOLD      { $errorCodes{ 'EC_DL_ACT_WARN_EXCEEDS_THRESHOLD' }{'CODE-NO'}  }
sub EC_DL_SWP_FAILED                      { $errorCodes{ 'EC_DL_SWP_FAILED' }{'CODE-NO'}  }
sub EC_DOWNLOAD_DB_FAILED                 { $errorCodes{ 'EC_DOWNLOAD_DB_FAILED' }{'CODE-NO'}  }
sub EC_EMERGENCY_CALL_ACTIVE              { $errorCodes{ 'EC_EMERGENCY_CALL_ACTIVE' }{'CODE-NO'}  }
sub EC_EMPTY_NT_DETECTED                  { $errorCodes{ 'EC_EMPTY_NT_DETECTED' }{'CODE-NO'}  }
sub EC_EMPTY_RESPONSE                     { $errorCodes{ 'EC_EMPTY_RESPONSE' }{'CODE-NO'}  }
sub EC_ENABLE_WR_FAILED                   { $errorCodes{ 'EC_ENABLE_WR_FAILED' }{'CODE-NO'}  }
sub EC_ENGINE_PARSE_ERROR                 { $errorCodes{ 'EC_ENGINE_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_ENV_NOT_OK                         { $errorCodes{ 'EC_ENV_NOT_OK' }{'CODE-NO'}  }
sub EC_ETSI_NOT_SUPPORTED                 { $errorCodes{ 'EC_ETSI_NOT_SUPPORTED' }{'CODE-NO'}  }
sub EC_EXTR_MCAST_FAILED                  { $errorCodes{ 'EC_EXTR_MCAST_FAILED' }{'CODE-NO'}  }
sub EC_FILEXFRMODE_NOTSUPPORTED           { $errorCodes{ 'EC_FILEXFRMODE_NOTSUPPORTED' }{'CODE-NO'}  }
sub EC_GETCOMMTY_ERROR                    { $errorCodes{ 'EC_GETCOMMTY_ERROR' }{'CODE-NO'}  }
sub EC_GET_ACC_BITS_FAILED                { $errorCodes{ 'EC_GET_ACC_BITS_FAILED' }{'CODE-NO'}  }
sub EC_GET_ACC_OPERID_FAILED              { $errorCodes{ 'EC_GET_ACC_OPERID_FAILED' }{'CODE-NO'}  }
sub EC_GET_ACTIVATE_FAIL                  { $errorCodes{ 'EC_GET_ACTIVATE_FAIL' }{'CODE-NO'}  }
sub EC_GET_ALARM_FAILED                   { $errorCodes{ 'EC_GET_ALARM_FAILED' }{'CODE-NO'}  }
sub EC_GET_ALL_ONT_FAILED                 { $errorCodes{ 'EC_GET_ALL_ONT_FAILED' }{'CODE-NO'}  }
sub EC_GET_ATMPVC_ERROR                   { $errorCodes{ 'EC_GET_ATMPVC_ERROR' }{'CODE-NO'}  }
sub EC_GET_AUTHENT_FAILED                 { $errorCodes{ 'EC_GET_AUTHENT_FAILED' }{'CODE-NO'}  }
sub EC_GET_BRIDGEINFO_ERROR               { $errorCodes{ 'EC_GET_BRIDGEINFO_ERROR' }{'CODE-NO'}  }
sub EC_GET_BRIDGE_FAILED                  { $errorCodes{ 'EC_GET_BRIDGE_FAILED' }{'CODE-NO'}  }
sub EC_GET_CLIMODE_FAIL                   { $errorCodes{ 'EC_GET_CLIMODE_FAIL' }{'CODE-NO'}  }
sub EC_GET_CLIMODE_FAILED                 { $errorCodes{ 'EC_GET_CLIMODE_FAILED' }{'CODE-NO'}  }
sub EC_GET_COMMIT_FAIL                    { $errorCodes{ 'EC_GET_COMMIT_FAIL' }{'CODE-NO'}  }
sub EC_GET_DB_INFO_FAILED                 { $errorCodes{ 'EC_GET_DB_INFO_FAILED' }{'CODE-NO'}  }
sub EC_GET_FILEXFR_FAILED                 { $errorCodes{ 'EC_GET_FILEXFR_FAILED' }{'CODE-NO'}  }
sub EC_GET_FTP_FAILED                     { $errorCodes{ 'EC_GET_FTP_FAILED' }{'CODE-NO'}  }
sub EC_GET_H248_FAILED                    { $errorCodes{ 'EC_GET_H248_FAILED' }{'CODE-NO'}  }
sub EC_GET_IGMPCH_INFO_ERROR              { $errorCodes{ 'EC_GET_IGMPCH_INFO_ERROR' }{'CODE-NO'}  }
sub EC_GET_LT_FAILED                      { $errorCodes{ 'EC_GET_LT_FAILED' }{'CODE-NO'}  }
sub EC_GET_LT_INFO_FAILED                 { $errorCodes{ 'EC_GET_LT_INFO_FAILED' }{'CODE-NO'}  }
sub EC_GET_NEW_FREE_FAIL                  { $errorCodes{ 'EC_GET_NEW_FREE_FAIL' }{'CODE-NO'}  }
sub EC_GET_NT_INFO_FAILED                 { $errorCodes{ 'EC_GET_NT_INFO_FAILED' }{'CODE-NO'}  }
sub EC_GET_NT_TYPE_FAILED                 { $errorCodes{ 'EC_GET_NT_TYPE_FAILED' }{'CODE-NO'}  }
sub EC_GET_OSPF_FAILED                    { $errorCodes{ 'EC_GET_OSPF_FAILED' }{'CODE-NO'}  }
sub EC_GET_OSWP_FAIL                      { $errorCodes{ 'EC_GET_OSWP_FAIL' }{'CODE-NO'}  }
sub EC_GET_OSWP_FAILED                    { $errorCodes{ 'EC_GET_OSWP_FAILED' }{'CODE-NO'}  }
sub EC_GET_OSWP_NAME_FAIL                 { $errorCodes{ 'EC_GET_OSWP_NAME_FAIL' }{'CODE-NO'}  }
sub EC_GET_SFP_ASSIGNMENT_FAILED          { $errorCodes{ 'EC_GET_SFP_ASSIGNMENT_FAILED' }{'CODE-NO'}  }
sub EC_GET_SHELFCONFIG_FAIL               { $errorCodes{ 'EC_GET_SHELFCONFIG_FAIL' }{'CODE-NO'}  }
sub EC_GET_SHUB_CONF_FAILED               { $errorCodes{ 'EC_GET_SHUB_CONF_FAILED' }{'CODE-NO'}  }
sub EC_GET_SHUB_SWSIZE_FAILED             { $errorCodes{ 'EC_GET_SHUB_SWSIZE_FAILED' }{'CODE-NO'}  }
sub EC_GET_SNMP_ACC_FAILED                { $errorCodes{ 'EC_GET_SNMP_ACC_FAILED' }{'CODE-NO'}  }
sub EC_GET_SNMP_SEC_FAIL                  { $errorCodes{ 'EC_GET_SNMP_SEC_FAIL' }{'CODE-NO'}  }
sub EC_GET_SW_PARSIZE_FAILED              { $errorCodes{ 'EC_GET_SW_PARSIZE_FAILED' }{'CODE-NO'}  }
sub EC_GET_TFTPIP_FAIL                    { $errorCodes{ 'EC_GET_TFTPIP_FAIL' }{'CODE-NO'}  }
sub EC_TRANSFER_SNMPGET_FAILED            { $errorCodes{ 'EC_TRANSFER_SNMPGET_FAILED' }{'CODE-NO'}  }
sub EC_GET_TRANS_TO_FAILED                { $errorCodes{ 'EC_GET_TRANS_TO_FAILED' }{'CODE-NO'}  }
sub EC_GET_VOICEIP_FAILED                 { $errorCodes{ 'EC_GET_VOICEIP_FAILED' }{'CODE-NO'}  }
sub EC_GET_VVLAN_FAILED                   { $errorCodes{ 'EC_GET_VVLAN_FAILED' }{'CODE-NO'}  }
sub EC_GET_WARN_TYPE_COULD_NOT_FOUND      { $errorCodes{ 'EC_GET_WARN_TYPE_COULD_NOT_FOUND' }{'CODE-NO'}  }
sub EC_GET_XDSL_FAILED                    { $errorCodes{ 'EC_GET_XDSL_FAILED' }{'CODE-NO'}  }
sub EC_HIGHER_PASSIVE_RESTRICTED          { $errorCodes{ 'EC_HIGHER_PASSIVE_RESTRICTED' }{'CODE-NO'}  }
sub EC_INCOMPAT_NT                        { $errorCodes{ 'EC_INCOMPAT_NT' }{'CODE-NO'}  }
sub EC_INCOMPAT_NT_ICS_STATE              { $errorCodes{ 'EC_INCOMPAT_NT_ICS_STATE' }{'CODE-NO'}  }
sub EC_INCOMPAT_NT_STATE                  { $errorCodes{ 'EC_INCOMPAT_NT_STATE' }{'CODE-NO'}  }
sub EC_INCORR_SHUB_OFLMT                  { $errorCodes{ 'EC_INCORR_SHUB_OFLMT' }{'CODE-NO'}  }
sub EC_INCORR_SHUB_OFLMT_NANT_B           { $errorCodes{ 'EC_INCORR_SHUB_OFLMT_NANT_B' }{'CODE-NO'}  }
sub EC_INFO_CONF_BRIDGEPORT_ERROR         { $errorCodes{ 'EC_INFO_CONF_BRIDGEPORT_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_CCPORT_ERROR             { $errorCodes{ 'EC_INFO_CONF_CCPORT_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_CLIENTPORT_ERROR         { $errorCodes{ 'EC_INFO_CONF_CLIENTPORT_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_DHCPRELAY_ERROR          { $errorCodes{ 'EC_INFO_CONF_DHCPRELAY_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_ENGINE_ERROR             { $errorCodes{ 'EC_INFO_CONF_ENGINE_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_INTERFACE_ERROR          { $errorCodes{ 'EC_INFO_CONF_INTERFACE_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_MCAST_ERROR              { $errorCodes{ 'EC_INFO_CONF_MCAST_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_NTVRF_ERROR              { $errorCodes{ 'EC_INFO_CONF_NTVRF_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_NT_VLAN_ERROR            { $errorCodes{ 'EC_INFO_CONF_NT_VLAN_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_PVCCONFIG_ERROR          { $errorCodes{ 'EC_INFO_CONF_PVCCONFIG_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_QOS_ERROR                { $errorCodes{ 'EC_INFO_CONF_QOS_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_SHUBVLAN_ERROR           { $errorCodes{ 'EC_INFO_CONF_SHUBVLAN_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_SHUBVRF_ERROR            { $errorCodes{ 'EC_INFO_CONF_SHUBVRF_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_SHUB_BRG_ERROR           { $errorCodes{ 'EC_INFO_CONF_SHUB_BRG_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_SHUB_VLAN_ERROR          { $errorCodes{ 'EC_INFO_CONF_SHUB_VLAN_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_VLANBIND_ERROR           { $errorCodes{ 'EC_INFO_CONF_VLANBIND_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_VLANPORT_ERROR           { $errorCodes{ 'EC_INFO_CONF_VLANPORT_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_VLANPP_ERROR             { $errorCodes{ 'EC_INFO_CONF_VLANPP_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_VLAN_ERROR               { $errorCodes{ 'EC_INFO_CONF_VLAN_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_VLAN_PP_ERROR            { $errorCodes{ 'EC_INFO_CONF_VLAN_PP_ERROR' }{'CODE-NO'}  }
sub EC_INFO_CONF_VRF_ERROR                { $errorCodes{ 'EC_INFO_CONF_VRF_ERROR' }{'CODE-NO'}  }
sub EC_INHBT_MORE_FAILED                  { $errorCodes{ 'EC_INHBT_MORE_FAILED' }{'CODE-NO'}  }
sub EC_INHIBIT_ALRM_FAILED                { $errorCodes{ 'EC_INHIBIT_ALRM_FAILED' }{'CODE-NO'}  }
sub EC_INH_ALARM_FAILED                   { $errorCodes{ 'EC_INH_ALARM_FAILED' }{'CODE-NO'}  }
sub EC_INIT_COPY_FAILED                   { $errorCodes{ 'EC_INIT_COPY_FAILED' }{'CODE-NO'}  }
sub EC_INPUTDB_PROBLEM                    { $errorCodes{ 'EC_INPUTDB_PROBLEM' }{'CODE-NO'}  }
sub EC_INVALID_2_ECNT_C                   { $errorCodes{ 'EC_INVALID_2_ECNT_C' }{'CODE-NO'}  }
sub EC_INVALID_2_ECNT_E                   { $errorCodes{ 'EC_INVALID_2_ECNT_E' }{'CODE-NO'}  }
sub EC_INVALID_2_NANT_B                   { $errorCodes{ 'EC_INVALID_2_NANT_B' }{'CODE-NO'}  }
sub EC_INVALID_CONFIG                     { $errorCodes{ 'EC_INVALID_CONFIG' }{'CODE-NO'}  }
sub EC_INVALID_FILE_CONTENT               { $errorCodes{ 'EC_INVALID_FILE_CONTENT' }{'CODE-NO'}  }
sub EC_INVALID_TFTP_IP                    { $errorCodes{ 'EC_INVALID_TFTP_IP' }{'CODE-NO'}  }
sub EC_INV_MIB_VERSION                    { $errorCodes{ 'EC_INV_MIB_VERSION' }{'CODE-NO'}  }
sub EC_INV_NT_RB_TYPE                     { $errorCodes{ 'EC_INV_NT_RB_TYPE' }{'CODE-NO'}  }
sub EC_INV_SHUB_RB_TYPE                   { $errorCodes{ 'EC_INV_SHUB_RB_TYPE' }{'CODE-NO'}  }
sub EC_ISAMV_OFLMIG_NO_CONF               { $errorCodes{ 'EC_ISAMV_OFLMIG_NO_CONF' }{'CODE-NO'}  }
sub EC_ISAM_HAS_VOICE                     { $errorCodes{ 'EC_ISAM_HAS_VOICE' }{'CODE-NO'}  }
sub EC_ISAM_NT_SYNC_FAILED                { $errorCodes{ 'EC_ISAM_NT_SYNC_FAILED' }{'CODE-NO'}  }
sub EC_ISAM_TRANSACT_TAKEN                { $errorCodes{ 'EC_ISAM_TRANSACT_TAKEN' }{'CODE-NO'}  }
sub EC_ITFCONFIG_PARSE_ERROR              { $errorCodes{ 'EC_ITFCONFIG_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_IVPS_ADD_CDE_FAILED                { $errorCodes{ 'EC_IVPS_ADD_CDE_FAILED' }{'CODE-NO'}  }
sub EC_IVPS_DB_FAIL                       { $errorCodes{ 'EC_IVPS_DB_FAIL' }{'CODE-NO'}  }
sub EC_IVPS_GETDEP_ERROR                  { $errorCodes{ 'EC_IVPS_GETDEP_ERROR' }{'CODE-NO'}  }
sub EC_IVPS_GETIP_ERROR                   { $errorCodes{ 'EC_IVPS_GETIP_ERROR' }{'CODE-NO'}  }
sub EC_IVPS_MIG_FAILED                    { $errorCodes{ 'EC_IVPS_MIG_FAILED' }{'CODE-NO'}  }
sub EC_LINEOOS_THR_EXCEEDED               { $errorCodes{ 'EC_LINEOOS_THR_EXCEEDED' }{'CODE-NO'}  }
sub EC_LIST_PARTLY_PROCD                  { $errorCodes{ 'EC_LIST_PARTLY_PROCD' }{'CODE-NO'}  }
sub EC_LIST_SWFILES_FAILED                { $errorCodes{ 'EC_LIST_SWFILES_FAILED' }{'CODE-NO'}  }
sub EC_LTOOS_THR_EXCEEDED                 { $errorCodes{ 'EC_LTOOS_THR_EXCEEDED' }{'CODE-NO'}  }
sub EC_LT_NOT_SUPPORTED                   { $errorCodes{ 'EC_LT_NOT_SUPPORTED' }{'CODE-NO'}  }
sub EC_MAC_CONC_ENABLED                   { $errorCodes{ 'EC_MAC_CONC_ENABLED' }{'CODE-NO'}  }
sub EC_MAC_LIMITS_EXCEEDED                { $errorCodes{ 'EC_MAC_LIMITS_EXCEEDED' }{'CODE-NO'}  }
sub EC_MAKE_KEYFILE_FAILED                { $errorCodes{ 'EC_MAKE_KEYFILE_FAILED' }{'CODE-NO'}  }
sub EC_MAPPING_FILE_FAILED                { $errorCodes{ 'EC_MAPPING_FILE_FAILED' }{'CODE-NO'}  }
sub EC_MAP_ERR_REC_FAILED                 { $errorCodes{ 'EC_MAP_ERR_REC_FAILED' }{'CODE-NO'}  }
sub EC_MCASTSRC_PARSE_ERROR               { $errorCodes{ 'EC_MCASTSRC_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_MIG_NOT_ADVISED                    { $errorCodes{ 'EC_MIG_NOT_ADVISED' }{'CODE-NO'}  }
sub EC_MIG_RESTRICTED                     { $errorCodes{ 'EC_MIG_RESTRICTED' }{'CODE-NO'}  }
sub EC_MISSING_FAILED                     { $errorCodes{ 'EC_MISSING_FAILED' }{'CODE-NO'}  }
sub EC_MKDIR_FAILED                       { $errorCodes{ 'EC_MKDIR_FAILED' }{'CODE-NO'}  }
sub EC_NEEDS_MIGRATION                    { $errorCodes{ 'EC_NEEDS_MIGRATION' }{'CODE-NO'}  }
sub EC_NEEDS_UPGRADE                      { $errorCodes{ 'EC_NEEDS_UPGRADE' }{'CODE-NO'}  }
sub EC_NEW_ALARM                          { $errorCodes{ 'EC_NEW_ALARM' }{'CODE-NO'}  }
sub EC_NEW_CDE_NOTFOUND                   { $errorCodes{ 'EC_NEW_CDE_NOTFOUND' }{'CODE-NO'}  }
sub EC_NEW_DB_MISSING                     { $errorCodes{ 'EC_NEW_DB_MISSING' }{'CODE-NO'}  }
sub EC_NEW_DB_NOT_ACTIVE                  { $errorCodes{ 'EC_NEW_DB_NOT_ACTIVE' }{'CODE-NO'}  }
sub EC_NEW_OSWP_FILE_FAILED               { $errorCodes{ 'EC_NEW_OSWP_FILE_FAILED' }{'CODE-NO'}  }
sub EC_NEW_SWP_NOT_ENABLED                { $errorCodes{ 'EC_NEW_SWP_NOT_ENABLED' }{'CODE-NO'}  }
sub EC_NEW_SWP_NOT_PRESENT                { $errorCodes{ 'EC_NEW_SWP_NOT_PRESENT' }{'CODE-NO'}  }
sub EC_NEW_SW_NOT_ACT                     { $errorCodes{ 'EC_NEW_SW_NOT_ACT' }{'CODE-NO'}  }
sub EC_NEW_SW_PRESENT                     { $errorCodes{ 'EC_NEW_SW_PRESENT' }{'CODE-NO'}  }
sub EC_NOT_SUPPORTED                      { $errorCodes{ 'EC_NOT_SUPPORTED' }{'CODE-NO'}  }
sub EC_NOT_SUPPORTED_LOWER_RELEASE        { $errorCodes{ 'EC_NOT_SUPPORTED_LOWER_RELEASE' }{'CODE-NO'}  }
sub EC_NOT_VOICE_SRV                      { $errorCodes{ 'EC_NOT_VOICE_SRV' }{'CODE-NO'}  }
sub EC_NO_ACT_OSWP                        { $errorCodes{ 'EC_NO_ACT_OSWP' }{'CODE-NO'}  }
sub EC_NO_ACT_OSWP_2ACTIVATE              { $errorCodes{ 'EC_NO_ACT_OSWP_2ACTIVATE' }{'CODE-NO'}  }
sub EC_NO_ACT_SWP                         { $errorCodes{ 'EC_NO_ACT_SWP' }{'CODE-NO'}  }
sub EC_NO_ALT_NT_CAP                      { $errorCodes{ 'EC_NO_ALT_NT_CAP' }{'CODE-NO'}  }
sub EC_NO_ASAMCORE_FOUND                  { $errorCodes{ 'EC_NO_ASAMCORE_FOUND' }{'CODE-NO'}  }
sub EC_NO_BOARDS                          { $errorCodes{ 'EC_NO_BOARDS' }{'CODE-NO'}  }
sub EC_NO_FREE_SPACE                      { $errorCodes{ 'EC_NO_FREE_SPACE' }{'CODE-NO'}  }
sub EC_NO_INPUT_DIR                       { $errorCodes{ 'EC_NO_INPUT_DIR' }{'CODE-NO'}  }
sub EC_NO_LT_SWP_FOUND                    { $errorCodes{ 'EC_NO_LT_SWP_FOUND' }{'CODE-NO'}  }
sub EC_NO_MIB_VERSION                     { $errorCodes{ 'EC_NO_MIB_VERSION' }{'CODE-NO'}  }
sub EC_NO_NT_CAP                          { $errorCodes{ 'EC_NO_NT_CAP' }{'CODE-NO'}  }
sub EC_NO_PASS_SWPIDX_FOUND               { $errorCodes{ 'EC_NO_PASS_SWPIDX_FOUND' }{'CODE-NO'}  }
sub EC_NO_PING                            { $errorCodes{ 'EC_NO_PING' }{'CODE-NO'}  }
sub EC_NO_SHUB_SW                         { $errorCodes{ 'EC_NO_SHUB_SW' }{'CODE-NO'}  }
sub EC_NO_SNMP_VERSION_SEL                { $errorCodes{ 'EC_NO_SNMP_VERSION_SEL' }{'CODE-NO'}  }
sub EC_NO_SSH_POSS                        { $errorCodes{ 'EC_NO_SSH_POSS' }{'CODE-NO'}  }
sub EC_NO_SSH_POSSIBLE                    { $errorCodes{ 'EC_NO_SSH_POSSIBLE' }{'CODE-NO'}  }
sub EC_NO_STATUS_FILE                     { $errorCodes{ 'EC_NO_STATUS_FILE' }{'CODE-NO'}  }
sub EC_NO_SUCH_BRIDGEPORT                 { $errorCodes{ 'EC_NO_SUCH_BRIDGEPORT' }{'CODE-NO'}  }
sub EC_NO_SUCH_CLIENTPORT                 { $errorCodes{ 'EC_NO_SUCH_CLIENTPORT' }{'CODE-NO'}  }
sub EC_NO_SUCH_ENGINE                     { $errorCodes{ 'EC_NO_SUCH_ENGINE' }{'CODE-NO'}  }
sub EC_NO_SUCH_IPVLAN                     { $errorCodes{ 'EC_NO_SUCH_IPVLAN' }{'CODE-NO'}  }
sub EC_NO_SUCH_NT_VLAN                    { $errorCodes{ 'EC_NO_SUCH_NT_VLAN' }{'CODE-NO'}  }
sub EC_NO_SUCH_PPPVLAN                    { $errorCodes{ 'EC_NO_SUCH_PPPVLAN' }{'CODE-NO'}  }
sub EC_NO_SUCH_SHUB_VLAN                  { $errorCodes{ 'EC_NO_SUCH_SHUB_VLAN' }{'CODE-NO'}  }
sub EC_NO_SUCH_VLAN                       { $errorCodes{ 'EC_NO_SUCH_VLAN' }{'CODE-NO'}  }
sub EC_NO_SUCH_VLAN_PP                    { $errorCodes{ 'EC_NO_SUCH_VLAN_PP' }{'CODE-NO'}  }
sub EC_NO_SUCH_VRF                        { $errorCodes{ 'EC_NO_SUCH_VRF' }{'CODE-NO'}  }
sub EC_NO_TTL0_FWD                        { $errorCodes{ 'EC_NO_TTL0_FWD' }{'CODE-NO'}  }
sub EC_NTTYPE_MAPFILE_FAILED              { $errorCodes{ 'EC_NTTYPE_MAPFILE_FAILED' }{'CODE-NO'}  }
sub EC_NTTYPE_OSWP_FAILED                 { $errorCodes{ 'EC_NTTYPE_OSWP_FAILED' }{'CODE-NO'}  }
sub EC_NTTYPE_SWPNAME_FAILED              { $errorCodes{ 'EC_NTTYPE_SWPNAME_FAILED' }{'CODE-NO'}  }
sub EC_NT_DB_BU_FAIL                      { $errorCodes{ 'EC_NT_DB_BU_FAIL' }{'CODE-NO'}  }
sub EC_NT_DB_COPY_FAILED                  { $errorCodes{ 'EC_NT_DB_COPY_FAILED' }{'CODE-NO'}  }
sub EC_NT_DB_CVT_FAIL                     { $errorCodes{ 'EC_NT_DB_CVT_FAIL' }{'CODE-NO'}  }
sub EC_NT_DB_EMPTY                        { $errorCodes{ 'EC_NT_DB_EMPTY' }{'CODE-NO'}  }
sub EC_NT_DB_NOT_PRESENT                  { $errorCodes{ 'EC_NT_DB_NOT_PRESENT' }{'CODE-NO'}  }
sub EC_NT_ICS_ISSUE                       { $errorCodes{ 'EC_NT_ICS_ISSUE' }{'CODE-NO'}  }
sub EC_NT_MIG_FAILED                      { $errorCodes{ 'EC_NT_MIG_FAILED' }{'CODE-NO'}  }
sub EC_NT_NOT_ECNT_A                      { $errorCodes{ 'EC_NT_NOT_ECNT_A' }{'CODE-NO'}  }
sub EC_NT_NOT_ECNT_A_OR_C                 { $errorCodes{ 'EC_NT_NOT_ECNT_A_OR_C' }{'CODE-NO'}  }
sub EC_NT_NOT_INSYNC                      { $errorCodes{ 'EC_NT_NOT_INSYNC' }{'CODE-NO'}  }
sub EC_NT_NOT_IN_SWP                      { $errorCodes{ 'EC_NT_NOT_IN_SWP' }{'CODE-NO'}  }
sub EC_NT_NOT_NANT_A                      { $errorCodes{ 'EC_NT_NOT_NANT_A' }{'CODE-NO'}  }
sub EC_NT_NOT_SUPPORTED                   { $errorCodes{ 'EC_NT_NOT_SUPPORTED' }{'CODE-NO'}  }
sub EC_NT_SHUB_SW_MM                      { $errorCodes{ 'EC_NT_SHUB_SW_MM' }{'CODE-NO'}  }
sub EC_NT_TYPE_UNKNOWN                    { $errorCodes{ 'EC_NT_TYPE_UNKNOWN' }{'CODE-NO'}  }
sub EC_NT_VLAN_PARSE_ERROR                { $errorCodes{ 'EC_NT_VLAN_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_OFLDB_NOTREQ                       { $errorCodes{ 'EC_OFLDB_NOTREQ' }{'CODE-NO'}  }
sub EC_ONTOOS_THR_EXCEEDED                { $errorCodes{ 'EC_ONTOOS_THR_EXCEEDED' }{'CODE-NO'}  }
sub EC_ONT_RELMAP_MISSING                 { $errorCodes{ 'EC_ONT_RELMAP_MISSING' }{'CODE-NO'}  }
sub EC_OPENCLI_FAILED                     { $errorCodes{ 'EC_OPENCLI_FAILED' }{'CODE-NO'}  }
sub EC_OPENFILE_FAILED                    { $errorCodes{ 'EC_OPENFILE_FAILED' }{'CODE-NO'}  }
sub EC_OPENTL1_FAILED                     { $errorCodes{ 'EC_OPENTL1_FAILED' }{'CODE-NO'}  }
sub EC_OPEN_OSWP_FAIL                     { $errorCodes{ 'EC_OPEN_OSWP_FAIL' }{'CODE-NO'}  }
sub EC_OPEN_OSWP_NEW_FAIL                 { $errorCodes{ 'EC_OPEN_OSWP_NEW_FAIL' }{'CODE-NO'}  }
sub EC_OPEN_POST_CHECK_FAILED             { $errorCodes{ 'EC_OPEN_POST_CHECK_FAILED' }{'CODE-NO'}  }
sub EC_OPEN_PRE_CHECK_FAILED              { $errorCodes{ 'EC_OPEN_PRE_CHECK_FAILED' }{'CODE-NO'}  }
sub EC_OPEN_SWP_FAIL                      { $errorCodes{ 'EC_OPEN_SWP_FAIL' }{'CODE-NO'}  }
sub EC_OPEN_WARNFILE_FAILED               { $errorCodes{ 'EC_OPEN_WARNFILE_FAILED' }{'CODE-NO'}  }
sub EC_OPEN_WARN_FILE_FAILED              { $errorCodes{ 'EC_OPEN_WARN_FILE_FAILED' }{'CODE-NO'}  }
sub EC_OSWP_ACT_FAILED                    { $errorCodes{ 'EC_OSWP_ACT_FAILED' }{'CODE-NO'}  }
sub EC_OSWP_FILE_NO_IP                    { $errorCodes{ 'EC_OSWP_FILE_NO_IP' }{'CODE-NO'}  }
sub EC_OSWP_MISSING                       { $errorCodes{ 'EC_OSWP_MISSING' }{'CODE-NO'}  }
sub EC_OSWP_OPER_FAIL                     { $errorCodes{ 'EC_OSWP_OPER_FAIL' }{'CODE-NO'}  }
sub EC_PARAM_CFG_NOT_OK                   { $errorCodes{ 'EC_PARAM_CFG_NOT_OK' }{'CODE-NO'}  }
sub EC_PARSE_SIP_SERV_FAILED              { $errorCodes{ 'EC_PARSE_SIP_SERV_FAILED' }{'CODE-NO'}  }
sub EC_PARSE_SIP_SITE_FAILED              { $errorCodes{ 'EC_PARSE_SIP_SITE_FAILED' }{'CODE-NO'}  }
sub EC_PARSE_SWP_FAILED                   { $errorCodes{ 'EC_PARSE_SWP_FAILED' }{'CODE-NO'}  }
sub EC_PAR_SIZE_TOO_SMALL                 { $errorCodes{ 'EC_PAR_SIZE_TOO_SMALL' }{'CODE-NO'}  }
sub EC_PASSIVE_DB_FAILED                  { $errorCodes{ 'EC_PASSIVE_DB_FAILED' }{'CODE-NO'}  }
sub EC_PASSIVE_OSWP_EMPTY                 { $errorCodes{ 'EC_PASSIVE_OSWP_EMPTY' }{'CODE-NO'}  }
sub EC_PASSIVE_SWP_NOT_COMMITTED          { $errorCodes{ 'EC_PASSIVE_SWP_NOT_COMMITTED' }{'CODE-NO'}  }
sub EC_PASSIVE_SWP_NOT_ENABLED            { $errorCodes{ 'EC_PASSIVE_SWP_NOT_ENABLED' }{'CODE-NO'}  }
sub EC_PING_NO_RESP                       { $errorCodes{ 'EC_PING_NO_RESP' }{'CODE-NO'}  }
sub EC_POST_UPGR_ERROR                    { $errorCodes{ 'EC_POST_UPGR_ERROR' }{'CODE-NO'}  }
sub EC_PPPOESESS_PARSE_ERROR              { $errorCodes{ 'EC_PPPOESESS_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_PRECHECK_NOTRUN                    { $errorCodes{ 'EC_PRECHECK_NOTRUN' }{'CODE-NO'}  }
sub EC_PREPARE_CREATE_ERROR               { $errorCodes{ 'EC_PREPARE_CREATE_ERROR' }{'CODE-NO'}  }
sub EC_PREP_RECONF_FAIL                   { $errorCodes{ 'EC_PREP_RECONF_FAIL' }{'CODE-NO'}  }
sub EC_PRE_UPGR_ERROR                     { $errorCodes{ 'EC_PRE_UPGR_ERROR' }{'CODE-NO'}  }
sub EC_PRNT_NOMORE_FAILED                 { $errorCodes{ 'EC_PRNT_NOMORE_FAILED' }{'CODE-NO'}  }
sub EC_PROMPT_FAILED                      { $errorCodes{ 'EC_PROMPT_FAILED' }{'CODE-NO'}  }
sub EC_PVCCONFIG_PARSE_ERROR              { $errorCodes{ 'EC_PVCCONFIG_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_QOS_PARSE_ERROR                    { $errorCodes{ 'EC_QOS_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_RB_FAILED                          { $errorCodes{ 'EC_RB_FAILED' }{'CODE-NO'}  }
sub EC_REDUCE_OPR_NOTALLOW                { $errorCodes{ 'EC_REDUCE_OPR_NOTALLOW' }{'CODE-NO'}  }
sub EC_RELEASE_TOO_LOW                    { $errorCodes{ 'EC_RELEASE_TOO_LOW' }{'CODE-NO'}  }
sub EC_REL_CVT_FAILED                     { $errorCodes{ 'EC_REL_CVT_FAILED' }{'CODE-NO'}  }
sub EC_RESET_NT_FAILED                    { $errorCodes{ 'EC_RESET_NT_FAILED' }{'CODE-NO'}  }
sub EC_RESET_TRANSACTION_FAILED           { $errorCodes{ 'EC_RESET_TRANSACTION_FAILED' }{'CODE-NO'}  }
sub EC_RESTART_FAILED                     { $errorCodes{ 'EC_RESTART_FAILED' }{'CODE-NO'}  }
sub EC_ROLLBACK_ERROR                     { $errorCodes{ 'EC_ROLLBACK_ERROR' }{'CODE-NO'}  }
sub EC_RTRV_ONT_EMPTY                     { $errorCodes{ 'EC_RTRV_ONT_EMPTY' }{'CODE-NO'}  }
sub EC_RTRV_ONT_FAILED                    { $errorCodes{ 'EC_RTRV_ONT_FAILED' }{'CODE-NO'}  }
sub EC_RTRV_REL_FAILED                    { $errorCodes{ 'EC_RTRV_REL_FAILED' }{'CODE-NO'}  }
sub EC_RTRV_REL_INFO_FAILED               { $errorCodes{ 'EC_RTRV_REL_INFO_FAILED' }{'CODE-NO'}  }
sub EC_SEC_SSH_ENABLE_WRITE_FAIL          { $errorCodes{ 'EC_SEC_SSH_ENABLE_WRITE_FAIL' }{'CODE-NO'}  }
sub EC_SEC_SSH_REMOVE_SWPKG_FAIL          { $errorCodes{ 'EC_SEC_SSH_REMOVE_SWPKG_FAIL' }{'CODE-NO'}  }
sub EC_SETUP_ISSU_FAILED                  { $errorCodes{ 'EC_SETUP_ISSU_FAILED' }{'CODE-NO'}  }
sub EC_SET_COMMIT_FAIL                    { $errorCodes{ 'EC_SET_COMMIT_FAIL' }{'CODE-NO'}  }
sub EC_SET_OSWP_FAIL                      { $errorCodes{ 'EC_SET_OSWP_FAIL' }{'CODE-NO'}  }
sub EC_SET_SESSION_TO_FAILED              { $errorCodes{ 'EC_SET_SESSION_TO_FAILED' }{'CODE-NO'}  }
sub EC_SET_SFP_ASSIGNMENT_FAILED          { $errorCodes{ 'EC_SET_SFP_ASSIGNMENT_FAILED' }{'CODE-NO'}  }
sub EC_SET_TIMEOUT_FAILED                 { $errorCodes{ 'EC_SET_TIMEOUT_FAILED' }{'CODE-NO'}  }
sub EC_SET_TRANSACTION_FAILED             { $errorCodes{ 'EC_SET_TRANSACTION_FAILED' }{'CODE-NO'}  }
sub EC_SET_TRANS_TO_FAILED                { $errorCodes{ 'EC_SET_TRANS_TO_FAILED' }{'CODE-NO'}  }
sub EC_SHOW_EQPT_FAIL                     { $errorCodes{ 'EC_SHOW_EQPT_FAIL' }{'CODE-NO'}  }
sub EC_SHOW_EQPT_FAILED                   { $errorCodes{ 'EC_SHOW_EQPT_FAILED' }{'CODE-NO'}  }
sub EC_SHOW_PPPOESESS_ERROR               { $errorCodes{ 'EC_SHOW_PPPOESESS_ERROR' }{'CODE-NO'}  }
sub EC_SHUB_BU_FAIL                       { $errorCodes{ 'EC_SHUB_BU_FAIL' }{'CODE-NO'}  }
sub EC_SHUB_DB_DL_FAILED                  { $errorCodes{ 'EC_SHUB_DB_DL_FAILED' }{'CODE-NO'}  }
sub EC_SHUB_DB_SAVE_FAIL                  { $errorCodes{ 'EC_SHUB_DB_SAVE_FAIL' }{'CODE-NO'}  }
sub EC_SHUB_MIG_FAILED                    { $errorCodes{ 'EC_SHUB_MIG_FAILED' }{'CODE-NO'}  }
sub EC_SHUB_NO_TFTP_BOOT                  { $errorCodes{ 'EC_SHUB_NO_TFTP_BOOT' }{'CODE-NO'}  }
sub EC_SHUB_PARSE_FAIL                    { $errorCodes{ 'EC_SHUB_PARSE_FAIL' }{'CODE-NO'}  }
sub EC_SHUB_REBOOT_FAILED                 { $errorCodes{ 'EC_SHUB_REBOOT_FAILED' }{'CODE-NO'}  }
sub EC_SHUB_RECONFIG_MISSING              { $errorCodes{ 'EC_SHUB_RECONFIG_MISSING' }{'CODE-NO'}  }
sub EC_SHUB_SET_NEW_CONF_FAILED           { $errorCodes{ 'EC_SHUB_SET_NEW_CONF_FAILED' }{'CODE-NO'}  }
sub EC_SHUB_SW_DL_FAILED                  { $errorCodes{ 'EC_SHUB_SW_DL_FAILED' }{'CODE-NO'}  }
sub EC_SHUB_SW_NOT_FOUND                  { $errorCodes{ 'EC_SHUB_SW_NOT_FOUND' }{'CODE-NO'}  }
sub EC_SHUB_VLAN_PARSE_ERROR              { $errorCodes{ 'EC_SHUB_VLAN_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_SIP_TERM_ERROR                     { $errorCodes{ 'EC_SIP_TERM_ERROR' }{'CODE-NO'}  }
sub EC_SIP_TERM_MISSING                   { $errorCodes{ 'EC_SIP_TERM_MISSING' }{'CODE-NO'}  }
sub EC_SNMP_ACT_FAIL                      { $errorCodes{ 'EC_SNMP_ACT_FAIL' }{'CODE-NO'}  }
sub EC_SNMP_CLIENT_CREDENTIAL_MISMATCH    { $errorCodes{ 'EC_SNMP_CLIENT_CREDENTIAL_MISMATCH' }{'CODE-NO'}  }
sub EC_SNMP_MAX_SIZE_FAIL                 { $errorCodes{ 'EC_SNMP_MAX_SIZE_FAIL' }{'CODE-NO'}  }
sub EC_SNMP_NO_GET                        { $errorCodes{ 'EC_SNMP_NO_GET' }{'CODE-NO'}  }
sub EC_SNMP_OPEN_FAIL                     { $errorCodes{ 'EC_SNMP_OPEN_FAIL' }{'CODE-NO'}  }
sub EC_SNMP_SESS_ERR                      { $errorCodes{ 'EC_SNMP_SESS_ERR' }{'CODE-NO'}  }
sub EC_SNMP_SESS_ESTB_ERR                 { $errorCodes{ 'EC_SNMP_SESS_ESTB_ERR' }{'CODE-NO'}  }
sub EC_STAGE_GET_DISK_SPACE_FAILED        { $errorCodes{ 'EC_STAGE_GET_DISK_SPACE_FAILED' }{'CODE-NO'}  }
sub EC_STAGE_GET_ONTSWVER_FAILED          { $errorCodes{ 'EC_STAGE_GET_ONTSWVER_FAILED' }{'CODE-NO'}  }
sub EC_STAGE_PUT_FAILED                   { $errorCodes{ 'EC_STAGE_PUT_FAILED' }{'CODE-NO'}  }
sub EC_STAGE_TELNET_CLOSE_FAILED          { $errorCodes{ 'EC_STAGE_TELNET_CLOSE_FAILED' }{'CODE-NO'}  }
sub EC_STAGE_TELNET_INVALID_OPTIONS       { $errorCodes{ 'EC_STAGE_TELNET_INVALID_OPTIONS' }{'CODE-NO'}  }
sub EC_STAGE_TELNET_OPEN_FAILED           { $errorCodes{ 'EC_STAGE_TELNET_OPEN_FAILED' }{'CODE-NO'}  }
sub EC_STAGE_TYPE_FILE_MAPPING_FAILED     { $errorCodes{ 'EC_STAGE_TYPE_FILE_MAPPING_FAILED' }{'CODE-NO'}  }
sub EC_STAGE_WARN_EXCEEDS_THRESHOLD       { $errorCodes{ 'EC_STAGE_WARN_EXCEEDS_THRESHOLD' }{'CODE-NO'}  }
sub EC_SUCCESS                            { $errorCodes{ 'EC_SUCCESS' }{'CODE-NO'}  }
sub EC_SWPFILE_NOT_SUPPORTED              { $errorCodes{ 'EC_SWPFILE_NOT_SUPPORTED' }{'CODE-NO'}  }
sub EC_SW_CHECKS_NOK                      { $errorCodes{ 'EC_SW_CHECKS_NOK' }{'CODE-NO'}  }
sub EC_SW_MGNT_SIZE_NOK                   { $errorCodes{ 'EC_SW_MGNT_SIZE_NOK' }{'CODE-NO'}  }
sub EC_SW_NAME_ERROR                      { $errorCodes{ 'EC_SW_NAME_ERROR' }{'CODE-NO'}  }
sub EC_SW_RED_FAILED                      { $errorCodes{ 'EC_SW_RED_FAILED' }{'CODE-NO'}  }
sub EC_SYNC_TO_STBY_FAILED                { $errorCodes{ 'EC_SYNC_TO_STBY_FAILED' }{'CODE-NO'}  }
sub EC_TARGET_DIFF_FROM_ACTIVE_OSWP       { $errorCodes{ 'EC_TARGET_DIFF_FROM_ACTIVE_OSWP' }{'CODE-NO'}  }
sub EC_TARGET_DIFF_FROM_PASSIVE_OSWP      { $errorCodes{ 'EC_TARGET_DIFF_FROM_PASSIVE_OSWP' }{'CODE-NO'}  }
sub EC_TFTP_MISSING_FAILED                { $errorCodes{ 'EC_TFTP_MISSING_FAILED' }{'CODE-NO'}  }
sub EC_TFTP_OPEN_FAIL                     { $errorCodes{ 'EC_TFTP_OPEN_FAIL' }{'CODE-NO'}  }
sub EC_TFTP_OPEN_FAILED                   { $errorCodes{ 'EC_TFTP_OPEN_FAILED' }{'CODE-NO'}  }
sub EC_TFTP_PUT_FAILED                    { $errorCodes{ 'EC_TFTP_PUT_FAILED' }{'CODE-NO'}  }
sub EC_TFTP_SUSPEND_FAILED                { $errorCodes{ 'EC_TFTP_SUSPEND_FAILED' }{'CODE-NO'}  }
sub EC_TRANSFER_WAITTIME_EXCEED           { $errorCodes{ 'EC_TRANSFER_WAITTIME_EXCEED' }{'CODE-NO'}  }
sub EC_THRESHOLD_WARN                     { $errorCodes{ 'EC_THRESHOLD_WARN' }{'CODE-NO'}  }
sub EC_THRESHOLD_WARN_EXCEED              { $errorCodes{ 'EC_THRESHOLD_WARN_EXCEED' }{'CODE-NO'}  }
sub EC_TOO_MANY_MC_SRC_PKG                { $errorCodes{ 'EC_TOO_MANY_MC_SRC_PKG' }{'CODE-NO'}  }
sub EC_TO_NTVLAN_EXISTS                   { $errorCodes{ 'EC_TO_NTVLAN_EXISTS' }{'CODE-NO'}  }
sub EC_TO_SHUBVLAN_EXISTS                 { $errorCodes{ 'EC_TO_SHUBVLAN_EXISTS' }{'CODE-NO'}  }
sub EC_TRX_PROTOCOL_MISMATCH              { $errorCodes{ 'EC_TRX_PROTOCOL_MISMATCH' }{'CODE-NO'}  }
sub EC_TTL_0_FWD_FAILED                   { $errorCodes{ 'EC_TTL_0_FWD_FAILED' }{'CODE-NO'}  }
sub EC_UNKNOWN_NE_FAM                     { $errorCodes{ 'EC_UNKNOWN_NE_FAM' }{'CODE-NO'}  }
sub EC_UPGR_NOT_ADVISED                   { $errorCodes{ 'EC_UPGR_NOT_ADVISED' }{'CODE-NO'}  }
sub EC_UPGR_RESTRICTED                    { $errorCodes{ 'EC_UPGR_RESTRICTED' }{'CODE-NO'}  }
sub EC_UPLOAD_DB_FAILED                   { $errorCodes{ 'EC_UPLOAD_DB_FAILED' }{'CODE-NO'}  }
sub EC_USED_CCPORT                        { $errorCodes{ 'EC_USED_CCPORT' }{'CODE-NO'}  }
sub EC_USED_NT_VRF                        { $errorCodes{ 'EC_USED_NT_VRF' }{'CODE-NO'}  }
sub EC_USED_SHUBPVID                      { $errorCodes{ 'EC_USED_SHUBPVID' }{'CODE-NO'}  }
sub EC_USED_SHUB_ROUTEDEST                { $errorCodes{ 'EC_USED_SHUB_ROUTEDEST' }{'CODE-NO'}  }
sub EC_USED_VLANPP                        { $errorCodes{ 'EC_USED_VLANPP' }{'CODE-NO'}  }
sub EC_VLANBIND_PARSE_ERROR               { $errorCodes{ 'EC_VLANBIND_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_VLANPORT_PARSE_ERROR               { $errorCodes{ 'EC_VLANPORT_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_VLAN_PARSE_ERROR                   { $errorCodes{ 'EC_VLAN_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_VLAN_PP_PARSE_ERROR                { $errorCodes{ 'EC_VLAN_PP_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_VLAN_USED                          { $errorCodes{ 'EC_VLAN_USED' }{'CODE-NO'}  }
sub EC_VMAC_HOST_ID_LIMITS_EXCEED         { $errorCodes{ 'EC_VMAC_HOST_ID_LIMITS_EXCEED' }{'CODE-NO'}  }
sub EC_VRF_PARSE_ERROR                    { $errorCodes{ 'EC_VRF_PARSE_ERROR' }{'CODE-NO'}  }
sub EC_WRONG_VARIANT                      { $errorCodes{ 'EC_WRONG_VARIANT' }{'CODE-NO'}  }
sub EC_XFTP_GET_OSWP_FAIL                 { $errorCodes{ 'EC_XFTP_GET_OSWP_FAIL' }{'CODE-NO'}  }
sub EC_XFTP_GET_SWP_FAIL                  { $errorCodes{ 'EC_XFTP_GET_SWP_FAIL' }{'CODE-NO'}  }
sub EC_XFTP_PUT_OSWP_FAIL                 { $errorCodes{ 'EC_XFTP_PUT_OSWP_FAIL' }{'CODE-NO'}  }
sub EC_XFTP_PUT_SWP_FAIL                  { $errorCodes{ 'EC_XFTP_PUT_SWP_FAIL' }{'CODE-NO'}  }


my $IGNORE = 1;
my $WARNING = 3;
my $ERROR = 5;

my %warningLevelStrings = ($IGNORE => "Ignored",
			   $WARNING => "Warning",
			   $ERROR => "Error");

%exitCodes  = (
	       # Success
	       SUCCESS =>                 [0, "Success"],
	       # Warnings
	       EXIT_WARN =>               [1, "Warnings generated by one or more child processes, check log file"],
	       EXIT_CHILD_FATAL =>        [2, "Fatal error generated by one or more child processes, check log file(s)"],
	       EXIT_SIGNAL =>             [3, "Aborted due to signal"],
	       EXIT_DIED =>               [4, "Program terminated via PERL die()"],
	       CONDITONAL_SUCCESS =>      [5, "Conditional success from SNMP to YANG upgrade"],
	       # Fatal errors
	       CMD_INCOMPL =>               [10, "**FATAL** Command incomplete, mandatory parameters missing"],
	       PARAM_CFG_NOT_OK =>          [11, "**FATAL** param.cfg contents not correct on the regular expression part"],
	       CREATE_DIR_FAILED =>         [12, "**FATAL** Creation of directory failed"],
	       CREATE_ISS_CONF_FAIL =>      [13, "**FATAL** Touching iss.conf failed"],
	       CREATE_NVRAM_FAIL =>         [14, "**FATAL** Touching issnvram.txt failed"],
	       NT_RESTART_FAIL =>           [15, "**FATAL** ISAM lost after NT restart"],
	       SHUB_RESTART_FAIL =>         [16, "**FATAL** ISAM lost after SHUB restart, check log files"],
	       SHUB_CONF_SELECT_FAIL =>     [17, "**FATAL** Failed to set new config on SHUB"],
	       CHG_OSWP_FAIL =>             [18, "**FATAL** Activation of the new OSWP failed"],
	       CREATE_FILE_FAIL =>          [19, "**FATAL** File creation failed"],
	       OPENDIR_FAIL =>              [20, "**FATAL** Opening the new database directory failed"],
	       FILE_OPEN_FAIL =>            [21, "**FATAL** Opening a file failed"],
	       OSWP_FILE_NO_IP =>           [22, "**FATAL** OSWP misses TFTP server IP address"],
	       PATH_DEAD_END =>             [23, "**FATAL** Dead-ended migration/upgrade path"],
	       UPG_IMPOSSIBLE =>            [24, "**FATAL** Migration/upgrade not possible"],
	       UPG_RESTRICTED =>            [25, "**FATAL** Migration/upgrade restricted"],
	       NO_NT_OFLMT_VER =>           [26, "**FATAL** Could not obtain NT offline migration tool version"],
	       NO_SHUB_OFLMT_VER =>         [27, "**FATAL** Could not obtain SHUB offline migration tool version"],
	       NT_OFLMT_NO_EXE =>           [28, "**FATAL** NT offline migration tool not executable"],
	       OCTOPUS_NO_EXE =>            [29, "**FATAL** Octopus not executable"],
	       VERS_DET_FAILED =>           [30, "**FATAL** Version determination failed (unix)"],
	       CREATE_WORK_DIR_FAILED =>    [31, "**FATAL** Creation of temporaty working directory failed"],
	       CREATE_NODE_FILE_FAIL   =>   [32, "**FATAL** Creation of node log file failed"],
	       CREATE_STAT_FILE_FAIL   =>   [33, "**FATAL** Creation of node status result file failed"],
	       HOSTS_FILE_OPEN_FAILED =>    [34, "**FATAL** Opening the hosts file failed"],
	       SHUB_OFLMT_VERSION_2_HIGH => [35, "**FATAL** SHUB OFLMT version too high for the version of this toolset"],
	       CREATE_CHCK_FILE_FAILED =>   [36, "**FATAL** Creation of node status check file failed"],
	       OSWP_FILE_INV_CHARS =>       [37, "**FATAL** OSWP contains invalid characters: only 'A-Z', 'a-z', '0-9', '.', '/', ':' and '_' are allowed"],
	       PARAM_INV_MACMODE =>         [38, "**FATAL** Invalid MAC-mode when migrating from ECNT-A to ECNT-C, only native or legacy allowed"],
	       CALC_IP_CKSUM_NO_EXEC =>     [39, "**FATAL** CalcIpCheckSum (for SHUB DB migration) not executable"],
	       NT_OFLMT_VERSION_2_HIGH =>   [40, "**FATAL** NT OFLMT version too high for the version of this toolset"],
	       PARAM_BOARD_CONV_ERR =>      [41, "**FATAL** Cannot use toecntc and tonantb together, they are mutually exclusive"],
	       WORK_NOT_WRITEABLE =>        [42, "**FATAL** Temporary working directory not writeable."],
	       NO_IVPS_OFLMT_VER =>         [43, "**FATAL** Could not obtain xVPS offline migration tool version"],
	       IVPS_OFLMT_NO_EXE =>         [44, "**FATAL** xVPS offline migration tool not executable"],
	       IVPS_OFLMT_VERSION_2_HIGH => [45, "**FATAL** xVPS OFLMT version too high for the version of this toolset"],
	       IVPS_INV_FAILED =>           [46, "**FATAL** Failed to determine xVPS/IP address mapping"],
	       NO_IVPS_OFLMT =>             [47, "**FATAL** IVPS offline migration tool not found"],
	       NO_NT_OFLMT =>               [48, "**FATAL** NT offline migration tool not found"],
	       PARAM_CFG_MISSING =>         [49, "**FATAL** param.cfg missing: run Config.pl first"],
	       PARAM_CFG_NEEDS_UPDATE =>    [50, "**FATAL** param.cfg needs update: run Config.pl first"],
	       BAD_CKSUM_SYNTAX =>          [51, "**FATAL** cksum output not according to expectation"],
	       UNKNOWN_SCRIPT =>            [52, "**FATAL** script unknown to toolset"],
	       BAD_CKSUM =>                 [53, "**FATAL** bad checksum"],
	       SHUB_OFLMT_VER_TOO_LOW =>    [54, "**FATAL** SHUB offline migration tool version too low for migration to target release"],
	       
	       #added for UE Merging
	       INVALID_ISAM_IP =>         [56, "**FATAL** Invalid ISAM IP"],
	       UNABLE_TO_RESOLVE_HOST_NAME =>   [57, "**FATAL** Unable to resolve the ip address for the given NE name"],
               SWP_FILE_OPEN_FAILED =>    [58, "**FATAL** Opening the swp file failed"],

	       # Internal errors -- should normally not occur!!!
	       OCTO_ENV_NOT_SET =>          [100, "**INTERNAL** Octopus environment not set"],
	       NT_CAP_NOTSET =>             [101, "**INTERNAL** SwMgntIni crititcal size not defined"],
	       NT_CAP_ALT_NOTSET =>         [102, "**INTERNAL** Alternate SwMgntIni crititcal size not defined"],
	       NT_SWPARSIZE_NOTSET =>       [103, "**INTERNAL** SW partition size not defined"],
	       COMMON_ENV_NOT_SET =>        [104, "**INTERNAL** Common environment not set"],

	       #added for start of activity
	       START_ACTIVITY    =>         [200, "Processing Started for activity"],
	       
	       #added for UE Merging
	       PERL_FATAL_RUNTIME_ERR =>    [255, "**FATAL** Perl run-time error caused premature exit"]
	      );

%errorCodes = (
	       'EC_SUCCESS'                             => {
	       				     'CODE-NO'  => 0,
	       				     'MESSAGE'  => "Operation Succeed",
	       				     'USED-IN'  => [ 'SWRD', 'SWMM' ],
	                                         },
	       'EC_NO_PING'                             => {
	       				     'CODE-NO'  => 1001,
	       				     'MESSAGE'  => "ISAM is not responding to ping command",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
### OCTOPUS SESSION DETAILS
	       'EC_OCTO_ACTIVE'                         => {
	       				     'CODE-NO'  => 1002,
	       				     'MESSAGE'  => "Octopus session is already open to this ISAM, Please wait to complete/close",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'PRCK' ],
	                                         },
	       'EC_OCTO_FAIL'                           => {
	       				     'CODE-NO'  => 1003,
	       				     'MESSAGE'  => "Executing 'octopus' failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_SNMP_SESS_ERR'                       => {
	       				     'CODE-NO'  => 1021,
	       				     'MESSAGE'  => "Failed to establish SNMP Session. SNMP correctly configured?",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'CHKC', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'TONANTD', 'TOSIP', 'IPINV', 'TONANTE', 'RLBK' ],
	                                         },
	       'EC_SNMP_MAX_SIZE_FAIL'                  => {
	       				     'CODE-NO'  => 1022,
	       				     'MESSAGE'  => "Failed to set maximum SNMP message size",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'PRCK', 'SCHK', 'CHKC', 'RLBK' ],
	                                         },
	       'EC_SNMP_NO_GET'                         => {
	       				     'CODE-NO'  => 1023,
	       				     'MESSAGE'  => "Could not retrieve system value using SNMP.",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'CHKC', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'ONT', 'RLBK' ],
	                                         },
	       'EC_SNMP_ACT_FAIL'                       => {
	       				     'CODE-NO'  => 1024,
	       				     'MESSAGE'  => "Activating new OSWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_SNMP_CLIENT_CREDENTIAL_MISMATCH'     => {
	       				     'CODE-NO'  => 1025,
	       				     'MESSAGE'  => "SFTP Client credential mismatch with NE configuration",
	       				     'USED-IN'  => [ 'PRCK' ],
	                                         },
	       'EC_SNMP_OPEN_FAIL'                      => {
	       				     'CODE-NO'  => 1026,
	       				     'MESSAGE'  => "Failed to SNMP session",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_SNMP_SESS_ESTB_ERR'                  => {
	       				     'CODE-NO'  => 1027,
	       				     'MESSAGE'  => "Could not open SNMP session",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_OCTO_PS_ERROR'                       => {
	       				     'CODE-NO'  => 1028,
	       				     'MESSAGE'  => "Failed to determine whether an octopus session is already open to this ISAM",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'PRCK' ],
	                                         },
	       'EC_CONFIG_LOCK_ACQUIRE_FAILED'           => {
	       				     'CODE-NO'  => 1051,
	       				     'MESSAGE'  => "Failed to Acquire configuration-lock ownership",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'ISAM', 'RBLK', 'RLBK' ],
	                                         },
	       'EC_CONFIG_LOCK_RELEASE_FAILED'          => {
	       				     'CODE-NO'  => 1052,
	       				     'MESSAGE'  => "Failed to release configuration-lock ownership",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'ISAM', 'RLBK' ],
	                                         },
	       'EC_CONFIG_LOCK_ALREADY_TAKEN'           => {
	       				     'CODE-NO'  => 1053,
	       				     'MESSAGE'  => "Ownership for configuration-lock acquired by others or Release period expired",
	       				     'USED-IN'  => [ 'SWDL', 'SCHK', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'TOSIP', 'ISAM', 'RBLK' ],
	                                         },
	       'EC_ABORT_DL_FAIL'                       => {
	       				     'CODE-NO'  => 1600,
	       				     'MESSAGE'  => "Abort SW download failed",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_ACC_OPERID_MISMATCH'                 => {
	       				     'CODE-NO'  => 1601,
	       				     'MESSAGE'  => "Access operator ID mismatch",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR' ],
	                                         },
	       'EC_ACTIVATE_FAIL'                       => {
	       				     'CODE-NO'  => 1603,
	       				     'MESSAGE'  => "New OSWP not active",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_ALW_ALARM_FAILED'                    => {
	       				     'CODE-NO'  => 1604,
	       				     'MESSAGE'  => "Unable to allow Alarms",
	       				     'USED-IN'  => [ 'SWDL', 'SCHK', 'ISAM' ],
	                                         },
	       'EC_AUTHOOS_THR_EXCEEDED'                => {
	       				     'CODE-NO'  => 1605,
	       				     'MESSAGE'  => "No-authentication threshold exceeded",
	       				     'USED-IN'  => [ 'CHKC' ],
                             'CHCK'     => $IGNORE,
	                                         },
	       'EC_BACKUPDB_FAILED'                     => {
	       				     'CODE-NO'  => 1606,
	       				     'MESSAGE'  => "Failed to take NT database backup",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_BATCHMODE_FAILED'                    => {
	       				     'CODE-NO'  => 1607,
	       				     'MESSAGE'  => "Enabling CLI batch mode failed",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_BRIDGECONFMM_THR_EXCEEDED'           => {
	       				     'CODE-NO'  => 1608,
	       				     'MESSAGE'  => "Bridge configuration mismatch",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_BRIDGEPORT_PARSE_ERROR'              => {
	       				     'CODE-NO'  => 1609,
	       				     'MESSAGE'  => "Bridge port configuration parsing error",
	       				     'USED-IN'  => [ 'CCCE', 'CNWV' ],
	                                         },
	       'EC_CDE_NOTFOUND'                        => {
	       				     'CODE-NO'  => 1610,
	       				     'MESSAGE'  => "CDE file not found",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_CLIENTPORT_PARSE_ERROR'              => {
	       				     'CODE-NO'  => 1611,
	       				     'MESSAGE'  => "PPPoX-relay CC client port configuration parsing error",
	       				     'USED-IN'  => [ 'CCCE', 'PPPC' ],
	                                         },
	       'EC_CLI_CMD_FAILED'                      => {
	       				     'CODE-NO'  => 1612,
	       				     'MESSAGE'  => "Failed to execute a CLI command",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_CLI_PREPARE_FAILED'                  => {
	       				     'CODE-NO'  => 1613,
	       				     'MESSAGE'  => "Preparing CLI environment failed",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_CMD_INCOMPL'                         => {
	       				     'CODE-NO'  => 1614,
	       				     'MESSAGE'  => "Command incomplete",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_COMMIT_FAIL'                         => {
	       				     'CODE-NO'  => 1615,
	       				     'MESSAGE'  => "New OSWP not commited",
	       				     'USED-IN'  => [ 'SWDL', 'SWRD' ],
	                                         },
	       'EC_COMMIT_FAILED'                       => {
	       				     'CODE-NO'  => 1616,
	       				     'MESSAGE'  => "SW commit failed",
	       				     'USED-IN'  => [ 'COMT' ],
	                                         },
	       'EC_CONFIG_DL_FAILED'                    => {
	       				     'CODE-NO'  => 1617,
	       				     'MESSAGE'  => "Failed to configure the SWP download parameters",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_CONFIG_OPRID_FAILED'                 => {
	       				     'CODE-NO'  => 1618,
	       				     'MESSAGE'  => "Failed to configure accessOperatorId",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR' ],
	                                         },
	       'EC_CONF_FILE_READ_FAILED'               => {
	       				     'CODE-NO'  => 1619,
	       				     'MESSAGE'  => "Failed to open board config file",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_COPY_FAILED'                         => {
	       				     'CODE-NO'  => 1620,
	       				     'MESSAGE'  => "Failed to copy file to target directory",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_COPY_OFLDB_FAILED'                   => {
	       				     'CODE-NO'  => 1621,
	       				     'MESSAGE'  => "Copying the offline DB to dm_complete.tar failed",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_CORR_VLAN_TBL_FAILED'                => {
	       				     'CODE-NO'  => 1622,
	       				     'MESSAGE'  => "Failed to prepare the VLAN static table for migration",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_CREATE_CONFIG_ERROR'                 => {
	       				     'CODE-NO'  => 1623,
	       				     'MESSAGE'  => "Creation of configuration failed",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_CREATE_ISS_FAIL'                     => {
	       				     'CODE-NO'  => 1624,
	       				     'MESSAGE'  => "Could not create empty iss.conf",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_CREATE_NVR_FAIL'                     => {
	       				     'CODE-NO'  => 1625,
	       				     'MESSAGE'  => "Could not create empty issnvram.txt",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_CREATE_OSWP_FAIL'                    => {
	       				     'CODE-NO'  => 1626,
	       				     'MESSAGE'  => "Failed to create OSWP file",
	       				     'USED-IN'  => [ 'SWRD', 'SWMM' ],
	                                         },
	       'EC_CREATE_RESTOREDIR_FAILED'            => {
	       				     'CODE-NO'  => 1627,
	       				     'MESSAGE'  => "Failed to create RestoreDB directory",
	       				     'USED-IN'  => [ 'PRCK' ],
	                                         },
	       'EC_CREATE_SWP_FAIL'                     => {
	       				     'CODE-NO'  => 1628,
	       				     'MESSAGE'  => "Failed to create SWP file",
	       				     'USED-IN'  => [ 'SWRD', 'SWMM' ],
	                                         },
	       'EC_CREATE_TARGET_DIR_FAILED'            => {
	       				     'CODE-NO'  => 1629,
	       				     'MESSAGE'  => "Failed to create target directory",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_CREATE_WORKSPACE_FAILED'             => {
	       				     'CODE-NO'  => 1630,
	       				     'MESSAGE'  => "Failed to create node workspace",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_CVT_COMBINE_FAIL'                    => {
	       				     'CODE-NO'  => 1631,
	       				     'MESSAGE'  => "Failed to generate converted database",
	       				     'USED-IN'  => [ 'TONANTD', 'TOSIP', 'TONANTE' ],
	                                         },
	       'EC_CVT_IHUB_FAIL'                       => {
	       				     'CODE-NO'  => 1632,
	       				     'MESSAGE'  => "IHUB reconfiguration failed",
	       				     'USED-IN'  => [ 'TONANTE' ],
	                                         },
	       'EC_CVT_TO_IHUB_FAIL'                    => {
	       				     'CODE-NO'  => 1633,
	       				     'MESSAGE'  => "Conversion to IHUB CLI commands failed",
	       				     'USED-IN'  => [ 'TONANTD' ],
	                                         },
	       'EC_DB_AUTO_ACTIVATE_ON_DWLD'            => {
	       				     'CODE-NO'  => 1634,
	       				     'MESSAGE'  => "Automatic DB activation on Download is enabled. Please retry migration after configuring manual DB activation",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_DB_COMBINE_FAILED'                   => {
	       				     'CODE-NO'  => 1635,
	       				     'MESSAGE'  => "Creation of integrated DB failed",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_DB_COUNT_FILES_FAILED'               => {
	       				     'CODE-NO'  => 1636,
	       				     'MESSAGE'  => "Failed to count the number of files in the migrated database",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_DB_DL_FAILED'                        => {
	       				     'CODE-NO'  => 1637,
	       				     'MESSAGE'  => "Failed to download database",
	       				     'USED-IN'  => [ 'MIGR', 'TOSIP' ],
	                                         },
	       'EC_DB_TOO_MANY_FILES'                   => {
	       				     'CODE-NO'  => 1638,
	       				     'MESSAGE'  => "Migrated database exceeds safe file limit for this release",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_DB_VERS_UNEXPECTED'                  => {
	       				     'CODE-NO'  => 1639,
	       				     'MESSAGE'  => "DB version not as expected",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_DB_VER_UNDEFINED'                    => {
	       				     'CODE-NO'  => 1640,
	       				     'MESSAGE'  => "DB Version is not specified",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_DEL_CONFIG_ERROR'                    => {
	       				     'CODE-NO'  => 1641,
	       				     'MESSAGE'  => "Deletion of configuration failed",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_DEL_SHUB_SW_FAILED'                  => {
	       				     'CODE-NO'  => 1642,
	       				     'MESSAGE'  => "Failed to remove SHUB SW from SW directory",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR' ],
	                                         },
	       'EC_DETERMINE_LOG_FAILED'                => {
	       				     'CODE-NO'  => 1643,
	       				     'MESSAGE'  => "Determine Log failed",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'CHKC', 'TONANTD', 'TOSIP', 'IPINV', 'TONANTE', 'RLBK' ],
	                                         },
	       'EC_DETERMINE_OPER_FAILED'               => {
	       				     'CODE-NO'  => 1644,
	       				     'MESSAGE'  => "Failed to determine operation (migration or upgrade)",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR' ],
	                                         },
	       'EC_DHCPRELAY_PARSE_ERROR'               => {
	       				     'CODE-NO'  => 1645,
	       				     'MESSAGE'  => "DHCP relay-agent configuration parsing error",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_DIR_SW_LIST_FAILED'                  => {
	       				     'CODE-NO'  => 1646,
	       				     'MESSAGE'  => "Failed to list software under /Sw on ISAM",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_DL_ACT_GET_ONT_LIST_FAILED'          => {
	       				     'CODE-NO'  => 1647,
	       				     'MESSAGE'  => "No matching ONT(s) found on the OLT for the given release / type",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_DL_ACT_RTRV_ONTSWCTRL_FAILED'        => {
	       				     'CODE-NO'  => 1648,
	       				     'MESSAGE'  => "Failed to retrieve ONT software control table",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_DL_ACT_WARN_EXCEEDS_THRESHOLD'       => {
	       				     'CODE-NO'  => 1649,
	       				     'MESSAGE'  => "Download failures exceeds The Warning Threshold",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_DL_SWP_FAILED'                       => {
	       				     'CODE-NO'  => 1650,
	       				     'MESSAGE'  => "Failed to download SW package",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_DOWNLOAD_DB_FAILED'                  => {
	       				     'CODE-NO'  => 1651,
	       				     'MESSAGE'  => "Download restored DB to ISAM failed",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_EMERGENCY_CALL_ACTIVE'               => {
	       				     'CODE-NO'  => 1652,
	       				     'MESSAGE'  => "Emergency call is ongoing, max retries reached",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'PRCK', 'RLBK' ],
	                                         },
	       'EC_EMPTY_NT_DETECTED'                   => {
	       				     'CODE-NO'  => 1653,
	       				     'MESSAGE'  => "Only EMPTY NT types detected (impossible...)",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_EMPTY_RESPONSE'                      => {
	       				     'CODE-NO'  => 1654,
	       				     'MESSAGE'  => "Empty CLI response received",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_ENABLE_WR_FAILED'                    => {
	       				     'CODE-NO'  => 1655,
	       				     'MESSAGE'  => "Enable write /Sw failed",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_ENGINE_PARSE_ERROR'                  => {
	       				     'CODE-NO'  => 1656,
	       				     'MESSAGE'  => "PPPoX-relay CC engine configuration parsing error",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_ENV_NOT_OK'                          => {
	       				     'CODE-NO'  => 1657,
	       				     'MESSAGE'  => "Migration environment not OK",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_ETSI_NOT_SUPPORTED'                  => {
	       				     'CODE-NO'  => 1658,
	       				     'MESSAGE'  => "Rollback not supported for this NT/Family",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_EXTR_MCAST_FAILED'                   => {
	       				     'CODE-NO'  => 1659,
	       				     'MESSAGE'  => "Failed to extract IgmpMcTable",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_FILEXFRMODE_NOTSUPPORTED'            => {
	       				     'CODE-NO'  => 1660,
	       				     'MESSAGE'  => "Filetransfer protocol not supported by PBMT",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'PRCK' ],
	                                         },
	       'EC_GETCOMMTY_ERROR'                     => {
	       				     'CODE-NO'  => 1661,
	       				     'MESSAGE'  => "Failed to retrieve community strings",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_GET_ACC_BITS_FAILED'                 => {
	       				     'CODE-NO'  => 1662,
	       				     'MESSAGE'  => "Get Acc Bits failed",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'TONANTD', 'TONANTE', 'RLBK' ],
	                                         },
	       'EC_GET_ACC_OPERID_FAILED'               => {
	       				     'CODE-NO'  => 1663,
	       				     'MESSAGE'  => "Failed to retrieve access operator ID",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR' ],
	                                         },
	       'EC_GET_ACTIVATE_FAIL'                   => {
	       				     'CODE-NO'  => 1664,
	       				     'MESSAGE'  => "Failed to read OSWP activation state",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_GET_ALARM_FAILED'                    => {
	       				     'CODE-NO'  => 1665,
	       				     'MESSAGE'  => "Failed to retrieve outstanding alarms",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_GET_ALL_ONT_FAILED'                  => {
	       				     'CODE-NO'  => 1666,
	       				     'MESSAGE'  => "All ONT Types could not be found",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_GET_ATMPVC_ERROR'                    => {
	       				     'CODE-NO'  => 1667,
	       				     'MESSAGE'  => "Failed to retrieve atm pvc info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_GET_AUTHENT_FAILED'                  => {
	       				     'CODE-NO'  => 1668,
	       				     'MESSAGE'  => "Failed to retrieve authentication status",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_GET_BRIDGEINFO_ERROR'                => {
	       				     'CODE-NO'  => 1669,
	       				     'MESSAGE'  => "Failed to retrieve bridge info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_GET_BRIDGE_FAILED'                   => {
	       				     'CODE-NO'  => 1670,
	       				     'MESSAGE'  => "Failed to retrieve bridge configuration data",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_GET_CLIMODE_FAIL'                    => {
	       				     'CODE-NO'  => 1671,
	       				     'MESSAGE'  => "Failed to determine CLI profile mode",
	       				     'USED-IN'  => [ 'TONANTD', 'TOSIP', 'TONANTE' ],
	                                         },
	       'EC_GET_CLIMODE_FAILED'                  => {
	       				     'CODE-NO'  => 1672,
	       				     'MESSAGE'  => "Failed to determine operator profile-mode",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK', 'SCHK' ],
	                                         },
	       'EC_GET_COMMIT_FAIL'                     => {
	       				     'CODE-NO'  => 1673,
	       				     'MESSAGE'  => "Reading commit state failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_GET_DB_INFO_FAILED'                  => {
	       				     'CODE-NO'  => 1674,
	       				     'MESSAGE'  => "Failed to retrieve DB info",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'PRCK' ],
	                                         },
	       'EC_GET_FILEXFR_FAILED'                  => {
	       				     'CODE-NO'  => 1675,
	       				     'MESSAGE'  => "Failed to retrieve security filetransfer settings",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'PRCK' ],
	                                         },
	       'EC_GET_FTP_FAILED'                      => {
	       				     'CODE-NO'  => 1676,
	       				     'MESSAGE'  => "Failed to get File Transfer Protocol  settings",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_GET_H248_FAILED'                     => {
	       				     'CODE-NO'  => 1677,
	       				     'MESSAGE'  => "Obtaining the H.248 data failed",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_GET_IGMPCH_INFO_ERROR'               => {
	       				     'CODE-NO'  => 1678,
	       				     'MESSAGE'  => "Failed to retrieve IGMP channel info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_GET_LT_FAILED'                       => {
	       				     'CODE-NO'  => 1679,
	       				     'MESSAGE'  => "Failed to retrieve LT configuration/operational data/status",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK', 'SCHK' ],
	                                         },
	       'EC_GET_LT_INFO_FAILED'                  => {
	       				     'CODE-NO'  => 1680,
	       				     'MESSAGE'  => "Unable to get LT Info",
	       				     'USED-IN'  => [ 'PRCK' ],
	                                         },
	       'EC_GET_NEW_FREE_FAIL'                   => {
	       				     'CODE-NO'  => 1681,
	       				     'MESSAGE'  => "Failed to read free space",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_GET_NT_INFO_FAILED'                  => {
	       				     'CODE-NO'  => 1682,
	       				     'MESSAGE'  => "Obtaining (extended) NT info failed",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'TOSIP', 'ONT', 'RLBK' ],
	                                         },
	       'EC_GET_NT_TYPE_FAILED'                  => {
	       				     'CODE-NO'  => 1683,
	       				     'MESSAGE'  => "NT (board) Type not found",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'ONT', 'RLBK' ],
	                                         },
	       'EC_GET_OSPF_FAILED'                     => {
	       				     'CODE-NO'  => 1684,
	       				     'MESSAGE'  => "Failed to obtain OSPF info",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_GET_OSWP_FAIL'                       => {
	       				     'CODE-NO'  => 1685,
	       				     'MESSAGE'  => "Reading new active OSWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_GET_OSWP_FAILED'                     => {
	       				     'CODE-NO'  => 1686,
	       				     'MESSAGE'  => "Get OSWP failed",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'TOSIP', 'RLBK' ],
	                                         },
	       'EC_GET_OSWP_NAME_FAIL'                  => {
	       				     'CODE-NO'  => 1687,
	       				     'MESSAGE'  => "Reading OSWP name failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_GET_SFP_ASSIGNMENT_FAILED'           => {
	       				     'CODE-NO'  => 1688,
	       				     'MESSAGE'  => "Failed to get SFP Assignments from ISAM",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_GET_SHELFCONFIG_FAIL'                => {
	       				     'CODE-NO'  => 1689,
	       				     'MESSAGE'  => "Failed to retrieve shelf configuration",
	       				     'USED-IN'  => [ 'TONANTD', 'TONANTE' ],
	                                         },
	       'EC_GET_SHUB_CONF_FAILED'                => {
	       				     'CODE-NO'  => 1690,
	       				     'MESSAGE'  => "Could not retrieve SHUB active configuration",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK' ],
	                                         },
	       'EC_GET_SHUB_SWSIZE_FAILED'              => {
	       				     'CODE-NO'  => 1691,
	       				     'MESSAGE'  => "Failed to determine SHUB SW size",
	       				     'USED-IN'  => [ 'PRCK' ],
	                                         },
	       'EC_GET_SNMP_ACC_FAILED'                 => {
	       				     'CODE-NO'  => 1692,
	       				     'MESSAGE'  => "Failed to obtain SNMP communities",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_GET_SNMP_SEC_FAIL'                   => {
	       				     'CODE-NO'  => 1693,
	       				     'MESSAGE'  => "Failed to retrieve SNMP security settings",
	       				     'USED-IN'  => [ 'TONANTD', 'TOSIP', 'TONANTE' ],
	                                         },
	       'EC_GET_SW_PARSIZE_FAILED'               => {
	       				     'CODE-NO'  => 1694,
	       				     'MESSAGE'  => "Failed to retrieve SW partition size",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK' ],
	                                         },
	       'EC_GET_TFTPIP_FAIL'                     => {
	       				     'CODE-NO'  => 1695,
	       				     'MESSAGE'  => "Failed to obtain File transer server IP address",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_TRANSFER_SNMPGET_FAILED' => {
	       				     'CODE-NO'  => 1696,
	       				     'MESSAGE'  => "Failed to determine whether a file transfer is currently active",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'TOSIP' ],
	                                         },
	       'EC_GET_TRANS_TO_FAILED'                 => {
	       				     'CODE-NO'  => 1697,
	       				     'MESSAGE'  => "Failed to get transaction timeout",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_GET_VOICEIP_FAILED'                  => {
	       				     'CODE-NO'  => 1698,
	       				     'MESSAGE'  => "Failed to obtaine Voice IP info",
	       				     'USED-IN'  => [ 'IPINV' ],
	                                         },
	       'EC_GET_VVLAN_FAILED'                    => {
	       				     'CODE-NO'  => 1699,
	       				     'MESSAGE'  => "Failed to obtain Voice VLAN info",
	       				     'USED-IN'  => [ 'IPINV' ],
	                                         },
	       'EC_GET_WARN_TYPE_COULD_NOT_FOUND'       => {
	       				     'CODE-NO'  => 1700,
	       				     'MESSAGE'  => "ONT Type could not be found",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_GET_XDSL_FAILED'                     => {
	       				     'CODE-NO'  => 1701,
	       				     'MESSAGE'  => "Failed to retrieve XDSL line configuration/operational data/status",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_HIGHER_PASSIVE_RESTRICTED'           => {
	       				     'CODE-NO'  => 1702,
	       				     'MESSAGE'  => "Restricted :Passive Software Release version is higher than Active Release",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_INCOMPAT_NT'                         => {
	       				     'CODE-NO'  => 1703,
	       				     'MESSAGE'  => "NT-A and NT-B are not incompatible NT types",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'RLBK' ],
	                                         },
	       'EC_INCOMPAT_NT_ICS_STATE'               => {
	       				     'CODE-NO'  => 1704,
	       				     'MESSAGE'  => "NT-A and NT-B are not in valid ICS states",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INCOMPAT_NT_STATE'                   => {
	       				     'CODE-NO'  => 1705,
	       				     'MESSAGE'  => "Incompatible NT state",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_INCORR_SHUB_OFLMT'                   => {
	       				     'CODE-NO'  => 1706,
	       				     'MESSAGE'  => "Incorrect SHUB OFLMT for ECNT-A to ECNT-C migration, need 1.26 or higher",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INCORR_SHUB_OFLMT_NANT_B'            => {
	       				     'CODE-NO'  => 1707,
	       				     'MESSAGE'  => "Incorrect SHUB OFLMT for NANT-A to NANT-B migration, needs1.32 or higher",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INFO_CONF_BRIDGEPORT_ERROR'          => {
	       				     'CODE-NO'  => 1708,
	       				     'MESSAGE'  => "Could not retrieve bridge port info",
	       				     'USED-IN'  => [ 'CCCE', 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_CCPORT_ERROR'              => {
	       				     'CODE-NO'  => 1709,
	       				     'MESSAGE'  => "Could not retrieve CC client-port configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_CLIENTPORT_ERROR'          => {
	       				     'CODE-NO'  => 1710,
	       				     'MESSAGE'  => "Could not retrieve PPPoX-relay CC client port info",
	       				     'USED-IN'  => [ 'CCCE', 'PPPC' ],
	                                         },
	       'EC_INFO_CONF_DHCPRELAY_ERROR'           => {
	       				     'CODE-NO'  => 1711,
	       				     'MESSAGE'  => "Could not retrieve DHCP relay-agent info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_INFO_CONF_ENGINE_ERROR'              => {
	       				     'CODE-NO'  => 1712,
	       				     'MESSAGE'  => "Could not retrieve PPPoX-relay CC engine info",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_INFO_CONF_INTERFACE_ERROR'           => {
	       				     'CODE-NO'  => 1713,
	       				     'MESSAGE'  => "Could not retrieve customer IDs",
	       				     'USED-IN'  => [ 'PPPC' ],
	                                         },
	       'EC_INFO_CONF_MCAST_ERROR'               => {
	       				     'CODE-NO'  => 1714,
	       				     'MESSAGE'  => "Could not retrieve MCAST configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_NTVRF_ERROR'               => {
	       				     'CODE-NO'  => 1715,
	       				     'MESSAGE'  => "Could not retrieve NT IP VRF configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_NT_VLAN_ERROR'             => {
	       				     'CODE-NO'  => 1716,
	       				     'MESSAGE'  => "Failed to retrieve NT VLAN configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_PVCCONFIG_ERROR'           => {
	       				     'CODE-NO'  => 1717,
	       				     'MESSAGE'  => "Could not retrieve PVC configuration",
	       				     'USED-IN'  => [ 'PPPC' ],
	                                         },
	       'EC_INFO_CONF_QOS_ERROR'                 => {
	       				     'CODE-NO'  => 1718,
	       				     'MESSAGE'  => "Could not retrieve qos session profile info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_INFO_CONF_SHUBVLAN_ERROR'            => {
	       				     'CODE-NO'  => 1719,
	       				     'MESSAGE'  => "Failed to retrieve NT VLAN configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_SHUBVRF_ERROR'             => {
	       				     'CODE-NO'  => 1720,
	       				     'MESSAGE'  => "Could not retrieve SHUB IP VRF configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_SHUB_BRG_ERROR'            => {
	       				     'CODE-NO'  => 1721,
	       				     'MESSAGE'  => "Could not retrieve SHUB bridge port configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_SHUB_VLAN_ERROR'           => {
	       				     'CODE-NO'  => 1722,
	       				     'MESSAGE'  => "Failed to retrieve SHUB VLAN configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_VLANBIND_ERROR'            => {
	       				     'CODE-NO'  => 1723,
	       				     'MESSAGE'  => "Could not retrieve vlan-bind info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_INFO_CONF_VLANPORT_ERROR'            => {
	       				     'CODE-NO'  => 1724,
	       				     'MESSAGE'  => "Could not retrieve VLAN port customer IDs",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_VLANPP_ERROR'              => {
	       				     'CODE-NO'  => 1725,
	       				     'MESSAGE'  => "Could not retrieve VLAN port-protcol configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INFO_CONF_VLAN_ERROR'                => {
	       				     'CODE-NO'  => 1726,
	       				     'MESSAGE'  => "Could not retrieve VLAN info",
	       				     'USED-IN'  => [ 'IBRG', 'PPPC' ],
	                                         },
	       'EC_INFO_CONF_VLAN_PP_ERROR'             => {
	       				     'CODE-NO'  => 1727,
	       				     'MESSAGE'  => "Could not retrieve VLAN port-protocol info",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_INFO_CONF_VRF_ERROR'                 => {
	       				     'CODE-NO'  => 1728,
	       				     'MESSAGE'  => "Could not retrieve VRF info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_INHBT_MORE_FAILED'                   => {
	       				     'CODE-NO'  => 1729,
	       				     'MESSAGE'  => "Enabling environment 'print no-more' failed",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK' ],
	                                         },
	       'EC_INHIBIT_ALRM_FAILED'                 => {
	       				     'CODE-NO'  => 1730,
	       				     'MESSAGE'  => "CLI alarm inhibit failed",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_INH_ALARM_FAILED'                    => {
	       				     'CODE-NO'  => 1731,
	       				     'MESSAGE'  => "Unable to inhibit Alarms",
	       				     'USED-IN'  => [ 'SWDL', 'SCHK', 'ISAM' ],
	                                         },
	       'EC_INIT_COPY_FAILED'                    => {
	       				     'CODE-NO'  => 1732,
	       				     'MESSAGE'  => "Failed to copy init file",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'TOSIP' ],
	                                         },
	       'EC_INPUTDB_PROBLEM'                     => {
	       				     'CODE-NO'  => 1733,
	       				     'MESSAGE'  => "Problem determining NT type or DB version from input database",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INVALID_2_ECNT_C'                    => {
	       				     'CODE-NO'  => 1734,
	       				     'MESSAGE'  => "ECNT-A to ECNT-C conversion only possible from R2.4.01 onwards",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INVALID_2_ECNT_E'                    => {
	       				     'CODE-NO'  => 1735,
	       				     'MESSAGE'  => "ECNT-A/ECNT-C to ECNT-E conversion only possible from R4.1.0 onwards",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INVALID_2_NANT_B'                    => {
	       				     'CODE-NO'  => 1736,
	       				     'MESSAGE'  => "NANT-A to NANT-B conversion only possible from R3.5 onwards",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_INVALID_CONFIG'                      => {
	       				     'CODE-NO'  => 1737,
	       				     'MESSAGE'  => "Invalid IP-aware configuration",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_INVALID_FILE_CONTENT'                => {
	       				     'CODE-NO'  => 1738,
	       				     'MESSAGE'  => "File doesn't contains valid data for status check compare",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_INVALID_TFTP_IP'                     => {
	       				     'CODE-NO'  => 1739,
	       				     'MESSAGE'  => "This UNIX machine is not the file transfer server as configured in the ISAM",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_INV_MIB_VERSION'                     => {
	       				     'CODE-NO'  => 1740,
	       				     'MESSAGE'  => "Unsupported MIB version read",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_INV_NT_RB_TYPE'                      => {
	       				     'CODE-NO'  => 1741,
	       				     'MESSAGE'  => "Invalid VLAN-mode on NT",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_INV_SHUB_RB_TYPE'                    => {
	       				     'CODE-NO'  => 1742,
	       				     'MESSAGE'  => "Invalid VLAN-mode on SHUB",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_ISAMV_OFLMIG_NO_CONF'                => {
	       				     'CODE-NO'  => 1743,
	       				     'MESSAGE'  => "Fake IvPsLiSt.txt file missing: needed for ISAM-V migration with offline DB",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_ISAM_HAS_VOICE'                      => {
	       				     'CODE-NO'  => 1745,
	       				     'MESSAGE'  => "ISAM has voice but no voice migration possible",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_ISAM_NT_SYNC_FAILED'                 => {
	       				     'CODE-NO'  => 1746,
	       				     'MESSAGE'  => "Failed to sync SW to standby NT",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_ISAM_TRANSACT_TAKEN'                 => {
	       				     'CODE-NO'  => 1747,
	       				     'MESSAGE'  => "Config-lock Ownership acquired by others or Release period expired",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_ITFCONFIG_PARSE_ERROR'               => {
	       				     'CODE-NO'  => 1748,
	       				     'MESSAGE'  => "ATM-PVC interface parsing error",
	       				     'USED-IN'  => [ 'PPPC' ],
	                                         },
	       'EC_IVPS_ADD_CDE_FAILED'                 => {
	       				     'CODE-NO'  => 1749,
	       				     'MESSAGE'  => "Adding CDE files for IVPS failed",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_IVPS_DB_FAIL'                        => {
	       				     'CODE-NO'  => 1750,
	       				     'MESSAGE'  => "IVPS DB Failure ",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_IVPS_GETDEP_ERROR'                   => {
	       				     'CODE-NO'  => 1751,
	       				     'MESSAGE'  => "Failed to retrieve xVPS dependancy list",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_IVPS_GETIP_ERROR'                    => {
	       				     'CODE-NO'  => 1752,
	       				     'MESSAGE'  => "Failed to retrieve xVPS IP addresses",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_IVPS_MIG_FAILED'                     => {
	       				     'CODE-NO'  => 1753,
	       				     'MESSAGE'  => "xVPS DB offline migration failure",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_LINEOOS_THR_EXCEEDED'                => {
	       				     'CODE-NO'  => 1754,
	       				     'MESSAGE'  => "Lines OOS threshold exceeded",
	       				     'USED-IN'  => [ 'CHKC' ],
                             'CHCK'     => $IGNORE,
	                                         },
	       'EC_LIST_PARTLY_PROCD'                   => {
	       				     'CODE-NO'  => 36021,  ## DON'T CHANGE, IT HAS HIGH DEPENDECY IN AMS
	       				     'MESSAGE'  => "ONT Type List Partially Processed",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_LIST_SWFILES_FAILED'                 => {
	       				     'CODE-NO'  => 1756,
	       				     'MESSAGE'  => "List SW disk files failed",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_LTOOS_THR_EXCEEDED'                  => {
	       				     'CODE-NO'  => 1757,
	       				     'MESSAGE'  => "LT OOS threshold exceeded",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_LT_NOT_SUPPORTED'                    => {
	       				     'CODE-NO'  => 1758,
	       				     'MESSAGE'  => "No SW support for one or more boards in this SW package",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK' ],
	                                         },
	       'EC_MAC_CONC_ENABLED'                    => {
	       				     'CODE-NO'  => 1759,
	       				     'MESSAGE'  => "Crossconnect Engine already has MAC@ concentration ebaled",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_MAC_LIMITS_EXCEEDED'                 => {
	       				     'CODE-NO'  => 1760,
	       				     'MESSAGE'  => "MAC address limits exceeded",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_MAKE_KEYFILE_FAILED'                 => {
	       				     'CODE-NO'  => 1761,
	       				     'MESSAGE'  => "Creating temporary keyfile failed",
	       				     'USED-IN'  => [ 'PRCK' ],
	                                         },
	       'EC_MAPPING_FILE_FAILED'                 => {
	       				     'CODE-NO'  => 1762,
	       				     'MESSAGE'  => "ONT Mapping File not found or could not be processed",
	       				     'USED-IN'  => [ 'OPEN' ],
	                                         },
	       'EC_MAP_ERR_REC_FAILED'                  => {
	       				     'CODE-NO'  => 1763,
	       				     'MESSAGE'  => "Failed to re-map error record",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'TOSIP' ],
	                                         },
	       'EC_MCASTSRC_PARSE_ERROR'                => {
	       				     'CODE-NO'  => 1764,
	       				     'MESSAGE'  => "MCAST SRC configuration parsing error",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_MIG_NOT_ADVISED'                     => {
	       				     'CODE-NO'  => 1765,
	       				     'MESSAGE'  => "Migration not advised",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'PRCK' ],
	                                         },
	       'EC_MIG_RESTRICTED'                      => {
	       				     'CODE-NO'  => 1766,
	       				     'MESSAGE'  => "Migration restricted",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'PRCK' ],
	                                         },
	       'EC_MISSING_FAILED'                      => {
	       				     'CODE-NO'  => 1767,
	       				     'MESSAGE'  => "Failed to transfer missing files to the ISAM",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_MKDIR_FAILED'                        => {
	       				     'CODE-NO'  => 1768,
	       				     'MESSAGE'  => "Failed to create directory",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NEEDS_MIGRATION'                     => {
	       				     'CODE-NO'  => 1769,
	       				     'MESSAGE'  => "DB version mismatch, migration is needed instead of SW upgrade",
	       				     'USED-IN'  => [ 'UPGR' ],
	                                         },
	       'EC_NEEDS_UPGRADE'                       => {
	       				     'CODE-NO'  => 1770,
	       				     'MESSAGE'  => "DB version equal, SW upgrade is needed instead of migration",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NEW_ALARM'                           => {
	       				     'CODE-NO'  => 1771,
	       				     'MESSAGE'  => "New alarm(s) outstanding not exisiting before migration/upgrade",
	       				     'USED-IN'  => [ 'CHKC' ],
                             'CHCK'     => $IGNORE,
	                                         },
	       'EC_NEW_CDE_NOTFOUND'                    => {
	       				     'CODE-NO'  => 1772,
	       				     'MESSAGE'  => "New CDE file not found",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NEW_DB_MISSING'                      => {
	       				     'CODE-NO'  => 1773,
	       				     'MESSAGE'  => "No converted database found",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_NEW_DB_NOT_ACTIVE'                   => {
	       				     'CODE-NO'  => 1774,
	       				     'MESSAGE'  => "Node did not restart with the converted database",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_NEW_OSWP_FILE_FAILED'                => {
	       				     'CODE-NO'  => 1775,
	       				     'MESSAGE'  => "Failed to generate new OSWP file based on build NTMAP file",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_NEW_SWP_NOT_ENABLED'                 => {
	       				     'CODE-NO'  => 1776,
	       				     'MESSAGE'  => "New OSWP not enabled",
	       				     'USED-IN'  => [ 'UPGR', 'RLBK' ],
	                                         },
	       'EC_NEW_SWP_NOT_PRESENT'                 => {
	       				     'CODE-NO'  => 1777,
	       				     'MESSAGE'  => "New SWP not present",
	       				     'USED-IN'  => [ 'UPGR' ],
	                                         },
	       'EC_NEW_SW_NOT_ACT'                      => {
	       				     'CODE-NO'  => 1778,
	       				     'MESSAGE'  => "Targeted SW is not activated, System rolled backed to old one",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'COMT', 'RLBK' ],
	                                         },
	       'EC_NEW_SW_PRESENT'                      => {
	       				     'CODE-NO'  => 1779,
	       				     'MESSAGE'  => "New SW already present",
	       				     'USED-IN'  => [ 'PRCK' ],
                             'PRCK'     => $IGNORE,
	                                         },
	       'EC_NOT_SUPPORTED'                       => {
	       				     'CODE-NO'  => 1780,
	       				     'MESSAGE'  => "Detected an unsupported hardware configuration",
	       				     'USED-IN'  => [ 'TONANTD', 'TONANTE' ],
	                                         },
	       'EC_NOT_SUPPORTED_LOWER_RELEASE'         => {
	       				     'CODE-NO'  => 1781,
	       				     'MESSAGE'  => "Rollback supported only from R5.1 release",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_NOT_VOICE_SRV'                       => {
	       				     'CODE-NO'  => 1782,
	       				     'MESSAGE'  => "This node is not an ISAM voice Server node",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_NO_ACT_OSWP'                         => {
	       				     'CODE-NO'  => 1783,
	       				     'MESSAGE'  => "No active OSWP found",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_NO_ACT_OSWP_2ACTIVATE'               => {
	       				     'CODE-NO'  => 1784,
	       				     'MESSAGE'  => "No OSWP to activate",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_NO_ACT_SWP'                          => {
	       				     'CODE-NO'  => 1785,
	       				     'MESSAGE'  => "No active SWP found",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_NO_ALT_NT_CAP'                       => {
	       				     'CODE-NO'  => 1786,
	       				     'MESSAGE'  => "Alternative SwMgnt.ini file size not found",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_NO_ASAMCORE_FOUND'                   => {
	       				     'CODE-NO'  => 1787,
	       				     'MESSAGE'  => "Failed to find ASAM-CORE line in uploaded OSWP file",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_NO_BOARDS'                           => {
	       				     'CODE-NO'  => 1788,
	       				     'MESSAGE'  => "No boards found in planned boards returned by NT",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_NO_FREE_SPACE'                       => {
	       				     'CODE-NO'  => 1789,
	       				     'MESSAGE'  => "Free space could not be read",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_NO_INPUT_DIR'                        => {
	       				     'CODE-NO'  => 1790,
	       				     'MESSAGE'  => "Input directory containing expected check results not existing",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_NO_LT_SWP_FOUND'                     => {
	       				     'CODE-NO'  => 1791,
	       				     'MESSAGE'  => "ISAM board SWP missing",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_NO_MIB_VERSION'                      => {
	       				     'CODE-NO'  => 1792,
	       				     'MESSAGE'  => "Could not determine NE MIB version",
	       				     'USED-IN'  => [ 'SWRD', 'SWMM' ],
	                                         },
	       'EC_NO_NT_CAP'                           => {
	       				     'CODE-NO'  => 1793,
	       				     'MESSAGE'  => "SwMgnt.ini file size not found",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_NO_PASS_SWPIDX_FOUND'                => {
	       				     'CODE-NO'  => 1794,
	       				     'MESSAGE'  => "Failed to determine passive OSWP index",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_NO_SHUB_SW'                          => {
	       				     'CODE-NO'  => 1796,
	       				     'MESSAGE'  => "Unable to find SHUB SW associated with this OSWP",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_NO_SNMP_VERSION_SEL'                 => {
	       				     'CODE-NO'  => 1797,
	       				     'MESSAGE'  => "No SNMP version configured in param.cfg",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'TONANTD', 'ONT', 'TONANTE' ],
	                                         },
	       'EC_NO_SSH_POSS'                         => {
	       				     'CODE-NO'  => 1798,
	       				     'MESSAGE'  => "No SSH access possible",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_NO_SSH_POSSIBLE'                     => {
	       				     'CODE-NO'  => 1799,
	       				     'MESSAGE'  => "No SSH possible",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_NO_STATUS_FILE'                      => {
	       				     'CODE-NO'  => 1800,
	       				     'MESSAGE'  => "No status result file found",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_NO_SUCH_BRIDGEPORT'                  => {
	       				     'CODE-NO'  => 1801,
	       				     'MESSAGE'  => "No such bridge port",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_NO_SUCH_CLIENTPORT'                  => {
	       				     'CODE-NO'  => 1802,
	       				     'MESSAGE'  => "No such PPPoX-relay CC client port",
	       				     'USED-IN'  => [ 'CCCE', 'PPPC' ],
	                                         },
	       'EC_NO_SUCH_ENGINE'                      => {
	       				     'CODE-NO'  => 1803,
	       				     'MESSAGE'  => "No such PPPoX-relay CC engine",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_NO_SUCH_IPVLAN'                      => {
	       				     'CODE-NO'  => 1804,
	       				     'MESSAGE'  => "IP VLAN not defined",
	       				     'USED-IN'  => [ 'PPPC' ],
	                                         },
	       'EC_NO_SUCH_NT_VLAN'                     => {
	       				     'CODE-NO'  => 1805,
	       				     'MESSAGE'  => "VLAN does not exist in ISAM (NT)",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_NO_SUCH_PPPVLAN'                     => {
	       				     'CODE-NO'  => 1806,
	       				     'MESSAGE'  => "PPP VLAN not defined",
	       				     'USED-IN'  => [ 'PPPC' ],
	                                         },
	       'EC_NO_SUCH_SHUB_VLAN'                   => {
	       				     'CODE-NO'  => 1807,
	       				     'MESSAGE'  => "VLAN does not exist in ISAM (SHUB)",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_NO_SUCH_VLAN'                        => {
	       				     'CODE-NO'  => 1808,
	       				     'MESSAGE'  => "No such VLAN ID",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_NO_SUCH_VLAN_PP'                     => {
	       				     'CODE-NO'  => 1809,
	       				     'MESSAGE'  => "No such VLAN port-protocol",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_NO_SUCH_VRF'                         => {
	       				     'CODE-NO'  => 1810,
	       				     'MESSAGE'  => "No such VRF ID",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_NO_TTL0_FWD'                         => {
	       				     'CODE-NO'  => 1811,
	       				     'MESSAGE'  => "TTL=0 packet forwarding only possible from R3.5 onwards",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NTTYPE_MAPFILE_FAILED'               => {
	       				     'CODE-NO'  => 1812,
	       				     'MESSAGE'  => "Failed to determine SW Descriptor from build NTMAP file",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'PRCK', 'SWMM' ],
	                                         },
	       'EC_NTTYPE_OSWP_FAILED'                  => {
	       				     'CODE-NO'  => 1813,
	       				     'MESSAGE'  => "OSWP filename mistmatch found based on build NTMAP file",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK', 'SWMM' ],
	                                         },
	       'EC_NTTYPE_SWPNAME_FAILED'               => {
	       				     'CODE-NO'  => 1814,
	       				     'MESSAGE'  => "Failed to fetch SWP file based on build NT-SWP Map file",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK', 'SWMM' ],
	                                         },
	       'EC_NT_DB_BU_FAIL'                       => {
	       				     'CODE-NO'  => 1815,
	       				     'MESSAGE'  => "DB backup failed",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'TONANTD', 'TOSIP', 'TONANTE' ],
	                                         },
	       'EC_NT_DB_COPY_FAILED'                   => {
	       				     'CODE-NO'  => 1816,
	       				     'MESSAGE'  => "Copy migrated NT DB failed",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_DB_CVT_FAIL'                      => {
	       				     'CODE-NO'  => 1817,
	       				     'MESSAGE'  => "NT DB conversion failed",
	       				     'USED-IN'  => [ 'TONANTD', 'TOSIP', 'TONANTE' ],
	                                         },
	       'EC_NT_DB_EMPTY'                         => {
	       				     'CODE-NO'  => 1818,
	       				     'MESSAGE'  => "NT DB backup failed (size 0)",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_DB_NOT_PRESENT'                   => {
	       				     'CODE-NO'  => 1819,
	       				     'MESSAGE'  => "NT DB not present",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_ICS_ISSUE'                        => {
	       				     'CODE-NO'  => 1820,
	       				     'MESSAGE'  => "NT ICS version too low",
	       				     'USED-IN'  => [ 'PRCK' ],
                             'PRCK'     => $IGNORE,
	                                         },
	       'EC_NT_MIG_FAILED'                       => {
	       				     'CODE-NO'  => 1821,
	       				     'MESSAGE'  => "NT DB offline migration failure",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_NOT_ECNT_A'                       => {
	       				     'CODE-NO'  => 1822,
	       				     'MESSAGE'  => "Source NT not ECNT-A",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_NOT_ECNT_A_OR_C'                  => {
	       				     'CODE-NO'  => 1823,
	       				     'MESSAGE'  => "Source NT should be ECNT-A or ECNT-C",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_NOT_INSYNC'                       => {
	       				     'CODE-NO'  => 1824,
	       				     'MESSAGE'  => "NT not in sync",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_NT_NOT_IN_SWP'                       => {
	       				     'CODE-NO'  => 1825,
	       				     'MESSAGE'  => "NT could not be found in software package",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK' ],
	                                         },
	       'EC_NT_NOT_NANT_A'                       => {
	       				     'CODE-NO'  => 1826,
	       				     'MESSAGE'  => "Source NT not NANT-A",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_NOT_SUPPORTED'                    => {
	       				     'CODE-NO'  => 1827,
	       				     'MESSAGE'  => "NT type not supported by toolset",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'RLBK' ],
	                                         },
	       'EC_NT_SHUB_SW_MM'                       => {
	       				     'CODE-NO'  => 1828,
	       				     'MESSAGE'  => "SW Version mismatch between NT and SHUB",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_NT_TYPE_UNKNOWN'                     => {
	       				     'CODE-NO'  => 1829,
	       				     'MESSAGE'  => "Unsupported NT type",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'PRCK' ],
	                                         },
	       'EC_NT_VLAN_PARSE_ERROR'                 => {
	       				     'CODE-NO'  => 1830,
	       				     'MESSAGE'  => "NT VLAN configuration parsing error",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_OFLDB_NOTREQ'                        => {
	       				     'CODE-NO'  => 1834,
	       				     'MESSAGE'  => "DB Migration is not required, Since both DB Versions are same",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_ONTOOS_THR_EXCEEDED'                 => {
	       				     'CODE-NO'  => 1835,
	       				     'MESSAGE'  => "ONT OOS after migration",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_ONT_RELMAP_MISSING'                  => {
	       				     'CODE-NO'  => 1836,
	       				     'MESSAGE'  => "ONT Release mapping file not found in the location specified",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_OPENCLI_FAILED'                      => {
	       				     'CODE-NO'  => 1837,
	       				     'MESSAGE'  => "Unable to establish CLI session from ISAM",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'UPGR', 'COMT', 'PRCK', 'SCHK', 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'TONANTD', 'TOSIP', 'IPINV', 'TONANTE', 'RLBK' ],
	                                         },
	       'EC_OPENFILE_FAILED'                     => {
	       				     'CODE-NO'  => 1838,
	       				     'MESSAGE'  => "Failed to open file",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_OPENTL1_FAILED'                      => {
	       				     'CODE-NO'  => 1839,
	       				     'MESSAGE'  => "Could not open TL1",
	       				     'USED-IN'  => [ 'SWDL', 'SCHK', 'ONT' ],
	                                         },
	       'EC_OPEN_OSWP_FAIL'                      => {
	       				     'CODE-NO'  => 1840,
	       				     'MESSAGE'  => "Failed to open OSWP file",
	       				     'USED-IN'  => [ 'SWRD', 'SWMM' ],
	                                         },
	       'EC_OPEN_OSWP_NEW_FAIL'                  => {
	       				     'CODE-NO'  => 1841,
	       				     'MESSAGE'  => "Failed to open OSWP file",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_OPEN_POST_CHECK_FAILED'              => {
	       				     'CODE-NO'  => 1842,
	       				     'MESSAGE'  => "Failed to store post-check results",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_OPEN_PRE_CHECK_FAILED'               => {
	       				     'CODE-NO'  => 1843,
	       				     'MESSAGE'  => "Failed to store pre-check results",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_OPEN_SWP_FAIL'                       => {
	       				     'CODE-NO'  => 1844,
	       				     'MESSAGE'  => "Failed to open SWP file",
	       				     'USED-IN'  => [ 'SWRD', 'SWMM' ],
	                                         },
	       'EC_OPEN_WARNFILE_FAILED'                => {
	       				     'CODE-NO'  => 1845,
	       				     'MESSAGE'  => "Could not open warnings file",
	       				     'USED-IN'  => [ 'TONANTD', 'TONANTE' ],
	                                         },
	       'EC_OPEN_WARN_FILE_FAILED'               => {
	       				     'CODE-NO'  => 1846,
	       				     'MESSAGE'  => "Failed to read warning level file",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_OSWP_ACT_FAILED'                     => {
	       				     'CODE-NO'  => 1847,
	       				     'MESSAGE'  => "Failed to activate (target) OSWP",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'TOSIP', 'RLBK' ],
	                                         },
	       'EC_OSWP_FILE_NO_IP'                     => {
	       				     'CODE-NO'  => 1848,
	       				     'MESSAGE'  => "No tftp IP address found in OSWP file",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_OSWP_MISSING'                        => {
	       				     'CODE-NO'  => 1849,
	       				     'MESSAGE'  => "OSWP is not specified",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_OSWP_OPER_FAIL'                      => {
	       				     'CODE-NO'  => 1850,
	       				     'MESSAGE'  => "Reading OSWP operational state failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_PARAM_CFG_NOT_OK'                    => {
	       				     'CODE-NO'  => 1851,
	       				     'MESSAGE'  => "param.cfg contents not correct on the regular expression part",
	       				     'USED-IN'  => [ 'SWMM' ],
	                                         },
	       'EC_PARSE_SIP_SERV_FAILED'               => {
	       				     'CODE-NO'  => 1852,
	       				     'MESSAGE'  => "Parsing the SIP service configuration file failed",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_PARSE_SIP_SITE_FAILED'               => {
	       				     'CODE-NO'  => 1853,
	       				     'MESSAGE'  => "Parsing the SIP site configuration file failed",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_PARSE_SWP_FAILED'                    => {
	       				     'CODE-NO'  => 1854,
	       				     'MESSAGE'  => "Failed to parse the SWP file",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_PAR_SIZE_TOO_SMALL'                  => {
	       				     'CODE-NO'  => 1855,
	       				     'MESSAGE'  => "Board not fitted to accomodate this release, please upgrade NT HW",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK' ],
	                                         },
	       'EC_PASSIVE_DB_FAILED'                   => {
	       				     'CODE-NO'  => 1856,
	       				     'MESSAGE'  => "Passive DB failed",
	       				     'USED-IN'  => [ 'RLBK', 'RLBK' ],
	                                         },
	       'EC_PASSIVE_OSWP_EMPTY'                  => {
	       				     'CODE-NO'  => 1857,
	       				     'MESSAGE'  => "Passive OSWP is empty",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_PASSIVE_SWP_NOT_COMMITTED'           => {
	       				     'CODE-NO'  => 1858,
	       				     'MESSAGE'  => "Passive OSWP is not committed version",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_PASSIVE_SWP_NOT_ENABLED'             => {
	       				     'CODE-NO'  => 1859,
	       				     'MESSAGE'  => "Passive OSWP is not enabled",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_PING_NO_RESP'                        => {
	       				     'CODE-NO'  => 1860,
	       				     'MESSAGE'  => "Ping No response",
	       				     'USED-IN'  => [ 'ONT', 'RLBK' ],
	                                         },
	       'EC_POST_UPGR_ERROR'                     => {
	       				     'CODE-NO'  => 1861,
	       				     'MESSAGE'  => "Posti-Action script error",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR' ],
	                                         },
	       'EC_PPPOESESS_PARSE_ERROR'               => {
	       				     'CODE-NO'  => 1862,
	       				     'MESSAGE'  => "pppoe session info parsing error",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_PRECHECK_NOTRUN'                     => {
	       				     'CODE-NO'  => 1863,
	       				     'MESSAGE'  => "The restored DB copy not available",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_PREPARE_CREATE_ERROR'                => {
	       				     'CODE-NO'  => 1864,
	       				     'MESSAGE'  => "Creation preparation failed",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_PREP_RECONF_FAIL'                    => {
	       				     'CODE-NO'  => 1865,
	       				     'MESSAGE'  => "Failed to generate SHUB post-reconfiguration file",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_PRE_UPGR_ERROR'                      => {
	       				     'CODE-NO'  => 1866,
	       				     'MESSAGE'  => "Pre-Action script error",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR' ],
	                                         },
	       'EC_PRNT_NOMORE_FAILED'                  => {
	       				     'CODE-NO'  => 1867,
	       				     'MESSAGE'  => "Disabling interactive mode failed",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_PROMPT_FAILED'                       => {
	       				     'CODE-NO'  => 1868,
	       				     'MESSAGE'  => "Changing environment prompt failed",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_PVCCONFIG_PARSE_ERROR'               => {
	       				     'CODE-NO'  => 1869,
	       				     'MESSAGE'  => "ATM-PVC configuration parsing error",
	       				     'USED-IN'  => [ 'PPPC' ],
	                                         },
	       'EC_QOS_PARSE_ERROR'                     => {
	       				     'CODE-NO'  => 1870,
	       				     'MESSAGE'  => "qos session profile configuration parsing error",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_RB_FAILED'                           => {
	       				     'CODE-NO'  => 1871,
	       				     'MESSAGE'  => "Rolling back to H.248 configuration failed",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_REDUCE_OPR_NOTALLOW'                 => {
	       				     'CODE-NO'  => 1872,
	       				     'MESSAGE'  => "Reduce download operation suppressed for this platform",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_RELEASE_TOO_LOW'                     => {
	       				     'CODE-NO'  => 1873,
	       				     'MESSAGE'  => "H.248 to SIP can only be done for R4.1 or higher",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV', 'TOSIP' ],
	                                         },
	       'EC_REL_CVT_FAILED'                      => {
	       				     'CODE-NO'  => 1875,
	       				     'MESSAGE'  => "Failed to convert 1540 release number to 7302 release number",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_RESET_NT_FAILED'                     => {
	       				     'CODE-NO'  => 1876,
	       				     'MESSAGE'  => "Resetting NT using T&D failed",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'TOSIP' ],
	                                         },
	       'EC_RESET_TRANSACTION_FAILED'            => {
	       				     'CODE-NO'  => 1877,
	       				     'MESSAGE'  => "Failed to reset transaction owner",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_RESTART_FAILED'                      => {
	       				     'CODE-NO'  => 1878,
	       				     'MESSAGE'  => "ISAM restart failed and lost connectivity to the node",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'TOSIP' ],
	                                         },
	       'EC_ROLLBACK_ERROR'                      => {
	       				     'CODE-NO'  => 1879,
	       				     'MESSAGE'  => "Rollback of configuration failed, please restore backup",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_RTRV_ONT_EMPTY'                      => {
	       				     'CODE-NO'  => 1880,
	       				     'MESSAGE'  => "No ONT provisioned on given OLT",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_RTRV_ONT_FAILED'                     => {
	       				     'CODE-NO'  => 1881,
	       				     'MESSAGE'  => "Failed to retrieve ONT configuration",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_RTRV_REL_FAILED'                     => {
	       				     'CODE-NO'  => 1882,
	       				     'MESSAGE'  => "Failed to retrieve Release information",
	       				     'USED-IN'  => [ 'SCHK', 'CHKC' ],
	                                         },
	       'EC_RTRV_REL_INFO_FAILED'                => {
	       				     'CODE-NO'  => 1883,
	       				     'MESSAGE'  => "Failed to retrieve information from ONT_Release_Mapping.txt",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_SEC_SSH_ENABLE_WRITE_FAIL'           => {
	       				     'CODE-NO'  => 1884,
	       				     'MESSAGE'  => "Executing 'Secure SSH' enable write failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_SEC_SSH_REMOVE_SWPKG_FAIL'           => {
	       				     'CODE-NO'  => 1885,
	       				     'MESSAGE'  => "Executing 'Secure SSH' remove SW Pkg failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_SETUP_ISSU_FAILED'                   => {
	       				     'CODE-NO'  => 1886,
	       				     'MESSAGE'  => "ISSU setting/setup is not properly done",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'PRCK', 'RLBK' ],
	                                         },
	       'EC_SET_COMMIT_FAIL'                     => {
	       				     'CODE-NO'  => 1887,
	       				     'MESSAGE'  => "Commit OSWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_SET_OSWP_FAIL'                       => {
	       				     'CODE-NO'  => 1888,
	       				     'MESSAGE'  => "Setting new active OSWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_SET_SESSION_TO_FAILED'               => {
	       				     'CODE-NO'  => 1889,
	       				     'MESSAGE'  => "Failed to set session timeout value",
	       				     'USED-IN'  => [ 'SCHK' ],
	                                         },
	       'EC_SET_SFP_ASSIGNMENT_FAILED'           => {
	       				     'CODE-NO'  => 1890,
	       				     'MESSAGE'  => "Failed to configure SFP Assignments",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SET_TIMEOUT_FAILED'                  => {
	       				     'CODE-NO'  => 1891,
	       				     'MESSAGE'  => "Failed to set CLI session timeout",
	       				     'USED-IN'  => [ 'IBRG', 'CCCE', 'PPPC', 'CNWV' ],
	                                         },
	       'EC_SET_TRANSACTION_FAILED'              => {
	       				     'CODE-NO'  => 1892,
	       				     'MESSAGE'  => "Failed to set transaction owner",
	       				     'USED-IN'  => [ 'MIGR', 'UPGR', 'SWRD', 'TOSIP', 'RLBK' ],
	                                         },
	       'EC_SET_TRANS_TO_FAILED'                 => {
	       				     'CODE-NO'  => 1893,
	       				     'MESSAGE'  => "Failed to set transaction timeout",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHOW_EQPT_FAIL'                      => {
	       				     'CODE-NO'  => 1894,
	       				     'MESSAGE'  => "Failed to retrieve equipment data",
	       				     'USED-IN'  => [ 'TONANTD', 'TONANTE' ],
	                                         },
	       'EC_SHOW_EQPT_FAILED'                    => {
	       				     'CODE-NO'  => 1895,
	       				     'MESSAGE'  => "Failed to retrieve equipment data",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK', 'TOSIP' ],
	                                         },
	       'EC_SHOW_PPPOESESS_ERROR'                => {
	       				     'CODE-NO'  => 1896,
	       				     'MESSAGE'  => "Could not retrieve pppoe session info",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_SHUB_BU_FAIL'                        => {
	       				     'CODE-NO'  => 1897,
	       				     'MESSAGE'  => "SHUB backup failed",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_DB_DL_FAILED'                   => {
	       				     'CODE-NO'  => 1898,
	       				     'MESSAGE'  => "Failed to download SHUB database",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_DB_SAVE_FAIL'                   => {
	       				     'CODE-NO'  => 1899,
	       				     'MESSAGE'  => "SHUB DB save failed",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_MIG_FAILED'                     => {
	       				     'CODE-NO'  => 1900,
	       				     'MESSAGE'  => "SHUB DB offline migration failure",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_NO_TFTP_BOOT'                   => {
	       				     'CODE-NO'  => 1901,
	       				     'MESSAGE'  => "SHUB not booting from tftp",
	       				     'USED-IN'  => [ 'SWDL', 'PRCK' ],
	                                         },
	       'EC_SHUB_PARSE_FAIL'                     => {
	       				     'CODE-NO'  => 1902,
	       				     'MESSAGE'  => "Parsing SHUB CLI configuration failed",
	       				     'USED-IN'  => [ 'TONANTD', 'TONANTE' ],
	                                         },
	       'EC_SHUB_REBOOT_FAILED'                  => {
	       				     'CODE-NO'  => 1903,
	       				     'MESSAGE'  => "Failed to reboot SHUB from tftp config settings",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_RECONFIG_MISSING'               => {
	       				     'CODE-NO'  => 1904,
	       				     'MESSAGE'  => "SHUB post-reconfiguration file missing",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_SHUB_SET_NEW_CONF_FAILED'            => {
	       				     'CODE-NO'  => 1905,
	       				     'MESSAGE'  => "Failed to configure SHUB for other config",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_SW_DL_FAILED'                   => {
	       				     'CODE-NO'  => 1906,
	       				     'MESSAGE'  => "Downloading SHUB SW failed",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_SHUB_SW_NOT_FOUND'                   => {
	       				     'CODE-NO'  => 1907,
	       				     'MESSAGE'  => "SHUB SW not found in Sw directory",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SHUB_VLAN_PARSE_ERROR'               => {
	       				     'CODE-NO'  => 1908,
	       				     'MESSAGE'  => "SHUB VLAN configuration parsing error",
	       				     'USED-IN'  => [ 'IBRG', 'CNWV' ],
	                                         },
	       'EC_SIP_TERM_ERROR'                      => {
	       				     'CODE-NO'  => 1909,
	       				     'MESSAGE'  => "Failed to obtain SIP termination info",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_SIP_TERM_MISSING'                    => {
	       				     'CODE-NO'  => 1910,
	       				     'MESSAGE'  => "Number of SIP terminations present in the node does not equal the amount of generated terminations",
	       				     'USED-IN'  => [ 'TOSIP' ],
	                                         },
	       'EC_STAGE_GET_DISK_SPACE_FAILED'         => {
	       				     'CODE-NO'  => 1918,
	       				     'MESSAGE'  => "Failed to retrieve the free disk space of OLT",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_GET_ONTSWVER_FAILED'           => {
	       				     'CODE-NO'  => 1919,
	       				     'MESSAGE'  => "Failed to determine ont software loads present on NE",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_PUT_FAILED'                    => {
	       				     'CODE-NO'  => 1920,
	       				     'MESSAGE'  => "Software PUT Operation failed",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_TELNET_CLOSE_FAILED'           => {
	       				     'CODE-NO'  => 1921,
	       				     'MESSAGE'  => "Failed to Close Telnet",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_TELNET_INVALID_OPTIONS'        => {
	       				     'CODE-NO'  => 1922,
	       				     'MESSAGE'  => "Failed due to Invalid telnet options",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_TELNET_OPEN_FAILED'            => {
	       				     'CODE-NO'  => 1923,
	       				     'MESSAGE'  => "Failed to Open Telnet",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_TYPE_FILE_MAPPING_FAILED'      => {
	       				     'CODE-NO'  => 1924,
	       				     'MESSAGE'  => "Failed to retrieve ONT information from the Release Mapping File.",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_STAGE_WARN_EXCEEDS_THRESHOLD'        => {
	       				     'CODE-NO'  => 1925,
	       				     'MESSAGE'  => "Software Staging Failures exceeds The Warning Threshold",
	       				     'USED-IN'  => [ 'ONT' ],
	                                         },
	       'EC_SWPFILE_NOT_SUPPORTED'               => {
	       				     'CODE-NO'  => 1927,
	       				     'MESSAGE'  => "Option --swpFile not supported for HiCap systems",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_SW_CHECKS_NOK'                       => {
	       				     'CODE-NO'  => 1928,
	       				     'MESSAGE'  => "SW pre-check failure",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_SW_MGNT_SIZE_NOK'                    => {
	       				     'CODE-NO'  => 1929,
	       				     'MESSAGE'  => "SwMgnt.ini size not OK",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_SW_NAME_ERROR'                       => {
	       				     'CODE-NO'  => 1930,
	       				     'MESSAGE'  => "SWP name not conforming to regular expression (should not occur)",
	       				     'USED-IN'  => [ 'UPGR' ],
	                                         },
	       'EC_SW_RED_FAILED'                       => {
	       				     'CODE-NO'  => 1931,
	       				     'MESSAGE'  => "SW reduction failed",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_SYNC_TO_STBY_FAILED'                 => {
	       				     'CODE-NO'  => 1932,
	       				     'MESSAGE'  => "Failed to sync SW to standby NT",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_TARGET_DIFF_FROM_ACTIVE_OSWP'        => {
	       				     'CODE-NO'  => 1933,
	       				     'MESSAGE'  => "Target SWP is different from Active OSWP",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_TARGET_DIFF_FROM_PASSIVE_OSWP'       => {
	       				     'CODE-NO'  => 1934,
	       				     'MESSAGE'  => "Target SWP is different from Passive OSWP",
	       				     'USED-IN'  => [ 'RLBK' ],
	                                         },
	       'EC_TFTP_MISSING_FAILED'                 => {
	       				     'CODE-NO'  => 1935,
	       				     'MESSAGE'  => "Failed to tftp missing files to the ISAM",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_TFTP_OPEN_FAIL'                      => {
	       				     'CODE-NO'  => 1936,
	       				     'MESSAGE'  => "Opening TFTP session failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_TFTP_OPEN_FAILED'                    => {
	       				     'CODE-NO'  => 1937,
	       				     'MESSAGE'  => "Failed to open tftp session",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_TFTP_PUT_FAILED'                     => {
	       				     'CODE-NO'  => 1938,
	       				     'MESSAGE'  => "Failed to transfer file via tftp",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_TFTP_SUSPEND_FAILED'                 => {
	       				     'CODE-NO'  => 1939,
	       				     'MESSAGE'  => "Failed to suspend NT tftp task",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_TRANSFER_WAITTIME_EXCEED'            => {
	       				     'CODE-NO'  => 1940,
	       				     'MESSAGE'  => "Wait time exceeded to complete active file transfer",
	       				     'USED-IN'  => [ 'SWDL', 'MIGR', 'TOSIP' ],
	                                         },
	       'EC_THRESHOLD_WARN'                      => {
	       				     'CODE-NO'  => 1941,
	       				     'MESSAGE'  => "Differences greater than zero and less than or equal to The Warning Threshold",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_THRESHOLD_WARN_EXCEED'               => {
	       				     'CODE-NO'  => 1942,
	       				     'MESSAGE'  => "Differences exceeds The Warning Threshold",
	       				     'USED-IN'  => [ 'CHKC' ],
	                                         },
	       'EC_TOO_MANY_MC_SRC_PKG'                 => {
	       				     'CODE-NO'  => 1943,
	       				     'MESSAGE'  => "Too many multicast source packages",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_TO_NTVLAN_EXISTS'                    => {
	       				     'CODE-NO'  => 1944,
	       				     'MESSAGE'  => "To-VLAN exists on NT",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_TO_SHUBVLAN_EXISTS'                  => {
	       				     'CODE-NO'  => 1945,
	       				     'MESSAGE'  => "To-VLAN exists on SHUB",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_TRX_PROTOCOL_MISMATCH'               => {
	       				     'CODE-NO'  => 1948,
	       				     'MESSAGE'  => "File Transfer Protocol Mismatch",
	       				     'USED-IN'  => [ 'FILE' ],
	                                         },
	       'EC_TTL_0_FWD_FAILED'                    => {
	       				     'CODE-NO'  => 1949,
	       				     'MESSAGE'  => "Failed to enable TTL=0 packet forwarding",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_UNKNOWN_NE_FAM'                      => {
	       				     'CODE-NO'  => 1950,
	       				     'MESSAGE'  => "Unknown NE family",
	       				     'USED-IN'  => [ 'MIGR' ],
	                                         },
	       'EC_UPGR_NOT_ADVISED'                    => {
	       				     'CODE-NO'  => 1951,
	       				     'MESSAGE'  => "Upgrade not advised",
	       				     'USED-IN'  => [ 'SWDL', 'UPGR', 'PRCK' ],
	                                         },
	       'EC_UPGR_RESTRICTED'                     => {
	       				     'CODE-NO'  => 1952,
	       				     'MESSAGE'  => "Upgrade restricted",
	       				     'USED-IN'  => [ 'SWDL', 'UPGR', 'PRCK' ],
	                                         },
	       'EC_UPLOAD_DB_FAILED'                    => {
	       				     'CODE-NO'  => 1953,
	       				     'MESSAGE'  => "Failed to upload active DB in RestoreDB directory",
	       				     'USED-IN'  => [ 'PRCK' ],
	                                         },
	       'EC_USED_CCPORT'                         => {
	       				     'CODE-NO'  => 1954,
	       				     'MESSAGE'  => "From-VLAN used in cross-connect client ports",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_USED_NT_VRF'                         => {
	       				     'CODE-NO'  => 1955,
	       				     'MESSAGE'  => "From-VLAN used in VRF on NT",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_USED_SHUBPVID'                       => {
	       				     'CODE-NO'  => 1956,
	       				     'MESSAGE'  => "Network VLAN used as PVID on SHUB bridgeport",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_USED_SHUB_ROUTEDEST'                 => {
	       				     'CODE-NO'  => 1957,
	       				     'MESSAGE'  => "From-VLAN used in route-destinations on SHUB",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_USED_VLANPP'                         => {
	       				     'CODE-NO'  => 1958,
	       				     'MESSAGE'  => "From-VLAN used in VLAN port-protocol configuration",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_VLANBIND_PARSE_ERROR'                => {
	       				     'CODE-NO'  => 1959,
	       				     'MESSAGE'  => "vlan-bind configuration parsing error",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_VLANPORT_PARSE_ERROR'                => {
	       				     'CODE-NO'  => 1960,
	       				     'MESSAGE'  => "VLAN port interface parsing error",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_VLAN_PARSE_ERROR'                    => {
	       				     'CODE-NO'  => 1961,
	       				     'MESSAGE'  => "NT VLAN configuration parsing error",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_VLAN_PP_PARSE_ERROR'                 => {
	       				     'CODE-NO'  => 1962,
	       				     'MESSAGE'  => "VLAN port-protocol configuration parsing error",
	       				     'USED-IN'  => [ 'CCCE' ],
	                                         },
	       'EC_VLAN_USED'                           => {
	       				     'CODE-NO'  => 1963,
	       				     'MESSAGE'  => "From VLAN used in SHUB filters",
	       				     'USED-IN'  => [ 'CNWV' ],
	                                         },
	       'EC_VMAC_HOST_ID_LIMITS_EXCEED'          => {
	       				     'CODE-NO'  => 1964,
	       				     'MESSAGE'  => "VMAC HOSTID limits exceeded",
	       				     'USED-IN'  => [ 'MIGR', 'PRCK' ],
	                                         },
	       'EC_VRF_PARSE_ERROR'                     => {
	       				     'CODE-NO'  => 1965,
	       				     'MESSAGE'  => "VRF configuration parsing error",
	       				     'USED-IN'  => [ 'IBRG' ],
	                                         },
	       'EC_WRONG_VARIANT'                       => {
	       				     'CODE-NO'  => 1966,
	       				     'MESSAGE'  => "Incorrect OSWP variant for this NE family",
	       				     'USED-IN'  => [ 'SWDL' ],
	                                         },
	       'EC_XFTP_GET_OSWP_FAIL'                  => {
	       				     'CODE-NO'  => 1967,
	       				     'MESSAGE'  => "Get OSWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_XFTP_GET_SWP_FAIL'                   => {
	       				     'CODE-NO'  => 1968,
	       				     'MESSAGE'  => "Get SWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_XFTP_PUT_OSWP_FAIL'                  => {
	       				     'CODE-NO'  => 1969,
	       				     'MESSAGE'  => "Put OSWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },
	       'EC_XFTP_PUT_SWP_FAIL'                   => {
	       				     'CODE-NO'  => 1970,
	       				     'MESSAGE'  => "Put SWP failed",
	       				     'USED-IN'  => [ 'SWRD' ],
	                                         },

	      );

my %errorCategories = (
		       1000 => "SWDL",
		       2000 => "MIGR",
		       3000 => "UPGR",
		       4000 => "COMT",
		       5000 => "PRCK",
		       6000 => "SCHK",
		       7000 => "CHKC",
		       10000 => "SWRD",
		       20000 => "SWMM",
		       30000 => "IBRG",
		       31000 => "CCCE",
		       32000 => "PPPC",
		       33000 => "CNWV",
		       34000 => "2NANTD",
		       35000 => "2SIP",
		       36000 => "ONT",
		       37000 => "2NANTE",
		       38000 => "RLBK");

my %errorRanges = (
                   "SWDL" => [1000, 1999],
                   "MIGR" => [2000, 2999],
                   "UPGR" => [3000, 3999],
                   "COMT" => [4000, 4999],
                   "PRCK" => [5000, 5999],
                   "SCHK" => [6000, 6999],
                   "CHKC" => [7000, 7999],
                   "SWRD" => [10000, 10999],
                   "SWMM" => [20000, 20999],
		   "IBRG" => [30000, 30999],
		   "CCCE" => [31000, 31999],
		   "PPPC" => [32000, 32999],
		   "CNWV" => [33000, 33999],
		   "2NANTD" => [34000, 34999],
		   "2SIP" => [35000, 35999],
		   "ONT" => [36000, 36999],
		   "2NANTE" => [37000, 37999],
		   "RLBK" => [38000, 38999]);
=head1 FUNCTIONS

=head2 GetVersion()

=begin html
Return the version of this PERL module.

=end html

=cut

sub GetVersion() {
  return $VERSION;
}

=head2 WarningLevelString($)

=begin html
Return the string associated to $level (ignore/warning/error).

=end html

=cut

sub WarningLevelString($) {
  my ($level) = @_;

  return (defined($warningLevelStrings{$level}) ? $warningLevelStrings{$level} : "Unknown ($level)");
}

=head2 WarningLevelIsError($)

=begin html
Returns true in case the warning level associated to $level is set to error.

=end html

=cut

sub WarningLevelIsError($) {
  my ($level) = @_;

  return $level == $ERROR ? 1 : 0;
}

=head2 WarningLevelIsWarning($)

=begin html
Returns true in case the warning level associated to $level is set to warning.

=end html

=cut

sub WarningLevelIsWarning($) {
  my ($level) = @_;

  return $level == $WARNING ? 1 : 0;
}

=head2 WarningLevelIsIgnore($)

=begin html
Returns true in case the warning level associated to $level is set to ignore.

=end html

=cut

sub WarningLevelIsIgnore($) {
  my ($level) = @_;

  return $level == $IGNORE ? 1 : 0;
}


=head2 ExitString($)

=begin html
Return the string associated to exitcode $code.

=end html

=cut

sub ExitString($) {
  my ($code) = @_;
  foreach my $key (keys %exitCodes) {
    return sprintf("%d: %s", $code, $exitCodes{$key}[1]) if ($exitCodes{$key}[0] == $code);
  }
  return "unknown exit: $code";
}

=head2 CvtErrorString($)

=begin html
Convert the errorcode $code into the associated error category (PBMT tool) and the associated error message.

=end html

=cut

sub CvtErrorString($) {
  my ($code) = @_;
  my $cat = int ($code / 1000) * 1000;
  foreach my $key (sort keys %errorCodes) {
    if( defined $errorCodes{$key} and defined $errorCodes{$key}{'CODE-NO'} and ($errorCodes{$key}{'CODE-NO'} == $code) ) {
        return sprintf("%d: %s", $code, $errorCodes{$key}{'MESSAGE'});
    }
  }
  return "unknown error code: $code";
}

=head2 ErrorString($)

=begin html
Return the string associated to exitcode $code.

=end html

=cut

sub ErrorString($) {
  my ($code) = @_;
  my $cat = int ($code / 1000) * 1000;

  foreach my $key (sort keys %errorCodes) {
    if( defined $errorCodes{$key} and defined $errorCodes{$key}{'CODE-NO'} and ($errorCodes{$key}{'CODE-NO'} == $code) ) {
        return sprintf("%d: %s", $code, $errorCodes{$key}{'MESSAGE'});
    }
  }
  return "unknown error code: $code";
}

sub IsErrorUsedIIn($$){
    my($code,$usedIn) = @_;
    my $matched = 0;
    foreach my $key (sort keys %errorCodes) {
       if( defined $errorCodes{$key} and  defined $errorCodes{$key}{'CODE-NO'} and $errorCodes{$key}{'CODE-NO'} == $code ) {
           my $matched = grep{ $_ eq $usedIn } @{$errorCodes{$key}{'USED-IN'}} if( defined $errorCodes{$key}{'USED-IN'} );
           if( $matched ) {
              return 1;
           }
           return 0;
       }
    }
    return 0;
}
=head2 IsPreCheckError($)

=begin html
Returns true if the specified error code is an error of the preCheck script

=end html

=cut


sub IsPreCheckError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"PRCK");
}

=head2 IsSwDlError($)

=begin html
Returns true if the specified error code is an error of the software download script

=end html

=cut

sub IsSwDlError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"SWDL");
}

=head2 IsMigrationError($)

=begin html
Returns true if the specified error code is an error of the migration script

=end html

=cut

sub IsMigrationError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"MIGR");
}

=head2 IsUpgradeError($)

=begin html
Returns true if the specified error code is an error of the upgrade script

=end html

=cut

sub IsUpgradeError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"UPGR");
}

=head2 IsCommitError($)

=begin html
Returns true if the specified error code is an error of the commit script

=end html

=cut

sub IsCommitError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"COMT");
}

=head2 IsStatusCheckError($)

=begin html
Returns true if the specified error code is an error of the status check script

=end html

=cut

sub IsStatusCheckError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"SCHK");
}

=head2 IsStatusCompareError($)

=begin html
Returns true if the specified error code is an error of the status check comparison script

=end html

=cut

sub IsStatusCompareError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"CHKC");
}

=head2 IsSwReduceError($)

=begin html
Returns true if the specified error code is an error of the online SW reduction script

=end html

=cut

sub IsSwReduceError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"SWRD");
}

=head2 IsSwMinimizeError($)

=begin html
Returns true if the specified error code is an error of the SW minimizer script

=end html

=cut

sub IsSwMinimizeError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"SWMM");
}


=head2 IsRollbackError($)

=begin html
Returns true if the specified error code is an error of the Rollback script

=end html

=cut

sub IsRollbackError($) {
  my ($code) = @_;
  return IsErrorUsedIIn($code,"RLBK");
}


=head2 MakePrckWarningFile($$)

=begin html
Regenerate a warning level file defining a level to every check carried out by the script.  The values are taken from the $errorCodes hash related
to the pre-checks.<br>
The following parameters are used:
<table>
  <tr><td>pwd</td><td>Directory where PBMT tools have been installed</td></tr>
  <tr><td>fileName</td><td>Name of the precheck warning file</td></tr>
</table>

=end html

=cut

sub MakeWarningFile($$$) {
  my ($reportType, $pwd, $fileName) = @_;
  my $wfile;

  if (-e "$pwd/$fileName") {
    if (unlink("$pwd/$fileName") == 0) {
      print STDERR "Deleting '$pwd/$fileName' failed: $!\n";
      exit(EXIT_WARN_CODE);
    }
  }
  my @errorList = ();
  if( $reportType eq "PRCK" ) {
      #@errorList = @PRCK_ERRORS;
      @errorList = @{$EXPORT_TAGS{'PRCK'}}
  } elsif( $reportType eq "CHKC" ) {
      @errorList = @CHKC_ERRORS;
  } else {
      print STDERR "Inavlid report type '$reportType' passed\n";
      exit(EXIT_WARN_CODE);
  }

  my $error_found = 0;
  if (open($wfile, ">$pwd/$fileName")) {
    my %numeric_based_error_codes = ();
    foreach my $key ( @errorList) {
        if( defined $errorCodes{$key}{'CODE-NO'}  ) {
           $numeric_based_error_codes{ $errorCodes{$key}{'CODE-NO'} } = $key;
        } else {
           print STDERR "Error code is not found for '$key'\n";
           $error_found = 1;
        }
    }
    if( $error_found ) {
        print STDERR "Some error key is missed with proper details'\n";
        exit(EXIT_WARN_CODE);
    }
    foreach my $key ( sort keys %numeric_based_error_codes ) {
       $key = $numeric_based_error_codes{$key};
       if( defined $errorCodes{$key} ) {
           my $report_mode = defined( $errorCodes{$key}{'PRCK'} ) ? $errorCodes{$key}{'PRCK'} : $ERROR;
	       print $wfile sprintf( "###Code No - %d; Key - %s; Mode - %s;\n###Message - %s\n%s=%d\n",
	     		           $errorCodes{$key}{'CODE-NO'},
	     		           $key,
	     		           ErrorCodes::WarningLevelString( $report_mode ),
	     		           $errorCodes{$key}{'MESSAGE'},
	     		           $errorCodes{$key}{'CODE-NO'},
	     		           $report_mode);
       }
    }
    close $wfile;
  } else {
    print STDERR "Opening file '$pwd/$fileName' for writing failed: $!\n";
    exit(EXIT_WARN_CODE);
  }
}

#-- End of library
1;
